/* Generated by Cython 3.0.11 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "define_macros": [
            [
                "NPY_NO_DEPRECATED_API",
                "NPY_1_7_API_VERSION"
            ]
        ],
        "depends": [],
        "include_dirs": [
            "/tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/_core/include"
        ],
        "name": "serializor.unicode.des",
        "sources": [
            "src/serializor/unicode/des.py"
        ]
    },
    "module_name": "serializor.unicode.des"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#if defined(CYTHON_LIMITED_API) && 0
  #ifndef Py_LIMITED_API
    #if CYTHON_LIMITED_API+0 > 0x03030000
      #define Py_LIMITED_API CYTHON_LIMITED_API
    #else
      #define Py_LIMITED_API 0x03030000
    #endif
  #endif
#endif

#include "Python.h"

    #if PY_MAJOR_VERSION >= 3
      #define __Pyx_PyFloat_FromString(obj)  PyFloat_FromString(obj)
    #else
      #define __Pyx_PyFloat_FromString(obj)  PyFloat_FromString(obj, NULL)
    #endif
    

    #if PY_MAJOR_VERSION <= 2
    #define PyDict_GetItemWithError _PyDict_GetItemWithError
    #endif
    

    #if (PY_VERSION_HEX < 0x030700b1 || (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM < 0x07030600)) && !defined(PyContextVar_Get)
    #define PyContextVar_Get(var, d, v)         ((d) ?             ((void)(var), Py_INCREF(d), (v)[0] = (d), 0) :             ((v)[0] = NULL, 0)         )
    #endif
    
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02070000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.7+ or Python 3.3+.
#else
#if defined(CYTHON_LIMITED_API) && CYTHON_LIMITED_API
#define __PYX_EXTRA_ABI_MODULE_NAME "limited"
#else
#define __PYX_EXTRA_ABI_MODULE_NAME ""
#endif
#define CYTHON_ABI "3_0_11" __PYX_EXTRA_ABI_MODULE_NAME
#define __PYX_ABI_MODULE_NAME "_cython_" CYTHON_ABI
#define __PYX_TYPE_MODULE_PREFIX __PYX_ABI_MODULE_NAME "."
#define CYTHON_HEX_VERSION 0x03000BF0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(_WIN32) && !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #define HAVE_LONG_LONG
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#define __PYX_LIMITED_VERSION_HEX PY_VERSION_HEX
#if defined(GRAALVM_PYTHON)
  /* For very preliminary testing purposes. Most variables are set the same as PyPy.
     The existence of this section does not imply that anything works or is even tested */
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 1
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS (PY_MAJOR_VERSION >= 3)
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
#elif defined(PYPY_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS (PY_MAJOR_VERSION >= 3)
  #endif
  #if PY_VERSION_HEX < 0x03090000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1 && PYPY_VERSION_NUM >= 0x07030C00)
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
#elif defined(CYTHON_LIMITED_API)
  #ifdef Py_LIMITED_API
    #undef __PYX_LIMITED_VERSION_HEX
    #define __PYX_LIMITED_VERSION_HEX Py_LIMITED_API
  #endif
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 1
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_CLINE_IN_TRACEBACK
  #define CYTHON_CLINE_IN_TRACEBACK 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 1
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #endif
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 1
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
#elif defined(Py_GIL_DISABLED) || defined(Py_NOGIL)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #ifndef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #ifndef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #ifndef CYTHON_METH_FASTCALL
    #define CYTHON_METH_FASTCALL 1
  #endif
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
  #ifndef CYTHON_USE_FREELISTS
    #define CYTHON_USE_FREELISTS 0
  #endif
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #ifndef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #ifndef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_GIL
    #define CYTHON_FAST_GIL (PY_MAJOR_VERSION < 3 || PY_VERSION_HEX >= 0x03060000 && PY_VERSION_HEX < 0x030C00A6)
  #endif
  #ifndef CYTHON_METH_FASTCALL
    #define CYTHON_METH_FASTCALL (PY_VERSION_HEX >= 0x030700A1)
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #if PY_VERSION_HEX < 0x030400a1
    #undef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #elif !defined(CYTHON_USE_TP_FINALIZE)
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #if PY_VERSION_HEX < 0x030600B1
    #undef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS 0
  #elif !defined(CYTHON_USE_DICT_VERSIONS)
    #define CYTHON_USE_DICT_VERSIONS  (PY_VERSION_HEX < 0x030C00A5)
  #endif
  #if PY_VERSION_HEX < 0x030700A3
    #undef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 0
  #elif !defined(CYTHON_USE_EXC_INFO_STACK)
    #define CYTHON_USE_EXC_INFO_STACK 1
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
  #ifndef CYTHON_USE_FREELISTS
    #define CYTHON_USE_FREELISTS 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if !defined(CYTHON_VECTORCALL)
#define CYTHON_VECTORCALL  (CYTHON_FAST_PYCCALL && PY_VERSION_HEX >= 0x030800B1)
#endif
#define CYTHON_BACKPORT_VECTORCALL (CYTHON_METH_FASTCALL && PY_VERSION_HEX < 0x030800B1)
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(maybe_unused) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(maybe_unused)
        #define CYTHON_UNUSED [[maybe_unused]]
      #endif
    #endif
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
  #define CYTHON_MAYBE_UNUSED_VAR(x) CYTHON_UNUSED_VAR(x)
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_USE_CPP_STD_MOVE
  #if defined(__cplusplus) && (\
    __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1600))
    #define CYTHON_USE_CPP_STD_MOVE 1
  #else
    #define CYTHON_USE_CPP_STD_MOVE 0
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
            typedef unsigned char     uint8_t;
            typedef unsigned short    uint16_t;
            typedef unsigned int      uint32_t;
        #else
            typedef unsigned __int8   uint8_t;
            typedef unsigned __int16  uint16_t;
            typedef unsigned __int32  uint32_t;
        #endif
    #endif
    #if _MSC_VER < 1300
        #ifdef _WIN64
            typedef unsigned long long  __pyx_uintptr_t;
        #else
            typedef unsigned int        __pyx_uintptr_t;
        #endif
    #else
        #ifdef _WIN64
            typedef unsigned __int64    __pyx_uintptr_t;
        #else
            typedef unsigned __int32    __pyx_uintptr_t;
        #endif
    #endif
#else
    #include <stdint.h>
    typedef uintptr_t  __pyx_uintptr_t;
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(fallthrough) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(fallthrough)
        #define CYTHON_FALLTHROUGH [[fallthrough]]
      #endif
    #endif
    #ifndef CYTHON_FALLTHROUGH
      #if __has_cpp_attribute(clang::fallthrough)
        #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
      #elif __has_cpp_attribute(gnu::fallthrough)
        #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
      #endif
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif
#ifdef __cplusplus
  template <typename T>
  struct __PYX_IS_UNSIGNED_IMPL {static const bool value = T(0) < T(-1);};
  #define __PYX_IS_UNSIGNED(type) (__PYX_IS_UNSIGNED_IMPL<type>::value)
#else
  #define __PYX_IS_UNSIGNED(type) (((type)-1) > 0)
#endif
#if CYTHON_COMPILING_IN_PYPY == 1
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x030A0000)
#else
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000)
#endif
#define __PYX_REINTERPRET_FUNCION(func_pointer, other_pointer) ((func_pointer)(void(*)(void))(other_pointer))

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_DefaultClassType PyClass_Type
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if CYTHON_COMPILING_IN_LIMITED_API
    static CYTHON_INLINE PyObject* __Pyx_PyCode_New(int a, int p, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *exception_table = NULL;
        PyObject *types_module=NULL, *code_type=NULL, *result=NULL;
        #if __PYX_LIMITED_VERSION_HEX < 0x030B0000
        PyObject *version_info;
        PyObject *py_minor_version = NULL;
        #endif
        long minor_version = 0;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        #if __PYX_LIMITED_VERSION_HEX >= 0x030B0000
        minor_version = 11;
        #else
        if (!(version_info = PySys_GetObject("version_info"))) goto end;
        if (!(py_minor_version = PySequence_GetItem(version_info, 1))) goto end;
        minor_version = PyLong_AsLong(py_minor_version);
        Py_DECREF(py_minor_version);
        if (minor_version == -1 && PyErr_Occurred()) goto end;
        #endif
        if (!(types_module = PyImport_ImportModule("types"))) goto end;
        if (!(code_type = PyObject_GetAttrString(types_module, "CodeType"))) goto end;
        if (minor_version <= 7) {
            (void)p;
            result = PyObject_CallFunction(code_type, "iiiiiOOOOOOiOO", a, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else if (minor_version <= 10) {
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOiOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else {
            if (!(exception_table = PyBytes_FromStringAndSize(NULL, 0))) goto end;
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOOiOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, name, fline, lnos, exception_table, fv, cell);
        }
    end:
        Py_XDECREF(code_type);
        Py_XDECREF(exception_table);
        Py_XDECREF(types_module);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return result;
    }
    #ifndef CO_OPTIMIZED
    #define CO_OPTIMIZED 0x0001
    #endif
    #ifndef CO_NEWLOCALS
    #define CO_NEWLOCALS 0x0002
    #endif
    #ifndef CO_VARARGS
    #define CO_VARARGS 0x0004
    #endif
    #ifndef CO_VARKEYWORDS
    #define CO_VARKEYWORDS 0x0008
    #endif
    #ifndef CO_ASYNC_GENERATOR
    #define CO_ASYNC_GENERATOR 0x0200
    #endif
    #ifndef CO_GENERATOR
    #define CO_GENERATOR 0x0020
    #endif
    #ifndef CO_COROUTINE
    #define CO_COROUTINE 0x0080
    #endif
#elif PY_VERSION_HEX >= 0x030B0000
  static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int p, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
    PyCodeObject *result;
    PyObject *empty_bytes = PyBytes_FromStringAndSize("", 0);
    if (!empty_bytes) return NULL;
    result =
      #if PY_VERSION_HEX >= 0x030C0000
        PyUnstable_Code_NewWithPosOnlyArgs
      #else
        PyCode_NewWithPosOnlyArgs
      #endif
        (a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, name, fline, lnos, empty_bytes);
    Py_DECREF(empty_bytes);
    return result;
  }
#elif PY_VERSION_HEX >= 0x030800B2 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_NewWithPosOnlyArgs(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
#endif
#if PY_VERSION_HEX >= 0x030900A4 || defined(Py_IS_TYPE)
  #define __Pyx_IS_TYPE(ob, type) Py_IS_TYPE(ob, type)
#else
  #define __Pyx_IS_TYPE(ob, type) (((const PyObject*)ob)->ob_type == (type))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_Is)
  #define __Pyx_Py_Is(x, y)  Py_Is(x, y)
#else
  #define __Pyx_Py_Is(x, y) ((x) == (y))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsNone)
  #define __Pyx_Py_IsNone(ob) Py_IsNone(ob)
#else
  #define __Pyx_Py_IsNone(ob) __Pyx_Py_Is((ob), Py_None)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsTrue)
  #define __Pyx_Py_IsTrue(ob) Py_IsTrue(ob)
#else
  #define __Pyx_Py_IsTrue(ob) __Pyx_Py_Is((ob), Py_True)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsFalse)
  #define __Pyx_Py_IsFalse(ob) Py_IsFalse(ob)
#else
  #define __Pyx_Py_IsFalse(ob) __Pyx_Py_Is((ob), Py_False)
#endif
#define __Pyx_NoneAsNull(obj)  (__Pyx_Py_IsNone(obj) ? NULL : (obj))
#if PY_VERSION_HEX >= 0x030900F0 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyObject_GC_IsFinalized(o) PyObject_GC_IsFinalized(o)
#else
  #define __Pyx_PyObject_GC_IsFinalized(o) _PyGC_FINALIZED(o)
#endif
#ifndef CO_COROUTINE
  #define CO_COROUTINE 0x80
#endif
#ifndef CO_ASYNC_GENERATOR
  #define CO_ASYNC_GENERATOR 0x200
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef Py_TPFLAGS_SEQUENCE
  #define Py_TPFLAGS_SEQUENCE 0
#endif
#ifndef Py_TPFLAGS_MAPPING
  #define Py_TPFLAGS_MAPPING 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #if PY_VERSION_HEX >= 0x030d00A4
  #  define __Pyx_PyCFunctionFast PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords PyCFunctionFastWithKeywords
  #else
  #  define __Pyx_PyCFunctionFast _PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
  #endif
#endif
#if CYTHON_METH_FASTCALL
  #define __Pyx_METH_FASTCALL METH_FASTCALL
  #define __Pyx_PyCFunction_FastCall __Pyx_PyCFunctionFast
  #define __Pyx_PyCFunction_FastCallWithKeywords __Pyx_PyCFunctionFastWithKeywords
#else
  #define __Pyx_METH_FASTCALL METH_VARARGS
  #define __Pyx_PyCFunction_FastCall PyCFunction
  #define __Pyx_PyCFunction_FastCallWithKeywords PyCFunctionWithKeywords
#endif
#if CYTHON_VECTORCALL
  #define __pyx_vectorcallfunc vectorcallfunc
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  PY_VECTORCALL_ARGUMENTS_OFFSET
  #define __Pyx_PyVectorcall_NARGS(n)  PyVectorcall_NARGS((size_t)(n))
#elif CYTHON_BACKPORT_VECTORCALL
  typedef PyObject *(*__pyx_vectorcallfunc)(PyObject *callable, PyObject *const *args,
                                            size_t nargsf, PyObject *kwnames);
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  ((size_t)1 << (8 * sizeof(size_t) - 1))
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(((size_t)(n)) & ~__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET))
#else
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  0
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(n))
#endif
#if PY_MAJOR_VERSION >= 0x030900B1
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_CheckExact(func)
#else
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_Check(func)
#endif
#define __Pyx_CyOrPyCFunction_Check(func)  PyCFunction_Check(func)
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  (((PyCFunctionObject*)(func))->m_ml->ml_meth)
#elif !CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  PyCFunction_GET_FUNCTION(func)
#endif
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FLAGS(func)  (((PyCFunctionObject*)(func))->m_ml->ml_flags)
static CYTHON_INLINE PyObject* __Pyx_CyOrPyCFunction_GET_SELF(PyObject *func) {
    return (__Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_STATIC) ? NULL : ((PyCFunctionObject*)func)->m_self;
}
#endif
static CYTHON_INLINE int __Pyx__IsSameCFunction(PyObject *func, void *cfunc) {
#if CYTHON_COMPILING_IN_LIMITED_API
    return PyCFunction_Check(func) && PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
#else
    return PyCFunction_Check(func) && PyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
#endif
}
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCFunction(func, cfunc)
#if __PYX_LIMITED_VERSION_HEX < 0x030900B1
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  ((void)m, PyType_FromSpecWithBases(s, b))
  typedef PyObject *(*__Pyx_PyCMethod)(PyObject *, PyTypeObject *, PyObject *const *, size_t, PyObject *);
#else
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  PyType_FromModuleAndSpec(m, s, b)
  #define __Pyx_PyCMethod  PyCMethod
#endif
#ifndef METH_METHOD
  #define METH_METHOD 0x200
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyThreadState_Current PyThreadState_Get()
#elif !CYTHON_FAST_THREAD_STATE
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyThreadState_Current PyThreadState_GetUnchecked()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE void *__Pyx_PyModule_GetState(PyObject *op)
{
    void *result;
    result = PyModule_GetState(op);
    if (!result)
        Py_FatalError("Couldn't find the module state");
    return result;
}
#endif
#define __Pyx_PyObject_GetSlot(obj, name, func_ctype)  __Pyx_PyType_GetSlot(Py_TYPE(obj), name, func_ctype)
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((func_ctype) PyType_GetSlot((type), Py_##name))
#else
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((type)->name)
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if PY_MAJOR_VERSION < 3
    #if CYTHON_COMPILING_IN_PYPY
        #if PYPY_VERSION_NUM < 0x07030600
            #if defined(__cplusplus) && __cplusplus >= 201402L
                [[deprecated("`with nogil:` inside a nogil function will not release the GIL in PyPy2 < 7.3.6")]]
            #elif defined(__GNUC__) || defined(__clang__)
                __attribute__ ((__deprecated__("`with nogil:` inside a nogil function will not release the GIL in PyPy2 < 7.3.6")))
            #elif defined(_MSC_VER)
                __declspec(deprecated("`with nogil:` inside a nogil function will not release the GIL in PyPy2 < 7.3.6"))
            #endif
            static CYTHON_INLINE int PyGILState_Check(void) {
                return 0;
            }
        #else  // PYPY_VERSION_NUM < 0x07030600
        #endif  // PYPY_VERSION_NUM < 0x07030600
    #else
        static CYTHON_INLINE int PyGILState_Check(void) {
            PyThreadState * tstate = _PyThreadState_Current;
            return tstate && (tstate == PyGILState_GetThisThreadState());
        }
    #endif
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030d0000 || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX > 0x030600B4 && PY_VERSION_HEX < 0x030d0000 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStrWithError(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStr(PyObject *dict, PyObject *name) {
    PyObject *res = __Pyx_PyDict_GetItemStrWithError(dict, name);
    if (res == NULL) PyErr_Clear();
    return res;
}
#elif PY_MAJOR_VERSION >= 3 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000)
#define __Pyx_PyDict_GetItemStrWithError  PyDict_GetItemWithError
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#else
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStrWithError(PyObject *dict, PyObject *name) {
#if CYTHON_COMPILING_IN_PYPY
    return PyDict_GetItem(dict, name);
#else
    PyDictEntry *ep;
    PyDictObject *mp = (PyDictObject*) dict;
    long hash = ((PyStringObject *) name)->ob_shash;
    assert(hash != -1);
    ep = (mp->ma_lookup)(mp, name, hash);
    if (ep == NULL) {
        return NULL;
    }
    return ep->me_value;
#endif
}
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#endif
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetFlags(tp)   (((PyTypeObject *)tp)->tp_flags)
  #define __Pyx_PyType_HasFeature(type, feature)  ((__Pyx_PyType_GetFlags(type) & (feature)) != 0)
  #define __Pyx_PyObject_GetIterNextFunc(obj)  (Py_TYPE(obj)->tp_iternext)
#else
  #define __Pyx_PyType_GetFlags(tp)   (PyType_GetFlags((PyTypeObject *)tp))
  #define __Pyx_PyType_HasFeature(type, feature)  PyType_HasFeature(type, feature)
  #define __Pyx_PyObject_GetIterNextFunc(obj)  PyIter_Next
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_SetItemOnTypeDict(tp, k, v) PyObject_GenericSetAttr((PyObject*)tp, k, v)
#else
  #define __Pyx_SetItemOnTypeDict(tp, k, v) PyDict_SetItem(tp->tp_dict, k, v)
#endif
#if CYTHON_USE_TYPE_SPECS && PY_VERSION_HEX >= 0x03080000
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  {\
    PyTypeObject *type = Py_TYPE((PyObject*)obj);\
    assert(__Pyx_PyType_HasFeature(type, Py_TPFLAGS_HEAPTYPE));\
    PyObject_GC_Del(obj);\
    Py_DECREF(type);\
}
#else
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  PyObject_GC_Del(obj)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GetLength(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_ReadChar(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((void)u, 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((void)u, (0))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)u)
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)k, PyUnicode_ReadChar((PyObject*)(d), i))
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GetLength(u))
#elif PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_READY(op)       (0)
  #else
    #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                                0 : _PyUnicode_Ready((PyObject *)(op)))
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         ((int)PyUnicode_KIND(u))
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, (Py_UCS4) ch)
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #else
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
    #else
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
    #endif
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535U : 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((int)sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = (Py_UNICODE) ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #if !defined(PyUnicode_DecodeUnicodeEscape)
    #define PyUnicode_DecodeUnicodeEscape(s, size, errors)  PyUnicode_Decode(s, size, "unicode_escape", errors)
  #endif
  #if !defined(PyUnicode_Contains) || (PY_MAJOR_VERSION == 2 && PYPY_VERSION_NUM < 0x07030500)
    #undef PyUnicode_Contains
    #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
  #endif
  #if !defined(PyByteArray_Check)
    #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
  #endif
  #if !defined(PyObject_Format)
    #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
  #endif
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#if CYTHON_COMPILING_IN_CPYTHON
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && Py_REFCNT(obj) == 1) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#else
  #define __Pyx_PySequence_ListKeepNew(obj)  PySequence_List(obj)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        __Pyx_IS_TYPE(obj, &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_ITEM(o, i) PySequence_ITEM(o, i)
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) (PyTuple_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyList_SET_ITEM(o, i, v) (PyList_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_GET_SIZE(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_GET_SIZE(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_GET_SIZE(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_GET_SIZE(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_GET_SIZE(o)
#else
  #define __Pyx_PySequence_ITEM(o, i) PySequence_GetItem(o, i)
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) PyTuple_SetItem(o, i, v)
  #define __Pyx_PyList_SET_ITEM(o, i, v) PyList_SetItem(o, i, v)
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_Size(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_Size(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_Size(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_Size(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_Size(o)
#endif
#if __PYX_LIMITED_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#else
  static CYTHON_INLINE PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *module = PyImport_AddModule(name);
      Py_XINCREF(module);
      return module;
  }
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define __Pyx_Py3Int_Check(op)       PyLong_Check(op)
  #define __Pyx_Py3Int_CheckExact(op)  PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#else
  #define __Pyx_Py3Int_Check(op)       (PyLong_Check(op) || PyInt_Check(op))
  #define __Pyx_Py3Int_CheckExact(op)  (PyLong_CheckExact(op) || PyInt_CheckExact(op))
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #if !defined(_USE_MATH_DEFINES)
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__;  (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifdef CYTHON_EXTERN_C
    #undef __PYX_EXTERN_C
    #define __PYX_EXTERN_C CYTHON_EXTERN_C
#elif defined(__PYX_EXTERN_C)
    #ifdef _MSC_VER
    #pragma message ("Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.")
    #else
    #warning Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.
    #endif
#else
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__serializor__unicode__des
#define __PYX_HAVE_API__serializor__unicode__des
/* Early includes */
#include <stddef.h>
#include <limits.h>
#include <string.h>
#include <stdio.h>

    /* Using NumPy API declarations from "numpy/__init__.cython-30.pxd" */
    
#include "numpy/arrayobject.h"
#include "numpy/ndarrayobject.h"
#include "numpy/ndarraytypes.h"
#include "numpy/arrayscalars.h"
#include "numpy/ufuncobject.h"
#include "pythread.h"
#include "datetime.h"

    /* Backport for Python 2.x */
    #if PY_MAJOR_VERSION < 3
        #ifndef PyDateTime_DELTA_GET_DAYS
            #define PyDateTime_DELTA_GET_DAYS(o) (((PyDateTime_Delta*)o)->days)
        #endif
        #ifndef PyDateTime_DELTA_GET_SECONDS
            #define PyDateTime_DELTA_GET_SECONDS(o) (((PyDateTime_Delta*)o)->seconds)
        #endif
        #ifndef PyDateTime_DELTA_GET_MICROSECONDS
            #define PyDateTime_DELTA_GET_MICROSECONDS(o) (((PyDateTime_Delta*)o)->microseconds)
        #endif
    #endif

    /* Backport for Python < 3.6 */
    #if PY_VERSION_HEX < 0x030600a4
        #ifndef PyDateTime_TIME_GET_FOLD
            #define PyDateTime_TIME_GET_FOLD(o) ((void)(o), 0)
        #endif
        #ifndef PyDateTime_DATE_GET_FOLD
            #define PyDateTime_DATE_GET_FOLD(o) ((void)(o), 0)
        #endif
    #endif

    /* Backport for Python < 3.6 */
    #if PY_VERSION_HEX < 0x030600a4
        #define __Pyx_DateTime_DateTimeWithFold(year, month, day, hour, minute, second, microsecond, tz, fold)             ((void)(fold), PyDateTimeAPI->DateTime_FromDateAndTime(year, month, day, hour, minute, second,                 microsecond, tz, PyDateTimeAPI->DateTimeType))
        #define __Pyx_DateTime_TimeWithFold(hour, minute, second, microsecond, tz, fold)             ((void)(fold), PyDateTimeAPI->Time_FromTime(hour, minute, second, microsecond, tz, PyDateTimeAPI->TimeType))
    #else /* For Python 3.6+ so that we can pass tz */
        #define __Pyx_DateTime_DateTimeWithFold(year, month, day, hour, minute, second, microsecond, tz, fold)             PyDateTimeAPI->DateTime_FromDateAndTimeAndFold(year, month, day, hour, minute, second,                 microsecond, tz, fold, PyDateTimeAPI->DateTimeType)
        #define __Pyx_DateTime_TimeWithFold(hour, minute, second, microsecond, tz, fold)             PyDateTimeAPI->Time_FromTimeAndFold(hour, minute, second, microsecond, tz, fold, PyDateTimeAPI->TimeType)
    #endif

    /* Backport for Python < 3.7 */
    #if PY_VERSION_HEX < 0x030700b1
        #define __Pyx_TimeZone_UTC NULL
        #define __Pyx_TimeZone_FromOffsetAndName(offset, name) ((void)(offset), (void)(name), (PyObject*)NULL)
    #else
        #define __Pyx_TimeZone_UTC PyDateTime_TimeZone_UTC
        #define __Pyx_TimeZone_FromOffsetAndName(offset, name) PyTimeZone_FromOffsetAndName(offset, name)
    #endif

    /* Backport for Python < 3.10 */
    #if PY_VERSION_HEX < 0x030a00a1
        #ifndef PyDateTime_TIME_GET_TZINFO
            #define PyDateTime_TIME_GET_TZINFO(o)                 ((((PyDateTime_Time*)o)->hastzinfo) ? ((PyDateTime_Time*)o)->tzinfo : Py_None)
        #endif
        #ifndef PyDateTime_DATE_GET_TZINFO
            #define PyDateTime_DATE_GET_TZINFO(o)                 ((((PyDateTime_DateTime*)o)->hastzinfo) ? ((PyDateTime_DateTime*)o)->tzinfo : Py_None)
        #endif
    #endif
    
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s);
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char*);
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#define __Pyx_PyUnicode_FromOrdinal(o)       PyUnicode_FromOrdinal((int)o)
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_VERSION_HEX >= 0x030C00A7
  #ifndef _PyLong_SIGN_MASK
    #define _PyLong_SIGN_MASK 3
  #endif
  #ifndef _PyLong_NON_SIZE_BITS
    #define _PyLong_NON_SIZE_BITS 3
  #endif
  #define __Pyx_PyLong_Sign(x)  (((PyLongObject*)x)->long_value.lv_tag & _PyLong_SIGN_MASK)
  #define __Pyx_PyLong_IsNeg(x)  ((__Pyx_PyLong_Sign(x) & 2) != 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (!__Pyx_PyLong_IsNeg(x))
  #define __Pyx_PyLong_IsZero(x)  (__Pyx_PyLong_Sign(x) & 1)
  #define __Pyx_PyLong_IsPos(x)  (__Pyx_PyLong_Sign(x) == 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  (__Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  ((Py_ssize_t) (((PyLongObject*)x)->long_value.lv_tag >> _PyLong_NON_SIZE_BITS))
  #define __Pyx_PyLong_SignedDigitCount(x)\
        ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * __Pyx_PyLong_DigitCount(x))
  #if defined(PyUnstable_Long_IsCompact) && defined(PyUnstable_Long_CompactValue)
    #define __Pyx_PyLong_IsCompact(x)     PyUnstable_Long_IsCompact((PyLongObject*) x)
    #define __Pyx_PyLong_CompactValue(x)  PyUnstable_Long_CompactValue((PyLongObject*) x)
  #else
    #define __Pyx_PyLong_IsCompact(x)     (((PyLongObject*)x)->long_value.lv_tag < (2 << _PyLong_NON_SIZE_BITS))
    #define __Pyx_PyLong_CompactValue(x)  ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * (Py_ssize_t) __Pyx_PyLong_Digits(x)[0])
  #endif
  typedef Py_ssize_t  __Pyx_compact_pylong;
  typedef size_t  __Pyx_compact_upylong;
  #else
  #define __Pyx_PyLong_IsNeg(x)  (Py_SIZE(x) < 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (Py_SIZE(x) >= 0)
  #define __Pyx_PyLong_IsZero(x)  (Py_SIZE(x) == 0)
  #define __Pyx_PyLong_IsPos(x)  (Py_SIZE(x) > 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  ((Py_SIZE(x) == 0) ? 0 : __Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  __Pyx_sst_abs(Py_SIZE(x))
  #define __Pyx_PyLong_SignedDigitCount(x)  Py_SIZE(x)
  #define __Pyx_PyLong_IsCompact(x)  (Py_SIZE(x) == 0 || Py_SIZE(x) == 1 || Py_SIZE(x) == -1)
  #define __Pyx_PyLong_CompactValue(x)\
        ((Py_SIZE(x) == 0) ? (sdigit) 0 : ((Py_SIZE(x) < 0) ? -(sdigit)__Pyx_PyLong_Digits(x)[0] : (sdigit)__Pyx_PyLong_Digits(x)[0]))
  typedef sdigit  __Pyx_compact_pylong;
  typedef digit  __Pyx_compact_upylong;
  #endif
  #if PY_VERSION_HEX >= 0x030C00A5
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->long_value.ob_digit)
  #else
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->ob_digit)
  #endif
#endif
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
#include <string.h>
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = (char) c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#include <string.h>
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

#if !CYTHON_USE_MODULE_STATE
static PyObject *__pyx_m = NULL;
#endif
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm = __FILE__;
static const char *__pyx_filename;

/* Header.proto */
#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif (defined(_Complex_I) && !defined(_MSC_VER)) || ((defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L) && !defined(__STDC_NO_COMPLEX__) && !defined(_MSC_VER))
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif

/* #### Code section: filename_table ### */

static const char *__pyx_f[] = {
  "src/serializor/unicode/des.pxd",
  "src/serializor/unicode/des.py",
  "src/serializor/unicode/ser.pxd",
  "__init__.cython-30.pxd",
  "contextvars.pxd",
  "datetime.pxd",
  "src/serializor/utils.pxd",
  "type.pxd",
  "bool.pxd",
  "complex.pxd",
};
/* #### Code section: utility_code_proto_before_types ### */
/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

/* #### Code section: numeric_typedefs ### */

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":769
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":770
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":771
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":772
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":776
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":777
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":778
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":779
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":783
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":784
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":791
 * ctypedef double complex complex128_t
 * 
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":792
 * 
 * ctypedef npy_longlong   longlong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":794
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":795
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":797
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":798
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":799
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef float complex       cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;
/* #### Code section: complex_type_declarations ### */
/* Declarations.proto */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif
static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

/* Declarations.proto */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif
static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);

/* Declarations.proto */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
  #ifdef __cplusplus
    typedef ::std::complex< long double > __pyx_t_long_double_complex;
  #else
    typedef long double _Complex __pyx_t_long_double_complex;
  #endif
#else
    typedef struct { long double real, imag; } __pyx_t_long_double_complex;
#endif
static CYTHON_INLINE __pyx_t_long_double_complex __pyx_t_long_double_complex_from_parts(long double, long double);

/* #### Code section: type_declarations ### */

/*--- Type declarations ---*/
struct __pyx_opt_args_7cpython_11contextvars_get_value;
struct __pyx_opt_args_7cpython_11contextvars_get_value_no_default;

/* "cpython/contextvars.pxd":112
 * 
 * 
 * cdef inline object get_value(var, default_value=None):             # <<<<<<<<<<<<<<
 *     """Return a new reference to the value of the context variable,
 *     or the default value of the context variable,
 */
struct __pyx_opt_args_7cpython_11contextvars_get_value {
  int __pyx_n;
  PyObject *default_value;
};

/* "cpython/contextvars.pxd":129
 * 
 * 
 * cdef inline object get_value_no_default(var, default_value=None):             # <<<<<<<<<<<<<<
 *     """Return a new reference to the value of the context variable,
 *     or the provided default value if no such value was found.
 */
struct __pyx_opt_args_7cpython_11contextvars_get_value_no_default {
  int __pyx_n;
  PyObject *default_value;
};

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1095
 * 
 * # Iterator API added in v1.6
 * ctypedef int (*NpyIter_IterNextFunc)(NpyIter* it) noexcept nogil             # <<<<<<<<<<<<<<
 * ctypedef void (*NpyIter_GetMultiIndexFunc)(NpyIter* it, npy_intp* outcoords) noexcept nogil
 * 
 */
typedef int (*__pyx_t_5numpy_NpyIter_IterNextFunc)(NpyIter *);

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1096
 * # Iterator API added in v1.6
 * ctypedef int (*NpyIter_IterNextFunc)(NpyIter* it) noexcept nogil
 * ctypedef void (*NpyIter_GetMultiIndexFunc)(NpyIter* it, npy_intp* outcoords) noexcept nogil             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "numpy/arrayobject.h":
 */
typedef void (*__pyx_t_5numpy_NpyIter_GetMultiIndexFunc)(NpyIter *, npy_intp *);
struct __pyx_opt_args_7cpython_8datetime_time_new;
struct __pyx_opt_args_7cpython_8datetime_datetime_new;
struct __pyx_opt_args_7cpython_8datetime_timezone_new;
struct __pyx_opt_args_7cpython_8datetime_datetime_from_timestamp;

/* "datetime.pxd":292
 * # Create time object using DateTime CAPI factory function
 * # Note, there are no range checks for any of the arguments.
 * cdef inline time time_new(int hour, int minute, int second, int microsecond, object tz, int fold=0):             # <<<<<<<<<<<<<<
 *     return __Pyx_DateTime_TimeWithFold(hour, minute, second, microsecond, tz, fold)
 * 
 */
struct __pyx_opt_args_7cpython_8datetime_time_new {
  int __pyx_n;
  int fold;
};

/* "datetime.pxd":297
 * # Create datetime object using DateTime CAPI factory function.
 * # Note, there are no range checks for any of the arguments.
 * cdef inline datetime datetime_new(int year, int month, int day, int hour, int minute, int second, int microsecond, object tz, int fold=0):             # <<<<<<<<<<<<<<
 *     return __Pyx_DateTime_DateTimeWithFold(year, month, day, hour, minute, second, microsecond, tz, fold)
 * 
 */
struct __pyx_opt_args_7cpython_8datetime_datetime_new {
  int __pyx_n;
  int fold;
};

/* "datetime.pxd":306
 * 
 * # Create timedelta object using DateTime CAPI factory function.
 * cdef inline object timezone_new(object offset, object name=None):             # <<<<<<<<<<<<<<
 *     if PY_VERSION_HEX < 0x030700b1:
 *         raise RuntimeError('Time zones are not available from the C-API.')
 */
struct __pyx_opt_args_7cpython_8datetime_timezone_new {
  int __pyx_n;
  PyObject *name;
};

/* "datetime.pxd":312
 * 
 * # Create datetime object using DB API constructor.
 * cdef inline datetime datetime_from_timestamp(timestamp, tz=None):             # <<<<<<<<<<<<<<
 *     return PyDateTimeAPI.DateTime_FromTimestamp(
 *         <PyObject*>PyDateTimeAPI.DateTimeType, (timestamp, tz) if tz is not None else (timestamp,), NULL)
 */
struct __pyx_opt_args_7cpython_8datetime_datetime_from_timestamp {
  int __pyx_n;
  PyObject *tz;
};
/* #### Code section: utility_code_proto ### */

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, Py_ssize_t);
    void (*DECREF)(void*, PyObject*, Py_ssize_t);
    void (*GOTREF)(void*, PyObject*, Py_ssize_t);
    void (*GIVEREF)(void*, PyObject*, Py_ssize_t);
    void* (*SetupContext)(const char*, Py_ssize_t, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__))
  #define __Pyx_RefNannyFinishContextNogil() __Pyx_RefNannyFinishContext()
#endif
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_XINCREF(r)  do { if((r) == NULL); else {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) == NULL); else {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) == NULL); else {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) == NULL); else {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContextNogil()
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_Py_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; Py_XDECREF(tmp);\
    } while (0)
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#if PY_VERSION_HEX >= 0x030C00A6
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->current_exception != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->current_exception ? (PyObject*) Py_TYPE(__pyx_tstate->current_exception) : (PyObject*) NULL)
#else
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->curexc_type != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->curexc_type)
#endif
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  (PyErr_Occurred() != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A6
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* IncludeStringH.proto */
#include <string.h>

/* decode_c_string_utf16.proto */
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 0;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16LE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = -1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16BE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}

/* decode_c_string.proto */
static CYTHON_INLINE PyObject* __Pyx_decode_c_string(
         const char* cstring, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors));

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#if !CYTHON_VECTORCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if !CYTHON_VECTORCALL
#if PY_VERSION_HEX >= 0x03080000
  #include "frameobject.h"
#if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
  #define __Pxy_PyFrame_Initialize_Offsets()
  #define __Pyx_PyFrame_GetLocalsplus(frame)  ((frame)->f_localsplus)
#else
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif
#endif
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectFastCall.proto */
#define __Pyx_PyObject_FastCall(func, args, nargs)  __Pyx_PyObject_FastCallDict(func, args, (size_t)(nargs), NULL)
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject **args, size_t nargs, PyObject *kwargs);

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* RaiseUnexpectedTypeError.proto */
static int __Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj);

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  do {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  do {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyUnicode_Substring.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Substring(
            PyObject* text, Py_ssize_t start, Py_ssize_t stop);

/* pybytes_as_double.proto */
static double __Pyx_SlowPyString_AsDouble(PyObject *obj);
static double __Pyx__PyBytes_AsDouble(PyObject *obj, const char* start, Py_ssize_t length);
static CYTHON_INLINE double __Pyx_PyBytes_AsDouble(PyObject *obj) {
    char* as_c_string;
    Py_ssize_t size;
#if CYTHON_ASSUME_SAFE_MACROS
    as_c_string = PyBytes_AS_STRING(obj);
    size = PyBytes_GET_SIZE(obj);
#else
    if (PyBytes_AsStringAndSize(obj, &as_c_string, &size) < 0) {
        return (double)-1;
    }
#endif
    return __Pyx__PyBytes_AsDouble(obj, as_c_string, size);
}
static CYTHON_INLINE double __Pyx_PyByteArray_AsDouble(PyObject *obj) {
    char* as_c_string;
    Py_ssize_t size;
#if CYTHON_ASSUME_SAFE_MACROS
    as_c_string = PyByteArray_AS_STRING(obj);
    size = PyByteArray_GET_SIZE(obj);
#else
    as_c_string = PyByteArray_AsString(obj);
    if (as_c_string == NULL) {
        return (double)-1;
    }
    size = PyByteArray_Size(obj);
#endif
    return __Pyx__PyBytes_AsDouble(obj, as_c_string, size);
}

/* pyunicode_as_double.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY && CYTHON_ASSUME_SAFE_MACROS
static const char* __Pyx__PyUnicode_AsDouble_Copy(const void* data, const int kind, char* buffer, Py_ssize_t start, Py_ssize_t end) {
    int last_was_punctuation;
    Py_ssize_t i;
    last_was_punctuation = 1;
    for (i=start; i <= end; i++) {
        Py_UCS4 chr = PyUnicode_READ(kind, data, i);
        int is_punctuation = (chr == '_') | (chr == '.');
        *buffer = (char)chr;
        buffer += (chr != '_');
        if (unlikely(chr > 127)) goto parse_failure;
        if (unlikely(last_was_punctuation & is_punctuation)) goto parse_failure;
        last_was_punctuation = is_punctuation;
    }
    if (unlikely(last_was_punctuation)) goto parse_failure;
    *buffer = '\0';
    return buffer;
parse_failure:
    return NULL;
}
static double __Pyx__PyUnicode_AsDouble_inf_nan(const void* data, int kind, Py_ssize_t start, Py_ssize_t length) {
    int matches = 1;
    Py_UCS4 chr;
    Py_UCS4 sign = PyUnicode_READ(kind, data, start);
    int is_signed = (sign == '-') | (sign == '+');
    start += is_signed;
    length -= is_signed;
    switch (PyUnicode_READ(kind, data, start)) {
        #ifdef Py_NAN
        case 'n':
        case 'N':
            if (unlikely(length != 3)) goto parse_failure;
            chr = PyUnicode_READ(kind, data, start+1);
            matches &= (chr == 'a') | (chr == 'A');
            chr = PyUnicode_READ(kind, data, start+2);
            matches &= (chr == 'n') | (chr == 'N');
            if (unlikely(!matches)) goto parse_failure;
            return (sign == '-') ? -Py_NAN : Py_NAN;
        #endif
        case 'i':
        case 'I':
            if (unlikely(length < 3)) goto parse_failure;
            chr = PyUnicode_READ(kind, data, start+1);
            matches &= (chr == 'n') | (chr == 'N');
            chr = PyUnicode_READ(kind, data, start+2);
            matches &= (chr == 'f') | (chr == 'F');
            if (likely(length == 3 && matches))
                return (sign == '-') ? -Py_HUGE_VAL : Py_HUGE_VAL;
            if (unlikely(length != 8)) goto parse_failure;
            chr = PyUnicode_READ(kind, data, start+3);
            matches &= (chr == 'i') | (chr == 'I');
            chr = PyUnicode_READ(kind, data, start+4);
            matches &= (chr == 'n') | (chr == 'N');
            chr = PyUnicode_READ(kind, data, start+5);
            matches &= (chr == 'i') | (chr == 'I');
            chr = PyUnicode_READ(kind, data, start+6);
            matches &= (chr == 't') | (chr == 'T');
            chr = PyUnicode_READ(kind, data, start+7);
            matches &= (chr == 'y') | (chr == 'Y');
            if (unlikely(!matches)) goto parse_failure;
            return (sign == '-') ? -Py_HUGE_VAL : Py_HUGE_VAL;
        case '.': case '0': case '1': case '2': case '3': case '4': case '5': case '6': case '7': case '8': case '9':
            break;
        default:
            goto parse_failure;
    }
    return 0.0;
parse_failure:
    return -1.0;
}
static double __Pyx_PyUnicode_AsDouble_WithSpaces(PyObject *obj) {
    double value;
    const char *last;
    char *end;
    Py_ssize_t start, length = PyUnicode_GET_LENGTH(obj);
    const int kind = PyUnicode_KIND(obj);
    const void* data = PyUnicode_DATA(obj);
    start = 0;
    while (Py_UNICODE_ISSPACE(PyUnicode_READ(kind, data, start)))
        start++;
    while (start < length - 1 && Py_UNICODE_ISSPACE(PyUnicode_READ(kind, data, length - 1)))
        length--;
    length -= start;
    if (unlikely(length <= 0)) goto fallback;
    value = __Pyx__PyUnicode_AsDouble_inf_nan(data, kind, start, length);
    if (unlikely(value == -1.0)) goto fallback;
    if (value != 0.0) return value;
    if (length < 40) {
        char number[40];
        last = __Pyx__PyUnicode_AsDouble_Copy(data, kind, number, start, start + length);
        if (unlikely(!last)) goto fallback;
        value = PyOS_string_to_double(number, &end, NULL);
    } else {
        char *number = (char*) PyMem_Malloc((length + 1) * sizeof(char));
        if (unlikely(!number)) goto fallback;
        last = __Pyx__PyUnicode_AsDouble_Copy(data, kind, number, start, start + length);
        if (unlikely(!last)) {
            PyMem_Free(number);
            goto fallback;
        }
        value = PyOS_string_to_double(number, &end, NULL);
        PyMem_Free(number);
    }
    if (likely(end == last) || (value == (double)-1 && PyErr_Occurred())) {
        return value;
    }
fallback:
    return __Pyx_SlowPyString_AsDouble(obj);
}
#endif
static CYTHON_INLINE double __Pyx_PyUnicode_AsDouble(PyObject *obj) {
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY && CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(__Pyx_PyUnicode_READY(obj) == -1))
        return (double)-1;
    if (likely(PyUnicode_IS_ASCII(obj))) {
        const char *s;
        Py_ssize_t length;
        s = PyUnicode_AsUTF8AndSize(obj, &length);
        return __Pyx__PyBytes_AsDouble(obj, s, length);
    }
    return __Pyx_PyUnicode_AsDouble_WithSpaces(obj);
#else
    return __Pyx_SlowPyString_AsDouble(obj);
#endif
}

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
        L->ob_item[len] = x;
        #else
        PyList_SET_ITEM(list, len, x);
        #endif
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* pyfrozenset_new.proto */
static CYTHON_INLINE PyObject* __Pyx_PyFrozenSet_New(PyObject* it);

/* IterNext.proto */
#define __Pyx_PyIter_Next(obj) __Pyx_PyIter_Next2(obj, NULL)
static CYTHON_INLINE PyObject *__Pyx_PyIter_Next2(PyObject *, PyObject *);

/* GetItemIntUnicode.proto */
#define __Pyx_GetItemInt_Unicode(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Unicode_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "string index out of range"), (Py_UCS4)-1))
static CYTHON_INLINE Py_UCS4 __Pyx_GetItemInt_Unicode_Fast(PyObject* ustring, Py_ssize_t i,
                                                           int wraparound, int boundscheck);

/* GCCDiagnostics.proto */
#if !defined(__INTEL_COMPILER) && defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* BuildPyUnicode.proto */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, char* chars, int clength,
                                                int prepend_sign, char padding_char);

/* CIntToPyUnicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_Py_ssize_t(Py_ssize_t value, Py_ssize_t width, char padding_char, char format_char);

/* JoinPyUnicode.proto */
static PyObject* __Pyx_PyUnicode_Join(PyObject* value_tuple, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char);

/* PyObjectFormatAndDecref.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f);
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f);

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* TupleAndListFromArray.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n);
static CYTHON_INLINE PyObject* __Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n);
#endif

/* fastcall.proto */
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_Arg_VARARGS(args, i) PySequence_GetItem(args, i)
#elif CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_Arg_VARARGS(args, i) PyTuple_GET_ITEM(args, i)
#else
    #define __Pyx_Arg_VARARGS(args, i) PyTuple_GetItem(args, i)
#endif
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_Arg_NewRef_VARARGS(arg) __Pyx_NewRef(arg)
    #define __Pyx_Arg_XDECREF_VARARGS(arg) Py_XDECREF(arg)
#else
    #define __Pyx_Arg_NewRef_VARARGS(arg) arg
    #define __Pyx_Arg_XDECREF_VARARGS(arg)
#endif
#define __Pyx_NumKwargs_VARARGS(kwds) PyDict_Size(kwds)
#define __Pyx_KwValues_VARARGS(args, nargs) NULL
#define __Pyx_GetKwValue_VARARGS(kw, kwvalues, s) __Pyx_PyDict_GetItemStrWithError(kw, s)
#define __Pyx_KwargsAsDict_VARARGS(kw, kwvalues) PyDict_Copy(kw)
#if CYTHON_METH_FASTCALL
    #define __Pyx_Arg_FASTCALL(args, i) args[i]
    #define __Pyx_NumKwargs_FASTCALL(kwds) PyTuple_GET_SIZE(kwds)
    #define __Pyx_KwValues_FASTCALL(args, nargs) ((args) + (nargs))
    static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
    CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues);
  #else
    #define __Pyx_KwargsAsDict_FASTCALL(kw, kwvalues) _PyStack_AsDict(kwvalues, kw)
  #endif
    #define __Pyx_Arg_NewRef_FASTCALL(arg) arg  /* no-op, __Pyx_Arg_FASTCALL is direct and this needs
                                                   to have the same reference counting */
    #define __Pyx_Arg_XDECREF_FASTCALL(arg)
#else
    #define __Pyx_Arg_FASTCALL __Pyx_Arg_VARARGS
    #define __Pyx_NumKwargs_FASTCALL __Pyx_NumKwargs_VARARGS
    #define __Pyx_KwValues_FASTCALL __Pyx_KwValues_VARARGS
    #define __Pyx_GetKwValue_FASTCALL __Pyx_GetKwValue_VARARGS
    #define __Pyx_KwargsAsDict_FASTCALL __Pyx_KwargsAsDict_VARARGS
    #define __Pyx_Arg_NewRef_FASTCALL(arg) __Pyx_Arg_NewRef_VARARGS(arg)
    #define __Pyx_Arg_XDECREF_FASTCALL(arg) __Pyx_Arg_XDECREF_VARARGS(arg)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) __Pyx_PyTuple_FromArray(&__Pyx_Arg_VARARGS(args, start), stop - start)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) __Pyx_PyTuple_FromArray(&__Pyx_Arg_FASTCALL(args, start), stop - start)
#else
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) PyTuple_GetSlice(args, start, stop)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) PyTuple_GetSlice(args, start, stop)
#endif

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject *const *kwvalues,
    PyObject **argnames[],
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely(__Pyx_IS_TYPE(obj, type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportDottedModule.proto */
static PyObject *__Pyx_ImportDottedModule(PyObject *name, PyObject *parts_tuple);
#if PY_MAJOR_VERSION >= 3
static PyObject *__Pyx_ImportDottedModule_WalkParts(PyObject *module, PyObject *name, PyObject *parts_tuple);
#endif

/* AssertionsEnabled.proto */
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define __Pyx_init_assertions_enabled()  (0)
  #define __pyx_assertions_enabled()  (1)
#elif CYTHON_COMPILING_IN_LIMITED_API  ||  (CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030C0000)
  static int __pyx_assertions_enabled_flag;
  #define __pyx_assertions_enabled() (__pyx_assertions_enabled_flag)
  static int __Pyx_init_assertions_enabled(void) {
    PyObject *builtins, *debug, *debug_str;
    int flag;
    builtins = PyEval_GetBuiltins();
    if (!builtins) goto bad;
    debug_str = PyUnicode_FromStringAndSize("__debug__", 9);
    if (!debug_str) goto bad;
    debug = PyObject_GetItem(builtins, debug_str);
    Py_DECREF(debug_str);
    if (!debug) goto bad;
    flag = PyObject_IsTrue(debug);
    Py_DECREF(debug);
    if (flag == -1) goto bad;
    __pyx_assertions_enabled_flag = flag;
    return 0;
  bad:
    __pyx_assertions_enabled_flag = 1;
    return -1;
  }
#else
  #define __Pyx_init_assertions_enabled()  (0)
  #define __pyx_assertions_enabled()  (!Py_OptimizeFlag)
#endif

/* PyObjectFormatSimple.proto */
#if CYTHON_COMPILING_IN_PYPY
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#elif PY_MAJOR_VERSION < 3
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        likely(PyString_CheckExact(s)) ? PyUnicode_FromEncodedObject(s, NULL, "strict") :\
        PyObject_Format(s, f))
#elif CYTHON_USE_TYPE_SLOTS
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        likely(PyLong_CheckExact(s)) ? PyLong_Type.tp_repr(s) :\
        likely(PyFloat_CheckExact(s)) ? PyFloat_Type.tp_repr(s) :\
        PyObject_Format(s, f))
#else
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#endif

/* CIntToPyUnicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_signed_char(signed char value, Py_ssize_t width, char padding_char, char format_char);

/* CIntToPyUnicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_unsigned_char(unsigned char value, Py_ssize_t width, char padding_char, char format_char);

/* CIntToPyUnicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_short(short value, Py_ssize_t width, char padding_char, char format_char);

/* CIntToPyUnicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_unsigned_short(unsigned short value, Py_ssize_t width, char padding_char, char format_char);

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* CIntToPyUnicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_int(int value, Py_ssize_t width, char padding_char, char format_char);

/* CIntToPyUnicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_unsigned_int(unsigned int value, Py_ssize_t width, char padding_char, char format_char);

/* CIntToPyUnicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_PY_LONG_LONG(PY_LONG_LONG value, Py_ssize_t width, char padding_char, char format_char);

/* CIntToPyUnicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_unsigned_PY_LONG_LONG(unsigned PY_LONG_LONG value, Py_ssize_t width, char padding_char, char format_char);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto_3_0_11
#define __PYX_HAVE_RT_ImportType_proto_3_0_11
#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L
#include <stdalign.h>
#endif
#if (defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L) || __cplusplus >= 201103L
#define __PYX_GET_STRUCT_ALIGNMENT_3_0_11(s) alignof(s)
#else
#define __PYX_GET_STRUCT_ALIGNMENT_3_0_11(s) sizeof(void*)
#endif
enum __Pyx_ImportType_CheckSize_3_0_11 {
   __Pyx_ImportType_CheckSize_Error_3_0_11 = 0,
   __Pyx_ImportType_CheckSize_Warn_3_0_11 = 1,
   __Pyx_ImportType_CheckSize_Ignore_3_0_11 = 2
};
static PyTypeObject *__Pyx_ImportType_3_0_11(PyObject* module, const char *module_name, const char *class_name, size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_0_11 check_size);
#endif

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* IncludeStructmemberH.proto */
#include <structmember.h>

/* FixUpExtensionType.proto */
#if CYTHON_USE_TYPE_SPECS
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type);
#endif

/* FetchSharedCythonModule.proto */
static PyObject *__Pyx_FetchSharedCythonABIModule(void);

/* FetchCommonType.proto */
#if !CYTHON_USE_TYPE_SPECS
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);
#else
static PyTypeObject* __Pyx_FetchCommonTypeFromSpec(PyObject *module, PyType_Spec *spec, PyObject *bases);
#endif

/* PyMethodNew.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    PyObject *typesModule=NULL, *methodType=NULL, *result=NULL;
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    typesModule = PyImport_ImportModule("types");
    if (!typesModule) return NULL;
    methodType = PyObject_GetAttrString(typesModule, "MethodType");
    Py_DECREF(typesModule);
    if (!methodType) return NULL;
    result = PyObject_CallFunctionObjArgs(methodType, func, self, NULL);
    Py_DECREF(methodType);
    return result;
}
#elif PY_MAJOR_VERSION >= 3
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    return PyMethod_New(func, self);
}
#else
    #define __Pyx_PyMethod_New PyMethod_New
#endif

/* PyVectorcallFastCallDict.proto */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw);
#endif

/* CythonFunctionShared.proto */
#define __Pyx_CyFunction_USED
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CYFUNCTION_COROUTINE     0x08
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_CyFunction_GetClassObj(f)\
      (((__pyx_CyFunctionObject *) (f))->func_classobj)
#else
  #define __Pyx_CyFunction_GetClassObj(f)\
      ((PyObject*) ((PyCMethodObject *) (f))->mm_class)
#endif
#define __Pyx_CyFunction_SetClassObj(f, classobj)\
    __Pyx__CyFunction_SetClassObj((__pyx_CyFunctionObject *) (f), (classobj))
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject_HEAD
    PyObject *func;
#elif PY_VERSION_HEX < 0x030900B1
    PyCFunctionObject func;
#else
    PyCMethodObject func;
#endif
#if CYTHON_BACKPORT_VECTORCALL
    __pyx_vectorcallfunc func_vectorcall;
#endif
#if PY_VERSION_HEX < 0x030500A0 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_classobj;
#endif
    void *defaults;
    int defaults_pyobjects;
    size_t defaults_size;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
    PyObject *func_is_coroutine;
} __pyx_CyFunctionObject;
#undef __Pyx_CyOrPyCFunction_Check
#define __Pyx_CyFunction_Check(obj)  __Pyx_TypeCheck(obj, __pyx_CyFunctionType)
#define __Pyx_CyOrPyCFunction_Check(obj)  __Pyx_TypeCheck2(obj, __pyx_CyFunctionType, &PyCFunction_Type)
#define __Pyx_CyFunction_CheckExact(obj)  __Pyx_IS_TYPE(obj, __pyx_CyFunctionType)
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void *cfunc);
#undef __Pyx_IsSameCFunction
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCyOrCFunction(func, cfunc)
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(PyObject *module);
#if CYTHON_METH_FASTCALL
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#if CYTHON_BACKPORT_VECTORCALL
#define __Pyx_CyFunction_func_vectorcall(f) (((__pyx_CyFunctionObject*)f)->func_vectorcall)
#else
#define __Pyx_CyFunction_func_vectorcall(f) (((PyCFunctionObject*)f)->vectorcall)
#endif
#endif

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
#if !CYTHON_COMPILING_IN_LIMITED_API
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);
#endif

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* RealImag.proto */
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if defined(__cplusplus) && CYTHON_CCOMPLEX\
        && (defined(_WIN32) || defined(__clang__) || (defined(__GNUC__) && (__GNUC__ >= 5 || __GNUC__ == 4 && __GNUC_MINOR__ >= 4 )) || __cplusplus >= 201103)
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
    #define __Pyx_c_eq_float(a, b)   ((a)==(b))
    #define __Pyx_c_sum_float(a, b)  ((a)+(b))
    #define __Pyx_c_diff_float(a, b) ((a)-(b))
    #define __Pyx_c_prod_float(a, b) ((a)*(b))
    #define __Pyx_c_quot_float(a, b) ((a)/(b))
    #define __Pyx_c_neg_float(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_float(z) ((z)==(float)0)
    #define __Pyx_c_conj_float(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (::std::abs(z))
        #define __Pyx_c_pow_float(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_float(z) ((z)==0)
    #define __Pyx_c_conj_float(z)    (conjf(z))
    #if 1
        #define __Pyx_c_abs_float(z)     (cabsf(z))
        #define __Pyx_c_pow_float(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
    #define __Pyx_c_eq_double(a, b)   ((a)==(b))
    #define __Pyx_c_sum_double(a, b)  ((a)+(b))
    #define __Pyx_c_diff_double(a, b) ((a)-(b))
    #define __Pyx_c_prod_double(a, b) ((a)*(b))
    #define __Pyx_c_quot_double(a, b) ((a)/(b))
    #define __Pyx_c_neg_double(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_double(z) ((z)==(double)0)
    #define __Pyx_c_conj_double(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (::std::abs(z))
        #define __Pyx_c_pow_double(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_double(z) ((z)==0)
    #define __Pyx_c_conj_double(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs_double(z)     (cabs(z))
        #define __Pyx_c_pow_double(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

/* Arithmetic.proto */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
    #define __Pyx_c_eq_long__double(a, b)   ((a)==(b))
    #define __Pyx_c_sum_long__double(a, b)  ((a)+(b))
    #define __Pyx_c_diff_long__double(a, b) ((a)-(b))
    #define __Pyx_c_prod_long__double(a, b) ((a)*(b))
    #define __Pyx_c_quot_long__double(a, b) ((a)/(b))
    #define __Pyx_c_neg_long__double(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero_long__double(z) ((z)==(long double)0)
    #define __Pyx_c_conj_long__double(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs_long__double(z)     (::std::abs(z))
        #define __Pyx_c_pow_long__double(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero_long__double(z) ((z)==0)
    #define __Pyx_c_conj_long__double(z)    (conjl(z))
    #if 1
        #define __Pyx_c_abs_long__double(z)     (cabsl(z))
        #define __Pyx_c_pow_long__double(a, b)  (cpowl(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq_long__double(__pyx_t_long_double_complex, __pyx_t_long_double_complex);
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_sum_long__double(__pyx_t_long_double_complex, __pyx_t_long_double_complex);
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_diff_long__double(__pyx_t_long_double_complex, __pyx_t_long_double_complex);
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_prod_long__double(__pyx_t_long_double_complex, __pyx_t_long_double_complex);
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_quot_long__double(__pyx_t_long_double_complex, __pyx_t_long_double_complex);
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_neg_long__double(__pyx_t_long_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero_long__double(__pyx_t_long_double_complex);
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_conj_long__double(__pyx_t_long_double_complex);
    #if 1
        static CYTHON_INLINE long double __Pyx_c_abs_long__double(__pyx_t_long_double_complex);
        static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_pow_long__double(__pyx_t_long_double_complex, __pyx_t_long_double_complex);
    #endif
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_char(unsigned char value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_PY_LONG_LONG(unsigned PY_LONG_LONG value);

/* CIntFromPy.proto */
static CYTHON_INLINE unsigned PY_LONG_LONG __Pyx_PyInt_As_unsigned_PY_LONG_LONG(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_PY_LONG_LONG(PY_LONG_LONG value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_char(char value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_signed_char(signed char value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_short(short value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_short(unsigned short value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_int(unsigned int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* FormatTypeName.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%U"
static __Pyx_TypeName __Pyx_PyType_GetName(PyTypeObject* tp);
#define __Pyx_DECREF_TypeName(obj) Py_XDECREF(obj)
#else
typedef const char *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%.200s"
#define __Pyx_PyType_GetName(tp) ((tp)->tp_name)
#define __Pyx_DECREF_TypeName(obj)
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) __Pyx_IsAnySubtype2(Py_TYPE(obj), (PyTypeObject *)type1, (PyTypeObject *)type2)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) (PyObject_TypeCheck(obj, (PyTypeObject *)type1) || PyObject_TypeCheck(obj, (PyTypeObject *)type2))
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyErr_ExceptionMatches2(err1, err2)  __Pyx_PyErr_GivenExceptionMatches2(__Pyx_PyErr_CurrentExceptionType(), err1, err2)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static unsigned long __Pyx_get_runtime_version(void);
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer);

/* FunctionExport.proto */
static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig);

/* VoidPtrImport.proto */
static int __Pyx_ImportVoidPtr_3_0_11(PyObject *module, const char *name, void **p, const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

/* #### Code section: module_declarations ### */
static CYTHON_INLINE npy_intp __pyx_f_5numpy_5dtype_8itemsize_itemsize(PyArray_Descr *__pyx_v_self); /* proto*/
static CYTHON_INLINE npy_intp __pyx_f_5numpy_5dtype_9alignment_alignment(PyArray_Descr *__pyx_v_self); /* proto*/
static CYTHON_INLINE PyObject *__pyx_f_5numpy_5dtype_6fields_fields(PyArray_Descr *__pyx_v_self); /* proto*/
static CYTHON_INLINE PyObject *__pyx_f_5numpy_5dtype_5names_names(PyArray_Descr *__pyx_v_self); /* proto*/
static CYTHON_INLINE PyArray_ArrayDescr *__pyx_f_5numpy_5dtype_8subarray_subarray(PyArray_Descr *__pyx_v_self); /* proto*/
static CYTHON_INLINE npy_uint64 __pyx_f_5numpy_5dtype_5flags_flags(PyArray_Descr *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_5numpy_9broadcast_7numiter_numiter(PyArrayMultiIterObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE npy_intp __pyx_f_5numpy_9broadcast_4size_size(PyArrayMultiIterObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE npy_intp __pyx_f_5numpy_9broadcast_5index_index(PyArrayMultiIterObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_5numpy_9broadcast_2nd_nd(PyArrayMultiIterObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE npy_intp *__pyx_f_5numpy_9broadcast_10dimensions_dimensions(PyArrayMultiIterObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE void **__pyx_f_5numpy_9broadcast_5iters_iters(PyArrayMultiIterObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE PyObject *__pyx_f_5numpy_7ndarray_4base_base(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE PyArray_Descr *__pyx_f_5numpy_7ndarray_5descr_descr(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_5numpy_7ndarray_4ndim_ndim(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE npy_intp *__pyx_f_5numpy_7ndarray_5shape_shape(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE npy_intp *__pyx_f_5numpy_7ndarray_7strides_strides(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE npy_intp __pyx_f_5numpy_7ndarray_4size_size(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE char *__pyx_f_5numpy_7ndarray_4data_data(PyArrayObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE double __pyx_f_7cpython_7complex_7complex_4real_real(PyComplexObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE double __pyx_f_7cpython_7complex_7complex_4imag_imag(PyComplexObject *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_7cpython_8datetime_4date_4year_year(PyDateTime_Date *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_7cpython_8datetime_4date_5month_month(PyDateTime_Date *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_7cpython_8datetime_4date_3day_day(PyDateTime_Date *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_7cpython_8datetime_4time_4hour_hour(PyDateTime_Time *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_7cpython_8datetime_4time_6minute_minute(PyDateTime_Time *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_7cpython_8datetime_4time_6second_second(PyDateTime_Time *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_7cpython_8datetime_4time_11microsecond_microsecond(PyDateTime_Time *__pyx_v_self); /* proto*/
static CYTHON_INLINE PyObject *__pyx_f_7cpython_8datetime_4time_6tzinfo_tzinfo(PyDateTime_Time *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_7cpython_8datetime_4time_4fold_fold(PyDateTime_Time *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_7cpython_8datetime_8datetime_4year_year(PyDateTime_DateTime *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_7cpython_8datetime_8datetime_5month_month(PyDateTime_DateTime *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_7cpython_8datetime_8datetime_3day_day(PyDateTime_DateTime *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_7cpython_8datetime_8datetime_4hour_hour(PyDateTime_DateTime *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_7cpython_8datetime_8datetime_6minute_minute(PyDateTime_DateTime *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_7cpython_8datetime_8datetime_6second_second(PyDateTime_DateTime *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_7cpython_8datetime_8datetime_11microsecond_microsecond(PyDateTime_DateTime *__pyx_v_self); /* proto*/
static CYTHON_INLINE PyObject *__pyx_f_7cpython_8datetime_8datetime_6tzinfo_tzinfo(PyDateTime_DateTime *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_7cpython_8datetime_8datetime_4fold_fold(PyDateTime_DateTime *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_7cpython_8datetime_9timedelta_3day_day(PyDateTime_Delta *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_7cpython_8datetime_9timedelta_6second_second(PyDateTime_Delta *__pyx_v_self); /* proto*/
static CYTHON_INLINE int __pyx_f_7cpython_8datetime_9timedelta_11microsecond_microsecond(PyDateTime_Delta *__pyx_v_self); /* proto*/

/* Module declarations from "libc.stddef" */

/* Module declarations from "cpython.unicode" */

/* Module declarations from "libc.limits" */

/* Module declarations from "serializor.unicode.ser" */
static PyObject **__pyx_vp_10serializor_7unicode_3ser_OPT_SERIALIZE_NUMPY = 0;
#define __pyx_v_10serializor_7unicode_3ser_OPT_SERIALIZE_NUMPY (*__pyx_vp_10serializor_7unicode_3ser_OPT_SERIALIZE_NUMPY)
static Py_UCS4 *__pyx_vp_10serializor_7unicode_3ser_UINT8_ENCODE_VALUE = 0;
#define __pyx_v_10serializor_7unicode_3ser_UINT8_ENCODE_VALUE (*__pyx_vp_10serializor_7unicode_3ser_UINT8_ENCODE_VALUE)
static Py_UCS4 *__pyx_vp_10serializor_7unicode_3ser_UINT16_ENCODE_VALUE = 0;
#define __pyx_v_10serializor_7unicode_3ser_UINT16_ENCODE_VALUE (*__pyx_vp_10serializor_7unicode_3ser_UINT16_ENCODE_VALUE)
static Py_UCS4 *__pyx_vp_10serializor_7unicode_3ser_UINT32_ENCODE_VALUE = 0;
#define __pyx_v_10serializor_7unicode_3ser_UINT32_ENCODE_VALUE (*__pyx_vp_10serializor_7unicode_3ser_UINT32_ENCODE_VALUE)
static Py_UCS4 *__pyx_vp_10serializor_7unicode_3ser_UINT64_ENCODE_VALUE = 0;
#define __pyx_v_10serializor_7unicode_3ser_UINT64_ENCODE_VALUE (*__pyx_vp_10serializor_7unicode_3ser_UINT64_ENCODE_VALUE)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_DATETIME_ID = 0;
#define __pyx_v_10serializor_7unicode_3ser_DATETIME_ID (*__pyx_vp_10serializor_7unicode_3ser_DATETIME_ID)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_DATE_ID = 0;
#define __pyx_v_10serializor_7unicode_3ser_DATE_ID (*__pyx_vp_10serializor_7unicode_3ser_DATE_ID)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_TIME_ID = 0;
#define __pyx_v_10serializor_7unicode_3ser_TIME_ID (*__pyx_vp_10serializor_7unicode_3ser_TIME_ID)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_TIMEDELTA_ID = 0;
#define __pyx_v_10serializor_7unicode_3ser_TIMEDELTA_ID (*__pyx_vp_10serializor_7unicode_3ser_TIMEDELTA_ID)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_STRUCT_TIME_ID = 0;
#define __pyx_v_10serializor_7unicode_3ser_STRUCT_TIME_ID (*__pyx_vp_10serializor_7unicode_3ser_STRUCT_TIME_ID)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_COMPLEX_ID = 0;
#define __pyx_v_10serializor_7unicode_3ser_COMPLEX_ID (*__pyx_vp_10serializor_7unicode_3ser_COMPLEX_ID)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_LIST_ID = 0;
#define __pyx_v_10serializor_7unicode_3ser_LIST_ID (*__pyx_vp_10serializor_7unicode_3ser_LIST_ID)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_TUPLE_ID = 0;
#define __pyx_v_10serializor_7unicode_3ser_TUPLE_ID (*__pyx_vp_10serializor_7unicode_3ser_TUPLE_ID)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_SET_ID = 0;
#define __pyx_v_10serializor_7unicode_3ser_SET_ID (*__pyx_vp_10serializor_7unicode_3ser_SET_ID)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_FROZENSET_ID = 0;
#define __pyx_v_10serializor_7unicode_3ser_FROZENSET_ID (*__pyx_vp_10serializor_7unicode_3ser_FROZENSET_ID)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_RANGE_ID = 0;
#define __pyx_v_10serializor_7unicode_3ser_RANGE_ID (*__pyx_vp_10serializor_7unicode_3ser_RANGE_ID)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_DICT_ID = 0;
#define __pyx_v_10serializor_7unicode_3ser_DICT_ID (*__pyx_vp_10serializor_7unicode_3ser_DICT_ID)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_DT64_ID = 0;
#define __pyx_v_10serializor_7unicode_3ser_DT64_ID (*__pyx_vp_10serializor_7unicode_3ser_DT64_ID)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_TD64_ID = 0;
#define __pyx_v_10serializor_7unicode_3ser_TD64_ID (*__pyx_vp_10serializor_7unicode_3ser_TD64_ID)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_NDARRAY_ID = 0;
#define __pyx_v_10serializor_7unicode_3ser_NDARRAY_ID (*__pyx_vp_10serializor_7unicode_3ser_NDARRAY_ID)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_NDARRAY_OBJECT_DT = 0;
#define __pyx_v_10serializor_7unicode_3ser_NDARRAY_OBJECT_DT (*__pyx_vp_10serializor_7unicode_3ser_NDARRAY_OBJECT_DT)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_NDARRAY_INT_DT = 0;
#define __pyx_v_10serializor_7unicode_3ser_NDARRAY_INT_DT (*__pyx_vp_10serializor_7unicode_3ser_NDARRAY_INT_DT)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_NDARRAY_UINT_DT = 0;
#define __pyx_v_10serializor_7unicode_3ser_NDARRAY_UINT_DT (*__pyx_vp_10serializor_7unicode_3ser_NDARRAY_UINT_DT)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_NDARRAY_FLOAT_DT = 0;
#define __pyx_v_10serializor_7unicode_3ser_NDARRAY_FLOAT_DT (*__pyx_vp_10serializor_7unicode_3ser_NDARRAY_FLOAT_DT)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_NDARRAY_BOOL_DT = 0;
#define __pyx_v_10serializor_7unicode_3ser_NDARRAY_BOOL_DT (*__pyx_vp_10serializor_7unicode_3ser_NDARRAY_BOOL_DT)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_NDARRAY_DT64_DT = 0;
#define __pyx_v_10serializor_7unicode_3ser_NDARRAY_DT64_DT (*__pyx_vp_10serializor_7unicode_3ser_NDARRAY_DT64_DT)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_NDARRAY_TD64_DT = 0;
#define __pyx_v_10serializor_7unicode_3ser_NDARRAY_TD64_DT (*__pyx_vp_10serializor_7unicode_3ser_NDARRAY_TD64_DT)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_NDARRAY_COMPLEX_DT = 0;
#define __pyx_v_10serializor_7unicode_3ser_NDARRAY_COMPLEX_DT (*__pyx_vp_10serializor_7unicode_3ser_NDARRAY_COMPLEX_DT)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_NDARRAY_BYTES_DT = 0;
#define __pyx_v_10serializor_7unicode_3ser_NDARRAY_BYTES_DT (*__pyx_vp_10serializor_7unicode_3ser_NDARRAY_BYTES_DT)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_NDARRAY_UNICODE_DT = 0;
#define __pyx_v_10serializor_7unicode_3ser_NDARRAY_UNICODE_DT (*__pyx_vp_10serializor_7unicode_3ser_NDARRAY_UNICODE_DT)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_NDARRAY_OBJECT_IDDT = 0;
#define __pyx_v_10serializor_7unicode_3ser_NDARRAY_OBJECT_IDDT (*__pyx_vp_10serializor_7unicode_3ser_NDARRAY_OBJECT_IDDT)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_NDARRAY_INT_IDDT = 0;
#define __pyx_v_10serializor_7unicode_3ser_NDARRAY_INT_IDDT (*__pyx_vp_10serializor_7unicode_3ser_NDARRAY_INT_IDDT)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_NDARRAY_UINT_IDDT = 0;
#define __pyx_v_10serializor_7unicode_3ser_NDARRAY_UINT_IDDT (*__pyx_vp_10serializor_7unicode_3ser_NDARRAY_UINT_IDDT)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_NDARRAY_FLOAT_IDDT = 0;
#define __pyx_v_10serializor_7unicode_3ser_NDARRAY_FLOAT_IDDT (*__pyx_vp_10serializor_7unicode_3ser_NDARRAY_FLOAT_IDDT)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_NDARRAY_BOOL_IDDT = 0;
#define __pyx_v_10serializor_7unicode_3ser_NDARRAY_BOOL_IDDT (*__pyx_vp_10serializor_7unicode_3ser_NDARRAY_BOOL_IDDT)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_NDARRAY_DT64_IDDT = 0;
#define __pyx_v_10serializor_7unicode_3ser_NDARRAY_DT64_IDDT (*__pyx_vp_10serializor_7unicode_3ser_NDARRAY_DT64_IDDT)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_NDARRAY_TD64_IDDT = 0;
#define __pyx_v_10serializor_7unicode_3ser_NDARRAY_TD64_IDDT (*__pyx_vp_10serializor_7unicode_3ser_NDARRAY_TD64_IDDT)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_NDARRAY_COMPLEX_IDDT = 0;
#define __pyx_v_10serializor_7unicode_3ser_NDARRAY_COMPLEX_IDDT (*__pyx_vp_10serializor_7unicode_3ser_NDARRAY_COMPLEX_IDDT)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_NDARRAY_BYTES_IDDT = 0;
#define __pyx_v_10serializor_7unicode_3ser_NDARRAY_BYTES_IDDT (*__pyx_vp_10serializor_7unicode_3ser_NDARRAY_BYTES_IDDT)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_NDARRAY_UNICODE_IDDT = 0;
#define __pyx_v_10serializor_7unicode_3ser_NDARRAY_UNICODE_IDDT (*__pyx_vp_10serializor_7unicode_3ser_NDARRAY_UNICODE_IDDT)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_PD_TIMESTAMP_ID = 0;
#define __pyx_v_10serializor_7unicode_3ser_PD_TIMESTAMP_ID (*__pyx_vp_10serializor_7unicode_3ser_PD_TIMESTAMP_ID)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_PD_TIMEDELTA_ID = 0;
#define __pyx_v_10serializor_7unicode_3ser_PD_TIMEDELTA_ID (*__pyx_vp_10serializor_7unicode_3ser_PD_TIMEDELTA_ID)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_SERIES_ID = 0;
#define __pyx_v_10serializor_7unicode_3ser_SERIES_ID (*__pyx_vp_10serializor_7unicode_3ser_SERIES_ID)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_DATETIMEINDEX_ID = 0;
#define __pyx_v_10serializor_7unicode_3ser_DATETIMEINDEX_ID (*__pyx_vp_10serializor_7unicode_3ser_DATETIMEINDEX_ID)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_TIMEDELTAINDEX_ID = 0;
#define __pyx_v_10serializor_7unicode_3ser_TIMEDELTAINDEX_ID (*__pyx_vp_10serializor_7unicode_3ser_TIMEDELTAINDEX_ID)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_DATAFRAME_ID = 0;
#define __pyx_v_10serializor_7unicode_3ser_DATAFRAME_ID (*__pyx_vp_10serializor_7unicode_3ser_DATAFRAME_ID)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_TRUE_VALUE = 0;
#define __pyx_v_10serializor_7unicode_3ser_TRUE_VALUE (*__pyx_vp_10serializor_7unicode_3ser_TRUE_VALUE)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_FALSE_VALUE = 0;
#define __pyx_v_10serializor_7unicode_3ser_FALSE_VALUE (*__pyx_vp_10serializor_7unicode_3ser_FALSE_VALUE)
static PyObject **__pyx_vp_10serializor_7unicode_3ser_NONE_VALUE = 0;
#define __pyx_v_10serializor_7unicode_3ser_NONE_VALUE (*__pyx_vp_10serializor_7unicode_3ser_NONE_VALUE)

/* Module declarations from "cython" */

/* Module declarations from "libc.string" */

/* Module declarations from "libc.stdio" */

/* Module declarations from "__builtin__" */

/* Module declarations from "cpython.type" */

/* Module declarations from "cpython.version" */

/* Module declarations from "cpython.exc" */

/* Module declarations from "cpython.module" */

/* Module declarations from "cpython.mem" */

/* Module declarations from "cpython.tuple" */

/* Module declarations from "cpython.list" */

/* Module declarations from "cpython.sequence" */

/* Module declarations from "cpython.mapping" */

/* Module declarations from "cpython.iterator" */

/* Module declarations from "cpython.number" */

/* Module declarations from "cpython.int" */

/* Module declarations from "__builtin__" */

/* Module declarations from "cpython.bool" */

/* Module declarations from "cpython.long" */

/* Module declarations from "cpython.float" */

/* Module declarations from "__builtin__" */

/* Module declarations from "cpython.complex" */

/* Module declarations from "cpython.string" */

/* Module declarations from "cpython.pyport" */

/* Module declarations from "cpython.dict" */

/* Module declarations from "cpython.instance" */

/* Module declarations from "cpython.function" */

/* Module declarations from "cpython.method" */

/* Module declarations from "cpython.weakref" */

/* Module declarations from "cpython.getargs" */

/* Module declarations from "cpython.pythread" */

/* Module declarations from "cpython.pystate" */

/* Module declarations from "cpython.cobject" */

/* Module declarations from "cpython.oldbuffer" */

/* Module declarations from "cpython.set" */

/* Module declarations from "cpython.buffer" */

/* Module declarations from "cpython.bytes" */

/* Module declarations from "cpython.pycapsule" */

/* Module declarations from "cpython.contextvars" */

/* Module declarations from "cpython" */

/* Module declarations from "cpython.object" */

/* Module declarations from "cpython.ref" */

/* Module declarations from "numpy" */

/* Module declarations from "numpy" */
static CYTHON_INLINE int __pyx_f_5numpy_import_array(void); /*proto*/
static CYTHON_INLINE int __pyx_f_5numpy_import_umath(void); /*proto*/

/* Module declarations from "datetime" */

/* Module declarations from "cpython.datetime" */
static CYTHON_INLINE void __pyx_f_7cpython_8datetime_import_datetime(void); /*proto*/
static CYTHON_INLINE PyDateTime_Date *__pyx_f_7cpython_8datetime_date_new(int, int, int); /*proto*/
static CYTHON_INLINE PyDateTime_Time *__pyx_f_7cpython_8datetime_time_new(int, int, int, int, PyObject *, struct __pyx_opt_args_7cpython_8datetime_time_new *__pyx_optional_args); /*proto*/
static CYTHON_INLINE PyDateTime_DateTime *__pyx_f_7cpython_8datetime_datetime_new(int, int, int, int, int, int, int, PyObject *, struct __pyx_opt_args_7cpython_8datetime_datetime_new *__pyx_optional_args); /*proto*/
static CYTHON_INLINE PyDateTime_Delta *__pyx_f_7cpython_8datetime_timedelta_new(int, int, int); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_7cpython_8datetime_timezone_new(PyObject *, struct __pyx_opt_args_7cpython_8datetime_timezone_new *__pyx_optional_args); /*proto*/

/* Module declarations from "serializor" */

/* Module declarations from "serializor.identifier" */
static char *__pyx_vp_10serializor_10identifier_STR = 0;
#define __pyx_v_10serializor_10identifier_STR (*__pyx_vp_10serializor_10identifier_STR)
static char *__pyx_vp_10serializor_10identifier_BOOL = 0;
#define __pyx_v_10serializor_10identifier_BOOL (*__pyx_vp_10serializor_10identifier_BOOL)
static char *__pyx_vp_10serializor_10identifier_FLOAT = 0;
#define __pyx_v_10serializor_10identifier_FLOAT (*__pyx_vp_10serializor_10identifier_FLOAT)
static char *__pyx_vp_10serializor_10identifier_INT = 0;
#define __pyx_v_10serializor_10identifier_INT (*__pyx_vp_10serializor_10identifier_INT)
static char *__pyx_vp_10serializor_10identifier_NONE = 0;
#define __pyx_v_10serializor_10identifier_NONE (*__pyx_vp_10serializor_10identifier_NONE)
static char *__pyx_vp_10serializor_10identifier_DATE = 0;
#define __pyx_v_10serializor_10identifier_DATE (*__pyx_vp_10serializor_10identifier_DATE)
static char *__pyx_vp_10serializor_10identifier_TIME = 0;
#define __pyx_v_10serializor_10identifier_TIME (*__pyx_vp_10serializor_10identifier_TIME)
static char *__pyx_vp_10serializor_10identifier_DATETIME = 0;
#define __pyx_v_10serializor_10identifier_DATETIME (*__pyx_vp_10serializor_10identifier_DATETIME)
static char *__pyx_vp_10serializor_10identifier_TIMEDELTA = 0;
#define __pyx_v_10serializor_10identifier_TIMEDELTA (*__pyx_vp_10serializor_10identifier_TIMEDELTA)
static char *__pyx_vp_10serializor_10identifier_STRUCT_TIME = 0;
#define __pyx_v_10serializor_10identifier_STRUCT_TIME (*__pyx_vp_10serializor_10identifier_STRUCT_TIME)
static char *__pyx_vp_10serializor_10identifier_DECIMAL = 0;
#define __pyx_v_10serializor_10identifier_DECIMAL (*__pyx_vp_10serializor_10identifier_DECIMAL)
static char *__pyx_vp_10serializor_10identifier_COMPLEX = 0;
#define __pyx_v_10serializor_10identifier_COMPLEX (*__pyx_vp_10serializor_10identifier_COMPLEX)
static char *__pyx_vp_10serializor_10identifier_BYTES = 0;
#define __pyx_v_10serializor_10identifier_BYTES (*__pyx_vp_10serializor_10identifier_BYTES)
static char *__pyx_vp_10serializor_10identifier_BYTEARRAY = 0;
#define __pyx_v_10serializor_10identifier_BYTEARRAY (*__pyx_vp_10serializor_10identifier_BYTEARRAY)
static char *__pyx_vp_10serializor_10identifier_LIST = 0;
#define __pyx_v_10serializor_10identifier_LIST (*__pyx_vp_10serializor_10identifier_LIST)
static char *__pyx_vp_10serializor_10identifier_TUPLE = 0;
#define __pyx_v_10serializor_10identifier_TUPLE (*__pyx_vp_10serializor_10identifier_TUPLE)
static char *__pyx_vp_10serializor_10identifier_SET = 0;
#define __pyx_v_10serializor_10identifier_SET (*__pyx_vp_10serializor_10identifier_SET)
static char *__pyx_vp_10serializor_10identifier_FROZENSET = 0;
#define __pyx_v_10serializor_10identifier_FROZENSET (*__pyx_vp_10serializor_10identifier_FROZENSET)
static char *__pyx_vp_10serializor_10identifier_RANGE = 0;
#define __pyx_v_10serializor_10identifier_RANGE (*__pyx_vp_10serializor_10identifier_RANGE)
static char *__pyx_vp_10serializor_10identifier_DICT = 0;
#define __pyx_v_10serializor_10identifier_DICT (*__pyx_vp_10serializor_10identifier_DICT)
static char *__pyx_vp_10serializor_10identifier_DATETIME64 = 0;
#define __pyx_v_10serializor_10identifier_DATETIME64 (*__pyx_vp_10serializor_10identifier_DATETIME64)
static char *__pyx_vp_10serializor_10identifier_TIMEDELTA64 = 0;
#define __pyx_v_10serializor_10identifier_TIMEDELTA64 (*__pyx_vp_10serializor_10identifier_TIMEDELTA64)
static char *__pyx_vp_10serializor_10identifier_NDARRAY = 0;
#define __pyx_v_10serializor_10identifier_NDARRAY (*__pyx_vp_10serializor_10identifier_NDARRAY)
static char *__pyx_vp_10serializor_10identifier_NDARRAY_OBJECT = 0;
#define __pyx_v_10serializor_10identifier_NDARRAY_OBJECT (*__pyx_vp_10serializor_10identifier_NDARRAY_OBJECT)
static char *__pyx_vp_10serializor_10identifier_NDARRAY_INT = 0;
#define __pyx_v_10serializor_10identifier_NDARRAY_INT (*__pyx_vp_10serializor_10identifier_NDARRAY_INT)
static char *__pyx_vp_10serializor_10identifier_NDARRAY_UINT = 0;
#define __pyx_v_10serializor_10identifier_NDARRAY_UINT (*__pyx_vp_10serializor_10identifier_NDARRAY_UINT)
static char *__pyx_vp_10serializor_10identifier_NDARRAY_FLOAT = 0;
#define __pyx_v_10serializor_10identifier_NDARRAY_FLOAT (*__pyx_vp_10serializor_10identifier_NDARRAY_FLOAT)
static char *__pyx_vp_10serializor_10identifier_NDARRAY_BOOL = 0;
#define __pyx_v_10serializor_10identifier_NDARRAY_BOOL (*__pyx_vp_10serializor_10identifier_NDARRAY_BOOL)
static char *__pyx_vp_10serializor_10identifier_NDARRAY_DT64 = 0;
#define __pyx_v_10serializor_10identifier_NDARRAY_DT64 (*__pyx_vp_10serializor_10identifier_NDARRAY_DT64)
static char *__pyx_vp_10serializor_10identifier_NDARRAY_TD64 = 0;
#define __pyx_v_10serializor_10identifier_NDARRAY_TD64 (*__pyx_vp_10serializor_10identifier_NDARRAY_TD64)
static char *__pyx_vp_10serializor_10identifier_NDARRAY_COMPLEX = 0;
#define __pyx_v_10serializor_10identifier_NDARRAY_COMPLEX (*__pyx_vp_10serializor_10identifier_NDARRAY_COMPLEX)
static char *__pyx_vp_10serializor_10identifier_NDARRAY_BYTES = 0;
#define __pyx_v_10serializor_10identifier_NDARRAY_BYTES (*__pyx_vp_10serializor_10identifier_NDARRAY_BYTES)
static char *__pyx_vp_10serializor_10identifier_NDARRAY_UNICODE = 0;
#define __pyx_v_10serializor_10identifier_NDARRAY_UNICODE (*__pyx_vp_10serializor_10identifier_NDARRAY_UNICODE)
static char *__pyx_vp_10serializor_10identifier_SERIES = 0;
#define __pyx_v_10serializor_10identifier_SERIES (*__pyx_vp_10serializor_10identifier_SERIES)
static char *__pyx_vp_10serializor_10identifier_DATAFRAME = 0;
#define __pyx_v_10serializor_10identifier_DATAFRAME (*__pyx_vp_10serializor_10identifier_DATAFRAME)
static char *__pyx_vp_10serializor_10identifier_PD_TIMESTAMP = 0;
#define __pyx_v_10serializor_10identifier_PD_TIMESTAMP (*__pyx_vp_10serializor_10identifier_PD_TIMESTAMP)
static char *__pyx_vp_10serializor_10identifier_PD_TIMEDELTA = 0;
#define __pyx_v_10serializor_10identifier_PD_TIMEDELTA (*__pyx_vp_10serializor_10identifier_PD_TIMEDELTA)
static char *__pyx_vp_10serializor_10identifier_DATETIMEINDEX = 0;
#define __pyx_v_10serializor_10identifier_DATETIMEINDEX (*__pyx_vp_10serializor_10identifier_DATETIMEINDEX)
static char *__pyx_vp_10serializor_10identifier_TIMEDELTAINDEX = 0;
#define __pyx_v_10serializor_10identifier_TIMEDELTAINDEX (*__pyx_vp_10serializor_10identifier_TIMEDELTAINDEX)
static int *__pyx_vp_10serializor_10identifier__duplicates_test = 0;
#define __pyx_v_10serializor_10identifier__duplicates_test (*__pyx_vp_10serializor_10identifier__duplicates_test)

/* Module declarations from "serializor.typeref" */
static PyObject **__pyx_vp_10serializor_7typeref_NONE = 0;
#define __pyx_v_10serializor_7typeref_NONE (*__pyx_vp_10serializor_7typeref_NONE)
static PyObject **__pyx_vp_10serializor_7typeref_DICT_KEYS = 0;
#define __pyx_v_10serializor_7typeref_DICT_KEYS (*__pyx_vp_10serializor_7typeref_DICT_KEYS)
static PyObject **__pyx_vp_10serializor_7typeref_DICT_VALUES = 0;
#define __pyx_v_10serializor_7typeref_DICT_VALUES (*__pyx_vp_10serializor_7typeref_DICT_VALUES)
static PyObject **__pyx_vp_10serializor_7typeref_DICT_ITEMS = 0;
#define __pyx_v_10serializor_7typeref_DICT_ITEMS (*__pyx_vp_10serializor_7typeref_DICT_ITEMS)
static PyObject **__pyx_vp_10serializor_7typeref_DECIMAL = 0;
#define __pyx_v_10serializor_7typeref_DECIMAL (*__pyx_vp_10serializor_7typeref_DECIMAL)
static PyObject **__pyx_vp_10serializor_7typeref_STRUCT_TIME = 0;
#define __pyx_v_10serializor_7typeref_STRUCT_TIME (*__pyx_vp_10serializor_7typeref_STRUCT_TIME)
static PyObject **__pyx_vp_10serializor_7typeref_DEQUE = 0;
#define __pyx_v_10serializor_7typeref_DEQUE (*__pyx_vp_10serializor_7typeref_DEQUE)
static PyObject **__pyx_vp_10serializor_7typeref_FLOAT16 = 0;
#define __pyx_v_10serializor_7typeref_FLOAT16 (*__pyx_vp_10serializor_7typeref_FLOAT16)
static PyObject **__pyx_vp_10serializor_7typeref_FLOAT32 = 0;
#define __pyx_v_10serializor_7typeref_FLOAT32 (*__pyx_vp_10serializor_7typeref_FLOAT32)
static PyObject **__pyx_vp_10serializor_7typeref_FLOAT64 = 0;
#define __pyx_v_10serializor_7typeref_FLOAT64 (*__pyx_vp_10serializor_7typeref_FLOAT64)
static PyObject **__pyx_vp_10serializor_7typeref_INT8 = 0;
#define __pyx_v_10serializor_7typeref_INT8 (*__pyx_vp_10serializor_7typeref_INT8)
static PyObject **__pyx_vp_10serializor_7typeref_INT16 = 0;
#define __pyx_v_10serializor_7typeref_INT16 (*__pyx_vp_10serializor_7typeref_INT16)
static PyObject **__pyx_vp_10serializor_7typeref_INT32 = 0;
#define __pyx_v_10serializor_7typeref_INT32 (*__pyx_vp_10serializor_7typeref_INT32)
static PyObject **__pyx_vp_10serializor_7typeref_INT64 = 0;
#define __pyx_v_10serializor_7typeref_INT64 (*__pyx_vp_10serializor_7typeref_INT64)
static PyObject **__pyx_vp_10serializor_7typeref_UINT8 = 0;
#define __pyx_v_10serializor_7typeref_UINT8 (*__pyx_vp_10serializor_7typeref_UINT8)
static PyObject **__pyx_vp_10serializor_7typeref_UINT16 = 0;
#define __pyx_v_10serializor_7typeref_UINT16 (*__pyx_vp_10serializor_7typeref_UINT16)
static PyObject **__pyx_vp_10serializor_7typeref_UINT32 = 0;
#define __pyx_v_10serializor_7typeref_UINT32 (*__pyx_vp_10serializor_7typeref_UINT32)
static PyObject **__pyx_vp_10serializor_7typeref_UINT64 = 0;
#define __pyx_v_10serializor_7typeref_UINT64 (*__pyx_vp_10serializor_7typeref_UINT64)
static PyObject **__pyx_vp_10serializor_7typeref_COMPLEX64 = 0;
#define __pyx_v_10serializor_7typeref_COMPLEX64 (*__pyx_vp_10serializor_7typeref_COMPLEX64)
static PyObject **__pyx_vp_10serializor_7typeref_COMPLEX128 = 0;
#define __pyx_v_10serializor_7typeref_COMPLEX128 (*__pyx_vp_10serializor_7typeref_COMPLEX128)
static PyObject **__pyx_vp_10serializor_7typeref_DATETIME64 = 0;
#define __pyx_v_10serializor_7typeref_DATETIME64 (*__pyx_vp_10serializor_7typeref_DATETIME64)
static PyObject **__pyx_vp_10serializor_7typeref_TIMEDELTA64 = 0;
#define __pyx_v_10serializor_7typeref_TIMEDELTA64 (*__pyx_vp_10serializor_7typeref_TIMEDELTA64)
static PyObject **__pyx_vp_10serializor_7typeref_STR_ = 0;
#define __pyx_v_10serializor_7typeref_STR_ (*__pyx_vp_10serializor_7typeref_STR_)
static PyObject **__pyx_vp_10serializor_7typeref_BOOL_ = 0;
#define __pyx_v_10serializor_7typeref_BOOL_ (*__pyx_vp_10serializor_7typeref_BOOL_)
static PyObject **__pyx_vp_10serializor_7typeref_BYTES_ = 0;
#define __pyx_v_10serializor_7typeref_BYTES_ (*__pyx_vp_10serializor_7typeref_BYTES_)
static PyObject **__pyx_vp_10serializor_7typeref_RECORD = 0;
#define __pyx_v_10serializor_7typeref_RECORD (*__pyx_vp_10serializor_7typeref_RECORD)
static PyObject **__pyx_vp_10serializor_7typeref_SERIES = 0;
#define __pyx_v_10serializor_7typeref_SERIES (*__pyx_vp_10serializor_7typeref_SERIES)
static PyObject **__pyx_vp_10serializor_7typeref_DATAFRAME = 0;
#define __pyx_v_10serializor_7typeref_DATAFRAME (*__pyx_vp_10serializor_7typeref_DATAFRAME)
static PyObject **__pyx_vp_10serializor_7typeref_PD_TIMESTAMP = 0;
#define __pyx_v_10serializor_7typeref_PD_TIMESTAMP (*__pyx_vp_10serializor_7typeref_PD_TIMESTAMP)
static PyObject **__pyx_vp_10serializor_7typeref_PD_TIMEDELTA = 0;
#define __pyx_v_10serializor_7typeref_PD_TIMEDELTA (*__pyx_vp_10serializor_7typeref_PD_TIMEDELTA)
static PyObject **__pyx_vp_10serializor_7typeref_DATETIMEINDEX = 0;
#define __pyx_v_10serializor_7typeref_DATETIMEINDEX (*__pyx_vp_10serializor_7typeref_DATETIMEINDEX)
static PyObject **__pyx_vp_10serializor_7typeref_TIMEDELTAINDEX = 0;
#define __pyx_v_10serializor_7typeref_TIMEDELTAINDEX (*__pyx_vp_10serializor_7typeref_TIMEDELTAINDEX)
static int *__pyx_vp_10serializor_7typeref_CYTIMES_AVAILABLE = 0;
#define __pyx_v_10serializor_7typeref_CYTIMES_AVAILABLE (*__pyx_vp_10serializor_7typeref_CYTIMES_AVAILABLE)
static PyObject **__pyx_vp_10serializor_7typeref_PYDT = 0;
#define __pyx_v_10serializor_7typeref_PYDT (*__pyx_vp_10serializor_7typeref_PYDT)
static PyObject **__pyx_vp_10serializor_7typeref_PDDT = 0;
#define __pyx_v_10serializor_7typeref_PDDT (*__pyx_vp_10serializor_7typeref_PDDT)

/* Module declarations from "serializor.utils" */
static PyObject **__pyx_vp_10serializor_5utils_AVAILABLE_TIMEZONES = 0;
#define __pyx_v_10serializor_5utils_AVAILABLE_TIMEZONES (*__pyx_vp_10serializor_5utils_AVAILABLE_TIMEZONES)
static CYTHON_INLINE PyObject *__pyx_f_10serializor_5utils_encode_str(PyObject *, char *); /*proto*/
static CYTHON_INLINE int __pyx_f_10serializor_5utils_arr_setitem_1d(PyArrayObject *, PyObject *, npy_intp); /*proto*/
static CYTHON_INLINE int __pyx_f_10serializor_5utils_arr_setitem_2d(PyArrayObject *, PyObject *, npy_intp, npy_intp); /*proto*/
static CYTHON_INLINE int __pyx_f_10serializor_5utils_arr_setitem_3d(PyArrayObject *, PyObject *, npy_intp, npy_intp, npy_intp); /*proto*/
static CYTHON_INLINE int __pyx_f_10serializor_5utils_arr_setitem_4d(PyArrayObject *, PyObject *, npy_intp, npy_intp, npy_intp, npy_intp); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_10serializor_5utils_map_nptime_unit_int2str(int); /*proto*/
static CYTHON_INLINE int __pyx_f_10serializor_5utils_map_nptime_unit_str2int(PyObject *); /*proto*/

/* Module declarations from "serializor.unicode.des" */
static CYTHON_INLINE unsigned char __pyx_f_10serializor_7unicode_3des_unpack_uint8(PyObject *, Py_ssize_t); /*proto*/
static CYTHON_INLINE unsigned short __pyx_f_10serializor_7unicode_3des_unpack_uint16(PyObject *, Py_ssize_t); /*proto*/
static CYTHON_INLINE unsigned int __pyx_f_10serializor_7unicode_3des_unpack_uint24(PyObject *, Py_ssize_t); /*proto*/
static CYTHON_INLINE unsigned int __pyx_f_10serializor_7unicode_3des_unpack_uint32(PyObject *, Py_ssize_t); /*proto*/
static CYTHON_INLINE unsigned PY_LONG_LONG __pyx_f_10serializor_7unicode_3des_unpack_uint64(PyObject *, Py_ssize_t); /*proto*/
static CYTHON_INLINE signed char __pyx_f_10serializor_7unicode_3des_unpack_int8(PyObject *, Py_ssize_t); /*proto*/
static CYTHON_INLINE short __pyx_f_10serializor_7unicode_3des_unpack_int16(PyObject *, Py_ssize_t); /*proto*/
static CYTHON_INLINE int __pyx_f_10serializor_7unicode_3des_unpack_int24(PyObject *, Py_ssize_t); /*proto*/
static CYTHON_INLINE int __pyx_f_10serializor_7unicode_3des_unpack_int32(PyObject *, Py_ssize_t); /*proto*/
static CYTHON_INLINE PY_LONG_LONG __pyx_f_10serializor_7unicode_3des_unpack_int64(PyObject *, Py_ssize_t); /*proto*/
static CYTHON_INLINE unsigned PY_LONG_LONG __pyx_f_10serializor_7unicode_3des_dec_encoded_int(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des_deserialize(PyObject *, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__orjson_loads(PyObject *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_str(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_int(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_float(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_bool(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_none(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_datetime(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_date(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_time(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_timedelta(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_struct_time(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_decimal(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_complex(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_bytes(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_bytearray(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_list(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_tuple(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_set(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_frozenset(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_range(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_dict(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_datetime64(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_timedelta64(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_ndarray(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_ndarray_object(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_ndarray_numeric(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_ndarray_nptime(PyObject *, Py_ssize_t *, int); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_ndarray_complex(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_ndarray_bytes(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_ndarray_unicode(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_pd_timestamp(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_pd_timedelta(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_series(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_datetime_index(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_timedelta_index(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_dataframe(PyObject *, Py_ssize_t *); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des__des_data(PyObject *, Py_ssize_t *); /*proto*/
/* #### Code section: typeinfo ### */
/* #### Code section: before_global_var ### */
#define __Pyx_MODULE_NAME "serializor.unicode.des"
extern int __pyx_module_is_main_serializor__unicode__des;
int __pyx_module_is_main_serializor__unicode__des = 0;

/* Implementation of "serializor.unicode.des" */
/* #### Code section: global_var ### */
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_chr;
static PyObject *__pyx_builtin_AssertionError;
static PyObject *__pyx_builtin_print;
static PyObject *__pyx_builtin_ImportError;
static PyObject *__pyx_builtin_RuntimeError;
/* #### Code section: string_decls ### */
static const char __pyx_k_B[] = "<B";
static const char __pyx_k_D[] = "D";
static const char __pyx_k_H[] = "<H";
static const char __pyx_k_I[] = "<I";
static const char __pyx_k_M[] = "M";
static const char __pyx_k_Q[] = "<Q";
static const char __pyx_k_W[] = "W";
static const char __pyx_k_Y[] = "Y";
static const char __pyx_k_b[] = "<b";
static const char __pyx_k_d[] = "d";
static const char __pyx_k_h[] = "h";
static const char __pyx_k_i[] = "<i";
static const char __pyx_k_m[] = "m";
static const char __pyx_k_q[] = "<q";
static const char __pyx_k_s[] = "s";
static const char __pyx_k__5[] = ".";
static const char __pyx_k__6[] = "\n";
static const char __pyx_k__7[] = "*";
static const char __pyx_k__8[] = "\r";
static const char __pyx_k_as[] = "as";
static const char __pyx_k_dt[] = "dt";
static const char __pyx_k_fs[] = "fs";
static const char __pyx_k_ms[] = "ms";
static const char __pyx_k_np[] = "np";
static const char __pyx_k_ns[] = "ns";
static const char __pyx_k_ps[] = "ps";
static const char __pyx_k_tz[] = "tz";
static const char __pyx_k_us[] = "us";
static const char __pyx_k_S_d[] = "S%d";
static const char __pyx_k_UTC[] = "UTC";
static const char __pyx_k_U_d[] = "U%d";
static const char __pyx_k__16[] = "\303\273";
static const char __pyx_k__18[] = "\303\274\303\274\000";
static const char __pyx_k__20[] = "\303\274\303\277\303\277";
static const char __pyx_k__22[] = "\303\275\000\000\001\000";
static const char __pyx_k__24[] = "\303\275\303\277\303\277\303\277\303\277";
static const char __pyx_k__26[] = "\303\276\000\000\000\000\001\000\000\000";
static const char __pyx_k__28[] = "\303\276\000\000\000\000\002\000\000\000";
static const char __pyx_k__47[] = "?";
static const char __pyx_k_all[] = "__all__";
static const char __pyx_k_b_2[] = "b";
static const char __pyx_k_chr[] = "chr";
static const char __pyx_k_enc[] = "enc";
static const char __pyx_k_end[] = "end";
static const char __pyx_k_eof[] = " / [eof] ";
static const char __pyx_k_h_2[] = "<h";
static const char __pyx_k_i_2[] = "i";
static const char __pyx_k_num[] = "num ";
static const char __pyx_k_str[] = "str";
static const char __pyx_k_val[] = "val ";
static const char __pyx_k_None[] = "None";
static const char __pyx_k_copy[] = "copy";
static const char __pyx_k_data[] = "data";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "name";
static const char __pyx_k_pack[] = "pack";
static const char __pyx_k_spec[] = "__spec__";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_vs_i[] = " vs i ";
static const char __pyx_k_Error[] = "\nError: ";
static const char __pyx_k_dtype[] = "dtype";
static const char __pyx_k_empty[] = "empty";
static const char __pyx_k_ljust[] = "ljust";
static const char __pyx_k_loads[] = "_loads";
static const char __pyx_k_num_2[] = "num";
static const char __pyx_k_numpy[] = "numpy";
static const char __pyx_k_print[] = "print";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_utils[] = "utils";
static const char __pyx_k_val_2[] = "val";
static const char __pyx_k_decode[] = "decode";
static const char __pyx_k_encode[] = " - encode: ";
static const char __pyx_k_errors[] = "errors";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_latin1[] = "latin1";
static const char __pyx_k_name_2[] = "__name__";
static const char __pyx_k_orjson[] = "orjson";
static const char __pyx_k_pass_s[] = "- pass: %s";
static const char __pyx_k_return[] = "return";
static const char __pyx_k_struct[] = "struct";
static const char __pyx_k_tzinfo[] = "tzinfo";
static const char __pyx_k_columns[] = "columns";
static const char __pyx_k_loads_2[] = "loads";
static const char __pyx_k_typeref[] = "typeref";
static const char __pyx_k_ZoneInfo[] = "ZoneInfo";
static const char __pyx_k_datetime[] = "datetime";
static const char __pyx_k_zoneinfo[] = "zoneinfo";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_identifier[] = "identifier";
static const char __pyx_k_serializor[] = "serializor";
static const char __pyx_k_test_utils[] = "_test_utils";
static const char __pyx_k_tz_convert[] = "tz_convert";
static const char __pyx_k_ImportError[] = "ImportError";
static const char __pyx_k_deserialize[] = "deserialize";
static const char __pyx_k_tz_localize[] = "tz_localize";
static const char __pyx_k_RuntimeError[] = "RuntimeError";
static const char __pyx_k_datetime64_s[] = "datetime64[%s]";
static const char __pyx_k_improper_pos[] = "improper [pos] ";
static const char __pyx_k_initializing[] = "_initializing";
static const char __pyx_k_is_coroutine[] = "_is_coroutine";
static const char __pyx_k_data_is_empty[] = "'data' is empty.";
static const char __pyx_k_timedelta64_s[] = "timedelta64[%s]";
static const char __pyx_k_AssertionError[] = "AssertionError";
static const char __pyx_k_DeserializeError[] = "DeserializeError";
static const char __pyx_k_Pass_Unpack_int8[] = "Pass Unpack int8";
static const char __pyx_k_test_unpack_int8[] = "_test_unpack_int8";
static const char __pyx_k_Pass_Unpack_int16[] = "Pass Unpack int16";
static const char __pyx_k_Pass_Unpack_int24[] = "Pass Unpack int24";
static const char __pyx_k_Pass_Unpack_int32[] = "Pass Unpack int32";
static const char __pyx_k_Pass_Unpack_int64[] = "Pass Unpack int64";
static const char __pyx_k_Pass_Unpack_uint8[] = "Pass Unpack uint8";
static const char __pyx_k_test_unpack_int16[] = "_test_unpack_int16";
static const char __pyx_k_test_unpack_int24[] = "_test_unpack_int24";
static const char __pyx_k_test_unpack_int32[] = "_test_unpack_int32";
static const char __pyx_k_test_unpack_int64[] = "_test_unpack_int64";
static const char __pyx_k_test_unpack_uint8[] = "_test_unpack_uint8";
static const char __pyx_k_Pass_Unpack_uint16[] = "Pass Unpack uint16";
static const char __pyx_k_Pass_Unpack_uint24[] = "Pass Unpack uint24";
static const char __pyx_k_Pass_Unpack_uint32[] = "Pass Unpack uint32";
static const char __pyx_k_Pass_Unpack_uint64[] = "Pass Unpack uint64";
static const char __pyx_k_asyncio_coroutines[] = "asyncio.coroutines";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_test_unpack_uint16[] = "_test_unpack_uint16";
static const char __pyx_k_test_unpack_uint24[] = "_test_unpack_uint24";
static const char __pyx_k_test_unpack_uint32[] = "_test_unpack_uint32";
static const char __pyx_k_test_unpack_uint64[] = "_test_unpack_uint64";
static const char __pyx_k_test_dec_encoded_int[] = "_test_dec_encoded_int";
static const char __pyx_k_serializor_unicode_des[] = "serializor.unicode.des";
static const char __pyx_k_Pass_Dec_Encoded_Integer[] = "Pass Dec Encoded Integer";
static const char __pyx_k_unknown_data_identifer_s[] = "unknown 'data' identifer '%s'.";
static const char __pyx_k_Unsupported_numpy_time_unit_d[] = "Unsupported numpy time unit: %d.";
static const char __pyx_k_Unsupported_numpy_time_unit_s[] = "Unsupported numpy time unit: %s.";
static const char __pyx_k_src_serializor_unicode_des_py[] = "src/serializor/unicode/des.py";
static const char __pyx_k_Invalid_encoded_integer_value_d[] = "Invalid encoded integer value: %d.";
static const char __pyx_k_Serializor_Failed_to_deserializ[] = "<'Serializor'>\nFailed to deserialize from: ";
static const char __pyx_k_unsupported_numpy_ndarray_dtype[] = "unsupported <'numpy.ndarray'> dtype '%s'.";
static const char __pyx_k_unsupported_pandas_Series_dtype[] = "unsupported <'pandas.Series'> dtype '%s'.";
static const char __pyx_k_Failed_to_parse_arr_time_unit_fr[] = "Failed to parse arr time unit from: '%s'.";
static const char __pyx_k_Time_zones_are_not_available_fro[] = "Time zones are not available from the C-API.";
static const char __pyx_k_numpy__core_multiarray_failed_to[] = "numpy._core.multiarray failed to import";
static const char __pyx_k_numpy__core_umath_failed_to_impo[] = "numpy._core.umath failed to import";
static const char __pyx_k_unsupported_numpy_ndarray_dimens[] = "unsupported <'numpy.ndarray'> dimension [%d].";
static const char __pyx_k_unsupported_pandas_DataFrame_col[] = "unsupported <'pandas.DataFrame'> column's dtype '%s'.";
/* #### Code section: decls ### */
static PyObject *__pyx_pf_10serializor_7unicode_3des_deserialize(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_10serializor_7unicode_3des_2_test_utils(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_10serializor_7unicode_3des_4_test_unpack_int8(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_10serializor_7unicode_3des_6_test_unpack_uint8(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_10serializor_7unicode_3des_8_test_unpack_int16(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_10serializor_7unicode_3des_10_test_unpack_uint16(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_10serializor_7unicode_3des_12_test_unpack_int24(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_10serializor_7unicode_3des_14_test_unpack_uint24(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_10serializor_7unicode_3des_16_test_unpack_int32(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_10serializor_7unicode_3des_18_test_unpack_uint32(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_10serializor_7unicode_3des_20_test_unpack_int64(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_10serializor_7unicode_3des_22_test_unpack_uint64(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_10serializor_7unicode_3des_24_test_dec_encoded_int(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
/* #### Code section: late_includes ### */
/* #### Code section: module_state ### */
typedef struct {
  PyObject *__pyx_d;
  PyObject *__pyx_b;
  PyObject *__pyx_cython_runtime;
  PyObject *__pyx_empty_tuple;
  PyObject *__pyx_empty_bytes;
  PyObject *__pyx_empty_unicode;
  #ifdef __Pyx_CyFunction_USED
  PyTypeObject *__pyx_CyFunctionType;
  #endif
  #ifdef __Pyx_FusedFunction_USED
  PyTypeObject *__pyx_FusedFunctionType;
  #endif
  #ifdef __Pyx_Generator_USED
  PyTypeObject *__pyx_GeneratorType;
  #endif
  #ifdef __Pyx_IterableCoroutine_USED
  PyTypeObject *__pyx_IterableCoroutineType;
  #endif
  #ifdef __Pyx_Coroutine_USED
  PyTypeObject *__pyx_CoroutineAwaitType;
  #endif
  #ifdef __Pyx_Coroutine_USED
  PyTypeObject *__pyx_CoroutineType;
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  PyTypeObject *__pyx_ptype_7cpython_4type_type;
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  PyTypeObject *__pyx_ptype_7cpython_4bool_bool;
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  PyTypeObject *__pyx_ptype_7cpython_7complex_complex;
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  PyTypeObject *__pyx_ptype_5numpy_dtype;
  PyTypeObject *__pyx_ptype_5numpy_flatiter;
  PyTypeObject *__pyx_ptype_5numpy_broadcast;
  PyTypeObject *__pyx_ptype_5numpy_ndarray;
  PyTypeObject *__pyx_ptype_5numpy_generic;
  PyTypeObject *__pyx_ptype_5numpy_number;
  PyTypeObject *__pyx_ptype_5numpy_integer;
  PyTypeObject *__pyx_ptype_5numpy_signedinteger;
  PyTypeObject *__pyx_ptype_5numpy_unsignedinteger;
  PyTypeObject *__pyx_ptype_5numpy_inexact;
  PyTypeObject *__pyx_ptype_5numpy_floating;
  PyTypeObject *__pyx_ptype_5numpy_complexfloating;
  PyTypeObject *__pyx_ptype_5numpy_flexible;
  PyTypeObject *__pyx_ptype_5numpy_character;
  PyTypeObject *__pyx_ptype_5numpy_ufunc;
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  PyTypeObject *__pyx_ptype_7cpython_8datetime_date;
  PyTypeObject *__pyx_ptype_7cpython_8datetime_time;
  PyTypeObject *__pyx_ptype_7cpython_8datetime_datetime;
  PyTypeObject *__pyx_ptype_7cpython_8datetime_timedelta;
  PyTypeObject *__pyx_ptype_7cpython_8datetime_tzinfo;
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  #if CYTHON_USE_MODULE_STATE
  #endif
  PyObject *__pyx_n_s_AssertionError;
  PyObject *__pyx_kp_u_B;
  PyObject *__pyx_n_u_D;
  PyObject *__pyx_n_s_DeserializeError;
  PyObject *__pyx_kp_u_Error;
  PyObject *__pyx_kp_u_Failed_to_parse_arr_time_unit_fr;
  PyObject *__pyx_kp_u_H;
  PyObject *__pyx_kp_u_I;
  PyObject *__pyx_n_s_ImportError;
  PyObject *__pyx_kp_u_Invalid_encoded_integer_value_d;
  PyObject *__pyx_n_u_M;
  PyObject *__pyx_n_s_None;
  PyObject *__pyx_kp_u_Pass_Dec_Encoded_Integer;
  PyObject *__pyx_kp_u_Pass_Unpack_int16;
  PyObject *__pyx_kp_u_Pass_Unpack_int24;
  PyObject *__pyx_kp_u_Pass_Unpack_int32;
  PyObject *__pyx_kp_u_Pass_Unpack_int64;
  PyObject *__pyx_kp_u_Pass_Unpack_int8;
  PyObject *__pyx_kp_u_Pass_Unpack_uint16;
  PyObject *__pyx_kp_u_Pass_Unpack_uint24;
  PyObject *__pyx_kp_u_Pass_Unpack_uint32;
  PyObject *__pyx_kp_u_Pass_Unpack_uint64;
  PyObject *__pyx_kp_u_Pass_Unpack_uint8;
  PyObject *__pyx_kp_u_Q;
  PyObject *__pyx_n_s_RuntimeError;
  PyObject *__pyx_kp_u_S_d;
  PyObject *__pyx_kp_u_Serializor_Failed_to_deserializ;
  PyObject *__pyx_kp_u_Time_zones_are_not_available_fro;
  PyObject *__pyx_n_u_UTC;
  PyObject *__pyx_kp_u_U_d;
  PyObject *__pyx_kp_u_Unsupported_numpy_time_unit_d;
  PyObject *__pyx_kp_u_Unsupported_numpy_time_unit_s;
  PyObject *__pyx_n_s_ValueError;
  PyObject *__pyx_n_u_W;
  PyObject *__pyx_n_u_Y;
  PyObject *__pyx_n_s_ZoneInfo;
  PyObject *__pyx_n_u__16;
  PyObject *__pyx_kp_u__18;
  PyObject *__pyx_n_u__20;
  PyObject *__pyx_kp_u__22;
  PyObject *__pyx_n_u__24;
  PyObject *__pyx_kp_u__26;
  PyObject *__pyx_kp_u__28;
  PyObject *__pyx_n_s__47;
  PyObject *__pyx_kp_u__5;
  PyObject *__pyx_kp_u__6;
  PyObject *__pyx_n_s__7;
  PyObject *__pyx_kp_u__8;
  PyObject *__pyx_n_s_all;
  PyObject *__pyx_n_u_as;
  PyObject *__pyx_n_s_asyncio_coroutines;
  PyObject *__pyx_kp_u_b;
  PyObject *__pyx_n_s_b_2;
  PyObject *__pyx_n_s_chr;
  PyObject *__pyx_n_s_cline_in_traceback;
  PyObject *__pyx_n_s_columns;
  PyObject *__pyx_n_s_copy;
  PyObject *__pyx_n_u_d;
  PyObject *__pyx_n_s_data;
  PyObject *__pyx_kp_u_data_is_empty;
  PyObject *__pyx_n_s_datetime;
  PyObject *__pyx_kp_u_datetime64_s;
  PyObject *__pyx_n_s_decode;
  PyObject *__pyx_n_s_deserialize;
  PyObject *__pyx_n_u_deserialize;
  PyObject *__pyx_n_s_dt;
  PyObject *__pyx_n_s_dtype;
  PyObject *__pyx_n_s_empty;
  PyObject *__pyx_n_s_enc;
  PyObject *__pyx_kp_u_encode;
  PyObject *__pyx_n_s_end;
  PyObject *__pyx_kp_u_eof;
  PyObject *__pyx_n_s_errors;
  PyObject *__pyx_n_u_fs;
  PyObject *__pyx_n_u_h;
  PyObject *__pyx_kp_u_h_2;
  PyObject *__pyx_kp_u_i;
  PyObject *__pyx_n_s_i_2;
  PyObject *__pyx_n_s_identifier;
  PyObject *__pyx_n_s_import;
  PyObject *__pyx_kp_u_improper_pos;
  PyObject *__pyx_n_s_initializing;
  PyObject *__pyx_n_s_is_coroutine;
  PyObject *__pyx_n_u_latin1;
  PyObject *__pyx_n_s_ljust;
  PyObject *__pyx_n_s_loads;
  PyObject *__pyx_n_s_loads_2;
  PyObject *__pyx_n_u_m;
  PyObject *__pyx_n_s_main;
  PyObject *__pyx_n_u_ms;
  PyObject *__pyx_n_s_name;
  PyObject *__pyx_n_s_name_2;
  PyObject *__pyx_n_s_np;
  PyObject *__pyx_n_u_ns;
  PyObject *__pyx_kp_u_num;
  PyObject *__pyx_n_s_num_2;
  PyObject *__pyx_n_s_numpy;
  PyObject *__pyx_kp_u_numpy__core_multiarray_failed_to;
  PyObject *__pyx_kp_u_numpy__core_umath_failed_to_impo;
  PyObject *__pyx_n_s_orjson;
  PyObject *__pyx_n_s_pack;
  PyObject *__pyx_kp_u_pass_s;
  PyObject *__pyx_n_s_print;
  PyObject *__pyx_n_u_ps;
  PyObject *__pyx_kp_u_q;
  PyObject *__pyx_n_s_range;
  PyObject *__pyx_n_s_return;
  PyObject *__pyx_n_u_s;
  PyObject *__pyx_n_s_serializor;
  PyObject *__pyx_n_s_serializor_unicode_des;
  PyObject *__pyx_n_s_spec;
  PyObject *__pyx_kp_s_src_serializor_unicode_des_py;
  PyObject *__pyx_n_s_str;
  PyObject *__pyx_n_s_struct;
  PyObject *__pyx_n_s_test;
  PyObject *__pyx_n_s_test_dec_encoded_int;
  PyObject *__pyx_n_s_test_unpack_int16;
  PyObject *__pyx_n_s_test_unpack_int24;
  PyObject *__pyx_n_s_test_unpack_int32;
  PyObject *__pyx_n_s_test_unpack_int64;
  PyObject *__pyx_n_s_test_unpack_int8;
  PyObject *__pyx_n_s_test_unpack_uint16;
  PyObject *__pyx_n_s_test_unpack_uint24;
  PyObject *__pyx_n_s_test_unpack_uint32;
  PyObject *__pyx_n_s_test_unpack_uint64;
  PyObject *__pyx_n_s_test_unpack_uint8;
  PyObject *__pyx_n_s_test_utils;
  PyObject *__pyx_kp_u_timedelta64_s;
  PyObject *__pyx_n_s_typeref;
  PyObject *__pyx_n_s_tz;
  PyObject *__pyx_n_s_tz_convert;
  PyObject *__pyx_n_s_tz_localize;
  PyObject *__pyx_n_s_tzinfo;
  PyObject *__pyx_kp_u_unknown_data_identifer_s;
  PyObject *__pyx_kp_u_unsupported_numpy_ndarray_dimens;
  PyObject *__pyx_kp_u_unsupported_numpy_ndarray_dtype;
  PyObject *__pyx_kp_u_unsupported_pandas_DataFrame_col;
  PyObject *__pyx_kp_u_unsupported_pandas_Series_dtype;
  PyObject *__pyx_n_u_us;
  PyObject *__pyx_n_s_utils;
  PyObject *__pyx_kp_u_val;
  PyObject *__pyx_n_s_val_2;
  PyObject *__pyx_kp_u_vs_i;
  PyObject *__pyx_n_s_zoneinfo;
  PyObject *__pyx_int_0;
  PyObject *__pyx_int_1;
  PyObject *__pyx_int_3;
  PyObject *__pyx_int_80;
  PyObject *__pyx_int_251;
  PyObject *__pyx_int_252;
  PyObject *__pyx_int_65535;
  PyObject *__pyx_int_65536;
  PyObject *__pyx_int_8388606;
  PyObject *__pyx_int_8388607;
  PyObject *__pyx_int_16777213;
  PyObject *__pyx_int_16777214;
  PyObject *__pyx_int_16777215;
  PyObject *__pyx_int_2147483646;
  PyObject *__pyx_int_2147483647;
  PyObject *__pyx_int_4294967293;
  PyObject *__pyx_int_4294967294;
  PyObject *__pyx_int_4294967295;
  PyObject *__pyx_int_4294967296;
  PyObject *__pyx_int_8589934592;
  PyObject *__pyx_int_0x7ffffffffffffffe;
  PyObject *__pyx_int_0x7fffffffffffffff;
  PyObject *__pyx_int_0xfffffffffffffffd;
  PyObject *__pyx_int_0xfffffffffffffffe;
  PyObject *__pyx_int_0xffffffffffffffff;
  PyObject *__pyx_int_neg_8388607;
  PyObject *__pyx_int_neg_8388608;
  PyObject *__pyx_int_neg_2147483647;
  PyObject *__pyx_int_neg_2147483648;
  PyObject *__pyx_int_neg_9223372036854775807;
  PyObject *__pyx_int_neg_9223372036854775808;
  PyObject *__pyx_tuple_;
  PyObject *__pyx_tuple__2;
  PyObject *__pyx_tuple__3;
  PyObject *__pyx_tuple__4;
  PyObject *__pyx_tuple__9;
  PyObject *__pyx_slice__10;
  PyObject *__pyx_tuple__11;
  PyObject *__pyx_tuple__12;
  PyObject *__pyx_tuple__13;
  PyObject *__pyx_tuple__14;
  PyObject *__pyx_tuple__15;
  PyObject *__pyx_tuple__17;
  PyObject *__pyx_tuple__19;
  PyObject *__pyx_tuple__21;
  PyObject *__pyx_tuple__23;
  PyObject *__pyx_tuple__25;
  PyObject *__pyx_tuple__27;
  PyObject *__pyx_tuple__29;
  PyObject *__pyx_tuple__30;
  PyObject *__pyx_tuple__31;
  PyObject *__pyx_tuple__34;
  PyObject *__pyx_tuple__45;
  PyObject *__pyx_codeobj__32;
  PyObject *__pyx_codeobj__33;
  PyObject *__pyx_codeobj__35;
  PyObject *__pyx_codeobj__36;
  PyObject *__pyx_codeobj__37;
  PyObject *__pyx_codeobj__38;
  PyObject *__pyx_codeobj__39;
  PyObject *__pyx_codeobj__40;
  PyObject *__pyx_codeobj__41;
  PyObject *__pyx_codeobj__42;
  PyObject *__pyx_codeobj__43;
  PyObject *__pyx_codeobj__44;
  PyObject *__pyx_codeobj__46;
} __pyx_mstate;

#if CYTHON_USE_MODULE_STATE
#ifdef __cplusplus
namespace {
  extern struct PyModuleDef __pyx_moduledef;
} /* anonymous namespace */
#else
static struct PyModuleDef __pyx_moduledef;
#endif

#define __pyx_mstate(o) ((__pyx_mstate *)__Pyx_PyModule_GetState(o))

#define __pyx_mstate_global (__pyx_mstate(PyState_FindModule(&__pyx_moduledef)))

#define __pyx_m (PyState_FindModule(&__pyx_moduledef))
#else
static __pyx_mstate __pyx_mstate_global_static =
#ifdef __cplusplus
    {};
#else
    {0};
#endif
static __pyx_mstate *__pyx_mstate_global = &__pyx_mstate_global_static;
#endif
/* #### Code section: module_state_clear ### */
#if CYTHON_USE_MODULE_STATE
static int __pyx_m_clear(PyObject *m) {
  __pyx_mstate *clear_module_state = __pyx_mstate(m);
  if (!clear_module_state) return 0;
  Py_CLEAR(clear_module_state->__pyx_d);
  Py_CLEAR(clear_module_state->__pyx_b);
  Py_CLEAR(clear_module_state->__pyx_cython_runtime);
  Py_CLEAR(clear_module_state->__pyx_empty_tuple);
  Py_CLEAR(clear_module_state->__pyx_empty_bytes);
  Py_CLEAR(clear_module_state->__pyx_empty_unicode);
  #ifdef __Pyx_CyFunction_USED
  Py_CLEAR(clear_module_state->__pyx_CyFunctionType);
  #endif
  #ifdef __Pyx_FusedFunction_USED
  Py_CLEAR(clear_module_state->__pyx_FusedFunctionType);
  #endif
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_4type_type);
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_4bool_bool);
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_7complex_complex);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_dtype);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_flatiter);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_broadcast);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_ndarray);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_generic);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_number);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_integer);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_signedinteger);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_unsignedinteger);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_inexact);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_floating);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_complexfloating);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_flexible);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_character);
  Py_CLEAR(clear_module_state->__pyx_ptype_5numpy_ufunc);
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_8datetime_date);
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_8datetime_time);
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_8datetime_datetime);
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_8datetime_timedelta);
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_8datetime_tzinfo);
  Py_CLEAR(clear_module_state->__pyx_n_s_AssertionError);
  Py_CLEAR(clear_module_state->__pyx_kp_u_B);
  Py_CLEAR(clear_module_state->__pyx_n_u_D);
  Py_CLEAR(clear_module_state->__pyx_n_s_DeserializeError);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Error);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Failed_to_parse_arr_time_unit_fr);
  Py_CLEAR(clear_module_state->__pyx_kp_u_H);
  Py_CLEAR(clear_module_state->__pyx_kp_u_I);
  Py_CLEAR(clear_module_state->__pyx_n_s_ImportError);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Invalid_encoded_integer_value_d);
  Py_CLEAR(clear_module_state->__pyx_n_u_M);
  Py_CLEAR(clear_module_state->__pyx_n_s_None);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Pass_Dec_Encoded_Integer);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Pass_Unpack_int16);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Pass_Unpack_int24);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Pass_Unpack_int32);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Pass_Unpack_int64);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Pass_Unpack_int8);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Pass_Unpack_uint16);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Pass_Unpack_uint24);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Pass_Unpack_uint32);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Pass_Unpack_uint64);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Pass_Unpack_uint8);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Q);
  Py_CLEAR(clear_module_state->__pyx_n_s_RuntimeError);
  Py_CLEAR(clear_module_state->__pyx_kp_u_S_d);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Serializor_Failed_to_deserializ);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Time_zones_are_not_available_fro);
  Py_CLEAR(clear_module_state->__pyx_n_u_UTC);
  Py_CLEAR(clear_module_state->__pyx_kp_u_U_d);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Unsupported_numpy_time_unit_d);
  Py_CLEAR(clear_module_state->__pyx_kp_u_Unsupported_numpy_time_unit_s);
  Py_CLEAR(clear_module_state->__pyx_n_s_ValueError);
  Py_CLEAR(clear_module_state->__pyx_n_u_W);
  Py_CLEAR(clear_module_state->__pyx_n_u_Y);
  Py_CLEAR(clear_module_state->__pyx_n_s_ZoneInfo);
  Py_CLEAR(clear_module_state->__pyx_n_u__16);
  Py_CLEAR(clear_module_state->__pyx_kp_u__18);
  Py_CLEAR(clear_module_state->__pyx_n_u__20);
  Py_CLEAR(clear_module_state->__pyx_kp_u__22);
  Py_CLEAR(clear_module_state->__pyx_n_u__24);
  Py_CLEAR(clear_module_state->__pyx_kp_u__26);
  Py_CLEAR(clear_module_state->__pyx_kp_u__28);
  Py_CLEAR(clear_module_state->__pyx_n_s__47);
  Py_CLEAR(clear_module_state->__pyx_kp_u__5);
  Py_CLEAR(clear_module_state->__pyx_kp_u__6);
  Py_CLEAR(clear_module_state->__pyx_n_s__7);
  Py_CLEAR(clear_module_state->__pyx_kp_u__8);
  Py_CLEAR(clear_module_state->__pyx_n_s_all);
  Py_CLEAR(clear_module_state->__pyx_n_u_as);
  Py_CLEAR(clear_module_state->__pyx_n_s_asyncio_coroutines);
  Py_CLEAR(clear_module_state->__pyx_kp_u_b);
  Py_CLEAR(clear_module_state->__pyx_n_s_b_2);
  Py_CLEAR(clear_module_state->__pyx_n_s_chr);
  Py_CLEAR(clear_module_state->__pyx_n_s_cline_in_traceback);
  Py_CLEAR(clear_module_state->__pyx_n_s_columns);
  Py_CLEAR(clear_module_state->__pyx_n_s_copy);
  Py_CLEAR(clear_module_state->__pyx_n_u_d);
  Py_CLEAR(clear_module_state->__pyx_n_s_data);
  Py_CLEAR(clear_module_state->__pyx_kp_u_data_is_empty);
  Py_CLEAR(clear_module_state->__pyx_n_s_datetime);
  Py_CLEAR(clear_module_state->__pyx_kp_u_datetime64_s);
  Py_CLEAR(clear_module_state->__pyx_n_s_decode);
  Py_CLEAR(clear_module_state->__pyx_n_s_deserialize);
  Py_CLEAR(clear_module_state->__pyx_n_u_deserialize);
  Py_CLEAR(clear_module_state->__pyx_n_s_dt);
  Py_CLEAR(clear_module_state->__pyx_n_s_dtype);
  Py_CLEAR(clear_module_state->__pyx_n_s_empty);
  Py_CLEAR(clear_module_state->__pyx_n_s_enc);
  Py_CLEAR(clear_module_state->__pyx_kp_u_encode);
  Py_CLEAR(clear_module_state->__pyx_n_s_end);
  Py_CLEAR(clear_module_state->__pyx_kp_u_eof);
  Py_CLEAR(clear_module_state->__pyx_n_s_errors);
  Py_CLEAR(clear_module_state->__pyx_n_u_fs);
  Py_CLEAR(clear_module_state->__pyx_n_u_h);
  Py_CLEAR(clear_module_state->__pyx_kp_u_h_2);
  Py_CLEAR(clear_module_state->__pyx_kp_u_i);
  Py_CLEAR(clear_module_state->__pyx_n_s_i_2);
  Py_CLEAR(clear_module_state->__pyx_n_s_identifier);
  Py_CLEAR(clear_module_state->__pyx_n_s_import);
  Py_CLEAR(clear_module_state->__pyx_kp_u_improper_pos);
  Py_CLEAR(clear_module_state->__pyx_n_s_initializing);
  Py_CLEAR(clear_module_state->__pyx_n_s_is_coroutine);
  Py_CLEAR(clear_module_state->__pyx_n_u_latin1);
  Py_CLEAR(clear_module_state->__pyx_n_s_ljust);
  Py_CLEAR(clear_module_state->__pyx_n_s_loads);
  Py_CLEAR(clear_module_state->__pyx_n_s_loads_2);
  Py_CLEAR(clear_module_state->__pyx_n_u_m);
  Py_CLEAR(clear_module_state->__pyx_n_s_main);
  Py_CLEAR(clear_module_state->__pyx_n_u_ms);
  Py_CLEAR(clear_module_state->__pyx_n_s_name);
  Py_CLEAR(clear_module_state->__pyx_n_s_name_2);
  Py_CLEAR(clear_module_state->__pyx_n_s_np);
  Py_CLEAR(clear_module_state->__pyx_n_u_ns);
  Py_CLEAR(clear_module_state->__pyx_kp_u_num);
  Py_CLEAR(clear_module_state->__pyx_n_s_num_2);
  Py_CLEAR(clear_module_state->__pyx_n_s_numpy);
  Py_CLEAR(clear_module_state->__pyx_kp_u_numpy__core_multiarray_failed_to);
  Py_CLEAR(clear_module_state->__pyx_kp_u_numpy__core_umath_failed_to_impo);
  Py_CLEAR(clear_module_state->__pyx_n_s_orjson);
  Py_CLEAR(clear_module_state->__pyx_n_s_pack);
  Py_CLEAR(clear_module_state->__pyx_kp_u_pass_s);
  Py_CLEAR(clear_module_state->__pyx_n_s_print);
  Py_CLEAR(clear_module_state->__pyx_n_u_ps);
  Py_CLEAR(clear_module_state->__pyx_kp_u_q);
  Py_CLEAR(clear_module_state->__pyx_n_s_range);
  Py_CLEAR(clear_module_state->__pyx_n_s_return);
  Py_CLEAR(clear_module_state->__pyx_n_u_s);
  Py_CLEAR(clear_module_state->__pyx_n_s_serializor);
  Py_CLEAR(clear_module_state->__pyx_n_s_serializor_unicode_des);
  Py_CLEAR(clear_module_state->__pyx_n_s_spec);
  Py_CLEAR(clear_module_state->__pyx_kp_s_src_serializor_unicode_des_py);
  Py_CLEAR(clear_module_state->__pyx_n_s_str);
  Py_CLEAR(clear_module_state->__pyx_n_s_struct);
  Py_CLEAR(clear_module_state->__pyx_n_s_test);
  Py_CLEAR(clear_module_state->__pyx_n_s_test_dec_encoded_int);
  Py_CLEAR(clear_module_state->__pyx_n_s_test_unpack_int16);
  Py_CLEAR(clear_module_state->__pyx_n_s_test_unpack_int24);
  Py_CLEAR(clear_module_state->__pyx_n_s_test_unpack_int32);
  Py_CLEAR(clear_module_state->__pyx_n_s_test_unpack_int64);
  Py_CLEAR(clear_module_state->__pyx_n_s_test_unpack_int8);
  Py_CLEAR(clear_module_state->__pyx_n_s_test_unpack_uint16);
  Py_CLEAR(clear_module_state->__pyx_n_s_test_unpack_uint24);
  Py_CLEAR(clear_module_state->__pyx_n_s_test_unpack_uint32);
  Py_CLEAR(clear_module_state->__pyx_n_s_test_unpack_uint64);
  Py_CLEAR(clear_module_state->__pyx_n_s_test_unpack_uint8);
  Py_CLEAR(clear_module_state->__pyx_n_s_test_utils);
  Py_CLEAR(clear_module_state->__pyx_kp_u_timedelta64_s);
  Py_CLEAR(clear_module_state->__pyx_n_s_typeref);
  Py_CLEAR(clear_module_state->__pyx_n_s_tz);
  Py_CLEAR(clear_module_state->__pyx_n_s_tz_convert);
  Py_CLEAR(clear_module_state->__pyx_n_s_tz_localize);
  Py_CLEAR(clear_module_state->__pyx_n_s_tzinfo);
  Py_CLEAR(clear_module_state->__pyx_kp_u_unknown_data_identifer_s);
  Py_CLEAR(clear_module_state->__pyx_kp_u_unsupported_numpy_ndarray_dimens);
  Py_CLEAR(clear_module_state->__pyx_kp_u_unsupported_numpy_ndarray_dtype);
  Py_CLEAR(clear_module_state->__pyx_kp_u_unsupported_pandas_DataFrame_col);
  Py_CLEAR(clear_module_state->__pyx_kp_u_unsupported_pandas_Series_dtype);
  Py_CLEAR(clear_module_state->__pyx_n_u_us);
  Py_CLEAR(clear_module_state->__pyx_n_s_utils);
  Py_CLEAR(clear_module_state->__pyx_kp_u_val);
  Py_CLEAR(clear_module_state->__pyx_n_s_val_2);
  Py_CLEAR(clear_module_state->__pyx_kp_u_vs_i);
  Py_CLEAR(clear_module_state->__pyx_n_s_zoneinfo);
  Py_CLEAR(clear_module_state->__pyx_int_0);
  Py_CLEAR(clear_module_state->__pyx_int_1);
  Py_CLEAR(clear_module_state->__pyx_int_3);
  Py_CLEAR(clear_module_state->__pyx_int_80);
  Py_CLEAR(clear_module_state->__pyx_int_251);
  Py_CLEAR(clear_module_state->__pyx_int_252);
  Py_CLEAR(clear_module_state->__pyx_int_65535);
  Py_CLEAR(clear_module_state->__pyx_int_65536);
  Py_CLEAR(clear_module_state->__pyx_int_8388606);
  Py_CLEAR(clear_module_state->__pyx_int_8388607);
  Py_CLEAR(clear_module_state->__pyx_int_16777213);
  Py_CLEAR(clear_module_state->__pyx_int_16777214);
  Py_CLEAR(clear_module_state->__pyx_int_16777215);
  Py_CLEAR(clear_module_state->__pyx_int_2147483646);
  Py_CLEAR(clear_module_state->__pyx_int_2147483647);
  Py_CLEAR(clear_module_state->__pyx_int_4294967293);
  Py_CLEAR(clear_module_state->__pyx_int_4294967294);
  Py_CLEAR(clear_module_state->__pyx_int_4294967295);
  Py_CLEAR(clear_module_state->__pyx_int_4294967296);
  Py_CLEAR(clear_module_state->__pyx_int_8589934592);
  Py_CLEAR(clear_module_state->__pyx_int_0x7ffffffffffffffe);
  Py_CLEAR(clear_module_state->__pyx_int_0x7fffffffffffffff);
  Py_CLEAR(clear_module_state->__pyx_int_0xfffffffffffffffd);
  Py_CLEAR(clear_module_state->__pyx_int_0xfffffffffffffffe);
  Py_CLEAR(clear_module_state->__pyx_int_0xffffffffffffffff);
  Py_CLEAR(clear_module_state->__pyx_int_neg_8388607);
  Py_CLEAR(clear_module_state->__pyx_int_neg_8388608);
  Py_CLEAR(clear_module_state->__pyx_int_neg_2147483647);
  Py_CLEAR(clear_module_state->__pyx_int_neg_2147483648);
  Py_CLEAR(clear_module_state->__pyx_int_neg_9223372036854775807);
  Py_CLEAR(clear_module_state->__pyx_int_neg_9223372036854775808);
  Py_CLEAR(clear_module_state->__pyx_tuple_);
  Py_CLEAR(clear_module_state->__pyx_tuple__2);
  Py_CLEAR(clear_module_state->__pyx_tuple__3);
  Py_CLEAR(clear_module_state->__pyx_tuple__4);
  Py_CLEAR(clear_module_state->__pyx_tuple__9);
  Py_CLEAR(clear_module_state->__pyx_slice__10);
  Py_CLEAR(clear_module_state->__pyx_tuple__11);
  Py_CLEAR(clear_module_state->__pyx_tuple__12);
  Py_CLEAR(clear_module_state->__pyx_tuple__13);
  Py_CLEAR(clear_module_state->__pyx_tuple__14);
  Py_CLEAR(clear_module_state->__pyx_tuple__15);
  Py_CLEAR(clear_module_state->__pyx_tuple__17);
  Py_CLEAR(clear_module_state->__pyx_tuple__19);
  Py_CLEAR(clear_module_state->__pyx_tuple__21);
  Py_CLEAR(clear_module_state->__pyx_tuple__23);
  Py_CLEAR(clear_module_state->__pyx_tuple__25);
  Py_CLEAR(clear_module_state->__pyx_tuple__27);
  Py_CLEAR(clear_module_state->__pyx_tuple__29);
  Py_CLEAR(clear_module_state->__pyx_tuple__30);
  Py_CLEAR(clear_module_state->__pyx_tuple__31);
  Py_CLEAR(clear_module_state->__pyx_tuple__34);
  Py_CLEAR(clear_module_state->__pyx_tuple__45);
  Py_CLEAR(clear_module_state->__pyx_codeobj__32);
  Py_CLEAR(clear_module_state->__pyx_codeobj__33);
  Py_CLEAR(clear_module_state->__pyx_codeobj__35);
  Py_CLEAR(clear_module_state->__pyx_codeobj__36);
  Py_CLEAR(clear_module_state->__pyx_codeobj__37);
  Py_CLEAR(clear_module_state->__pyx_codeobj__38);
  Py_CLEAR(clear_module_state->__pyx_codeobj__39);
  Py_CLEAR(clear_module_state->__pyx_codeobj__40);
  Py_CLEAR(clear_module_state->__pyx_codeobj__41);
  Py_CLEAR(clear_module_state->__pyx_codeobj__42);
  Py_CLEAR(clear_module_state->__pyx_codeobj__43);
  Py_CLEAR(clear_module_state->__pyx_codeobj__44);
  Py_CLEAR(clear_module_state->__pyx_codeobj__46);
  return 0;
}
#endif
/* #### Code section: module_state_traverse ### */
#if CYTHON_USE_MODULE_STATE
static int __pyx_m_traverse(PyObject *m, visitproc visit, void *arg) {
  __pyx_mstate *traverse_module_state = __pyx_mstate(m);
  if (!traverse_module_state) return 0;
  Py_VISIT(traverse_module_state->__pyx_d);
  Py_VISIT(traverse_module_state->__pyx_b);
  Py_VISIT(traverse_module_state->__pyx_cython_runtime);
  Py_VISIT(traverse_module_state->__pyx_empty_tuple);
  Py_VISIT(traverse_module_state->__pyx_empty_bytes);
  Py_VISIT(traverse_module_state->__pyx_empty_unicode);
  #ifdef __Pyx_CyFunction_USED
  Py_VISIT(traverse_module_state->__pyx_CyFunctionType);
  #endif
  #ifdef __Pyx_FusedFunction_USED
  Py_VISIT(traverse_module_state->__pyx_FusedFunctionType);
  #endif
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_4type_type);
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_4bool_bool);
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_7complex_complex);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_dtype);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_flatiter);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_broadcast);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_ndarray);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_generic);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_number);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_integer);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_signedinteger);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_unsignedinteger);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_inexact);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_floating);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_complexfloating);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_flexible);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_character);
  Py_VISIT(traverse_module_state->__pyx_ptype_5numpy_ufunc);
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_8datetime_date);
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_8datetime_time);
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_8datetime_datetime);
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_8datetime_timedelta);
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_8datetime_tzinfo);
  Py_VISIT(traverse_module_state->__pyx_n_s_AssertionError);
  Py_VISIT(traverse_module_state->__pyx_kp_u_B);
  Py_VISIT(traverse_module_state->__pyx_n_u_D);
  Py_VISIT(traverse_module_state->__pyx_n_s_DeserializeError);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Error);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Failed_to_parse_arr_time_unit_fr);
  Py_VISIT(traverse_module_state->__pyx_kp_u_H);
  Py_VISIT(traverse_module_state->__pyx_kp_u_I);
  Py_VISIT(traverse_module_state->__pyx_n_s_ImportError);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Invalid_encoded_integer_value_d);
  Py_VISIT(traverse_module_state->__pyx_n_u_M);
  Py_VISIT(traverse_module_state->__pyx_n_s_None);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Pass_Dec_Encoded_Integer);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Pass_Unpack_int16);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Pass_Unpack_int24);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Pass_Unpack_int32);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Pass_Unpack_int64);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Pass_Unpack_int8);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Pass_Unpack_uint16);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Pass_Unpack_uint24);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Pass_Unpack_uint32);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Pass_Unpack_uint64);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Pass_Unpack_uint8);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Q);
  Py_VISIT(traverse_module_state->__pyx_n_s_RuntimeError);
  Py_VISIT(traverse_module_state->__pyx_kp_u_S_d);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Serializor_Failed_to_deserializ);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Time_zones_are_not_available_fro);
  Py_VISIT(traverse_module_state->__pyx_n_u_UTC);
  Py_VISIT(traverse_module_state->__pyx_kp_u_U_d);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Unsupported_numpy_time_unit_d);
  Py_VISIT(traverse_module_state->__pyx_kp_u_Unsupported_numpy_time_unit_s);
  Py_VISIT(traverse_module_state->__pyx_n_s_ValueError);
  Py_VISIT(traverse_module_state->__pyx_n_u_W);
  Py_VISIT(traverse_module_state->__pyx_n_u_Y);
  Py_VISIT(traverse_module_state->__pyx_n_s_ZoneInfo);
  Py_VISIT(traverse_module_state->__pyx_n_u__16);
  Py_VISIT(traverse_module_state->__pyx_kp_u__18);
  Py_VISIT(traverse_module_state->__pyx_n_u__20);
  Py_VISIT(traverse_module_state->__pyx_kp_u__22);
  Py_VISIT(traverse_module_state->__pyx_n_u__24);
  Py_VISIT(traverse_module_state->__pyx_kp_u__26);
  Py_VISIT(traverse_module_state->__pyx_kp_u__28);
  Py_VISIT(traverse_module_state->__pyx_n_s__47);
  Py_VISIT(traverse_module_state->__pyx_kp_u__5);
  Py_VISIT(traverse_module_state->__pyx_kp_u__6);
  Py_VISIT(traverse_module_state->__pyx_n_s__7);
  Py_VISIT(traverse_module_state->__pyx_kp_u__8);
  Py_VISIT(traverse_module_state->__pyx_n_s_all);
  Py_VISIT(traverse_module_state->__pyx_n_u_as);
  Py_VISIT(traverse_module_state->__pyx_n_s_asyncio_coroutines);
  Py_VISIT(traverse_module_state->__pyx_kp_u_b);
  Py_VISIT(traverse_module_state->__pyx_n_s_b_2);
  Py_VISIT(traverse_module_state->__pyx_n_s_chr);
  Py_VISIT(traverse_module_state->__pyx_n_s_cline_in_traceback);
  Py_VISIT(traverse_module_state->__pyx_n_s_columns);
  Py_VISIT(traverse_module_state->__pyx_n_s_copy);
  Py_VISIT(traverse_module_state->__pyx_n_u_d);
  Py_VISIT(traverse_module_state->__pyx_n_s_data);
  Py_VISIT(traverse_module_state->__pyx_kp_u_data_is_empty);
  Py_VISIT(traverse_module_state->__pyx_n_s_datetime);
  Py_VISIT(traverse_module_state->__pyx_kp_u_datetime64_s);
  Py_VISIT(traverse_module_state->__pyx_n_s_decode);
  Py_VISIT(traverse_module_state->__pyx_n_s_deserialize);
  Py_VISIT(traverse_module_state->__pyx_n_u_deserialize);
  Py_VISIT(traverse_module_state->__pyx_n_s_dt);
  Py_VISIT(traverse_module_state->__pyx_n_s_dtype);
  Py_VISIT(traverse_module_state->__pyx_n_s_empty);
  Py_VISIT(traverse_module_state->__pyx_n_s_enc);
  Py_VISIT(traverse_module_state->__pyx_kp_u_encode);
  Py_VISIT(traverse_module_state->__pyx_n_s_end);
  Py_VISIT(traverse_module_state->__pyx_kp_u_eof);
  Py_VISIT(traverse_module_state->__pyx_n_s_errors);
  Py_VISIT(traverse_module_state->__pyx_n_u_fs);
  Py_VISIT(traverse_module_state->__pyx_n_u_h);
  Py_VISIT(traverse_module_state->__pyx_kp_u_h_2);
  Py_VISIT(traverse_module_state->__pyx_kp_u_i);
  Py_VISIT(traverse_module_state->__pyx_n_s_i_2);
  Py_VISIT(traverse_module_state->__pyx_n_s_identifier);
  Py_VISIT(traverse_module_state->__pyx_n_s_import);
  Py_VISIT(traverse_module_state->__pyx_kp_u_improper_pos);
  Py_VISIT(traverse_module_state->__pyx_n_s_initializing);
  Py_VISIT(traverse_module_state->__pyx_n_s_is_coroutine);
  Py_VISIT(traverse_module_state->__pyx_n_u_latin1);
  Py_VISIT(traverse_module_state->__pyx_n_s_ljust);
  Py_VISIT(traverse_module_state->__pyx_n_s_loads);
  Py_VISIT(traverse_module_state->__pyx_n_s_loads_2);
  Py_VISIT(traverse_module_state->__pyx_n_u_m);
  Py_VISIT(traverse_module_state->__pyx_n_s_main);
  Py_VISIT(traverse_module_state->__pyx_n_u_ms);
  Py_VISIT(traverse_module_state->__pyx_n_s_name);
  Py_VISIT(traverse_module_state->__pyx_n_s_name_2);
  Py_VISIT(traverse_module_state->__pyx_n_s_np);
  Py_VISIT(traverse_module_state->__pyx_n_u_ns);
  Py_VISIT(traverse_module_state->__pyx_kp_u_num);
  Py_VISIT(traverse_module_state->__pyx_n_s_num_2);
  Py_VISIT(traverse_module_state->__pyx_n_s_numpy);
  Py_VISIT(traverse_module_state->__pyx_kp_u_numpy__core_multiarray_failed_to);
  Py_VISIT(traverse_module_state->__pyx_kp_u_numpy__core_umath_failed_to_impo);
  Py_VISIT(traverse_module_state->__pyx_n_s_orjson);
  Py_VISIT(traverse_module_state->__pyx_n_s_pack);
  Py_VISIT(traverse_module_state->__pyx_kp_u_pass_s);
  Py_VISIT(traverse_module_state->__pyx_n_s_print);
  Py_VISIT(traverse_module_state->__pyx_n_u_ps);
  Py_VISIT(traverse_module_state->__pyx_kp_u_q);
  Py_VISIT(traverse_module_state->__pyx_n_s_range);
  Py_VISIT(traverse_module_state->__pyx_n_s_return);
  Py_VISIT(traverse_module_state->__pyx_n_u_s);
  Py_VISIT(traverse_module_state->__pyx_n_s_serializor);
  Py_VISIT(traverse_module_state->__pyx_n_s_serializor_unicode_des);
  Py_VISIT(traverse_module_state->__pyx_n_s_spec);
  Py_VISIT(traverse_module_state->__pyx_kp_s_src_serializor_unicode_des_py);
  Py_VISIT(traverse_module_state->__pyx_n_s_str);
  Py_VISIT(traverse_module_state->__pyx_n_s_struct);
  Py_VISIT(traverse_module_state->__pyx_n_s_test);
  Py_VISIT(traverse_module_state->__pyx_n_s_test_dec_encoded_int);
  Py_VISIT(traverse_module_state->__pyx_n_s_test_unpack_int16);
  Py_VISIT(traverse_module_state->__pyx_n_s_test_unpack_int24);
  Py_VISIT(traverse_module_state->__pyx_n_s_test_unpack_int32);
  Py_VISIT(traverse_module_state->__pyx_n_s_test_unpack_int64);
  Py_VISIT(traverse_module_state->__pyx_n_s_test_unpack_int8);
  Py_VISIT(traverse_module_state->__pyx_n_s_test_unpack_uint16);
  Py_VISIT(traverse_module_state->__pyx_n_s_test_unpack_uint24);
  Py_VISIT(traverse_module_state->__pyx_n_s_test_unpack_uint32);
  Py_VISIT(traverse_module_state->__pyx_n_s_test_unpack_uint64);
  Py_VISIT(traverse_module_state->__pyx_n_s_test_unpack_uint8);
  Py_VISIT(traverse_module_state->__pyx_n_s_test_utils);
  Py_VISIT(traverse_module_state->__pyx_kp_u_timedelta64_s);
  Py_VISIT(traverse_module_state->__pyx_n_s_typeref);
  Py_VISIT(traverse_module_state->__pyx_n_s_tz);
  Py_VISIT(traverse_module_state->__pyx_n_s_tz_convert);
  Py_VISIT(traverse_module_state->__pyx_n_s_tz_localize);
  Py_VISIT(traverse_module_state->__pyx_n_s_tzinfo);
  Py_VISIT(traverse_module_state->__pyx_kp_u_unknown_data_identifer_s);
  Py_VISIT(traverse_module_state->__pyx_kp_u_unsupported_numpy_ndarray_dimens);
  Py_VISIT(traverse_module_state->__pyx_kp_u_unsupported_numpy_ndarray_dtype);
  Py_VISIT(traverse_module_state->__pyx_kp_u_unsupported_pandas_DataFrame_col);
  Py_VISIT(traverse_module_state->__pyx_kp_u_unsupported_pandas_Series_dtype);
  Py_VISIT(traverse_module_state->__pyx_n_u_us);
  Py_VISIT(traverse_module_state->__pyx_n_s_utils);
  Py_VISIT(traverse_module_state->__pyx_kp_u_val);
  Py_VISIT(traverse_module_state->__pyx_n_s_val_2);
  Py_VISIT(traverse_module_state->__pyx_kp_u_vs_i);
  Py_VISIT(traverse_module_state->__pyx_n_s_zoneinfo);
  Py_VISIT(traverse_module_state->__pyx_int_0);
  Py_VISIT(traverse_module_state->__pyx_int_1);
  Py_VISIT(traverse_module_state->__pyx_int_3);
  Py_VISIT(traverse_module_state->__pyx_int_80);
  Py_VISIT(traverse_module_state->__pyx_int_251);
  Py_VISIT(traverse_module_state->__pyx_int_252);
  Py_VISIT(traverse_module_state->__pyx_int_65535);
  Py_VISIT(traverse_module_state->__pyx_int_65536);
  Py_VISIT(traverse_module_state->__pyx_int_8388606);
  Py_VISIT(traverse_module_state->__pyx_int_8388607);
  Py_VISIT(traverse_module_state->__pyx_int_16777213);
  Py_VISIT(traverse_module_state->__pyx_int_16777214);
  Py_VISIT(traverse_module_state->__pyx_int_16777215);
  Py_VISIT(traverse_module_state->__pyx_int_2147483646);
  Py_VISIT(traverse_module_state->__pyx_int_2147483647);
  Py_VISIT(traverse_module_state->__pyx_int_4294967293);
  Py_VISIT(traverse_module_state->__pyx_int_4294967294);
  Py_VISIT(traverse_module_state->__pyx_int_4294967295);
  Py_VISIT(traverse_module_state->__pyx_int_4294967296);
  Py_VISIT(traverse_module_state->__pyx_int_8589934592);
  Py_VISIT(traverse_module_state->__pyx_int_0x7ffffffffffffffe);
  Py_VISIT(traverse_module_state->__pyx_int_0x7fffffffffffffff);
  Py_VISIT(traverse_module_state->__pyx_int_0xfffffffffffffffd);
  Py_VISIT(traverse_module_state->__pyx_int_0xfffffffffffffffe);
  Py_VISIT(traverse_module_state->__pyx_int_0xffffffffffffffff);
  Py_VISIT(traverse_module_state->__pyx_int_neg_8388607);
  Py_VISIT(traverse_module_state->__pyx_int_neg_8388608);
  Py_VISIT(traverse_module_state->__pyx_int_neg_2147483647);
  Py_VISIT(traverse_module_state->__pyx_int_neg_2147483648);
  Py_VISIT(traverse_module_state->__pyx_int_neg_9223372036854775807);
  Py_VISIT(traverse_module_state->__pyx_int_neg_9223372036854775808);
  Py_VISIT(traverse_module_state->__pyx_tuple_);
  Py_VISIT(traverse_module_state->__pyx_tuple__2);
  Py_VISIT(traverse_module_state->__pyx_tuple__3);
  Py_VISIT(traverse_module_state->__pyx_tuple__4);
  Py_VISIT(traverse_module_state->__pyx_tuple__9);
  Py_VISIT(traverse_module_state->__pyx_slice__10);
  Py_VISIT(traverse_module_state->__pyx_tuple__11);
  Py_VISIT(traverse_module_state->__pyx_tuple__12);
  Py_VISIT(traverse_module_state->__pyx_tuple__13);
  Py_VISIT(traverse_module_state->__pyx_tuple__14);
  Py_VISIT(traverse_module_state->__pyx_tuple__15);
  Py_VISIT(traverse_module_state->__pyx_tuple__17);
  Py_VISIT(traverse_module_state->__pyx_tuple__19);
  Py_VISIT(traverse_module_state->__pyx_tuple__21);
  Py_VISIT(traverse_module_state->__pyx_tuple__23);
  Py_VISIT(traverse_module_state->__pyx_tuple__25);
  Py_VISIT(traverse_module_state->__pyx_tuple__27);
  Py_VISIT(traverse_module_state->__pyx_tuple__29);
  Py_VISIT(traverse_module_state->__pyx_tuple__30);
  Py_VISIT(traverse_module_state->__pyx_tuple__31);
  Py_VISIT(traverse_module_state->__pyx_tuple__34);
  Py_VISIT(traverse_module_state->__pyx_tuple__45);
  Py_VISIT(traverse_module_state->__pyx_codeobj__32);
  Py_VISIT(traverse_module_state->__pyx_codeobj__33);
  Py_VISIT(traverse_module_state->__pyx_codeobj__35);
  Py_VISIT(traverse_module_state->__pyx_codeobj__36);
  Py_VISIT(traverse_module_state->__pyx_codeobj__37);
  Py_VISIT(traverse_module_state->__pyx_codeobj__38);
  Py_VISIT(traverse_module_state->__pyx_codeobj__39);
  Py_VISIT(traverse_module_state->__pyx_codeobj__40);
  Py_VISIT(traverse_module_state->__pyx_codeobj__41);
  Py_VISIT(traverse_module_state->__pyx_codeobj__42);
  Py_VISIT(traverse_module_state->__pyx_codeobj__43);
  Py_VISIT(traverse_module_state->__pyx_codeobj__44);
  Py_VISIT(traverse_module_state->__pyx_codeobj__46);
  return 0;
}
#endif
/* #### Code section: module_state_defines ### */
#define __pyx_d __pyx_mstate_global->__pyx_d
#define __pyx_b __pyx_mstate_global->__pyx_b
#define __pyx_cython_runtime __pyx_mstate_global->__pyx_cython_runtime
#define __pyx_empty_tuple __pyx_mstate_global->__pyx_empty_tuple
#define __pyx_empty_bytes __pyx_mstate_global->__pyx_empty_bytes
#define __pyx_empty_unicode __pyx_mstate_global->__pyx_empty_unicode
#ifdef __Pyx_CyFunction_USED
#define __pyx_CyFunctionType __pyx_mstate_global->__pyx_CyFunctionType
#endif
#ifdef __Pyx_FusedFunction_USED
#define __pyx_FusedFunctionType __pyx_mstate_global->__pyx_FusedFunctionType
#endif
#ifdef __Pyx_Generator_USED
#define __pyx_GeneratorType __pyx_mstate_global->__pyx_GeneratorType
#endif
#ifdef __Pyx_IterableCoroutine_USED
#define __pyx_IterableCoroutineType __pyx_mstate_global->__pyx_IterableCoroutineType
#endif
#ifdef __Pyx_Coroutine_USED
#define __pyx_CoroutineAwaitType __pyx_mstate_global->__pyx_CoroutineAwaitType
#endif
#ifdef __Pyx_Coroutine_USED
#define __pyx_CoroutineType __pyx_mstate_global->__pyx_CoroutineType
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#define __pyx_ptype_7cpython_4type_type __pyx_mstate_global->__pyx_ptype_7cpython_4type_type
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#define __pyx_ptype_7cpython_4bool_bool __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#define __pyx_ptype_7cpython_7complex_complex __pyx_mstate_global->__pyx_ptype_7cpython_7complex_complex
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#define __pyx_ptype_5numpy_dtype __pyx_mstate_global->__pyx_ptype_5numpy_dtype
#define __pyx_ptype_5numpy_flatiter __pyx_mstate_global->__pyx_ptype_5numpy_flatiter
#define __pyx_ptype_5numpy_broadcast __pyx_mstate_global->__pyx_ptype_5numpy_broadcast
#define __pyx_ptype_5numpy_ndarray __pyx_mstate_global->__pyx_ptype_5numpy_ndarray
#define __pyx_ptype_5numpy_generic __pyx_mstate_global->__pyx_ptype_5numpy_generic
#define __pyx_ptype_5numpy_number __pyx_mstate_global->__pyx_ptype_5numpy_number
#define __pyx_ptype_5numpy_integer __pyx_mstate_global->__pyx_ptype_5numpy_integer
#define __pyx_ptype_5numpy_signedinteger __pyx_mstate_global->__pyx_ptype_5numpy_signedinteger
#define __pyx_ptype_5numpy_unsignedinteger __pyx_mstate_global->__pyx_ptype_5numpy_unsignedinteger
#define __pyx_ptype_5numpy_inexact __pyx_mstate_global->__pyx_ptype_5numpy_inexact
#define __pyx_ptype_5numpy_floating __pyx_mstate_global->__pyx_ptype_5numpy_floating
#define __pyx_ptype_5numpy_complexfloating __pyx_mstate_global->__pyx_ptype_5numpy_complexfloating
#define __pyx_ptype_5numpy_flexible __pyx_mstate_global->__pyx_ptype_5numpy_flexible
#define __pyx_ptype_5numpy_character __pyx_mstate_global->__pyx_ptype_5numpy_character
#define __pyx_ptype_5numpy_ufunc __pyx_mstate_global->__pyx_ptype_5numpy_ufunc
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#define __pyx_ptype_7cpython_8datetime_date __pyx_mstate_global->__pyx_ptype_7cpython_8datetime_date
#define __pyx_ptype_7cpython_8datetime_time __pyx_mstate_global->__pyx_ptype_7cpython_8datetime_time
#define __pyx_ptype_7cpython_8datetime_datetime __pyx_mstate_global->__pyx_ptype_7cpython_8datetime_datetime
#define __pyx_ptype_7cpython_8datetime_timedelta __pyx_mstate_global->__pyx_ptype_7cpython_8datetime_timedelta
#define __pyx_ptype_7cpython_8datetime_tzinfo __pyx_mstate_global->__pyx_ptype_7cpython_8datetime_tzinfo
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#if CYTHON_USE_MODULE_STATE
#endif
#define __pyx_n_s_AssertionError __pyx_mstate_global->__pyx_n_s_AssertionError
#define __pyx_kp_u_B __pyx_mstate_global->__pyx_kp_u_B
#define __pyx_n_u_D __pyx_mstate_global->__pyx_n_u_D
#define __pyx_n_s_DeserializeError __pyx_mstate_global->__pyx_n_s_DeserializeError
#define __pyx_kp_u_Error __pyx_mstate_global->__pyx_kp_u_Error
#define __pyx_kp_u_Failed_to_parse_arr_time_unit_fr __pyx_mstate_global->__pyx_kp_u_Failed_to_parse_arr_time_unit_fr
#define __pyx_kp_u_H __pyx_mstate_global->__pyx_kp_u_H
#define __pyx_kp_u_I __pyx_mstate_global->__pyx_kp_u_I
#define __pyx_n_s_ImportError __pyx_mstate_global->__pyx_n_s_ImportError
#define __pyx_kp_u_Invalid_encoded_integer_value_d __pyx_mstate_global->__pyx_kp_u_Invalid_encoded_integer_value_d
#define __pyx_n_u_M __pyx_mstate_global->__pyx_n_u_M
#define __pyx_n_s_None __pyx_mstate_global->__pyx_n_s_None
#define __pyx_kp_u_Pass_Dec_Encoded_Integer __pyx_mstate_global->__pyx_kp_u_Pass_Dec_Encoded_Integer
#define __pyx_kp_u_Pass_Unpack_int16 __pyx_mstate_global->__pyx_kp_u_Pass_Unpack_int16
#define __pyx_kp_u_Pass_Unpack_int24 __pyx_mstate_global->__pyx_kp_u_Pass_Unpack_int24
#define __pyx_kp_u_Pass_Unpack_int32 __pyx_mstate_global->__pyx_kp_u_Pass_Unpack_int32
#define __pyx_kp_u_Pass_Unpack_int64 __pyx_mstate_global->__pyx_kp_u_Pass_Unpack_int64
#define __pyx_kp_u_Pass_Unpack_int8 __pyx_mstate_global->__pyx_kp_u_Pass_Unpack_int8
#define __pyx_kp_u_Pass_Unpack_uint16 __pyx_mstate_global->__pyx_kp_u_Pass_Unpack_uint16
#define __pyx_kp_u_Pass_Unpack_uint24 __pyx_mstate_global->__pyx_kp_u_Pass_Unpack_uint24
#define __pyx_kp_u_Pass_Unpack_uint32 __pyx_mstate_global->__pyx_kp_u_Pass_Unpack_uint32
#define __pyx_kp_u_Pass_Unpack_uint64 __pyx_mstate_global->__pyx_kp_u_Pass_Unpack_uint64
#define __pyx_kp_u_Pass_Unpack_uint8 __pyx_mstate_global->__pyx_kp_u_Pass_Unpack_uint8
#define __pyx_kp_u_Q __pyx_mstate_global->__pyx_kp_u_Q
#define __pyx_n_s_RuntimeError __pyx_mstate_global->__pyx_n_s_RuntimeError
#define __pyx_kp_u_S_d __pyx_mstate_global->__pyx_kp_u_S_d
#define __pyx_kp_u_Serializor_Failed_to_deserializ __pyx_mstate_global->__pyx_kp_u_Serializor_Failed_to_deserializ
#define __pyx_kp_u_Time_zones_are_not_available_fro __pyx_mstate_global->__pyx_kp_u_Time_zones_are_not_available_fro
#define __pyx_n_u_UTC __pyx_mstate_global->__pyx_n_u_UTC
#define __pyx_kp_u_U_d __pyx_mstate_global->__pyx_kp_u_U_d
#define __pyx_kp_u_Unsupported_numpy_time_unit_d __pyx_mstate_global->__pyx_kp_u_Unsupported_numpy_time_unit_d
#define __pyx_kp_u_Unsupported_numpy_time_unit_s __pyx_mstate_global->__pyx_kp_u_Unsupported_numpy_time_unit_s
#define __pyx_n_s_ValueError __pyx_mstate_global->__pyx_n_s_ValueError
#define __pyx_n_u_W __pyx_mstate_global->__pyx_n_u_W
#define __pyx_n_u_Y __pyx_mstate_global->__pyx_n_u_Y
#define __pyx_n_s_ZoneInfo __pyx_mstate_global->__pyx_n_s_ZoneInfo
#define __pyx_n_u__16 __pyx_mstate_global->__pyx_n_u__16
#define __pyx_kp_u__18 __pyx_mstate_global->__pyx_kp_u__18
#define __pyx_n_u__20 __pyx_mstate_global->__pyx_n_u__20
#define __pyx_kp_u__22 __pyx_mstate_global->__pyx_kp_u__22
#define __pyx_n_u__24 __pyx_mstate_global->__pyx_n_u__24
#define __pyx_kp_u__26 __pyx_mstate_global->__pyx_kp_u__26
#define __pyx_kp_u__28 __pyx_mstate_global->__pyx_kp_u__28
#define __pyx_n_s__47 __pyx_mstate_global->__pyx_n_s__47
#define __pyx_kp_u__5 __pyx_mstate_global->__pyx_kp_u__5
#define __pyx_kp_u__6 __pyx_mstate_global->__pyx_kp_u__6
#define __pyx_n_s__7 __pyx_mstate_global->__pyx_n_s__7
#define __pyx_kp_u__8 __pyx_mstate_global->__pyx_kp_u__8
#define __pyx_n_s_all __pyx_mstate_global->__pyx_n_s_all
#define __pyx_n_u_as __pyx_mstate_global->__pyx_n_u_as
#define __pyx_n_s_asyncio_coroutines __pyx_mstate_global->__pyx_n_s_asyncio_coroutines
#define __pyx_kp_u_b __pyx_mstate_global->__pyx_kp_u_b
#define __pyx_n_s_b_2 __pyx_mstate_global->__pyx_n_s_b_2
#define __pyx_n_s_chr __pyx_mstate_global->__pyx_n_s_chr
#define __pyx_n_s_cline_in_traceback __pyx_mstate_global->__pyx_n_s_cline_in_traceback
#define __pyx_n_s_columns __pyx_mstate_global->__pyx_n_s_columns
#define __pyx_n_s_copy __pyx_mstate_global->__pyx_n_s_copy
#define __pyx_n_u_d __pyx_mstate_global->__pyx_n_u_d
#define __pyx_n_s_data __pyx_mstate_global->__pyx_n_s_data
#define __pyx_kp_u_data_is_empty __pyx_mstate_global->__pyx_kp_u_data_is_empty
#define __pyx_n_s_datetime __pyx_mstate_global->__pyx_n_s_datetime
#define __pyx_kp_u_datetime64_s __pyx_mstate_global->__pyx_kp_u_datetime64_s
#define __pyx_n_s_decode __pyx_mstate_global->__pyx_n_s_decode
#define __pyx_n_s_deserialize __pyx_mstate_global->__pyx_n_s_deserialize
#define __pyx_n_u_deserialize __pyx_mstate_global->__pyx_n_u_deserialize
#define __pyx_n_s_dt __pyx_mstate_global->__pyx_n_s_dt
#define __pyx_n_s_dtype __pyx_mstate_global->__pyx_n_s_dtype
#define __pyx_n_s_empty __pyx_mstate_global->__pyx_n_s_empty
#define __pyx_n_s_enc __pyx_mstate_global->__pyx_n_s_enc
#define __pyx_kp_u_encode __pyx_mstate_global->__pyx_kp_u_encode
#define __pyx_n_s_end __pyx_mstate_global->__pyx_n_s_end
#define __pyx_kp_u_eof __pyx_mstate_global->__pyx_kp_u_eof
#define __pyx_n_s_errors __pyx_mstate_global->__pyx_n_s_errors
#define __pyx_n_u_fs __pyx_mstate_global->__pyx_n_u_fs
#define __pyx_n_u_h __pyx_mstate_global->__pyx_n_u_h
#define __pyx_kp_u_h_2 __pyx_mstate_global->__pyx_kp_u_h_2
#define __pyx_kp_u_i __pyx_mstate_global->__pyx_kp_u_i
#define __pyx_n_s_i_2 __pyx_mstate_global->__pyx_n_s_i_2
#define __pyx_n_s_identifier __pyx_mstate_global->__pyx_n_s_identifier
#define __pyx_n_s_import __pyx_mstate_global->__pyx_n_s_import
#define __pyx_kp_u_improper_pos __pyx_mstate_global->__pyx_kp_u_improper_pos
#define __pyx_n_s_initializing __pyx_mstate_global->__pyx_n_s_initializing
#define __pyx_n_s_is_coroutine __pyx_mstate_global->__pyx_n_s_is_coroutine
#define __pyx_n_u_latin1 __pyx_mstate_global->__pyx_n_u_latin1
#define __pyx_n_s_ljust __pyx_mstate_global->__pyx_n_s_ljust
#define __pyx_n_s_loads __pyx_mstate_global->__pyx_n_s_loads
#define __pyx_n_s_loads_2 __pyx_mstate_global->__pyx_n_s_loads_2
#define __pyx_n_u_m __pyx_mstate_global->__pyx_n_u_m
#define __pyx_n_s_main __pyx_mstate_global->__pyx_n_s_main
#define __pyx_n_u_ms __pyx_mstate_global->__pyx_n_u_ms
#define __pyx_n_s_name __pyx_mstate_global->__pyx_n_s_name
#define __pyx_n_s_name_2 __pyx_mstate_global->__pyx_n_s_name_2
#define __pyx_n_s_np __pyx_mstate_global->__pyx_n_s_np
#define __pyx_n_u_ns __pyx_mstate_global->__pyx_n_u_ns
#define __pyx_kp_u_num __pyx_mstate_global->__pyx_kp_u_num
#define __pyx_n_s_num_2 __pyx_mstate_global->__pyx_n_s_num_2
#define __pyx_n_s_numpy __pyx_mstate_global->__pyx_n_s_numpy
#define __pyx_kp_u_numpy__core_multiarray_failed_to __pyx_mstate_global->__pyx_kp_u_numpy__core_multiarray_failed_to
#define __pyx_kp_u_numpy__core_umath_failed_to_impo __pyx_mstate_global->__pyx_kp_u_numpy__core_umath_failed_to_impo
#define __pyx_n_s_orjson __pyx_mstate_global->__pyx_n_s_orjson
#define __pyx_n_s_pack __pyx_mstate_global->__pyx_n_s_pack
#define __pyx_kp_u_pass_s __pyx_mstate_global->__pyx_kp_u_pass_s
#define __pyx_n_s_print __pyx_mstate_global->__pyx_n_s_print
#define __pyx_n_u_ps __pyx_mstate_global->__pyx_n_u_ps
#define __pyx_kp_u_q __pyx_mstate_global->__pyx_kp_u_q
#define __pyx_n_s_range __pyx_mstate_global->__pyx_n_s_range
#define __pyx_n_s_return __pyx_mstate_global->__pyx_n_s_return
#define __pyx_n_u_s __pyx_mstate_global->__pyx_n_u_s
#define __pyx_n_s_serializor __pyx_mstate_global->__pyx_n_s_serializor
#define __pyx_n_s_serializor_unicode_des __pyx_mstate_global->__pyx_n_s_serializor_unicode_des
#define __pyx_n_s_spec __pyx_mstate_global->__pyx_n_s_spec
#define __pyx_kp_s_src_serializor_unicode_des_py __pyx_mstate_global->__pyx_kp_s_src_serializor_unicode_des_py
#define __pyx_n_s_str __pyx_mstate_global->__pyx_n_s_str
#define __pyx_n_s_struct __pyx_mstate_global->__pyx_n_s_struct
#define __pyx_n_s_test __pyx_mstate_global->__pyx_n_s_test
#define __pyx_n_s_test_dec_encoded_int __pyx_mstate_global->__pyx_n_s_test_dec_encoded_int
#define __pyx_n_s_test_unpack_int16 __pyx_mstate_global->__pyx_n_s_test_unpack_int16
#define __pyx_n_s_test_unpack_int24 __pyx_mstate_global->__pyx_n_s_test_unpack_int24
#define __pyx_n_s_test_unpack_int32 __pyx_mstate_global->__pyx_n_s_test_unpack_int32
#define __pyx_n_s_test_unpack_int64 __pyx_mstate_global->__pyx_n_s_test_unpack_int64
#define __pyx_n_s_test_unpack_int8 __pyx_mstate_global->__pyx_n_s_test_unpack_int8
#define __pyx_n_s_test_unpack_uint16 __pyx_mstate_global->__pyx_n_s_test_unpack_uint16
#define __pyx_n_s_test_unpack_uint24 __pyx_mstate_global->__pyx_n_s_test_unpack_uint24
#define __pyx_n_s_test_unpack_uint32 __pyx_mstate_global->__pyx_n_s_test_unpack_uint32
#define __pyx_n_s_test_unpack_uint64 __pyx_mstate_global->__pyx_n_s_test_unpack_uint64
#define __pyx_n_s_test_unpack_uint8 __pyx_mstate_global->__pyx_n_s_test_unpack_uint8
#define __pyx_n_s_test_utils __pyx_mstate_global->__pyx_n_s_test_utils
#define __pyx_kp_u_timedelta64_s __pyx_mstate_global->__pyx_kp_u_timedelta64_s
#define __pyx_n_s_typeref __pyx_mstate_global->__pyx_n_s_typeref
#define __pyx_n_s_tz __pyx_mstate_global->__pyx_n_s_tz
#define __pyx_n_s_tz_convert __pyx_mstate_global->__pyx_n_s_tz_convert
#define __pyx_n_s_tz_localize __pyx_mstate_global->__pyx_n_s_tz_localize
#define __pyx_n_s_tzinfo __pyx_mstate_global->__pyx_n_s_tzinfo
#define __pyx_kp_u_unknown_data_identifer_s __pyx_mstate_global->__pyx_kp_u_unknown_data_identifer_s
#define __pyx_kp_u_unsupported_numpy_ndarray_dimens __pyx_mstate_global->__pyx_kp_u_unsupported_numpy_ndarray_dimens
#define __pyx_kp_u_unsupported_numpy_ndarray_dtype __pyx_mstate_global->__pyx_kp_u_unsupported_numpy_ndarray_dtype
#define __pyx_kp_u_unsupported_pandas_DataFrame_col __pyx_mstate_global->__pyx_kp_u_unsupported_pandas_DataFrame_col
#define __pyx_kp_u_unsupported_pandas_Series_dtype __pyx_mstate_global->__pyx_kp_u_unsupported_pandas_Series_dtype
#define __pyx_n_u_us __pyx_mstate_global->__pyx_n_u_us
#define __pyx_n_s_utils __pyx_mstate_global->__pyx_n_s_utils
#define __pyx_kp_u_val __pyx_mstate_global->__pyx_kp_u_val
#define __pyx_n_s_val_2 __pyx_mstate_global->__pyx_n_s_val_2
#define __pyx_kp_u_vs_i __pyx_mstate_global->__pyx_kp_u_vs_i
#define __pyx_n_s_zoneinfo __pyx_mstate_global->__pyx_n_s_zoneinfo
#define __pyx_int_0 __pyx_mstate_global->__pyx_int_0
#define __pyx_int_1 __pyx_mstate_global->__pyx_int_1
#define __pyx_int_3 __pyx_mstate_global->__pyx_int_3
#define __pyx_int_80 __pyx_mstate_global->__pyx_int_80
#define __pyx_int_251 __pyx_mstate_global->__pyx_int_251
#define __pyx_int_252 __pyx_mstate_global->__pyx_int_252
#define __pyx_int_65535 __pyx_mstate_global->__pyx_int_65535
#define __pyx_int_65536 __pyx_mstate_global->__pyx_int_65536
#define __pyx_int_8388606 __pyx_mstate_global->__pyx_int_8388606
#define __pyx_int_8388607 __pyx_mstate_global->__pyx_int_8388607
#define __pyx_int_16777213 __pyx_mstate_global->__pyx_int_16777213
#define __pyx_int_16777214 __pyx_mstate_global->__pyx_int_16777214
#define __pyx_int_16777215 __pyx_mstate_global->__pyx_int_16777215
#define __pyx_int_2147483646 __pyx_mstate_global->__pyx_int_2147483646
#define __pyx_int_2147483647 __pyx_mstate_global->__pyx_int_2147483647
#define __pyx_int_4294967293 __pyx_mstate_global->__pyx_int_4294967293
#define __pyx_int_4294967294 __pyx_mstate_global->__pyx_int_4294967294
#define __pyx_int_4294967295 __pyx_mstate_global->__pyx_int_4294967295
#define __pyx_int_4294967296 __pyx_mstate_global->__pyx_int_4294967296
#define __pyx_int_8589934592 __pyx_mstate_global->__pyx_int_8589934592
#define __pyx_int_0x7ffffffffffffffe __pyx_mstate_global->__pyx_int_0x7ffffffffffffffe
#define __pyx_int_0x7fffffffffffffff __pyx_mstate_global->__pyx_int_0x7fffffffffffffff
#define __pyx_int_0xfffffffffffffffd __pyx_mstate_global->__pyx_int_0xfffffffffffffffd
#define __pyx_int_0xfffffffffffffffe __pyx_mstate_global->__pyx_int_0xfffffffffffffffe
#define __pyx_int_0xffffffffffffffff __pyx_mstate_global->__pyx_int_0xffffffffffffffff
#define __pyx_int_neg_8388607 __pyx_mstate_global->__pyx_int_neg_8388607
#define __pyx_int_neg_8388608 __pyx_mstate_global->__pyx_int_neg_8388608
#define __pyx_int_neg_2147483647 __pyx_mstate_global->__pyx_int_neg_2147483647
#define __pyx_int_neg_2147483648 __pyx_mstate_global->__pyx_int_neg_2147483648
#define __pyx_int_neg_9223372036854775807 __pyx_mstate_global->__pyx_int_neg_9223372036854775807
#define __pyx_int_neg_9223372036854775808 __pyx_mstate_global->__pyx_int_neg_9223372036854775808
#define __pyx_tuple_ __pyx_mstate_global->__pyx_tuple_
#define __pyx_tuple__2 __pyx_mstate_global->__pyx_tuple__2
#define __pyx_tuple__3 __pyx_mstate_global->__pyx_tuple__3
#define __pyx_tuple__4 __pyx_mstate_global->__pyx_tuple__4
#define __pyx_tuple__9 __pyx_mstate_global->__pyx_tuple__9
#define __pyx_slice__10 __pyx_mstate_global->__pyx_slice__10
#define __pyx_tuple__11 __pyx_mstate_global->__pyx_tuple__11
#define __pyx_tuple__12 __pyx_mstate_global->__pyx_tuple__12
#define __pyx_tuple__13 __pyx_mstate_global->__pyx_tuple__13
#define __pyx_tuple__14 __pyx_mstate_global->__pyx_tuple__14
#define __pyx_tuple__15 __pyx_mstate_global->__pyx_tuple__15
#define __pyx_tuple__17 __pyx_mstate_global->__pyx_tuple__17
#define __pyx_tuple__19 __pyx_mstate_global->__pyx_tuple__19
#define __pyx_tuple__21 __pyx_mstate_global->__pyx_tuple__21
#define __pyx_tuple__23 __pyx_mstate_global->__pyx_tuple__23
#define __pyx_tuple__25 __pyx_mstate_global->__pyx_tuple__25
#define __pyx_tuple__27 __pyx_mstate_global->__pyx_tuple__27
#define __pyx_tuple__29 __pyx_mstate_global->__pyx_tuple__29
#define __pyx_tuple__30 __pyx_mstate_global->__pyx_tuple__30
#define __pyx_tuple__31 __pyx_mstate_global->__pyx_tuple__31
#define __pyx_tuple__34 __pyx_mstate_global->__pyx_tuple__34
#define __pyx_tuple__45 __pyx_mstate_global->__pyx_tuple__45
#define __pyx_codeobj__32 __pyx_mstate_global->__pyx_codeobj__32
#define __pyx_codeobj__33 __pyx_mstate_global->__pyx_codeobj__33
#define __pyx_codeobj__35 __pyx_mstate_global->__pyx_codeobj__35
#define __pyx_codeobj__36 __pyx_mstate_global->__pyx_codeobj__36
#define __pyx_codeobj__37 __pyx_mstate_global->__pyx_codeobj__37
#define __pyx_codeobj__38 __pyx_mstate_global->__pyx_codeobj__38
#define __pyx_codeobj__39 __pyx_mstate_global->__pyx_codeobj__39
#define __pyx_codeobj__40 __pyx_mstate_global->__pyx_codeobj__40
#define __pyx_codeobj__41 __pyx_mstate_global->__pyx_codeobj__41
#define __pyx_codeobj__42 __pyx_mstate_global->__pyx_codeobj__42
#define __pyx_codeobj__43 __pyx_mstate_global->__pyx_codeobj__43
#define __pyx_codeobj__44 __pyx_mstate_global->__pyx_codeobj__44
#define __pyx_codeobj__46 __pyx_mstate_global->__pyx_codeobj__46
/* #### Code section: module_code ### */

/* "serializor/unicode/ser.pxd":63
 * 
 * # Utils: Pack unsigned integers
 * cdef inline str pack_uint8(unsigned int value):             # <<<<<<<<<<<<<<
 *     """Pack `UNSIGNED` 8-bit integer in little-endian order to `<'str'>`.
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_10serializor_7unicode_3ser_pack_uint8(unsigned int __pyx_v_value) {
  char __pyx_v_buffer[1];
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pack_uint8", 1);

  /* "serializor/unicode/ser.pxd":70
 *     """
 *     cdef char buffer[1]
 *     buffer[0] = value & 0xFF             # <<<<<<<<<<<<<<
 *     return buffer[0:1].decode("latin1")
 * 
 */
  (__pyx_v_buffer[0]) = (__pyx_v_value & 0xFF);

  /* "serializor/unicode/ser.pxd":71
 *     cdef char buffer[1]
 *     buffer[0] = value & 0xFF
 *     return buffer[0:1].decode("latin1")             # <<<<<<<<<<<<<<
 * 
 * cdef inline str pack_uint16(unsigned int value):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_decode_c_string(__pyx_v_buffer, 0, 1, NULL, NULL, PyUnicode_DecodeLatin1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/ser.pxd":63
 * 
 * # Utils: Pack unsigned integers
 * cdef inline str pack_uint8(unsigned int value):             # <<<<<<<<<<<<<<
 *     """Pack `UNSIGNED` 8-bit integer in little-endian order to `<'str'>`.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("serializor.unicode.ser.pack_uint8", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/ser.pxd":73
 *     return buffer[0:1].decode("latin1")
 * 
 * cdef inline str pack_uint16(unsigned int value):             # <<<<<<<<<<<<<<
 *     """Pack `UNSIGNED` 16-bit integer in little-endian order to `<'str'>`.
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_10serializor_7unicode_3ser_pack_uint16(unsigned int __pyx_v_value) {
  char __pyx_v_buffer[2];
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pack_uint16", 1);

  /* "serializor/unicode/ser.pxd":80
 *     """
 *     cdef char buffer[2]
 *     buffer[0] = value & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[1] = (value >> 8) & 0xFF
 *     return buffer[0:2].decode("latin1")
 */
  (__pyx_v_buffer[0]) = (__pyx_v_value & 0xFF);

  /* "serializor/unicode/ser.pxd":81
 *     cdef char buffer[2]
 *     buffer[0] = value & 0xFF
 *     buffer[1] = (value >> 8) & 0xFF             # <<<<<<<<<<<<<<
 *     return buffer[0:2].decode("latin1")
 * 
 */
  (__pyx_v_buffer[1]) = ((__pyx_v_value >> 8) & 0xFF);

  /* "serializor/unicode/ser.pxd":82
 *     buffer[0] = value & 0xFF
 *     buffer[1] = (value >> 8) & 0xFF
 *     return buffer[0:2].decode("latin1")             # <<<<<<<<<<<<<<
 * 
 * cdef inline str pack_uint24(unsigned int value):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_decode_c_string(__pyx_v_buffer, 0, 2, NULL, NULL, PyUnicode_DecodeLatin1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/ser.pxd":73
 *     return buffer[0:1].decode("latin1")
 * 
 * cdef inline str pack_uint16(unsigned int value):             # <<<<<<<<<<<<<<
 *     """Pack `UNSIGNED` 16-bit integer in little-endian order to `<'str'>`.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("serializor.unicode.ser.pack_uint16", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/ser.pxd":84
 *     return buffer[0:2].decode("latin1")
 * 
 * cdef inline str pack_uint24(unsigned int value):             # <<<<<<<<<<<<<<
 *     """Pack `UNSIGNED` 24-bit integer in little-endian order to `<'str'>`.
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_10serializor_7unicode_3ser_pack_uint24(unsigned int __pyx_v_value) {
  char __pyx_v_buffer[3];
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pack_uint24", 1);

  /* "serializor/unicode/ser.pxd":91
 *     """
 *     cdef char buffer[3]
 *     buffer[0] = value & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[1] = (value >> 8) & 0xFF
 *     buffer[2] = (value >> 16) & 0xFF
 */
  (__pyx_v_buffer[0]) = (__pyx_v_value & 0xFF);

  /* "serializor/unicode/ser.pxd":92
 *     cdef char buffer[3]
 *     buffer[0] = value & 0xFF
 *     buffer[1] = (value >> 8) & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[2] = (value >> 16) & 0xFF
 *     return buffer[0:3].decode("latin1")
 */
  (__pyx_v_buffer[1]) = ((__pyx_v_value >> 8) & 0xFF);

  /* "serializor/unicode/ser.pxd":93
 *     buffer[0] = value & 0xFF
 *     buffer[1] = (value >> 8) & 0xFF
 *     buffer[2] = (value >> 16) & 0xFF             # <<<<<<<<<<<<<<
 *     return buffer[0:3].decode("latin1")
 * 
 */
  (__pyx_v_buffer[2]) = ((__pyx_v_value >> 16) & 0xFF);

  /* "serializor/unicode/ser.pxd":94
 *     buffer[1] = (value >> 8) & 0xFF
 *     buffer[2] = (value >> 16) & 0xFF
 *     return buffer[0:3].decode("latin1")             # <<<<<<<<<<<<<<
 * 
 * cdef inline str pack_uint32(unsigned long long value):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_decode_c_string(__pyx_v_buffer, 0, 3, NULL, NULL, PyUnicode_DecodeLatin1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/ser.pxd":84
 *     return buffer[0:2].decode("latin1")
 * 
 * cdef inline str pack_uint24(unsigned int value):             # <<<<<<<<<<<<<<
 *     """Pack `UNSIGNED` 24-bit integer in little-endian order to `<'str'>`.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("serializor.unicode.ser.pack_uint24", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/ser.pxd":96
 *     return buffer[0:3].decode("latin1")
 * 
 * cdef inline str pack_uint32(unsigned long long value):             # <<<<<<<<<<<<<<
 *     """Pack `UNSIGNED` 32-bit integer in little-endian order to `<'str'>`.
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_10serializor_7unicode_3ser_pack_uint32(unsigned PY_LONG_LONG __pyx_v_value) {
  char __pyx_v_buffer[4];
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pack_uint32", 1);

  /* "serializor/unicode/ser.pxd":103
 *     """
 *     cdef char buffer[4]
 *     buffer[0] = value & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[1] = (value >> 8) & 0xFF
 *     buffer[2] = (value >> 16) & 0xFF
 */
  (__pyx_v_buffer[0]) = (__pyx_v_value & 0xFF);

  /* "serializor/unicode/ser.pxd":104
 *     cdef char buffer[4]
 *     buffer[0] = value & 0xFF
 *     buffer[1] = (value >> 8) & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[2] = (value >> 16) & 0xFF
 *     buffer[3] = (value >> 24) & 0xFF
 */
  (__pyx_v_buffer[1]) = ((__pyx_v_value >> 8) & 0xFF);

  /* "serializor/unicode/ser.pxd":105
 *     buffer[0] = value & 0xFF
 *     buffer[1] = (value >> 8) & 0xFF
 *     buffer[2] = (value >> 16) & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[3] = (value >> 24) & 0xFF
 *     return buffer[0:4].decode("latin1")
 */
  (__pyx_v_buffer[2]) = ((__pyx_v_value >> 16) & 0xFF);

  /* "serializor/unicode/ser.pxd":106
 *     buffer[1] = (value >> 8) & 0xFF
 *     buffer[2] = (value >> 16) & 0xFF
 *     buffer[3] = (value >> 24) & 0xFF             # <<<<<<<<<<<<<<
 *     return buffer[0:4].decode("latin1")
 * 
 */
  (__pyx_v_buffer[3]) = ((__pyx_v_value >> 24) & 0xFF);

  /* "serializor/unicode/ser.pxd":107
 *     buffer[2] = (value >> 16) & 0xFF
 *     buffer[3] = (value >> 24) & 0xFF
 *     return buffer[0:4].decode("latin1")             # <<<<<<<<<<<<<<
 * 
 * cdef inline str pack_uint64(unsigned long long value):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_decode_c_string(__pyx_v_buffer, 0, 4, NULL, NULL, PyUnicode_DecodeLatin1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/ser.pxd":96
 *     return buffer[0:3].decode("latin1")
 * 
 * cdef inline str pack_uint32(unsigned long long value):             # <<<<<<<<<<<<<<
 *     """Pack `UNSIGNED` 32-bit integer in little-endian order to `<'str'>`.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("serializor.unicode.ser.pack_uint32", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/ser.pxd":109
 *     return buffer[0:4].decode("latin1")
 * 
 * cdef inline str pack_uint64(unsigned long long value):             # <<<<<<<<<<<<<<
 *     """Pack `UNSIGNED` 64-bit integer in little-endian order to `<'str'>`.
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_10serializor_7unicode_3ser_pack_uint64(unsigned PY_LONG_LONG __pyx_v_value) {
  char __pyx_v_buffer[8];
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pack_uint64", 1);

  /* "serializor/unicode/ser.pxd":116
 *     """
 *     cdef char buffer[8]
 *     buffer[0] = value & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[1] = (value >> 8) & 0xFF
 *     buffer[2] = (value >> 16) & 0xFF
 */
  (__pyx_v_buffer[0]) = (__pyx_v_value & 0xFF);

  /* "serializor/unicode/ser.pxd":117
 *     cdef char buffer[8]
 *     buffer[0] = value & 0xFF
 *     buffer[1] = (value >> 8) & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[2] = (value >> 16) & 0xFF
 *     buffer[3] = (value >> 24) & 0xFF
 */
  (__pyx_v_buffer[1]) = ((__pyx_v_value >> 8) & 0xFF);

  /* "serializor/unicode/ser.pxd":118
 *     buffer[0] = value & 0xFF
 *     buffer[1] = (value >> 8) & 0xFF
 *     buffer[2] = (value >> 16) & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[3] = (value >> 24) & 0xFF
 *     buffer[4] = (value >> 32) & 0xFF
 */
  (__pyx_v_buffer[2]) = ((__pyx_v_value >> 16) & 0xFF);

  /* "serializor/unicode/ser.pxd":119
 *     buffer[1] = (value >> 8) & 0xFF
 *     buffer[2] = (value >> 16) & 0xFF
 *     buffer[3] = (value >> 24) & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[4] = (value >> 32) & 0xFF
 *     buffer[5] = (value >> 40) & 0xFF
 */
  (__pyx_v_buffer[3]) = ((__pyx_v_value >> 24) & 0xFF);

  /* "serializor/unicode/ser.pxd":120
 *     buffer[2] = (value >> 16) & 0xFF
 *     buffer[3] = (value >> 24) & 0xFF
 *     buffer[4] = (value >> 32) & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[5] = (value >> 40) & 0xFF
 *     buffer[6] = (value >> 48) & 0xFF
 */
  (__pyx_v_buffer[4]) = ((__pyx_v_value >> 32) & 0xFF);

  /* "serializor/unicode/ser.pxd":121
 *     buffer[3] = (value >> 24) & 0xFF
 *     buffer[4] = (value >> 32) & 0xFF
 *     buffer[5] = (value >> 40) & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[6] = (value >> 48) & 0xFF
 *     buffer[7] = (value >> 56) & 0xFF
 */
  (__pyx_v_buffer[5]) = ((__pyx_v_value >> 40) & 0xFF);

  /* "serializor/unicode/ser.pxd":122
 *     buffer[4] = (value >> 32) & 0xFF
 *     buffer[5] = (value >> 40) & 0xFF
 *     buffer[6] = (value >> 48) & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[7] = (value >> 56) & 0xFF
 *     return buffer[0:8].decode("latin1")
 */
  (__pyx_v_buffer[6]) = ((__pyx_v_value >> 48) & 0xFF);

  /* "serializor/unicode/ser.pxd":123
 *     buffer[5] = (value >> 40) & 0xFF
 *     buffer[6] = (value >> 48) & 0xFF
 *     buffer[7] = (value >> 56) & 0xFF             # <<<<<<<<<<<<<<
 *     return buffer[0:8].decode("latin1")
 * 
 */
  (__pyx_v_buffer[7]) = ((__pyx_v_value >> 56) & 0xFF);

  /* "serializor/unicode/ser.pxd":124
 *     buffer[6] = (value >> 48) & 0xFF
 *     buffer[7] = (value >> 56) & 0xFF
 *     return buffer[0:8].decode("latin1")             # <<<<<<<<<<<<<<
 * 
 * # Utils: Pack signed integer
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_decode_c_string(__pyx_v_buffer, 0, 8, NULL, NULL, PyUnicode_DecodeLatin1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/ser.pxd":109
 *     return buffer[0:4].decode("latin1")
 * 
 * cdef inline str pack_uint64(unsigned long long value):             # <<<<<<<<<<<<<<
 *     """Pack `UNSIGNED` 64-bit integer in little-endian order to `<'str'>`.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("serializor.unicode.ser.pack_uint64", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/ser.pxd":127
 * 
 * # Utils: Pack signed integer
 * cdef inline str pack_int8(int value):             # <<<<<<<<<<<<<<
 *     """Pack `SIGNED` 8-bit integer in little-endian order to `<'str'>`.
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_10serializor_7unicode_3ser_pack_int8(int __pyx_v_value) {
  char __pyx_v_buffer[1];
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pack_int8", 1);

  /* "serializor/unicode/ser.pxd":134
 *     """
 *     cdef char buffer[1]
 *     buffer[0] = value & 0xFF             # <<<<<<<<<<<<<<
 *     return buffer[0:1].decode("latin1")
 * 
 */
  (__pyx_v_buffer[0]) = (__pyx_v_value & 0xFF);

  /* "serializor/unicode/ser.pxd":135
 *     cdef char buffer[1]
 *     buffer[0] = value & 0xFF
 *     return buffer[0:1].decode("latin1")             # <<<<<<<<<<<<<<
 * 
 * cdef inline str pack_int16(int value):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_decode_c_string(__pyx_v_buffer, 0, 1, NULL, NULL, PyUnicode_DecodeLatin1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 135, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/ser.pxd":127
 * 
 * # Utils: Pack signed integer
 * cdef inline str pack_int8(int value):             # <<<<<<<<<<<<<<
 *     """Pack `SIGNED` 8-bit integer in little-endian order to `<'str'>`.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("serializor.unicode.ser.pack_int8", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/ser.pxd":137
 *     return buffer[0:1].decode("latin1")
 * 
 * cdef inline str pack_int16(int value):             # <<<<<<<<<<<<<<
 *     """Pack `SIGNED` 16-bit integer in little-endian order to `<'str'>`.
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_10serializor_7unicode_3ser_pack_int16(int __pyx_v_value) {
  char __pyx_v_buffer[2];
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pack_int16", 1);

  /* "serializor/unicode/ser.pxd":144
 *     """
 *     cdef char buffer[2]
 *     buffer[0] = value & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[1] = (value >> 8) & 0xFF
 *     return buffer[0:2].decode("latin1")
 */
  (__pyx_v_buffer[0]) = (__pyx_v_value & 0xFF);

  /* "serializor/unicode/ser.pxd":145
 *     cdef char buffer[2]
 *     buffer[0] = value & 0xFF
 *     buffer[1] = (value >> 8) & 0xFF             # <<<<<<<<<<<<<<
 *     return buffer[0:2].decode("latin1")
 * 
 */
  (__pyx_v_buffer[1]) = ((__pyx_v_value >> 8) & 0xFF);

  /* "serializor/unicode/ser.pxd":146
 *     buffer[0] = value & 0xFF
 *     buffer[1] = (value >> 8) & 0xFF
 *     return buffer[0:2].decode("latin1")             # <<<<<<<<<<<<<<
 * 
 * cdef inline str pack_int24(int value):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_decode_c_string(__pyx_v_buffer, 0, 2, NULL, NULL, PyUnicode_DecodeLatin1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 146, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/ser.pxd":137
 *     return buffer[0:1].decode("latin1")
 * 
 * cdef inline str pack_int16(int value):             # <<<<<<<<<<<<<<
 *     """Pack `SIGNED` 16-bit integer in little-endian order to `<'str'>`.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("serializor.unicode.ser.pack_int16", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/ser.pxd":148
 *     return buffer[0:2].decode("latin1")
 * 
 * cdef inline str pack_int24(int value):             # <<<<<<<<<<<<<<
 *     """Pack `SIGNED` 24-bit integer in little-endian order to `<'str'>`.
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_10serializor_7unicode_3ser_pack_int24(int __pyx_v_value) {
  char __pyx_v_buffer[3];
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pack_int24", 1);

  /* "serializor/unicode/ser.pxd":155
 *     """
 *     cdef char buffer[3]
 *     buffer[0] = value & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[1] = (value >> 8) & 0xFF
 *     buffer[2] = (value >> 16) & 0xFF
 */
  (__pyx_v_buffer[0]) = (__pyx_v_value & 0xFF);

  /* "serializor/unicode/ser.pxd":156
 *     cdef char buffer[3]
 *     buffer[0] = value & 0xFF
 *     buffer[1] = (value >> 8) & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[2] = (value >> 16) & 0xFF
 *     return buffer[0:3].decode("latin1")
 */
  (__pyx_v_buffer[1]) = ((__pyx_v_value >> 8) & 0xFF);

  /* "serializor/unicode/ser.pxd":157
 *     buffer[0] = value & 0xFF
 *     buffer[1] = (value >> 8) & 0xFF
 *     buffer[2] = (value >> 16) & 0xFF             # <<<<<<<<<<<<<<
 *     return buffer[0:3].decode("latin1")
 * 
 */
  (__pyx_v_buffer[2]) = ((__pyx_v_value >> 16) & 0xFF);

  /* "serializor/unicode/ser.pxd":158
 *     buffer[1] = (value >> 8) & 0xFF
 *     buffer[2] = (value >> 16) & 0xFF
 *     return buffer[0:3].decode("latin1")             # <<<<<<<<<<<<<<
 * 
 * cdef inline str pack_int32(long long value):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_decode_c_string(__pyx_v_buffer, 0, 3, NULL, NULL, PyUnicode_DecodeLatin1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 158, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/ser.pxd":148
 *     return buffer[0:2].decode("latin1")
 * 
 * cdef inline str pack_int24(int value):             # <<<<<<<<<<<<<<
 *     """Pack `SIGNED` 24-bit integer in little-endian order to `<'str'>`.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("serializor.unicode.ser.pack_int24", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/ser.pxd":160
 *     return buffer[0:3].decode("latin1")
 * 
 * cdef inline str pack_int32(long long value):             # <<<<<<<<<<<<<<
 *     """Pack `SIGNED` 32-bit integer in little-endian order to `<'str'>`.
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_10serializor_7unicode_3ser_pack_int32(PY_LONG_LONG __pyx_v_value) {
  char __pyx_v_buffer[4];
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pack_int32", 1);

  /* "serializor/unicode/ser.pxd":167
 *     """
 *     cdef char buffer[4]
 *     buffer[0] = value & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[1] = (value >> 8) & 0xFF
 *     buffer[2] = (value >> 16) & 0xFF
 */
  (__pyx_v_buffer[0]) = (__pyx_v_value & 0xFF);

  /* "serializor/unicode/ser.pxd":168
 *     cdef char buffer[4]
 *     buffer[0] = value & 0xFF
 *     buffer[1] = (value >> 8) & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[2] = (value >> 16) & 0xFF
 *     buffer[3] = (value >> 24) & 0xFF
 */
  (__pyx_v_buffer[1]) = ((__pyx_v_value >> 8) & 0xFF);

  /* "serializor/unicode/ser.pxd":169
 *     buffer[0] = value & 0xFF
 *     buffer[1] = (value >> 8) & 0xFF
 *     buffer[2] = (value >> 16) & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[3] = (value >> 24) & 0xFF
 *     return buffer[0:4].decode("latin1")
 */
  (__pyx_v_buffer[2]) = ((__pyx_v_value >> 16) & 0xFF);

  /* "serializor/unicode/ser.pxd":170
 *     buffer[1] = (value >> 8) & 0xFF
 *     buffer[2] = (value >> 16) & 0xFF
 *     buffer[3] = (value >> 24) & 0xFF             # <<<<<<<<<<<<<<
 *     return buffer[0:4].decode("latin1")
 * 
 */
  (__pyx_v_buffer[3]) = ((__pyx_v_value >> 24) & 0xFF);

  /* "serializor/unicode/ser.pxd":171
 *     buffer[2] = (value >> 16) & 0xFF
 *     buffer[3] = (value >> 24) & 0xFF
 *     return buffer[0:4].decode("latin1")             # <<<<<<<<<<<<<<
 * 
 * cdef inline str pack_int64(long long value):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_decode_c_string(__pyx_v_buffer, 0, 4, NULL, NULL, PyUnicode_DecodeLatin1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/ser.pxd":160
 *     return buffer[0:3].decode("latin1")
 * 
 * cdef inline str pack_int32(long long value):             # <<<<<<<<<<<<<<
 *     """Pack `SIGNED` 32-bit integer in little-endian order to `<'str'>`.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("serializor.unicode.ser.pack_int32", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/ser.pxd":173
 *     return buffer[0:4].decode("latin1")
 * 
 * cdef inline str pack_int64(long long value):             # <<<<<<<<<<<<<<
 *     """Pack `SIGNED` 64-bit integer in little-endian order to `<'str'>`.
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_10serializor_7unicode_3ser_pack_int64(PY_LONG_LONG __pyx_v_value) {
  char __pyx_v_buffer[8];
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pack_int64", 1);

  /* "serializor/unicode/ser.pxd":180
 *     """
 *     cdef char buffer[8]
 *     buffer[0] = value & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[1] = (value >> 8) & 0xFF
 *     buffer[2] = (value >> 16) & 0xFF
 */
  (__pyx_v_buffer[0]) = (__pyx_v_value & 0xFF);

  /* "serializor/unicode/ser.pxd":181
 *     cdef char buffer[8]
 *     buffer[0] = value & 0xFF
 *     buffer[1] = (value >> 8) & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[2] = (value >> 16) & 0xFF
 *     buffer[3] = (value >> 24) & 0xFF
 */
  (__pyx_v_buffer[1]) = ((__pyx_v_value >> 8) & 0xFF);

  /* "serializor/unicode/ser.pxd":182
 *     buffer[0] = value & 0xFF
 *     buffer[1] = (value >> 8) & 0xFF
 *     buffer[2] = (value >> 16) & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[3] = (value >> 24) & 0xFF
 *     buffer[4] = (value >> 32) & 0xFF
 */
  (__pyx_v_buffer[2]) = ((__pyx_v_value >> 16) & 0xFF);

  /* "serializor/unicode/ser.pxd":183
 *     buffer[1] = (value >> 8) & 0xFF
 *     buffer[2] = (value >> 16) & 0xFF
 *     buffer[3] = (value >> 24) & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[4] = (value >> 32) & 0xFF
 *     buffer[5] = (value >> 40) & 0xFF
 */
  (__pyx_v_buffer[3]) = ((__pyx_v_value >> 24) & 0xFF);

  /* "serializor/unicode/ser.pxd":184
 *     buffer[2] = (value >> 16) & 0xFF
 *     buffer[3] = (value >> 24) & 0xFF
 *     buffer[4] = (value >> 32) & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[5] = (value >> 40) & 0xFF
 *     buffer[6] = (value >> 48) & 0xFF
 */
  (__pyx_v_buffer[4]) = ((__pyx_v_value >> 32) & 0xFF);

  /* "serializor/unicode/ser.pxd":185
 *     buffer[3] = (value >> 24) & 0xFF
 *     buffer[4] = (value >> 32) & 0xFF
 *     buffer[5] = (value >> 40) & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[6] = (value >> 48) & 0xFF
 *     buffer[7] = (value >> 56) & 0xFF
 */
  (__pyx_v_buffer[5]) = ((__pyx_v_value >> 40) & 0xFF);

  /* "serializor/unicode/ser.pxd":186
 *     buffer[4] = (value >> 32) & 0xFF
 *     buffer[5] = (value >> 40) & 0xFF
 *     buffer[6] = (value >> 48) & 0xFF             # <<<<<<<<<<<<<<
 *     buffer[7] = (value >> 56) & 0xFF
 *     return buffer[0:8].decode("latin1")
 */
  (__pyx_v_buffer[6]) = ((__pyx_v_value >> 48) & 0xFF);

  /* "serializor/unicode/ser.pxd":187
 *     buffer[5] = (value >> 40) & 0xFF
 *     buffer[6] = (value >> 48) & 0xFF
 *     buffer[7] = (value >> 56) & 0xFF             # <<<<<<<<<<<<<<
 *     return buffer[0:8].decode("latin1")
 * 
 */
  (__pyx_v_buffer[7]) = ((__pyx_v_value >> 56) & 0xFF);

  /* "serializor/unicode/ser.pxd":188
 *     buffer[6] = (value >> 48) & 0xFF
 *     buffer[7] = (value >> 56) & 0xFF
 *     return buffer[0:8].decode("latin1")             # <<<<<<<<<<<<<<
 * 
 * # Encoded integer
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_decode_c_string(__pyx_v_buffer, 0, 8, NULL, NULL, PyUnicode_DecodeLatin1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/ser.pxd":173
 *     return buffer[0:4].decode("latin1")
 * 
 * cdef inline str pack_int64(long long value):             # <<<<<<<<<<<<<<
 *     """Pack `SIGNED` 64-bit integer in little-endian order to `<'str'>`.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("serializor.unicode.ser.pack_int64", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/ser.pxd":191
 * 
 * # Encoded integer
 * cdef inline str gen_encoded_int(unsigned long long i):             # <<<<<<<<<<<<<<
 *     """Generate encoded integer `<'str'>`."""
 *     cdef char buffer[9]
 */

static CYTHON_INLINE PyObject *__pyx_f_10serializor_7unicode_3ser_gen_encoded_int(unsigned PY_LONG_LONG __pyx_v_i) {
  char __pyx_v_buffer[9];
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gen_encoded_int", 1);

  /* "serializor/unicode/ser.pxd":194
 *     """Generate encoded integer `<'str'>`."""
 *     cdef char buffer[9]
 *     if i <= UINT8_ENCODE_VALUE:             # <<<<<<<<<<<<<<
 *         buffer[0] = i & 0xFF
 *         return buffer[0:1].decode("latin1")
 */
  __pyx_t_1 = (__pyx_v_i <= __pyx_v_10serializor_7unicode_3ser_UINT8_ENCODE_VALUE);
  if (__pyx_t_1) {

    /* "serializor/unicode/ser.pxd":195
 *     cdef char buffer[9]
 *     if i <= UINT8_ENCODE_VALUE:
 *         buffer[0] = i & 0xFF             # <<<<<<<<<<<<<<
 *         return buffer[0:1].decode("latin1")
 *     elif i <= USHRT_MAX:
 */
    (__pyx_v_buffer[0]) = (__pyx_v_i & 0xFF);

    /* "serializor/unicode/ser.pxd":196
 *     if i <= UINT8_ENCODE_VALUE:
 *         buffer[0] = i & 0xFF
 *         return buffer[0:1].decode("latin1")             # <<<<<<<<<<<<<<
 *     elif i <= USHRT_MAX:
 *         buffer[0] = UINT16_ENCODE_VALUE
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_decode_c_string(__pyx_v_buffer, 0, 1, NULL, NULL, PyUnicode_DecodeLatin1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 196, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/ser.pxd":194
 *     """Generate encoded integer `<'str'>`."""
 *     cdef char buffer[9]
 *     if i <= UINT8_ENCODE_VALUE:             # <<<<<<<<<<<<<<
 *         buffer[0] = i & 0xFF
 *         return buffer[0:1].decode("latin1")
 */
  }

  /* "serializor/unicode/ser.pxd":197
 *         buffer[0] = i & 0xFF
 *         return buffer[0:1].decode("latin1")
 *     elif i <= USHRT_MAX:             # <<<<<<<<<<<<<<
 *         buffer[0] = UINT16_ENCODE_VALUE
 *         buffer[1] = i & 0xFF
 */
  __pyx_t_1 = (__pyx_v_i <= USHRT_MAX);
  if (__pyx_t_1) {

    /* "serializor/unicode/ser.pxd":198
 *         return buffer[0:1].decode("latin1")
 *     elif i <= USHRT_MAX:
 *         buffer[0] = UINT16_ENCODE_VALUE             # <<<<<<<<<<<<<<
 *         buffer[1] = i & 0xFF
 *         buffer[2] = (i >> 8) & 0xFF
 */
    (__pyx_v_buffer[0]) = __pyx_v_10serializor_7unicode_3ser_UINT16_ENCODE_VALUE;

    /* "serializor/unicode/ser.pxd":199
 *     elif i <= USHRT_MAX:
 *         buffer[0] = UINT16_ENCODE_VALUE
 *         buffer[1] = i & 0xFF             # <<<<<<<<<<<<<<
 *         buffer[2] = (i >> 8) & 0xFF
 *         return buffer[0:3].decode("latin1")
 */
    (__pyx_v_buffer[1]) = (__pyx_v_i & 0xFF);

    /* "serializor/unicode/ser.pxd":200
 *         buffer[0] = UINT16_ENCODE_VALUE
 *         buffer[1] = i & 0xFF
 *         buffer[2] = (i >> 8) & 0xFF             # <<<<<<<<<<<<<<
 *         return buffer[0:3].decode("latin1")
 *     elif i <= UINT_MAX:
 */
    (__pyx_v_buffer[2]) = ((__pyx_v_i >> 8) & 0xFF);

    /* "serializor/unicode/ser.pxd":201
 *         buffer[1] = i & 0xFF
 *         buffer[2] = (i >> 8) & 0xFF
 *         return buffer[0:3].decode("latin1")             # <<<<<<<<<<<<<<
 *     elif i <= UINT_MAX:
 *         buffer[0] = UINT32_ENCODE_VALUE
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_decode_c_string(__pyx_v_buffer, 0, 3, NULL, NULL, PyUnicode_DecodeLatin1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 201, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/ser.pxd":197
 *         buffer[0] = i & 0xFF
 *         return buffer[0:1].decode("latin1")
 *     elif i <= USHRT_MAX:             # <<<<<<<<<<<<<<
 *         buffer[0] = UINT16_ENCODE_VALUE
 *         buffer[1] = i & 0xFF
 */
  }

  /* "serializor/unicode/ser.pxd":202
 *         buffer[2] = (i >> 8) & 0xFF
 *         return buffer[0:3].decode("latin1")
 *     elif i <= UINT_MAX:             # <<<<<<<<<<<<<<
 *         buffer[0] = UINT32_ENCODE_VALUE
 *         buffer[1] = i & 0xFF
 */
  __pyx_t_1 = (__pyx_v_i <= UINT_MAX);
  if (__pyx_t_1) {

    /* "serializor/unicode/ser.pxd":203
 *         return buffer[0:3].decode("latin1")
 *     elif i <= UINT_MAX:
 *         buffer[0] = UINT32_ENCODE_VALUE             # <<<<<<<<<<<<<<
 *         buffer[1] = i & 0xFF
 *         buffer[2] = (i >> 8) & 0xFF
 */
    (__pyx_v_buffer[0]) = __pyx_v_10serializor_7unicode_3ser_UINT32_ENCODE_VALUE;

    /* "serializor/unicode/ser.pxd":204
 *     elif i <= UINT_MAX:
 *         buffer[0] = UINT32_ENCODE_VALUE
 *         buffer[1] = i & 0xFF             # <<<<<<<<<<<<<<
 *         buffer[2] = (i >> 8) & 0xFF
 *         buffer[3] = (i >> 16) & 0xFF
 */
    (__pyx_v_buffer[1]) = (__pyx_v_i & 0xFF);

    /* "serializor/unicode/ser.pxd":205
 *         buffer[0] = UINT32_ENCODE_VALUE
 *         buffer[1] = i & 0xFF
 *         buffer[2] = (i >> 8) & 0xFF             # <<<<<<<<<<<<<<
 *         buffer[3] = (i >> 16) & 0xFF
 *         buffer[4] = (i >> 24) & 0xFF
 */
    (__pyx_v_buffer[2]) = ((__pyx_v_i >> 8) & 0xFF);

    /* "serializor/unicode/ser.pxd":206
 *         buffer[1] = i & 0xFF
 *         buffer[2] = (i >> 8) & 0xFF
 *         buffer[3] = (i >> 16) & 0xFF             # <<<<<<<<<<<<<<
 *         buffer[4] = (i >> 24) & 0xFF
 *         return buffer[:5].decode("latin1")
 */
    (__pyx_v_buffer[3]) = ((__pyx_v_i >> 16) & 0xFF);

    /* "serializor/unicode/ser.pxd":207
 *         buffer[2] = (i >> 8) & 0xFF
 *         buffer[3] = (i >> 16) & 0xFF
 *         buffer[4] = (i >> 24) & 0xFF             # <<<<<<<<<<<<<<
 *         return buffer[:5].decode("latin1")
 *     else:
 */
    (__pyx_v_buffer[4]) = ((__pyx_v_i >> 24) & 0xFF);

    /* "serializor/unicode/ser.pxd":208
 *         buffer[3] = (i >> 16) & 0xFF
 *         buffer[4] = (i >> 24) & 0xFF
 *         return buffer[:5].decode("latin1")             # <<<<<<<<<<<<<<
 *     else:
 *         buffer[0] = UINT64_ENCODE_VALUE
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_decode_c_string(__pyx_v_buffer, 0, 5, NULL, NULL, PyUnicode_DecodeLatin1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 208, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/ser.pxd":202
 *         buffer[2] = (i >> 8) & 0xFF
 *         return buffer[0:3].decode("latin1")
 *     elif i <= UINT_MAX:             # <<<<<<<<<<<<<<
 *         buffer[0] = UINT32_ENCODE_VALUE
 *         buffer[1] = i & 0xFF
 */
  }

  /* "serializor/unicode/ser.pxd":210
 *         return buffer[:5].decode("latin1")
 *     else:
 *         buffer[0] = UINT64_ENCODE_VALUE             # <<<<<<<<<<<<<<
 *         buffer[1] = i & 0xFF
 *         buffer[2] = (i >> 8) & 0xFF
 */
  /*else*/ {
    (__pyx_v_buffer[0]) = __pyx_v_10serializor_7unicode_3ser_UINT64_ENCODE_VALUE;

    /* "serializor/unicode/ser.pxd":211
 *     else:
 *         buffer[0] = UINT64_ENCODE_VALUE
 *         buffer[1] = i & 0xFF             # <<<<<<<<<<<<<<
 *         buffer[2] = (i >> 8) & 0xFF
 *         buffer[3] = (i >> 16) & 0xFF
 */
    (__pyx_v_buffer[1]) = (__pyx_v_i & 0xFF);

    /* "serializor/unicode/ser.pxd":212
 *         buffer[0] = UINT64_ENCODE_VALUE
 *         buffer[1] = i & 0xFF
 *         buffer[2] = (i >> 8) & 0xFF             # <<<<<<<<<<<<<<
 *         buffer[3] = (i >> 16) & 0xFF
 *         buffer[4] = (i >> 24) & 0xFF
 */
    (__pyx_v_buffer[2]) = ((__pyx_v_i >> 8) & 0xFF);

    /* "serializor/unicode/ser.pxd":213
 *         buffer[1] = i & 0xFF
 *         buffer[2] = (i >> 8) & 0xFF
 *         buffer[3] = (i >> 16) & 0xFF             # <<<<<<<<<<<<<<
 *         buffer[4] = (i >> 24) & 0xFF
 *         buffer[5] = (i >> 32) & 0xFF
 */
    (__pyx_v_buffer[3]) = ((__pyx_v_i >> 16) & 0xFF);

    /* "serializor/unicode/ser.pxd":214
 *         buffer[2] = (i >> 8) & 0xFF
 *         buffer[3] = (i >> 16) & 0xFF
 *         buffer[4] = (i >> 24) & 0xFF             # <<<<<<<<<<<<<<
 *         buffer[5] = (i >> 32) & 0xFF
 *         buffer[6] = (i >> 40) & 0xFF
 */
    (__pyx_v_buffer[4]) = ((__pyx_v_i >> 24) & 0xFF);

    /* "serializor/unicode/ser.pxd":215
 *         buffer[3] = (i >> 16) & 0xFF
 *         buffer[4] = (i >> 24) & 0xFF
 *         buffer[5] = (i >> 32) & 0xFF             # <<<<<<<<<<<<<<
 *         buffer[6] = (i >> 40) & 0xFF
 *         buffer[7] = (i >> 48) & 0xFF
 */
    (__pyx_v_buffer[5]) = ((__pyx_v_i >> 32) & 0xFF);

    /* "serializor/unicode/ser.pxd":216
 *         buffer[4] = (i >> 24) & 0xFF
 *         buffer[5] = (i >> 32) & 0xFF
 *         buffer[6] = (i >> 40) & 0xFF             # <<<<<<<<<<<<<<
 *         buffer[7] = (i >> 48) & 0xFF
 *         buffer[8] = (i >> 56) & 0xFF
 */
    (__pyx_v_buffer[6]) = ((__pyx_v_i >> 40) & 0xFF);

    /* "serializor/unicode/ser.pxd":217
 *         buffer[5] = (i >> 32) & 0xFF
 *         buffer[6] = (i >> 40) & 0xFF
 *         buffer[7] = (i >> 48) & 0xFF             # <<<<<<<<<<<<<<
 *         buffer[8] = (i >> 56) & 0xFF
 *         return buffer[0:9].decode("latin1")
 */
    (__pyx_v_buffer[7]) = ((__pyx_v_i >> 48) & 0xFF);

    /* "serializor/unicode/ser.pxd":218
 *         buffer[6] = (i >> 40) & 0xFF
 *         buffer[7] = (i >> 48) & 0xFF
 *         buffer[8] = (i >> 56) & 0xFF             # <<<<<<<<<<<<<<
 *         return buffer[0:9].decode("latin1")
 * 
 */
    (__pyx_v_buffer[8]) = ((__pyx_v_i >> 56) & 0xFF);

    /* "serializor/unicode/ser.pxd":219
 *         buffer[7] = (i >> 48) & 0xFF
 *         buffer[8] = (i >> 56) & 0xFF
 *         return buffer[0:9].decode("latin1")             # <<<<<<<<<<<<<<
 * 
 * # Header
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_decode_c_string(__pyx_v_buffer, 0, 9, NULL, NULL, PyUnicode_DecodeLatin1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 219, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "serializor/unicode/ser.pxd":191
 * 
 * # Encoded integer
 * cdef inline str gen_encoded_int(unsigned long long i):             # <<<<<<<<<<<<<<
 *     """Generate encoded integer `<'str'>`."""
 *     cdef char buffer[9]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("serializor.unicode.ser.gen_encoded_int", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/ser.pxd":222
 * 
 * # Header
 * cdef inline str gen_header(unsigned char identifier, unsigned long long i):             # <<<<<<<<<<<<<<
 *     """Generate header '[identifier]+[encoded integer]' `<'str'>`."""
 *     cdef char buffer[10]
 */

static CYTHON_INLINE PyObject *__pyx_f_10serializor_7unicode_3ser_gen_header(unsigned char __pyx_v_identifier, unsigned PY_LONG_LONG __pyx_v_i) {
  char __pyx_v_buffer[10];
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gen_header", 1);

  /* "serializor/unicode/ser.pxd":225
 *     """Generate header '[identifier]+[encoded integer]' `<'str'>`."""
 *     cdef char buffer[10]
 *     if i <= UINT8_ENCODE_VALUE:             # <<<<<<<<<<<<<<
 *         buffer[0] = identifier
 *         buffer[1] = i & 0xFF
 */
  __pyx_t_1 = (__pyx_v_i <= __pyx_v_10serializor_7unicode_3ser_UINT8_ENCODE_VALUE);
  if (__pyx_t_1) {

    /* "serializor/unicode/ser.pxd":226
 *     cdef char buffer[10]
 *     if i <= UINT8_ENCODE_VALUE:
 *         buffer[0] = identifier             # <<<<<<<<<<<<<<
 *         buffer[1] = i & 0xFF
 *         return buffer[0:2].decode("latin1")
 */
    (__pyx_v_buffer[0]) = __pyx_v_identifier;

    /* "serializor/unicode/ser.pxd":227
 *     if i <= UINT8_ENCODE_VALUE:
 *         buffer[0] = identifier
 *         buffer[1] = i & 0xFF             # <<<<<<<<<<<<<<
 *         return buffer[0:2].decode("latin1")
 *     elif i <= USHRT_MAX:
 */
    (__pyx_v_buffer[1]) = (__pyx_v_i & 0xFF);

    /* "serializor/unicode/ser.pxd":228
 *         buffer[0] = identifier
 *         buffer[1] = i & 0xFF
 *         return buffer[0:2].decode("latin1")             # <<<<<<<<<<<<<<
 *     elif i <= USHRT_MAX:
 *         buffer[0] = identifier
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_decode_c_string(__pyx_v_buffer, 0, 2, NULL, NULL, PyUnicode_DecodeLatin1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 228, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/ser.pxd":225
 *     """Generate header '[identifier]+[encoded integer]' `<'str'>`."""
 *     cdef char buffer[10]
 *     if i <= UINT8_ENCODE_VALUE:             # <<<<<<<<<<<<<<
 *         buffer[0] = identifier
 *         buffer[1] = i & 0xFF
 */
  }

  /* "serializor/unicode/ser.pxd":229
 *         buffer[1] = i & 0xFF
 *         return buffer[0:2].decode("latin1")
 *     elif i <= USHRT_MAX:             # <<<<<<<<<<<<<<
 *         buffer[0] = identifier
 *         buffer[1] = UINT16_ENCODE_VALUE
 */
  __pyx_t_1 = (__pyx_v_i <= USHRT_MAX);
  if (__pyx_t_1) {

    /* "serializor/unicode/ser.pxd":230
 *         return buffer[0:2].decode("latin1")
 *     elif i <= USHRT_MAX:
 *         buffer[0] = identifier             # <<<<<<<<<<<<<<
 *         buffer[1] = UINT16_ENCODE_VALUE
 *         buffer[2] = i & 0xFF
 */
    (__pyx_v_buffer[0]) = __pyx_v_identifier;

    /* "serializor/unicode/ser.pxd":231
 *     elif i <= USHRT_MAX:
 *         buffer[0] = identifier
 *         buffer[1] = UINT16_ENCODE_VALUE             # <<<<<<<<<<<<<<
 *         buffer[2] = i & 0xFF
 *         buffer[3] = (i >> 8) & 0xFF
 */
    (__pyx_v_buffer[1]) = __pyx_v_10serializor_7unicode_3ser_UINT16_ENCODE_VALUE;

    /* "serializor/unicode/ser.pxd":232
 *         buffer[0] = identifier
 *         buffer[1] = UINT16_ENCODE_VALUE
 *         buffer[2] = i & 0xFF             # <<<<<<<<<<<<<<
 *         buffer[3] = (i >> 8) & 0xFF
 *         return buffer[0:4].decode("latin1")
 */
    (__pyx_v_buffer[2]) = (__pyx_v_i & 0xFF);

    /* "serializor/unicode/ser.pxd":233
 *         buffer[1] = UINT16_ENCODE_VALUE
 *         buffer[2] = i & 0xFF
 *         buffer[3] = (i >> 8) & 0xFF             # <<<<<<<<<<<<<<
 *         return buffer[0:4].decode("latin1")
 *     elif i <= UINT_MAX:
 */
    (__pyx_v_buffer[3]) = ((__pyx_v_i >> 8) & 0xFF);

    /* "serializor/unicode/ser.pxd":234
 *         buffer[2] = i & 0xFF
 *         buffer[3] = (i >> 8) & 0xFF
 *         return buffer[0:4].decode("latin1")             # <<<<<<<<<<<<<<
 *     elif i <= UINT_MAX:
 *         buffer[0] = identifier
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_decode_c_string(__pyx_v_buffer, 0, 4, NULL, NULL, PyUnicode_DecodeLatin1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 234, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/ser.pxd":229
 *         buffer[1] = i & 0xFF
 *         return buffer[0:2].decode("latin1")
 *     elif i <= USHRT_MAX:             # <<<<<<<<<<<<<<
 *         buffer[0] = identifier
 *         buffer[1] = UINT16_ENCODE_VALUE
 */
  }

  /* "serializor/unicode/ser.pxd":235
 *         buffer[3] = (i >> 8) & 0xFF
 *         return buffer[0:4].decode("latin1")
 *     elif i <= UINT_MAX:             # <<<<<<<<<<<<<<
 *         buffer[0] = identifier
 *         buffer[1] = UINT32_ENCODE_VALUE
 */
  __pyx_t_1 = (__pyx_v_i <= UINT_MAX);
  if (__pyx_t_1) {

    /* "serializor/unicode/ser.pxd":236
 *         return buffer[0:4].decode("latin1")
 *     elif i <= UINT_MAX:
 *         buffer[0] = identifier             # <<<<<<<<<<<<<<
 *         buffer[1] = UINT32_ENCODE_VALUE
 *         buffer[2] = i & 0xFF
 */
    (__pyx_v_buffer[0]) = __pyx_v_identifier;

    /* "serializor/unicode/ser.pxd":237
 *     elif i <= UINT_MAX:
 *         buffer[0] = identifier
 *         buffer[1] = UINT32_ENCODE_VALUE             # <<<<<<<<<<<<<<
 *         buffer[2] = i & 0xFF
 *         buffer[3] = (i >> 8) & 0xFF
 */
    (__pyx_v_buffer[1]) = __pyx_v_10serializor_7unicode_3ser_UINT32_ENCODE_VALUE;

    /* "serializor/unicode/ser.pxd":238
 *         buffer[0] = identifier
 *         buffer[1] = UINT32_ENCODE_VALUE
 *         buffer[2] = i & 0xFF             # <<<<<<<<<<<<<<
 *         buffer[3] = (i >> 8) & 0xFF
 *         buffer[4] = (i >> 16) & 0xFF
 */
    (__pyx_v_buffer[2]) = (__pyx_v_i & 0xFF);

    /* "serializor/unicode/ser.pxd":239
 *         buffer[1] = UINT32_ENCODE_VALUE
 *         buffer[2] = i & 0xFF
 *         buffer[3] = (i >> 8) & 0xFF             # <<<<<<<<<<<<<<
 *         buffer[4] = (i >> 16) & 0xFF
 *         buffer[5] = (i >> 24) & 0xFF
 */
    (__pyx_v_buffer[3]) = ((__pyx_v_i >> 8) & 0xFF);

    /* "serializor/unicode/ser.pxd":240
 *         buffer[2] = i & 0xFF
 *         buffer[3] = (i >> 8) & 0xFF
 *         buffer[4] = (i >> 16) & 0xFF             # <<<<<<<<<<<<<<
 *         buffer[5] = (i >> 24) & 0xFF
 *         return buffer[:6].decode("latin1")
 */
    (__pyx_v_buffer[4]) = ((__pyx_v_i >> 16) & 0xFF);

    /* "serializor/unicode/ser.pxd":241
 *         buffer[3] = (i >> 8) & 0xFF
 *         buffer[4] = (i >> 16) & 0xFF
 *         buffer[5] = (i >> 24) & 0xFF             # <<<<<<<<<<<<<<
 *         return buffer[:6].decode("latin1")
 *     else:
 */
    (__pyx_v_buffer[5]) = ((__pyx_v_i >> 24) & 0xFF);

    /* "serializor/unicode/ser.pxd":242
 *         buffer[4] = (i >> 16) & 0xFF
 *         buffer[5] = (i >> 24) & 0xFF
 *         return buffer[:6].decode("latin1")             # <<<<<<<<<<<<<<
 *     else:
 *         buffer[0] = identifier
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_decode_c_string(__pyx_v_buffer, 0, 6, NULL, NULL, PyUnicode_DecodeLatin1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 242, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/ser.pxd":235
 *         buffer[3] = (i >> 8) & 0xFF
 *         return buffer[0:4].decode("latin1")
 *     elif i <= UINT_MAX:             # <<<<<<<<<<<<<<
 *         buffer[0] = identifier
 *         buffer[1] = UINT32_ENCODE_VALUE
 */
  }

  /* "serializor/unicode/ser.pxd":244
 *         return buffer[:6].decode("latin1")
 *     else:
 *         buffer[0] = identifier             # <<<<<<<<<<<<<<
 *         buffer[1] = UINT64_ENCODE_VALUE
 *         buffer[2] = i & 0xFF
 */
  /*else*/ {
    (__pyx_v_buffer[0]) = __pyx_v_identifier;

    /* "serializor/unicode/ser.pxd":245
 *     else:
 *         buffer[0] = identifier
 *         buffer[1] = UINT64_ENCODE_VALUE             # <<<<<<<<<<<<<<
 *         buffer[2] = i & 0xFF
 *         buffer[3] = (i >> 8) & 0xFF
 */
    (__pyx_v_buffer[1]) = __pyx_v_10serializor_7unicode_3ser_UINT64_ENCODE_VALUE;

    /* "serializor/unicode/ser.pxd":246
 *         buffer[0] = identifier
 *         buffer[1] = UINT64_ENCODE_VALUE
 *         buffer[2] = i & 0xFF             # <<<<<<<<<<<<<<
 *         buffer[3] = (i >> 8) & 0xFF
 *         buffer[4] = (i >> 16) & 0xFF
 */
    (__pyx_v_buffer[2]) = (__pyx_v_i & 0xFF);

    /* "serializor/unicode/ser.pxd":247
 *         buffer[1] = UINT64_ENCODE_VALUE
 *         buffer[2] = i & 0xFF
 *         buffer[3] = (i >> 8) & 0xFF             # <<<<<<<<<<<<<<
 *         buffer[4] = (i >> 16) & 0xFF
 *         buffer[5] = (i >> 24) & 0xFF
 */
    (__pyx_v_buffer[3]) = ((__pyx_v_i >> 8) & 0xFF);

    /* "serializor/unicode/ser.pxd":248
 *         buffer[2] = i & 0xFF
 *         buffer[3] = (i >> 8) & 0xFF
 *         buffer[4] = (i >> 16) & 0xFF             # <<<<<<<<<<<<<<
 *         buffer[5] = (i >> 24) & 0xFF
 *         buffer[6] = (i >> 32) & 0xFF
 */
    (__pyx_v_buffer[4]) = ((__pyx_v_i >> 16) & 0xFF);

    /* "serializor/unicode/ser.pxd":249
 *         buffer[3] = (i >> 8) & 0xFF
 *         buffer[4] = (i >> 16) & 0xFF
 *         buffer[5] = (i >> 24) & 0xFF             # <<<<<<<<<<<<<<
 *         buffer[6] = (i >> 32) & 0xFF
 *         buffer[7] = (i >> 40) & 0xFF
 */
    (__pyx_v_buffer[5]) = ((__pyx_v_i >> 24) & 0xFF);

    /* "serializor/unicode/ser.pxd":250
 *         buffer[4] = (i >> 16) & 0xFF
 *         buffer[5] = (i >> 24) & 0xFF
 *         buffer[6] = (i >> 32) & 0xFF             # <<<<<<<<<<<<<<
 *         buffer[7] = (i >> 40) & 0xFF
 *         buffer[8] = (i >> 48) & 0xFF
 */
    (__pyx_v_buffer[6]) = ((__pyx_v_i >> 32) & 0xFF);

    /* "serializor/unicode/ser.pxd":251
 *         buffer[5] = (i >> 24) & 0xFF
 *         buffer[6] = (i >> 32) & 0xFF
 *         buffer[7] = (i >> 40) & 0xFF             # <<<<<<<<<<<<<<
 *         buffer[8] = (i >> 48) & 0xFF
 *         buffer[9] = (i >> 56) & 0xFF
 */
    (__pyx_v_buffer[7]) = ((__pyx_v_i >> 40) & 0xFF);

    /* "serializor/unicode/ser.pxd":252
 *         buffer[6] = (i >> 32) & 0xFF
 *         buffer[7] = (i >> 40) & 0xFF
 *         buffer[8] = (i >> 48) & 0xFF             # <<<<<<<<<<<<<<
 *         buffer[9] = (i >> 56) & 0xFF
 *         return buffer[0:10].decode("latin1")
 */
    (__pyx_v_buffer[8]) = ((__pyx_v_i >> 48) & 0xFF);

    /* "serializor/unicode/ser.pxd":253
 *         buffer[7] = (i >> 40) & 0xFF
 *         buffer[8] = (i >> 48) & 0xFF
 *         buffer[9] = (i >> 56) & 0xFF             # <<<<<<<<<<<<<<
 *         return buffer[0:10].decode("latin1")
 * 
 */
    (__pyx_v_buffer[9]) = ((__pyx_v_i >> 56) & 0xFF);

    /* "serializor/unicode/ser.pxd":254
 *         buffer[8] = (i >> 48) & 0xFF
 *         buffer[9] = (i >> 56) & 0xFF
 *         return buffer[0:10].decode("latin1")             # <<<<<<<<<<<<<<
 * 
 * # Serialize
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_decode_c_string(__pyx_v_buffer, 0, 10, NULL, NULL, PyUnicode_DecodeLatin1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 254, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "serializor/unicode/ser.pxd":222
 * 
 * # Header
 * cdef inline str gen_header(unsigned char identifier, unsigned long long i):             # <<<<<<<<<<<<<<
 *     """Generate header '[identifier]+[encoded integer]' `<'str'>`."""
 *     cdef char buffer[10]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("serializor.unicode.ser.gen_header", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.pxd":11
 * 
 * # Utils: Unpack unsigned integers from string
 * cdef inline unsigned char unpack_uint8(str data, Py_ssize_t pos):             # <<<<<<<<<<<<<<
 *     """Unpack (read) `UNSIGNED` 8-bit integer from 'data'
 *     at the given 'pos' in little-endian order `<'int'>`.
 */

static CYTHON_INLINE unsigned char __pyx_f_10serializor_7unicode_3des_unpack_uint8(PyObject *__pyx_v_data, Py_ssize_t __pyx_v_pos) {
  unsigned char __pyx_r;

  /* "serializor/unicode/des.pxd":15
 *     at the given 'pos' in little-endian order `<'int'>`.
 *     """
 *     return <unsigned char> read_char(data, pos)             # <<<<<<<<<<<<<<
 * 
 * cdef inline unsigned short unpack_uint16(str data, Py_ssize_t pos):
 */
  __pyx_r = ((unsigned char)PyUnicode_READ_CHAR(__pyx_v_data, __pyx_v_pos));
  goto __pyx_L0;

  /* "serializor/unicode/des.pxd":11
 * 
 * # Utils: Unpack unsigned integers from string
 * cdef inline unsigned char unpack_uint8(str data, Py_ssize_t pos):             # <<<<<<<<<<<<<<
 *     """Unpack (read) `UNSIGNED` 8-bit integer from 'data'
 *     at the given 'pos' in little-endian order `<'int'>`.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "serializor/unicode/des.pxd":17
 *     return <unsigned char> read_char(data, pos)
 * 
 * cdef inline unsigned short unpack_uint16(str data, Py_ssize_t pos):             # <<<<<<<<<<<<<<
 *     """Unpack (read) `UNSIGNED` 16-bit integer from 'data'
 *     at the given 'pos' in little-endian order `<'int'>`.
 */

static CYTHON_INLINE unsigned short __pyx_f_10serializor_7unicode_3des_unpack_uint16(PyObject *__pyx_v_data, Py_ssize_t __pyx_v_pos) {
  unsigned short __pyx_v_p0;
  unsigned short __pyx_v_p1;
  unsigned short __pyx_v_res;
  unsigned short __pyx_r;

  /* "serializor/unicode/des.pxd":22
 *     """
 *     cdef:
 *         unsigned short p0 = <unsigned char> read_char(data, pos)             # <<<<<<<<<<<<<<
 *         unsigned short p1 = <unsigned char> read_char(data, pos + 1)
 *         unsigned short res = p0 | (p1 << 8)
 */
  __pyx_v_p0 = ((unsigned char)PyUnicode_READ_CHAR(__pyx_v_data, __pyx_v_pos));

  /* "serializor/unicode/des.pxd":23
 *     cdef:
 *         unsigned short p0 = <unsigned char> read_char(data, pos)
 *         unsigned short p1 = <unsigned char> read_char(data, pos + 1)             # <<<<<<<<<<<<<<
 *         unsigned short res = p0 | (p1 << 8)
 *     return res
 */
  __pyx_v_p1 = ((unsigned char)PyUnicode_READ_CHAR(__pyx_v_data, (__pyx_v_pos + 1)));

  /* "serializor/unicode/des.pxd":24
 *         unsigned short p0 = <unsigned char> read_char(data, pos)
 *         unsigned short p1 = <unsigned char> read_char(data, pos + 1)
 *         unsigned short res = p0 | (p1 << 8)             # <<<<<<<<<<<<<<
 *     return res
 * 
 */
  __pyx_v_res = (__pyx_v_p0 | (__pyx_v_p1 << 8));

  /* "serializor/unicode/des.pxd":25
 *         unsigned short p1 = <unsigned char> read_char(data, pos + 1)
 *         unsigned short res = p0 | (p1 << 8)
 *     return res             # <<<<<<<<<<<<<<
 * 
 * cdef inline unsigned int unpack_uint24(str data, Py_ssize_t pos):
 */
  __pyx_r = __pyx_v_res;
  goto __pyx_L0;

  /* "serializor/unicode/des.pxd":17
 *     return <unsigned char> read_char(data, pos)
 * 
 * cdef inline unsigned short unpack_uint16(str data, Py_ssize_t pos):             # <<<<<<<<<<<<<<
 *     """Unpack (read) `UNSIGNED` 16-bit integer from 'data'
 *     at the given 'pos' in little-endian order `<'int'>`.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "serializor/unicode/des.pxd":27
 *     return res
 * 
 * cdef inline unsigned int unpack_uint24(str data, Py_ssize_t pos):             # <<<<<<<<<<<<<<
 *     """Unpack (read) `UNSIGNED` 24-bit integer from 'data'
 *     at the given 'pos' in little-endian order `<'int'>`.
 */

static CYTHON_INLINE unsigned int __pyx_f_10serializor_7unicode_3des_unpack_uint24(PyObject *__pyx_v_data, Py_ssize_t __pyx_v_pos) {
  unsigned int __pyx_v_p0;
  unsigned int __pyx_v_p1;
  unsigned int __pyx_v_p2;
  unsigned int __pyx_v_res;
  unsigned int __pyx_r;

  /* "serializor/unicode/des.pxd":32
 *     """
 *     cdef:
 *         unsigned int p0 = <unsigned char> read_char(data, pos)             # <<<<<<<<<<<<<<
 *         unsigned int p1 = <unsigned char> read_char(data, pos + 1)
 *         unsigned int p2 = <unsigned char> read_char(data, pos + 2)
 */
  __pyx_v_p0 = ((unsigned char)PyUnicode_READ_CHAR(__pyx_v_data, __pyx_v_pos));

  /* "serializor/unicode/des.pxd":33
 *     cdef:
 *         unsigned int p0 = <unsigned char> read_char(data, pos)
 *         unsigned int p1 = <unsigned char> read_char(data, pos + 1)             # <<<<<<<<<<<<<<
 *         unsigned int p2 = <unsigned char> read_char(data, pos + 2)
 *         unsigned int res = p0 | (p1 << 8) | (p2 << 16)
 */
  __pyx_v_p1 = ((unsigned char)PyUnicode_READ_CHAR(__pyx_v_data, (__pyx_v_pos + 1)));

  /* "serializor/unicode/des.pxd":34
 *         unsigned int p0 = <unsigned char> read_char(data, pos)
 *         unsigned int p1 = <unsigned char> read_char(data, pos + 1)
 *         unsigned int p2 = <unsigned char> read_char(data, pos + 2)             # <<<<<<<<<<<<<<
 *         unsigned int res = p0 | (p1 << 8) | (p2 << 16)
 *     return res
 */
  __pyx_v_p2 = ((unsigned char)PyUnicode_READ_CHAR(__pyx_v_data, (__pyx_v_pos + 2)));

  /* "serializor/unicode/des.pxd":35
 *         unsigned int p1 = <unsigned char> read_char(data, pos + 1)
 *         unsigned int p2 = <unsigned char> read_char(data, pos + 2)
 *         unsigned int res = p0 | (p1 << 8) | (p2 << 16)             # <<<<<<<<<<<<<<
 *     return res
 * 
 */
  __pyx_v_res = ((__pyx_v_p0 | (__pyx_v_p1 << 8)) | (__pyx_v_p2 << 16));

  /* "serializor/unicode/des.pxd":36
 *         unsigned int p2 = <unsigned char> read_char(data, pos + 2)
 *         unsigned int res = p0 | (p1 << 8) | (p2 << 16)
 *     return res             # <<<<<<<<<<<<<<
 * 
 * cdef inline unsigned int unpack_uint32(str data, Py_ssize_t pos):
 */
  __pyx_r = __pyx_v_res;
  goto __pyx_L0;

  /* "serializor/unicode/des.pxd":27
 *     return res
 * 
 * cdef inline unsigned int unpack_uint24(str data, Py_ssize_t pos):             # <<<<<<<<<<<<<<
 *     """Unpack (read) `UNSIGNED` 24-bit integer from 'data'
 *     at the given 'pos' in little-endian order `<'int'>`.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "serializor/unicode/des.pxd":38
 *     return res
 * 
 * cdef inline unsigned int unpack_uint32(str data, Py_ssize_t pos):             # <<<<<<<<<<<<<<
 *     """Unpack (read) `UNSIGNED` 32-bit integer from 'data'
 *     at the given 'pos' in little-endian order `<'int'>`.
 */

static CYTHON_INLINE unsigned int __pyx_f_10serializor_7unicode_3des_unpack_uint32(PyObject *__pyx_v_data, Py_ssize_t __pyx_v_pos) {
  unsigned int __pyx_v_p0;
  unsigned int __pyx_v_p1;
  unsigned int __pyx_v_p2;
  unsigned int __pyx_v_p3;
  unsigned int __pyx_v_res;
  unsigned int __pyx_r;

  /* "serializor/unicode/des.pxd":43
 *     """
 *     cdef:
 *         unsigned int p0 = <unsigned char> read_char(data, pos)             # <<<<<<<<<<<<<<
 *         unsigned int p1 = <unsigned char> read_char(data, pos + 1)
 *         unsigned int p2 = <unsigned char> read_char(data, pos + 2)
 */
  __pyx_v_p0 = ((unsigned char)PyUnicode_READ_CHAR(__pyx_v_data, __pyx_v_pos));

  /* "serializor/unicode/des.pxd":44
 *     cdef:
 *         unsigned int p0 = <unsigned char> read_char(data, pos)
 *         unsigned int p1 = <unsigned char> read_char(data, pos + 1)             # <<<<<<<<<<<<<<
 *         unsigned int p2 = <unsigned char> read_char(data, pos + 2)
 *         unsigned int p3 = <unsigned char> read_char(data, pos + 3)
 */
  __pyx_v_p1 = ((unsigned char)PyUnicode_READ_CHAR(__pyx_v_data, (__pyx_v_pos + 1)));

  /* "serializor/unicode/des.pxd":45
 *         unsigned int p0 = <unsigned char> read_char(data, pos)
 *         unsigned int p1 = <unsigned char> read_char(data, pos + 1)
 *         unsigned int p2 = <unsigned char> read_char(data, pos + 2)             # <<<<<<<<<<<<<<
 *         unsigned int p3 = <unsigned char> read_char(data, pos + 3)
 *         unsigned int res = p0 | (p1 << 8) | (p2 << 16) | (p3 << 24)
 */
  __pyx_v_p2 = ((unsigned char)PyUnicode_READ_CHAR(__pyx_v_data, (__pyx_v_pos + 2)));

  /* "serializor/unicode/des.pxd":46
 *         unsigned int p1 = <unsigned char> read_char(data, pos + 1)
 *         unsigned int p2 = <unsigned char> read_char(data, pos + 2)
 *         unsigned int p3 = <unsigned char> read_char(data, pos + 3)             # <<<<<<<<<<<<<<
 *         unsigned int res = p0 | (p1 << 8) | (p2 << 16) | (p3 << 24)
 *     return res
 */
  __pyx_v_p3 = ((unsigned char)PyUnicode_READ_CHAR(__pyx_v_data, (__pyx_v_pos + 3)));

  /* "serializor/unicode/des.pxd":47
 *         unsigned int p2 = <unsigned char> read_char(data, pos + 2)
 *         unsigned int p3 = <unsigned char> read_char(data, pos + 3)
 *         unsigned int res = p0 | (p1 << 8) | (p2 << 16) | (p3 << 24)             # <<<<<<<<<<<<<<
 *     return res
 * 
 */
  __pyx_v_res = (((__pyx_v_p0 | (__pyx_v_p1 << 8)) | (__pyx_v_p2 << 16)) | (__pyx_v_p3 << 24));

  /* "serializor/unicode/des.pxd":48
 *         unsigned int p3 = <unsigned char> read_char(data, pos + 3)
 *         unsigned int res = p0 | (p1 << 8) | (p2 << 16) | (p3 << 24)
 *     return res             # <<<<<<<<<<<<<<
 * 
 * cdef inline unsigned long long unpack_uint64(str data, Py_ssize_t pos):
 */
  __pyx_r = __pyx_v_res;
  goto __pyx_L0;

  /* "serializor/unicode/des.pxd":38
 *     return res
 * 
 * cdef inline unsigned int unpack_uint32(str data, Py_ssize_t pos):             # <<<<<<<<<<<<<<
 *     """Unpack (read) `UNSIGNED` 32-bit integer from 'data'
 *     at the given 'pos' in little-endian order `<'int'>`.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "serializor/unicode/des.pxd":50
 *     return res
 * 
 * cdef inline unsigned long long unpack_uint64(str data, Py_ssize_t pos):             # <<<<<<<<<<<<<<
 *     """Unpack (read) `UNSIGNED` 64-bit integer from 'data'
 *     at the given 'pos' in little-endian order `<'int'>`.
 */

static CYTHON_INLINE unsigned PY_LONG_LONG __pyx_f_10serializor_7unicode_3des_unpack_uint64(PyObject *__pyx_v_data, Py_ssize_t __pyx_v_pos) {
  unsigned PY_LONG_LONG __pyx_v_p0;
  unsigned PY_LONG_LONG __pyx_v_p1;
  unsigned PY_LONG_LONG __pyx_v_p2;
  unsigned PY_LONG_LONG __pyx_v_p3;
  unsigned PY_LONG_LONG __pyx_v_p4;
  unsigned PY_LONG_LONG __pyx_v_p5;
  unsigned PY_LONG_LONG __pyx_v_p6;
  unsigned PY_LONG_LONG __pyx_v_p7;
  unsigned PY_LONG_LONG __pyx_v_res;
  unsigned PY_LONG_LONG __pyx_r;

  /* "serializor/unicode/des.pxd":55
 *     """
 *     cdef:
 *         unsigned long long p0 = <unsigned char> read_char(data, pos)             # <<<<<<<<<<<<<<
 *         unsigned long long p1 = <unsigned char> read_char(data, pos + 1)
 *         unsigned long long p2 = <unsigned char> read_char(data, pos + 2)
 */
  __pyx_v_p0 = ((unsigned char)PyUnicode_READ_CHAR(__pyx_v_data, __pyx_v_pos));

  /* "serializor/unicode/des.pxd":56
 *     cdef:
 *         unsigned long long p0 = <unsigned char> read_char(data, pos)
 *         unsigned long long p1 = <unsigned char> read_char(data, pos + 1)             # <<<<<<<<<<<<<<
 *         unsigned long long p2 = <unsigned char> read_char(data, pos + 2)
 *         unsigned long long p3 = <unsigned char> read_char(data, pos + 3)
 */
  __pyx_v_p1 = ((unsigned char)PyUnicode_READ_CHAR(__pyx_v_data, (__pyx_v_pos + 1)));

  /* "serializor/unicode/des.pxd":57
 *         unsigned long long p0 = <unsigned char> read_char(data, pos)
 *         unsigned long long p1 = <unsigned char> read_char(data, pos + 1)
 *         unsigned long long p2 = <unsigned char> read_char(data, pos + 2)             # <<<<<<<<<<<<<<
 *         unsigned long long p3 = <unsigned char> read_char(data, pos + 3)
 *         unsigned long long p4 = <unsigned char> read_char(data, pos + 4)
 */
  __pyx_v_p2 = ((unsigned char)PyUnicode_READ_CHAR(__pyx_v_data, (__pyx_v_pos + 2)));

  /* "serializor/unicode/des.pxd":58
 *         unsigned long long p1 = <unsigned char> read_char(data, pos + 1)
 *         unsigned long long p2 = <unsigned char> read_char(data, pos + 2)
 *         unsigned long long p3 = <unsigned char> read_char(data, pos + 3)             # <<<<<<<<<<<<<<
 *         unsigned long long p4 = <unsigned char> read_char(data, pos + 4)
 *         unsigned long long p5 = <unsigned char> read_char(data, pos + 5)
 */
  __pyx_v_p3 = ((unsigned char)PyUnicode_READ_CHAR(__pyx_v_data, (__pyx_v_pos + 3)));

  /* "serializor/unicode/des.pxd":59
 *         unsigned long long p2 = <unsigned char> read_char(data, pos + 2)
 *         unsigned long long p3 = <unsigned char> read_char(data, pos + 3)
 *         unsigned long long p4 = <unsigned char> read_char(data, pos + 4)             # <<<<<<<<<<<<<<
 *         unsigned long long p5 = <unsigned char> read_char(data, pos + 5)
 *         unsigned long long p6 = <unsigned char> read_char(data, pos + 6)
 */
  __pyx_v_p4 = ((unsigned char)PyUnicode_READ_CHAR(__pyx_v_data, (__pyx_v_pos + 4)));

  /* "serializor/unicode/des.pxd":60
 *         unsigned long long p3 = <unsigned char> read_char(data, pos + 3)
 *         unsigned long long p4 = <unsigned char> read_char(data, pos + 4)
 *         unsigned long long p5 = <unsigned char> read_char(data, pos + 5)             # <<<<<<<<<<<<<<
 *         unsigned long long p6 = <unsigned char> read_char(data, pos + 6)
 *         unsigned long long p7 = <unsigned char> read_char(data, pos + 7)
 */
  __pyx_v_p5 = ((unsigned char)PyUnicode_READ_CHAR(__pyx_v_data, (__pyx_v_pos + 5)));

  /* "serializor/unicode/des.pxd":61
 *         unsigned long long p4 = <unsigned char> read_char(data, pos + 4)
 *         unsigned long long p5 = <unsigned char> read_char(data, pos + 5)
 *         unsigned long long p6 = <unsigned char> read_char(data, pos + 6)             # <<<<<<<<<<<<<<
 *         unsigned long long p7 = <unsigned char> read_char(data, pos + 7)
 *         unsigned long long res = (
 */
  __pyx_v_p6 = ((unsigned char)PyUnicode_READ_CHAR(__pyx_v_data, (__pyx_v_pos + 6)));

  /* "serializor/unicode/des.pxd":62
 *         unsigned long long p5 = <unsigned char> read_char(data, pos + 5)
 *         unsigned long long p6 = <unsigned char> read_char(data, pos + 6)
 *         unsigned long long p7 = <unsigned char> read_char(data, pos + 7)             # <<<<<<<<<<<<<<
 *         unsigned long long res = (
 *             p0 | (p1 << 8) | (p2 << 16) | (p3 << 24) | (p4 << 32)
 */
  __pyx_v_p7 = ((unsigned char)PyUnicode_READ_CHAR(__pyx_v_data, (__pyx_v_pos + 7)));

  /* "serializor/unicode/des.pxd":65
 *         unsigned long long res = (
 *             p0 | (p1 << 8) | (p2 << 16) | (p3 << 24) | (p4 << 32)
 *             | (p5 << 40) | (p6 << 48) | (p7 << 56) )             # <<<<<<<<<<<<<<
 *     return res
 * 
 */
  __pyx_v_res = (((((((__pyx_v_p0 | (__pyx_v_p1 << 8)) | (__pyx_v_p2 << 16)) | (__pyx_v_p3 << 24)) | (__pyx_v_p4 << 32)) | (__pyx_v_p5 << 40)) | (__pyx_v_p6 << 48)) | (__pyx_v_p7 << 56));

  /* "serializor/unicode/des.pxd":66
 *             p0 | (p1 << 8) | (p2 << 16) | (p3 << 24) | (p4 << 32)
 *             | (p5 << 40) | (p6 << 48) | (p7 << 56) )
 *     return res             # <<<<<<<<<<<<<<
 * 
 * # Utils: Unpack signed integer from string
 */
  __pyx_r = __pyx_v_res;
  goto __pyx_L0;

  /* "serializor/unicode/des.pxd":50
 *     return res
 * 
 * cdef inline unsigned long long unpack_uint64(str data, Py_ssize_t pos):             # <<<<<<<<<<<<<<
 *     """Unpack (read) `UNSIGNED` 64-bit integer from 'data'
 *     at the given 'pos' in little-endian order `<'int'>`.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "serializor/unicode/des.pxd":69
 * 
 * # Utils: Unpack signed integer from string
 * cdef inline signed char unpack_int8(str data, Py_ssize_t pos):             # <<<<<<<<<<<<<<
 *     """Read (unpack) `SIGNED` 8-bit integer from 'data'
 *     at givent 'pos' in little-endian order `<'int'>`."""
 */

static CYTHON_INLINE signed char __pyx_f_10serializor_7unicode_3des_unpack_int8(PyObject *__pyx_v_data, Py_ssize_t __pyx_v_pos) {
  signed char __pyx_r;

  /* "serializor/unicode/des.pxd":72
 *     """Read (unpack) `SIGNED` 8-bit integer from 'data'
 *     at givent 'pos' in little-endian order `<'int'>`."""
 *     return <signed char> read_char(data, pos)             # <<<<<<<<<<<<<<
 * 
 * cdef inline short unpack_int16(str data, Py_ssize_t pos):
 */
  __pyx_r = ((signed char)PyUnicode_READ_CHAR(__pyx_v_data, __pyx_v_pos));
  goto __pyx_L0;

  /* "serializor/unicode/des.pxd":69
 * 
 * # Utils: Unpack signed integer from string
 * cdef inline signed char unpack_int8(str data, Py_ssize_t pos):             # <<<<<<<<<<<<<<
 *     """Read (unpack) `SIGNED` 8-bit integer from 'data'
 *     at givent 'pos' in little-endian order `<'int'>`."""
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "serializor/unicode/des.pxd":74
 *     return <signed char> read_char(data, pos)
 * 
 * cdef inline short unpack_int16(str data, Py_ssize_t pos):             # <<<<<<<<<<<<<<
 *     """Read (unpack) `SIGNED` 16-bit integer from 'data'
 *     at givent 'pos' in little-endian order `<'int'>`."""
 */

static CYTHON_INLINE short __pyx_f_10serializor_7unicode_3des_unpack_int16(PyObject *__pyx_v_data, Py_ssize_t __pyx_v_pos) {
  short __pyx_r;
  unsigned short __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "serializor/unicode/des.pxd":77
 *     """Read (unpack) `SIGNED` 16-bit integer from 'data'
 *     at givent 'pos' in little-endian order `<'int'>`."""
 *     return <short> unpack_uint16(data, pos)             # <<<<<<<<<<<<<<
 * 
 * cdef inline int unpack_int24(str data, Py_ssize_t pos):
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint16(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_1 == ((unsigned short)-1) && PyErr_Occurred())) __PYX_ERR(0, 77, __pyx_L1_error)
  __pyx_r = ((short)__pyx_t_1);
  goto __pyx_L0;

  /* "serializor/unicode/des.pxd":74
 *     return <signed char> read_char(data, pos)
 * 
 * cdef inline short unpack_int16(str data, Py_ssize_t pos):             # <<<<<<<<<<<<<<
 *     """Read (unpack) `SIGNED` 16-bit integer from 'data'
 *     at givent 'pos' in little-endian order `<'int'>`."""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("serializor.unicode.des.unpack_int16", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "serializor/unicode/des.pxd":79
 *     return <short> unpack_uint16(data, pos)
 * 
 * cdef inline int unpack_int24(str data, Py_ssize_t pos):             # <<<<<<<<<<<<<<
 *     """Read (unpack) `SIGNED` 24-bit integer from 'data'
 *     at givent 'pos' in little-endian order `<'int'>`."""
 */

static CYTHON_INLINE int __pyx_f_10serializor_7unicode_3des_unpack_int24(PyObject *__pyx_v_data, Py_ssize_t __pyx_v_pos) {
  int __pyx_v_res;
  int __pyx_r;
  unsigned int __pyx_t_1;
  long __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "serializor/unicode/des.pxd":82
 *     """Read (unpack) `SIGNED` 24-bit integer from 'data'
 *     at givent 'pos' in little-endian order `<'int'>`."""
 *     cdef int res = <int> unpack_uint24(data, pos)             # <<<<<<<<<<<<<<
 *     return res if res < 0x800000 else res - 0x1000000
 * 
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint24(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_1 == ((unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 82, __pyx_L1_error)
  __pyx_v_res = ((int)__pyx_t_1);

  /* "serializor/unicode/des.pxd":83
 *     at givent 'pos' in little-endian order `<'int'>`."""
 *     cdef int res = <int> unpack_uint24(data, pos)
 *     return res if res < 0x800000 else res - 0x1000000             # <<<<<<<<<<<<<<
 * 
 * cdef inline int unpack_int32(str data, Py_ssize_t pos):
 */
  __pyx_t_3 = (__pyx_v_res < 0x800000);
  if (__pyx_t_3) {
    __pyx_t_2 = __pyx_v_res;
  } else {
    __pyx_t_2 = (__pyx_v_res - 0x1000000);
  }
  __pyx_r = __pyx_t_2;
  goto __pyx_L0;

  /* "serializor/unicode/des.pxd":79
 *     return <short> unpack_uint16(data, pos)
 * 
 * cdef inline int unpack_int24(str data, Py_ssize_t pos):             # <<<<<<<<<<<<<<
 *     """Read (unpack) `SIGNED` 24-bit integer from 'data'
 *     at givent 'pos' in little-endian order `<'int'>`."""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("serializor.unicode.des.unpack_int24", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "serializor/unicode/des.pxd":85
 *     return res if res < 0x800000 else res - 0x1000000
 * 
 * cdef inline int unpack_int32(str data, Py_ssize_t pos):             # <<<<<<<<<<<<<<
 *     """Read (unpack) `SIGNED` 32-bit integer from 'data'
 *     at givent 'pos' in little-endian order `<'int'>`."""
 */

static CYTHON_INLINE int __pyx_f_10serializor_7unicode_3des_unpack_int32(PyObject *__pyx_v_data, Py_ssize_t __pyx_v_pos) {
  int __pyx_r;
  unsigned int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "serializor/unicode/des.pxd":88
 *     """Read (unpack) `SIGNED` 32-bit integer from 'data'
 *     at givent 'pos' in little-endian order `<'int'>`."""
 *     return <int> unpack_uint32(data, pos)             # <<<<<<<<<<<<<<
 * 
 * cdef inline long long unpack_int64(str data, Py_ssize_t pos):
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint32(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_1 == ((unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 88, __pyx_L1_error)
  __pyx_r = ((int)__pyx_t_1);
  goto __pyx_L0;

  /* "serializor/unicode/des.pxd":85
 *     return res if res < 0x800000 else res - 0x1000000
 * 
 * cdef inline int unpack_int32(str data, Py_ssize_t pos):             # <<<<<<<<<<<<<<
 *     """Read (unpack) `SIGNED` 32-bit integer from 'data'
 *     at givent 'pos' in little-endian order `<'int'>`."""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("serializor.unicode.des.unpack_int32", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "serializor/unicode/des.pxd":90
 *     return <int> unpack_uint32(data, pos)
 * 
 * cdef inline long long unpack_int64(str data, Py_ssize_t pos):             # <<<<<<<<<<<<<<
 *     """Read (unpack) `SIGNED` 64-bit integer from 'data'
 *     at givent 'pos' in little-endian order `<'int'>`."""
 */

static CYTHON_INLINE PY_LONG_LONG __pyx_f_10serializor_7unicode_3des_unpack_int64(PyObject *__pyx_v_data, Py_ssize_t __pyx_v_pos) {
  PY_LONG_LONG __pyx_r;
  unsigned PY_LONG_LONG __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "serializor/unicode/des.pxd":93
 *     """Read (unpack) `SIGNED` 64-bit integer from 'data'
 *     at givent 'pos' in little-endian order `<'int'>`."""
 *     return <long long> unpack_uint64(data, pos)             # <<<<<<<<<<<<<<
 * 
 * # Encoded integer
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint64(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_1 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(0, 93, __pyx_L1_error)
  __pyx_r = ((PY_LONG_LONG)__pyx_t_1);
  goto __pyx_L0;

  /* "serializor/unicode/des.pxd":90
 *     return <int> unpack_uint32(data, pos)
 * 
 * cdef inline long long unpack_int64(str data, Py_ssize_t pos):             # <<<<<<<<<<<<<<
 *     """Read (unpack) `SIGNED` 64-bit integer from 'data'
 *     at givent 'pos' in little-endian order `<'int'>`."""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("serializor.unicode.des.unpack_int64", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "serializor/unicode/des.pxd":96
 * 
 * # Encoded integer
 * cdef inline unsigned long long dec_encoded_int(str data, Py_ssize_t pos[1]):             # <<<<<<<<<<<<<<
 *     """Decode encoded integer from 'data' at the given 'pos' pointer `<'int'>`.
 *     Note:
 */

static CYTHON_INLINE unsigned PY_LONG_LONG __pyx_f_10serializor_7unicode_3des_dec_encoded_int(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  Py_ssize_t __pyx_v_idx;
  unsigned char __pyx_v_code;
  unsigned PY_LONG_LONG __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  unsigned short __pyx_t_2;
  unsigned int __pyx_t_3;
  unsigned PY_LONG_LONG __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("dec_encoded_int", 1);

  /* "serializor/unicode/des.pxd":103
 *     """
 *     cdef:
 *         Py_ssize_t idx = pos[0]             # <<<<<<<<<<<<<<
 *         unsigned char code = <unsigned char> read_char(data, idx)
 * 
 */
  __pyx_v_idx = (__pyx_v_pos[0]);

  /* "serializor/unicode/des.pxd":104
 *     cdef:
 *         Py_ssize_t idx = pos[0]
 *         unsigned char code = <unsigned char> read_char(data, idx)             # <<<<<<<<<<<<<<
 * 
 *     if code <= UINT8_ENCODE_VALUE:
 */
  __pyx_v_code = ((unsigned char)PyUnicode_READ_CHAR(__pyx_v_data, __pyx_v_idx));

  /* "serializor/unicode/des.pxd":106
 *         unsigned char code = <unsigned char> read_char(data, idx)
 * 
 *     if code <= UINT8_ENCODE_VALUE:             # <<<<<<<<<<<<<<
 *         pos[0] = idx + 1
 *         return code
 */
  __pyx_t_1 = (__pyx_v_code <= __pyx_v_10serializor_7unicode_3ser_UINT8_ENCODE_VALUE);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.pxd":107
 * 
 *     if code <= UINT8_ENCODE_VALUE:
 *         pos[0] = idx + 1             # <<<<<<<<<<<<<<
 *         return code
 *     elif code == UINT16_ENCODE_VALUE:
 */
    (__pyx_v_pos[0]) = (__pyx_v_idx + 1);

    /* "serializor/unicode/des.pxd":108
 *     if code <= UINT8_ENCODE_VALUE:
 *         pos[0] = idx + 1
 *         return code             # <<<<<<<<<<<<<<
 *     elif code == UINT16_ENCODE_VALUE:
 *         pos[0] = idx + 3
 */
    __pyx_r = __pyx_v_code;
    goto __pyx_L0;

    /* "serializor/unicode/des.pxd":106
 *         unsigned char code = <unsigned char> read_char(data, idx)
 * 
 *     if code <= UINT8_ENCODE_VALUE:             # <<<<<<<<<<<<<<
 *         pos[0] = idx + 1
 *         return code
 */
  }

  /* "serializor/unicode/des.pxd":109
 *         pos[0] = idx + 1
 *         return code
 *     elif code == UINT16_ENCODE_VALUE:             # <<<<<<<<<<<<<<
 *         pos[0] = idx + 3
 *         return unpack_uint16(data, idx + 1)
 */
  __pyx_t_1 = (__pyx_v_code == __pyx_v_10serializor_7unicode_3ser_UINT16_ENCODE_VALUE);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.pxd":110
 *         return code
 *     elif code == UINT16_ENCODE_VALUE:
 *         pos[0] = idx + 3             # <<<<<<<<<<<<<<
 *         return unpack_uint16(data, idx + 1)
 *     elif code == UINT32_ENCODE_VALUE:
 */
    (__pyx_v_pos[0]) = (__pyx_v_idx + 3);

    /* "serializor/unicode/des.pxd":111
 *     elif code == UINT16_ENCODE_VALUE:
 *         pos[0] = idx + 3
 *         return unpack_uint16(data, idx + 1)             # <<<<<<<<<<<<<<
 *     elif code == UINT32_ENCODE_VALUE:
 *         pos[0] = idx + 5
 */
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des_unpack_uint16(__pyx_v_data, (__pyx_v_idx + 1)); if (unlikely(__pyx_t_2 == ((unsigned short)-1) && PyErr_Occurred())) __PYX_ERR(0, 111, __pyx_L1_error)
    __pyx_r = __pyx_t_2;
    goto __pyx_L0;

    /* "serializor/unicode/des.pxd":109
 *         pos[0] = idx + 1
 *         return code
 *     elif code == UINT16_ENCODE_VALUE:             # <<<<<<<<<<<<<<
 *         pos[0] = idx + 3
 *         return unpack_uint16(data, idx + 1)
 */
  }

  /* "serializor/unicode/des.pxd":112
 *         pos[0] = idx + 3
 *         return unpack_uint16(data, idx + 1)
 *     elif code == UINT32_ENCODE_VALUE:             # <<<<<<<<<<<<<<
 *         pos[0] = idx + 5
 *         return unpack_uint32(data, idx + 1)
 */
  __pyx_t_1 = (__pyx_v_code == __pyx_v_10serializor_7unicode_3ser_UINT32_ENCODE_VALUE);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.pxd":113
 *         return unpack_uint16(data, idx + 1)
 *     elif code == UINT32_ENCODE_VALUE:
 *         pos[0] = idx + 5             # <<<<<<<<<<<<<<
 *         return unpack_uint32(data, idx + 1)
 *     elif code == UINT64_ENCODE_VALUE:
 */
    (__pyx_v_pos[0]) = (__pyx_v_idx + 5);

    /* "serializor/unicode/des.pxd":114
 *     elif code == UINT32_ENCODE_VALUE:
 *         pos[0] = idx + 5
 *         return unpack_uint32(data, idx + 1)             # <<<<<<<<<<<<<<
 *     elif code == UINT64_ENCODE_VALUE:
 *         pos[0] = idx + 9
 */
    __pyx_t_3 = __pyx_f_10serializor_7unicode_3des_unpack_uint32(__pyx_v_data, (__pyx_v_idx + 1)); if (unlikely(__pyx_t_3 == ((unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 114, __pyx_L1_error)
    __pyx_r = __pyx_t_3;
    goto __pyx_L0;

    /* "serializor/unicode/des.pxd":112
 *         pos[0] = idx + 3
 *         return unpack_uint16(data, idx + 1)
 *     elif code == UINT32_ENCODE_VALUE:             # <<<<<<<<<<<<<<
 *         pos[0] = idx + 5
 *         return unpack_uint32(data, idx + 1)
 */
  }

  /* "serializor/unicode/des.pxd":115
 *         pos[0] = idx + 5
 *         return unpack_uint32(data, idx + 1)
 *     elif code == UINT64_ENCODE_VALUE:             # <<<<<<<<<<<<<<
 *         pos[0] = idx + 9
 *         return unpack_uint64(data, idx + 1)
 */
  __pyx_t_1 = (__pyx_v_code == __pyx_v_10serializor_7unicode_3ser_UINT64_ENCODE_VALUE);
  if (likely(__pyx_t_1)) {

    /* "serializor/unicode/des.pxd":116
 *         return unpack_uint32(data, idx + 1)
 *     elif code == UINT64_ENCODE_VALUE:
 *         pos[0] = idx + 9             # <<<<<<<<<<<<<<
 *         return unpack_uint64(data, idx + 1)
 *     else:
 */
    (__pyx_v_pos[0]) = (__pyx_v_idx + 9);

    /* "serializor/unicode/des.pxd":117
 *     elif code == UINT64_ENCODE_VALUE:
 *         pos[0] = idx + 9
 *         return unpack_uint64(data, idx + 1)             # <<<<<<<<<<<<<<
 *     else:
 *         raise ValueError("Invalid encoded integer value: %d." % code)
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_unpack_uint64(__pyx_v_data, (__pyx_v_idx + 1)); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(0, 117, __pyx_L1_error)
    __pyx_r = __pyx_t_4;
    goto __pyx_L0;

    /* "serializor/unicode/des.pxd":115
 *         pos[0] = idx + 5
 *         return unpack_uint32(data, idx + 1)
 *     elif code == UINT64_ENCODE_VALUE:             # <<<<<<<<<<<<<<
 *         pos[0] = idx + 9
 *         return unpack_uint64(data, idx + 1)
 */
  }

  /* "serializor/unicode/des.pxd":119
 *         return unpack_uint64(data, idx + 1)
 *     else:
 *         raise ValueError("Invalid encoded integer value: %d." % code)             # <<<<<<<<<<<<<<
 * 
 * # Deserialize
 */
  /*else*/ {
    __pyx_t_5 = __Pyx_PyInt_From_unsigned_char(__pyx_v_code); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 119, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyUnicode_Format(__pyx_kp_u_Invalid_encoded_integer_value_d, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 119, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 119, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 119, __pyx_L1_error)
  }

  /* "serializor/unicode/des.pxd":96
 * 
 * # Encoded integer
 * cdef inline unsigned long long dec_encoded_int(str data, Py_ssize_t pos[1]):             # <<<<<<<<<<<<<<
 *     """Decode encoded integer from 'data' at the given 'pos' pointer `<'int'>`.
 *     Note:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("serializor.unicode.des.dec_encoded_int", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":286
 * 
 *         @property
 *         cdef inline npy_intp itemsize(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             return PyDataType_ELSIZE(self)
 * 
 */

static CYTHON_INLINE npy_intp __pyx_f_5numpy_5dtype_8itemsize_itemsize(PyArray_Descr *__pyx_v_self) {
  npy_intp __pyx_r;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":287
 *         @property
 *         cdef inline npy_intp itemsize(self) noexcept nogil:
 *             return PyDataType_ELSIZE(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyDataType_ELSIZE(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":286
 * 
 *         @property
 *         cdef inline npy_intp itemsize(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             return PyDataType_ELSIZE(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":290
 * 
 *         @property
 *         cdef inline npy_intp alignment(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             return PyDataType_ALIGNMENT(self)
 * 
 */

static CYTHON_INLINE npy_intp __pyx_f_5numpy_5dtype_9alignment_alignment(PyArray_Descr *__pyx_v_self) {
  npy_intp __pyx_r;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":291
 *         @property
 *         cdef inline npy_intp alignment(self) noexcept nogil:
 *             return PyDataType_ALIGNMENT(self)             # <<<<<<<<<<<<<<
 * 
 *         # Use fields/names with care as they may be NULL.  You must check
 */
  __pyx_r = PyDataType_ALIGNMENT(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":290
 * 
 *         @property
 *         cdef inline npy_intp alignment(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             return PyDataType_ALIGNMENT(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":296
 *         # for this using PyDataType_HASFIELDS.
 *         @property
 *         cdef inline object fields(self):             # <<<<<<<<<<<<<<
 *             return <object>PyDataType_FIELDS(self)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_5dtype_6fields_fields(PyArray_Descr *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1;
  __Pyx_RefNannySetupContext("fields", 1);

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":297
 *         @property
 *         cdef inline object fields(self):
 *             return <object>PyDataType_FIELDS(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyDataType_FIELDS(__pyx_v_self);
  __Pyx_INCREF(((PyObject *)__pyx_t_1));
  __pyx_r = ((PyObject *)__pyx_t_1);
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":296
 *         # for this using PyDataType_HASFIELDS.
 *         @property
 *         cdef inline object fields(self):             # <<<<<<<<<<<<<<
 *             return <object>PyDataType_FIELDS(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":300
 * 
 *         @property
 *         cdef inline tuple names(self):             # <<<<<<<<<<<<<<
 *             return <tuple>PyDataType_NAMES(self)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_5dtype_5names_names(PyArray_Descr *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1;
  __Pyx_RefNannySetupContext("names", 1);

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":301
 *         @property
 *         cdef inline tuple names(self):
 *             return <tuple>PyDataType_NAMES(self)             # <<<<<<<<<<<<<<
 * 
 *         # Use PyDataType_HASSUBARRAY to test whether this field is
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyDataType_NAMES(__pyx_v_self);
  __Pyx_INCREF(((PyObject*)__pyx_t_1));
  __pyx_r = ((PyObject*)__pyx_t_1);
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":300
 * 
 *         @property
 *         cdef inline tuple names(self):             # <<<<<<<<<<<<<<
 *             return <tuple>PyDataType_NAMES(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":307
 *         # this field via the inline helper method PyDataType_SHAPE.
 *         @property
 *         cdef inline PyArray_ArrayDescr* subarray(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             return PyDataType_SUBARRAY(self)
 * 
 */

static CYTHON_INLINE PyArray_ArrayDescr *__pyx_f_5numpy_5dtype_8subarray_subarray(PyArray_Descr *__pyx_v_self) {
  PyArray_ArrayDescr *__pyx_r;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":308
 *         @property
 *         cdef inline PyArray_ArrayDescr* subarray(self) noexcept nogil:
 *             return PyDataType_SUBARRAY(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyDataType_SUBARRAY(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":307
 *         # this field via the inline helper method PyDataType_SHAPE.
 *         @property
 *         cdef inline PyArray_ArrayDescr* subarray(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             return PyDataType_SUBARRAY(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":311
 * 
 *         @property
 *         cdef inline npy_uint64 flags(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """The data types flags."""
 *             return PyDataType_FLAGS(self)
 */

static CYTHON_INLINE npy_uint64 __pyx_f_5numpy_5dtype_5flags_flags(PyArray_Descr *__pyx_v_self) {
  npy_uint64 __pyx_r;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":313
 *         cdef inline npy_uint64 flags(self) noexcept nogil:
 *             """The data types flags."""
 *             return PyDataType_FLAGS(self)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = PyDataType_FLAGS(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":311
 * 
 *         @property
 *         cdef inline npy_uint64 flags(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """The data types flags."""
 *             return PyDataType_FLAGS(self)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":323
 * 
 *         @property
 *         cdef inline int numiter(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """The number of arrays that need to be broadcast to the same shape."""
 *             return PyArray_MultiIter_NUMITER(self)
 */

static CYTHON_INLINE int __pyx_f_5numpy_9broadcast_7numiter_numiter(PyArrayMultiIterObject *__pyx_v_self) {
  int __pyx_r;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":325
 *         cdef inline int numiter(self) noexcept nogil:
 *             """The number of arrays that need to be broadcast to the same shape."""
 *             return PyArray_MultiIter_NUMITER(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyArray_MultiIter_NUMITER(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":323
 * 
 *         @property
 *         cdef inline int numiter(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """The number of arrays that need to be broadcast to the same shape."""
 *             return PyArray_MultiIter_NUMITER(self)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":328
 * 
 *         @property
 *         cdef inline npy_intp size(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """The total broadcasted size."""
 *             return PyArray_MultiIter_SIZE(self)
 */

static CYTHON_INLINE npy_intp __pyx_f_5numpy_9broadcast_4size_size(PyArrayMultiIterObject *__pyx_v_self) {
  npy_intp __pyx_r;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":330
 *         cdef inline npy_intp size(self) noexcept nogil:
 *             """The total broadcasted size."""
 *             return PyArray_MultiIter_SIZE(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyArray_MultiIter_SIZE(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":328
 * 
 *         @property
 *         cdef inline npy_intp size(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """The total broadcasted size."""
 *             return PyArray_MultiIter_SIZE(self)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":333
 * 
 *         @property
 *         cdef inline npy_intp index(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """The current (1-d) index into the broadcasted result."""
 *             return PyArray_MultiIter_INDEX(self)
 */

static CYTHON_INLINE npy_intp __pyx_f_5numpy_9broadcast_5index_index(PyArrayMultiIterObject *__pyx_v_self) {
  npy_intp __pyx_r;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":335
 *         cdef inline npy_intp index(self) noexcept nogil:
 *             """The current (1-d) index into the broadcasted result."""
 *             return PyArray_MultiIter_INDEX(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyArray_MultiIter_INDEX(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":333
 * 
 *         @property
 *         cdef inline npy_intp index(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """The current (1-d) index into the broadcasted result."""
 *             return PyArray_MultiIter_INDEX(self)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":338
 * 
 *         @property
 *         cdef inline int nd(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """The number of dimensions in the broadcasted result."""
 *             return PyArray_MultiIter_NDIM(self)
 */

static CYTHON_INLINE int __pyx_f_5numpy_9broadcast_2nd_nd(PyArrayMultiIterObject *__pyx_v_self) {
  int __pyx_r;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":340
 *         cdef inline int nd(self) noexcept nogil:
 *             """The number of dimensions in the broadcasted result."""
 *             return PyArray_MultiIter_NDIM(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyArray_MultiIter_NDIM(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":338
 * 
 *         @property
 *         cdef inline int nd(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """The number of dimensions in the broadcasted result."""
 *             return PyArray_MultiIter_NDIM(self)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":343
 * 
 *         @property
 *         cdef inline npy_intp* dimensions(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """The shape of the broadcasted result."""
 *             return PyArray_MultiIter_DIMS(self)
 */

static CYTHON_INLINE npy_intp *__pyx_f_5numpy_9broadcast_10dimensions_dimensions(PyArrayMultiIterObject *__pyx_v_self) {
  npy_intp *__pyx_r;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":345
 *         cdef inline npy_intp* dimensions(self) noexcept nogil:
 *             """The shape of the broadcasted result."""
 *             return PyArray_MultiIter_DIMS(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyArray_MultiIter_DIMS(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":343
 * 
 *         @property
 *         cdef inline npy_intp* dimensions(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """The shape of the broadcasted result."""
 *             return PyArray_MultiIter_DIMS(self)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":348
 * 
 *         @property
 *         cdef inline void** iters(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """An array of iterator objects that holds the iterators for the arrays to be broadcast together.
 *             On return, the iterators are adjusted for broadcasting."""
 */

static CYTHON_INLINE void **__pyx_f_5numpy_9broadcast_5iters_iters(PyArrayMultiIterObject *__pyx_v_self) {
  void **__pyx_r;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":351
 *             """An array of iterator objects that holds the iterators for the arrays to be broadcast together.
 *             On return, the iterators are adjusted for broadcasting."""
 *             return PyArray_MultiIter_ITERS(self)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = PyArray_MultiIter_ITERS(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":348
 * 
 *         @property
 *         cdef inline void** iters(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """An array of iterator objects that holds the iterators for the arrays to be broadcast together.
 *             On return, the iterators are adjusted for broadcasting."""
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":366
 * 
 *         @property
 *         cdef inline PyObject* base(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """Returns a borrowed reference to the object owning the data/memory.
 *             """
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_7ndarray_4base_base(PyArrayObject *__pyx_v_self) {
  PyObject *__pyx_r;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":369
 *             """Returns a borrowed reference to the object owning the data/memory.
 *             """
 *             return PyArray_BASE(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyArray_BASE(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":366
 * 
 *         @property
 *         cdef inline PyObject* base(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """Returns a borrowed reference to the object owning the data/memory.
 *             """
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":372
 * 
 *         @property
 *         cdef inline dtype descr(self):             # <<<<<<<<<<<<<<
 *             """Returns an owned reference to the dtype of the array.
 *             """
 */

static CYTHON_INLINE PyArray_Descr *__pyx_f_5numpy_7ndarray_5descr_descr(PyArrayObject *__pyx_v_self) {
  PyArray_Descr *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyArray_Descr *__pyx_t_1;
  __Pyx_RefNannySetupContext("descr", 1);

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":375
 *             """Returns an owned reference to the dtype of the array.
 *             """
 *             return <dtype>PyArray_DESCR(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_1 = PyArray_DESCR(__pyx_v_self);
  __Pyx_INCREF((PyObject *)((PyArray_Descr *)__pyx_t_1));
  __pyx_r = ((PyArray_Descr *)__pyx_t_1);
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":372
 * 
 *         @property
 *         cdef inline dtype descr(self):             # <<<<<<<<<<<<<<
 *             """Returns an owned reference to the dtype of the array.
 *             """
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":378
 * 
 *         @property
 *         cdef inline int ndim(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """Returns the number of dimensions in the array.
 *             """
 */

static CYTHON_INLINE int __pyx_f_5numpy_7ndarray_4ndim_ndim(PyArrayObject *__pyx_v_self) {
  int __pyx_r;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":381
 *             """Returns the number of dimensions in the array.
 *             """
 *             return PyArray_NDIM(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyArray_NDIM(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":378
 * 
 *         @property
 *         cdef inline int ndim(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """Returns the number of dimensions in the array.
 *             """
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":384
 * 
 *         @property
 *         cdef inline npy_intp *shape(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """Returns a pointer to the dimensions/shape of the array.
 *             The number of elements matches the number of dimensions of the array (ndim).
 */

static CYTHON_INLINE npy_intp *__pyx_f_5numpy_7ndarray_5shape_shape(PyArrayObject *__pyx_v_self) {
  npy_intp *__pyx_r;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":389
 *             Can return NULL for 0-dimensional arrays.
 *             """
 *             return PyArray_DIMS(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyArray_DIMS(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":384
 * 
 *         @property
 *         cdef inline npy_intp *shape(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """Returns a pointer to the dimensions/shape of the array.
 *             The number of elements matches the number of dimensions of the array (ndim).
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":392
 * 
 *         @property
 *         cdef inline npy_intp *strides(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """Returns a pointer to the strides of the array.
 *             The number of elements matches the number of dimensions of the array (ndim).
 */

static CYTHON_INLINE npy_intp *__pyx_f_5numpy_7ndarray_7strides_strides(PyArrayObject *__pyx_v_self) {
  npy_intp *__pyx_r;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":396
 *             The number of elements matches the number of dimensions of the array (ndim).
 *             """
 *             return PyArray_STRIDES(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyArray_STRIDES(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":392
 * 
 *         @property
 *         cdef inline npy_intp *strides(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """Returns a pointer to the strides of the array.
 *             The number of elements matches the number of dimensions of the array (ndim).
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":399
 * 
 *         @property
 *         cdef inline npy_intp size(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """Returns the total size (in number of elements) of the array.
 *             """
 */

static CYTHON_INLINE npy_intp __pyx_f_5numpy_7ndarray_4size_size(PyArrayObject *__pyx_v_self) {
  npy_intp __pyx_r;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":402
 *             """Returns the total size (in number of elements) of the array.
 *             """
 *             return PyArray_SIZE(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyArray_SIZE(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":399
 * 
 *         @property
 *         cdef inline npy_intp size(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """Returns the total size (in number of elements) of the array.
 *             """
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":405
 * 
 *         @property
 *         cdef inline char* data(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """The pointer to the data buffer as a char*.
 *             This is provided for legacy reasons to avoid direct struct field access.
 */

static CYTHON_INLINE char *__pyx_f_5numpy_7ndarray_4data_data(PyArrayObject *__pyx_v_self) {
  char *__pyx_r;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":411
 *             of `PyArray_DATA()` instead, which returns a 'void*'.
 *             """
 *             return PyArray_BYTES(self)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = PyArray_BYTES(__pyx_v_self);
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":405
 * 
 *         @property
 *         cdef inline char* data(self) noexcept nogil:             # <<<<<<<<<<<<<<
 *             """The pointer to the data buffer as a char*.
 *             This is provided for legacy reasons to avoid direct struct field access.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":806
 * ctypedef long double complex clongdouble_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 1);

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":807
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 807, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":806
 * ctypedef long double complex clongdouble_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":809
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 1);

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":810
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 810, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":809
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":812
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 1);

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":813
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 813, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":812
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":815
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 1);

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":816
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 816, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":815
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":818
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 1);

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":819
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 819, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":818
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":821
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyDataType_SHAPE(PyArray_Descr *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2;
  __Pyx_RefNannySetupContext("PyDataType_SHAPE", 1);

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":822
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  __pyx_t_1 = PyDataType_HASSUBARRAY(__pyx_v_d);
  if (__pyx_t_1) {

    /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":823
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape             # <<<<<<<<<<<<<<
 *     else:
 *         return ()
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_5numpy_5dtype_8subarray_subarray(__pyx_v_d)->shape;
    __Pyx_INCREF(((PyObject*)__pyx_t_2));
    __pyx_r = ((PyObject*)__pyx_t_2);
    goto __pyx_L0;

    /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":822
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):
 *     if PyDataType_HASSUBARRAY(d):             # <<<<<<<<<<<<<<
 *         return <tuple>d.subarray.shape
 *     else:
 */
  }

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":825
 *         return <tuple>d.subarray.shape
 *     else:
 *         return ()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_empty_tuple);
    __pyx_r = __pyx_empty_tuple;
    goto __pyx_L0;
  }

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":821
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline tuple PyDataType_SHAPE(dtype d):             # <<<<<<<<<<<<<<
 *     if PyDataType_HASSUBARRAY(d):
 *         return <tuple>d.subarray.shape
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1009
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base) except *:             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1010
 * 
 * cdef inline void set_array_base(ndarray arr, object base) except *:
 *     Py_INCREF(base) # important to do this before stealing the reference below!             # <<<<<<<<<<<<<<
 *     PyArray_SetBaseObject(arr, base)
 * 
 */
  Py_INCREF(__pyx_v_base);

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1011
 * cdef inline void set_array_base(ndarray arr, object base) except *:
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  __pyx_t_1 = PyArray_SetBaseObject(__pyx_v_arr, __pyx_v_base); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(3, 1011, __pyx_L1_error)

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1009
 *     int _import_umath() except -1
 * 
 * cdef inline void set_array_base(ndarray arr, object base) except *:             # <<<<<<<<<<<<<<
 *     Py_INCREF(base) # important to do this before stealing the reference below!
 *     PyArray_SetBaseObject(arr, base)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("numpy.set_array_base", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1013
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_v_base;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 1);

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1014
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)             # <<<<<<<<<<<<<<
 *     if base is NULL:
 *         return None
 */
  __pyx_v_base = PyArray_BASE(__pyx_v_arr);

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1015
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  __pyx_t_1 = (__pyx_v_base == NULL);
  if (__pyx_t_1) {

    /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1016
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     return <object>base
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1015
 * cdef inline object get_array_base(ndarray arr):
 *     base = PyArray_BASE(arr)
 *     if base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     return <object>base
 */
  }

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1017
 *     if base is NULL:
 *         return None
 *     return <object>base             # <<<<<<<<<<<<<<
 * 
 * # Versions of the import_* functions which are more suitable for
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_base));
  __pyx_r = ((PyObject *)__pyx_v_base);
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1013
 *     PyArray_SetBaseObject(arr, base)
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     base = PyArray_BASE(arr)
 *     if base is NULL:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1021
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_array(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_array", 1);

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1022
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1023
 * cdef inline int import_array() except -1:
 *     try:
 *         __pyx_import_array()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy._core.multiarray failed to import")
 */
      __pyx_t_4 = _import_array(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(3, 1023, __pyx_L3_error)

      /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1022
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1024
 *     try:
 *         __pyx_import_array()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy._core.multiarray failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(3, 1024, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);

      /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1025
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy._core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 1025, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(3, 1025, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;

    /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1022
 * # Cython code.
 * cdef inline int import_array() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         __pyx_import_array()
 *     except Exception:
 */
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1021
 * # Versions of the import_* functions which are more suitable for
 * # Cython code.
 * cdef inline int import_array() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         __pyx_import_array()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_array", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1027
 *         raise ImportError("numpy._core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_umath(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_umath", 1);

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1028
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1029
 * cdef inline int import_umath() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy._core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(3, 1029, __pyx_L3_error)

      /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1028
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1030
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy._core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(3, 1030, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);

      /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1031
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy._core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 1031, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(3, 1031, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;

    /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1028
 * 
 * cdef inline int import_umath() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1027
 *         raise ImportError("numpy._core.multiarray failed to import")
 * 
 * cdef inline int import_umath() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_umath", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1033
 *         raise ImportError("numpy._core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

static CYTHON_INLINE int __pyx_f_5numpy_import_ufunc(void) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("import_ufunc", 1);

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1034
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1035
 * cdef inline int import_ufunc() except -1:
 *     try:
 *         _import_umath()             # <<<<<<<<<<<<<<
 *     except Exception:
 *         raise ImportError("numpy._core.umath failed to import")
 */
      __pyx_t_4 = _import_umath(); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(3, 1035, __pyx_L3_error)

      /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1034
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1036
 *     try:
 *         _import_umath()
 *     except Exception:             # <<<<<<<<<<<<<<
 *         raise ImportError("numpy._core.umath failed to import")
 * 
 */
    __pyx_t_4 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_4) {
      __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(3, 1036, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);

      /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1037
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy._core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ImportError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(3, 1037, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(3, 1037, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;

    /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1034
 * 
 * cdef inline int import_ufunc() except -1:
 *     try:             # <<<<<<<<<<<<<<
 *         _import_umath()
 *     except Exception:
 */
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1033
 *         raise ImportError("numpy._core.umath failed to import")
 * 
 * cdef inline int import_ufunc() except -1:             # <<<<<<<<<<<<<<
 *     try:
 *         _import_umath()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("numpy.import_ufunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1040
 * 
 * 
 * cdef inline bint is_timedelta64_object(object obj) noexcept:             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.timedelta64)`
 */

static CYTHON_INLINE int __pyx_f_5numpy_is_timedelta64_object(PyObject *__pyx_v_obj) {
  int __pyx_r;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1052
 *     bool
 *     """
 *     return PyObject_TypeCheck(obj, &PyTimedeltaArrType_Type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = PyObject_TypeCheck(__pyx_v_obj, (&PyTimedeltaArrType_Type));
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1040
 * 
 * 
 * cdef inline bint is_timedelta64_object(object obj) noexcept:             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.timedelta64)`
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1055
 * 
 * 
 * cdef inline bint is_datetime64_object(object obj) noexcept:             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.datetime64)`
 */

static CYTHON_INLINE int __pyx_f_5numpy_is_datetime64_object(PyObject *__pyx_v_obj) {
  int __pyx_r;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1067
 *     bool
 *     """
 *     return PyObject_TypeCheck(obj, &PyDatetimeArrType_Type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = PyObject_TypeCheck(__pyx_v_obj, (&PyDatetimeArrType_Type));
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1055
 * 
 * 
 * cdef inline bint is_datetime64_object(object obj) noexcept:             # <<<<<<<<<<<<<<
 *     """
 *     Cython equivalent of `isinstance(obj, np.datetime64)`
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1070
 * 
 * 
 * cdef inline npy_datetime get_datetime64_value(object obj) noexcept nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy datetime64 object
 */

static CYTHON_INLINE npy_datetime __pyx_f_5numpy_get_datetime64_value(PyObject *__pyx_v_obj) {
  npy_datetime __pyx_r;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1077
 *     also needed.  That can be found using `get_datetime64_unit`.
 *     """
 *     return (<PyDatetimeScalarObject*>obj).obval             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((PyDatetimeScalarObject *)__pyx_v_obj)->obval;
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1070
 * 
 * 
 * cdef inline npy_datetime get_datetime64_value(object obj) noexcept nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy datetime64 object
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1080
 * 
 * 
 * cdef inline npy_timedelta get_timedelta64_value(object obj) noexcept nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy timedelta64 object
 */

static CYTHON_INLINE npy_timedelta __pyx_f_5numpy_get_timedelta64_value(PyObject *__pyx_v_obj) {
  npy_timedelta __pyx_r;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1084
 *     returns the int64 value underlying scalar numpy timedelta64 object
 *     """
 *     return (<PyTimedeltaScalarObject*>obj).obval             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((PyTimedeltaScalarObject *)__pyx_v_obj)->obval;
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1080
 * 
 * 
 * cdef inline npy_timedelta get_timedelta64_value(object obj) noexcept nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the int64 value underlying scalar numpy timedelta64 object
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1087
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) noexcept nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

static CYTHON_INLINE NPY_DATETIMEUNIT __pyx_f_5numpy_get_datetime64_unit(PyObject *__pyx_v_obj) {
  NPY_DATETIMEUNIT __pyx_r;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1091
 *     returns the unit part of the dtype for a numpy datetime64 object.
 *     """
 *     return <NPY_DATETIMEUNIT>(<PyDatetimeScalarObject*>obj).obmeta.base             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((NPY_DATETIMEUNIT)((PyDatetimeScalarObject *)__pyx_v_obj)->obmeta.base);
  goto __pyx_L0;

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1087
 * 
 * 
 * cdef inline NPY_DATETIMEUNIT get_datetime64_unit(object obj) noexcept nogil:             # <<<<<<<<<<<<<<
 *     """
 *     returns the unit part of the dtype for a numpy datetime64 object.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "cpython/complex.pxd":19
 * 
 *         @property
 *         cdef inline double real(self) noexcept:             # <<<<<<<<<<<<<<
 *             return self.cval.real
 * 
 */

static CYTHON_INLINE double __pyx_f_7cpython_7complex_7complex_4real_real(PyComplexObject *__pyx_v_self) {
  double __pyx_r;

  /* "cpython/complex.pxd":20
 *         @property
 *         cdef inline double real(self) noexcept:
 *             return self.cval.real             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = __pyx_v_self->cval.real;
  goto __pyx_L0;

  /* "cpython/complex.pxd":19
 * 
 *         @property
 *         cdef inline double real(self) noexcept:             # <<<<<<<<<<<<<<
 *             return self.cval.real
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "cpython/complex.pxd":23
 * 
 *         @property
 *         cdef inline double imag(self) noexcept:             # <<<<<<<<<<<<<<
 *             return self.cval.imag
 * 
 */

static CYTHON_INLINE double __pyx_f_7cpython_7complex_7complex_4imag_imag(PyComplexObject *__pyx_v_self) {
  double __pyx_r;

  /* "cpython/complex.pxd":24
 *         @property
 *         cdef inline double imag(self) noexcept:
 *             return self.cval.imag             # <<<<<<<<<<<<<<
 * 
 *     # PyTypeObject PyComplex_Type
 */
  __pyx_r = __pyx_v_self->cval.imag;
  goto __pyx_L0;

  /* "cpython/complex.pxd":23
 * 
 *         @property
 *         cdef inline double imag(self) noexcept:             # <<<<<<<<<<<<<<
 *             return self.cval.imag
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "cpython/contextvars.pxd":112
 * 
 * 
 * cdef inline object get_value(var, default_value=None):             # <<<<<<<<<<<<<<
 *     """Return a new reference to the value of the context variable,
 *     or the default value of the context variable,
 */

static CYTHON_INLINE PyObject *__pyx_f_7cpython_11contextvars_get_value(PyObject *__pyx_v_var, struct __pyx_opt_args_7cpython_11contextvars_get_value *__pyx_optional_args) {
  PyObject *__pyx_v_default_value = ((PyObject *)Py_None);
  PyObject *__pyx_v_value;
  PyObject *__pyx_v_pyvalue = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_value", 1);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_default_value = __pyx_optional_args->default_value;
    }
  }

  /* "cpython/contextvars.pxd":117
 *     or None if no such value or default was found.
 *     """
 *     cdef PyObject *value = NULL             # <<<<<<<<<<<<<<
 *     PyContextVar_Get(var, NULL, &value)
 *     if value is NULL:
 */
  __pyx_v_value = NULL;

  /* "cpython/contextvars.pxd":118
 *     """
 *     cdef PyObject *value = NULL
 *     PyContextVar_Get(var, NULL, &value)             # <<<<<<<<<<<<<<
 *     if value is NULL:
 *         # context variable does not have a default
 */
  __pyx_t_1 = PyContextVar_Get(__pyx_v_var, NULL, (&__pyx_v_value)); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(4, 118, __pyx_L1_error)

  /* "cpython/contextvars.pxd":119
 *     cdef PyObject *value = NULL
 *     PyContextVar_Get(var, NULL, &value)
 *     if value is NULL:             # <<<<<<<<<<<<<<
 *         # context variable does not have a default
 *         pyvalue = default_value
 */
  __pyx_t_2 = (__pyx_v_value == NULL);
  if (__pyx_t_2) {

    /* "cpython/contextvars.pxd":121
 *     if value is NULL:
 *         # context variable does not have a default
 *         pyvalue = default_value             # <<<<<<<<<<<<<<
 *     else:
 *         # value or default value of context variable
 */
    __Pyx_INCREF(__pyx_v_default_value);
    __pyx_v_pyvalue = __pyx_v_default_value;

    /* "cpython/contextvars.pxd":119
 *     cdef PyObject *value = NULL
 *     PyContextVar_Get(var, NULL, &value)
 *     if value is NULL:             # <<<<<<<<<<<<<<
 *         # context variable does not have a default
 *         pyvalue = default_value
 */
    goto __pyx_L3;
  }

  /* "cpython/contextvars.pxd":124
 *     else:
 *         # value or default value of context variable
 *         pyvalue = <object>value             # <<<<<<<<<<<<<<
 *         Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'
 *     return pyvalue
 */
  /*else*/ {
    __pyx_t_3 = ((PyObject *)__pyx_v_value);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_v_pyvalue = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cpython/contextvars.pxd":125
 *         # value or default value of context variable
 *         pyvalue = <object>value
 *         Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'             # <<<<<<<<<<<<<<
 *     return pyvalue
 * 
 */
    Py_XDECREF(__pyx_v_value);
  }
  __pyx_L3:;

  /* "cpython/contextvars.pxd":126
 *         pyvalue = <object>value
 *         Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'
 *     return pyvalue             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_pyvalue);
  __pyx_r = __pyx_v_pyvalue;
  goto __pyx_L0;

  /* "cpython/contextvars.pxd":112
 * 
 * 
 * cdef inline object get_value(var, default_value=None):             # <<<<<<<<<<<<<<
 *     """Return a new reference to the value of the context variable,
 *     or the default value of the context variable,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cpython.contextvars.get_value", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_pyvalue);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cpython/contextvars.pxd":129
 * 
 * 
 * cdef inline object get_value_no_default(var, default_value=None):             # <<<<<<<<<<<<<<
 *     """Return a new reference to the value of the context variable,
 *     or the provided default value if no such value was found.
 */

static CYTHON_INLINE PyObject *__pyx_f_7cpython_11contextvars_get_value_no_default(PyObject *__pyx_v_var, struct __pyx_opt_args_7cpython_11contextvars_get_value_no_default *__pyx_optional_args) {
  PyObject *__pyx_v_default_value = ((PyObject *)Py_None);
  PyObject *__pyx_v_value;
  PyObject *__pyx_v_pyvalue = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_value_no_default", 1);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_default_value = __pyx_optional_args->default_value;
    }
  }

  /* "cpython/contextvars.pxd":135
 *     Ignores the default value of the context variable, if any.
 *     """
 *     cdef PyObject *value = NULL             # <<<<<<<<<<<<<<
 *     PyContextVar_Get(var, <PyObject*>default_value, &value)
 *     # value of context variable or 'default_value'
 */
  __pyx_v_value = NULL;

  /* "cpython/contextvars.pxd":136
 *     """
 *     cdef PyObject *value = NULL
 *     PyContextVar_Get(var, <PyObject*>default_value, &value)             # <<<<<<<<<<<<<<
 *     # value of context variable or 'default_value'
 *     pyvalue = <object>value
 */
  __pyx_t_1 = PyContextVar_Get(__pyx_v_var, ((PyObject *)__pyx_v_default_value), (&__pyx_v_value)); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(4, 136, __pyx_L1_error)

  /* "cpython/contextvars.pxd":138
 *     PyContextVar_Get(var, <PyObject*>default_value, &value)
 *     # value of context variable or 'default_value'
 *     pyvalue = <object>value             # <<<<<<<<<<<<<<
 *     Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'
 *     return pyvalue
 */
  __pyx_t_2 = ((PyObject *)__pyx_v_value);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_v_pyvalue = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cpython/contextvars.pxd":139
 *     # value of context variable or 'default_value'
 *     pyvalue = <object>value
 *     Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'             # <<<<<<<<<<<<<<
 *     return pyvalue
 */
  Py_XDECREF(__pyx_v_value);

  /* "cpython/contextvars.pxd":140
 *     pyvalue = <object>value
 *     Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'
 *     return pyvalue             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_pyvalue);
  __pyx_r = __pyx_v_pyvalue;
  goto __pyx_L0;

  /* "cpython/contextvars.pxd":129
 * 
 * 
 * cdef inline object get_value_no_default(var, default_value=None):             # <<<<<<<<<<<<<<
 *     """Return a new reference to the value of the context variable,
 *     or the provided default value if no such value was found.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cpython.contextvars.get_value_no_default", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_pyvalue);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "datetime.pxd":72
 *     ctypedef extern class datetime.date[object PyDateTime_Date]:
 *         @property
 *         cdef inline int year(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_GET_YEAR(self)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_4date_4year_year(PyDateTime_Date *__pyx_v_self) {
  int __pyx_r;

  /* "datetime.pxd":73
 *         @property
 *         cdef inline int year(self) noexcept:
 *             return PyDateTime_GET_YEAR(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyDateTime_GET_YEAR(((PyObject *)__pyx_v_self));
  goto __pyx_L0;

  /* "datetime.pxd":72
 *     ctypedef extern class datetime.date[object PyDateTime_Date]:
 *         @property
 *         cdef inline int year(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_GET_YEAR(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":76
 * 
 *         @property
 *         cdef inline int month(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_GET_MONTH(self)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_4date_5month_month(PyDateTime_Date *__pyx_v_self) {
  int __pyx_r;

  /* "datetime.pxd":77
 *         @property
 *         cdef inline int month(self) noexcept:
 *             return PyDateTime_GET_MONTH(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyDateTime_GET_MONTH(((PyObject *)__pyx_v_self));
  goto __pyx_L0;

  /* "datetime.pxd":76
 * 
 *         @property
 *         cdef inline int month(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_GET_MONTH(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":80
 * 
 *         @property
 *         cdef inline int day(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_GET_DAY(self)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_4date_3day_day(PyDateTime_Date *__pyx_v_self) {
  int __pyx_r;

  /* "datetime.pxd":81
 *         @property
 *         cdef inline int day(self) noexcept:
 *             return PyDateTime_GET_DAY(self)             # <<<<<<<<<<<<<<
 * 
 *     ctypedef extern class datetime.time[object PyDateTime_Time]:
 */
  __pyx_r = PyDateTime_GET_DAY(((PyObject *)__pyx_v_self));
  goto __pyx_L0;

  /* "datetime.pxd":80
 * 
 *         @property
 *         cdef inline int day(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_GET_DAY(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":85
 *     ctypedef extern class datetime.time[object PyDateTime_Time]:
 *         @property
 *         cdef inline int hour(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_TIME_GET_HOUR(self)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_4time_4hour_hour(PyDateTime_Time *__pyx_v_self) {
  int __pyx_r;

  /* "datetime.pxd":86
 *         @property
 *         cdef inline int hour(self) noexcept:
 *             return PyDateTime_TIME_GET_HOUR(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyDateTime_TIME_GET_HOUR(((PyObject *)__pyx_v_self));
  goto __pyx_L0;

  /* "datetime.pxd":85
 *     ctypedef extern class datetime.time[object PyDateTime_Time]:
 *         @property
 *         cdef inline int hour(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_TIME_GET_HOUR(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":89
 * 
 *         @property
 *         cdef inline int minute(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_TIME_GET_MINUTE(self)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_4time_6minute_minute(PyDateTime_Time *__pyx_v_self) {
  int __pyx_r;

  /* "datetime.pxd":90
 *         @property
 *         cdef inline int minute(self) noexcept:
 *             return PyDateTime_TIME_GET_MINUTE(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyDateTime_TIME_GET_MINUTE(((PyObject *)__pyx_v_self));
  goto __pyx_L0;

  /* "datetime.pxd":89
 * 
 *         @property
 *         cdef inline int minute(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_TIME_GET_MINUTE(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":93
 * 
 *         @property
 *         cdef inline int second(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_TIME_GET_SECOND(self)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_4time_6second_second(PyDateTime_Time *__pyx_v_self) {
  int __pyx_r;

  /* "datetime.pxd":94
 *         @property
 *         cdef inline int second(self) noexcept:
 *             return PyDateTime_TIME_GET_SECOND(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyDateTime_TIME_GET_SECOND(((PyObject *)__pyx_v_self));
  goto __pyx_L0;

  /* "datetime.pxd":93
 * 
 *         @property
 *         cdef inline int second(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_TIME_GET_SECOND(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":97
 * 
 *         @property
 *         cdef inline int microsecond(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_TIME_GET_MICROSECOND(self)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_4time_11microsecond_microsecond(PyDateTime_Time *__pyx_v_self) {
  int __pyx_r;

  /* "datetime.pxd":98
 *         @property
 *         cdef inline int microsecond(self) noexcept:
 *             return PyDateTime_TIME_GET_MICROSECOND(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyDateTime_TIME_GET_MICROSECOND(((PyObject *)__pyx_v_self));
  goto __pyx_L0;

  /* "datetime.pxd":97
 * 
 *         @property
 *         cdef inline int microsecond(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_TIME_GET_MICROSECOND(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":101
 * 
 *         @property
 *         cdef inline object tzinfo(self):             # <<<<<<<<<<<<<<
 *             return <object>PyDateTime_TIME_GET_TZINFO(self)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_7cpython_8datetime_4time_6tzinfo_tzinfo(PyDateTime_Time *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1;
  __Pyx_RefNannySetupContext("tzinfo", 1);

  /* "datetime.pxd":102
 *         @property
 *         cdef inline object tzinfo(self):
 *             return <object>PyDateTime_TIME_GET_TZINFO(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyDateTime_TIME_GET_TZINFO(((PyObject *)__pyx_v_self));
  __Pyx_INCREF(((PyObject *)__pyx_t_1));
  __pyx_r = ((PyObject *)__pyx_t_1);
  goto __pyx_L0;

  /* "datetime.pxd":101
 * 
 *         @property
 *         cdef inline object tzinfo(self):             # <<<<<<<<<<<<<<
 *             return <object>PyDateTime_TIME_GET_TZINFO(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "datetime.pxd":105
 * 
 *         @property
 *         cdef inline int fold(self) noexcept:             # <<<<<<<<<<<<<<
 *             # For Python < 3.6 this returns 0 no matter what
 *             return PyDateTime_TIME_GET_FOLD(self)
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_4time_4fold_fold(PyDateTime_Time *__pyx_v_self) {
  int __pyx_r;

  /* "datetime.pxd":107
 *         cdef inline int fold(self) noexcept:
 *             # For Python < 3.6 this returns 0 no matter what
 *             return PyDateTime_TIME_GET_FOLD(self)             # <<<<<<<<<<<<<<
 * 
 *     ctypedef extern class datetime.datetime[object PyDateTime_DateTime]:
 */
  __pyx_r = PyDateTime_TIME_GET_FOLD(((PyObject *)__pyx_v_self));
  goto __pyx_L0;

  /* "datetime.pxd":105
 * 
 *         @property
 *         cdef inline int fold(self) noexcept:             # <<<<<<<<<<<<<<
 *             # For Python < 3.6 this returns 0 no matter what
 *             return PyDateTime_TIME_GET_FOLD(self)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":111
 *     ctypedef extern class datetime.datetime[object PyDateTime_DateTime]:
 *         @property
 *         cdef inline int year(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_GET_YEAR(self)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_8datetime_4year_year(PyDateTime_DateTime *__pyx_v_self) {
  int __pyx_r;

  /* "datetime.pxd":112
 *         @property
 *         cdef inline int year(self) noexcept:
 *             return PyDateTime_GET_YEAR(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyDateTime_GET_YEAR(((PyObject *)__pyx_v_self));
  goto __pyx_L0;

  /* "datetime.pxd":111
 *     ctypedef extern class datetime.datetime[object PyDateTime_DateTime]:
 *         @property
 *         cdef inline int year(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_GET_YEAR(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":115
 * 
 *         @property
 *         cdef inline int month(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_GET_MONTH(self)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_8datetime_5month_month(PyDateTime_DateTime *__pyx_v_self) {
  int __pyx_r;

  /* "datetime.pxd":116
 *         @property
 *         cdef inline int month(self) noexcept:
 *             return PyDateTime_GET_MONTH(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyDateTime_GET_MONTH(((PyObject *)__pyx_v_self));
  goto __pyx_L0;

  /* "datetime.pxd":115
 * 
 *         @property
 *         cdef inline int month(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_GET_MONTH(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":119
 * 
 *         @property
 *         cdef inline int day(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_GET_DAY(self)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_8datetime_3day_day(PyDateTime_DateTime *__pyx_v_self) {
  int __pyx_r;

  /* "datetime.pxd":120
 *         @property
 *         cdef inline int day(self) noexcept:
 *             return PyDateTime_GET_DAY(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyDateTime_GET_DAY(((PyObject *)__pyx_v_self));
  goto __pyx_L0;

  /* "datetime.pxd":119
 * 
 *         @property
 *         cdef inline int day(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_GET_DAY(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":123
 * 
 *         @property
 *         cdef inline int hour(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_DATE_GET_HOUR(self)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_8datetime_4hour_hour(PyDateTime_DateTime *__pyx_v_self) {
  int __pyx_r;

  /* "datetime.pxd":124
 *         @property
 *         cdef inline int hour(self) noexcept:
 *             return PyDateTime_DATE_GET_HOUR(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyDateTime_DATE_GET_HOUR(((PyObject *)__pyx_v_self));
  goto __pyx_L0;

  /* "datetime.pxd":123
 * 
 *         @property
 *         cdef inline int hour(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_DATE_GET_HOUR(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":127
 * 
 *         @property
 *         cdef inline int minute(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_DATE_GET_MINUTE(self)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_8datetime_6minute_minute(PyDateTime_DateTime *__pyx_v_self) {
  int __pyx_r;

  /* "datetime.pxd":128
 *         @property
 *         cdef inline int minute(self) noexcept:
 *             return PyDateTime_DATE_GET_MINUTE(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyDateTime_DATE_GET_MINUTE(((PyObject *)__pyx_v_self));
  goto __pyx_L0;

  /* "datetime.pxd":127
 * 
 *         @property
 *         cdef inline int minute(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_DATE_GET_MINUTE(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":131
 * 
 *         @property
 *         cdef inline int second(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_DATE_GET_SECOND(self)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_8datetime_6second_second(PyDateTime_DateTime *__pyx_v_self) {
  int __pyx_r;

  /* "datetime.pxd":132
 *         @property
 *         cdef inline int second(self) noexcept:
 *             return PyDateTime_DATE_GET_SECOND(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyDateTime_DATE_GET_SECOND(((PyObject *)__pyx_v_self));
  goto __pyx_L0;

  /* "datetime.pxd":131
 * 
 *         @property
 *         cdef inline int second(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_DATE_GET_SECOND(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":135
 * 
 *         @property
 *         cdef inline int microsecond(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_DATE_GET_MICROSECOND(self)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_8datetime_11microsecond_microsecond(PyDateTime_DateTime *__pyx_v_self) {
  int __pyx_r;

  /* "datetime.pxd":136
 *         @property
 *         cdef inline int microsecond(self) noexcept:
 *             return PyDateTime_DATE_GET_MICROSECOND(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyDateTime_DATE_GET_MICROSECOND(((PyObject *)__pyx_v_self));
  goto __pyx_L0;

  /* "datetime.pxd":135
 * 
 *         @property
 *         cdef inline int microsecond(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_DATE_GET_MICROSECOND(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":139
 * 
 *         @property
 *         cdef inline object tzinfo(self):             # <<<<<<<<<<<<<<
 *             return <object>PyDateTime_DATE_GET_TZINFO(self)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_7cpython_8datetime_8datetime_6tzinfo_tzinfo(PyDateTime_DateTime *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1;
  __Pyx_RefNannySetupContext("tzinfo", 1);

  /* "datetime.pxd":140
 *         @property
 *         cdef inline object tzinfo(self):
 *             return <object>PyDateTime_DATE_GET_TZINFO(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyDateTime_DATE_GET_TZINFO(((PyObject *)__pyx_v_self));
  __Pyx_INCREF(((PyObject *)__pyx_t_1));
  __pyx_r = ((PyObject *)__pyx_t_1);
  goto __pyx_L0;

  /* "datetime.pxd":139
 * 
 *         @property
 *         cdef inline object tzinfo(self):             # <<<<<<<<<<<<<<
 *             return <object>PyDateTime_DATE_GET_TZINFO(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "datetime.pxd":143
 * 
 *         @property
 *         cdef inline int fold(self) noexcept:             # <<<<<<<<<<<<<<
 *             # For Python < 3.6 this returns 0 no matter what
 *             return PyDateTime_DATE_GET_FOLD(self)
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_8datetime_4fold_fold(PyDateTime_DateTime *__pyx_v_self) {
  int __pyx_r;

  /* "datetime.pxd":145
 *         cdef inline int fold(self) noexcept:
 *             # For Python < 3.6 this returns 0 no matter what
 *             return PyDateTime_DATE_GET_FOLD(self)             # <<<<<<<<<<<<<<
 * 
 *     ctypedef extern class datetime.timedelta[object PyDateTime_Delta]:
 */
  __pyx_r = PyDateTime_DATE_GET_FOLD(((PyObject *)__pyx_v_self));
  goto __pyx_L0;

  /* "datetime.pxd":143
 * 
 *         @property
 *         cdef inline int fold(self) noexcept:             # <<<<<<<<<<<<<<
 *             # For Python < 3.6 this returns 0 no matter what
 *             return PyDateTime_DATE_GET_FOLD(self)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":149
 *     ctypedef extern class datetime.timedelta[object PyDateTime_Delta]:
 *         @property
 *         cdef inline int day(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_DELTA_GET_DAYS(self)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_9timedelta_3day_day(PyDateTime_Delta *__pyx_v_self) {
  int __pyx_r;

  /* "datetime.pxd":150
 *         @property
 *         cdef inline int day(self) noexcept:
 *             return PyDateTime_DELTA_GET_DAYS(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyDateTime_DELTA_GET_DAYS(((PyObject *)__pyx_v_self));
  goto __pyx_L0;

  /* "datetime.pxd":149
 *     ctypedef extern class datetime.timedelta[object PyDateTime_Delta]:
 *         @property
 *         cdef inline int day(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_DELTA_GET_DAYS(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":153
 * 
 *         @property
 *         cdef inline int second(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_DELTA_GET_SECONDS(self)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_9timedelta_6second_second(PyDateTime_Delta *__pyx_v_self) {
  int __pyx_r;

  /* "datetime.pxd":154
 *         @property
 *         cdef inline int second(self) noexcept:
 *             return PyDateTime_DELTA_GET_SECONDS(self)             # <<<<<<<<<<<<<<
 * 
 *         @property
 */
  __pyx_r = PyDateTime_DELTA_GET_SECONDS(((PyObject *)__pyx_v_self));
  goto __pyx_L0;

  /* "datetime.pxd":153
 * 
 *         @property
 *         cdef inline int second(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_DELTA_GET_SECONDS(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":157
 * 
 *         @property
 *         cdef inline int microsecond(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_DELTA_GET_MICROSECONDS(self)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_9timedelta_11microsecond_microsecond(PyDateTime_Delta *__pyx_v_self) {
  int __pyx_r;

  /* "datetime.pxd":158
 *         @property
 *         cdef inline int microsecond(self) noexcept:
 *             return PyDateTime_DELTA_GET_MICROSECONDS(self)             # <<<<<<<<<<<<<<
 * 
 *     ctypedef extern class datetime.tzinfo[object PyDateTime_TZInfo]:
 */
  __pyx_r = PyDateTime_DELTA_GET_MICROSECONDS(((PyObject *)__pyx_v_self));
  goto __pyx_L0;

  /* "datetime.pxd":157
 * 
 *         @property
 *         cdef inline int microsecond(self) noexcept:             # <<<<<<<<<<<<<<
 *             return PyDateTime_DELTA_GET_MICROSECONDS(self)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":282
 * # Datetime C API initialization function.
 * # You have to call it before any usage of DateTime CAPI functions.
 * cdef inline void import_datetime() noexcept:             # <<<<<<<<<<<<<<
 *     PyDateTime_IMPORT
 * 
 */

static CYTHON_INLINE void __pyx_f_7cpython_8datetime_import_datetime(void) {

  /* "datetime.pxd":283
 * # You have to call it before any usage of DateTime CAPI functions.
 * cdef inline void import_datetime() noexcept:
 *     PyDateTime_IMPORT             # <<<<<<<<<<<<<<
 * 
 * # Create date object using DateTime CAPI factory function.
 */
  (void)(PyDateTime_IMPORT);

  /* "datetime.pxd":282
 * # Datetime C API initialization function.
 * # You have to call it before any usage of DateTime CAPI functions.
 * cdef inline void import_datetime() noexcept:             # <<<<<<<<<<<<<<
 *     PyDateTime_IMPORT
 * 
 */

  /* function exit code */
}

/* "datetime.pxd":287
 * # Create date object using DateTime CAPI factory function.
 * # Note, there are no range checks for any of the arguments.
 * cdef inline date date_new(int year, int month, int day):             # <<<<<<<<<<<<<<
 *     return PyDateTimeAPI.Date_FromDate(year, month, day, PyDateTimeAPI.DateType)
 * 
 */

static CYTHON_INLINE PyDateTime_Date *__pyx_f_7cpython_8datetime_date_new(int __pyx_v_year, int __pyx_v_month, int __pyx_v_day) {
  PyDateTime_Date *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("date_new", 1);

  /* "datetime.pxd":288
 * # Note, there are no range checks for any of the arguments.
 * cdef inline date date_new(int year, int month, int day):
 *     return PyDateTimeAPI.Date_FromDate(year, month, day, PyDateTimeAPI.DateType)             # <<<<<<<<<<<<<<
 * 
 * # Create time object using DateTime CAPI factory function
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_1 = ((PyObject *)PyDateTimeAPI->Date_FromDate(__pyx_v_year, __pyx_v_month, __pyx_v_day, PyDateTimeAPI->DateType)); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 288, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = ((PyDateTime_Date *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "datetime.pxd":287
 * # Create date object using DateTime CAPI factory function.
 * # Note, there are no range checks for any of the arguments.
 * cdef inline date date_new(int year, int month, int day):             # <<<<<<<<<<<<<<
 *     return PyDateTimeAPI.Date_FromDate(year, month, day, PyDateTimeAPI.DateType)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cpython.datetime.date_new", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "datetime.pxd":292
 * # Create time object using DateTime CAPI factory function
 * # Note, there are no range checks for any of the arguments.
 * cdef inline time time_new(int hour, int minute, int second, int microsecond, object tz, int fold=0):             # <<<<<<<<<<<<<<
 *     return __Pyx_DateTime_TimeWithFold(hour, minute, second, microsecond, tz, fold)
 * 
 */

static CYTHON_INLINE PyDateTime_Time *__pyx_f_7cpython_8datetime_time_new(int __pyx_v_hour, int __pyx_v_minute, int __pyx_v_second, int __pyx_v_microsecond, PyObject *__pyx_v_tz, struct __pyx_opt_args_7cpython_8datetime_time_new *__pyx_optional_args) {
  int __pyx_v_fold = ((int)0);
  PyDateTime_Time *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("time_new", 1);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_fold = __pyx_optional_args->fold;
    }
  }

  /* "datetime.pxd":293
 * # Note, there are no range checks for any of the arguments.
 * cdef inline time time_new(int hour, int minute, int second, int microsecond, object tz, int fold=0):
 *     return __Pyx_DateTime_TimeWithFold(hour, minute, second, microsecond, tz, fold)             # <<<<<<<<<<<<<<
 * 
 * # Create datetime object using DateTime CAPI factory function.
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_1 = ((PyObject *)__Pyx_DateTime_TimeWithFold(__pyx_v_hour, __pyx_v_minute, __pyx_v_second, __pyx_v_microsecond, __pyx_v_tz, __pyx_v_fold)); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 293, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_7cpython_8datetime_time))))) __PYX_ERR(5, 293, __pyx_L1_error)
  __pyx_r = ((PyDateTime_Time *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "datetime.pxd":292
 * # Create time object using DateTime CAPI factory function
 * # Note, there are no range checks for any of the arguments.
 * cdef inline time time_new(int hour, int minute, int second, int microsecond, object tz, int fold=0):             # <<<<<<<<<<<<<<
 *     return __Pyx_DateTime_TimeWithFold(hour, minute, second, microsecond, tz, fold)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cpython.datetime.time_new", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "datetime.pxd":297
 * # Create datetime object using DateTime CAPI factory function.
 * # Note, there are no range checks for any of the arguments.
 * cdef inline datetime datetime_new(int year, int month, int day, int hour, int minute, int second, int microsecond, object tz, int fold=0):             # <<<<<<<<<<<<<<
 *     return __Pyx_DateTime_DateTimeWithFold(year, month, day, hour, minute, second, microsecond, tz, fold)
 * 
 */

static CYTHON_INLINE PyDateTime_DateTime *__pyx_f_7cpython_8datetime_datetime_new(int __pyx_v_year, int __pyx_v_month, int __pyx_v_day, int __pyx_v_hour, int __pyx_v_minute, int __pyx_v_second, int __pyx_v_microsecond, PyObject *__pyx_v_tz, struct __pyx_opt_args_7cpython_8datetime_datetime_new *__pyx_optional_args) {
  int __pyx_v_fold = ((int)0);
  PyDateTime_DateTime *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("datetime_new", 1);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_fold = __pyx_optional_args->fold;
    }
  }

  /* "datetime.pxd":298
 * # Note, there are no range checks for any of the arguments.
 * cdef inline datetime datetime_new(int year, int month, int day, int hour, int minute, int second, int microsecond, object tz, int fold=0):
 *     return __Pyx_DateTime_DateTimeWithFold(year, month, day, hour, minute, second, microsecond, tz, fold)             # <<<<<<<<<<<<<<
 * 
 * # Create timedelta object using DateTime CAPI factory function.
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_1 = ((PyObject *)__Pyx_DateTime_DateTimeWithFold(__pyx_v_year, __pyx_v_month, __pyx_v_day, __pyx_v_hour, __pyx_v_minute, __pyx_v_second, __pyx_v_microsecond, __pyx_v_tz, __pyx_v_fold)); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 298, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = ((PyDateTime_DateTime *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "datetime.pxd":297
 * # Create datetime object using DateTime CAPI factory function.
 * # Note, there are no range checks for any of the arguments.
 * cdef inline datetime datetime_new(int year, int month, int day, int hour, int minute, int second, int microsecond, object tz, int fold=0):             # <<<<<<<<<<<<<<
 *     return __Pyx_DateTime_DateTimeWithFold(year, month, day, hour, minute, second, microsecond, tz, fold)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cpython.datetime.datetime_new", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "datetime.pxd":302
 * # Create timedelta object using DateTime CAPI factory function.
 * # Note, there are no range checks for any of the arguments.
 * cdef inline timedelta timedelta_new(int days, int seconds, int useconds):             # <<<<<<<<<<<<<<
 *     return PyDateTimeAPI.Delta_FromDelta(days, seconds, useconds, 1, PyDateTimeAPI.DeltaType)
 * 
 */

static CYTHON_INLINE PyDateTime_Delta *__pyx_f_7cpython_8datetime_timedelta_new(int __pyx_v_days, int __pyx_v_seconds, int __pyx_v_useconds) {
  PyDateTime_Delta *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("timedelta_new", 1);

  /* "datetime.pxd":303
 * # Note, there are no range checks for any of the arguments.
 * cdef inline timedelta timedelta_new(int days, int seconds, int useconds):
 *     return PyDateTimeAPI.Delta_FromDelta(days, seconds, useconds, 1, PyDateTimeAPI.DeltaType)             # <<<<<<<<<<<<<<
 * 
 * # Create timedelta object using DateTime CAPI factory function.
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_1 = ((PyObject *)PyDateTimeAPI->Delta_FromDelta(__pyx_v_days, __pyx_v_seconds, __pyx_v_useconds, 1, PyDateTimeAPI->DeltaType)); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 303, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = ((PyDateTime_Delta *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "datetime.pxd":302
 * # Create timedelta object using DateTime CAPI factory function.
 * # Note, there are no range checks for any of the arguments.
 * cdef inline timedelta timedelta_new(int days, int seconds, int useconds):             # <<<<<<<<<<<<<<
 *     return PyDateTimeAPI.Delta_FromDelta(days, seconds, useconds, 1, PyDateTimeAPI.DeltaType)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cpython.datetime.timedelta_new", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "datetime.pxd":306
 * 
 * # Create timedelta object using DateTime CAPI factory function.
 * cdef inline object timezone_new(object offset, object name=None):             # <<<<<<<<<<<<<<
 *     if PY_VERSION_HEX < 0x030700b1:
 *         raise RuntimeError('Time zones are not available from the C-API.')
 */

static CYTHON_INLINE PyObject *__pyx_f_7cpython_8datetime_timezone_new(PyObject *__pyx_v_offset, struct __pyx_opt_args_7cpython_8datetime_timezone_new *__pyx_optional_args) {
  PyObject *__pyx_v_name = ((PyObject *)Py_None);
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("timezone_new", 1);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_name = __pyx_optional_args->name;
    }
  }

  /* "datetime.pxd":307
 * # Create timedelta object using DateTime CAPI factory function.
 * cdef inline object timezone_new(object offset, object name=None):
 *     if PY_VERSION_HEX < 0x030700b1:             # <<<<<<<<<<<<<<
 *         raise RuntimeError('Time zones are not available from the C-API.')
 *     return __Pyx_TimeZone_FromOffsetAndName(offset, <PyObject*>name if name is not None else NULL)
 */
  __pyx_t_1 = (PY_VERSION_HEX < 0x030700b1);
  if (unlikely(__pyx_t_1)) {

    /* "datetime.pxd":308
 * cdef inline object timezone_new(object offset, object name=None):
 *     if PY_VERSION_HEX < 0x030700b1:
 *         raise RuntimeError('Time zones are not available from the C-API.')             # <<<<<<<<<<<<<<
 *     return __Pyx_TimeZone_FromOffsetAndName(offset, <PyObject*>name if name is not None else NULL)
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 308, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(5, 308, __pyx_L1_error)

    /* "datetime.pxd":307
 * # Create timedelta object using DateTime CAPI factory function.
 * cdef inline object timezone_new(object offset, object name=None):
 *     if PY_VERSION_HEX < 0x030700b1:             # <<<<<<<<<<<<<<
 *         raise RuntimeError('Time zones are not available from the C-API.')
 *     return __Pyx_TimeZone_FromOffsetAndName(offset, <PyObject*>name if name is not None else NULL)
 */
  }

  /* "datetime.pxd":309
 *     if PY_VERSION_HEX < 0x030700b1:
 *         raise RuntimeError('Time zones are not available from the C-API.')
 *     return __Pyx_TimeZone_FromOffsetAndName(offset, <PyObject*>name if name is not None else NULL)             # <<<<<<<<<<<<<<
 * 
 * # Create datetime object using DB API constructor.
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = (__pyx_v_name != Py_None);
  if (__pyx_t_1) {
    __pyx_t_3 = ((PyObject *)__pyx_v_name);
  } else {
    __pyx_t_3 = NULL;
  }
  __pyx_t_2 = __Pyx_TimeZone_FromOffsetAndName(__pyx_v_offset, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "datetime.pxd":306
 * 
 * # Create timedelta object using DateTime CAPI factory function.
 * cdef inline object timezone_new(object offset, object name=None):             # <<<<<<<<<<<<<<
 *     if PY_VERSION_HEX < 0x030700b1:
 *         raise RuntimeError('Time zones are not available from the C-API.')
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cpython.datetime.timezone_new", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "datetime.pxd":312
 * 
 * # Create datetime object using DB API constructor.
 * cdef inline datetime datetime_from_timestamp(timestamp, tz=None):             # <<<<<<<<<<<<<<
 *     return PyDateTimeAPI.DateTime_FromTimestamp(
 *         <PyObject*>PyDateTimeAPI.DateTimeType, (timestamp, tz) if tz is not None else (timestamp,), NULL)
 */

static CYTHON_INLINE PyDateTime_DateTime *__pyx_f_7cpython_8datetime_datetime_from_timestamp(PyObject *__pyx_v_timestamp, struct __pyx_opt_args_7cpython_8datetime_datetime_from_timestamp *__pyx_optional_args) {
  PyObject *__pyx_v_tz = ((PyObject *)Py_None);
  PyDateTime_DateTime *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("datetime_from_timestamp", 1);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_tz = __pyx_optional_args->tz;
    }
  }

  /* "datetime.pxd":313
 * # Create datetime object using DB API constructor.
 * cdef inline datetime datetime_from_timestamp(timestamp, tz=None):
 *     return PyDateTimeAPI.DateTime_FromTimestamp(             # <<<<<<<<<<<<<<
 *         <PyObject*>PyDateTimeAPI.DateTimeType, (timestamp, tz) if tz is not None else (timestamp,), NULL)
 * 
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);

  /* "datetime.pxd":314
 * cdef inline datetime datetime_from_timestamp(timestamp, tz=None):
 *     return PyDateTimeAPI.DateTime_FromTimestamp(
 *         <PyObject*>PyDateTimeAPI.DateTimeType, (timestamp, tz) if tz is not None else (timestamp,), NULL)             # <<<<<<<<<<<<<<
 * 
 * # Create date object using DB API constructor.
 */
  __pyx_t_2 = (__pyx_v_tz != Py_None);
  if (__pyx_t_2) {
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 314, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_timestamp);
    __Pyx_GIVEREF(__pyx_v_timestamp);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_timestamp)) __PYX_ERR(5, 314, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_tz);
    __Pyx_GIVEREF(__pyx_v_tz);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_tz)) __PYX_ERR(5, 314, __pyx_L1_error);
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
  } else {
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 314, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_timestamp);
    __Pyx_GIVEREF(__pyx_v_timestamp);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_timestamp)) __PYX_ERR(5, 314, __pyx_L1_error);
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
  }

  /* "datetime.pxd":313
 * # Create datetime object using DB API constructor.
 * cdef inline datetime datetime_from_timestamp(timestamp, tz=None):
 *     return PyDateTimeAPI.DateTime_FromTimestamp(             # <<<<<<<<<<<<<<
 *         <PyObject*>PyDateTimeAPI.DateTimeType, (timestamp, tz) if tz is not None else (timestamp,), NULL)
 * 
 */
  __pyx_t_3 = ((PyObject *)PyDateTimeAPI->DateTime_FromTimestamp(((PyObject *)PyDateTimeAPI->DateTimeType), __pyx_t_1, NULL)); if (unlikely(!__pyx_t_3)) __PYX_ERR(5, 313, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = ((PyDateTime_DateTime *)__pyx_t_3);
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "datetime.pxd":312
 * 
 * # Create datetime object using DB API constructor.
 * cdef inline datetime datetime_from_timestamp(timestamp, tz=None):             # <<<<<<<<<<<<<<
 *     return PyDateTimeAPI.DateTime_FromTimestamp(
 *         <PyObject*>PyDateTimeAPI.DateTimeType, (timestamp, tz) if tz is not None else (timestamp,), NULL)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cpython.datetime.datetime_from_timestamp", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "datetime.pxd":317
 * 
 * # Create date object using DB API constructor.
 * cdef inline date date_from_timestamp(timestamp):             # <<<<<<<<<<<<<<
 *     return PyDateTimeAPI.Date_FromTimestamp(<PyObject*>PyDateTimeAPI.DateType, (timestamp,))
 * 
 */

static CYTHON_INLINE PyDateTime_Date *__pyx_f_7cpython_8datetime_date_from_timestamp(PyObject *__pyx_v_timestamp) {
  PyDateTime_Date *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("date_from_timestamp", 1);

  /* "datetime.pxd":318
 * # Create date object using DB API constructor.
 * cdef inline date date_from_timestamp(timestamp):
 *     return PyDateTimeAPI.Date_FromTimestamp(<PyObject*>PyDateTimeAPI.DateType, (timestamp,))             # <<<<<<<<<<<<<<
 * 
 * # More recognizable getters for date/time/datetime/timedelta.
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_timestamp);
  __Pyx_GIVEREF(__pyx_v_timestamp);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_timestamp)) __PYX_ERR(5, 318, __pyx_L1_error);
  __pyx_t_2 = ((PyObject *)PyDateTimeAPI->Date_FromTimestamp(((PyObject *)PyDateTimeAPI->DateType), __pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = ((PyDateTime_Date *)__pyx_t_2);
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "datetime.pxd":317
 * 
 * # Create date object using DB API constructor.
 * cdef inline date date_from_timestamp(timestamp):             # <<<<<<<<<<<<<<
 *     return PyDateTimeAPI.Date_FromTimestamp(<PyObject*>PyDateTimeAPI.DateType, (timestamp,))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cpython.datetime.date_from_timestamp", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "datetime.pxd":326
 * 
 * # Get UTC singleton
 * cdef inline object get_utc():             # <<<<<<<<<<<<<<
 *     if PY_VERSION_HEX < 0x030700b1:
 *         raise RuntimeError('Time zones are not available from the C-API.')
 */

static CYTHON_INLINE PyObject *__pyx_f_7cpython_8datetime_get_utc(void) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_utc", 1);

  /* "datetime.pxd":327
 * # Get UTC singleton
 * cdef inline object get_utc():
 *     if PY_VERSION_HEX < 0x030700b1:             # <<<<<<<<<<<<<<
 *         raise RuntimeError('Time zones are not available from the C-API.')
 *     return <object>__Pyx_TimeZone_UTC
 */
  __pyx_t_1 = (PY_VERSION_HEX < 0x030700b1);
  if (unlikely(__pyx_t_1)) {

    /* "datetime.pxd":328
 * cdef inline object get_utc():
 *     if PY_VERSION_HEX < 0x030700b1:
 *         raise RuntimeError('Time zones are not available from the C-API.')             # <<<<<<<<<<<<<<
 *     return <object>__Pyx_TimeZone_UTC
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(5, 328, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(5, 328, __pyx_L1_error)

    /* "datetime.pxd":327
 * # Get UTC singleton
 * cdef inline object get_utc():
 *     if PY_VERSION_HEX < 0x030700b1:             # <<<<<<<<<<<<<<
 *         raise RuntimeError('Time zones are not available from the C-API.')
 *     return <object>__Pyx_TimeZone_UTC
 */
  }

  /* "datetime.pxd":329
 *     if PY_VERSION_HEX < 0x030700b1:
 *         raise RuntimeError('Time zones are not available from the C-API.')
 *     return <object>__Pyx_TimeZone_UTC             # <<<<<<<<<<<<<<
 * 
 * # Get tzinfo of time
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__Pyx_TimeZone_UTC));
  __pyx_r = ((PyObject *)__Pyx_TimeZone_UTC);
  goto __pyx_L0;

  /* "datetime.pxd":326
 * 
 * # Get UTC singleton
 * cdef inline object get_utc():             # <<<<<<<<<<<<<<
 *     if PY_VERSION_HEX < 0x030700b1:
 *         raise RuntimeError('Time zones are not available from the C-API.')
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cpython.datetime.get_utc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "datetime.pxd":332
 * 
 * # Get tzinfo of time
 * cdef inline object time_tzinfo(object o):             # <<<<<<<<<<<<<<
 *     return <object>PyDateTime_TIME_GET_TZINFO(o)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_7cpython_8datetime_time_tzinfo(PyObject *__pyx_v_o) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1;
  __Pyx_RefNannySetupContext("time_tzinfo", 1);

  /* "datetime.pxd":333
 * # Get tzinfo of time
 * cdef inline object time_tzinfo(object o):
 *     return <object>PyDateTime_TIME_GET_TZINFO(o)             # <<<<<<<<<<<<<<
 * 
 * # Get tzinfo of datetime
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyDateTime_TIME_GET_TZINFO(__pyx_v_o);
  __Pyx_INCREF(((PyObject *)__pyx_t_1));
  __pyx_r = ((PyObject *)__pyx_t_1);
  goto __pyx_L0;

  /* "datetime.pxd":332
 * 
 * # Get tzinfo of time
 * cdef inline object time_tzinfo(object o):             # <<<<<<<<<<<<<<
 *     return <object>PyDateTime_TIME_GET_TZINFO(o)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "datetime.pxd":336
 * 
 * # Get tzinfo of datetime
 * cdef inline object datetime_tzinfo(object o):             # <<<<<<<<<<<<<<
 *     return <object>PyDateTime_DATE_GET_TZINFO(o)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_7cpython_8datetime_datetime_tzinfo(PyObject *__pyx_v_o) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1;
  __Pyx_RefNannySetupContext("datetime_tzinfo", 1);

  /* "datetime.pxd":337
 * # Get tzinfo of datetime
 * cdef inline object datetime_tzinfo(object o):
 *     return <object>PyDateTime_DATE_GET_TZINFO(o)             # <<<<<<<<<<<<<<
 * 
 * # Get year of date
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyDateTime_DATE_GET_TZINFO(__pyx_v_o);
  __Pyx_INCREF(((PyObject *)__pyx_t_1));
  __pyx_r = ((PyObject *)__pyx_t_1);
  goto __pyx_L0;

  /* "datetime.pxd":336
 * 
 * # Get tzinfo of datetime
 * cdef inline object datetime_tzinfo(object o):             # <<<<<<<<<<<<<<
 *     return <object>PyDateTime_DATE_GET_TZINFO(o)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "datetime.pxd":340
 * 
 * # Get year of date
 * cdef inline int date_year(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_GET_YEAR(o)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_date_year(PyObject *__pyx_v_o) {
  int __pyx_r;

  /* "datetime.pxd":341
 * # Get year of date
 * cdef inline int date_year(object o) noexcept:
 *     return PyDateTime_GET_YEAR(o)             # <<<<<<<<<<<<<<
 * 
 * # Get month of date
 */
  __pyx_r = PyDateTime_GET_YEAR(__pyx_v_o);
  goto __pyx_L0;

  /* "datetime.pxd":340
 * 
 * # Get year of date
 * cdef inline int date_year(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_GET_YEAR(o)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":344
 * 
 * # Get month of date
 * cdef inline int date_month(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_GET_MONTH(o)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_date_month(PyObject *__pyx_v_o) {
  int __pyx_r;

  /* "datetime.pxd":345
 * # Get month of date
 * cdef inline int date_month(object o) noexcept:
 *     return PyDateTime_GET_MONTH(o)             # <<<<<<<<<<<<<<
 * 
 * # Get day of date
 */
  __pyx_r = PyDateTime_GET_MONTH(__pyx_v_o);
  goto __pyx_L0;

  /* "datetime.pxd":344
 * 
 * # Get month of date
 * cdef inline int date_month(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_GET_MONTH(o)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":348
 * 
 * # Get day of date
 * cdef inline int date_day(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_GET_DAY(o)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_date_day(PyObject *__pyx_v_o) {
  int __pyx_r;

  /* "datetime.pxd":349
 * # Get day of date
 * cdef inline int date_day(object o) noexcept:
 *     return PyDateTime_GET_DAY(o)             # <<<<<<<<<<<<<<
 * 
 * # Get year of datetime
 */
  __pyx_r = PyDateTime_GET_DAY(__pyx_v_o);
  goto __pyx_L0;

  /* "datetime.pxd":348
 * 
 * # Get day of date
 * cdef inline int date_day(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_GET_DAY(o)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":352
 * 
 * # Get year of datetime
 * cdef inline int datetime_year(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_GET_YEAR(o)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_datetime_year(PyObject *__pyx_v_o) {
  int __pyx_r;

  /* "datetime.pxd":353
 * # Get year of datetime
 * cdef inline int datetime_year(object o) noexcept:
 *     return PyDateTime_GET_YEAR(o)             # <<<<<<<<<<<<<<
 * 
 * # Get month of datetime
 */
  __pyx_r = PyDateTime_GET_YEAR(__pyx_v_o);
  goto __pyx_L0;

  /* "datetime.pxd":352
 * 
 * # Get year of datetime
 * cdef inline int datetime_year(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_GET_YEAR(o)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":356
 * 
 * # Get month of datetime
 * cdef inline int datetime_month(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_GET_MONTH(o)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_datetime_month(PyObject *__pyx_v_o) {
  int __pyx_r;

  /* "datetime.pxd":357
 * # Get month of datetime
 * cdef inline int datetime_month(object o) noexcept:
 *     return PyDateTime_GET_MONTH(o)             # <<<<<<<<<<<<<<
 * 
 * # Get day of datetime
 */
  __pyx_r = PyDateTime_GET_MONTH(__pyx_v_o);
  goto __pyx_L0;

  /* "datetime.pxd":356
 * 
 * # Get month of datetime
 * cdef inline int datetime_month(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_GET_MONTH(o)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":360
 * 
 * # Get day of datetime
 * cdef inline int datetime_day(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_GET_DAY(o)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_datetime_day(PyObject *__pyx_v_o) {
  int __pyx_r;

  /* "datetime.pxd":361
 * # Get day of datetime
 * cdef inline int datetime_day(object o) noexcept:
 *     return PyDateTime_GET_DAY(o)             # <<<<<<<<<<<<<<
 * 
 * # Get hour of time
 */
  __pyx_r = PyDateTime_GET_DAY(__pyx_v_o);
  goto __pyx_L0;

  /* "datetime.pxd":360
 * 
 * # Get day of datetime
 * cdef inline int datetime_day(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_GET_DAY(o)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":364
 * 
 * # Get hour of time
 * cdef inline int time_hour(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_TIME_GET_HOUR(o)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_time_hour(PyObject *__pyx_v_o) {
  int __pyx_r;

  /* "datetime.pxd":365
 * # Get hour of time
 * cdef inline int time_hour(object o) noexcept:
 *     return PyDateTime_TIME_GET_HOUR(o)             # <<<<<<<<<<<<<<
 * 
 * # Get minute of time
 */
  __pyx_r = PyDateTime_TIME_GET_HOUR(__pyx_v_o);
  goto __pyx_L0;

  /* "datetime.pxd":364
 * 
 * # Get hour of time
 * cdef inline int time_hour(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_TIME_GET_HOUR(o)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":368
 * 
 * # Get minute of time
 * cdef inline int time_minute(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_TIME_GET_MINUTE(o)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_time_minute(PyObject *__pyx_v_o) {
  int __pyx_r;

  /* "datetime.pxd":369
 * # Get minute of time
 * cdef inline int time_minute(object o) noexcept:
 *     return PyDateTime_TIME_GET_MINUTE(o)             # <<<<<<<<<<<<<<
 * 
 * # Get second of time
 */
  __pyx_r = PyDateTime_TIME_GET_MINUTE(__pyx_v_o);
  goto __pyx_L0;

  /* "datetime.pxd":368
 * 
 * # Get minute of time
 * cdef inline int time_minute(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_TIME_GET_MINUTE(o)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":372
 * 
 * # Get second of time
 * cdef inline int time_second(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_TIME_GET_SECOND(o)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_time_second(PyObject *__pyx_v_o) {
  int __pyx_r;

  /* "datetime.pxd":373
 * # Get second of time
 * cdef inline int time_second(object o) noexcept:
 *     return PyDateTime_TIME_GET_SECOND(o)             # <<<<<<<<<<<<<<
 * 
 * # Get microsecond of time
 */
  __pyx_r = PyDateTime_TIME_GET_SECOND(__pyx_v_o);
  goto __pyx_L0;

  /* "datetime.pxd":372
 * 
 * # Get second of time
 * cdef inline int time_second(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_TIME_GET_SECOND(o)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":376
 * 
 * # Get microsecond of time
 * cdef inline int time_microsecond(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_TIME_GET_MICROSECOND(o)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_time_microsecond(PyObject *__pyx_v_o) {
  int __pyx_r;

  /* "datetime.pxd":377
 * # Get microsecond of time
 * cdef inline int time_microsecond(object o) noexcept:
 *     return PyDateTime_TIME_GET_MICROSECOND(o)             # <<<<<<<<<<<<<<
 * 
 * # Get fold of time
 */
  __pyx_r = PyDateTime_TIME_GET_MICROSECOND(__pyx_v_o);
  goto __pyx_L0;

  /* "datetime.pxd":376
 * 
 * # Get microsecond of time
 * cdef inline int time_microsecond(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_TIME_GET_MICROSECOND(o)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":380
 * 
 * # Get fold of time
 * cdef inline int time_fold(object o) noexcept:             # <<<<<<<<<<<<<<
 *     # For Python < 3.6 this returns 0 no matter what
 *     return PyDateTime_TIME_GET_FOLD(o)
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_time_fold(PyObject *__pyx_v_o) {
  int __pyx_r;

  /* "datetime.pxd":382
 * cdef inline int time_fold(object o) noexcept:
 *     # For Python < 3.6 this returns 0 no matter what
 *     return PyDateTime_TIME_GET_FOLD(o)             # <<<<<<<<<<<<<<
 * 
 * # Get hour of datetime
 */
  __pyx_r = PyDateTime_TIME_GET_FOLD(__pyx_v_o);
  goto __pyx_L0;

  /* "datetime.pxd":380
 * 
 * # Get fold of time
 * cdef inline int time_fold(object o) noexcept:             # <<<<<<<<<<<<<<
 *     # For Python < 3.6 this returns 0 no matter what
 *     return PyDateTime_TIME_GET_FOLD(o)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":385
 * 
 * # Get hour of datetime
 * cdef inline int datetime_hour(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_DATE_GET_HOUR(o)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_datetime_hour(PyObject *__pyx_v_o) {
  int __pyx_r;

  /* "datetime.pxd":386
 * # Get hour of datetime
 * cdef inline int datetime_hour(object o) noexcept:
 *     return PyDateTime_DATE_GET_HOUR(o)             # <<<<<<<<<<<<<<
 * 
 * # Get minute of datetime
 */
  __pyx_r = PyDateTime_DATE_GET_HOUR(__pyx_v_o);
  goto __pyx_L0;

  /* "datetime.pxd":385
 * 
 * # Get hour of datetime
 * cdef inline int datetime_hour(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_DATE_GET_HOUR(o)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":389
 * 
 * # Get minute of datetime
 * cdef inline int datetime_minute(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_DATE_GET_MINUTE(o)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_datetime_minute(PyObject *__pyx_v_o) {
  int __pyx_r;

  /* "datetime.pxd":390
 * # Get minute of datetime
 * cdef inline int datetime_minute(object o) noexcept:
 *     return PyDateTime_DATE_GET_MINUTE(o)             # <<<<<<<<<<<<<<
 * 
 * # Get second of datetime
 */
  __pyx_r = PyDateTime_DATE_GET_MINUTE(__pyx_v_o);
  goto __pyx_L0;

  /* "datetime.pxd":389
 * 
 * # Get minute of datetime
 * cdef inline int datetime_minute(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_DATE_GET_MINUTE(o)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":393
 * 
 * # Get second of datetime
 * cdef inline int datetime_second(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_DATE_GET_SECOND(o)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_datetime_second(PyObject *__pyx_v_o) {
  int __pyx_r;

  /* "datetime.pxd":394
 * # Get second of datetime
 * cdef inline int datetime_second(object o) noexcept:
 *     return PyDateTime_DATE_GET_SECOND(o)             # <<<<<<<<<<<<<<
 * 
 * # Get microsecond of datetime
 */
  __pyx_r = PyDateTime_DATE_GET_SECOND(__pyx_v_o);
  goto __pyx_L0;

  /* "datetime.pxd":393
 * 
 * # Get second of datetime
 * cdef inline int datetime_second(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_DATE_GET_SECOND(o)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":397
 * 
 * # Get microsecond of datetime
 * cdef inline int datetime_microsecond(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_DATE_GET_MICROSECOND(o)
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_datetime_microsecond(PyObject *__pyx_v_o) {
  int __pyx_r;

  /* "datetime.pxd":398
 * # Get microsecond of datetime
 * cdef inline int datetime_microsecond(object o) noexcept:
 *     return PyDateTime_DATE_GET_MICROSECOND(o)             # <<<<<<<<<<<<<<
 * 
 * # Get fold of datetime
 */
  __pyx_r = PyDateTime_DATE_GET_MICROSECOND(__pyx_v_o);
  goto __pyx_L0;

  /* "datetime.pxd":397
 * 
 * # Get microsecond of datetime
 * cdef inline int datetime_microsecond(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return PyDateTime_DATE_GET_MICROSECOND(o)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":401
 * 
 * # Get fold of datetime
 * cdef inline int datetime_fold(object o) noexcept:             # <<<<<<<<<<<<<<
 *     # For Python < 3.6 this returns 0 no matter what
 *     return PyDateTime_DATE_GET_FOLD(o)
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_datetime_fold(PyObject *__pyx_v_o) {
  int __pyx_r;

  /* "datetime.pxd":403
 * cdef inline int datetime_fold(object o) noexcept:
 *     # For Python < 3.6 this returns 0 no matter what
 *     return PyDateTime_DATE_GET_FOLD(o)             # <<<<<<<<<<<<<<
 * 
 * # Get days of timedelta
 */
  __pyx_r = PyDateTime_DATE_GET_FOLD(__pyx_v_o);
  goto __pyx_L0;

  /* "datetime.pxd":401
 * 
 * # Get fold of datetime
 * cdef inline int datetime_fold(object o) noexcept:             # <<<<<<<<<<<<<<
 *     # For Python < 3.6 this returns 0 no matter what
 *     return PyDateTime_DATE_GET_FOLD(o)
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":406
 * 
 * # Get days of timedelta
 * cdef inline int timedelta_days(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return (<PyDateTime_Delta*>o).days
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_timedelta_days(PyObject *__pyx_v_o) {
  int __pyx_r;

  /* "datetime.pxd":407
 * # Get days of timedelta
 * cdef inline int timedelta_days(object o) noexcept:
 *     return (<PyDateTime_Delta*>o).days             # <<<<<<<<<<<<<<
 * 
 * # Get seconds of timedelta
 */
  __pyx_r = ((PyDateTime_Delta *)__pyx_v_o)->days;
  goto __pyx_L0;

  /* "datetime.pxd":406
 * 
 * # Get days of timedelta
 * cdef inline int timedelta_days(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return (<PyDateTime_Delta*>o).days
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":410
 * 
 * # Get seconds of timedelta
 * cdef inline int timedelta_seconds(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return (<PyDateTime_Delta*>o).seconds
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_timedelta_seconds(PyObject *__pyx_v_o) {
  int __pyx_r;

  /* "datetime.pxd":411
 * # Get seconds of timedelta
 * cdef inline int timedelta_seconds(object o) noexcept:
 *     return (<PyDateTime_Delta*>o).seconds             # <<<<<<<<<<<<<<
 * 
 * # Get microseconds of timedelta
 */
  __pyx_r = ((PyDateTime_Delta *)__pyx_v_o)->seconds;
  goto __pyx_L0;

  /* "datetime.pxd":410
 * 
 * # Get seconds of timedelta
 * cdef inline int timedelta_seconds(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return (<PyDateTime_Delta*>o).seconds
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":414
 * 
 * # Get microseconds of timedelta
 * cdef inline int timedelta_microseconds(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return (<PyDateTime_Delta*>o).microseconds
 * 
 */

static CYTHON_INLINE int __pyx_f_7cpython_8datetime_timedelta_microseconds(PyObject *__pyx_v_o) {
  int __pyx_r;

  /* "datetime.pxd":415
 * # Get microseconds of timedelta
 * cdef inline int timedelta_microseconds(object o) noexcept:
 *     return (<PyDateTime_Delta*>o).microseconds             # <<<<<<<<<<<<<<
 * 
 * cdef inline double total_seconds(timedelta obj) noexcept:
 */
  __pyx_r = ((PyDateTime_Delta *)__pyx_v_o)->microseconds;
  goto __pyx_L0;

  /* "datetime.pxd":414
 * 
 * # Get microseconds of timedelta
 * cdef inline int timedelta_microseconds(object o) noexcept:             # <<<<<<<<<<<<<<
 *     return (<PyDateTime_Delta*>o).microseconds
 * 
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "datetime.pxd":417
 *     return (<PyDateTime_Delta*>o).microseconds
 * 
 * cdef inline double total_seconds(timedelta obj) noexcept:             # <<<<<<<<<<<<<<
 *     # Mirrors the "timedelta.total_seconds()" method.
 *     # Note that this implementation is not guaranteed to give *exactly* the same
 */

static CYTHON_INLINE double __pyx_f_7cpython_8datetime_total_seconds(PyDateTime_Delta *__pyx_v_obj) {
  double __pyx_v_days;
  double __pyx_v_seconds;
  double __pyx_v_micros;
  double __pyx_r;

  /* "datetime.pxd":423
 *     cdef:
 *         double days, seconds, micros
 *     days = <double>PyDateTime_DELTA_GET_DAYS(obj)             # <<<<<<<<<<<<<<
 *     seconds = <double>PyDateTime_DELTA_GET_SECONDS(obj)
 *     micros = <double>PyDateTime_DELTA_GET_MICROSECONDS(obj)
 */
  __pyx_v_days = ((double)PyDateTime_DELTA_GET_DAYS(((PyObject *)__pyx_v_obj)));

  /* "datetime.pxd":424
 *         double days, seconds, micros
 *     days = <double>PyDateTime_DELTA_GET_DAYS(obj)
 *     seconds = <double>PyDateTime_DELTA_GET_SECONDS(obj)             # <<<<<<<<<<<<<<
 *     micros = <double>PyDateTime_DELTA_GET_MICROSECONDS(obj)
 *     return days * 24 * 3600 + seconds + micros / 1_000_000
 */
  __pyx_v_seconds = ((double)PyDateTime_DELTA_GET_SECONDS(((PyObject *)__pyx_v_obj)));

  /* "datetime.pxd":425
 *     days = <double>PyDateTime_DELTA_GET_DAYS(obj)
 *     seconds = <double>PyDateTime_DELTA_GET_SECONDS(obj)
 *     micros = <double>PyDateTime_DELTA_GET_MICROSECONDS(obj)             # <<<<<<<<<<<<<<
 *     return days * 24 * 3600 + seconds + micros / 1_000_000
 */
  __pyx_v_micros = ((double)PyDateTime_DELTA_GET_MICROSECONDS(((PyObject *)__pyx_v_obj)));

  /* "datetime.pxd":426
 *     seconds = <double>PyDateTime_DELTA_GET_SECONDS(obj)
 *     micros = <double>PyDateTime_DELTA_GET_MICROSECONDS(obj)
 *     return days * 24 * 3600 + seconds + micros / 1_000_000             # <<<<<<<<<<<<<<
 */
  __pyx_r = ((((__pyx_v_days * 24.0) * 3600.0) + __pyx_v_seconds) + (__pyx_v_micros / 1000000.0));
  goto __pyx_L0;

  /* "datetime.pxd":417
 *     return (<PyDateTime_Delta*>o).microseconds
 * 
 * cdef inline double total_seconds(timedelta obj) noexcept:             # <<<<<<<<<<<<<<
 *     # Mirrors the "timedelta.total_seconds()" method.
 *     # Note that this implementation is not guaranteed to give *exactly* the same
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "utils.pxd":32
 * 
 * # Utils: encode / decode
 * cdef inline bytes encode_str(object obj, char* encoding):             # <<<<<<<<<<<<<<
 *     """Encode string to bytes using the 'encoding' with
 *     'surrogateescape' error handling `<'bytes'>`."""
 */

static CYTHON_INLINE PyObject *__pyx_f_10serializor_5utils_encode_str(PyObject *__pyx_v_obj, char *__pyx_v_encoding) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("encode_str", 1);

  /* "utils.pxd":35
 *     """Encode string to bytes using the 'encoding' with
 *     'surrogateescape' error handling `<'bytes'>`."""
 *     return PyUnicode_AsEncodedString(obj, encoding, b"surrogateescape")             # <<<<<<<<<<<<<<
 * 
 * cdef inline str decode_bytes(object data, char* encoding):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyUnicode_AsEncodedString(__pyx_v_obj, __pyx_v_encoding, ((char *)"surrogateescape")); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(6, 35, __pyx_L1_error)
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "utils.pxd":32
 * 
 * # Utils: encode / decode
 * cdef inline bytes encode_str(object obj, char* encoding):             # <<<<<<<<<<<<<<
 *     """Encode string to bytes using the 'encoding' with
 *     'surrogateescape' error handling `<'bytes'>`."""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("serializor.utils.encode_str", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "utils.pxd":37
 *     return PyUnicode_AsEncodedString(obj, encoding, b"surrogateescape")
 * 
 * cdef inline str decode_bytes(object data, char* encoding):             # <<<<<<<<<<<<<<
 *     """Decode bytes to string using the 'encoding' with
 *     "surrogateescape" error handling `<'str'>`."""
 */

static CYTHON_INLINE PyObject *__pyx_f_10serializor_5utils_decode_bytes(PyObject *__pyx_v_data, char *__pyx_v_encoding) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  char *__pyx_t_1;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("decode_bytes", 1);

  /* "utils.pxd":40
 *     """Decode bytes to string using the 'encoding' with
 *     "surrogateescape" error handling `<'str'>`."""
 *     return PyUnicode_Decode(bytes_to_chars(data), bytes_len(data), encoding, b"surrogateescape")             # <<<<<<<<<<<<<<
 * 
 * cdef inline str decode_bytes_utf8(object data):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyBytes_AsString(__pyx_v_data); if (unlikely(__pyx_t_1 == ((char *)NULL))) __PYX_ERR(6, 40, __pyx_L1_error)
  __pyx_t_2 = PyBytes_Size(__pyx_v_data); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1L))) __PYX_ERR(6, 40, __pyx_L1_error)
  __pyx_t_3 = PyUnicode_Decode(__pyx_t_1, __pyx_t_2, __pyx_v_encoding, ((char *)"surrogateescape")); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None) || __Pyx_RaiseUnexpectedTypeError("unicode", __pyx_t_3))) __PYX_ERR(6, 40, __pyx_L1_error)
  __pyx_r = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "utils.pxd":37
 *     return PyUnicode_AsEncodedString(obj, encoding, b"surrogateescape")
 * 
 * cdef inline str decode_bytes(object data, char* encoding):             # <<<<<<<<<<<<<<
 *     """Decode bytes to string using the 'encoding' with
 *     "surrogateescape" error handling `<'str'>`."""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("serializor.utils.decode_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "utils.pxd":42
 *     return PyUnicode_Decode(bytes_to_chars(data), bytes_len(data), encoding, b"surrogateescape")
 * 
 * cdef inline str decode_bytes_utf8(object data):             # <<<<<<<<<<<<<<
 *     """Decode bytes to string using 'utf-8' encoding with
 *     'surrogateescape' error handling `<'str'>`."""
 */

static CYTHON_INLINE PyObject *__pyx_f_10serializor_5utils_decode_bytes_utf8(PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  char *__pyx_t_1;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("decode_bytes_utf8", 1);

  /* "utils.pxd":45
 *     """Decode bytes to string using 'utf-8' encoding with
 *     'surrogateescape' error handling `<'str'>`."""
 *     return PyUnicode_DecodeUTF8(bytes_to_chars(data), bytes_len(data), b"surrogateescape")             # <<<<<<<<<<<<<<
 * 
 * cdef inline str decode_bytes_ascii(object data):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyBytes_AsString(__pyx_v_data); if (unlikely(__pyx_t_1 == ((char *)NULL))) __PYX_ERR(6, 45, __pyx_L1_error)
  __pyx_t_2 = PyBytes_Size(__pyx_v_data); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1L))) __PYX_ERR(6, 45, __pyx_L1_error)
  __pyx_t_3 = PyUnicode_DecodeUTF8(__pyx_t_1, __pyx_t_2, ((char *)"surrogateescape")); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "utils.pxd":42
 *     return PyUnicode_Decode(bytes_to_chars(data), bytes_len(data), encoding, b"surrogateescape")
 * 
 * cdef inline str decode_bytes_utf8(object data):             # <<<<<<<<<<<<<<
 *     """Decode bytes to string using 'utf-8' encoding with
 *     'surrogateescape' error handling `<'str'>`."""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("serializor.utils.decode_bytes_utf8", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "utils.pxd":47
 *     return PyUnicode_DecodeUTF8(bytes_to_chars(data), bytes_len(data), b"surrogateescape")
 * 
 * cdef inline str decode_bytes_ascii(object data):             # <<<<<<<<<<<<<<
 *     """Decode bytes to string using 'ascii' encoding with
 *     'surrogateescape' error handling `<'str'>`."""
 */

static CYTHON_INLINE PyObject *__pyx_f_10serializor_5utils_decode_bytes_ascii(PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  char *__pyx_t_1;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("decode_bytes_ascii", 1);

  /* "utils.pxd":50
 *     """Decode bytes to string using 'ascii' encoding with
 *     'surrogateescape' error handling `<'str'>`."""
 *     return PyUnicode_DecodeASCII(bytes_to_chars(data), bytes_len(data), b"surrogateescape")             # <<<<<<<<<<<<<<
 * 
 * # NumPy: ndarray get item
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyBytes_AsString(__pyx_v_data); if (unlikely(__pyx_t_1 == ((char *)NULL))) __PYX_ERR(6, 50, __pyx_L1_error)
  __pyx_t_2 = PyBytes_Size(__pyx_v_data); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1L))) __PYX_ERR(6, 50, __pyx_L1_error)
  __pyx_t_3 = PyUnicode_DecodeASCII(__pyx_t_1, __pyx_t_2, ((char *)"surrogateescape")); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "utils.pxd":47
 *     return PyUnicode_DecodeUTF8(bytes_to_chars(data), bytes_len(data), b"surrogateescape")
 * 
 * cdef inline str decode_bytes_ascii(object data):             # <<<<<<<<<<<<<<
 *     """Decode bytes to string using 'ascii' encoding with
 *     'surrogateescape' error handling `<'str'>`."""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("serializor.utils.decode_bytes_ascii", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "utils.pxd":53
 * 
 * # NumPy: ndarray get item
 * cdef inline object arr_getitem_1d(np.ndarray arr, np.npy_intp i):             # <<<<<<<<<<<<<<
 *     """Get item from 1-dimensional numpy ndarray as `<'object'>`."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR1(arr, i)
 */

static CYTHON_INLINE PyObject *__pyx_f_10serializor_5utils_arr_getitem_1d(PyArrayObject *__pyx_v_arr, npy_intp __pyx_v_i) {
  void *__pyx_v_itemptr;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("arr_getitem_1d", 1);

  /* "utils.pxd":55
 * cdef inline object arr_getitem_1d(np.ndarray arr, np.npy_intp i):
 *     """Get item from 1-dimensional numpy ndarray as `<'object'>`."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR1(arr, i)             # <<<<<<<<<<<<<<
 *     return PyArray_GETITEM(arr, itemptr)
 * 
 */
  __pyx_v_itemptr = ((void *)PyArray_GETPTR1(__pyx_v_arr, __pyx_v_i));

  /* "utils.pxd":56
 *     """Get item from 1-dimensional numpy ndarray as `<'object'>`."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR1(arr, i)
 *     return PyArray_GETITEM(arr, itemptr)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object arr_getitem_2d(np.ndarray arr, np.npy_intp i, np.npy_intp j):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_GETITEM(__pyx_v_arr, __pyx_v_itemptr); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "utils.pxd":53
 * 
 * # NumPy: ndarray get item
 * cdef inline object arr_getitem_1d(np.ndarray arr, np.npy_intp i):             # <<<<<<<<<<<<<<
 *     """Get item from 1-dimensional numpy ndarray as `<'object'>`."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR1(arr, i)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("serializor.utils.arr_getitem_1d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "utils.pxd":58
 *     return PyArray_GETITEM(arr, itemptr)
 * 
 * cdef inline object arr_getitem_2d(np.ndarray arr, np.npy_intp i, np.npy_intp j):             # <<<<<<<<<<<<<<
 *     """Get item from 2-dimensional numpy ndarray as `<'object'>`."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR2(arr, i, j)
 */

static CYTHON_INLINE PyObject *__pyx_f_10serializor_5utils_arr_getitem_2d(PyArrayObject *__pyx_v_arr, npy_intp __pyx_v_i, npy_intp __pyx_v_j) {
  void *__pyx_v_itemptr;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("arr_getitem_2d", 1);

  /* "utils.pxd":60
 * cdef inline object arr_getitem_2d(np.ndarray arr, np.npy_intp i, np.npy_intp j):
 *     """Get item from 2-dimensional numpy ndarray as `<'object'>`."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR2(arr, i, j)             # <<<<<<<<<<<<<<
 *     return PyArray_GETITEM(arr, itemptr)
 * 
 */
  __pyx_v_itemptr = ((void *)PyArray_GETPTR2(__pyx_v_arr, __pyx_v_i, __pyx_v_j));

  /* "utils.pxd":61
 *     """Get item from 2-dimensional numpy ndarray as `<'object'>`."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR2(arr, i, j)
 *     return PyArray_GETITEM(arr, itemptr)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object arr_getitem_3d(np.ndarray arr, np.npy_intp i, np.npy_intp j, np.npy_intp k):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_GETITEM(__pyx_v_arr, __pyx_v_itemptr); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "utils.pxd":58
 *     return PyArray_GETITEM(arr, itemptr)
 * 
 * cdef inline object arr_getitem_2d(np.ndarray arr, np.npy_intp i, np.npy_intp j):             # <<<<<<<<<<<<<<
 *     """Get item from 2-dimensional numpy ndarray as `<'object'>`."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR2(arr, i, j)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("serializor.utils.arr_getitem_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "utils.pxd":63
 *     return PyArray_GETITEM(arr, itemptr)
 * 
 * cdef inline object arr_getitem_3d(np.ndarray arr, np.npy_intp i, np.npy_intp j, np.npy_intp k):             # <<<<<<<<<<<<<<
 *     """Get item from 3-dimensional numpy ndarray as `<'object'>`."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR3(arr, i, j, k)
 */

static CYTHON_INLINE PyObject *__pyx_f_10serializor_5utils_arr_getitem_3d(PyArrayObject *__pyx_v_arr, npy_intp __pyx_v_i, npy_intp __pyx_v_j, npy_intp __pyx_v_k) {
  void *__pyx_v_itemptr;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("arr_getitem_3d", 1);

  /* "utils.pxd":65
 * cdef inline object arr_getitem_3d(np.ndarray arr, np.npy_intp i, np.npy_intp j, np.npy_intp k):
 *     """Get item from 3-dimensional numpy ndarray as `<'object'>`."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR3(arr, i, j, k)             # <<<<<<<<<<<<<<
 *     return PyArray_GETITEM(arr, itemptr)
 * 
 */
  __pyx_v_itemptr = ((void *)PyArray_GETPTR3(__pyx_v_arr, __pyx_v_i, __pyx_v_j, __pyx_v_k));

  /* "utils.pxd":66
 *     """Get item from 3-dimensional numpy ndarray as `<'object'>`."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR3(arr, i, j, k)
 *     return PyArray_GETITEM(arr, itemptr)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object arr_getitem_4d(np.ndarray arr, np.npy_intp i, np.npy_intp j, np.npy_intp k, np.npy_intp l):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_GETITEM(__pyx_v_arr, __pyx_v_itemptr); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "utils.pxd":63
 *     return PyArray_GETITEM(arr, itemptr)
 * 
 * cdef inline object arr_getitem_3d(np.ndarray arr, np.npy_intp i, np.npy_intp j, np.npy_intp k):             # <<<<<<<<<<<<<<
 *     """Get item from 3-dimensional numpy ndarray as `<'object'>`."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR3(arr, i, j, k)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("serializor.utils.arr_getitem_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "utils.pxd":68
 *     return PyArray_GETITEM(arr, itemptr)
 * 
 * cdef inline object arr_getitem_4d(np.ndarray arr, np.npy_intp i, np.npy_intp j, np.npy_intp k, np.npy_intp l):             # <<<<<<<<<<<<<<
 *     """Get item from 4-dimensional numpy ndarray as `<'object'>`."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR4(arr, i, j, k, l)
 */

static CYTHON_INLINE PyObject *__pyx_f_10serializor_5utils_arr_getitem_4d(PyArrayObject *__pyx_v_arr, npy_intp __pyx_v_i, npy_intp __pyx_v_j, npy_intp __pyx_v_k, npy_intp __pyx_v_l) {
  void *__pyx_v_itemptr;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("arr_getitem_4d", 1);

  /* "utils.pxd":70
 * cdef inline object arr_getitem_4d(np.ndarray arr, np.npy_intp i, np.npy_intp j, np.npy_intp k, np.npy_intp l):
 *     """Get item from 4-dimensional numpy ndarray as `<'object'>`."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR4(arr, i, j, k, l)             # <<<<<<<<<<<<<<
 *     return PyArray_GETITEM(arr, itemptr)
 * 
 */
  __pyx_v_itemptr = ((void *)PyArray_GETPTR4(__pyx_v_arr, __pyx_v_i, __pyx_v_j, __pyx_v_k, __pyx_v_l));

  /* "utils.pxd":71
 *     """Get item from 4-dimensional numpy ndarray as `<'object'>`."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR4(arr, i, j, k, l)
 *     return PyArray_GETITEM(arr, itemptr)             # <<<<<<<<<<<<<<
 * 
 * # NumPy: ndarray set item
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_GETITEM(__pyx_v_arr, __pyx_v_itemptr); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "utils.pxd":68
 *     return PyArray_GETITEM(arr, itemptr)
 * 
 * cdef inline object arr_getitem_4d(np.ndarray arr, np.npy_intp i, np.npy_intp j, np.npy_intp k, np.npy_intp l):             # <<<<<<<<<<<<<<
 *     """Get item from 4-dimensional numpy ndarray as `<'object'>`."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR4(arr, i, j, k, l)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("serializor.utils.arr_getitem_4d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "utils.pxd":74
 * 
 * # NumPy: ndarray set item
 * cdef inline bint arr_setitem_1d(np.ndarray arr, object item, np.npy_intp i) except -1:             # <<<<<<<<<<<<<<
 *     """Set item for 1-dimensional numpy ndarray."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR1(arr, i)
 */

static CYTHON_INLINE int __pyx_f_10serializor_5utils_arr_setitem_1d(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_item, npy_intp __pyx_v_i) {
  void *__pyx_v_itemptr;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "utils.pxd":76
 * cdef inline bint arr_setitem_1d(np.ndarray arr, object item, np.npy_intp i) except -1:
 *     """Set item for 1-dimensional numpy ndarray."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR1(arr, i)             # <<<<<<<<<<<<<<
 *     return PyArray_SETITEM(arr, itemptr, item)
 * 
 */
  __pyx_v_itemptr = ((void *)PyArray_GETPTR1(__pyx_v_arr, __pyx_v_i));

  /* "utils.pxd":77
 *     """Set item for 1-dimensional numpy ndarray."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR1(arr, i)
 *     return PyArray_SETITEM(arr, itemptr, item)             # <<<<<<<<<<<<<<
 * 
 * cdef inline bint arr_setitem_2d(np.ndarray arr, object item, np.npy_intp i, np.npy_intp j) except -1:
 */
  __pyx_t_1 = PyArray_SETITEM(__pyx_v_arr, __pyx_v_itemptr, __pyx_v_item); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(6, 77, __pyx_L1_error)
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "utils.pxd":74
 * 
 * # NumPy: ndarray set item
 * cdef inline bint arr_setitem_1d(np.ndarray arr, object item, np.npy_intp i) except -1:             # <<<<<<<<<<<<<<
 *     """Set item for 1-dimensional numpy ndarray."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR1(arr, i)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("serializor.utils.arr_setitem_1d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "utils.pxd":79
 *     return PyArray_SETITEM(arr, itemptr, item)
 * 
 * cdef inline bint arr_setitem_2d(np.ndarray arr, object item, np.npy_intp i, np.npy_intp j) except -1:             # <<<<<<<<<<<<<<
 *     """Set item for 2-dimensional numpy ndarray."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR2(arr, i, j)
 */

static CYTHON_INLINE int __pyx_f_10serializor_5utils_arr_setitem_2d(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_item, npy_intp __pyx_v_i, npy_intp __pyx_v_j) {
  void *__pyx_v_itemptr;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "utils.pxd":81
 * cdef inline bint arr_setitem_2d(np.ndarray arr, object item, np.npy_intp i, np.npy_intp j) except -1:
 *     """Set item for 2-dimensional numpy ndarray."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR2(arr, i, j)             # <<<<<<<<<<<<<<
 *     return PyArray_SETITEM(arr, itemptr, item)
 * 
 */
  __pyx_v_itemptr = ((void *)PyArray_GETPTR2(__pyx_v_arr, __pyx_v_i, __pyx_v_j));

  /* "utils.pxd":82
 *     """Set item for 2-dimensional numpy ndarray."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR2(arr, i, j)
 *     return PyArray_SETITEM(arr, itemptr, item)             # <<<<<<<<<<<<<<
 * 
 * cdef inline bint arr_setitem_3d(np.ndarray arr, object item, np.npy_intp i, np.npy_intp j, np.npy_intp k) except -1:
 */
  __pyx_t_1 = PyArray_SETITEM(__pyx_v_arr, __pyx_v_itemptr, __pyx_v_item); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(6, 82, __pyx_L1_error)
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "utils.pxd":79
 *     return PyArray_SETITEM(arr, itemptr, item)
 * 
 * cdef inline bint arr_setitem_2d(np.ndarray arr, object item, np.npy_intp i, np.npy_intp j) except -1:             # <<<<<<<<<<<<<<
 *     """Set item for 2-dimensional numpy ndarray."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR2(arr, i, j)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("serializor.utils.arr_setitem_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "utils.pxd":84
 *     return PyArray_SETITEM(arr, itemptr, item)
 * 
 * cdef inline bint arr_setitem_3d(np.ndarray arr, object item, np.npy_intp i, np.npy_intp j, np.npy_intp k) except -1:             # <<<<<<<<<<<<<<
 *     """Set item for 3-dimensional numpy ndarray."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR3(arr, i, j, k)
 */

static CYTHON_INLINE int __pyx_f_10serializor_5utils_arr_setitem_3d(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_item, npy_intp __pyx_v_i, npy_intp __pyx_v_j, npy_intp __pyx_v_k) {
  void *__pyx_v_itemptr;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "utils.pxd":86
 * cdef inline bint arr_setitem_3d(np.ndarray arr, object item, np.npy_intp i, np.npy_intp j, np.npy_intp k) except -1:
 *     """Set item for 3-dimensional numpy ndarray."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR3(arr, i, j, k)             # <<<<<<<<<<<<<<
 *     return PyArray_SETITEM(arr, itemptr, item)
 * 
 */
  __pyx_v_itemptr = ((void *)PyArray_GETPTR3(__pyx_v_arr, __pyx_v_i, __pyx_v_j, __pyx_v_k));

  /* "utils.pxd":87
 *     """Set item for 3-dimensional numpy ndarray."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR3(arr, i, j, k)
 *     return PyArray_SETITEM(arr, itemptr, item)             # <<<<<<<<<<<<<<
 * 
 * cdef inline bint arr_setitem_4d(np.ndarray arr, object item, np.npy_intp i, np.npy_intp j, np.npy_intp k, np.npy_intp l) except -1:
 */
  __pyx_t_1 = PyArray_SETITEM(__pyx_v_arr, __pyx_v_itemptr, __pyx_v_item); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(6, 87, __pyx_L1_error)
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "utils.pxd":84
 *     return PyArray_SETITEM(arr, itemptr, item)
 * 
 * cdef inline bint arr_setitem_3d(np.ndarray arr, object item, np.npy_intp i, np.npy_intp j, np.npy_intp k) except -1:             # <<<<<<<<<<<<<<
 *     """Set item for 3-dimensional numpy ndarray."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR3(arr, i, j, k)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("serializor.utils.arr_setitem_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "utils.pxd":89
 *     return PyArray_SETITEM(arr, itemptr, item)
 * 
 * cdef inline bint arr_setitem_4d(np.ndarray arr, object item, np.npy_intp i, np.npy_intp j, np.npy_intp k, np.npy_intp l) except -1:             # <<<<<<<<<<<<<<
 *     """Set item for 4-dimensional numpy ndarray."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR4(arr, i, j, k, l)
 */

static CYTHON_INLINE int __pyx_f_10serializor_5utils_arr_setitem_4d(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_item, npy_intp __pyx_v_i, npy_intp __pyx_v_j, npy_intp __pyx_v_k, npy_intp __pyx_v_l) {
  void *__pyx_v_itemptr;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "utils.pxd":91
 * cdef inline bint arr_setitem_4d(np.ndarray arr, object item, np.npy_intp i, np.npy_intp j, np.npy_intp k, np.npy_intp l) except -1:
 *     """Set item for 4-dimensional numpy ndarray."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR4(arr, i, j, k, l)             # <<<<<<<<<<<<<<
 *     return PyArray_SETITEM(arr, itemptr, item)
 * 
 */
  __pyx_v_itemptr = ((void *)PyArray_GETPTR4(__pyx_v_arr, __pyx_v_i, __pyx_v_j, __pyx_v_k, __pyx_v_l));

  /* "utils.pxd":92
 *     """Set item for 4-dimensional numpy ndarray."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR4(arr, i, j, k, l)
 *     return PyArray_SETITEM(arr, itemptr, item)             # <<<<<<<<<<<<<<
 * 
 * # NumPy: ndarray flatten
 */
  __pyx_t_1 = PyArray_SETITEM(__pyx_v_arr, __pyx_v_itemptr, __pyx_v_item); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(6, 92, __pyx_L1_error)
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "utils.pxd":89
 *     return PyArray_SETITEM(arr, itemptr, item)
 * 
 * cdef inline bint arr_setitem_4d(np.ndarray arr, object item, np.npy_intp i, np.npy_intp j, np.npy_intp k, np.npy_intp l) except -1:             # <<<<<<<<<<<<<<
 *     """Set item for 4-dimensional numpy ndarray."""
 *     cdef void* itemptr = <void*>PyArray_GETPTR4(arr, i, j, k, l)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("serializor.utils.arr_setitem_4d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "utils.pxd":95
 * 
 * # NumPy: ndarray flatten
 * cdef inline np.ndarray arr_flatten(np.ndarray arr):             # <<<<<<<<<<<<<<
 *     """Flatten multi-dimensional numpy ndarray into
 *     1-dimension with 'C-ORDER' `<'np.ndarray'>`."""
 */

static CYTHON_INLINE PyArrayObject *__pyx_f_10serializor_5utils_arr_flatten(PyArrayObject *__pyx_v_arr) {
  PyArrayObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("arr_flatten", 1);

  /* "utils.pxd":98
 *     """Flatten multi-dimensional numpy ndarray into
 *     1-dimension with 'C-ORDER' `<'np.ndarray'>`."""
 *     return PyArray_Flatten(arr, NPY_ORDER.NPY_CORDER)             # <<<<<<<<<<<<<<
 * 
 * # NumPy: nptime unit
 */
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_1 = PyArray_Flatten(__pyx_v_arr, NPY_CORDER); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(6, 98, __pyx_L1_error)
  __pyx_r = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "utils.pxd":95
 * 
 * # NumPy: ndarray flatten
 * cdef inline np.ndarray arr_flatten(np.ndarray arr):             # <<<<<<<<<<<<<<
 *     """Flatten multi-dimensional numpy ndarray into
 *     1-dimension with 'C-ORDER' `<'np.ndarray'>`."""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("serializor.utils.arr_flatten", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "utils.pxd":101
 * 
 * # NumPy: nptime unit
 * cdef inline str map_nptime_unit_int2str(int unit):             # <<<<<<<<<<<<<<
 *     """Map ndarray[datetime64/timedelta64] time unit from integer
 *     to the corresponding string representation `<'str'>`."""
 */

static CYTHON_INLINE PyObject *__pyx_f_10serializor_5utils_map_nptime_unit_int2str(int __pyx_v_unit) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("map_nptime_unit_int2str", 1);

  /* "utils.pxd":105
 *     to the corresponding string representation `<'str'>`."""
 *     # Common units
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_ns:             # <<<<<<<<<<<<<<
 *         return "ns"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_us:
 */
  __pyx_t_1 = (__pyx_v_unit == NPY_FR_ns);
  if (__pyx_t_1) {

    /* "utils.pxd":106
 *     # Common units
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_ns:
 *         return "ns"             # <<<<<<<<<<<<<<
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_us:
 *         return "us"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ns);
    __pyx_r = __pyx_n_u_ns;
    goto __pyx_L0;

    /* "utils.pxd":105
 *     to the corresponding string representation `<'str'>`."""
 *     # Common units
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_ns:             # <<<<<<<<<<<<<<
 *         return "ns"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_us:
 */
  }

  /* "utils.pxd":107
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_ns:
 *         return "ns"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_us:             # <<<<<<<<<<<<<<
 *         return "us"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_ms:
 */
  __pyx_t_1 = (__pyx_v_unit == NPY_FR_us);
  if (__pyx_t_1) {

    /* "utils.pxd":108
 *         return "ns"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_us:
 *         return "us"             # <<<<<<<<<<<<<<
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_ms:
 *         return "ms"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_us);
    __pyx_r = __pyx_n_u_us;
    goto __pyx_L0;

    /* "utils.pxd":107
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_ns:
 *         return "ns"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_us:             # <<<<<<<<<<<<<<
 *         return "us"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_ms:
 */
  }

  /* "utils.pxd":109
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_us:
 *         return "us"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_ms:             # <<<<<<<<<<<<<<
 *         return "ms"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_s:
 */
  __pyx_t_1 = (__pyx_v_unit == NPY_FR_ms);
  if (__pyx_t_1) {

    /* "utils.pxd":110
 *         return "us"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_ms:
 *         return "ms"             # <<<<<<<<<<<<<<
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_s:
 *         return "s"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ms);
    __pyx_r = __pyx_n_u_ms;
    goto __pyx_L0;

    /* "utils.pxd":109
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_us:
 *         return "us"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_ms:             # <<<<<<<<<<<<<<
 *         return "ms"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_s:
 */
  }

  /* "utils.pxd":111
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_ms:
 *         return "ms"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_s:             # <<<<<<<<<<<<<<
 *         return "s"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_m:
 */
  __pyx_t_1 = (__pyx_v_unit == NPY_FR_s);
  if (__pyx_t_1) {

    /* "utils.pxd":112
 *         return "ms"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_s:
 *         return "s"             # <<<<<<<<<<<<<<
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_m:
 *         return "m"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_s);
    __pyx_r = __pyx_n_u_s;
    goto __pyx_L0;

    /* "utils.pxd":111
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_ms:
 *         return "ms"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_s:             # <<<<<<<<<<<<<<
 *         return "s"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_m:
 */
  }

  /* "utils.pxd":113
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_s:
 *         return "s"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_m:             # <<<<<<<<<<<<<<
 *         return "m"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_h:
 */
  __pyx_t_1 = (__pyx_v_unit == NPY_FR_m);
  if (__pyx_t_1) {

    /* "utils.pxd":114
 *         return "s"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_m:
 *         return "m"             # <<<<<<<<<<<<<<
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_h:
 *         return "h"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_m);
    __pyx_r = __pyx_n_u_m;
    goto __pyx_L0;

    /* "utils.pxd":113
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_s:
 *         return "s"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_m:             # <<<<<<<<<<<<<<
 *         return "m"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_h:
 */
  }

  /* "utils.pxd":115
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_m:
 *         return "m"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_h:             # <<<<<<<<<<<<<<
 *         return "h"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_D:
 */
  __pyx_t_1 = (__pyx_v_unit == NPY_FR_h);
  if (__pyx_t_1) {

    /* "utils.pxd":116
 *         return "m"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_h:
 *         return "h"             # <<<<<<<<<<<<<<
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_D:
 *         return "D"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_h);
    __pyx_r = __pyx_n_u_h;
    goto __pyx_L0;

    /* "utils.pxd":115
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_m:
 *         return "m"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_h:             # <<<<<<<<<<<<<<
 *         return "h"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_D:
 */
  }

  /* "utils.pxd":117
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_h:
 *         return "h"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_D:             # <<<<<<<<<<<<<<
 *         return "D"
 *     # Uncommon units
 */
  __pyx_t_1 = (__pyx_v_unit == NPY_FR_D);
  if (__pyx_t_1) {

    /* "utils.pxd":118
 *         return "h"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_D:
 *         return "D"             # <<<<<<<<<<<<<<
 *     # Uncommon units
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_Y:
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_D);
    __pyx_r = __pyx_n_u_D;
    goto __pyx_L0;

    /* "utils.pxd":117
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_h:
 *         return "h"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_D:             # <<<<<<<<<<<<<<
 *         return "D"
 *     # Uncommon units
 */
  }

  /* "utils.pxd":120
 *         return "D"
 *     # Uncommon units
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_Y:             # <<<<<<<<<<<<<<
 *         return "Y"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_M:
 */
  __pyx_t_1 = (__pyx_v_unit == NPY_FR_Y);
  if (__pyx_t_1) {

    /* "utils.pxd":121
 *     # Uncommon units
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_Y:
 *         return "Y"             # <<<<<<<<<<<<<<
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_M:
 *         return "M"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_Y);
    __pyx_r = __pyx_n_u_Y;
    goto __pyx_L0;

    /* "utils.pxd":120
 *         return "D"
 *     # Uncommon units
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_Y:             # <<<<<<<<<<<<<<
 *         return "Y"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_M:
 */
  }

  /* "utils.pxd":122
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_Y:
 *         return "Y"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_M:             # <<<<<<<<<<<<<<
 *         return "M"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_W:
 */
  __pyx_t_1 = (__pyx_v_unit == NPY_FR_M);
  if (__pyx_t_1) {

    /* "utils.pxd":123
 *         return "Y"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_M:
 *         return "M"             # <<<<<<<<<<<<<<
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_W:
 *         return "W"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_M);
    __pyx_r = __pyx_n_u_M;
    goto __pyx_L0;

    /* "utils.pxd":122
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_Y:
 *         return "Y"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_M:             # <<<<<<<<<<<<<<
 *         return "M"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_W:
 */
  }

  /* "utils.pxd":124
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_M:
 *         return "M"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_W:             # <<<<<<<<<<<<<<
 *         return "W"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_ps:
 */
  __pyx_t_1 = (__pyx_v_unit == NPY_FR_W);
  if (__pyx_t_1) {

    /* "utils.pxd":125
 *         return "M"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_W:
 *         return "W"             # <<<<<<<<<<<<<<
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_ps:
 *         return "ps"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_W);
    __pyx_r = __pyx_n_u_W;
    goto __pyx_L0;

    /* "utils.pxd":124
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_M:
 *         return "M"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_W:             # <<<<<<<<<<<<<<
 *         return "W"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_ps:
 */
  }

  /* "utils.pxd":126
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_W:
 *         return "W"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_ps:             # <<<<<<<<<<<<<<
 *         return "ps"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_fs:
 */
  __pyx_t_1 = (__pyx_v_unit == NPY_FR_ps);
  if (__pyx_t_1) {

    /* "utils.pxd":127
 *         return "W"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_ps:
 *         return "ps"             # <<<<<<<<<<<<<<
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_fs:
 *         return "fs"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_ps);
    __pyx_r = __pyx_n_u_ps;
    goto __pyx_L0;

    /* "utils.pxd":126
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_W:
 *         return "W"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_ps:             # <<<<<<<<<<<<<<
 *         return "ps"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_fs:
 */
  }

  /* "utils.pxd":128
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_ps:
 *         return "ps"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_fs:             # <<<<<<<<<<<<<<
 *         return "fs"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_as:
 */
  __pyx_t_1 = (__pyx_v_unit == NPY_FR_fs);
  if (__pyx_t_1) {

    /* "utils.pxd":129
 *         return "ps"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_fs:
 *         return "fs"             # <<<<<<<<<<<<<<
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_as:
 *         return "as"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_fs);
    __pyx_r = __pyx_n_u_fs;
    goto __pyx_L0;

    /* "utils.pxd":128
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_ps:
 *         return "ps"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_fs:             # <<<<<<<<<<<<<<
 *         return "fs"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_as:
 */
  }

  /* "utils.pxd":130
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_fs:
 *         return "fs"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_as:             # <<<<<<<<<<<<<<
 *         return "as"
 *     # if unit == np.NPY_DATETIMEUNIT.NPY_FR_B:
 */
  __pyx_t_1 = (__pyx_v_unit == NPY_FR_as);
  if (__pyx_t_1) {

    /* "utils.pxd":131
 *         return "fs"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_as:
 *         return "as"             # <<<<<<<<<<<<<<
 *     # if unit == np.NPY_DATETIMEUNIT.NPY_FR_B:
 *     #     return "B"
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_n_u_as);
    __pyx_r = __pyx_n_u_as;
    goto __pyx_L0;

    /* "utils.pxd":130
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_fs:
 *         return "fs"
 *     if unit == np.NPY_DATETIMEUNIT.NPY_FR_as:             # <<<<<<<<<<<<<<
 *         return "as"
 *     # if unit == np.NPY_DATETIMEUNIT.NPY_FR_B:
 */
  }

  /* "utils.pxd":134
 *     # if unit == np.NPY_DATETIMEUNIT.NPY_FR_B:
 *     #     return "B"
 *     raise ValueError("Unsupported numpy time unit: %d." % (unit))             # <<<<<<<<<<<<<<
 * 
 * cdef inline int map_nptime_unit_str2int(str unit):
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_unit); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 134, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyUnicode_Format(__pyx_kp_u_Unsupported_numpy_time_unit_d, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 134, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 134, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_Raise(__pyx_t_2, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(6, 134, __pyx_L1_error)

  /* "utils.pxd":101
 * 
 * # NumPy: nptime unit
 * cdef inline str map_nptime_unit_int2str(int unit):             # <<<<<<<<<<<<<<
 *     """Map ndarray[datetime64/timedelta64] time unit from integer
 *     to the corresponding string representation `<'str'>`."""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("serializor.utils.map_nptime_unit_int2str", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "utils.pxd":136
 *     raise ValueError("Unsupported numpy time unit: %d." % (unit))
 * 
 * cdef inline int map_nptime_unit_str2int(str unit):             # <<<<<<<<<<<<<<
 *     """Map ndarray[datetime64/timedelta64] time unit from string
 *     representation to the corresponding integer `<'int'>`."""
 */

static CYTHON_INLINE int __pyx_f_10serializor_5utils_map_nptime_unit_str2int(PyObject *__pyx_v_unit) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("map_nptime_unit_str2int", 1);

  /* "utils.pxd":140
 *     representation to the corresponding integer `<'int'>`."""
 *     # Common units
 *     if unit == "ns":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_ns
 *     if unit == "us":
 */
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_unit, __pyx_n_u_ns, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(6, 140, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "utils.pxd":141
 *     # Common units
 *     if unit == "ns":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_ns             # <<<<<<<<<<<<<<
 *     if unit == "us":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_us
 */
    __pyx_r = NPY_FR_ns;
    goto __pyx_L0;

    /* "utils.pxd":140
 *     representation to the corresponding integer `<'int'>`."""
 *     # Common units
 *     if unit == "ns":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_ns
 *     if unit == "us":
 */
  }

  /* "utils.pxd":142
 *     if unit == "ns":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_ns
 *     if unit == "us":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_us
 *     if unit == "ms":
 */
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_unit, __pyx_n_u_us, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(6, 142, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "utils.pxd":143
 *         return np.NPY_DATETIMEUNIT.NPY_FR_ns
 *     if unit == "us":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_us             # <<<<<<<<<<<<<<
 *     if unit == "ms":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_ms
 */
    __pyx_r = NPY_FR_us;
    goto __pyx_L0;

    /* "utils.pxd":142
 *     if unit == "ns":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_ns
 *     if unit == "us":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_us
 *     if unit == "ms":
 */
  }

  /* "utils.pxd":144
 *     if unit == "us":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_us
 *     if unit == "ms":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_ms
 *     if unit == "s":
 */
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_unit, __pyx_n_u_ms, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(6, 144, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "utils.pxd":145
 *         return np.NPY_DATETIMEUNIT.NPY_FR_us
 *     if unit == "ms":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_ms             # <<<<<<<<<<<<<<
 *     if unit == "s":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_s
 */
    __pyx_r = NPY_FR_ms;
    goto __pyx_L0;

    /* "utils.pxd":144
 *     if unit == "us":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_us
 *     if unit == "ms":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_ms
 *     if unit == "s":
 */
  }

  /* "utils.pxd":146
 *     if unit == "ms":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_ms
 *     if unit == "s":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_s
 *     if unit == "m":
 */
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_unit, __pyx_n_u_s, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(6, 146, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "utils.pxd":147
 *         return np.NPY_DATETIMEUNIT.NPY_FR_ms
 *     if unit == "s":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_s             # <<<<<<<<<<<<<<
 *     if unit == "m":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_m
 */
    __pyx_r = NPY_FR_s;
    goto __pyx_L0;

    /* "utils.pxd":146
 *     if unit == "ms":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_ms
 *     if unit == "s":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_s
 *     if unit == "m":
 */
  }

  /* "utils.pxd":148
 *     if unit == "s":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_s
 *     if unit == "m":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_m
 *     if unit == "h":
 */
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_unit, __pyx_n_u_m, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(6, 148, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "utils.pxd":149
 *         return np.NPY_DATETIMEUNIT.NPY_FR_s
 *     if unit == "m":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_m             # <<<<<<<<<<<<<<
 *     if unit == "h":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_h
 */
    __pyx_r = NPY_FR_m;
    goto __pyx_L0;

    /* "utils.pxd":148
 *     if unit == "s":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_s
 *     if unit == "m":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_m
 *     if unit == "h":
 */
  }

  /* "utils.pxd":150
 *     if unit == "m":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_m
 *     if unit == "h":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_h
 *     if unit == "D":
 */
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_unit, __pyx_n_u_h, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(6, 150, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "utils.pxd":151
 *         return np.NPY_DATETIMEUNIT.NPY_FR_m
 *     if unit == "h":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_h             # <<<<<<<<<<<<<<
 *     if unit == "D":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_D
 */
    __pyx_r = NPY_FR_h;
    goto __pyx_L0;

    /* "utils.pxd":150
 *     if unit == "m":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_m
 *     if unit == "h":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_h
 *     if unit == "D":
 */
  }

  /* "utils.pxd":152
 *     if unit == "h":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_h
 *     if unit == "D":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_D
 *     # Uncommon units
 */
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_unit, __pyx_n_u_D, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(6, 152, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "utils.pxd":153
 *         return np.NPY_DATETIMEUNIT.NPY_FR_h
 *     if unit == "D":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_D             # <<<<<<<<<<<<<<
 *     # Uncommon units
 *     if unit == "Y":
 */
    __pyx_r = NPY_FR_D;
    goto __pyx_L0;

    /* "utils.pxd":152
 *     if unit == "h":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_h
 *     if unit == "D":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_D
 *     # Uncommon units
 */
  }

  /* "utils.pxd":155
 *         return np.NPY_DATETIMEUNIT.NPY_FR_D
 *     # Uncommon units
 *     if unit == "Y":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_Y
 *     if unit == "M":
 */
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_unit, __pyx_n_u_Y, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(6, 155, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "utils.pxd":156
 *     # Uncommon units
 *     if unit == "Y":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_Y             # <<<<<<<<<<<<<<
 *     if unit == "M":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_M
 */
    __pyx_r = NPY_FR_Y;
    goto __pyx_L0;

    /* "utils.pxd":155
 *         return np.NPY_DATETIMEUNIT.NPY_FR_D
 *     # Uncommon units
 *     if unit == "Y":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_Y
 *     if unit == "M":
 */
  }

  /* "utils.pxd":157
 *     if unit == "Y":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_Y
 *     if unit == "M":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_M
 *     if unit == "W":
 */
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_unit, __pyx_n_u_M, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(6, 157, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "utils.pxd":158
 *         return np.NPY_DATETIMEUNIT.NPY_FR_Y
 *     if unit == "M":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_M             # <<<<<<<<<<<<<<
 *     if unit == "W":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_W
 */
    __pyx_r = NPY_FR_M;
    goto __pyx_L0;

    /* "utils.pxd":157
 *     if unit == "Y":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_Y
 *     if unit == "M":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_M
 *     if unit == "W":
 */
  }

  /* "utils.pxd":159
 *     if unit == "M":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_M
 *     if unit == "W":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_W
 *     if unit == "ps":
 */
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_unit, __pyx_n_u_W, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(6, 159, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "utils.pxd":160
 *         return np.NPY_DATETIMEUNIT.NPY_FR_M
 *     if unit == "W":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_W             # <<<<<<<<<<<<<<
 *     if unit == "ps":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_ps
 */
    __pyx_r = NPY_FR_W;
    goto __pyx_L0;

    /* "utils.pxd":159
 *     if unit == "M":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_M
 *     if unit == "W":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_W
 *     if unit == "ps":
 */
  }

  /* "utils.pxd":161
 *     if unit == "W":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_W
 *     if unit == "ps":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_ps
 *     if unit == "fs":
 */
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_unit, __pyx_n_u_ps, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(6, 161, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "utils.pxd":162
 *         return np.NPY_DATETIMEUNIT.NPY_FR_W
 *     if unit == "ps":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_ps             # <<<<<<<<<<<<<<
 *     if unit == "fs":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_fs
 */
    __pyx_r = NPY_FR_ps;
    goto __pyx_L0;

    /* "utils.pxd":161
 *     if unit == "W":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_W
 *     if unit == "ps":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_ps
 *     if unit == "fs":
 */
  }

  /* "utils.pxd":163
 *     if unit == "ps":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_ps
 *     if unit == "fs":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_fs
 *     if unit == "as":
 */
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_unit, __pyx_n_u_fs, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(6, 163, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "utils.pxd":164
 *         return np.NPY_DATETIMEUNIT.NPY_FR_ps
 *     if unit == "fs":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_fs             # <<<<<<<<<<<<<<
 *     if unit == "as":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_as
 */
    __pyx_r = NPY_FR_fs;
    goto __pyx_L0;

    /* "utils.pxd":163
 *     if unit == "ps":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_ps
 *     if unit == "fs":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_fs
 *     if unit == "as":
 */
  }

  /* "utils.pxd":165
 *     if unit == "fs":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_fs
 *     if unit == "as":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_as
 *     # if unit == "B":
 */
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_unit, __pyx_n_u_as, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(6, 165, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "utils.pxd":166
 *         return np.NPY_DATETIMEUNIT.NPY_FR_fs
 *     if unit == "as":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_as             # <<<<<<<<<<<<<<
 *     # if unit == "B":
 *     #     return np.NPY_DATETIMEUNIT.NPY_FR_B
 */
    __pyx_r = NPY_FR_as;
    goto __pyx_L0;

    /* "utils.pxd":165
 *     if unit == "fs":
 *         return np.NPY_DATETIMEUNIT.NPY_FR_fs
 *     if unit == "as":             # <<<<<<<<<<<<<<
 *         return np.NPY_DATETIMEUNIT.NPY_FR_as
 *     # if unit == "B":
 */
  }

  /* "utils.pxd":169
 *     # if unit == "B":
 *     #     return np.NPY_DATETIMEUNIT.NPY_FR_B
 *     raise ValueError("Unsupported numpy time unit: %s." % (unit))             # <<<<<<<<<<<<<<
 * 
 * cdef inline int parse_arr_nptime_unit(np.ndarray arr):
 */
  __pyx_t_2 = PyUnicode_Format(__pyx_kp_u_Unsupported_numpy_time_unit_s, __pyx_v_unit); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(6, 169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_Raise(__pyx_t_3, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(6, 169, __pyx_L1_error)

  /* "utils.pxd":136
 *     raise ValueError("Unsupported numpy time unit: %d." % (unit))
 * 
 * cdef inline int map_nptime_unit_str2int(str unit):             # <<<<<<<<<<<<<<
 *     """Map ndarray[datetime64/timedelta64] time unit from string
 *     representation to the corresponding integer `<'int'>`."""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("serializor.utils.map_nptime_unit_str2int", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "utils.pxd":171
 *     raise ValueError("Unsupported numpy time unit: %s." % (unit))
 * 
 * cdef inline int parse_arr_nptime_unit(np.ndarray arr):             # <<<<<<<<<<<<<<
 *     """Parse numpy datetime64/timedelta64 time unit from the
 *     given 'arr', returns the unit in `<'int'>`."""
 */

static CYTHON_INLINE int __pyx_f_10serializor_5utils_parse_arr_nptime_unit(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_v_dtype_str = 0;
  Py_ssize_t __pyx_v_length;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("parse_arr_nptime_unit", 1);

  /* "utils.pxd":175
 *     given 'arr', returns the unit in `<'int'>`."""
 *     cdef:
 *         str dtype_str = arr.dtype.str             # <<<<<<<<<<<<<<
 *         Py_ssize_t length = str_len(dtype_str)
 *     if length < 6:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_arr), __pyx_n_s_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_str); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(PyUnicode_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("unicode", __pyx_t_2))) __PYX_ERR(6, 175, __pyx_L1_error)
  __pyx_v_dtype_str = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "utils.pxd":176
 *     cdef:
 *         str dtype_str = arr.dtype.str
 *         Py_ssize_t length = str_len(dtype_str)             # <<<<<<<<<<<<<<
 *     if length < 6:
 *         raise ValueError("Failed to parse arr time unit from: '%s'." % dtype_str)
 */
  __pyx_v_length = PyUnicode_GET_LENGTH(__pyx_v_dtype_str);

  /* "utils.pxd":177
 *         str dtype_str = arr.dtype.str
 *         Py_ssize_t length = str_len(dtype_str)
 *     if length < 6:             # <<<<<<<<<<<<<<
 *         raise ValueError("Failed to parse arr time unit from: '%s'." % dtype_str)
 *     return map_nptime_unit_str2int(str_substr(dtype_str, 4, length - 1))
 */
  __pyx_t_3 = (__pyx_v_length < 6);
  if (unlikely(__pyx_t_3)) {

    /* "utils.pxd":178
 *         Py_ssize_t length = str_len(dtype_str)
 *     if length < 6:
 *         raise ValueError("Failed to parse arr time unit from: '%s'." % dtype_str)             # <<<<<<<<<<<<<<
 *     return map_nptime_unit_str2int(str_substr(dtype_str, 4, length - 1))
 */
    __pyx_t_2 = PyUnicode_Format(__pyx_kp_u_Failed_to_parse_arr_time_unit_fr, __pyx_v_dtype_str); if (unlikely(!__pyx_t_2)) __PYX_ERR(6, 178, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 178, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(6, 178, __pyx_L1_error)

    /* "utils.pxd":177
 *         str dtype_str = arr.dtype.str
 *         Py_ssize_t length = str_len(dtype_str)
 *     if length < 6:             # <<<<<<<<<<<<<<
 *         raise ValueError("Failed to parse arr time unit from: '%s'." % dtype_str)
 *     return map_nptime_unit_str2int(str_substr(dtype_str, 4, length - 1))
 */
  }

  /* "utils.pxd":179
 *     if length < 6:
 *         raise ValueError("Failed to parse arr time unit from: '%s'." % dtype_str)
 *     return map_nptime_unit_str2int(str_substr(dtype_str, 4, length - 1))             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = PyUnicode_Substring(__pyx_v_dtype_str, 4, (__pyx_v_length - 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 179, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __pyx_f_10serializor_5utils_map_nptime_unit_str2int(((PyObject*)__pyx_t_1)); if (unlikely(__pyx_t_4 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(6, 179, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_4;
  goto __pyx_L0;

  /* "utils.pxd":171
 *     raise ValueError("Unsupported numpy time unit: %s." % (unit))
 * 
 * cdef inline int parse_arr_nptime_unit(np.ndarray arr):             # <<<<<<<<<<<<<<
 *     """Parse numpy datetime64/timedelta64 time unit from the
 *     given 'arr', returns the unit in `<'int'>`."""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("serializor.utils.parse_arr_nptime_unit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dtype_str);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":29
 * 
 * # Orjson loads --------------------------------------------------------------------------------
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _orjson_loads(data: object) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__orjson_loads(PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  unsigned int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_orjson_loads", 1);

  /* "serializor/unicode/des.py":36
 *     Based on [orjson](https://github.com/ijl/orjson) `'loads()'` function.
 *     """
 *     return _loads(data)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_loads); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_data};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 1+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 36, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":29
 * 
 * # Orjson loads --------------------------------------------------------------------------------
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _orjson_loads(data: object) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("serializor.unicode.des._orjson_loads", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":40
 * 
 * # Basic Types ---------------------------------------------------------------------------------
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_str(data: str, pos: cython.Py_ssize_t[1]) -> str:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_str(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  unsigned PY_LONG_LONG __pyx_v_size;
  Py_ssize_t __pyx_v_idx;
  Py_ssize_t __pyx_v_end;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  unsigned PY_LONG_LONG __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_str", 1);

  /* "serializor/unicode/des.py":47
 *     Position updates to the next charactor after deserialization.
 *     """
 *     pos[0] += 1  # skip identifier             # <<<<<<<<<<<<<<
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *     idx: cython.Py_ssize_t = pos[0]
 */
  __pyx_t_1 = 0;
  (__pyx_v_pos[__pyx_t_1]) = ((__pyx_v_pos[__pyx_t_1]) + 1);

  /* "serializor/unicode/des.py":48
 *     """
 *     pos[0] += 1  # skip identifier
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *     idx: cython.Py_ssize_t = pos[0]
 *     end: cython.Py_ssize_t = idx + size
 */
  __pyx_t_2 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_2 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 48, __pyx_L1_error)
  __pyx_v_size = __pyx_t_2;

  /* "serializor/unicode/des.py":49
 *     pos[0] += 1  # skip identifier
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *     idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *     end: cython.Py_ssize_t = idx + size
 *     pos[0] = end  # set new position
 */
  __pyx_v_idx = (__pyx_v_pos[0]);

  /* "serializor/unicode/des.py":50
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *     idx: cython.Py_ssize_t = pos[0]
 *     end: cython.Py_ssize_t = idx + size             # <<<<<<<<<<<<<<
 *     pos[0] = end  # set new position
 *     return data[idx:end]
 */
  __pyx_v_end = (__pyx_v_idx + __pyx_v_size);

  /* "serializor/unicode/des.py":51
 *     idx: cython.Py_ssize_t = pos[0]
 *     end: cython.Py_ssize_t = idx + size
 *     pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *     return data[idx:end]
 * 
 */
  (__pyx_v_pos[0]) = __pyx_v_end;

  /* "serializor/unicode/des.py":52
 *     end: cython.Py_ssize_t = idx + size
 *     pos[0] = end  # set new position
 *     return data[idx:end]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":40
 * 
 * # Basic Types ---------------------------------------------------------------------------------
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_str(data: str, pos: cython.Py_ssize_t[1]) -> str:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("serializor.unicode.des._des_str", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":55
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_int(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_int(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_int", 1);

  /* "serializor/unicode/des.py":62
 *     Position updates to the next charactor after deserialization.
 *     """
 *     return int(_des_str(data, pos))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des__des_str(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 62, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyNumber_Int(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 62, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":55
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_int(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("serializor.unicode.des._des_int", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":65
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_float(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_float(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_float", 1);

  /* "serializor/unicode/des.py":72
 *     Position updates to the next charactor after deserialization.
 *     """
 *     return float(_des_str(data, pos))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des__des_str(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 72, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "float() argument must be a string or a number, not 'NoneType'");
    __PYX_ERR(1, 72, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_PyUnicode_AsDouble(__pyx_t_1); if (unlikely(__pyx_t_2 == ((double)((double)-1)) && PyErr_Occurred())) __PYX_ERR(1, 72, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyFloat_FromDouble(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 72, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":65
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_float(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("serializor.unicode.des._des_float", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":75
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_bool(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_bool(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  unsigned char __pyx_v_val;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("_des_bool", 1);

  /* "serializor/unicode/des.py":82
 *     Position updates to the next charactor after deserialization.
 *     """
 *     val: cython.uchar = read_char(data, pos[0] + 1)             # <<<<<<<<<<<<<<
 *     pos[0] += 2  # skip identifier & boolean value
 *     return True if val == 1 else False
 */
  __pyx_v_val = PyUnicode_READ_CHAR(__pyx_v_data, ((__pyx_v_pos[0]) + 1));

  /* "serializor/unicode/des.py":83
 *     """
 *     val: cython.uchar = read_char(data, pos[0] + 1)
 *     pos[0] += 2  # skip identifier & boolean value             # <<<<<<<<<<<<<<
 *     return True if val == 1 else False
 * 
 */
  __pyx_t_1 = 0;
  (__pyx_v_pos[__pyx_t_1]) = ((__pyx_v_pos[__pyx_t_1]) + 2);

  /* "serializor/unicode/des.py":84
 *     val: cython.uchar = read_char(data, pos[0] + 1)
 *     pos[0] += 2  # skip identifier & boolean value
 *     return True if val == 1 else False             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = (__pyx_v_val == 1);
  if (__pyx_t_3) {
    __Pyx_INCREF(Py_True);
    __pyx_t_2 = Py_True;
  } else {
    __Pyx_INCREF(Py_False);
    __pyx_t_2 = Py_False;
  }
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":75
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_bool(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":87
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_none(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_none(CYTHON_UNUSED PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  __Pyx_RefNannySetupContext("_des_none", 1);

  /* "serializor/unicode/des.py":94
 *     Position updates to the next charactor after deserialization.
 *     """
 *     pos[0] += 1  # skip identifier             # <<<<<<<<<<<<<<
 *     return None
 * 
 */
  __pyx_t_1 = 0;
  (__pyx_v_pos[__pyx_t_1]) = ((__pyx_v_pos[__pyx_t_1]) + 1);

  /* "serializor/unicode/des.py":95
 *     """
 *     pos[0] += 1  # skip identifier
 *     return None             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "serializor/unicode/des.py":87
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_none(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":99
 * 
 * # Date&Time Types -----------------------------------------------------------------------------
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_datetime(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_datetime(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  Py_ssize_t __pyx_v_idx;
  unsigned char __pyx_v_flag;
  int __pyx_v_yy;
  int __pyx_v_mm;
  int __pyx_v_dd;
  int __pyx_v_hh;
  int __pyx_v_mi;
  int __pyx_v_ss;
  int __pyx_v_us;
  unsigned PY_LONG_LONG __pyx_v_size;
  Py_ssize_t __pyx_v_end;
  PyObject *__pyx_v_tz_name = 0;
  PyDateTime_Delta *__pyx_v_offset = NULL;
  PyObject *__pyx_v_tzinfo = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  unsigned char __pyx_t_1;
  unsigned short __pyx_t_2;
  unsigned int __pyx_t_3;
  long __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  struct __pyx_opt_args_7cpython_8datetime_datetime_new __pyx_t_6;
  unsigned PY_LONG_LONG __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  struct __pyx_opt_args_7cpython_8datetime_timezone_new __pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_datetime", 1);

  /* "serializor/unicode/des.py":106
 *     Position updates to the next charactor after deserialization.
 *     """
 *     idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *     flag: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore
 *     yy: cython.int = unpack_uint16(data, idx + 2)  # type: ignore
 */
  __pyx_v_idx = (__pyx_v_pos[0]);

  /* "serializor/unicode/des.py":107
 *     """
 *     idx: cython.Py_ssize_t = pos[0]
 *     flag: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore             # <<<<<<<<<<<<<<
 *     yy: cython.int = unpack_uint16(data, idx + 2)  # type: ignore
 *     mm: cython.int = unpack_uint8(data, idx + 4)  # type: ignore
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_idx + 1)); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 107, __pyx_L1_error)
  __pyx_v_flag = __pyx_t_1;

  /* "serializor/unicode/des.py":108
 *     idx: cython.Py_ssize_t = pos[0]
 *     flag: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore
 *     yy: cython.int = unpack_uint16(data, idx + 2)  # type: ignore             # <<<<<<<<<<<<<<
 *     mm: cython.int = unpack_uint8(data, idx + 4)  # type: ignore
 *     dd: cython.int = unpack_uint8(data, idx + 5)  # type: ignore
 */
  __pyx_t_2 = __pyx_f_10serializor_7unicode_3des_unpack_uint16(__pyx_v_data, (__pyx_v_idx + 2)); if (unlikely(__pyx_t_2 == ((unsigned short)-1) && PyErr_Occurred())) __PYX_ERR(1, 108, __pyx_L1_error)
  __pyx_v_yy = __pyx_t_2;

  /* "serializor/unicode/des.py":109
 *     flag: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore
 *     yy: cython.int = unpack_uint16(data, idx + 2)  # type: ignore
 *     mm: cython.int = unpack_uint8(data, idx + 4)  # type: ignore             # <<<<<<<<<<<<<<
 *     dd: cython.int = unpack_uint8(data, idx + 5)  # type: ignore
 *     hh: cython.int = unpack_uint8(data, idx + 6)  # type: ignore
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_idx + 4)); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 109, __pyx_L1_error)
  __pyx_v_mm = __pyx_t_1;

  /* "serializor/unicode/des.py":110
 *     yy: cython.int = unpack_uint16(data, idx + 2)  # type: ignore
 *     mm: cython.int = unpack_uint8(data, idx + 4)  # type: ignore
 *     dd: cython.int = unpack_uint8(data, idx + 5)  # type: ignore             # <<<<<<<<<<<<<<
 *     hh: cython.int = unpack_uint8(data, idx + 6)  # type: ignore
 *     mi: cython.int = unpack_uint8(data, idx + 7)  # type: ignore
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_idx + 5)); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 110, __pyx_L1_error)
  __pyx_v_dd = __pyx_t_1;

  /* "serializor/unicode/des.py":111
 *     mm: cython.int = unpack_uint8(data, idx + 4)  # type: ignore
 *     dd: cython.int = unpack_uint8(data, idx + 5)  # type: ignore
 *     hh: cython.int = unpack_uint8(data, idx + 6)  # type: ignore             # <<<<<<<<<<<<<<
 *     mi: cython.int = unpack_uint8(data, idx + 7)  # type: ignore
 *     ss: cython.int = unpack_uint8(data, idx + 8)  # type: ignore
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_idx + 6)); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 111, __pyx_L1_error)
  __pyx_v_hh = __pyx_t_1;

  /* "serializor/unicode/des.py":112
 *     dd: cython.int = unpack_uint8(data, idx + 5)  # type: ignore
 *     hh: cython.int = unpack_uint8(data, idx + 6)  # type: ignore
 *     mi: cython.int = unpack_uint8(data, idx + 7)  # type: ignore             # <<<<<<<<<<<<<<
 *     ss: cython.int = unpack_uint8(data, idx + 8)  # type: ignore
 *     us: cython.int = unpack_uint32(data, idx + 9)  # type: ignore
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_idx + 7)); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 112, __pyx_L1_error)
  __pyx_v_mi = __pyx_t_1;

  /* "serializor/unicode/des.py":113
 *     hh: cython.int = unpack_uint8(data, idx + 6)  # type: ignore
 *     mi: cython.int = unpack_uint8(data, idx + 7)  # type: ignore
 *     ss: cython.int = unpack_uint8(data, idx + 8)  # type: ignore             # <<<<<<<<<<<<<<
 *     us: cython.int = unpack_uint32(data, idx + 9)  # type: ignore
 *     # Without timezone
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_idx + 8)); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 113, __pyx_L1_error)
  __pyx_v_ss = __pyx_t_1;

  /* "serializor/unicode/des.py":114
 *     mi: cython.int = unpack_uint8(data, idx + 7)  # type: ignore
 *     ss: cython.int = unpack_uint8(data, idx + 8)  # type: ignore
 *     us: cython.int = unpack_uint32(data, idx + 9)  # type: ignore             # <<<<<<<<<<<<<<
 *     # Without timezone
 *     if flag == 0:
 */
  __pyx_t_3 = __pyx_f_10serializor_7unicode_3des_unpack_uint32(__pyx_v_data, (__pyx_v_idx + 9)); if (unlikely(__pyx_t_3 == ((unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(1, 114, __pyx_L1_error)
  __pyx_v_us = __pyx_t_3;

  /* "serializor/unicode/des.py":116
 *     us: cython.int = unpack_uint32(data, idx + 9)  # type: ignore
 *     # Without timezone
 *     if flag == 0:             # <<<<<<<<<<<<<<
 *         pos[0] += 13  # set new position
 *         return datetime.datetime_new(yy, mm, dd, hh, mi, ss, us, None, 0)
 */
  switch (__pyx_v_flag) {
    case 0:

    /* "serializor/unicode/des.py":117
 *     # Without timezone
 *     if flag == 0:
 *         pos[0] += 13  # set new position             # <<<<<<<<<<<<<<
 *         return datetime.datetime_new(yy, mm, dd, hh, mi, ss, us, None, 0)
 *     # Timezone: approach 1 (tz_name)
 */
    __pyx_t_4 = 0;
    (__pyx_v_pos[__pyx_t_4]) = ((__pyx_v_pos[__pyx_t_4]) + 13);

    /* "serializor/unicode/des.py":118
 *     if flag == 0:
 *         pos[0] += 13  # set new position
 *         return datetime.datetime_new(yy, mm, dd, hh, mi, ss, us, None, 0)             # <<<<<<<<<<<<<<
 *     # Timezone: approach 1 (tz_name)
 *     elif flag == 1:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_6.__pyx_n = 1;
    __pyx_t_6.fold = 0;
    __pyx_t_5 = ((PyObject *)__pyx_f_7cpython_8datetime_datetime_new(__pyx_v_yy, __pyx_v_mm, __pyx_v_dd, __pyx_v_hh, __pyx_v_mi, __pyx_v_ss, __pyx_v_us, Py_None, &__pyx_t_6)); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 118, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":116
 *     us: cython.int = unpack_uint32(data, idx + 9)  # type: ignore
 *     # Without timezone
 *     if flag == 0:             # <<<<<<<<<<<<<<
 *         pos[0] += 13  # set new position
 *         return datetime.datetime_new(yy, mm, dd, hh, mi, ss, us, None, 0)
 */
    break;
    case 1:

    /* "serializor/unicode/des.py":121
 *     # Timezone: approach 1 (tz_name)
 *     elif flag == 1:
 *         pos[0] += 13  # set new position             # <<<<<<<<<<<<<<
 *         size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx = pos[0]
 */
    __pyx_t_4 = 0;
    (__pyx_v_pos[__pyx_t_4]) = ((__pyx_v_pos[__pyx_t_4]) + 13);

    /* "serializor/unicode/des.py":122
 *     elif flag == 1:
 *         pos[0] += 13  # set new position
 *         size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         idx = pos[0]
 *         end: cython.Py_ssize_t = idx + size
 */
    __pyx_t_7 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_7 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 122, __pyx_L1_error)
    __pyx_v_size = __pyx_t_7;

    /* "serializor/unicode/des.py":123
 *         pos[0] += 13  # set new position
 *         size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx = pos[0]             # <<<<<<<<<<<<<<
 *         end: cython.Py_ssize_t = idx + size
 *         tz_name: str = data[idx:end]
 */
    __pyx_v_idx = (__pyx_v_pos[0]);

    /* "serializor/unicode/des.py":124
 *         size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx = pos[0]
 *         end: cython.Py_ssize_t = idx + size             # <<<<<<<<<<<<<<
 *         tz_name: str = data[idx:end]
 *         pos[0] = end  # set new position
 */
    __pyx_v_end = (__pyx_v_idx + __pyx_v_size);

    /* "serializor/unicode/des.py":125
 *         idx = pos[0]
 *         end: cython.Py_ssize_t = idx + size
 *         tz_name: str = data[idx:end]             # <<<<<<<<<<<<<<
 *         pos[0] = end  # set new position
 *         return datetime.datetime_new(yy, mm, dd, hh, mi, ss, us, ZoneInfo(tz_name), 0)
 */
    __pyx_t_5 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 125, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_v_tz_name = ((PyObject*)__pyx_t_5);
    __pyx_t_5 = 0;

    /* "serializor/unicode/des.py":126
 *         end: cython.Py_ssize_t = idx + size
 *         tz_name: str = data[idx:end]
 *         pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *         return datetime.datetime_new(yy, mm, dd, hh, mi, ss, us, ZoneInfo(tz_name), 0)
 *     # Timezone: approach 2 (tz_offset)
 */
    (__pyx_v_pos[0]) = __pyx_v_end;

    /* "serializor/unicode/des.py":127
 *         tz_name: str = data[idx:end]
 *         pos[0] = end  # set new position
 *         return datetime.datetime_new(yy, mm, dd, hh, mi, ss, us, ZoneInfo(tz_name), 0)             # <<<<<<<<<<<<<<
 *     # Timezone: approach 2 (tz_offset)
 *     else:
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_n_s_ZoneInfo); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 127, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = NULL;
    __pyx_t_3 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
        __pyx_t_3 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_v_tz_name};
      __pyx_t_5 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+1-__pyx_t_3, 1+__pyx_t_3);
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 127, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __pyx_t_6.__pyx_n = 1;
    __pyx_t_6.fold = 0;
    __pyx_t_8 = ((PyObject *)__pyx_f_7cpython_8datetime_datetime_new(__pyx_v_yy, __pyx_v_mm, __pyx_v_dd, __pyx_v_hh, __pyx_v_mi, __pyx_v_ss, __pyx_v_us, __pyx_t_5, &__pyx_t_6)); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 127, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":120
 *         return datetime.datetime_new(yy, mm, dd, hh, mi, ss, us, None, 0)
 *     # Timezone: approach 1 (tz_name)
 *     elif flag == 1:             # <<<<<<<<<<<<<<
 *         pos[0] += 13  # set new position
 *         size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 */
    break;
    default:

    /* "serializor/unicode/des.py":131
 *     else:
 *         offset = datetime.timedelta_new(
 *             unpack_int32(data, idx + 13),  # type: ignore             # <<<<<<<<<<<<<<
 *             unpack_int32(data, idx + 17),  # type: ignore
 *             unpack_int32(data, idx + 21),  # type: ignore
 */
    __pyx_t_10 = __pyx_f_10serializor_7unicode_3des_unpack_int32(__pyx_v_data, (__pyx_v_idx + 13)); if (unlikely(__pyx_t_10 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 131, __pyx_L1_error)

    /* "serializor/unicode/des.py":132
 *         offset = datetime.timedelta_new(
 *             unpack_int32(data, idx + 13),  # type: ignore
 *             unpack_int32(data, idx + 17),  # type: ignore             # <<<<<<<<<<<<<<
 *             unpack_int32(data, idx + 21),  # type: ignore
 *         )
 */
    __pyx_t_11 = __pyx_f_10serializor_7unicode_3des_unpack_int32(__pyx_v_data, (__pyx_v_idx + 17)); if (unlikely(__pyx_t_11 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 132, __pyx_L1_error)

    /* "serializor/unicode/des.py":133
 *             unpack_int32(data, idx + 13),  # type: ignore
 *             unpack_int32(data, idx + 17),  # type: ignore
 *             unpack_int32(data, idx + 21),  # type: ignore             # <<<<<<<<<<<<<<
 *         )
 *         pos[0] += 25  # set new position
 */
    __pyx_t_12 = __pyx_f_10serializor_7unicode_3des_unpack_int32(__pyx_v_data, (__pyx_v_idx + 21)); if (unlikely(__pyx_t_12 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 133, __pyx_L1_error)

    /* "serializor/unicode/des.py":130
 *     # Timezone: approach 2 (tz_offset)
 *     else:
 *         offset = datetime.timedelta_new(             # <<<<<<<<<<<<<<
 *             unpack_int32(data, idx + 13),  # type: ignore
 *             unpack_int32(data, idx + 17),  # type: ignore
 */
    __pyx_t_8 = ((PyObject *)__pyx_f_7cpython_8datetime_timedelta_new(__pyx_t_10, __pyx_t_11, __pyx_t_12)); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 130, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_v_offset = ((PyDateTime_Delta *)__pyx_t_8);
    __pyx_t_8 = 0;

    /* "serializor/unicode/des.py":135
 *             unpack_int32(data, idx + 21),  # type: ignore
 *         )
 *         pos[0] += 25  # set new position             # <<<<<<<<<<<<<<
 *         tzinfo = datetime.timezone_new(offset, None)
 *         return datetime.datetime_new(yy, mm, dd, hh, mi, ss, us, tzinfo, 0)
 */
    __pyx_t_4 = 0;
    (__pyx_v_pos[__pyx_t_4]) = ((__pyx_v_pos[__pyx_t_4]) + 25);

    /* "serializor/unicode/des.py":136
 *         )
 *         pos[0] += 25  # set new position
 *         tzinfo = datetime.timezone_new(offset, None)             # <<<<<<<<<<<<<<
 *         return datetime.datetime_new(yy, mm, dd, hh, mi, ss, us, tzinfo, 0)
 * 
 */
    __pyx_t_13.__pyx_n = 1;
    __pyx_t_13.name = Py_None;
    __pyx_t_8 = __pyx_f_7cpython_8datetime_timezone_new(((PyObject *)__pyx_v_offset), &__pyx_t_13); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 136, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_v_tzinfo = __pyx_t_8;
    __pyx_t_8 = 0;

    /* "serializor/unicode/des.py":137
 *         pos[0] += 25  # set new position
 *         tzinfo = datetime.timezone_new(offset, None)
 *         return datetime.datetime_new(yy, mm, dd, hh, mi, ss, us, tzinfo, 0)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_6.__pyx_n = 1;
    __pyx_t_6.fold = 0;
    __pyx_t_8 = ((PyObject *)__pyx_f_7cpython_8datetime_datetime_new(__pyx_v_yy, __pyx_v_mm, __pyx_v_dd, __pyx_v_hh, __pyx_v_mi, __pyx_v_ss, __pyx_v_us, __pyx_v_tzinfo, &__pyx_t_6)); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 137, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
    break;
  }

  /* "serializor/unicode/des.py":99
 * 
 * # Date&Time Types -----------------------------------------------------------------------------
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_datetime(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("serializor.unicode.des._des_datetime", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_tz_name);
  __Pyx_XDECREF((PyObject *)__pyx_v_offset);
  __Pyx_XDECREF(__pyx_v_tzinfo);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":140
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_date(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_date(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  Py_ssize_t __pyx_v_idx;
  int __pyx_v_year;
  int __pyx_v_month;
  int __pyx_v_day;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  unsigned short __pyx_t_1;
  unsigned char __pyx_t_2;
  long __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_date", 1);

  /* "serializor/unicode/des.py":147
 *     Position updates to the next charactor after deserialization.
 *     """
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip identifier             # <<<<<<<<<<<<<<
 *     year: cython.int = unpack_uint16(data, idx)  # type: ignore
 *     month: cython.int = unpack_uint8(data, idx + 2)  # type: ignore
 */
  __pyx_v_idx = ((__pyx_v_pos[0]) + 1);

  /* "serializor/unicode/des.py":148
 *     """
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip identifier
 *     year: cython.int = unpack_uint16(data, idx)  # type: ignore             # <<<<<<<<<<<<<<
 *     month: cython.int = unpack_uint8(data, idx + 2)  # type: ignore
 *     day: cython.int = unpack_uint8(data, idx + 3)  # type: ignore
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint16(__pyx_v_data, __pyx_v_idx); if (unlikely(__pyx_t_1 == ((unsigned short)-1) && PyErr_Occurred())) __PYX_ERR(1, 148, __pyx_L1_error)
  __pyx_v_year = __pyx_t_1;

  /* "serializor/unicode/des.py":149
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip identifier
 *     year: cython.int = unpack_uint16(data, idx)  # type: ignore
 *     month: cython.int = unpack_uint8(data, idx + 2)  # type: ignore             # <<<<<<<<<<<<<<
 *     day: cython.int = unpack_uint8(data, idx + 3)  # type: ignore
 *     pos[0] += 5  # set new position
 */
  __pyx_t_2 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_idx + 2)); if (unlikely(__pyx_t_2 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 149, __pyx_L1_error)
  __pyx_v_month = __pyx_t_2;

  /* "serializor/unicode/des.py":150
 *     year: cython.int = unpack_uint16(data, idx)  # type: ignore
 *     month: cython.int = unpack_uint8(data, idx + 2)  # type: ignore
 *     day: cython.int = unpack_uint8(data, idx + 3)  # type: ignore             # <<<<<<<<<<<<<<
 *     pos[0] += 5  # set new position
 *     return datetime.date_new(year, month, day)
 */
  __pyx_t_2 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_idx + 3)); if (unlikely(__pyx_t_2 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 150, __pyx_L1_error)
  __pyx_v_day = __pyx_t_2;

  /* "serializor/unicode/des.py":151
 *     month: cython.int = unpack_uint8(data, idx + 2)  # type: ignore
 *     day: cython.int = unpack_uint8(data, idx + 3)  # type: ignore
 *     pos[0] += 5  # set new position             # <<<<<<<<<<<<<<
 *     return datetime.date_new(year, month, day)
 * 
 */
  __pyx_t_3 = 0;
  (__pyx_v_pos[__pyx_t_3]) = ((__pyx_v_pos[__pyx_t_3]) + 5);

  /* "serializor/unicode/des.py":152
 *     day: cython.int = unpack_uint8(data, idx + 3)  # type: ignore
 *     pos[0] += 5  # set new position
 *     return datetime.date_new(year, month, day)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = ((PyObject *)__pyx_f_7cpython_8datetime_date_new(__pyx_v_year, __pyx_v_month, __pyx_v_day)); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":140
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_date(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("serializor.unicode.des._des_date", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":155
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_time(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_time(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  Py_ssize_t __pyx_v_idx;
  unsigned char __pyx_v_flag;
  int __pyx_v_hh;
  int __pyx_v_mi;
  int __pyx_v_ss;
  int __pyx_v_us;
  unsigned PY_LONG_LONG __pyx_v_size;
  Py_ssize_t __pyx_v_end;
  PyObject *__pyx_v_tz_name = 0;
  PyDateTime_Delta *__pyx_v_offset = NULL;
  PyObject *__pyx_v_tzinfo = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  unsigned char __pyx_t_1;
  unsigned int __pyx_t_2;
  long __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  struct __pyx_opt_args_7cpython_8datetime_time_new __pyx_t_5;
  unsigned PY_LONG_LONG __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  struct __pyx_opt_args_7cpython_8datetime_timezone_new __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_time", 1);

  /* "serializor/unicode/des.py":162
 *     Position updates to the next charactor after deserialization.
 *     """
 *     idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *     flag: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore
 *     hh: cython.int = unpack_uint8(data, idx + 2)  # type: ignore
 */
  __pyx_v_idx = (__pyx_v_pos[0]);

  /* "serializor/unicode/des.py":163
 *     """
 *     idx: cython.Py_ssize_t = pos[0]
 *     flag: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore             # <<<<<<<<<<<<<<
 *     hh: cython.int = unpack_uint8(data, idx + 2)  # type: ignore
 *     mi: cython.int = unpack_uint8(data, idx + 3)  # type: ignore
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_idx + 1)); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 163, __pyx_L1_error)
  __pyx_v_flag = __pyx_t_1;

  /* "serializor/unicode/des.py":164
 *     idx: cython.Py_ssize_t = pos[0]
 *     flag: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore
 *     hh: cython.int = unpack_uint8(data, idx + 2)  # type: ignore             # <<<<<<<<<<<<<<
 *     mi: cython.int = unpack_uint8(data, idx + 3)  # type: ignore
 *     ss: cython.int = unpack_uint8(data, idx + 4)  # type: ignore
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_idx + 2)); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 164, __pyx_L1_error)
  __pyx_v_hh = __pyx_t_1;

  /* "serializor/unicode/des.py":165
 *     flag: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore
 *     hh: cython.int = unpack_uint8(data, idx + 2)  # type: ignore
 *     mi: cython.int = unpack_uint8(data, idx + 3)  # type: ignore             # <<<<<<<<<<<<<<
 *     ss: cython.int = unpack_uint8(data, idx + 4)  # type: ignore
 *     us: cython.int = unpack_uint32(data, idx + 5)  # type: ignore
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_idx + 3)); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 165, __pyx_L1_error)
  __pyx_v_mi = __pyx_t_1;

  /* "serializor/unicode/des.py":166
 *     hh: cython.int = unpack_uint8(data, idx + 2)  # type: ignore
 *     mi: cython.int = unpack_uint8(data, idx + 3)  # type: ignore
 *     ss: cython.int = unpack_uint8(data, idx + 4)  # type: ignore             # <<<<<<<<<<<<<<
 *     us: cython.int = unpack_uint32(data, idx + 5)  # type: ignore
 *     # Without timezone
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_idx + 4)); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 166, __pyx_L1_error)
  __pyx_v_ss = __pyx_t_1;

  /* "serializor/unicode/des.py":167
 *     mi: cython.int = unpack_uint8(data, idx + 3)  # type: ignore
 *     ss: cython.int = unpack_uint8(data, idx + 4)  # type: ignore
 *     us: cython.int = unpack_uint32(data, idx + 5)  # type: ignore             # <<<<<<<<<<<<<<
 *     # Without timezone
 *     if flag == 0:
 */
  __pyx_t_2 = __pyx_f_10serializor_7unicode_3des_unpack_uint32(__pyx_v_data, (__pyx_v_idx + 5)); if (unlikely(__pyx_t_2 == ((unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(1, 167, __pyx_L1_error)
  __pyx_v_us = __pyx_t_2;

  /* "serializor/unicode/des.py":169
 *     us: cython.int = unpack_uint32(data, idx + 5)  # type: ignore
 *     # Without timezone
 *     if flag == 0:             # <<<<<<<<<<<<<<
 *         pos[0] += 9
 *         return datetime.time_new(hh, mi, ss, us, None, 0)
 */
  switch (__pyx_v_flag) {
    case 0:

    /* "serializor/unicode/des.py":170
 *     # Without timezone
 *     if flag == 0:
 *         pos[0] += 9             # <<<<<<<<<<<<<<
 *         return datetime.time_new(hh, mi, ss, us, None, 0)
 *     # Timezone: approach 1 (tz_name)
 */
    __pyx_t_3 = 0;
    (__pyx_v_pos[__pyx_t_3]) = ((__pyx_v_pos[__pyx_t_3]) + 9);

    /* "serializor/unicode/des.py":171
 *     if flag == 0:
 *         pos[0] += 9
 *         return datetime.time_new(hh, mi, ss, us, None, 0)             # <<<<<<<<<<<<<<
 *     # Timezone: approach 1 (tz_name)
 *     elif flag == 1:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_5.__pyx_n = 1;
    __pyx_t_5.fold = 0;
    __pyx_t_4 = ((PyObject *)__pyx_f_7cpython_8datetime_time_new(__pyx_v_hh, __pyx_v_mi, __pyx_v_ss, __pyx_v_us, Py_None, &__pyx_t_5)); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 171, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":169
 *     us: cython.int = unpack_uint32(data, idx + 5)  # type: ignore
 *     # Without timezone
 *     if flag == 0:             # <<<<<<<<<<<<<<
 *         pos[0] += 9
 *         return datetime.time_new(hh, mi, ss, us, None, 0)
 */
    break;
    case 1:

    /* "serializor/unicode/des.py":174
 *     # Timezone: approach 1 (tz_name)
 *     elif flag == 1:
 *         pos[0] += 9  # set new position             # <<<<<<<<<<<<<<
 *         size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx = pos[0]
 */
    __pyx_t_3 = 0;
    (__pyx_v_pos[__pyx_t_3]) = ((__pyx_v_pos[__pyx_t_3]) + 9);

    /* "serializor/unicode/des.py":175
 *     elif flag == 1:
 *         pos[0] += 9  # set new position
 *         size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         idx = pos[0]
 *         end: cython.Py_ssize_t = idx + size
 */
    __pyx_t_6 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_6 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 175, __pyx_L1_error)
    __pyx_v_size = __pyx_t_6;

    /* "serializor/unicode/des.py":176
 *         pos[0] += 9  # set new position
 *         size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx = pos[0]             # <<<<<<<<<<<<<<
 *         end: cython.Py_ssize_t = idx + size
 *         tz_name: str = data[idx:end]
 */
    __pyx_v_idx = (__pyx_v_pos[0]);

    /* "serializor/unicode/des.py":177
 *         size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx = pos[0]
 *         end: cython.Py_ssize_t = idx + size             # <<<<<<<<<<<<<<
 *         tz_name: str = data[idx:end]
 *         pos[0] = end  # set new position
 */
    __pyx_v_end = (__pyx_v_idx + __pyx_v_size);

    /* "serializor/unicode/des.py":178
 *         idx = pos[0]
 *         end: cython.Py_ssize_t = idx + size
 *         tz_name: str = data[idx:end]             # <<<<<<<<<<<<<<
 *         pos[0] = end  # set new position
 *         return datetime.time_new(hh, mi, ss, us, ZoneInfo(tz_name), 0)
 */
    __pyx_t_4 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 178, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_v_tz_name = ((PyObject*)__pyx_t_4);
    __pyx_t_4 = 0;

    /* "serializor/unicode/des.py":179
 *         end: cython.Py_ssize_t = idx + size
 *         tz_name: str = data[idx:end]
 *         pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *         return datetime.time_new(hh, mi, ss, us, ZoneInfo(tz_name), 0)
 *     # Timezone: approach 2 (tz_offset)
 */
    (__pyx_v_pos[0]) = __pyx_v_end;

    /* "serializor/unicode/des.py":180
 *         tz_name: str = data[idx:end]
 *         pos[0] = end  # set new position
 *         return datetime.time_new(hh, mi, ss, us, ZoneInfo(tz_name), 0)             # <<<<<<<<<<<<<<
 *     # Timezone: approach 2 (tz_offset)
 *     else:
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_ZoneInfo); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 180, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = NULL;
    __pyx_t_2 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
        __pyx_t_2 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_v_tz_name};
      __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+1-__pyx_t_2, 1+__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 180, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __pyx_t_5.__pyx_n = 1;
    __pyx_t_5.fold = 0;
    __pyx_t_7 = ((PyObject *)__pyx_f_7cpython_8datetime_time_new(__pyx_v_hh, __pyx_v_mi, __pyx_v_ss, __pyx_v_us, __pyx_t_4, &__pyx_t_5)); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 180, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_r = __pyx_t_7;
    __pyx_t_7 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":173
 *         return datetime.time_new(hh, mi, ss, us, None, 0)
 *     # Timezone: approach 1 (tz_name)
 *     elif flag == 1:             # <<<<<<<<<<<<<<
 *         pos[0] += 9  # set new position
 *         size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 */
    break;
    default:

    /* "serializor/unicode/des.py":184
 *     else:
 *         offset = datetime.timedelta_new(
 *             unpack_int32(data, idx + 9),  # type: ignore             # <<<<<<<<<<<<<<
 *             unpack_int32(data, idx + 13),  # type: ignore
 *             unpack_int32(data, idx + 17),  # type: ignore
 */
    __pyx_t_9 = __pyx_f_10serializor_7unicode_3des_unpack_int32(__pyx_v_data, (__pyx_v_idx + 9)); if (unlikely(__pyx_t_9 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 184, __pyx_L1_error)

    /* "serializor/unicode/des.py":185
 *         offset = datetime.timedelta_new(
 *             unpack_int32(data, idx + 9),  # type: ignore
 *             unpack_int32(data, idx + 13),  # type: ignore             # <<<<<<<<<<<<<<
 *             unpack_int32(data, idx + 17),  # type: ignore
 *         )
 */
    __pyx_t_10 = __pyx_f_10serializor_7unicode_3des_unpack_int32(__pyx_v_data, (__pyx_v_idx + 13)); if (unlikely(__pyx_t_10 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 185, __pyx_L1_error)

    /* "serializor/unicode/des.py":186
 *             unpack_int32(data, idx + 9),  # type: ignore
 *             unpack_int32(data, idx + 13),  # type: ignore
 *             unpack_int32(data, idx + 17),  # type: ignore             # <<<<<<<<<<<<<<
 *         )
 *         pos[0] += 21  # set new position
 */
    __pyx_t_11 = __pyx_f_10serializor_7unicode_3des_unpack_int32(__pyx_v_data, (__pyx_v_idx + 17)); if (unlikely(__pyx_t_11 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 186, __pyx_L1_error)

    /* "serializor/unicode/des.py":183
 *     # Timezone: approach 2 (tz_offset)
 *     else:
 *         offset = datetime.timedelta_new(             # <<<<<<<<<<<<<<
 *             unpack_int32(data, idx + 9),  # type: ignore
 *             unpack_int32(data, idx + 13),  # type: ignore
 */
    __pyx_t_7 = ((PyObject *)__pyx_f_7cpython_8datetime_timedelta_new(__pyx_t_9, __pyx_t_10, __pyx_t_11)); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_v_offset = ((PyDateTime_Delta *)__pyx_t_7);
    __pyx_t_7 = 0;

    /* "serializor/unicode/des.py":188
 *             unpack_int32(data, idx + 17),  # type: ignore
 *         )
 *         pos[0] += 21  # set new position             # <<<<<<<<<<<<<<
 *         tzinfo = datetime.timezone_new(offset, None)
 *         return datetime.time_new(hh, mi, ss, us, tzinfo, 0)
 */
    __pyx_t_3 = 0;
    (__pyx_v_pos[__pyx_t_3]) = ((__pyx_v_pos[__pyx_t_3]) + 21);

    /* "serializor/unicode/des.py":189
 *         )
 *         pos[0] += 21  # set new position
 *         tzinfo = datetime.timezone_new(offset, None)             # <<<<<<<<<<<<<<
 *         return datetime.time_new(hh, mi, ss, us, tzinfo, 0)
 * 
 */
    __pyx_t_12.__pyx_n = 1;
    __pyx_t_12.name = Py_None;
    __pyx_t_7 = __pyx_f_7cpython_8datetime_timezone_new(((PyObject *)__pyx_v_offset), &__pyx_t_12); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_v_tzinfo = __pyx_t_7;
    __pyx_t_7 = 0;

    /* "serializor/unicode/des.py":190
 *         pos[0] += 21  # set new position
 *         tzinfo = datetime.timezone_new(offset, None)
 *         return datetime.time_new(hh, mi, ss, us, tzinfo, 0)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_5.__pyx_n = 1;
    __pyx_t_5.fold = 0;
    __pyx_t_7 = ((PyObject *)__pyx_f_7cpython_8datetime_time_new(__pyx_v_hh, __pyx_v_mi, __pyx_v_ss, __pyx_v_us, __pyx_v_tzinfo, &__pyx_t_5)); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_r = __pyx_t_7;
    __pyx_t_7 = 0;
    goto __pyx_L0;
    break;
  }

  /* "serializor/unicode/des.py":155
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_time(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("serializor.unicode.des._des_time", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_tz_name);
  __Pyx_XDECREF((PyObject *)__pyx_v_offset);
  __Pyx_XDECREF(__pyx_v_tzinfo);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":193
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_timedelta(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_timedelta(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  Py_ssize_t __pyx_v_idx;
  int __pyx_v_days;
  int __pyx_v_seconds;
  int __pyx_v_microseconds;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  long __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_timedelta", 1);

  /* "serializor/unicode/des.py":200
 *     Position updates to the next charactor after deserialization.
 *     """
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip identifier             # <<<<<<<<<<<<<<
 *     days: cython.int = unpack_int32(data, idx)  # type: ignore
 *     seconds: cython.int = unpack_int32(data, idx + 4)  # type: ignore
 */
  __pyx_v_idx = ((__pyx_v_pos[0]) + 1);

  /* "serializor/unicode/des.py":201
 *     """
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip identifier
 *     days: cython.int = unpack_int32(data, idx)  # type: ignore             # <<<<<<<<<<<<<<
 *     seconds: cython.int = unpack_int32(data, idx + 4)  # type: ignore
 *     microseconds: cython.int = unpack_int32(data, idx + 8)  # type: ignore
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_int32(__pyx_v_data, __pyx_v_idx); if (unlikely(__pyx_t_1 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 201, __pyx_L1_error)
  __pyx_v_days = __pyx_t_1;

  /* "serializor/unicode/des.py":202
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip identifier
 *     days: cython.int = unpack_int32(data, idx)  # type: ignore
 *     seconds: cython.int = unpack_int32(data, idx + 4)  # type: ignore             # <<<<<<<<<<<<<<
 *     microseconds: cython.int = unpack_int32(data, idx + 8)  # type: ignore
 *     pos[0] += 13  # set new position
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_int32(__pyx_v_data, (__pyx_v_idx + 4)); if (unlikely(__pyx_t_1 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 202, __pyx_L1_error)
  __pyx_v_seconds = __pyx_t_1;

  /* "serializor/unicode/des.py":203
 *     days: cython.int = unpack_int32(data, idx)  # type: ignore
 *     seconds: cython.int = unpack_int32(data, idx + 4)  # type: ignore
 *     microseconds: cython.int = unpack_int32(data, idx + 8)  # type: ignore             # <<<<<<<<<<<<<<
 *     pos[0] += 13  # set new position
 *     return datetime.timedelta_new(days, seconds, microseconds)
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_int32(__pyx_v_data, (__pyx_v_idx + 8)); if (unlikely(__pyx_t_1 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 203, __pyx_L1_error)
  __pyx_v_microseconds = __pyx_t_1;

  /* "serializor/unicode/des.py":204
 *     seconds: cython.int = unpack_int32(data, idx + 4)  # type: ignore
 *     microseconds: cython.int = unpack_int32(data, idx + 8)  # type: ignore
 *     pos[0] += 13  # set new position             # <<<<<<<<<<<<<<
 *     return datetime.timedelta_new(days, seconds, microseconds)
 * 
 */
  __pyx_t_2 = 0;
  (__pyx_v_pos[__pyx_t_2]) = ((__pyx_v_pos[__pyx_t_2]) + 13);

  /* "serializor/unicode/des.py":205
 *     microseconds: cython.int = unpack_int32(data, idx + 8)  # type: ignore
 *     pos[0] += 13  # set new position
 *     return datetime.timedelta_new(days, seconds, microseconds)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = ((PyObject *)__pyx_f_7cpython_8datetime_timedelta_new(__pyx_v_days, __pyx_v_seconds, __pyx_v_microseconds)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":193
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_timedelta(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("serializor.unicode.des._des_timedelta", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":208
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_struct_time(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_struct_time(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  unsigned int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_struct_time", 1);

  /* "serializor/unicode/des.py":215
 *     Position updates to the next charactor after deserialization.
 *     """
 *     pos[0] += 1  # skip identifier             # <<<<<<<<<<<<<<
 *     return typeref.STRUCT_TIME(_des_list(data, pos))
 * 
 */
  __pyx_t_1 = 0;
  (__pyx_v_pos[__pyx_t_1]) = ((__pyx_v_pos[__pyx_t_1]) + 1);

  /* "serializor/unicode/des.py":216
 *     """
 *     pos[0] += 1  # skip identifier
 *     return typeref.STRUCT_TIME(_des_list(data, pos))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __pyx_f_10serializor_7unicode_3des__des_list(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 216, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_10serializor_7typeref_STRUCT_TIME);
  __pyx_t_4 = __pyx_v_10serializor_7typeref_STRUCT_TIME; __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
      __pyx_t_6 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_3};
    __pyx_t_2 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_6, 1+__pyx_t_6);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 216, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":208
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_struct_time(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("serializor.unicode.des._des_struct_time", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":220
 * 
 * # Numeric Types -------------------------------------------------------------------------------
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_decimal(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_decimal(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  unsigned PY_LONG_LONG __pyx_v_size;
  Py_ssize_t __pyx_v_idx;
  Py_ssize_t __pyx_v_end;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  unsigned PY_LONG_LONG __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  unsigned int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_decimal", 1);

  /* "serializor/unicode/des.py":227
 *     Position updates to the next charactor after deserialization.
 *     """
 *     pos[0] += 1  # skip identifier             # <<<<<<<<<<<<<<
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *     idx: cython.Py_ssize_t = pos[0]
 */
  __pyx_t_1 = 0;
  (__pyx_v_pos[__pyx_t_1]) = ((__pyx_v_pos[__pyx_t_1]) + 1);

  /* "serializor/unicode/des.py":228
 *     """
 *     pos[0] += 1  # skip identifier
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *     idx: cython.Py_ssize_t = pos[0]
 *     end: cython.Py_ssize_t = idx + size
 */
  __pyx_t_2 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_2 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 228, __pyx_L1_error)
  __pyx_v_size = __pyx_t_2;

  /* "serializor/unicode/des.py":229
 *     pos[0] += 1  # skip identifier
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *     idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *     end: cython.Py_ssize_t = idx + size
 *     pos[0] = end  # set new position
 */
  __pyx_v_idx = (__pyx_v_pos[0]);

  /* "serializor/unicode/des.py":230
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *     idx: cython.Py_ssize_t = pos[0]
 *     end: cython.Py_ssize_t = idx + size             # <<<<<<<<<<<<<<
 *     pos[0] = end  # set new position
 *     return typeref.DECIMAL(data[idx:end])
 */
  __pyx_v_end = (__pyx_v_idx + __pyx_v_size);

  /* "serializor/unicode/des.py":231
 *     idx: cython.Py_ssize_t = pos[0]
 *     end: cython.Py_ssize_t = idx + size
 *     pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *     return typeref.DECIMAL(data[idx:end])
 * 
 */
  (__pyx_v_pos[0]) = __pyx_v_end;

  /* "serializor/unicode/des.py":232
 *     end: cython.Py_ssize_t = idx + size
 *     pos[0] = end  # set new position
 *     return typeref.DECIMAL(data[idx:end])             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 232, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_10serializor_7typeref_DECIMAL);
  __pyx_t_5 = __pyx_v_10serializor_7typeref_DECIMAL; __pyx_t_6 = NULL;
  __pyx_t_7 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
      __pyx_t_7 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_4};
    __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 232, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":220
 * 
 * # Numeric Types -------------------------------------------------------------------------------
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_decimal(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("serializor.unicode.des._des_decimal", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":235
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_complex(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_complex(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  unsigned PY_LONG_LONG __pyx_v_size;
  Py_ssize_t __pyx_v_idx;
  Py_ssize_t __pyx_v_end;
  long double __pyx_v_real;
  long double __pyx_v_imag;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  unsigned PY_LONG_LONG __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  double __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_complex", 1);

  /* "serializor/unicode/des.py":242
 *     Position updates to the next charactor after deserialization.
 *     """
 *     pos[0] += 1  # skip identifier             # <<<<<<<<<<<<<<
 *     # Complex: real
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_1 = 0;
  (__pyx_v_pos[__pyx_t_1]) = ((__pyx_v_pos[__pyx_t_1]) + 1);

  /* "serializor/unicode/des.py":244
 *     pos[0] += 1  # skip identifier
 *     # Complex: real
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *     idx: cython.Py_ssize_t = pos[0]
 *     end: cython.Py_ssize_t = idx + size
 */
  __pyx_t_2 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_2 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 244, __pyx_L1_error)
  __pyx_v_size = __pyx_t_2;

  /* "serializor/unicode/des.py":245
 *     # Complex: real
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *     idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *     end: cython.Py_ssize_t = idx + size
 *     real: cython.longdouble = float(data[idx:end])
 */
  __pyx_v_idx = (__pyx_v_pos[0]);

  /* "serializor/unicode/des.py":246
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *     idx: cython.Py_ssize_t = pos[0]
 *     end: cython.Py_ssize_t = idx + size             # <<<<<<<<<<<<<<
 *     real: cython.longdouble = float(data[idx:end])
 *     pos[0] = end  # set new position
 */
  __pyx_v_end = (__pyx_v_idx + __pyx_v_size);

  /* "serializor/unicode/des.py":247
 *     idx: cython.Py_ssize_t = pos[0]
 *     end: cython.Py_ssize_t = idx + size
 *     real: cython.longdouble = float(data[idx:end])             # <<<<<<<<<<<<<<
 *     pos[0] = end  # set new position
 *     # Complex: imag
 */
  __pyx_t_3 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyUnicode_AsDouble(__pyx_t_3); if (unlikely(__pyx_t_4 == ((double)((double)-1)) && PyErr_Occurred())) __PYX_ERR(1, 247, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_real = __pyx_t_4;

  /* "serializor/unicode/des.py":248
 *     end: cython.Py_ssize_t = idx + size
 *     real: cython.longdouble = float(data[idx:end])
 *     pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *     # Complex: imag
 *     size = dec_encoded_int(data, pos)  # type: ignore
 */
  (__pyx_v_pos[0]) = __pyx_v_end;

  /* "serializor/unicode/des.py":250
 *     pos[0] = end  # set new position
 *     # Complex: imag
 *     size = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *     idx: cython.Py_ssize_t = pos[0]
 *     end: cython.Py_ssize_t = idx + size
 */
  __pyx_t_2 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_2 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 250, __pyx_L1_error)
  __pyx_v_size = __pyx_t_2;

  /* "serializor/unicode/des.py":251
 *     # Complex: imag
 *     size = dec_encoded_int(data, pos)  # type: ignore
 *     idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *     end: cython.Py_ssize_t = idx + size
 *     imag: cython.longdouble = float(data[idx:end])
 */
  __pyx_v_idx = (__pyx_v_pos[0]);

  /* "serializor/unicode/des.py":252
 *     size = dec_encoded_int(data, pos)  # type: ignore
 *     idx: cython.Py_ssize_t = pos[0]
 *     end: cython.Py_ssize_t = idx + size             # <<<<<<<<<<<<<<
 *     imag: cython.longdouble = float(data[idx:end])
 *     pos[0] = end  # set new position
 */
  __pyx_v_end = (__pyx_v_idx + __pyx_v_size);

  /* "serializor/unicode/des.py":253
 *     idx: cython.Py_ssize_t = pos[0]
 *     end: cython.Py_ssize_t = idx + size
 *     imag: cython.longdouble = float(data[idx:end])             # <<<<<<<<<<<<<<
 *     pos[0] = end  # set new position
 *     # Generate complex
 */
  __pyx_t_3 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 253, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyUnicode_AsDouble(__pyx_t_3); if (unlikely(__pyx_t_4 == ((double)((double)-1)) && PyErr_Occurred())) __PYX_ERR(1, 253, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_imag = __pyx_t_4;

  /* "serializor/unicode/des.py":254
 *     end: cython.Py_ssize_t = idx + size
 *     imag: cython.longdouble = float(data[idx:end])
 *     pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *     # Generate complex
 *     return gen_complex(real, imag)
 */
  (__pyx_v_pos[0]) = __pyx_v_end;

  /* "serializor/unicode/des.py":256
 *     pos[0] = end  # set new position
 *     # Generate complex
 *     return gen_complex(real, imag)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = PyComplex_FromDoubles(__pyx_v_real, __pyx_v_imag); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":235
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_complex(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("serializor.unicode.des._des_complex", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":260
 * 
 * # Bytes Types ---------------------------------------------------------------------------------
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_bytes(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_bytes(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  unsigned PY_LONG_LONG __pyx_v_size;
  Py_ssize_t __pyx_v_idx;
  Py_ssize_t __pyx_v_end;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  unsigned PY_LONG_LONG __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_bytes", 1);

  /* "serializor/unicode/des.py":267
 *     Position updates to the next charactor after deserialization.
 *     """
 *     pos[0] += 1  # skip identifier             # <<<<<<<<<<<<<<
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *     idx: cython.Py_ssize_t = pos[0]
 */
  __pyx_t_1 = 0;
  (__pyx_v_pos[__pyx_t_1]) = ((__pyx_v_pos[__pyx_t_1]) + 1);

  /* "serializor/unicode/des.py":268
 *     """
 *     pos[0] += 1  # skip identifier
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *     idx: cython.Py_ssize_t = pos[0]
 *     end: cython.Py_ssize_t = idx + size
 */
  __pyx_t_2 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_2 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 268, __pyx_L1_error)
  __pyx_v_size = __pyx_t_2;

  /* "serializor/unicode/des.py":269
 *     pos[0] += 1  # skip identifier
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *     idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *     end: cython.Py_ssize_t = idx + size
 *     pos[0] = end  # set new position
 */
  __pyx_v_idx = (__pyx_v_pos[0]);

  /* "serializor/unicode/des.py":270
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *     idx: cython.Py_ssize_t = pos[0]
 *     end: cython.Py_ssize_t = idx + size             # <<<<<<<<<<<<<<
 *     pos[0] = end  # set new position
 *     return utils.encode_str(data[idx:end], b"utf-8")
 */
  __pyx_v_end = (__pyx_v_idx + __pyx_v_size);

  /* "serializor/unicode/des.py":271
 *     idx: cython.Py_ssize_t = pos[0]
 *     end: cython.Py_ssize_t = idx + size
 *     pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *     return utils.encode_str(data[idx:end], b"utf-8")
 * 
 */
  (__pyx_v_pos[0]) = __pyx_v_end;

  /* "serializor/unicode/des.py":272
 *     end: cython.Py_ssize_t = idx + size
 *     pos[0] = end  # set new position
 *     return utils.encode_str(data[idx:end], b"utf-8")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_f_10serializor_5utils_encode_str(__pyx_t_3, ((char *)"utf-8")); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":260
 * 
 * # Bytes Types ---------------------------------------------------------------------------------
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_bytes(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("serializor.unicode.des._des_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":275
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_bytearray(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_bytearray(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_bytearray", 1);

  /* "serializor/unicode/des.py":282
 *     Position updates to the next charactor after deserialization.
 *     """
 *     return bytearray(_des_bytes(data, pos))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des__des_bytes(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 282, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyByteArray_Type)), __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 282, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":275
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_bytearray(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("serializor.unicode.des._des_bytearray", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":286
 * 
 * # Sequence Types ------------------------------------------------------------------------------
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_list(data: str, pos: cython.Py_ssize_t[1]) -> list:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_list(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  unsigned PY_LONG_LONG __pyx_v_size;
  CYTHON_UNUSED unsigned PY_LONG_LONG __pyx_7genexpr__pyx_v__;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  unsigned PY_LONG_LONG __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  unsigned PY_LONG_LONG __pyx_t_4;
  unsigned PY_LONG_LONG __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_list", 1);

  /* "serializor/unicode/des.py":293
 *     Position updates to the next charactor after deserialization.
 *     """
 *     pos[0] += 1  # skip identifier             # <<<<<<<<<<<<<<
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *     return [_des_data(data, pos) for _ in range(size)]
 */
  __pyx_t_1 = 0;
  (__pyx_v_pos[__pyx_t_1]) = ((__pyx_v_pos[__pyx_t_1]) + 1);

  /* "serializor/unicode/des.py":294
 *     """
 *     pos[0] += 1  # skip identifier
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *     return [_des_data(data, pos) for _ in range(size)]
 * 
 */
  __pyx_t_2 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_2 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 294, __pyx_L1_error)
  __pyx_v_size = __pyx_t_2;

  /* "serializor/unicode/des.py":295
 *     pos[0] += 1  # skip identifier
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *     return [_des_data(data, pos) for _ in range(size)]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  { /* enter inner scope */
    __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 295, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __pyx_v_size;
    __pyx_t_4 = __pyx_t_2;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_7genexpr__pyx_v__ = __pyx_t_5;
      __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__des_data(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 295, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_t_6))) __PYX_ERR(1, 295, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
  } /* exit inner scope */
  __pyx_r = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":286
 * 
 * # Sequence Types ------------------------------------------------------------------------------
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_list(data: str, pos: cython.Py_ssize_t[1]) -> list:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("serializor.unicode.des._des_list", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":298
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_tuple(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_tuple(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_tuple", 1);

  /* "serializor/unicode/des.py":305
 *     Position updates to the next charactor after deserialization.
 *     """
 *     return list_to_tuple(_des_list(data, pos))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des__des_list(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 305, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyList_AsTuple(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 305, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":298
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_tuple(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("serializor.unicode.des._des_tuple", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":308
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_set(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_set(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  unsigned PY_LONG_LONG __pyx_v_size;
  CYTHON_UNUSED unsigned PY_LONG_LONG __pyx_8genexpr1__pyx_v__;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  unsigned PY_LONG_LONG __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  unsigned PY_LONG_LONG __pyx_t_4;
  unsigned PY_LONG_LONG __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_set", 1);

  /* "serializor/unicode/des.py":315
 *     Position updates to the next charactor after deserialization.
 *     """
 *     pos[0] += 1  # skip identifier             # <<<<<<<<<<<<<<
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *     return {_des_data(data, pos) for _ in range(size)}
 */
  __pyx_t_1 = 0;
  (__pyx_v_pos[__pyx_t_1]) = ((__pyx_v_pos[__pyx_t_1]) + 1);

  /* "serializor/unicode/des.py":316
 *     """
 *     pos[0] += 1  # skip identifier
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *     return {_des_data(data, pos) for _ in range(size)}
 * 
 */
  __pyx_t_2 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_2 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 316, __pyx_L1_error)
  __pyx_v_size = __pyx_t_2;

  /* "serializor/unicode/des.py":317
 *     pos[0] += 1  # skip identifier
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *     return {_des_data(data, pos) for _ in range(size)}             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  { /* enter inner scope */
    __pyx_t_3 = PySet_New(NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 317, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __pyx_v_size;
    __pyx_t_4 = __pyx_t_2;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_8genexpr1__pyx_v__ = __pyx_t_5;
      __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__des_data(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 317, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (unlikely(PySet_Add(__pyx_t_3, (PyObject*)__pyx_t_6))) __PYX_ERR(1, 317, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
  } /* exit inner scope */
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":308
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_set(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("serializor.unicode.des._des_set", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":320
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_frozenset(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_frozenset(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  unsigned PY_LONG_LONG __pyx_v_size;
  CYTHON_UNUSED unsigned PY_LONG_LONG __pyx_8genexpr2__pyx_v__;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  unsigned PY_LONG_LONG __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  unsigned PY_LONG_LONG __pyx_t_4;
  unsigned PY_LONG_LONG __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_frozenset", 1);

  /* "serializor/unicode/des.py":327
 *     Position updates to the next charactor after deserialization.
 *     """
 *     pos[0] += 1  # skip identifier             # <<<<<<<<<<<<<<
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *     return frozenset([_des_data(data, pos) for _ in range(size)])
 */
  __pyx_t_1 = 0;
  (__pyx_v_pos[__pyx_t_1]) = ((__pyx_v_pos[__pyx_t_1]) + 1);

  /* "serializor/unicode/des.py":328
 *     """
 *     pos[0] += 1  # skip identifier
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *     return frozenset([_des_data(data, pos) for _ in range(size)])
 * 
 */
  __pyx_t_2 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_2 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 328, __pyx_L1_error)
  __pyx_v_size = __pyx_t_2;

  /* "serializor/unicode/des.py":329
 *     pos[0] += 1  # skip identifier
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *     return frozenset([_des_data(data, pos) for _ in range(size)])             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  { /* enter inner scope */
    __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 329, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __pyx_v_size;
    __pyx_t_4 = __pyx_t_2;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_8genexpr2__pyx_v__ = __pyx_t_5;
      __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__des_data(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 329, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_t_6))) __PYX_ERR(1, 329, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
  } /* exit inner scope */
  __pyx_t_6 = __Pyx_PyFrozenSet_New(__pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 329, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":320
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_frozenset(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("serializor.unicode.des._des_frozenset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":332
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_range(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_range(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_range", 1);

  /* "serializor/unicode/des.py":339
 *     Position updates to the next charactor after deserialization.
 *     """
 *     pos[0] += 1  # skip identifier             # <<<<<<<<<<<<<<
 *     return range(
 *         _des_int(data, pos),  # start
 */
  __pyx_t_1 = 0;
  (__pyx_v_pos[__pyx_t_1]) = ((__pyx_v_pos[__pyx_t_1]) + 1);

  /* "serializor/unicode/des.py":340
 *     """
 *     pos[0] += 1  # skip identifier
 *     return range(             # <<<<<<<<<<<<<<
 *         _des_int(data, pos),  # start
 *         _des_int(data, pos),  # stop
 */
  __Pyx_XDECREF(__pyx_r);

  /* "serializor/unicode/des.py":341
 *     pos[0] += 1  # skip identifier
 *     return range(
 *         _des_int(data, pos),  # start             # <<<<<<<<<<<<<<
 *         _des_int(data, pos),  # stop
 *         _des_int(data, pos),  # step
 */
  __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_int(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 341, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "serializor/unicode/des.py":342
 *     return range(
 *         _des_int(data, pos),  # start
 *         _des_int(data, pos),  # stop             # <<<<<<<<<<<<<<
 *         _des_int(data, pos),  # step
 *     )
 */
  __pyx_t_3 = __pyx_f_10serializor_7unicode_3des__des_int(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 342, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "serializor/unicode/des.py":343
 *         _des_int(data, pos),  # start
 *         _des_int(data, pos),  # stop
 *         _des_int(data, pos),  # step             # <<<<<<<<<<<<<<
 *     )
 * 
 */
  __pyx_t_4 = __pyx_f_10serializor_7unicode_3des__des_int(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 343, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "serializor/unicode/des.py":340
 *     """
 *     pos[0] += 1  # skip identifier
 *     return range(             # <<<<<<<<<<<<<<
 *         _des_int(data, pos),  # start
 *         _des_int(data, pos),  # stop
 */
  __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2)) __PYX_ERR(1, 340, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_3)) __PYX_ERR(1, 340, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_4);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_4)) __PYX_ERR(1, 340, __pyx_L1_error);
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_range, __pyx_t_5, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":332
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_range(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("serializor.unicode.des._des_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":348
 * 
 * # Mapping Types -------------------------------------------------------------------------------
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_dict(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_dict(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  unsigned PY_LONG_LONG __pyx_v_size;
  CYTHON_UNUSED unsigned PY_LONG_LONG __pyx_8genexpr3__pyx_v__;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  unsigned PY_LONG_LONG __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  unsigned PY_LONG_LONG __pyx_t_4;
  unsigned PY_LONG_LONG __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_dict", 1);

  /* "serializor/unicode/des.py":355
 *     Position updates to the next charactor after deserialization.
 *     """
 *     pos[0] += 1  # skip identifier             # <<<<<<<<<<<<<<
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *     return {_des_data(data, pos): _des_data(data, pos) for _ in range(size)}
 */
  __pyx_t_1 = 0;
  (__pyx_v_pos[__pyx_t_1]) = ((__pyx_v_pos[__pyx_t_1]) + 1);

  /* "serializor/unicode/des.py":356
 *     """
 *     pos[0] += 1  # skip identifier
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *     return {_des_data(data, pos): _des_data(data, pos) for _ in range(size)}
 * 
 */
  __pyx_t_2 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_2 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 356, __pyx_L1_error)
  __pyx_v_size = __pyx_t_2;

  /* "serializor/unicode/des.py":357
 *     pos[0] += 1  # skip identifier
 *     size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *     return {_des_data(data, pos): _des_data(data, pos) for _ in range(size)}             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  { /* enter inner scope */
    __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 357, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __pyx_v_size;
    __pyx_t_4 = __pyx_t_2;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_8genexpr3__pyx_v__ = __pyx_t_5;
      __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__des_data(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 357, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = __pyx_f_10serializor_7unicode_3des__des_data(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 357, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (unlikely(PyDict_SetItem(__pyx_t_3, (PyObject*)__pyx_t_6, (PyObject*)__pyx_t_7))) __PYX_ERR(1, 357, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
  } /* exit inner scope */
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":348
 * 
 * # Mapping Types -------------------------------------------------------------------------------
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_dict(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("serializor.unicode.des._des_dict", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":361
 * 
 * # NumPy Types ---------------------------------------------------------------------------------
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_datetime64(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_datetime64(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  Py_ssize_t __pyx_v_idx;
  unsigned char __pyx_v_unit;
  PY_LONG_LONG __pyx_v_val;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  unsigned char __pyx_t_1;
  PY_LONG_LONG __pyx_t_2;
  long __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  unsigned int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_datetime64", 1);

  /* "serializor/unicode/des.py":368
 *     Position updates to the next charactor after deserialization.
 *     """
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip identifier             # <<<<<<<<<<<<<<
 *     unit: cython.uchar = unpack_uint8(data, idx)  # type: ignore
 *     val: cython.longlong = unpack_int64(data, idx + 1)  # type: ignore
 */
  __pyx_v_idx = ((__pyx_v_pos[0]) + 1);

  /* "serializor/unicode/des.py":369
 *     """
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip identifier
 *     unit: cython.uchar = unpack_uint8(data, idx)  # type: ignore             # <<<<<<<<<<<<<<
 *     val: cython.longlong = unpack_int64(data, idx + 1)  # type: ignore
 *     pos[0] += 10  # set new position
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, __pyx_v_idx); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 369, __pyx_L1_error)
  __pyx_v_unit = __pyx_t_1;

  /* "serializor/unicode/des.py":370
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip identifier
 *     unit: cython.uchar = unpack_uint8(data, idx)  # type: ignore
 *     val: cython.longlong = unpack_int64(data, idx + 1)  # type: ignore             # <<<<<<<<<<<<<<
 *     pos[0] += 10  # set new position
 *     return typeref.DATETIME64(val, utils.map_nptime_unit_int2str(unit))
 */
  __pyx_t_2 = __pyx_f_10serializor_7unicode_3des_unpack_int64(__pyx_v_data, (__pyx_v_idx + 1)); if (unlikely(__pyx_t_2 == ((PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 370, __pyx_L1_error)
  __pyx_v_val = __pyx_t_2;

  /* "serializor/unicode/des.py":371
 *     unit: cython.uchar = unpack_uint8(data, idx)  # type: ignore
 *     val: cython.longlong = unpack_int64(data, idx + 1)  # type: ignore
 *     pos[0] += 10  # set new position             # <<<<<<<<<<<<<<
 *     return typeref.DATETIME64(val, utils.map_nptime_unit_int2str(unit))
 * 
 */
  __pyx_t_3 = 0;
  (__pyx_v_pos[__pyx_t_3]) = ((__pyx_v_pos[__pyx_t_3]) + 10);

  /* "serializor/unicode/des.py":372
 *     val: cython.longlong = unpack_int64(data, idx + 1)  # type: ignore
 *     pos[0] += 10  # set new position
 *     return typeref.DATETIME64(val, utils.map_nptime_unit_int2str(unit))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __Pyx_PyInt_From_PY_LONG_LONG(__pyx_v_val); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 372, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __pyx_f_10serializor_5utils_map_nptime_unit_int2str(__pyx_v_unit); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 372, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_10serializor_7typeref_DATETIME64);
  __pyx_t_7 = __pyx_v_10serializor_7typeref_DATETIME64; __pyx_t_8 = NULL;
  __pyx_t_9 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
      __pyx_t_9 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_8, __pyx_t_5, __pyx_t_6};
    __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+1-__pyx_t_9, 2+__pyx_t_9);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 372, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":361
 * 
 * # NumPy Types ---------------------------------------------------------------------------------
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_datetime64(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("serializor.unicode.des._des_datetime64", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":375
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_timedelta64(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_timedelta64(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  Py_ssize_t __pyx_v_idx;
  unsigned char __pyx_v_unit;
  PY_LONG_LONG __pyx_v_val;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  unsigned char __pyx_t_1;
  PY_LONG_LONG __pyx_t_2;
  long __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  unsigned int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_timedelta64", 1);

  /* "serializor/unicode/des.py":382
 *     Position updates to the next charactor after deserialization.
 *     """
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip identifier             # <<<<<<<<<<<<<<
 *     unit: cython.uchar = unpack_uint8(data, idx)  # type: ignore
 *     val: cython.longlong = unpack_int64(data, idx + 1)  # type: ignore
 */
  __pyx_v_idx = ((__pyx_v_pos[0]) + 1);

  /* "serializor/unicode/des.py":383
 *     """
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip identifier
 *     unit: cython.uchar = unpack_uint8(data, idx)  # type: ignore             # <<<<<<<<<<<<<<
 *     val: cython.longlong = unpack_int64(data, idx + 1)  # type: ignore
 *     pos[0] += 10  # set new position
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, __pyx_v_idx); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 383, __pyx_L1_error)
  __pyx_v_unit = __pyx_t_1;

  /* "serializor/unicode/des.py":384
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip identifier
 *     unit: cython.uchar = unpack_uint8(data, idx)  # type: ignore
 *     val: cython.longlong = unpack_int64(data, idx + 1)  # type: ignore             # <<<<<<<<<<<<<<
 *     pos[0] += 10  # set new position
 *     return typeref.TIMEDELTA64(val, utils.map_nptime_unit_int2str(unit))
 */
  __pyx_t_2 = __pyx_f_10serializor_7unicode_3des_unpack_int64(__pyx_v_data, (__pyx_v_idx + 1)); if (unlikely(__pyx_t_2 == ((PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 384, __pyx_L1_error)
  __pyx_v_val = __pyx_t_2;

  /* "serializor/unicode/des.py":385
 *     unit: cython.uchar = unpack_uint8(data, idx)  # type: ignore
 *     val: cython.longlong = unpack_int64(data, idx + 1)  # type: ignore
 *     pos[0] += 10  # set new position             # <<<<<<<<<<<<<<
 *     return typeref.TIMEDELTA64(val, utils.map_nptime_unit_int2str(unit))
 * 
 */
  __pyx_t_3 = 0;
  (__pyx_v_pos[__pyx_t_3]) = ((__pyx_v_pos[__pyx_t_3]) + 10);

  /* "serializor/unicode/des.py":386
 *     val: cython.longlong = unpack_int64(data, idx + 1)  # type: ignore
 *     pos[0] += 10  # set new position
 *     return typeref.TIMEDELTA64(val, utils.map_nptime_unit_int2str(unit))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __Pyx_PyInt_From_PY_LONG_LONG(__pyx_v_val); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 386, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __pyx_f_10serializor_5utils_map_nptime_unit_int2str(__pyx_v_unit); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 386, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_10serializor_7typeref_TIMEDELTA64);
  __pyx_t_7 = __pyx_v_10serializor_7typeref_TIMEDELTA64; __pyx_t_8 = NULL;
  __pyx_t_9 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
      __pyx_t_9 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_8, __pyx_t_5, __pyx_t_6};
    __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+1-__pyx_t_9, 2+__pyx_t_9);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 386, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":375
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_timedelta64(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("serializor.unicode.des._des_timedelta64", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":389
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_ndarray(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_ndarray(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  char __pyx_v_arr_dtype;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_ndarray", 1);

  /* "serializor/unicode/des.py":396
 *     Position updates to the next charactor after deserialization.
 *     """
 *     pos[0] += 1  # skip identifier             # <<<<<<<<<<<<<<
 *     # Get ndarray dtype
 *     arr_dtype: cython.char = read_char(data, pos[0])
 */
  __pyx_t_1 = 0;
  (__pyx_v_pos[__pyx_t_1]) = ((__pyx_v_pos[__pyx_t_1]) + 1);

  /* "serializor/unicode/des.py":398
 *     pos[0] += 1  # skip identifier
 *     # Get ndarray dtype
 *     arr_dtype: cython.char = read_char(data, pos[0])             # <<<<<<<<<<<<<<
 * 
 *     # Deserialize ndarray
 */
  __pyx_v_arr_dtype = PyUnicode_READ_CHAR(__pyx_v_data, (__pyx_v_pos[0]));

  /* "serializor/unicode/des.py":402
 *     # Deserialize ndarray
 *     # . ndarray[object]
 *     if arr_dtype == identifier.NDARRAY_OBJECT:             # <<<<<<<<<<<<<<
 *         return _des_ndarray_object(data, pos)
 *     # . ndarray[int]
 */
  __pyx_t_2 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_OBJECT);
  if (__pyx_t_2) {

    /* "serializor/unicode/des.py":403
 *     # . ndarray[object]
 *     if arr_dtype == identifier.NDARRAY_OBJECT:
 *         return _des_ndarray_object(data, pos)             # <<<<<<<<<<<<<<
 *     # . ndarray[int]
 *     if arr_dtype == identifier.NDARRAY_INT:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __pyx_f_10serializor_7unicode_3des__des_ndarray_object(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 403, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":402
 *     # Deserialize ndarray
 *     # . ndarray[object]
 *     if arr_dtype == identifier.NDARRAY_OBJECT:             # <<<<<<<<<<<<<<
 *         return _des_ndarray_object(data, pos)
 *     # . ndarray[int]
 */
  }

  /* "serializor/unicode/des.py":405
 *         return _des_ndarray_object(data, pos)
 *     # . ndarray[int]
 *     if arr_dtype == identifier.NDARRAY_INT:             # <<<<<<<<<<<<<<
 *         return _des_ndarray_numeric(data, pos)
 *     # . ndarray[uint]
 */
  __pyx_t_2 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_INT);
  if (__pyx_t_2) {

    /* "serializor/unicode/des.py":406
 *     # . ndarray[int]
 *     if arr_dtype == identifier.NDARRAY_INT:
 *         return _des_ndarray_numeric(data, pos)             # <<<<<<<<<<<<<<
 *     # . ndarray[uint]
 *     if arr_dtype == identifier.NDARRAY_UINT:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __pyx_f_10serializor_7unicode_3des__des_ndarray_numeric(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 406, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":405
 *         return _des_ndarray_object(data, pos)
 *     # . ndarray[int]
 *     if arr_dtype == identifier.NDARRAY_INT:             # <<<<<<<<<<<<<<
 *         return _des_ndarray_numeric(data, pos)
 *     # . ndarray[uint]
 */
  }

  /* "serializor/unicode/des.py":408
 *         return _des_ndarray_numeric(data, pos)
 *     # . ndarray[uint]
 *     if arr_dtype == identifier.NDARRAY_UINT:             # <<<<<<<<<<<<<<
 *         return _des_ndarray_numeric(data, pos)
 *     # . ndarray[float]
 */
  __pyx_t_2 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_UINT);
  if (__pyx_t_2) {

    /* "serializor/unicode/des.py":409
 *     # . ndarray[uint]
 *     if arr_dtype == identifier.NDARRAY_UINT:
 *         return _des_ndarray_numeric(data, pos)             # <<<<<<<<<<<<<<
 *     # . ndarray[float]
 *     if arr_dtype == identifier.NDARRAY_FLOAT:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __pyx_f_10serializor_7unicode_3des__des_ndarray_numeric(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 409, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":408
 *         return _des_ndarray_numeric(data, pos)
 *     # . ndarray[uint]
 *     if arr_dtype == identifier.NDARRAY_UINT:             # <<<<<<<<<<<<<<
 *         return _des_ndarray_numeric(data, pos)
 *     # . ndarray[float]
 */
  }

  /* "serializor/unicode/des.py":411
 *         return _des_ndarray_numeric(data, pos)
 *     # . ndarray[float]
 *     if arr_dtype == identifier.NDARRAY_FLOAT:             # <<<<<<<<<<<<<<
 *         return _des_ndarray_numeric(data, pos)
 *     # . ndarray[bool]
 */
  __pyx_t_2 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_FLOAT);
  if (__pyx_t_2) {

    /* "serializor/unicode/des.py":412
 *     # . ndarray[float]
 *     if arr_dtype == identifier.NDARRAY_FLOAT:
 *         return _des_ndarray_numeric(data, pos)             # <<<<<<<<<<<<<<
 *     # . ndarray[bool]
 *     if arr_dtype == identifier.NDARRAY_BOOL:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __pyx_f_10serializor_7unicode_3des__des_ndarray_numeric(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 412, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":411
 *         return _des_ndarray_numeric(data, pos)
 *     # . ndarray[float]
 *     if arr_dtype == identifier.NDARRAY_FLOAT:             # <<<<<<<<<<<<<<
 *         return _des_ndarray_numeric(data, pos)
 *     # . ndarray[bool]
 */
  }

  /* "serializor/unicode/des.py":414
 *         return _des_ndarray_numeric(data, pos)
 *     # . ndarray[bool]
 *     if arr_dtype == identifier.NDARRAY_BOOL:             # <<<<<<<<<<<<<<
 *         return _des_ndarray_numeric(data, pos)
 *     # . ndarray[datetime64]
 */
  __pyx_t_2 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_BOOL);
  if (__pyx_t_2) {

    /* "serializor/unicode/des.py":415
 *     # . ndarray[bool]
 *     if arr_dtype == identifier.NDARRAY_BOOL:
 *         return _des_ndarray_numeric(data, pos)             # <<<<<<<<<<<<<<
 *     # . ndarray[datetime64]
 *     if arr_dtype == identifier.NDARRAY_DT64:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __pyx_f_10serializor_7unicode_3des__des_ndarray_numeric(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 415, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":414
 *         return _des_ndarray_numeric(data, pos)
 *     # . ndarray[bool]
 *     if arr_dtype == identifier.NDARRAY_BOOL:             # <<<<<<<<<<<<<<
 *         return _des_ndarray_numeric(data, pos)
 *     # . ndarray[datetime64]
 */
  }

  /* "serializor/unicode/des.py":417
 *         return _des_ndarray_numeric(data, pos)
 *     # . ndarray[datetime64]
 *     if arr_dtype == identifier.NDARRAY_DT64:             # <<<<<<<<<<<<<<
 *         return _des_ndarray_nptime(data, pos, True)
 *     # . ndarray[timedelta64]
 */
  __pyx_t_2 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_DT64);
  if (__pyx_t_2) {

    /* "serializor/unicode/des.py":418
 *     # . ndarray[datetime64]
 *     if arr_dtype == identifier.NDARRAY_DT64:
 *         return _des_ndarray_nptime(data, pos, True)             # <<<<<<<<<<<<<<
 *     # . ndarray[timedelta64]
 *     if arr_dtype == identifier.NDARRAY_TD64:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __pyx_f_10serializor_7unicode_3des__des_ndarray_nptime(__pyx_v_data, __pyx_v_pos, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 418, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":417
 *         return _des_ndarray_numeric(data, pos)
 *     # . ndarray[datetime64]
 *     if arr_dtype == identifier.NDARRAY_DT64:             # <<<<<<<<<<<<<<
 *         return _des_ndarray_nptime(data, pos, True)
 *     # . ndarray[timedelta64]
 */
  }

  /* "serializor/unicode/des.py":420
 *         return _des_ndarray_nptime(data, pos, True)
 *     # . ndarray[timedelta64]
 *     if arr_dtype == identifier.NDARRAY_TD64:             # <<<<<<<<<<<<<<
 *         return _des_ndarray_nptime(data, pos, False)
 *     # . ndarray[complex]
 */
  __pyx_t_2 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_TD64);
  if (__pyx_t_2) {

    /* "serializor/unicode/des.py":421
 *     # . ndarray[timedelta64]
 *     if arr_dtype == identifier.NDARRAY_TD64:
 *         return _des_ndarray_nptime(data, pos, False)             # <<<<<<<<<<<<<<
 *     # . ndarray[complex]
 *     if arr_dtype == identifier.NDARRAY_COMPLEX:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __pyx_f_10serializor_7unicode_3des__des_ndarray_nptime(__pyx_v_data, __pyx_v_pos, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 421, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":420
 *         return _des_ndarray_nptime(data, pos, True)
 *     # . ndarray[timedelta64]
 *     if arr_dtype == identifier.NDARRAY_TD64:             # <<<<<<<<<<<<<<
 *         return _des_ndarray_nptime(data, pos, False)
 *     # . ndarray[complex]
 */
  }

  /* "serializor/unicode/des.py":423
 *         return _des_ndarray_nptime(data, pos, False)
 *     # . ndarray[complex]
 *     if arr_dtype == identifier.NDARRAY_COMPLEX:             # <<<<<<<<<<<<<<
 *         return _des_ndarray_complex(data, pos)
 *     # . ndarray[bytes]
 */
  __pyx_t_2 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_COMPLEX);
  if (__pyx_t_2) {

    /* "serializor/unicode/des.py":424
 *     # . ndarray[complex]
 *     if arr_dtype == identifier.NDARRAY_COMPLEX:
 *         return _des_ndarray_complex(data, pos)             # <<<<<<<<<<<<<<
 *     # . ndarray[bytes]
 *     if arr_dtype == identifier.NDARRAY_BYTES:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __pyx_f_10serializor_7unicode_3des__des_ndarray_complex(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 424, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":423
 *         return _des_ndarray_nptime(data, pos, False)
 *     # . ndarray[complex]
 *     if arr_dtype == identifier.NDARRAY_COMPLEX:             # <<<<<<<<<<<<<<
 *         return _des_ndarray_complex(data, pos)
 *     # . ndarray[bytes]
 */
  }

  /* "serializor/unicode/des.py":426
 *         return _des_ndarray_complex(data, pos)
 *     # . ndarray[bytes]
 *     if arr_dtype == identifier.NDARRAY_BYTES:             # <<<<<<<<<<<<<<
 *         return _des_ndarray_bytes(data, pos)
 *     # . ndarray[str]
 */
  __pyx_t_2 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_BYTES);
  if (__pyx_t_2) {

    /* "serializor/unicode/des.py":427
 *     # . ndarray[bytes]
 *     if arr_dtype == identifier.NDARRAY_BYTES:
 *         return _des_ndarray_bytes(data, pos)             # <<<<<<<<<<<<<<
 *     # . ndarray[str]
 *     if arr_dtype == identifier.NDARRAY_UNICODE:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __pyx_f_10serializor_7unicode_3des__des_ndarray_bytes(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 427, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":426
 *         return _des_ndarray_complex(data, pos)
 *     # . ndarray[bytes]
 *     if arr_dtype == identifier.NDARRAY_BYTES:             # <<<<<<<<<<<<<<
 *         return _des_ndarray_bytes(data, pos)
 *     # . ndarray[str]
 */
  }

  /* "serializor/unicode/des.py":429
 *         return _des_ndarray_bytes(data, pos)
 *     # . ndarray[str]
 *     if arr_dtype == identifier.NDARRAY_UNICODE:             # <<<<<<<<<<<<<<
 *         return _des_ndarray_unicode(data, pos)
 *     # . invalid dtype
 */
  __pyx_t_2 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_UNICODE);
  if (__pyx_t_2) {

    /* "serializor/unicode/des.py":430
 *     # . ndarray[str]
 *     if arr_dtype == identifier.NDARRAY_UNICODE:
 *         return _des_ndarray_unicode(data, pos)             # <<<<<<<<<<<<<<
 *     # . invalid dtype
 *     raise ValueError("unsupported <'numpy.ndarray'> dtype '%s'." % chr(arr_dtype))
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __pyx_f_10serializor_7unicode_3des__des_ndarray_unicode(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 430, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":429
 *         return _des_ndarray_bytes(data, pos)
 *     # . ndarray[str]
 *     if arr_dtype == identifier.NDARRAY_UNICODE:             # <<<<<<<<<<<<<<
 *         return _des_ndarray_unicode(data, pos)
 *     # . invalid dtype
 */
  }

  /* "serializor/unicode/des.py":432
 *         return _des_ndarray_unicode(data, pos)
 *     # . invalid dtype
 *     raise ValueError("unsupported <'numpy.ndarray'> dtype '%s'." % chr(arr_dtype))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = __Pyx_PyInt_From_char(__pyx_v_arr_dtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 432, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_chr, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 432, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyUnicode_FormatSafe(__pyx_kp_u_unsupported_numpy_ndarray_dtype, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 432, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 432, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_Raise(__pyx_t_4, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __PYX_ERR(1, 432, __pyx_L1_error)

  /* "serializor/unicode/des.py":389
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_ndarray(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("serializor.unicode.des._des_ndarray", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":435
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_ndarray_object(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_ndarray_object(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  Py_ssize_t __pyx_v_idx;
  unsigned char __pyx_v_npy_dtype;
  unsigned char __pyx_v_ndim;
  PyArrayObject *__pyx_v_arr = 0;
  Py_ssize_t __pyx_v_s_i;
  Py_ssize_t __pyx_v_i;
  Py_ssize_t __pyx_v_s_j;
  Py_ssize_t __pyx_v_j;
  Py_ssize_t __pyx_v_s_k;
  Py_ssize_t __pyx_v_k;
  Py_ssize_t __pyx_v_s_l;
  Py_ssize_t __pyx_v_l;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  unsigned char __pyx_t_1;
  long __pyx_t_2;
  int __pyx_t_3;
  unsigned PY_LONG_LONG __pyx_t_4;
  npy_intp __pyx_t_5[1];
  PyObject *__pyx_t_6 = NULL;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  npy_intp __pyx_t_10[2];
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  npy_intp __pyx_t_14[3];
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  npy_intp __pyx_t_18[4];
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  PyObject *__pyx_t_22 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_ndarray_object", 1);

  /* "serializor/unicode/des.py":444
 *     Position updates to the next charactor after deserialization.
 *     """
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip array dtype             # <<<<<<<<<<<<<<
 *     npy_dtype = unpack_uint8(data, idx)  # type: ignore
 *     ndim: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore
 */
  __pyx_v_idx = ((__pyx_v_pos[0]) + 1);

  /* "serializor/unicode/des.py":445
 *     """
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip array dtype
 *     npy_dtype = unpack_uint8(data, idx)  # type: ignore             # <<<<<<<<<<<<<<
 *     ndim: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore
 *     pos[0] += 3  # set new position
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, __pyx_v_idx); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 445, __pyx_L1_error)
  __pyx_v_npy_dtype = __pyx_t_1;

  /* "serializor/unicode/des.py":446
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip array dtype
 *     npy_dtype = unpack_uint8(data, idx)  # type: ignore
 *     ndim: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore             # <<<<<<<<<<<<<<
 *     pos[0] += 3  # set new position
 *     arr: np.ndarray
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_idx + 1)); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 446, __pyx_L1_error)
  __pyx_v_ndim = __pyx_t_1;

  /* "serializor/unicode/des.py":447
 *     npy_dtype = unpack_uint8(data, idx)  # type: ignore
 *     ndim: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore
 *     pos[0] += 3  # set new position             # <<<<<<<<<<<<<<
 *     arr: np.ndarray
 *     # 1-dimension
 */
  __pyx_t_2 = 0;
  (__pyx_v_pos[__pyx_t_2]) = ((__pyx_v_pos[__pyx_t_2]) + 3);

  /* "serializor/unicode/des.py":450
 *     arr: np.ndarray
 *     # 1-dimension
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 1);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":452
 *     if ndim == 1:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         arr = np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)
 *         if s_i == 0:
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 452, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":453
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)             # <<<<<<<<<<<<<<
 *         if s_i == 0:
 *             return arr  # exit: empty
 */
    __pyx_t_5[0] = __pyx_v_s_i;
    __pyx_t_6 = PyArray_EMPTY(__pyx_v_ndim, __pyx_t_5, __pyx_v_npy_dtype, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 453, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 453, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":454
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)
 *         if s_i == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    __pyx_t_3 = (__pyx_v_s_i == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":455
 *         arr = np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)
 *         if s_i == 0:
 *             return arr  # exit: empty             # <<<<<<<<<<<<<<
 *         # . deserialize values
 *         for i in range(s_i):
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF((PyObject *)__pyx_v_arr);
      __pyx_r = ((PyObject *)__pyx_v_arr);
      goto __pyx_L0;

      /* "serializor/unicode/des.py":454
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)
 *         if s_i == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    }

    /* "serializor/unicode/des.py":457
 *             return arr  # exit: empty
 *         # . deserialize values
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             utils.arr_setitem_1d(arr, _des_data(data, pos), i)
 *         return arr
 */
    __pyx_t_7 = __pyx_v_s_i;
    __pyx_t_8 = __pyx_t_7;
    for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
      __pyx_v_i = __pyx_t_9;

      /* "serializor/unicode/des.py":458
 *         # . deserialize values
 *         for i in range(s_i):
 *             utils.arr_setitem_1d(arr, _des_data(data, pos), i)             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
      __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__des_data(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 458, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_1d(__pyx_v_arr, __pyx_t_6, __pyx_v_i); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 458, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }

    /* "serializor/unicode/des.py":459
 *         for i in range(s_i):
 *             utils.arr_setitem_1d(arr, _des_data(data, pos), i)
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # 2-dimensions
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":450
 *     arr: np.ndarray
 *     # 1-dimension
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":462
 * 
 *     # 2-dimensions
 *     if ndim == 2:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 2);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":464
 *     if ndim == 2:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 464, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":465
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)
 *         if s_j == 0:
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 465, __pyx_L1_error)
    __pyx_v_s_j = __pyx_t_4;

    /* "serializor/unicode/des.py":466
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)             # <<<<<<<<<<<<<<
 *         if s_j == 0:
 *             return arr  # exit: empty
 */
    __pyx_t_10[0] = __pyx_v_s_i;
    __pyx_t_10[1] = __pyx_v_s_j;
    __pyx_t_6 = PyArray_EMPTY(__pyx_v_ndim, __pyx_t_10, __pyx_v_npy_dtype, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 466, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 466, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":467
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)
 *         if s_j == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    __pyx_t_3 = (__pyx_v_s_j == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":468
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)
 *         if s_j == 0:
 *             return arr  # exit: empty             # <<<<<<<<<<<<<<
 *         # . deserialize values
 *         for i in range(s_i):
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF((PyObject *)__pyx_v_arr);
      __pyx_r = ((PyObject *)__pyx_v_arr);
      goto __pyx_L0;

      /* "serializor/unicode/des.py":467
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)
 *         if s_j == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    }

    /* "serializor/unicode/des.py":470
 *             return arr  # exit: empty
 *         # . deserialize values
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             for j in range(s_j):
 *                 utils.arr_setitem_2d(arr, _des_data(data, pos), i, j)
 */
    __pyx_t_7 = __pyx_v_s_i;
    __pyx_t_8 = __pyx_t_7;
    for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
      __pyx_v_i = __pyx_t_9;

      /* "serializor/unicode/des.py":471
 *         # . deserialize values
 *         for i in range(s_i):
 *             for j in range(s_j):             # <<<<<<<<<<<<<<
 *                 utils.arr_setitem_2d(arr, _des_data(data, pos), i, j)
 *         return arr
 */
      __pyx_t_11 = __pyx_v_s_j;
      __pyx_t_12 = __pyx_t_11;
      for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
        __pyx_v_j = __pyx_t_13;

        /* "serializor/unicode/des.py":472
 *         for i in range(s_i):
 *             for j in range(s_j):
 *                 utils.arr_setitem_2d(arr, _des_data(data, pos), i, j)             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
        __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__des_data(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 472, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_2d(__pyx_v_arr, __pyx_t_6, __pyx_v_i, __pyx_v_j); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 472, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }

    /* "serializor/unicode/des.py":473
 *             for j in range(s_j):
 *                 utils.arr_setitem_2d(arr, _des_data(data, pos), i, j)
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # 3-dimensions
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":462
 * 
 *     # 2-dimensions
 *     if ndim == 2:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":476
 * 
 *     # 3-dimensions
 *     if ndim == 3:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 3);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":478
 *     if ndim == 3:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 478, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":479
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 479, __pyx_L1_error)
    __pyx_v_s_j = __pyx_t_4;

    /* "serializor/unicode/des.py":480
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)
 *         if s_k == 0:
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 480, __pyx_L1_error)
    __pyx_v_s_k = __pyx_t_4;

    /* "serializor/unicode/des.py":481
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)             # <<<<<<<<<<<<<<
 *         if s_k == 0:
 *             return arr  # exit: empty
 */
    __pyx_t_14[0] = __pyx_v_s_i;
    __pyx_t_14[1] = __pyx_v_s_j;
    __pyx_t_14[2] = __pyx_v_s_k;
    __pyx_t_6 = PyArray_EMPTY(__pyx_v_ndim, __pyx_t_14, __pyx_v_npy_dtype, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 481, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 481, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":482
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)
 *         if s_k == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    __pyx_t_3 = (__pyx_v_s_k == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":483
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)
 *         if s_k == 0:
 *             return arr  # exit: empty             # <<<<<<<<<<<<<<
 *         # . deserialize values
 *         for i in range(s_i):
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF((PyObject *)__pyx_v_arr);
      __pyx_r = ((PyObject *)__pyx_v_arr);
      goto __pyx_L0;

      /* "serializor/unicode/des.py":482
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)
 *         if s_k == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    }

    /* "serializor/unicode/des.py":485
 *             return arr  # exit: empty
 *         # . deserialize values
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             for j in range(s_j):
 *                 for k in range(s_k):
 */
    __pyx_t_7 = __pyx_v_s_i;
    __pyx_t_8 = __pyx_t_7;
    for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
      __pyx_v_i = __pyx_t_9;

      /* "serializor/unicode/des.py":486
 *         # . deserialize values
 *         for i in range(s_i):
 *             for j in range(s_j):             # <<<<<<<<<<<<<<
 *                 for k in range(s_k):
 *                     utils.arr_setitem_3d(arr, _des_data(data, pos), i, j, k)
 */
      __pyx_t_11 = __pyx_v_s_j;
      __pyx_t_12 = __pyx_t_11;
      for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
        __pyx_v_j = __pyx_t_13;

        /* "serializor/unicode/des.py":487
 *         for i in range(s_i):
 *             for j in range(s_j):
 *                 for k in range(s_k):             # <<<<<<<<<<<<<<
 *                     utils.arr_setitem_3d(arr, _des_data(data, pos), i, j, k)
 *         return arr
 */
        __pyx_t_15 = __pyx_v_s_k;
        __pyx_t_16 = __pyx_t_15;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "serializor/unicode/des.py":488
 *             for j in range(s_j):
 *                 for k in range(s_k):
 *                     utils.arr_setitem_3d(arr, _des_data(data, pos), i, j, k)             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
          __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__des_data(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 488, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_3d(__pyx_v_arr, __pyx_t_6, __pyx_v_i, __pyx_v_j, __pyx_v_k); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 488, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
      }
    }

    /* "serializor/unicode/des.py":489
 *                 for k in range(s_k):
 *                     utils.arr_setitem_3d(arr, _des_data(data, pos), i, j, k)
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # 4-dimensions
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":476
 * 
 *     # 3-dimensions
 *     if ndim == 3:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":492
 * 
 *     # 4-dimensions
 *     if ndim == 4:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 4);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":494
 *     if ndim == 4:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 494, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":495
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 495, __pyx_L1_error)
    __pyx_v_s_j = __pyx_t_4;

    /* "serializor/unicode/des.py":496
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0)
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 496, __pyx_L1_error)
    __pyx_v_s_k = __pyx_t_4;

    /* "serializor/unicode/des.py":497
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0)
 *         if s_l == 0:
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 497, __pyx_L1_error)
    __pyx_v_s_l = __pyx_t_4;

    /* "serializor/unicode/des.py":498
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0)             # <<<<<<<<<<<<<<
 *         if s_l == 0:
 *             return arr  # exit: empty
 */
    __pyx_t_18[0] = __pyx_v_s_i;
    __pyx_t_18[1] = __pyx_v_s_j;
    __pyx_t_18[2] = __pyx_v_s_k;
    __pyx_t_18[3] = __pyx_v_s_l;
    __pyx_t_6 = PyArray_EMPTY(__pyx_v_ndim, __pyx_t_18, __pyx_v_npy_dtype, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 498, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 498, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":499
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0)
 *         if s_l == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    __pyx_t_3 = (__pyx_v_s_l == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":500
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0)
 *         if s_l == 0:
 *             return arr  # exit: empty             # <<<<<<<<<<<<<<
 *         # . deserialize values
 *         for i in range(s_i):
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF((PyObject *)__pyx_v_arr);
      __pyx_r = ((PyObject *)__pyx_v_arr);
      goto __pyx_L0;

      /* "serializor/unicode/des.py":499
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0)
 *         if s_l == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    }

    /* "serializor/unicode/des.py":502
 *             return arr  # exit: empty
 *         # . deserialize values
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             for j in range(s_j):
 *                 for k in range(s_k):
 */
    __pyx_t_7 = __pyx_v_s_i;
    __pyx_t_8 = __pyx_t_7;
    for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
      __pyx_v_i = __pyx_t_9;

      /* "serializor/unicode/des.py":503
 *         # . deserialize values
 *         for i in range(s_i):
 *             for j in range(s_j):             # <<<<<<<<<<<<<<
 *                 for k in range(s_k):
 *                     for l in range(s_l):
 */
      __pyx_t_11 = __pyx_v_s_j;
      __pyx_t_12 = __pyx_t_11;
      for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
        __pyx_v_j = __pyx_t_13;

        /* "serializor/unicode/des.py":504
 *         for i in range(s_i):
 *             for j in range(s_j):
 *                 for k in range(s_k):             # <<<<<<<<<<<<<<
 *                     for l in range(s_l):
 *                         utils.arr_setitem_4d(arr, _des_data(data, pos), i, j, k, l)
 */
        __pyx_t_15 = __pyx_v_s_k;
        __pyx_t_16 = __pyx_t_15;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "serializor/unicode/des.py":505
 *             for j in range(s_j):
 *                 for k in range(s_k):
 *                     for l in range(s_l):             # <<<<<<<<<<<<<<
 *                         utils.arr_setitem_4d(arr, _des_data(data, pos), i, j, k, l)
 *         return arr
 */
          __pyx_t_19 = __pyx_v_s_l;
          __pyx_t_20 = __pyx_t_19;
          for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
            __pyx_v_l = __pyx_t_21;

            /* "serializor/unicode/des.py":506
 *                 for k in range(s_k):
 *                     for l in range(s_l):
 *                         utils.arr_setitem_4d(arr, _des_data(data, pos), i, j, k, l)             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
            __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__des_data(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 506, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_6);
            __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_4d(__pyx_v_arr, __pyx_t_6, __pyx_v_i, __pyx_v_j, __pyx_v_k, __pyx_v_l); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 506, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          }
        }
      }
    }

    /* "serializor/unicode/des.py":507
 *                     for l in range(s_l):
 *                         utils.arr_setitem_4d(arr, _des_data(data, pos), i, j, k, l)
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # Invalid
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":492
 * 
 *     # 4-dimensions
 *     if ndim == 4:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":510
 * 
 *     # Invalid
 *     raise ValueError("unsupported <'numpy.ndarray'> dimension [%d]." % ndim)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_6 = __Pyx_PyInt_From_unsigned_char(__pyx_v_ndim); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 510, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_22 = PyUnicode_Format(__pyx_kp_u_unsupported_numpy_ndarray_dimens, __pyx_t_6); if (unlikely(!__pyx_t_22)) __PYX_ERR(1, 510, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_22);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_22); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 510, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
  __Pyx_Raise(__pyx_t_6, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __PYX_ERR(1, 510, __pyx_L1_error)

  /* "serializor/unicode/des.py":435
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_ndarray_object(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_22);
  __Pyx_AddTraceback("serializor.unicode.des._des_ndarray_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_arr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":513
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_ndarray_numeric(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_ndarray_numeric(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  Py_ssize_t __pyx_v_idx;
  unsigned char __pyx_v_npy_dtype;
  unsigned char __pyx_v_ndim;
  PyArrayObject *__pyx_v_arr = 0;
  Py_ssize_t __pyx_v_s_i;
  unsigned PY_LONG_LONG __pyx_v_values_len;
  Py_ssize_t __pyx_v_end;
  PyObject *__pyx_v_values_json = 0;
  PyObject *__pyx_v_values = NULL;
  Py_ssize_t __pyx_v_i;
  Py_ssize_t __pyx_v_s_j;
  Py_ssize_t __pyx_v_j;
  Py_ssize_t __pyx_v_s_k;
  Py_ssize_t __pyx_v_k;
  Py_ssize_t __pyx_v_s_l;
  Py_ssize_t __pyx_v_l;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  unsigned char __pyx_t_1;
  long __pyx_t_2;
  int __pyx_t_3;
  unsigned PY_LONG_LONG __pyx_t_4;
  npy_intp __pyx_t_5[1];
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  npy_intp __pyx_t_11[2];
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  npy_intp __pyx_t_15[3];
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  npy_intp __pyx_t_19[4];
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_ndarray_numeric", 1);

  /* "serializor/unicode/des.py":522
 *     Position updates to the next charactor after deserialization.
 *     """
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip array dtype             # <<<<<<<<<<<<<<
 *     npy_dtype = unpack_uint8(data, idx)  # type: ignore
 *     ndim: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore
 */
  __pyx_v_idx = ((__pyx_v_pos[0]) + 1);

  /* "serializor/unicode/des.py":523
 *     """
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip array dtype
 *     npy_dtype = unpack_uint8(data, idx)  # type: ignore             # <<<<<<<<<<<<<<
 *     ndim: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore
 *     pos[0] += 3  # set new position
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, __pyx_v_idx); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 523, __pyx_L1_error)
  __pyx_v_npy_dtype = __pyx_t_1;

  /* "serializor/unicode/des.py":524
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip array dtype
 *     npy_dtype = unpack_uint8(data, idx)  # type: ignore
 *     ndim: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore             # <<<<<<<<<<<<<<
 *     pos[0] += 3  # set new position
 *     arr: np.ndarray
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_idx + 1)); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 524, __pyx_L1_error)
  __pyx_v_ndim = __pyx_t_1;

  /* "serializor/unicode/des.py":525
 *     npy_dtype = unpack_uint8(data, idx)  # type: ignore
 *     ndim: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore
 *     pos[0] += 3  # set new position             # <<<<<<<<<<<<<<
 *     arr: np.ndarray
 *     # 1-dimension
 */
  __pyx_t_2 = 0;
  (__pyx_v_pos[__pyx_t_2]) = ((__pyx_v_pos[__pyx_t_2]) + 3);

  /* "serializor/unicode/des.py":528
 *     arr: np.ndarray
 *     # 1-dimension
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 1);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":530
 *     if ndim == 1:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         arr = np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)
 *         if s_i == 0:
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 530, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":531
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)             # <<<<<<<<<<<<<<
 *         if s_i == 0:
 *             return arr  # exit: empty
 */
    __pyx_t_5[0] = __pyx_v_s_i;
    __pyx_t_6 = PyArray_EMPTY(__pyx_v_ndim, __pyx_t_5, __pyx_v_npy_dtype, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 531, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 531, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":532
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)
 *         if s_i == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    __pyx_t_3 = (__pyx_v_s_i == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":533
 *         arr = np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)
 *         if s_i == 0:
 *             return arr  # exit: empty             # <<<<<<<<<<<<<<
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF((PyObject *)__pyx_v_arr);
      __pyx_r = ((PyObject *)__pyx_v_arr);
      goto __pyx_L0;

      /* "serializor/unicode/des.py":532
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)
 *         if s_i == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    }

    /* "serializor/unicode/des.py":535
 *             return arr  # exit: empty
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 535, __pyx_L1_error)
    __pyx_v_values_len = __pyx_t_4;

    /* "serializor/unicode/des.py":536
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 */
    __pyx_v_idx = (__pyx_v_pos[0]);

    /* "serializor/unicode/des.py":537
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len             # <<<<<<<<<<<<<<
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 */
    __pyx_v_end = (__pyx_v_idx + __pyx_v_values_len);

    /* "serializor/unicode/des.py":538
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]             # <<<<<<<<<<<<<<
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 */
    __pyx_t_6 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 538, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_v_values_json = ((PyObject*)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":539
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 */
    (__pyx_v_pos[0]) = __pyx_v_end;

    /* "serializor/unicode/des.py":540
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))             # <<<<<<<<<<<<<<
 *         for i in range(s_i):
 *             utils.arr_setitem_1d(arr, next(values), i)
 */
    __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__orjson_loads(__pyx_v_values_json); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 540, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 540, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_values = __pyx_t_7;
    __pyx_t_7 = 0;

    /* "serializor/unicode/des.py":541
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             utils.arr_setitem_1d(arr, next(values), i)
 *         return arr
 */
    __pyx_t_8 = __pyx_v_s_i;
    __pyx_t_9 = __pyx_t_8;
    for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
      __pyx_v_i = __pyx_t_10;

      /* "serializor/unicode/des.py":542
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 *             utils.arr_setitem_1d(arr, next(values), i)             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
      __pyx_t_7 = __Pyx_PyIter_Next(__pyx_v_values); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 542, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_1d(__pyx_v_arr, __pyx_t_7, __pyx_v_i); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 542, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }

    /* "serializor/unicode/des.py":543
 *         for i in range(s_i):
 *             utils.arr_setitem_1d(arr, next(values), i)
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # 2-dimension
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":528
 *     arr: np.ndarray
 *     # 1-dimension
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":546
 * 
 *     # 2-dimension
 *     if ndim == 2:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 2);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":548
 *     if ndim == 2:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 548, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":549
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)
 *         if s_j == 0:
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 549, __pyx_L1_error)
    __pyx_v_s_j = __pyx_t_4;

    /* "serializor/unicode/des.py":550
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)             # <<<<<<<<<<<<<<
 *         if s_j == 0:
 *             return arr  # exit: empty
 */
    __pyx_t_11[0] = __pyx_v_s_i;
    __pyx_t_11[1] = __pyx_v_s_j;
    __pyx_t_7 = PyArray_EMPTY(__pyx_v_ndim, __pyx_t_11, __pyx_v_npy_dtype, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 550, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (!(likely(((__pyx_t_7) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_7, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 550, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_7);
    __pyx_t_7 = 0;

    /* "serializor/unicode/des.py":551
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)
 *         if s_j == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    __pyx_t_3 = (__pyx_v_s_j == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":552
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)
 *         if s_j == 0:
 *             return arr  # exit: empty             # <<<<<<<<<<<<<<
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF((PyObject *)__pyx_v_arr);
      __pyx_r = ((PyObject *)__pyx_v_arr);
      goto __pyx_L0;

      /* "serializor/unicode/des.py":551
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)
 *         if s_j == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    }

    /* "serializor/unicode/des.py":554
 *             return arr  # exit: empty
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 554, __pyx_L1_error)
    __pyx_v_values_len = __pyx_t_4;

    /* "serializor/unicode/des.py":555
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 */
    __pyx_v_idx = (__pyx_v_pos[0]);

    /* "serializor/unicode/des.py":556
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len             # <<<<<<<<<<<<<<
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 */
    __pyx_v_end = (__pyx_v_idx + __pyx_v_values_len);

    /* "serializor/unicode/des.py":557
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]             # <<<<<<<<<<<<<<
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 */
    __pyx_t_7 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 557, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_v_values_json = ((PyObject*)__pyx_t_7);
    __pyx_t_7 = 0;

    /* "serializor/unicode/des.py":558
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 */
    (__pyx_v_pos[0]) = __pyx_v_end;

    /* "serializor/unicode/des.py":559
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))             # <<<<<<<<<<<<<<
 *         for i in range(s_i):
 *             for j in range(s_j):
 */
    __pyx_t_7 = __pyx_f_10serializor_7unicode_3des__orjson_loads(__pyx_v_values_json); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 559, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 559, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_v_values = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":560
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             for j in range(s_j):
 *                 utils.arr_setitem_2d(arr, next(values), i, j)
 */
    __pyx_t_8 = __pyx_v_s_i;
    __pyx_t_9 = __pyx_t_8;
    for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
      __pyx_v_i = __pyx_t_10;

      /* "serializor/unicode/des.py":561
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 *             for j in range(s_j):             # <<<<<<<<<<<<<<
 *                 utils.arr_setitem_2d(arr, next(values), i, j)
 *         return arr
 */
      __pyx_t_12 = __pyx_v_s_j;
      __pyx_t_13 = __pyx_t_12;
      for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
        __pyx_v_j = __pyx_t_14;

        /* "serializor/unicode/des.py":562
 *         for i in range(s_i):
 *             for j in range(s_j):
 *                 utils.arr_setitem_2d(arr, next(values), i, j)             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
        __pyx_t_6 = __Pyx_PyIter_Next(__pyx_v_values); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 562, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_2d(__pyx_v_arr, __pyx_t_6, __pyx_v_i, __pyx_v_j); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 562, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }

    /* "serializor/unicode/des.py":563
 *             for j in range(s_j):
 *                 utils.arr_setitem_2d(arr, next(values), i, j)
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # 3-dimension
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":546
 * 
 *     # 2-dimension
 *     if ndim == 2:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":566
 * 
 *     # 3-dimension
 *     if ndim == 3:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 3);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":568
 *     if ndim == 3:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 568, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":569
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 569, __pyx_L1_error)
    __pyx_v_s_j = __pyx_t_4;

    /* "serializor/unicode/des.py":570
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)
 *         if s_k == 0:
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 570, __pyx_L1_error)
    __pyx_v_s_k = __pyx_t_4;

    /* "serializor/unicode/des.py":571
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)             # <<<<<<<<<<<<<<
 *         if s_k == 0:
 *             return arr  # exit: empty
 */
    __pyx_t_15[0] = __pyx_v_s_i;
    __pyx_t_15[1] = __pyx_v_s_j;
    __pyx_t_15[2] = __pyx_v_s_k;
    __pyx_t_6 = PyArray_EMPTY(__pyx_v_ndim, __pyx_t_15, __pyx_v_npy_dtype, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 571, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 571, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":572
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)
 *         if s_k == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    __pyx_t_3 = (__pyx_v_s_k == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":573
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)
 *         if s_k == 0:
 *             return arr  # exit: empty             # <<<<<<<<<<<<<<
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF((PyObject *)__pyx_v_arr);
      __pyx_r = ((PyObject *)__pyx_v_arr);
      goto __pyx_L0;

      /* "serializor/unicode/des.py":572
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)
 *         if s_k == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    }

    /* "serializor/unicode/des.py":575
 *             return arr  # exit: empty
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 575, __pyx_L1_error)
    __pyx_v_values_len = __pyx_t_4;

    /* "serializor/unicode/des.py":576
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 */
    __pyx_v_idx = (__pyx_v_pos[0]);

    /* "serializor/unicode/des.py":577
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len             # <<<<<<<<<<<<<<
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 */
    __pyx_v_end = (__pyx_v_idx + __pyx_v_values_len);

    /* "serializor/unicode/des.py":578
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]             # <<<<<<<<<<<<<<
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 */
    __pyx_t_6 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 578, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_v_values_json = ((PyObject*)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":579
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 */
    (__pyx_v_pos[0]) = __pyx_v_end;

    /* "serializor/unicode/des.py":580
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))             # <<<<<<<<<<<<<<
 *         for i in range(s_i):
 *             for j in range(s_j):
 */
    __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__orjson_loads(__pyx_v_values_json); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 580, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 580, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_values = __pyx_t_7;
    __pyx_t_7 = 0;

    /* "serializor/unicode/des.py":581
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             for j in range(s_j):
 *                 for k in range(s_k):
 */
    __pyx_t_8 = __pyx_v_s_i;
    __pyx_t_9 = __pyx_t_8;
    for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
      __pyx_v_i = __pyx_t_10;

      /* "serializor/unicode/des.py":582
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 *             for j in range(s_j):             # <<<<<<<<<<<<<<
 *                 for k in range(s_k):
 *                     utils.arr_setitem_3d(arr, next(values), i, j, k)
 */
      __pyx_t_12 = __pyx_v_s_j;
      __pyx_t_13 = __pyx_t_12;
      for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
        __pyx_v_j = __pyx_t_14;

        /* "serializor/unicode/des.py":583
 *         for i in range(s_i):
 *             for j in range(s_j):
 *                 for k in range(s_k):             # <<<<<<<<<<<<<<
 *                     utils.arr_setitem_3d(arr, next(values), i, j, k)
 *         return arr
 */
        __pyx_t_16 = __pyx_v_s_k;
        __pyx_t_17 = __pyx_t_16;
        for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
          __pyx_v_k = __pyx_t_18;

          /* "serializor/unicode/des.py":584
 *             for j in range(s_j):
 *                 for k in range(s_k):
 *                     utils.arr_setitem_3d(arr, next(values), i, j, k)             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
          __pyx_t_7 = __Pyx_PyIter_Next(__pyx_v_values); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 584, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_3d(__pyx_v_arr, __pyx_t_7, __pyx_v_i, __pyx_v_j, __pyx_v_k); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 584, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
      }
    }

    /* "serializor/unicode/des.py":585
 *                 for k in range(s_k):
 *                     utils.arr_setitem_3d(arr, next(values), i, j, k)
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # 4-dimension
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":566
 * 
 *     # 3-dimension
 *     if ndim == 3:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":588
 * 
 *     # 4-dimension
 *     if ndim == 4:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 4);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":590
 *     if ndim == 4:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 590, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":591
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 591, __pyx_L1_error)
    __pyx_v_s_j = __pyx_t_4;

    /* "serializor/unicode/des.py":592
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0)
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 592, __pyx_L1_error)
    __pyx_v_s_k = __pyx_t_4;

    /* "serializor/unicode/des.py":593
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0)
 *         if s_l == 0:
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 593, __pyx_L1_error)
    __pyx_v_s_l = __pyx_t_4;

    /* "serializor/unicode/des.py":594
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0)             # <<<<<<<<<<<<<<
 *         if s_l == 0:
 *             return arr
 */
    __pyx_t_19[0] = __pyx_v_s_i;
    __pyx_t_19[1] = __pyx_v_s_j;
    __pyx_t_19[2] = __pyx_v_s_k;
    __pyx_t_19[3] = __pyx_v_s_l;
    __pyx_t_7 = PyArray_EMPTY(__pyx_v_ndim, __pyx_t_19, __pyx_v_npy_dtype, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 594, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (!(likely(((__pyx_t_7) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_7, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 594, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_7);
    __pyx_t_7 = 0;

    /* "serializor/unicode/des.py":595
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0)
 *         if s_l == 0:             # <<<<<<<<<<<<<<
 *             return arr
 *         # . deserialize values
 */
    __pyx_t_3 = (__pyx_v_s_l == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":596
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0)
 *         if s_l == 0:
 *             return arr             # <<<<<<<<<<<<<<
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF((PyObject *)__pyx_v_arr);
      __pyx_r = ((PyObject *)__pyx_v_arr);
      goto __pyx_L0;

      /* "serializor/unicode/des.py":595
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0)
 *         if s_l == 0:             # <<<<<<<<<<<<<<
 *             return arr
 *         # . deserialize values
 */
    }

    /* "serializor/unicode/des.py":598
 *             return arr
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 598, __pyx_L1_error)
    __pyx_v_values_len = __pyx_t_4;

    /* "serializor/unicode/des.py":599
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 */
    __pyx_v_idx = (__pyx_v_pos[0]);

    /* "serializor/unicode/des.py":600
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len             # <<<<<<<<<<<<<<
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 */
    __pyx_v_end = (__pyx_v_idx + __pyx_v_values_len);

    /* "serializor/unicode/des.py":601
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]             # <<<<<<<<<<<<<<
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 */
    __pyx_t_7 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 601, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_v_values_json = ((PyObject*)__pyx_t_7);
    __pyx_t_7 = 0;

    /* "serializor/unicode/des.py":602
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 */
    (__pyx_v_pos[0]) = __pyx_v_end;

    /* "serializor/unicode/des.py":603
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))             # <<<<<<<<<<<<<<
 *         for i in range(s_i):
 *             for j in range(s_j):
 */
    __pyx_t_7 = __pyx_f_10serializor_7unicode_3des__orjson_loads(__pyx_v_values_json); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 603, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 603, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_v_values = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":604
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             for j in range(s_j):
 *                 for k in range(s_k):
 */
    __pyx_t_8 = __pyx_v_s_i;
    __pyx_t_9 = __pyx_t_8;
    for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
      __pyx_v_i = __pyx_t_10;

      /* "serializor/unicode/des.py":605
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 *             for j in range(s_j):             # <<<<<<<<<<<<<<
 *                 for k in range(s_k):
 *                     for l in range(s_l):
 */
      __pyx_t_12 = __pyx_v_s_j;
      __pyx_t_13 = __pyx_t_12;
      for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
        __pyx_v_j = __pyx_t_14;

        /* "serializor/unicode/des.py":606
 *         for i in range(s_i):
 *             for j in range(s_j):
 *                 for k in range(s_k):             # <<<<<<<<<<<<<<
 *                     for l in range(s_l):
 *                         utils.arr_setitem_4d(arr, next(values), i, j, k, l)
 */
        __pyx_t_16 = __pyx_v_s_k;
        __pyx_t_17 = __pyx_t_16;
        for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
          __pyx_v_k = __pyx_t_18;

          /* "serializor/unicode/des.py":607
 *             for j in range(s_j):
 *                 for k in range(s_k):
 *                     for l in range(s_l):             # <<<<<<<<<<<<<<
 *                         utils.arr_setitem_4d(arr, next(values), i, j, k, l)
 *         return arr
 */
          __pyx_t_20 = __pyx_v_s_l;
          __pyx_t_21 = __pyx_t_20;
          for (__pyx_t_22 = 0; __pyx_t_22 < __pyx_t_21; __pyx_t_22+=1) {
            __pyx_v_l = __pyx_t_22;

            /* "serializor/unicode/des.py":608
 *                 for k in range(s_k):
 *                     for l in range(s_l):
 *                         utils.arr_setitem_4d(arr, next(values), i, j, k, l)             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
            __pyx_t_6 = __Pyx_PyIter_Next(__pyx_v_values); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 608, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_6);
            __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_4d(__pyx_v_arr, __pyx_t_6, __pyx_v_i, __pyx_v_j, __pyx_v_k, __pyx_v_l); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 608, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          }
        }
      }
    }

    /* "serializor/unicode/des.py":609
 *                     for l in range(s_l):
 *                         utils.arr_setitem_4d(arr, next(values), i, j, k, l)
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # Invalid
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":588
 * 
 *     # 4-dimension
 *     if ndim == 4:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":612
 * 
 *     # Invalid
 *     raise ValueError("unsupported <'numpy.ndarray'> dimension [%d]." % ndim)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_6 = __Pyx_PyInt_From_unsigned_char(__pyx_v_ndim); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 612, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = PyUnicode_Format(__pyx_kp_u_unsupported_numpy_ndarray_dimens, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 612, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 612, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_Raise(__pyx_t_6, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __PYX_ERR(1, 612, __pyx_L1_error)

  /* "serializor/unicode/des.py":513
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_ndarray_numeric(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("serializor.unicode.des._des_ndarray_numeric", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_arr);
  __Pyx_XDECREF(__pyx_v_values_json);
  __Pyx_XDECREF(__pyx_v_values);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":615
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_ndarray_nptime(
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_ndarray_nptime(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos, int __pyx_v_is_dt64) {
  unsigned char __pyx_v_ndim;
  PyArrayObject *__pyx_v_arr = 0;
  Py_ssize_t __pyx_v_s_i;
  PyObject *__pyx_v_unit = 0;
  unsigned PY_LONG_LONG __pyx_v_values_len;
  Py_ssize_t __pyx_v_idx;
  Py_ssize_t __pyx_v_end;
  PyObject *__pyx_v_values_json = 0;
  PyObject *__pyx_v_values = NULL;
  Py_ssize_t __pyx_v_i;
  Py_ssize_t __pyx_v_s_j;
  Py_ssize_t __pyx_v_j;
  Py_ssize_t __pyx_v_s_k;
  Py_ssize_t __pyx_v_k;
  Py_ssize_t __pyx_v_s_l;
  Py_ssize_t __pyx_v_l;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  unsigned char __pyx_t_1;
  long __pyx_t_2;
  int __pyx_t_3;
  unsigned PY_LONG_LONG __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  unsigned int __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  PyObject *__pyx_t_17 = NULL;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  PyObject *__pyx_t_21 = NULL;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_ndarray_nptime", 1);

  /* "serializor/unicode/des.py":628
 *     Position updates to the next charactor after deserialization.
 *     """
 *     ndim: cython.uchar = unpack_uint8(data, pos[0] + 2)  # type: ignore             # <<<<<<<<<<<<<<
 *     pos[0] += 3  # set new position
 *     arr: np.ndarray
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, ((__pyx_v_pos[0]) + 2)); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 628, __pyx_L1_error)
  __pyx_v_ndim = __pyx_t_1;

  /* "serializor/unicode/des.py":629
 *     """
 *     ndim: cython.uchar = unpack_uint8(data, pos[0] + 2)  # type: ignore
 *     pos[0] += 3  # set new position             # <<<<<<<<<<<<<<
 *     arr: np.ndarray
 *     # 1-dimension
 */
  __pyx_t_2 = 0;
  (__pyx_v_pos[__pyx_t_2]) = ((__pyx_v_pos[__pyx_t_2]) + 3);

  /* "serializor/unicode/des.py":632
 *     arr: np.ndarray
 *     # 1-dimension
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 1);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":634
 *     if ndim == 1:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         unit: str = utils.map_nptime_unit_int2str(unpack_uint8(data, pos[0]))  # type: ignore
 *         pos[0] += 1  # skip unit
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 634, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":635
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         unit: str = utils.map_nptime_unit_int2str(unpack_uint8(data, pos[0]))  # type: ignore             # <<<<<<<<<<<<<<
 *         pos[0] += 1  # skip unit
 *         arr = np.empty(
 */
    __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_pos[0])); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 635, __pyx_L1_error)
    __pyx_t_5 = __pyx_f_10serializor_5utils_map_nptime_unit_int2str(__pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 635, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_v_unit = ((PyObject*)__pyx_t_5);
    __pyx_t_5 = 0;

    /* "serializor/unicode/des.py":636
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         unit: str = utils.map_nptime_unit_int2str(unpack_uint8(data, pos[0]))  # type: ignore
 *         pos[0] += 1  # skip unit             # <<<<<<<<<<<<<<
 *         arr = np.empty(
 *             s_i,
 */
    __pyx_t_2 = 0;
    (__pyx_v_pos[__pyx_t_2]) = ((__pyx_v_pos[__pyx_t_2]) + 1);

    /* "serializor/unicode/des.py":637
 *         unit: str = utils.map_nptime_unit_int2str(unpack_uint8(data, pos[0]))  # type: ignore
 *         pos[0] += 1  # skip unit
 *         arr = np.empty(             # <<<<<<<<<<<<<<
 *             s_i,
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,
 */
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_np); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 637, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_empty); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 637, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":638
 *         pos[0] += 1  # skip unit
 *         arr = np.empty(
 *             s_i,             # <<<<<<<<<<<<<<
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,
 *         )
 */
    __pyx_t_6 = PyInt_FromSsize_t(__pyx_v_s_i); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 638, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);

    /* "serializor/unicode/des.py":639
 *         arr = np.empty(
 *             s_i,
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,             # <<<<<<<<<<<<<<
 *         )
 *         if s_i == 0:
 */
    if (__pyx_v_is_dt64) {
      __pyx_t_9 = PyUnicode_Format(__pyx_kp_u_datetime64_s, __pyx_v_unit); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 639, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_8 = __pyx_t_9;
      __pyx_t_9 = 0;
    } else {
      __pyx_t_9 = PyUnicode_Format(__pyx_kp_u_timedelta64_s, __pyx_v_unit); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 639, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_8 = __pyx_t_9;
      __pyx_t_9 = 0;
    }
    __pyx_t_9 = NULL;
    __pyx_t_10 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
        __pyx_t_10 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_9, __pyx_t_6, __pyx_t_8};
      __pyx_t_5 = __Pyx_PyObject_FastCall(__pyx_t_7, __pyx_callargs+1-__pyx_t_10, 2+__pyx_t_10);
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 637, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }

    /* "serializor/unicode/des.py":637
 *         unit: str = utils.map_nptime_unit_int2str(unpack_uint8(data, pos[0]))  # type: ignore
 *         pos[0] += 1  # skip unit
 *         arr = np.empty(             # <<<<<<<<<<<<<<
 *             s_i,
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,
 */
    if (!(likely(((__pyx_t_5) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_5, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 637, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_5);
    __pyx_t_5 = 0;

    /* "serializor/unicode/des.py":641
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,
 *         )
 *         if s_i == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    __pyx_t_3 = (__pyx_v_s_i == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":642
 *         )
 *         if s_i == 0:
 *             return arr  # exit: empty             # <<<<<<<<<<<<<<
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF((PyObject *)__pyx_v_arr);
      __pyx_r = ((PyObject *)__pyx_v_arr);
      goto __pyx_L0;

      /* "serializor/unicode/des.py":641
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,
 *         )
 *         if s_i == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    }

    /* "serializor/unicode/des.py":644
 *             return arr  # exit: empty
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 644, __pyx_L1_error)
    __pyx_v_values_len = __pyx_t_4;

    /* "serializor/unicode/des.py":645
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 */
    __pyx_v_idx = (__pyx_v_pos[0]);

    /* "serializor/unicode/des.py":646
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len             # <<<<<<<<<<<<<<
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 */
    __pyx_v_end = (__pyx_v_idx + __pyx_v_values_len);

    /* "serializor/unicode/des.py":647
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]             # <<<<<<<<<<<<<<
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 */
    __pyx_t_5 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 647, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_v_values_json = ((PyObject*)__pyx_t_5);
    __pyx_t_5 = 0;

    /* "serializor/unicode/des.py":648
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 */
    (__pyx_v_pos[0]) = __pyx_v_end;

    /* "serializor/unicode/des.py":649
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))             # <<<<<<<<<<<<<<
 *         for i in range(s_i):
 *             utils.arr_setitem_1d(arr, next(values), i)
 */
    __pyx_t_5 = __pyx_f_10serializor_7unicode_3des__orjson_loads(__pyx_v_values_json); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 649, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_7 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 649, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_values = __pyx_t_7;
    __pyx_t_7 = 0;

    /* "serializor/unicode/des.py":650
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             utils.arr_setitem_1d(arr, next(values), i)
 *         return arr
 */
    __pyx_t_11 = __pyx_v_s_i;
    __pyx_t_12 = __pyx_t_11;
    for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
      __pyx_v_i = __pyx_t_13;

      /* "serializor/unicode/des.py":651
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 *             utils.arr_setitem_1d(arr, next(values), i)             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
      __pyx_t_7 = __Pyx_PyIter_Next(__pyx_v_values); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 651, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_1d(__pyx_v_arr, __pyx_t_7, __pyx_v_i); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 651, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }

    /* "serializor/unicode/des.py":652
 *         for i in range(s_i):
 *             utils.arr_setitem_1d(arr, next(values), i)
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # 2-dimension
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":632
 *     arr: np.ndarray
 *     # 1-dimension
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":655
 * 
 *     # 2-dimension
 *     if ndim == 2:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 2);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":657
 *     if ndim == 2:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         unit: str = utils.map_nptime_unit_int2str(unpack_uint8(data, pos[0]))  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 657, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":658
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         unit: str = utils.map_nptime_unit_int2str(unpack_uint8(data, pos[0]))  # type: ignore
 *         pos[0] += 1  # skip unit
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 658, __pyx_L1_error)
    __pyx_v_s_j = __pyx_t_4;

    /* "serializor/unicode/des.py":659
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         unit: str = utils.map_nptime_unit_int2str(unpack_uint8(data, pos[0]))  # type: ignore             # <<<<<<<<<<<<<<
 *         pos[0] += 1  # skip unit
 *         arr = np.empty(
 */
    __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_pos[0])); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 659, __pyx_L1_error)
    __pyx_t_7 = __pyx_f_10serializor_5utils_map_nptime_unit_int2str(__pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 659, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_v_unit = ((PyObject*)__pyx_t_7);
    __pyx_t_7 = 0;

    /* "serializor/unicode/des.py":660
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         unit: str = utils.map_nptime_unit_int2str(unpack_uint8(data, pos[0]))  # type: ignore
 *         pos[0] += 1  # skip unit             # <<<<<<<<<<<<<<
 *         arr = np.empty(
 *             [s_i, s_j],
 */
    __pyx_t_2 = 0;
    (__pyx_v_pos[__pyx_t_2]) = ((__pyx_v_pos[__pyx_t_2]) + 1);

    /* "serializor/unicode/des.py":661
 *         unit: str = utils.map_nptime_unit_int2str(unpack_uint8(data, pos[0]))  # type: ignore
 *         pos[0] += 1  # skip unit
 *         arr = np.empty(             # <<<<<<<<<<<<<<
 *             [s_i, s_j],
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,
 */
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 661, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_empty); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 661, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "serializor/unicode/des.py":662
 *         pos[0] += 1  # skip unit
 *         arr = np.empty(
 *             [s_i, s_j],             # <<<<<<<<<<<<<<
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,
 *         )
 */
    __pyx_t_5 = PyInt_FromSsize_t(__pyx_v_s_i); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 662, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyInt_FromSsize_t(__pyx_v_s_j); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 662, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_9 = PyList_New(2); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 662, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 0, __pyx_t_5)) __PYX_ERR(1, 662, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_6);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 1, __pyx_t_6)) __PYX_ERR(1, 662, __pyx_L1_error);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":663
 *         arr = np.empty(
 *             [s_i, s_j],
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,             # <<<<<<<<<<<<<<
 *         )
 *         if s_j == 0:
 */
    if (__pyx_v_is_dt64) {
      __pyx_t_5 = PyUnicode_Format(__pyx_kp_u_datetime64_s, __pyx_v_unit); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 663, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = __pyx_t_5;
      __pyx_t_5 = 0;
    } else {
      __pyx_t_5 = PyUnicode_Format(__pyx_kp_u_timedelta64_s, __pyx_v_unit); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 663, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = __pyx_t_5;
      __pyx_t_5 = 0;
    }
    __pyx_t_5 = NULL;
    __pyx_t_10 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
        __pyx_t_10 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_5, __pyx_t_9, __pyx_t_6};
      __pyx_t_7 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+1-__pyx_t_10, 2+__pyx_t_10);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 661, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }

    /* "serializor/unicode/des.py":661
 *         unit: str = utils.map_nptime_unit_int2str(unpack_uint8(data, pos[0]))  # type: ignore
 *         pos[0] += 1  # skip unit
 *         arr = np.empty(             # <<<<<<<<<<<<<<
 *             [s_i, s_j],
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,
 */
    if (!(likely(((__pyx_t_7) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_7, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 661, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_7);
    __pyx_t_7 = 0;

    /* "serializor/unicode/des.py":665
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,
 *         )
 *         if s_j == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    __pyx_t_3 = (__pyx_v_s_j == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":666
 *         )
 *         if s_j == 0:
 *             return arr  # exit: empty             # <<<<<<<<<<<<<<
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF((PyObject *)__pyx_v_arr);
      __pyx_r = ((PyObject *)__pyx_v_arr);
      goto __pyx_L0;

      /* "serializor/unicode/des.py":665
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,
 *         )
 *         if s_j == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    }

    /* "serializor/unicode/des.py":668
 *             return arr  # exit: empty
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 668, __pyx_L1_error)
    __pyx_v_values_len = __pyx_t_4;

    /* "serializor/unicode/des.py":669
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 */
    __pyx_v_idx = (__pyx_v_pos[0]);

    /* "serializor/unicode/des.py":670
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len             # <<<<<<<<<<<<<<
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 */
    __pyx_v_end = (__pyx_v_idx + __pyx_v_values_len);

    /* "serializor/unicode/des.py":671
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]             # <<<<<<<<<<<<<<
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 */
    __pyx_t_7 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 671, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_v_values_json = ((PyObject*)__pyx_t_7);
    __pyx_t_7 = 0;

    /* "serializor/unicode/des.py":672
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 */
    (__pyx_v_pos[0]) = __pyx_v_end;

    /* "serializor/unicode/des.py":673
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))             # <<<<<<<<<<<<<<
 *         for i in range(s_i):
 *             for j in range(s_j):
 */
    __pyx_t_7 = __pyx_f_10serializor_7unicode_3des__orjson_loads(__pyx_v_values_json); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 673, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 673, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_v_values = __pyx_t_8;
    __pyx_t_8 = 0;

    /* "serializor/unicode/des.py":674
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             for j in range(s_j):
 *                 utils.arr_setitem_2d(arr, next(values), i, j)
 */
    __pyx_t_11 = __pyx_v_s_i;
    __pyx_t_12 = __pyx_t_11;
    for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
      __pyx_v_i = __pyx_t_13;

      /* "serializor/unicode/des.py":675
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 *             for j in range(s_j):             # <<<<<<<<<<<<<<
 *                 utils.arr_setitem_2d(arr, next(values), i, j)
 *         return arr
 */
      __pyx_t_14 = __pyx_v_s_j;
      __pyx_t_15 = __pyx_t_14;
      for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
        __pyx_v_j = __pyx_t_16;

        /* "serializor/unicode/des.py":676
 *         for i in range(s_i):
 *             for j in range(s_j):
 *                 utils.arr_setitem_2d(arr, next(values), i, j)             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
        __pyx_t_8 = __Pyx_PyIter_Next(__pyx_v_values); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 676, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_2d(__pyx_v_arr, __pyx_t_8, __pyx_v_i, __pyx_v_j); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 676, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      }
    }

    /* "serializor/unicode/des.py":677
 *             for j in range(s_j):
 *                 utils.arr_setitem_2d(arr, next(values), i, j)
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # 3-dimension
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":655
 * 
 *     # 2-dimension
 *     if ndim == 2:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":680
 * 
 *     # 3-dimension
 *     if ndim == 3:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 3);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":682
 *     if ndim == 3:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 682, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":683
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         unit: str = utils.map_nptime_unit_int2str(unpack_uint8(data, pos[0]))  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 683, __pyx_L1_error)
    __pyx_v_s_j = __pyx_t_4;

    /* "serializor/unicode/des.py":684
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         unit: str = utils.map_nptime_unit_int2str(unpack_uint8(data, pos[0]))  # type: ignore
 *         pos[0] += 1  # skip unit
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 684, __pyx_L1_error)
    __pyx_v_s_k = __pyx_t_4;

    /* "serializor/unicode/des.py":685
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         unit: str = utils.map_nptime_unit_int2str(unpack_uint8(data, pos[0]))  # type: ignore             # <<<<<<<<<<<<<<
 *         pos[0] += 1  # skip unit
 *         arr = np.empty(
 */
    __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_pos[0])); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 685, __pyx_L1_error)
    __pyx_t_8 = __pyx_f_10serializor_5utils_map_nptime_unit_int2str(__pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 685, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_v_unit = ((PyObject*)__pyx_t_8);
    __pyx_t_8 = 0;

    /* "serializor/unicode/des.py":686
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         unit: str = utils.map_nptime_unit_int2str(unpack_uint8(data, pos[0]))  # type: ignore
 *         pos[0] += 1  # skip unit             # <<<<<<<<<<<<<<
 *         arr = np.empty(
 *             [s_i, s_j, s_k],
 */
    __pyx_t_2 = 0;
    (__pyx_v_pos[__pyx_t_2]) = ((__pyx_v_pos[__pyx_t_2]) + 1);

    /* "serializor/unicode/des.py":687
 *         unit: str = utils.map_nptime_unit_int2str(unpack_uint8(data, pos[0]))  # type: ignore
 *         pos[0] += 1  # skip unit
 *         arr = np.empty(             # <<<<<<<<<<<<<<
 *             [s_i, s_j, s_k],
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,
 */
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_np); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 687, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_empty); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 687, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "serializor/unicode/des.py":688
 *         pos[0] += 1  # skip unit
 *         arr = np.empty(
 *             [s_i, s_j, s_k],             # <<<<<<<<<<<<<<
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,
 *         )
 */
    __pyx_t_7 = PyInt_FromSsize_t(__pyx_v_s_i); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 688, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_v_s_j); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 688, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_5 = PyInt_FromSsize_t(__pyx_v_s_k); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 688, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_17 = PyList_New(3); if (unlikely(!__pyx_t_17)) __PYX_ERR(1, 688, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_17);
    __Pyx_GIVEREF(__pyx_t_7);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 0, __pyx_t_7)) __PYX_ERR(1, 688, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_9);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 1, __pyx_t_9)) __PYX_ERR(1, 688, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_17, 2, __pyx_t_5)) __PYX_ERR(1, 688, __pyx_L1_error);
    __pyx_t_7 = 0;
    __pyx_t_9 = 0;
    __pyx_t_5 = 0;

    /* "serializor/unicode/des.py":689
 *         arr = np.empty(
 *             [s_i, s_j, s_k],
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,             # <<<<<<<<<<<<<<
 *         )
 *         if s_k == 0:
 */
    if (__pyx_v_is_dt64) {
      __pyx_t_9 = PyUnicode_Format(__pyx_kp_u_datetime64_s, __pyx_v_unit); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 689, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_5 = __pyx_t_9;
      __pyx_t_9 = 0;
    } else {
      __pyx_t_9 = PyUnicode_Format(__pyx_kp_u_timedelta64_s, __pyx_v_unit); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 689, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_5 = __pyx_t_9;
      __pyx_t_9 = 0;
    }
    __pyx_t_9 = NULL;
    __pyx_t_10 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
        __pyx_t_10 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_9, __pyx_t_17, __pyx_t_5};
      __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_6, __pyx_callargs+1-__pyx_t_10, 2+__pyx_t_10);
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 687, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }

    /* "serializor/unicode/des.py":687
 *         unit: str = utils.map_nptime_unit_int2str(unpack_uint8(data, pos[0]))  # type: ignore
 *         pos[0] += 1  # skip unit
 *         arr = np.empty(             # <<<<<<<<<<<<<<
 *             [s_i, s_j, s_k],
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,
 */
    if (!(likely(((__pyx_t_8) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_8, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 687, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_8);
    __pyx_t_8 = 0;

    /* "serializor/unicode/des.py":691
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,
 *         )
 *         if s_k == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    __pyx_t_3 = (__pyx_v_s_k == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":692
 *         )
 *         if s_k == 0:
 *             return arr  # exit: empty             # <<<<<<<<<<<<<<
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF((PyObject *)__pyx_v_arr);
      __pyx_r = ((PyObject *)__pyx_v_arr);
      goto __pyx_L0;

      /* "serializor/unicode/des.py":691
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,
 *         )
 *         if s_k == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    }

    /* "serializor/unicode/des.py":694
 *             return arr  # exit: empty
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 694, __pyx_L1_error)
    __pyx_v_values_len = __pyx_t_4;

    /* "serializor/unicode/des.py":695
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 */
    __pyx_v_idx = (__pyx_v_pos[0]);

    /* "serializor/unicode/des.py":696
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len             # <<<<<<<<<<<<<<
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 */
    __pyx_v_end = (__pyx_v_idx + __pyx_v_values_len);

    /* "serializor/unicode/des.py":697
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]             # <<<<<<<<<<<<<<
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 */
    __pyx_t_8 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 697, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_v_values_json = ((PyObject*)__pyx_t_8);
    __pyx_t_8 = 0;

    /* "serializor/unicode/des.py":698
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 */
    (__pyx_v_pos[0]) = __pyx_v_end;

    /* "serializor/unicode/des.py":699
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))             # <<<<<<<<<<<<<<
 *         for i in range(s_i):
 *             for j in range(s_j):
 */
    __pyx_t_8 = __pyx_f_10serializor_7unicode_3des__orjson_loads(__pyx_v_values_json); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 699, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_6 = PyObject_GetIter(__pyx_t_8); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 699, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_v_values = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":700
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             for j in range(s_j):
 *                 for k in range(s_k):
 */
    __pyx_t_11 = __pyx_v_s_i;
    __pyx_t_12 = __pyx_t_11;
    for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
      __pyx_v_i = __pyx_t_13;

      /* "serializor/unicode/des.py":701
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 *             for j in range(s_j):             # <<<<<<<<<<<<<<
 *                 for k in range(s_k):
 *                     utils.arr_setitem_3d(arr, next(values), i, j, k)
 */
      __pyx_t_14 = __pyx_v_s_j;
      __pyx_t_15 = __pyx_t_14;
      for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
        __pyx_v_j = __pyx_t_16;

        /* "serializor/unicode/des.py":702
 *         for i in range(s_i):
 *             for j in range(s_j):
 *                 for k in range(s_k):             # <<<<<<<<<<<<<<
 *                     utils.arr_setitem_3d(arr, next(values), i, j, k)
 *         return arr
 */
        __pyx_t_18 = __pyx_v_s_k;
        __pyx_t_19 = __pyx_t_18;
        for (__pyx_t_20 = 0; __pyx_t_20 < __pyx_t_19; __pyx_t_20+=1) {
          __pyx_v_k = __pyx_t_20;

          /* "serializor/unicode/des.py":703
 *             for j in range(s_j):
 *                 for k in range(s_k):
 *                     utils.arr_setitem_3d(arr, next(values), i, j, k)             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
          __pyx_t_6 = __Pyx_PyIter_Next(__pyx_v_values); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 703, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_3d(__pyx_v_arr, __pyx_t_6, __pyx_v_i, __pyx_v_j, __pyx_v_k); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 703, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
      }
    }

    /* "serializor/unicode/des.py":704
 *                 for k in range(s_k):
 *                     utils.arr_setitem_3d(arr, next(values), i, j, k)
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # 4-dimension
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":680
 * 
 *     # 3-dimension
 *     if ndim == 3:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":707
 * 
 *     # 4-dimension
 *     if ndim == 4:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 4);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":709
 *     if ndim == 4:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 709, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":710
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 710, __pyx_L1_error)
    __pyx_v_s_j = __pyx_t_4;

    /* "serializor/unicode/des.py":711
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         unit: str = utils.map_nptime_unit_int2str(unpack_uint8(data, pos[0]))  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 711, __pyx_L1_error)
    __pyx_v_s_k = __pyx_t_4;

    /* "serializor/unicode/des.py":712
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         unit: str = utils.map_nptime_unit_int2str(unpack_uint8(data, pos[0]))  # type: ignore
 *         pos[0] += 1  # skip unit
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 712, __pyx_L1_error)
    __pyx_v_s_l = __pyx_t_4;

    /* "serializor/unicode/des.py":713
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         unit: str = utils.map_nptime_unit_int2str(unpack_uint8(data, pos[0]))  # type: ignore             # <<<<<<<<<<<<<<
 *         pos[0] += 1  # skip unit
 *         arr = np.empty(
 */
    __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_pos[0])); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 713, __pyx_L1_error)
    __pyx_t_6 = __pyx_f_10serializor_5utils_map_nptime_unit_int2str(__pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 713, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_v_unit = ((PyObject*)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":714
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         unit: str = utils.map_nptime_unit_int2str(unpack_uint8(data, pos[0]))  # type: ignore
 *         pos[0] += 1  # skip unit             # <<<<<<<<<<<<<<
 *         arr = np.empty(
 *             [s_i, s_j, s_k, s_l],
 */
    __pyx_t_2 = 0;
    (__pyx_v_pos[__pyx_t_2]) = ((__pyx_v_pos[__pyx_t_2]) + 1);

    /* "serializor/unicode/des.py":715
 *         unit: str = utils.map_nptime_unit_int2str(unpack_uint8(data, pos[0]))  # type: ignore
 *         pos[0] += 1  # skip unit
 *         arr = np.empty(             # <<<<<<<<<<<<<<
 *             [s_i, s_j, s_k, s_l],
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,
 */
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_n_s_np); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 715, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_empty); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 715, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "serializor/unicode/des.py":716
 *         pos[0] += 1  # skip unit
 *         arr = np.empty(
 *             [s_i, s_j, s_k, s_l],             # <<<<<<<<<<<<<<
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,
 *         )
 */
    __pyx_t_8 = PyInt_FromSsize_t(__pyx_v_s_i); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 716, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_17 = PyInt_FromSsize_t(__pyx_v_s_j); if (unlikely(!__pyx_t_17)) __PYX_ERR(1, 716, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_17);
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_v_s_k); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 716, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_7 = PyInt_FromSsize_t(__pyx_v_s_l); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 716, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_21 = PyList_New(4); if (unlikely(!__pyx_t_21)) __PYX_ERR(1, 716, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_21);
    __Pyx_GIVEREF(__pyx_t_8);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 0, __pyx_t_8)) __PYX_ERR(1, 716, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_17);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 1, __pyx_t_17)) __PYX_ERR(1, 716, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_9);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 2, __pyx_t_9)) __PYX_ERR(1, 716, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_7);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_21, 3, __pyx_t_7)) __PYX_ERR(1, 716, __pyx_L1_error);
    __pyx_t_8 = 0;
    __pyx_t_17 = 0;
    __pyx_t_9 = 0;
    __pyx_t_7 = 0;

    /* "serializor/unicode/des.py":717
 *         arr = np.empty(
 *             [s_i, s_j, s_k, s_l],
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,             # <<<<<<<<<<<<<<
 *         )
 *         if s_l == 0:
 */
    if (__pyx_v_is_dt64) {
      __pyx_t_9 = PyUnicode_Format(__pyx_kp_u_datetime64_s, __pyx_v_unit); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 717, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_7 = __pyx_t_9;
      __pyx_t_9 = 0;
    } else {
      __pyx_t_9 = PyUnicode_Format(__pyx_kp_u_timedelta64_s, __pyx_v_unit); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 717, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_7 = __pyx_t_9;
      __pyx_t_9 = 0;
    }
    __pyx_t_9 = NULL;
    __pyx_t_10 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_10 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_9, __pyx_t_21, __pyx_t_7};
      __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_10, 2+__pyx_t_10);
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 715, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }

    /* "serializor/unicode/des.py":715
 *         unit: str = utils.map_nptime_unit_int2str(unpack_uint8(data, pos[0]))  # type: ignore
 *         pos[0] += 1  # skip unit
 *         arr = np.empty(             # <<<<<<<<<<<<<<
 *             [s_i, s_j, s_k, s_l],
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,
 */
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 715, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":719
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,
 *         )
 *         if s_l == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    __pyx_t_3 = (__pyx_v_s_l == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":720
 *         )
 *         if s_l == 0:
 *             return arr  # exit: empty             # <<<<<<<<<<<<<<
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF((PyObject *)__pyx_v_arr);
      __pyx_r = ((PyObject *)__pyx_v_arr);
      goto __pyx_L0;

      /* "serializor/unicode/des.py":719
 *             "datetime64[%s]" % unit if is_dt64 else "timedelta64[%s]" % unit,
 *         )
 *         if s_l == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    }

    /* "serializor/unicode/des.py":722
 *             return arr  # exit: empty
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 722, __pyx_L1_error)
    __pyx_v_values_len = __pyx_t_4;

    /* "serializor/unicode/des.py":723
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 */
    __pyx_v_idx = (__pyx_v_pos[0]);

    /* "serializor/unicode/des.py":724
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len             # <<<<<<<<<<<<<<
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 */
    __pyx_v_end = (__pyx_v_idx + __pyx_v_values_len);

    /* "serializor/unicode/des.py":725
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]             # <<<<<<<<<<<<<<
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 */
    __pyx_t_6 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 725, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_v_values_json = ((PyObject*)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":726
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 */
    (__pyx_v_pos[0]) = __pyx_v_end;

    /* "serializor/unicode/des.py":727
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))             # <<<<<<<<<<<<<<
 *         for i in range(s_i):
 *             for j in range(s_j):
 */
    __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__orjson_loads(__pyx_v_values_json); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 727, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 727, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_values = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "serializor/unicode/des.py":728
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             for j in range(s_j):
 *                 for k in range(s_k):
 */
    __pyx_t_11 = __pyx_v_s_i;
    __pyx_t_12 = __pyx_t_11;
    for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
      __pyx_v_i = __pyx_t_13;

      /* "serializor/unicode/des.py":729
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 *             for j in range(s_j):             # <<<<<<<<<<<<<<
 *                 for k in range(s_k):
 *                     for l in range(s_l):
 */
      __pyx_t_14 = __pyx_v_s_j;
      __pyx_t_15 = __pyx_t_14;
      for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
        __pyx_v_j = __pyx_t_16;

        /* "serializor/unicode/des.py":730
 *         for i in range(s_i):
 *             for j in range(s_j):
 *                 for k in range(s_k):             # <<<<<<<<<<<<<<
 *                     for l in range(s_l):
 *                         utils.arr_setitem_4d(arr, next(values), i, j, k, l)
 */
        __pyx_t_18 = __pyx_v_s_k;
        __pyx_t_19 = __pyx_t_18;
        for (__pyx_t_20 = 0; __pyx_t_20 < __pyx_t_19; __pyx_t_20+=1) {
          __pyx_v_k = __pyx_t_20;

          /* "serializor/unicode/des.py":731
 *             for j in range(s_j):
 *                 for k in range(s_k):
 *                     for l in range(s_l):             # <<<<<<<<<<<<<<
 *                         utils.arr_setitem_4d(arr, next(values), i, j, k, l)
 *         return arr
 */
          __pyx_t_22 = __pyx_v_s_l;
          __pyx_t_23 = __pyx_t_22;
          for (__pyx_t_24 = 0; __pyx_t_24 < __pyx_t_23; __pyx_t_24+=1) {
            __pyx_v_l = __pyx_t_24;

            /* "serializor/unicode/des.py":732
 *                 for k in range(s_k):
 *                     for l in range(s_l):
 *                         utils.arr_setitem_4d(arr, next(values), i, j, k, l)             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
            __pyx_t_5 = __Pyx_PyIter_Next(__pyx_v_values); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 732, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_5);
            __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_4d(__pyx_v_arr, __pyx_t_5, __pyx_v_i, __pyx_v_j, __pyx_v_k, __pyx_v_l); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 732, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          }
        }
      }
    }

    /* "serializor/unicode/des.py":733
 *                     for l in range(s_l):
 *                         utils.arr_setitem_4d(arr, next(values), i, j, k, l)
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # Invalid
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":707
 * 
 *     # 4-dimension
 *     if ndim == 4:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":736
 * 
 *     # Invalid
 *     raise ValueError("unsupported <'numpy.ndarray'> dimension [%d]." % ndim)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_5 = __Pyx_PyInt_From_unsigned_char(__pyx_v_ndim); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 736, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyUnicode_Format(__pyx_kp_u_unsupported_numpy_ndarray_dimens, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 736, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 736, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_Raise(__pyx_t_5, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __PYX_ERR(1, 736, __pyx_L1_error)

  /* "serializor/unicode/des.py":615
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_ndarray_nptime(
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_21);
  __Pyx_AddTraceback("serializor.unicode.des._des_ndarray_nptime", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_arr);
  __Pyx_XDECREF(__pyx_v_unit);
  __Pyx_XDECREF(__pyx_v_values_json);
  __Pyx_XDECREF(__pyx_v_values);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":739
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_ndarray_complex(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_ndarray_complex(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  Py_ssize_t __pyx_v_idx;
  unsigned char __pyx_v_npy_dtype;
  unsigned char __pyx_v_ndim;
  PyArrayObject *__pyx_v_arr = 0;
  Py_ssize_t __pyx_v_s_i;
  unsigned PY_LONG_LONG __pyx_v_values_len;
  Py_ssize_t __pyx_v_end;
  PyObject *__pyx_v_values_json = 0;
  PyObject *__pyx_v_values = NULL;
  Py_ssize_t __pyx_v_i;
  Py_ssize_t __pyx_v_s_j;
  Py_ssize_t __pyx_v_j;
  Py_ssize_t __pyx_v_s_k;
  Py_ssize_t __pyx_v_k;
  Py_ssize_t __pyx_v_s_l;
  Py_ssize_t __pyx_v_l;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  unsigned char __pyx_t_1;
  long __pyx_t_2;
  int __pyx_t_3;
  unsigned PY_LONG_LONG __pyx_t_4;
  npy_intp __pyx_t_5[1];
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  double __pyx_t_11;
  double __pyx_t_12;
  npy_intp __pyx_t_13[2];
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  npy_intp __pyx_t_17[3];
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  npy_intp __pyx_t_21[4];
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_ndarray_complex", 1);

  /* "serializor/unicode/des.py":748
 *     Position updates to the next charactor after deserialization.
 *     """
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip array dtype             # <<<<<<<<<<<<<<
 *     npy_dtype = unpack_uint8(data, idx)  # type: ignore
 *     ndim: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore
 */
  __pyx_v_idx = ((__pyx_v_pos[0]) + 1);

  /* "serializor/unicode/des.py":749
 *     """
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip array dtype
 *     npy_dtype = unpack_uint8(data, idx)  # type: ignore             # <<<<<<<<<<<<<<
 *     ndim: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore
 *     pos[0] += 3  # set new position
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, __pyx_v_idx); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 749, __pyx_L1_error)
  __pyx_v_npy_dtype = __pyx_t_1;

  /* "serializor/unicode/des.py":750
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip array dtype
 *     npy_dtype = unpack_uint8(data, idx)  # type: ignore
 *     ndim: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore             # <<<<<<<<<<<<<<
 *     pos[0] += 3  # set new position
 *     arr: np.ndarray
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_idx + 1)); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 750, __pyx_L1_error)
  __pyx_v_ndim = __pyx_t_1;

  /* "serializor/unicode/des.py":751
 *     npy_dtype = unpack_uint8(data, idx)  # type: ignore
 *     ndim: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore
 *     pos[0] += 3  # set new position             # <<<<<<<<<<<<<<
 *     arr: np.ndarray
 *     # 1-dimension
 */
  __pyx_t_2 = 0;
  (__pyx_v_pos[__pyx_t_2]) = ((__pyx_v_pos[__pyx_t_2]) + 3);

  /* "serializor/unicode/des.py":754
 *     arr: np.ndarray
 *     # 1-dimension
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 1);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":756
 *     if ndim == 1:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         arr = np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)
 *         if s_i == 0:
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 756, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":757
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)             # <<<<<<<<<<<<<<
 *         if s_i == 0:
 *             return arr  # exit: empty
 */
    __pyx_t_5[0] = __pyx_v_s_i;
    __pyx_t_6 = PyArray_EMPTY(__pyx_v_ndim, __pyx_t_5, __pyx_v_npy_dtype, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 757, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 757, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":758
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)
 *         if s_i == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    __pyx_t_3 = (__pyx_v_s_i == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":759
 *         arr = np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)
 *         if s_i == 0:
 *             return arr  # exit: empty             # <<<<<<<<<<<<<<
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF((PyObject *)__pyx_v_arr);
      __pyx_r = ((PyObject *)__pyx_v_arr);
      goto __pyx_L0;

      /* "serializor/unicode/des.py":758
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)
 *         if s_i == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    }

    /* "serializor/unicode/des.py":761
 *             return arr  # exit: empty
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 761, __pyx_L1_error)
    __pyx_v_values_len = __pyx_t_4;

    /* "serializor/unicode/des.py":762
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 */
    __pyx_v_idx = (__pyx_v_pos[0]);

    /* "serializor/unicode/des.py":763
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len             # <<<<<<<<<<<<<<
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 */
    __pyx_v_end = (__pyx_v_idx + __pyx_v_values_len);

    /* "serializor/unicode/des.py":764
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]             # <<<<<<<<<<<<<<
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 */
    __pyx_t_6 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 764, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_v_values_json = ((PyObject*)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":765
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 */
    (__pyx_v_pos[0]) = __pyx_v_end;

    /* "serializor/unicode/des.py":766
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))             # <<<<<<<<<<<<<<
 *         for i in range(s_i):
 *             utils.arr_setitem_1d(arr, gen_complex(next(values), next(values)), i)
 */
    __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__orjson_loads(__pyx_v_values_json); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 766, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 766, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_values = __pyx_t_7;
    __pyx_t_7 = 0;

    /* "serializor/unicode/des.py":767
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             utils.arr_setitem_1d(arr, gen_complex(next(values), next(values)), i)
 *         return arr
 */
    __pyx_t_8 = __pyx_v_s_i;
    __pyx_t_9 = __pyx_t_8;
    for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
      __pyx_v_i = __pyx_t_10;

      /* "serializor/unicode/des.py":768
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 *             utils.arr_setitem_1d(arr, gen_complex(next(values), next(values)), i)             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
      __pyx_t_7 = __Pyx_PyIter_Next(__pyx_v_values); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 768, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_11 = __pyx_PyFloat_AsDouble(__pyx_t_7); if (unlikely((__pyx_t_11 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 768, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = __Pyx_PyIter_Next(__pyx_v_values); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 768, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_12 = __pyx_PyFloat_AsDouble(__pyx_t_7); if (unlikely((__pyx_t_12 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 768, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = PyComplex_FromDoubles(__pyx_t_11, __pyx_t_12); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 768, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_1d(__pyx_v_arr, __pyx_t_7, __pyx_v_i); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 768, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }

    /* "serializor/unicode/des.py":769
 *         for i in range(s_i):
 *             utils.arr_setitem_1d(arr, gen_complex(next(values), next(values)), i)
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # 2-dimension
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":754
 *     arr: np.ndarray
 *     # 1-dimension
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":772
 * 
 *     # 2-dimension
 *     if ndim == 2:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 2);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":774
 *     if ndim == 2:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 774, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":775
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)
 *         if s_j == 0:
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 775, __pyx_L1_error)
    __pyx_v_s_j = __pyx_t_4;

    /* "serializor/unicode/des.py":776
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)             # <<<<<<<<<<<<<<
 *         if s_j == 0:
 *             return arr  # exit: empty
 */
    __pyx_t_13[0] = __pyx_v_s_i;
    __pyx_t_13[1] = __pyx_v_s_j;
    __pyx_t_7 = PyArray_EMPTY(__pyx_v_ndim, __pyx_t_13, __pyx_v_npy_dtype, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 776, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (!(likely(((__pyx_t_7) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_7, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 776, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_7);
    __pyx_t_7 = 0;

    /* "serializor/unicode/des.py":777
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)
 *         if s_j == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    __pyx_t_3 = (__pyx_v_s_j == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":778
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)
 *         if s_j == 0:
 *             return arr  # exit: empty             # <<<<<<<<<<<<<<
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF((PyObject *)__pyx_v_arr);
      __pyx_r = ((PyObject *)__pyx_v_arr);
      goto __pyx_L0;

      /* "serializor/unicode/des.py":777
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)
 *         if s_j == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    }

    /* "serializor/unicode/des.py":780
 *             return arr  # exit: empty
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 780, __pyx_L1_error)
    __pyx_v_values_len = __pyx_t_4;

    /* "serializor/unicode/des.py":781
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 */
    __pyx_v_idx = (__pyx_v_pos[0]);

    /* "serializor/unicode/des.py":782
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len             # <<<<<<<<<<<<<<
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 */
    __pyx_v_end = (__pyx_v_idx + __pyx_v_values_len);

    /* "serializor/unicode/des.py":783
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]             # <<<<<<<<<<<<<<
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 */
    __pyx_t_7 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 783, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_v_values_json = ((PyObject*)__pyx_t_7);
    __pyx_t_7 = 0;

    /* "serializor/unicode/des.py":784
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 */
    (__pyx_v_pos[0]) = __pyx_v_end;

    /* "serializor/unicode/des.py":785
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))             # <<<<<<<<<<<<<<
 *         for i in range(s_i):
 *             for j in range(s_j):
 */
    __pyx_t_7 = __pyx_f_10serializor_7unicode_3des__orjson_loads(__pyx_v_values_json); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 785, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 785, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_v_values = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":786
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             for j in range(s_j):
 *                 utils.arr_setitem_2d(arr, gen_complex(next(values), next(values)), i, j)
 */
    __pyx_t_8 = __pyx_v_s_i;
    __pyx_t_9 = __pyx_t_8;
    for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
      __pyx_v_i = __pyx_t_10;

      /* "serializor/unicode/des.py":787
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 *             for j in range(s_j):             # <<<<<<<<<<<<<<
 *                 utils.arr_setitem_2d(arr, gen_complex(next(values), next(values)), i, j)
 *         return arr
 */
      __pyx_t_14 = __pyx_v_s_j;
      __pyx_t_15 = __pyx_t_14;
      for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
        __pyx_v_j = __pyx_t_16;

        /* "serializor/unicode/des.py":788
 *         for i in range(s_i):
 *             for j in range(s_j):
 *                 utils.arr_setitem_2d(arr, gen_complex(next(values), next(values)), i, j)             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
        __pyx_t_6 = __Pyx_PyIter_Next(__pyx_v_values); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 788, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_12 = __pyx_PyFloat_AsDouble(__pyx_t_6); if (unlikely((__pyx_t_12 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 788, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_6 = __Pyx_PyIter_Next(__pyx_v_values); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 788, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_11 = __pyx_PyFloat_AsDouble(__pyx_t_6); if (unlikely((__pyx_t_11 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 788, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_6 = PyComplex_FromDoubles(__pyx_t_12, __pyx_t_11); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 788, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_2d(__pyx_v_arr, __pyx_t_6, __pyx_v_i, __pyx_v_j); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 788, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }

    /* "serializor/unicode/des.py":789
 *             for j in range(s_j):
 *                 utils.arr_setitem_2d(arr, gen_complex(next(values), next(values)), i, j)
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # 3-dimension
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":772
 * 
 *     # 2-dimension
 *     if ndim == 2:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":792
 * 
 *     # 3-dimension
 *     if ndim == 3:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 3);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":794
 *     if ndim == 3:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 794, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":795
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 795, __pyx_L1_error)
    __pyx_v_s_j = __pyx_t_4;

    /* "serializor/unicode/des.py":796
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)
 *         if s_k == 0:
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 796, __pyx_L1_error)
    __pyx_v_s_k = __pyx_t_4;

    /* "serializor/unicode/des.py":797
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)             # <<<<<<<<<<<<<<
 *         if s_k == 0:
 *             return arr  # exit: empty
 */
    __pyx_t_17[0] = __pyx_v_s_i;
    __pyx_t_17[1] = __pyx_v_s_j;
    __pyx_t_17[2] = __pyx_v_s_k;
    __pyx_t_6 = PyArray_EMPTY(__pyx_v_ndim, __pyx_t_17, __pyx_v_npy_dtype, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 797, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 797, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":798
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)
 *         if s_k == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    __pyx_t_3 = (__pyx_v_s_k == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":799
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)
 *         if s_k == 0:
 *             return arr  # exit: empty             # <<<<<<<<<<<<<<
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF((PyObject *)__pyx_v_arr);
      __pyx_r = ((PyObject *)__pyx_v_arr);
      goto __pyx_L0;

      /* "serializor/unicode/des.py":798
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)
 *         if s_k == 0:             # <<<<<<<<<<<<<<
 *             return arr  # exit: empty
 *         # . deserialize values
 */
    }

    /* "serializor/unicode/des.py":801
 *             return arr  # exit: empty
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 801, __pyx_L1_error)
    __pyx_v_values_len = __pyx_t_4;

    /* "serializor/unicode/des.py":802
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 */
    __pyx_v_idx = (__pyx_v_pos[0]);

    /* "serializor/unicode/des.py":803
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len             # <<<<<<<<<<<<<<
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 */
    __pyx_v_end = (__pyx_v_idx + __pyx_v_values_len);

    /* "serializor/unicode/des.py":804
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]             # <<<<<<<<<<<<<<
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 */
    __pyx_t_6 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 804, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_v_values_json = ((PyObject*)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":805
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 */
    (__pyx_v_pos[0]) = __pyx_v_end;

    /* "serializor/unicode/des.py":806
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))             # <<<<<<<<<<<<<<
 *         for i in range(s_i):
 *             for j in range(s_j):
 */
    __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__orjson_loads(__pyx_v_values_json); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 806, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 806, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_values = __pyx_t_7;
    __pyx_t_7 = 0;

    /* "serializor/unicode/des.py":807
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             for j in range(s_j):
 *                 for k in range(s_k):
 */
    __pyx_t_8 = __pyx_v_s_i;
    __pyx_t_9 = __pyx_t_8;
    for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
      __pyx_v_i = __pyx_t_10;

      /* "serializor/unicode/des.py":808
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 *             for j in range(s_j):             # <<<<<<<<<<<<<<
 *                 for k in range(s_k):
 *                     utils.arr_setitem_3d(arr, gen_complex(next(values), next(values)), i, j, k)  # type: ignore
 */
      __pyx_t_14 = __pyx_v_s_j;
      __pyx_t_15 = __pyx_t_14;
      for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
        __pyx_v_j = __pyx_t_16;

        /* "serializor/unicode/des.py":809
 *         for i in range(s_i):
 *             for j in range(s_j):
 *                 for k in range(s_k):             # <<<<<<<<<<<<<<
 *                     utils.arr_setitem_3d(arr, gen_complex(next(values), next(values)), i, j, k)  # type: ignore
 *         return arr
 */
        __pyx_t_18 = __pyx_v_s_k;
        __pyx_t_19 = __pyx_t_18;
        for (__pyx_t_20 = 0; __pyx_t_20 < __pyx_t_19; __pyx_t_20+=1) {
          __pyx_v_k = __pyx_t_20;

          /* "serializor/unicode/des.py":810
 *             for j in range(s_j):
 *                 for k in range(s_k):
 *                     utils.arr_setitem_3d(arr, gen_complex(next(values), next(values)), i, j, k)  # type: ignore             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
          __pyx_t_7 = __Pyx_PyIter_Next(__pyx_v_values); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 810, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __pyx_t_11 = __pyx_PyFloat_AsDouble(__pyx_t_7); if (unlikely((__pyx_t_11 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 810, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __pyx_t_7 = __Pyx_PyIter_Next(__pyx_v_values); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 810, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __pyx_t_12 = __pyx_PyFloat_AsDouble(__pyx_t_7); if (unlikely((__pyx_t_12 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 810, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __pyx_t_7 = PyComplex_FromDoubles(__pyx_t_11, __pyx_t_12); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 810, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_3d(__pyx_v_arr, __pyx_t_7, __pyx_v_i, __pyx_v_j, __pyx_v_k); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 810, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
      }
    }

    /* "serializor/unicode/des.py":811
 *                 for k in range(s_k):
 *                     utils.arr_setitem_3d(arr, gen_complex(next(values), next(values)), i, j, k)  # type: ignore
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # 4-dimension
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":792
 * 
 *     # 3-dimension
 *     if ndim == 3:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":814
 * 
 *     # 4-dimension
 *     if ndim == 4:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 4);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":816
 *     if ndim == 4:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 816, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":817
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 817, __pyx_L1_error)
    __pyx_v_s_j = __pyx_t_4;

    /* "serializor/unicode/des.py":818
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0)  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 818, __pyx_L1_error)
    __pyx_v_s_k = __pyx_t_4;

    /* "serializor/unicode/des.py":819
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0)  # type: ignore
 *         if s_l == 0:
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 819, __pyx_L1_error)
    __pyx_v_s_l = __pyx_t_4;

    /* "serializor/unicode/des.py":820
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0)  # type: ignore             # <<<<<<<<<<<<<<
 *         if s_l == 0:
 *             return arr
 */
    __pyx_t_21[0] = __pyx_v_s_i;
    __pyx_t_21[1] = __pyx_v_s_j;
    __pyx_t_21[2] = __pyx_v_s_k;
    __pyx_t_21[3] = __pyx_v_s_l;
    __pyx_t_7 = PyArray_EMPTY(__pyx_v_ndim, __pyx_t_21, __pyx_v_npy_dtype, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 820, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (!(likely(((__pyx_t_7) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_7, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 820, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_7);
    __pyx_t_7 = 0;

    /* "serializor/unicode/des.py":821
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0)  # type: ignore
 *         if s_l == 0:             # <<<<<<<<<<<<<<
 *             return arr
 *         # . deserialize values
 */
    __pyx_t_3 = (__pyx_v_s_l == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":822
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0)  # type: ignore
 *         if s_l == 0:
 *             return arr             # <<<<<<<<<<<<<<
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF((PyObject *)__pyx_v_arr);
      __pyx_r = ((PyObject *)__pyx_v_arr);
      goto __pyx_L0;

      /* "serializor/unicode/des.py":821
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.PyArray_EMPTY(ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0)  # type: ignore
 *         if s_l == 0:             # <<<<<<<<<<<<<<
 *             return arr
 *         # . deserialize values
 */
    }

    /* "serializor/unicode/des.py":824
 *             return arr
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 824, __pyx_L1_error)
    __pyx_v_values_len = __pyx_t_4;

    /* "serializor/unicode/des.py":825
 *         # . deserialize values
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 */
    __pyx_v_idx = (__pyx_v_pos[0]);

    /* "serializor/unicode/des.py":826
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len             # <<<<<<<<<<<<<<
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 */
    __pyx_v_end = (__pyx_v_idx + __pyx_v_values_len);

    /* "serializor/unicode/des.py":827
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]             # <<<<<<<<<<<<<<
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 */
    __pyx_t_7 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 827, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_v_values_json = ((PyObject*)__pyx_t_7);
    __pyx_t_7 = 0;

    /* "serializor/unicode/des.py":828
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 */
    (__pyx_v_pos[0]) = __pyx_v_end;

    /* "serializor/unicode/des.py":829
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))             # <<<<<<<<<<<<<<
 *         for i in range(s_i):
 *             for j in range(s_j):
 */
    __pyx_t_7 = __pyx_f_10serializor_7unicode_3des__orjson_loads(__pyx_v_values_json); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 829, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 829, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_v_values = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":830
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             for j in range(s_j):
 *                 for k in range(s_k):
 */
    __pyx_t_8 = __pyx_v_s_i;
    __pyx_t_9 = __pyx_t_8;
    for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
      __pyx_v_i = __pyx_t_10;

      /* "serializor/unicode/des.py":831
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 *             for j in range(s_j):             # <<<<<<<<<<<<<<
 *                 for k in range(s_k):
 *                     for l in range(s_l):
 */
      __pyx_t_14 = __pyx_v_s_j;
      __pyx_t_15 = __pyx_t_14;
      for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
        __pyx_v_j = __pyx_t_16;

        /* "serializor/unicode/des.py":832
 *         for i in range(s_i):
 *             for j in range(s_j):
 *                 for k in range(s_k):             # <<<<<<<<<<<<<<
 *                     for l in range(s_l):
 *                         utils.arr_setitem_4d(arr, gen_complex(next(values), next(values)), i, j, k, l)  # type: ignore
 */
        __pyx_t_18 = __pyx_v_s_k;
        __pyx_t_19 = __pyx_t_18;
        for (__pyx_t_20 = 0; __pyx_t_20 < __pyx_t_19; __pyx_t_20+=1) {
          __pyx_v_k = __pyx_t_20;

          /* "serializor/unicode/des.py":833
 *             for j in range(s_j):
 *                 for k in range(s_k):
 *                     for l in range(s_l):             # <<<<<<<<<<<<<<
 *                         utils.arr_setitem_4d(arr, gen_complex(next(values), next(values)), i, j, k, l)  # type: ignore
 *         return arr
 */
          __pyx_t_22 = __pyx_v_s_l;
          __pyx_t_23 = __pyx_t_22;
          for (__pyx_t_24 = 0; __pyx_t_24 < __pyx_t_23; __pyx_t_24+=1) {
            __pyx_v_l = __pyx_t_24;

            /* "serializor/unicode/des.py":834
 *                 for k in range(s_k):
 *                     for l in range(s_l):
 *                         utils.arr_setitem_4d(arr, gen_complex(next(values), next(values)), i, j, k, l)  # type: ignore             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
            __pyx_t_6 = __Pyx_PyIter_Next(__pyx_v_values); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 834, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_6);
            __pyx_t_12 = __pyx_PyFloat_AsDouble(__pyx_t_6); if (unlikely((__pyx_t_12 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 834, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
            __pyx_t_6 = __Pyx_PyIter_Next(__pyx_v_values); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 834, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_6);
            __pyx_t_11 = __pyx_PyFloat_AsDouble(__pyx_t_6); if (unlikely((__pyx_t_11 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 834, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
            __pyx_t_6 = PyComplex_FromDoubles(__pyx_t_12, __pyx_t_11); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 834, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_6);
            __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_4d(__pyx_v_arr, __pyx_t_6, __pyx_v_i, __pyx_v_j, __pyx_v_k, __pyx_v_l); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 834, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          }
        }
      }
    }

    /* "serializor/unicode/des.py":835
 *                     for l in range(s_l):
 *                         utils.arr_setitem_4d(arr, gen_complex(next(values), next(values)), i, j, k, l)  # type: ignore
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # Invalid
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":814
 * 
 *     # 4-dimension
 *     if ndim == 4:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":838
 * 
 *     # Invalid
 *     raise ValueError("unsupported <'numpy.ndarray'> dimension [%d]." % ndim)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_6 = __Pyx_PyInt_From_unsigned_char(__pyx_v_ndim); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 838, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = PyUnicode_Format(__pyx_kp_u_unsupported_numpy_ndarray_dimens, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 838, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 838, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_Raise(__pyx_t_6, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __PYX_ERR(1, 838, __pyx_L1_error)

  /* "serializor/unicode/des.py":739
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_ndarray_complex(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("serializor.unicode.des._des_ndarray_complex", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_arr);
  __Pyx_XDECREF(__pyx_v_values_json);
  __Pyx_XDECREF(__pyx_v_values);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":841
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_ndarray_bytes(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_ndarray_bytes(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  Py_ssize_t __pyx_v_idx;
  unsigned char __pyx_v_npy_dtype;
  unsigned char __pyx_v_ndim;
  PyArrayObject *__pyx_v_arr = 0;
  Py_ssize_t __pyx_v_s_i;
  int __pyx_v_ch_size;
  Py_ssize_t __pyx_v_i;
  Py_ssize_t __pyx_v_s_j;
  Py_ssize_t __pyx_v_j;
  Py_ssize_t __pyx_v_s_k;
  Py_ssize_t __pyx_v_k;
  Py_ssize_t __pyx_v_s_l;
  Py_ssize_t __pyx_v_l;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  unsigned char __pyx_t_1;
  long __pyx_t_2;
  int __pyx_t_3;
  unsigned PY_LONG_LONG __pyx_t_4;
  npy_intp __pyx_t_5[1];
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  unsigned int __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  npy_intp __pyx_t_15[2];
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  npy_intp __pyx_t_19[3];
  PyObject *__pyx_t_20 = NULL;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  npy_intp __pyx_t_24[4];
  PyObject *__pyx_t_25 = NULL;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_ndarray_bytes", 1);

  /* "serializor/unicode/des.py":850
 *     Position updates to the next charactor after deserialization.
 *     """
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip array dtype             # <<<<<<<<<<<<<<
 *     npy_dtype = unpack_uint8(data, idx)  # type: ignore
 *     ndim: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore
 */
  __pyx_v_idx = ((__pyx_v_pos[0]) + 1);

  /* "serializor/unicode/des.py":851
 *     """
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip array dtype
 *     npy_dtype = unpack_uint8(data, idx)  # type: ignore             # <<<<<<<<<<<<<<
 *     ndim: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore
 *     pos[0] += 3  # set new position
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, __pyx_v_idx); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 851, __pyx_L1_error)
  __pyx_v_npy_dtype = __pyx_t_1;

  /* "serializor/unicode/des.py":852
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip array dtype
 *     npy_dtype = unpack_uint8(data, idx)  # type: ignore
 *     ndim: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore             # <<<<<<<<<<<<<<
 *     pos[0] += 3  # set new position
 *     arr: np.ndarray
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_idx + 1)); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 852, __pyx_L1_error)
  __pyx_v_ndim = __pyx_t_1;

  /* "serializor/unicode/des.py":853
 *     npy_dtype = unpack_uint8(data, idx)  # type: ignore
 *     ndim: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore
 *     pos[0] += 3  # set new position             # <<<<<<<<<<<<<<
 *     arr: np.ndarray
 *     # 1-dimension
 */
  __pyx_t_2 = 0;
  (__pyx_v_pos[__pyx_t_2]) = ((__pyx_v_pos[__pyx_t_2]) + 3);

  /* "serializor/unicode/des.py":856
 *     arr: np.ndarray
 *     # 1-dimension
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 1);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":858
 *     if ndim == 1:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         if s_i == 0:
 *             return np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)  # exit: empty
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 858, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":859
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_i == 0:             # <<<<<<<<<<<<<<
 *             return np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)  # exit: empty
 *         # . deserialize values
 */
    __pyx_t_3 = (__pyx_v_s_i == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":860
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_i == 0:
 *             return np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)  # exit: empty             # <<<<<<<<<<<<<<
 *         # . deserialize values
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_5[0] = __pyx_v_s_i;
      __pyx_t_6 = PyArray_EMPTY(__pyx_v_ndim, __pyx_t_5, __pyx_v_npy_dtype, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 860, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_r = __pyx_t_6;
      __pyx_t_6 = 0;
      goto __pyx_L0;

      /* "serializor/unicode/des.py":859
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_i == 0:             # <<<<<<<<<<<<<<
 *             return np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)  # exit: empty
 *         # . deserialize values
 */
    }

    /* "serializor/unicode/des.py":862
 *             return np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)  # exit: empty
 *         # . deserialize values
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         arr = np.empty(s_i, "S%d" % ch_size)
 *         for i in range(s_i):
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 862, __pyx_L1_error)
    __pyx_v_ch_size = __pyx_t_4;

    /* "serializor/unicode/des.py":863
 *         # . deserialize values
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.empty(s_i, "S%d" % ch_size)             # <<<<<<<<<<<<<<
 *         for i in range(s_i):
 *             utils.arr_setitem_1d(arr, _des_bytes(data, pos), i)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_np); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 863, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_empty); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 863, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = PyInt_FromSsize_t(__pyx_v_s_i); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 863, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = __Pyx_PyInt_From_int(__pyx_v_ch_size); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 863, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = PyUnicode_Format(__pyx_kp_u_S_d, __pyx_t_9); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 863, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = NULL;
    __pyx_t_11 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
        __pyx_t_11 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_9, __pyx_t_7, __pyx_t_10};
      __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+1-__pyx_t_11, 2+__pyx_t_11);
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 863, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 863, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":864
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.empty(s_i, "S%d" % ch_size)
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             utils.arr_setitem_1d(arr, _des_bytes(data, pos), i)
 *         return arr
 */
    __pyx_t_12 = __pyx_v_s_i;
    __pyx_t_13 = __pyx_t_12;
    for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
      __pyx_v_i = __pyx_t_14;

      /* "serializor/unicode/des.py":865
 *         arr = np.empty(s_i, "S%d" % ch_size)
 *         for i in range(s_i):
 *             utils.arr_setitem_1d(arr, _des_bytes(data, pos), i)             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
      __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__des_bytes(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 865, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_1d(__pyx_v_arr, __pyx_t_6, __pyx_v_i); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 865, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }

    /* "serializor/unicode/des.py":866
 *         for i in range(s_i):
 *             utils.arr_setitem_1d(arr, _des_bytes(data, pos), i)
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # 2-dimension
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":856
 *     arr: np.ndarray
 *     # 1-dimension
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":869
 * 
 *     # 2-dimension
 *     if ndim == 2:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 2);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":871
 *     if ndim == 2:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_j == 0:
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 871, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":872
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         if s_j == 0:
 *             return np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)  # exit: empty
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 872, __pyx_L1_error)
    __pyx_v_s_j = __pyx_t_4;

    /* "serializor/unicode/des.py":873
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_j == 0:             # <<<<<<<<<<<<<<
 *             return np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)  # exit: empty
 *         # . deserialize values
 */
    __pyx_t_3 = (__pyx_v_s_j == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":874
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_j == 0:
 *             return np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)  # exit: empty             # <<<<<<<<<<<<<<
 *         # . deserialize values
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_15[0] = __pyx_v_s_i;
      __pyx_t_15[1] = __pyx_v_s_j;
      __pyx_t_6 = PyArray_EMPTY(__pyx_v_ndim, __pyx_t_15, __pyx_v_npy_dtype, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 874, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_r = __pyx_t_6;
      __pyx_t_6 = 0;
      goto __pyx_L0;

      /* "serializor/unicode/des.py":873
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_j == 0:             # <<<<<<<<<<<<<<
 *             return np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)  # exit: empty
 *         # . deserialize values
 */
    }

    /* "serializor/unicode/des.py":876
 *             return np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)  # exit: empty
 *         # . deserialize values
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         arr = np.empty([s_i, s_j], "S%d" % ch_size)
 *         for i in range(s_i):
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 876, __pyx_L1_error)
    __pyx_v_ch_size = __pyx_t_4;

    /* "serializor/unicode/des.py":877
 *         # . deserialize values
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.empty([s_i, s_j], "S%d" % ch_size)             # <<<<<<<<<<<<<<
 *         for i in range(s_i):
 *             for j in range(s_j):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_n_s_np); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 877, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_empty); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 877, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = PyInt_FromSsize_t(__pyx_v_s_i); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 877, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_7 = PyInt_FromSsize_t(__pyx_v_s_j); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 877, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = PyList_New(2); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 877, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 0, __pyx_t_8)) __PYX_ERR(1, 877, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_7);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 1, __pyx_t_7)) __PYX_ERR(1, 877, __pyx_L1_error);
    __pyx_t_8 = 0;
    __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_ch_size); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 877, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = PyUnicode_Format(__pyx_kp_u_S_d, __pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 877, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = NULL;
    __pyx_t_11 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_10))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_10);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_10, function);
        __pyx_t_11 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_7, __pyx_t_9, __pyx_t_8};
      __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_10, __pyx_callargs+1-__pyx_t_11, 2+__pyx_t_11);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 877, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    }
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 877, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":878
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.empty([s_i, s_j], "S%d" % ch_size)
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             for j in range(s_j):
 *                 utils.arr_setitem_2d(arr, _des_bytes(data, pos), i, j)
 */
    __pyx_t_12 = __pyx_v_s_i;
    __pyx_t_13 = __pyx_t_12;
    for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
      __pyx_v_i = __pyx_t_14;

      /* "serializor/unicode/des.py":879
 *         arr = np.empty([s_i, s_j], "S%d" % ch_size)
 *         for i in range(s_i):
 *             for j in range(s_j):             # <<<<<<<<<<<<<<
 *                 utils.arr_setitem_2d(arr, _des_bytes(data, pos), i, j)
 *         return arr
 */
      __pyx_t_16 = __pyx_v_s_j;
      __pyx_t_17 = __pyx_t_16;
      for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
        __pyx_v_j = __pyx_t_18;

        /* "serializor/unicode/des.py":880
 *         for i in range(s_i):
 *             for j in range(s_j):
 *                 utils.arr_setitem_2d(arr, _des_bytes(data, pos), i, j)             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
        __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__des_bytes(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 880, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_2d(__pyx_v_arr, __pyx_t_6, __pyx_v_i, __pyx_v_j); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 880, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }

    /* "serializor/unicode/des.py":881
 *             for j in range(s_j):
 *                 utils.arr_setitem_2d(arr, _des_bytes(data, pos), i, j)
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # 3-dimension
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":869
 * 
 *     # 2-dimension
 *     if ndim == 2:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":884
 * 
 *     # 3-dimension
 *     if ndim == 3:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 3);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":886
 *     if ndim == 3:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 886, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":887
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_k == 0:
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 887, __pyx_L1_error)
    __pyx_v_s_j = __pyx_t_4;

    /* "serializor/unicode/des.py":888
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         if s_k == 0:
 *             return np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)  # exit: empty
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 888, __pyx_L1_error)
    __pyx_v_s_k = __pyx_t_4;

    /* "serializor/unicode/des.py":889
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_k == 0:             # <<<<<<<<<<<<<<
 *             return np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)  # exit: empty
 *         # . deserialize values
 */
    __pyx_t_3 = (__pyx_v_s_k == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":890
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_k == 0:
 *             return np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)  # exit: empty             # <<<<<<<<<<<<<<
 *         # . deserialize values
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_19[0] = __pyx_v_s_i;
      __pyx_t_19[1] = __pyx_v_s_j;
      __pyx_t_19[2] = __pyx_v_s_k;
      __pyx_t_6 = PyArray_EMPTY(__pyx_v_ndim, __pyx_t_19, __pyx_v_npy_dtype, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 890, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_r = __pyx_t_6;
      __pyx_t_6 = 0;
      goto __pyx_L0;

      /* "serializor/unicode/des.py":889
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_k == 0:             # <<<<<<<<<<<<<<
 *             return np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)  # exit: empty
 *         # . deserialize values
 */
    }

    /* "serializor/unicode/des.py":892
 *             return np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)  # exit: empty
 *         # . deserialize values
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         arr = np.empty([s_i, s_j, s_k], "S%d" % ch_size)
 *         for i in range(s_i):
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 892, __pyx_L1_error)
    __pyx_v_ch_size = __pyx_t_4;

    /* "serializor/unicode/des.py":893
 *         # . deserialize values
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.empty([s_i, s_j, s_k], "S%d" % ch_size)             # <<<<<<<<<<<<<<
 *         for i in range(s_i):
 *             for j in range(s_j):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_np); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 893, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_empty); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 893, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = PyInt_FromSsize_t(__pyx_v_s_i); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 893, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_v_s_j); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 893, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_7 = PyInt_FromSsize_t(__pyx_v_s_k); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 893, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_20 = PyList_New(3); if (unlikely(!__pyx_t_20)) __PYX_ERR(1, 893, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_20);
    __Pyx_GIVEREF(__pyx_t_10);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 0, __pyx_t_10)) __PYX_ERR(1, 893, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_9);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 1, __pyx_t_9)) __PYX_ERR(1, 893, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_7);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 2, __pyx_t_7)) __PYX_ERR(1, 893, __pyx_L1_error);
    __pyx_t_10 = 0;
    __pyx_t_9 = 0;
    __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_ch_size); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 893, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = PyUnicode_Format(__pyx_kp_u_S_d, __pyx_t_7); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 893, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = NULL;
    __pyx_t_11 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
        __pyx_t_11 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_7, __pyx_t_20, __pyx_t_9};
      __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+1-__pyx_t_11, 2+__pyx_t_11);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 893, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 893, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":894
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.empty([s_i, s_j, s_k], "S%d" % ch_size)
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             for j in range(s_j):
 *                 for k in range(s_k):
 */
    __pyx_t_12 = __pyx_v_s_i;
    __pyx_t_13 = __pyx_t_12;
    for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
      __pyx_v_i = __pyx_t_14;

      /* "serializor/unicode/des.py":895
 *         arr = np.empty([s_i, s_j, s_k], "S%d" % ch_size)
 *         for i in range(s_i):
 *             for j in range(s_j):             # <<<<<<<<<<<<<<
 *                 for k in range(s_k):
 *                     utils.arr_setitem_3d(arr, _des_bytes(data, pos), i, j, k)
 */
      __pyx_t_16 = __pyx_v_s_j;
      __pyx_t_17 = __pyx_t_16;
      for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
        __pyx_v_j = __pyx_t_18;

        /* "serializor/unicode/des.py":896
 *         for i in range(s_i):
 *             for j in range(s_j):
 *                 for k in range(s_k):             # <<<<<<<<<<<<<<
 *                     utils.arr_setitem_3d(arr, _des_bytes(data, pos), i, j, k)
 *         return arr
 */
        __pyx_t_21 = __pyx_v_s_k;
        __pyx_t_22 = __pyx_t_21;
        for (__pyx_t_23 = 0; __pyx_t_23 < __pyx_t_22; __pyx_t_23+=1) {
          __pyx_v_k = __pyx_t_23;

          /* "serializor/unicode/des.py":897
 *             for j in range(s_j):
 *                 for k in range(s_k):
 *                     utils.arr_setitem_3d(arr, _des_bytes(data, pos), i, j, k)             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
          __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__des_bytes(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 897, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_3d(__pyx_v_arr, __pyx_t_6, __pyx_v_i, __pyx_v_j, __pyx_v_k); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 897, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
      }
    }

    /* "serializor/unicode/des.py":898
 *                 for k in range(s_k):
 *                     utils.arr_setitem_3d(arr, _des_bytes(data, pos), i, j, k)
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # 4-dimension
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":884
 * 
 *     # 3-dimension
 *     if ndim == 3:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":901
 * 
 *     # 4-dimension
 *     if ndim == 4:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 4);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":903
 *     if ndim == 4:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 903, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":904
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 904, __pyx_L1_error)
    __pyx_v_s_j = __pyx_t_4;

    /* "serializor/unicode/des.py":905
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_l == 0:
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 905, __pyx_L1_error)
    __pyx_v_s_k = __pyx_t_4;

    /* "serializor/unicode/des.py":906
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         if s_l == 0:
 *             return np.PyArray_EMPTY(
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 906, __pyx_L1_error)
    __pyx_v_s_l = __pyx_t_4;

    /* "serializor/unicode/des.py":907
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_l == 0:             # <<<<<<<<<<<<<<
 *             return np.PyArray_EMPTY(
 *                 ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0
 */
    __pyx_t_3 = (__pyx_v_s_l == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":908
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_l == 0:
 *             return np.PyArray_EMPTY(             # <<<<<<<<<<<<<<
 *                 ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0
 *             )  # exit: empty
 */
      __Pyx_XDECREF(__pyx_r);

      /* "serializor/unicode/des.py":909
 *         if s_l == 0:
 *             return np.PyArray_EMPTY(
 *                 ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0             # <<<<<<<<<<<<<<
 *             )  # exit: empty
 *         # . deserialize values
 */
      __pyx_t_24[0] = __pyx_v_s_i;
      __pyx_t_24[1] = __pyx_v_s_j;
      __pyx_t_24[2] = __pyx_v_s_k;
      __pyx_t_24[3] = __pyx_v_s_l;

      /* "serializor/unicode/des.py":908
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_l == 0:
 *             return np.PyArray_EMPTY(             # <<<<<<<<<<<<<<
 *                 ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0
 *             )  # exit: empty
 */
      __pyx_t_6 = PyArray_EMPTY(__pyx_v_ndim, __pyx_t_24, __pyx_v_npy_dtype, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 908, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_r = __pyx_t_6;
      __pyx_t_6 = 0;
      goto __pyx_L0;

      /* "serializor/unicode/des.py":907
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_l == 0:             # <<<<<<<<<<<<<<
 *             return np.PyArray_EMPTY(
 *                 ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0
 */
    }

    /* "serializor/unicode/des.py":912
 *             )  # exit: empty
 *         # . deserialize values
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         arr = np.empty([s_i, s_j, s_k, s_l], "S%d" % ch_size)
 *         for i in range(s_i):
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 912, __pyx_L1_error)
    __pyx_v_ch_size = __pyx_t_4;

    /* "serializor/unicode/des.py":913
 *         # . deserialize values
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.empty([s_i, s_j, s_k, s_l], "S%d" % ch_size)             # <<<<<<<<<<<<<<
 *         for i in range(s_i):
 *             for j in range(s_j):
 */
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_n_s_np); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 913, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_empty); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 913, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = PyInt_FromSsize_t(__pyx_v_s_i); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 913, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_20 = PyInt_FromSsize_t(__pyx_v_s_j); if (unlikely(!__pyx_t_20)) __PYX_ERR(1, 913, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_20);
    __pyx_t_7 = PyInt_FromSsize_t(__pyx_v_s_k); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 913, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_10 = PyInt_FromSsize_t(__pyx_v_s_l); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 913, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_25 = PyList_New(4); if (unlikely(!__pyx_t_25)) __PYX_ERR(1, 913, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_25);
    __Pyx_GIVEREF(__pyx_t_8);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 0, __pyx_t_8)) __PYX_ERR(1, 913, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_20);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 1, __pyx_t_20)) __PYX_ERR(1, 913, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_7);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 2, __pyx_t_7)) __PYX_ERR(1, 913, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_10);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 3, __pyx_t_10)) __PYX_ERR(1, 913, __pyx_L1_error);
    __pyx_t_8 = 0;
    __pyx_t_20 = 0;
    __pyx_t_7 = 0;
    __pyx_t_10 = 0;
    __pyx_t_10 = __Pyx_PyInt_From_int(__pyx_v_ch_size); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 913, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_7 = PyUnicode_Format(__pyx_kp_u_S_d, __pyx_t_10); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 913, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = NULL;
    __pyx_t_11 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
        __pyx_t_11 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_10, __pyx_t_25, __pyx_t_7};
      __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_9, __pyx_callargs+1-__pyx_t_11, 2+__pyx_t_11);
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 913, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    }
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 913, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":914
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.empty([s_i, s_j, s_k, s_l], "S%d" % ch_size)
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             for j in range(s_j):
 *                 for k in range(s_k):
 */
    __pyx_t_12 = __pyx_v_s_i;
    __pyx_t_13 = __pyx_t_12;
    for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
      __pyx_v_i = __pyx_t_14;

      /* "serializor/unicode/des.py":915
 *         arr = np.empty([s_i, s_j, s_k, s_l], "S%d" % ch_size)
 *         for i in range(s_i):
 *             for j in range(s_j):             # <<<<<<<<<<<<<<
 *                 for k in range(s_k):
 *                     for l in range(s_l):
 */
      __pyx_t_16 = __pyx_v_s_j;
      __pyx_t_17 = __pyx_t_16;
      for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
        __pyx_v_j = __pyx_t_18;

        /* "serializor/unicode/des.py":916
 *         for i in range(s_i):
 *             for j in range(s_j):
 *                 for k in range(s_k):             # <<<<<<<<<<<<<<
 *                     for l in range(s_l):
 *                         utils.arr_setitem_4d(arr, _des_bytes(data, pos), i, j, k, l)
 */
        __pyx_t_21 = __pyx_v_s_k;
        __pyx_t_22 = __pyx_t_21;
        for (__pyx_t_23 = 0; __pyx_t_23 < __pyx_t_22; __pyx_t_23+=1) {
          __pyx_v_k = __pyx_t_23;

          /* "serializor/unicode/des.py":917
 *             for j in range(s_j):
 *                 for k in range(s_k):
 *                     for l in range(s_l):             # <<<<<<<<<<<<<<
 *                         utils.arr_setitem_4d(arr, _des_bytes(data, pos), i, j, k, l)
 *         return arr
 */
          __pyx_t_26 = __pyx_v_s_l;
          __pyx_t_27 = __pyx_t_26;
          for (__pyx_t_28 = 0; __pyx_t_28 < __pyx_t_27; __pyx_t_28+=1) {
            __pyx_v_l = __pyx_t_28;

            /* "serializor/unicode/des.py":918
 *                 for k in range(s_k):
 *                     for l in range(s_l):
 *                         utils.arr_setitem_4d(arr, _des_bytes(data, pos), i, j, k, l)             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
            __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__des_bytes(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 918, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_6);
            __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_4d(__pyx_v_arr, __pyx_t_6, __pyx_v_i, __pyx_v_j, __pyx_v_k, __pyx_v_l); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 918, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          }
        }
      }
    }

    /* "serializor/unicode/des.py":919
 *                     for l in range(s_l):
 *                         utils.arr_setitem_4d(arr, _des_bytes(data, pos), i, j, k, l)
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # Invalid
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":901
 * 
 *     # 4-dimension
 *     if ndim == 4:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":922
 * 
 *     # Invalid
 *     raise ValueError("unsupported <'numpy.ndarray'> dimension [%d]." % ndim)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_6 = __Pyx_PyInt_From_unsigned_char(__pyx_v_ndim); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 922, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_9 = PyUnicode_Format(__pyx_kp_u_unsupported_numpy_ndarray_dimens, __pyx_t_6); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 922, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_9); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 922, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_Raise(__pyx_t_6, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __PYX_ERR(1, 922, __pyx_L1_error)

  /* "serializor/unicode/des.py":841
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_ndarray_bytes(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_20);
  __Pyx_XDECREF(__pyx_t_25);
  __Pyx_AddTraceback("serializor.unicode.des._des_ndarray_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_arr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":925
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_ndarray_unicode(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_ndarray_unicode(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  Py_ssize_t __pyx_v_idx;
  unsigned char __pyx_v_npy_dtype;
  unsigned char __pyx_v_ndim;
  PyArrayObject *__pyx_v_arr = 0;
  Py_ssize_t __pyx_v_s_i;
  int __pyx_v_ch_size;
  unsigned PY_LONG_LONG __pyx_v_values_len;
  Py_ssize_t __pyx_v_end;
  PyObject *__pyx_v_values_json = 0;
  PyObject *__pyx_v_values = NULL;
  Py_ssize_t __pyx_v_i;
  Py_ssize_t __pyx_v_s_j;
  Py_ssize_t __pyx_v_j;
  Py_ssize_t __pyx_v_s_k;
  Py_ssize_t __pyx_v_k;
  Py_ssize_t __pyx_v_s_l;
  Py_ssize_t __pyx_v_l;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  unsigned char __pyx_t_1;
  long __pyx_t_2;
  int __pyx_t_3;
  unsigned PY_LONG_LONG __pyx_t_4;
  npy_intp __pyx_t_5[1];
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  unsigned int __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  npy_intp __pyx_t_15[2];
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  npy_intp __pyx_t_19[3];
  PyObject *__pyx_t_20 = NULL;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  npy_intp __pyx_t_24[4];
  PyObject *__pyx_t_25 = NULL;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_ndarray_unicode", 1);

  /* "serializor/unicode/des.py":934
 *     Position updates to the next charactor after deserialization.
 *     """
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip array dtype             # <<<<<<<<<<<<<<
 *     npy_dtype = unpack_uint8(data, idx)  # type: ignore
 *     ndim: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore
 */
  __pyx_v_idx = ((__pyx_v_pos[0]) + 1);

  /* "serializor/unicode/des.py":935
 *     """
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip array dtype
 *     npy_dtype = unpack_uint8(data, idx)  # type: ignore             # <<<<<<<<<<<<<<
 *     ndim: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore
 *     pos[0] += 3  # set new position
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, __pyx_v_idx); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 935, __pyx_L1_error)
  __pyx_v_npy_dtype = __pyx_t_1;

  /* "serializor/unicode/des.py":936
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip array dtype
 *     npy_dtype = unpack_uint8(data, idx)  # type: ignore
 *     ndim: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore             # <<<<<<<<<<<<<<
 *     pos[0] += 3  # set new position
 *     arr: np.ndarray
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_idx + 1)); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 936, __pyx_L1_error)
  __pyx_v_ndim = __pyx_t_1;

  /* "serializor/unicode/des.py":937
 *     npy_dtype = unpack_uint8(data, idx)  # type: ignore
 *     ndim: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore
 *     pos[0] += 3  # set new position             # <<<<<<<<<<<<<<
 *     arr: np.ndarray
 *     # 1-dimension
 */
  __pyx_t_2 = 0;
  (__pyx_v_pos[__pyx_t_2]) = ((__pyx_v_pos[__pyx_t_2]) + 3);

  /* "serializor/unicode/des.py":940
 *     arr: np.ndarray
 *     # 1-dimension
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 1);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":942
 *     if ndim == 1:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         if s_i == 0:
 *             return np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)  # exit: empty
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 942, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":943
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_i == 0:             # <<<<<<<<<<<<<<
 *             return np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)  # exit: empty
 *         # . deserialize values
 */
    __pyx_t_3 = (__pyx_v_s_i == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":944
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_i == 0:
 *             return np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)  # exit: empty             # <<<<<<<<<<<<<<
 *         # . deserialize values
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_5[0] = __pyx_v_s_i;
      __pyx_t_6 = PyArray_EMPTY(__pyx_v_ndim, __pyx_t_5, __pyx_v_npy_dtype, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 944, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_r = __pyx_t_6;
      __pyx_t_6 = 0;
      goto __pyx_L0;

      /* "serializor/unicode/des.py":943
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_i == 0:             # <<<<<<<<<<<<<<
 *             return np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)  # exit: empty
 *         # . deserialize values
 */
    }

    /* "serializor/unicode/des.py":946
 *             return np.PyArray_EMPTY(ndim, [s_i], npy_dtype, 0)  # exit: empty
 *         # . deserialize values
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         arr = np.empty(s_i, "U%d" % ch_size)
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 946, __pyx_L1_error)
    __pyx_v_ch_size = __pyx_t_4;

    /* "serializor/unicode/des.py":947
 *         # . deserialize values
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.empty(s_i, "U%d" % ch_size)             # <<<<<<<<<<<<<<
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]
 */
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_np); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 947, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_empty); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 947, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = PyInt_FromSsize_t(__pyx_v_s_i); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 947, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = __Pyx_PyInt_From_int(__pyx_v_ch_size); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 947, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = PyUnicode_Format(__pyx_kp_u_U_d, __pyx_t_9); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 947, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = NULL;
    __pyx_t_11 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
        __pyx_t_11 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_9, __pyx_t_7, __pyx_t_10};
      __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_8, __pyx_callargs+1-__pyx_t_11, 2+__pyx_t_11);
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 947, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 947, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":948
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.empty(s_i, "U%d" % ch_size)
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 948, __pyx_L1_error)
    __pyx_v_values_len = __pyx_t_4;

    /* "serializor/unicode/des.py":949
 *         arr = np.empty(s_i, "U%d" % ch_size)
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 */
    __pyx_v_idx = (__pyx_v_pos[0]);

    /* "serializor/unicode/des.py":950
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len             # <<<<<<<<<<<<<<
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 */
    __pyx_v_end = (__pyx_v_idx + __pyx_v_values_len);

    /* "serializor/unicode/des.py":951
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]             # <<<<<<<<<<<<<<
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 */
    __pyx_t_6 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 951, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_v_values_json = ((PyObject*)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":952
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 */
    (__pyx_v_pos[0]) = __pyx_v_end;

    /* "serializor/unicode/des.py":953
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))             # <<<<<<<<<<<<<<
 *         for i in range(s_i):
 *             utils.arr_setitem_1d(arr, next(values), i)
 */
    __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__orjson_loads(__pyx_v_values_json); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 953, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 953, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_values = __pyx_t_8;
    __pyx_t_8 = 0;

    /* "serializor/unicode/des.py":954
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             utils.arr_setitem_1d(arr, next(values), i)
 *         return arr
 */
    __pyx_t_12 = __pyx_v_s_i;
    __pyx_t_13 = __pyx_t_12;
    for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
      __pyx_v_i = __pyx_t_14;

      /* "serializor/unicode/des.py":955
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 *             utils.arr_setitem_1d(arr, next(values), i)             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
      __pyx_t_8 = __Pyx_PyIter_Next(__pyx_v_values); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 955, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_1d(__pyx_v_arr, __pyx_t_8, __pyx_v_i); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 955, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }

    /* "serializor/unicode/des.py":956
 *         for i in range(s_i):
 *             utils.arr_setitem_1d(arr, next(values), i)
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # 2-dimension
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":940
 *     arr: np.ndarray
 *     # 1-dimension
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":959
 * 
 *     # 2-dimension
 *     if ndim == 2:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 2);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":961
 *     if ndim == 2:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_j == 0:
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 961, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":962
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         if s_j == 0:
 *             return np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)  # exit: empty
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 962, __pyx_L1_error)
    __pyx_v_s_j = __pyx_t_4;

    /* "serializor/unicode/des.py":963
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_j == 0:             # <<<<<<<<<<<<<<
 *             return np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)  # exit: empty
 *         # . deserialize values
 */
    __pyx_t_3 = (__pyx_v_s_j == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":964
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_j == 0:
 *             return np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)  # exit: empty             # <<<<<<<<<<<<<<
 *         # . deserialize values
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_15[0] = __pyx_v_s_i;
      __pyx_t_15[1] = __pyx_v_s_j;
      __pyx_t_8 = PyArray_EMPTY(__pyx_v_ndim, __pyx_t_15, __pyx_v_npy_dtype, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 964, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_r = __pyx_t_8;
      __pyx_t_8 = 0;
      goto __pyx_L0;

      /* "serializor/unicode/des.py":963
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_j == 0:             # <<<<<<<<<<<<<<
 *             return np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)  # exit: empty
 *         # . deserialize values
 */
    }

    /* "serializor/unicode/des.py":966
 *             return np.PyArray_EMPTY(ndim, [s_i, s_j], npy_dtype, 0)  # exit: empty
 *         # . deserialize values
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         arr = np.empty([s_i, s_j], "U%d" % ch_size)
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 966, __pyx_L1_error)
    __pyx_v_ch_size = __pyx_t_4;

    /* "serializor/unicode/des.py":967
 *         # . deserialize values
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.empty([s_i, s_j], "U%d" % ch_size)             # <<<<<<<<<<<<<<
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]
 */
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_np); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 967, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_empty); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 967, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = PyInt_FromSsize_t(__pyx_v_s_i); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 967, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = PyInt_FromSsize_t(__pyx_v_s_j); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 967, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = PyList_New(2); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 967, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_6);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 0, __pyx_t_6)) __PYX_ERR(1, 967, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_7);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_9, 1, __pyx_t_7)) __PYX_ERR(1, 967, __pyx_L1_error);
    __pyx_t_6 = 0;
    __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_ch_size); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 967, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = PyUnicode_Format(__pyx_kp_u_U_d, __pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 967, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = NULL;
    __pyx_t_11 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_10))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_10);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_10, function);
        __pyx_t_11 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_7, __pyx_t_9, __pyx_t_6};
      __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_10, __pyx_callargs+1-__pyx_t_11, 2+__pyx_t_11);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 967, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    }
    if (!(likely(((__pyx_t_8) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_8, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 967, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_8);
    __pyx_t_8 = 0;

    /* "serializor/unicode/des.py":968
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.empty([s_i, s_j], "U%d" % ch_size)
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 968, __pyx_L1_error)
    __pyx_v_values_len = __pyx_t_4;

    /* "serializor/unicode/des.py":969
 *         arr = np.empty([s_i, s_j], "U%d" % ch_size)
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 */
    __pyx_v_idx = (__pyx_v_pos[0]);

    /* "serializor/unicode/des.py":970
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len             # <<<<<<<<<<<<<<
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 */
    __pyx_v_end = (__pyx_v_idx + __pyx_v_values_len);

    /* "serializor/unicode/des.py":971
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]             # <<<<<<<<<<<<<<
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 */
    __pyx_t_8 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 971, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_v_values_json = ((PyObject*)__pyx_t_8);
    __pyx_t_8 = 0;

    /* "serializor/unicode/des.py":972
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 */
    (__pyx_v_pos[0]) = __pyx_v_end;

    /* "serializor/unicode/des.py":973
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))             # <<<<<<<<<<<<<<
 *         for i in range(s_i):
 *             for j in range(s_j):
 */
    __pyx_t_8 = __pyx_f_10serializor_7unicode_3des__orjson_loads(__pyx_v_values_json); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 973, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = PyObject_GetIter(__pyx_t_8); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 973, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_v_values = __pyx_t_10;
    __pyx_t_10 = 0;

    /* "serializor/unicode/des.py":974
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             for j in range(s_j):
 *                 utils.arr_setitem_2d(arr, next(values), i, j)
 */
    __pyx_t_12 = __pyx_v_s_i;
    __pyx_t_13 = __pyx_t_12;
    for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
      __pyx_v_i = __pyx_t_14;

      /* "serializor/unicode/des.py":975
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 *             for j in range(s_j):             # <<<<<<<<<<<<<<
 *                 utils.arr_setitem_2d(arr, next(values), i, j)
 *         return arr
 */
      __pyx_t_16 = __pyx_v_s_j;
      __pyx_t_17 = __pyx_t_16;
      for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
        __pyx_v_j = __pyx_t_18;

        /* "serializor/unicode/des.py":976
 *         for i in range(s_i):
 *             for j in range(s_j):
 *                 utils.arr_setitem_2d(arr, next(values), i, j)             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
        __pyx_t_10 = __Pyx_PyIter_Next(__pyx_v_values); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 976, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_2d(__pyx_v_arr, __pyx_t_10, __pyx_v_i, __pyx_v_j); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 976, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      }
    }

    /* "serializor/unicode/des.py":977
 *             for j in range(s_j):
 *                 utils.arr_setitem_2d(arr, next(values), i, j)
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # 3-dimension
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":959
 * 
 *     # 2-dimension
 *     if ndim == 2:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":980
 * 
 *     # 3-dimension
 *     if ndim == 3:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 3);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":982
 *     if ndim == 3:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 982, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":983
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_k == 0:
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 983, __pyx_L1_error)
    __pyx_v_s_j = __pyx_t_4;

    /* "serializor/unicode/des.py":984
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         if s_k == 0:
 *             return np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)  # exit: empty
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 984, __pyx_L1_error)
    __pyx_v_s_k = __pyx_t_4;

    /* "serializor/unicode/des.py":985
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_k == 0:             # <<<<<<<<<<<<<<
 *             return np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)  # exit: empty
 *         # . deserialize values
 */
    __pyx_t_3 = (__pyx_v_s_k == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":986
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_k == 0:
 *             return np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)  # exit: empty             # <<<<<<<<<<<<<<
 *         # . deserialize values
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_19[0] = __pyx_v_s_i;
      __pyx_t_19[1] = __pyx_v_s_j;
      __pyx_t_19[2] = __pyx_v_s_k;
      __pyx_t_10 = PyArray_EMPTY(__pyx_v_ndim, __pyx_t_19, __pyx_v_npy_dtype, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 986, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_r = __pyx_t_10;
      __pyx_t_10 = 0;
      goto __pyx_L0;

      /* "serializor/unicode/des.py":985
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_k == 0:             # <<<<<<<<<<<<<<
 *             return np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)  # exit: empty
 *         # . deserialize values
 */
    }

    /* "serializor/unicode/des.py":988
 *             return np.PyArray_EMPTY(ndim, [s_i, s_j, s_k], npy_dtype, 0)  # exit: empty
 *         # . deserialize values
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         arr = np.empty([s_i, s_j, s_k], "U%d" % ch_size)
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 988, __pyx_L1_error)
    __pyx_v_ch_size = __pyx_t_4;

    /* "serializor/unicode/des.py":989
 *         # . deserialize values
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.empty([s_i, s_j, s_k], "U%d" % ch_size)             # <<<<<<<<<<<<<<
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]
 */
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_n_s_np); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 989, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_empty); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 989, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = PyInt_FromSsize_t(__pyx_v_s_i); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 989, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_v_s_j); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 989, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_7 = PyInt_FromSsize_t(__pyx_v_s_k); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 989, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_20 = PyList_New(3); if (unlikely(!__pyx_t_20)) __PYX_ERR(1, 989, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_20);
    __Pyx_GIVEREF(__pyx_t_8);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 0, __pyx_t_8)) __PYX_ERR(1, 989, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_9);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 1, __pyx_t_9)) __PYX_ERR(1, 989, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_7);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_20, 2, __pyx_t_7)) __PYX_ERR(1, 989, __pyx_L1_error);
    __pyx_t_8 = 0;
    __pyx_t_9 = 0;
    __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_ch_size); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 989, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = PyUnicode_Format(__pyx_kp_u_U_d, __pyx_t_7); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 989, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = NULL;
    __pyx_t_11 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
        __pyx_t_11 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_7, __pyx_t_20, __pyx_t_9};
      __pyx_t_10 = __Pyx_PyObject_FastCall(__pyx_t_6, __pyx_callargs+1-__pyx_t_11, 2+__pyx_t_11);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 989, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
    if (!(likely(((__pyx_t_10) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_10, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 989, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_10);
    __pyx_t_10 = 0;

    /* "serializor/unicode/des.py":990
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.empty([s_i, s_j, s_k], "U%d" % ch_size)
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 990, __pyx_L1_error)
    __pyx_v_values_len = __pyx_t_4;

    /* "serializor/unicode/des.py":991
 *         arr = np.empty([s_i, s_j, s_k], "U%d" % ch_size)
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 */
    __pyx_v_idx = (__pyx_v_pos[0]);

    /* "serializor/unicode/des.py":992
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len             # <<<<<<<<<<<<<<
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 */
    __pyx_v_end = (__pyx_v_idx + __pyx_v_values_len);

    /* "serializor/unicode/des.py":993
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]             # <<<<<<<<<<<<<<
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 */
    __pyx_t_10 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 993, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_v_values_json = ((PyObject*)__pyx_t_10);
    __pyx_t_10 = 0;

    /* "serializor/unicode/des.py":994
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 */
    (__pyx_v_pos[0]) = __pyx_v_end;

    /* "serializor/unicode/des.py":995
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))             # <<<<<<<<<<<<<<
 *         for i in range(s_i):
 *             for j in range(s_j):
 */
    __pyx_t_10 = __pyx_f_10serializor_7unicode_3des__orjson_loads(__pyx_v_values_json); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 995, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_6 = PyObject_GetIter(__pyx_t_10); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 995, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_v_values = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":996
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             for j in range(s_j):
 *                 for k in range(s_k):
 */
    __pyx_t_12 = __pyx_v_s_i;
    __pyx_t_13 = __pyx_t_12;
    for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
      __pyx_v_i = __pyx_t_14;

      /* "serializor/unicode/des.py":997
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 *             for j in range(s_j):             # <<<<<<<<<<<<<<
 *                 for k in range(s_k):
 *                     utils.arr_setitem_3d(arr, next(values), i, j, k)
 */
      __pyx_t_16 = __pyx_v_s_j;
      __pyx_t_17 = __pyx_t_16;
      for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
        __pyx_v_j = __pyx_t_18;

        /* "serializor/unicode/des.py":998
 *         for i in range(s_i):
 *             for j in range(s_j):
 *                 for k in range(s_k):             # <<<<<<<<<<<<<<
 *                     utils.arr_setitem_3d(arr, next(values), i, j, k)
 *         return arr
 */
        __pyx_t_21 = __pyx_v_s_k;
        __pyx_t_22 = __pyx_t_21;
        for (__pyx_t_23 = 0; __pyx_t_23 < __pyx_t_22; __pyx_t_23+=1) {
          __pyx_v_k = __pyx_t_23;

          /* "serializor/unicode/des.py":999
 *             for j in range(s_j):
 *                 for k in range(s_k):
 *                     utils.arr_setitem_3d(arr, next(values), i, j, k)             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
          __pyx_t_6 = __Pyx_PyIter_Next(__pyx_v_values); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 999, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_3d(__pyx_v_arr, __pyx_t_6, __pyx_v_i, __pyx_v_j, __pyx_v_k); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 999, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
      }
    }

    /* "serializor/unicode/des.py":1000
 *                 for k in range(s_k):
 *                     utils.arr_setitem_3d(arr, next(values), i, j, k)
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # 4-dimension
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":980
 * 
 *     # 3-dimension
 *     if ndim == 3:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":1003
 * 
 *     # 4-dimension
 *     if ndim == 4:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  __pyx_t_3 = (__pyx_v_ndim == 4);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":1005
 *     if ndim == 4:
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 1005, __pyx_L1_error)
    __pyx_v_s_i = __pyx_t_4;

    /* "serializor/unicode/des.py":1006
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 1006, __pyx_L1_error)
    __pyx_v_s_j = __pyx_t_4;

    /* "serializor/unicode/des.py":1007
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_l == 0:
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 1007, __pyx_L1_error)
    __pyx_v_s_k = __pyx_t_4;

    /* "serializor/unicode/des.py":1008
 *         s_j: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         if s_l == 0:
 *             return np.PyArray_EMPTY(
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 1008, __pyx_L1_error)
    __pyx_v_s_l = __pyx_t_4;

    /* "serializor/unicode/des.py":1009
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_l == 0:             # <<<<<<<<<<<<<<
 *             return np.PyArray_EMPTY(
 *                 ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0
 */
    __pyx_t_3 = (__pyx_v_s_l == 0);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":1010
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_l == 0:
 *             return np.PyArray_EMPTY(             # <<<<<<<<<<<<<<
 *                 ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0
 *             )  # exit: empty
 */
      __Pyx_XDECREF(__pyx_r);

      /* "serializor/unicode/des.py":1011
 *         if s_l == 0:
 *             return np.PyArray_EMPTY(
 *                 ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0             # <<<<<<<<<<<<<<
 *             )  # exit: empty
 *         # . deserialize values
 */
      __pyx_t_24[0] = __pyx_v_s_i;
      __pyx_t_24[1] = __pyx_v_s_j;
      __pyx_t_24[2] = __pyx_v_s_k;
      __pyx_t_24[3] = __pyx_v_s_l;

      /* "serializor/unicode/des.py":1010
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_l == 0:
 *             return np.PyArray_EMPTY(             # <<<<<<<<<<<<<<
 *                 ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0
 *             )  # exit: empty
 */
      __pyx_t_6 = PyArray_EMPTY(__pyx_v_ndim, __pyx_t_24, __pyx_v_npy_dtype, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1010, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_r = __pyx_t_6;
      __pyx_t_6 = 0;
      goto __pyx_L0;

      /* "serializor/unicode/des.py":1009
 *         s_k: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         s_l: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 *         if s_l == 0:             # <<<<<<<<<<<<<<
 *             return np.PyArray_EMPTY(
 *                 ndim, [s_i, s_j, s_k, s_l], npy_dtype, 0
 */
    }

    /* "serializor/unicode/des.py":1014
 *             )  # exit: empty
 *         # . deserialize values
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         arr = np.empty([s_i, s_j, s_k, s_l], "U%d" % ch_size)
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 1014, __pyx_L1_error)
    __pyx_v_ch_size = __pyx_t_4;

    /* "serializor/unicode/des.py":1015
 *         # . deserialize values
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.empty([s_i, s_j, s_k, s_l], "U%d" % ch_size)             # <<<<<<<<<<<<<<
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]
 */
    __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_np); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 1015, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_empty); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 1015, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = PyInt_FromSsize_t(__pyx_v_s_i); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 1015, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_20 = PyInt_FromSsize_t(__pyx_v_s_j); if (unlikely(!__pyx_t_20)) __PYX_ERR(1, 1015, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_20);
    __pyx_t_7 = PyInt_FromSsize_t(__pyx_v_s_k); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 1015, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = PyInt_FromSsize_t(__pyx_v_s_l); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 1015, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_25 = PyList_New(4); if (unlikely(!__pyx_t_25)) __PYX_ERR(1, 1015, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_25);
    __Pyx_GIVEREF(__pyx_t_10);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 0, __pyx_t_10)) __PYX_ERR(1, 1015, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_20);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 1, __pyx_t_20)) __PYX_ERR(1, 1015, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_7);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 2, __pyx_t_7)) __PYX_ERR(1, 1015, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_8);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_25, 3, __pyx_t_8)) __PYX_ERR(1, 1015, __pyx_L1_error);
    __pyx_t_10 = 0;
    __pyx_t_20 = 0;
    __pyx_t_7 = 0;
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_v_ch_size); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 1015, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_7 = PyUnicode_Format(__pyx_kp_u_U_d, __pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 1015, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = NULL;
    __pyx_t_11 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
        __pyx_t_11 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_8, __pyx_t_25, __pyx_t_7};
      __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_9, __pyx_callargs+1-__pyx_t_11, 2+__pyx_t_11);
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1015, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    }
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5numpy_ndarray))))) __PYX_ERR(1, 1015, __pyx_L1_error)
    __pyx_v_arr = ((PyArrayObject *)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":1016
 *         ch_size: cython.int = dec_encoded_int(data, pos)  # type: ignore
 *         arr = np.empty([s_i, s_j, s_k, s_l], "U%d" % ch_size)
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 */
    __pyx_t_4 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_4 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 1016, __pyx_L1_error)
    __pyx_v_values_len = __pyx_t_4;

    /* "serializor/unicode/des.py":1017
 *         arr = np.empty([s_i, s_j, s_k, s_l], "U%d" % ch_size)
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 */
    __pyx_v_idx = (__pyx_v_pos[0]);

    /* "serializor/unicode/des.py":1018
 *         values_len: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len             # <<<<<<<<<<<<<<
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 */
    __pyx_v_end = (__pyx_v_idx + __pyx_v_values_len);

    /* "serializor/unicode/des.py":1019
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]             # <<<<<<<<<<<<<<
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 */
    __pyx_t_6 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1019, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_v_values_json = ((PyObject*)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":1020
 *         end: cython.Py_ssize_t = idx + values_len
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 */
    (__pyx_v_pos[0]) = __pyx_v_end;

    /* "serializor/unicode/des.py":1021
 *         values_json: str = data[idx:end]
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))             # <<<<<<<<<<<<<<
 *         for i in range(s_i):
 *             for j in range(s_j):
 */
    __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__orjson_loads(__pyx_v_values_json); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1021, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_9 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 1021, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_values = __pyx_t_9;
    __pyx_t_9 = 0;

    /* "serializor/unicode/des.py":1022
 *         pos[0] = end  # set new position
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):             # <<<<<<<<<<<<<<
 *             for j in range(s_j):
 *                 for k in range(s_k):
 */
    __pyx_t_12 = __pyx_v_s_i;
    __pyx_t_13 = __pyx_t_12;
    for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
      __pyx_v_i = __pyx_t_14;

      /* "serializor/unicode/des.py":1023
 *         values = iter(_orjson_loads(values_json))
 *         for i in range(s_i):
 *             for j in range(s_j):             # <<<<<<<<<<<<<<
 *                 for k in range(s_k):
 *                     for l in range(s_l):
 */
      __pyx_t_16 = __pyx_v_s_j;
      __pyx_t_17 = __pyx_t_16;
      for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
        __pyx_v_j = __pyx_t_18;

        /* "serializor/unicode/des.py":1024
 *         for i in range(s_i):
 *             for j in range(s_j):
 *                 for k in range(s_k):             # <<<<<<<<<<<<<<
 *                     for l in range(s_l):
 *                         utils.arr_setitem_4d(arr, next(values), i, j, k, l)
 */
        __pyx_t_21 = __pyx_v_s_k;
        __pyx_t_22 = __pyx_t_21;
        for (__pyx_t_23 = 0; __pyx_t_23 < __pyx_t_22; __pyx_t_23+=1) {
          __pyx_v_k = __pyx_t_23;

          /* "serializor/unicode/des.py":1025
 *             for j in range(s_j):
 *                 for k in range(s_k):
 *                     for l in range(s_l):             # <<<<<<<<<<<<<<
 *                         utils.arr_setitem_4d(arr, next(values), i, j, k, l)
 *         return arr
 */
          __pyx_t_26 = __pyx_v_s_l;
          __pyx_t_27 = __pyx_t_26;
          for (__pyx_t_28 = 0; __pyx_t_28 < __pyx_t_27; __pyx_t_28+=1) {
            __pyx_v_l = __pyx_t_28;

            /* "serializor/unicode/des.py":1026
 *                 for k in range(s_k):
 *                     for l in range(s_l):
 *                         utils.arr_setitem_4d(arr, next(values), i, j, k, l)             # <<<<<<<<<<<<<<
 *         return arr
 * 
 */
            __pyx_t_9 = __Pyx_PyIter_Next(__pyx_v_values); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 1026, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_9);
            __pyx_t_3 = __pyx_f_10serializor_5utils_arr_setitem_4d(__pyx_v_arr, __pyx_t_9, __pyx_v_i, __pyx_v_j, __pyx_v_k, __pyx_v_l); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 1026, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          }
        }
      }
    }

    /* "serializor/unicode/des.py":1027
 *                     for l in range(s_l):
 *                         utils.arr_setitem_4d(arr, next(values), i, j, k, l)
 *         return arr             # <<<<<<<<<<<<<<
 * 
 *     # Invalid
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_arr);
    __pyx_r = ((PyObject *)__pyx_v_arr);
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1003
 * 
 *     # 4-dimension
 *     if ndim == 4:             # <<<<<<<<<<<<<<
 *         # . array shape
 *         s_i: cython.Py_ssize_t = dec_encoded_int(data, pos)  # type: ignore
 */
  }

  /* "serializor/unicode/des.py":1030
 * 
 *     # Invalid
 *     raise ValueError("unsupported <'numpy.ndarray'> dimension [%d]." % ndim)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_9 = __Pyx_PyInt_From_unsigned_char(__pyx_v_ndim); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 1030, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_6 = PyUnicode_Format(__pyx_kp_u_unsupported_numpy_ndarray_dimens, __pyx_t_9); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1030, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_6); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 1030, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_Raise(__pyx_t_9, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __PYX_ERR(1, 1030, __pyx_L1_error)

  /* "serializor/unicode/des.py":925
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_ndarray_unicode(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_20);
  __Pyx_XDECREF(__pyx_t_25);
  __Pyx_AddTraceback("serializor.unicode.des._des_ndarray_unicode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_arr);
  __Pyx_XDECREF(__pyx_v_values_json);
  __Pyx_XDECREF(__pyx_v_values);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":1034
 * 
 * # Pandas Types --------------------------------------------------------------------------------
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_pd_timestamp(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_pd_timestamp(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  Py_ssize_t __pyx_v_idx;
  unsigned char __pyx_v_flag;
  PY_LONG_LONG __pyx_v_val;
  unsigned PY_LONG_LONG __pyx_v_size;
  unsigned PY_LONG_LONG __pyx_v_end;
  PyObject *__pyx_v_tz_name = 0;
  PyDateTime_Delta *__pyx_v_offset = NULL;
  PyObject *__pyx_v_tzinfo = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  unsigned char __pyx_t_1;
  PY_LONG_LONG __pyx_t_2;
  long __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  unsigned int __pyx_t_8;
  unsigned PY_LONG_LONG __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  struct __pyx_opt_args_7cpython_8datetime_timezone_new __pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_pd_timestamp", 1);

  /* "serializor/unicode/des.py":1041
 *     Position updates to the next byte after deserialization.
 *     """
 *     idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *     flag: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore
 *     val: cython.longlong = unpack_int64(data, idx + 2)  # type: ignore
 */
  __pyx_v_idx = (__pyx_v_pos[0]);

  /* "serializor/unicode/des.py":1042
 *     """
 *     idx: cython.Py_ssize_t = pos[0]
 *     flag: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore             # <<<<<<<<<<<<<<
 *     val: cython.longlong = unpack_int64(data, idx + 2)  # type: ignore
 *     # Without timezone
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_idx + 1)); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 1042, __pyx_L1_error)
  __pyx_v_flag = __pyx_t_1;

  /* "serializor/unicode/des.py":1043
 *     idx: cython.Py_ssize_t = pos[0]
 *     flag: cython.uchar = unpack_uint8(data, idx + 1)  # type: ignore
 *     val: cython.longlong = unpack_int64(data, idx + 2)  # type: ignore             # <<<<<<<<<<<<<<
 *     # Without timezone
 *     if flag == 0:
 */
  __pyx_t_2 = __pyx_f_10serializor_7unicode_3des_unpack_int64(__pyx_v_data, (__pyx_v_idx + 2)); if (unlikely(__pyx_t_2 == ((PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 1043, __pyx_L1_error)
  __pyx_v_val = __pyx_t_2;

  /* "serializor/unicode/des.py":1045
 *     val: cython.longlong = unpack_int64(data, idx + 2)  # type: ignore
 *     # Without timezone
 *     if flag == 0:             # <<<<<<<<<<<<<<
 *         pos[0] += 10  # set new position
 *         return typeref.PD_TIMESTAMP(val)
 */
  switch (__pyx_v_flag) {
    case 0:

    /* "serializor/unicode/des.py":1046
 *     # Without timezone
 *     if flag == 0:
 *         pos[0] += 10  # set new position             # <<<<<<<<<<<<<<
 *         return typeref.PD_TIMESTAMP(val)
 *     # Timezone: approach 1 (tz_name)
 */
    __pyx_t_3 = 0;
    (__pyx_v_pos[__pyx_t_3]) = ((__pyx_v_pos[__pyx_t_3]) + 10);

    /* "serializor/unicode/des.py":1047
 *     if flag == 0:
 *         pos[0] += 10  # set new position
 *         return typeref.PD_TIMESTAMP(val)             # <<<<<<<<<<<<<<
 *     # Timezone: approach 1 (tz_name)
 *     elif flag == 1:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_5 = __Pyx_PyInt_From_PY_LONG_LONG(__pyx_v_val); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1047, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v_10serializor_7typeref_PD_TIMESTAMP);
    __pyx_t_6 = __pyx_v_10serializor_7typeref_PD_TIMESTAMP; __pyx_t_7 = NULL;
    __pyx_t_8 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
        __pyx_t_8 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_t_5};
      __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_6, __pyx_callargs+1-__pyx_t_8, 1+__pyx_t_8);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1047, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1045
 *     val: cython.longlong = unpack_int64(data, idx + 2)  # type: ignore
 *     # Without timezone
 *     if flag == 0:             # <<<<<<<<<<<<<<
 *         pos[0] += 10  # set new position
 *         return typeref.PD_TIMESTAMP(val)
 */
    break;
    case 1:

    /* "serializor/unicode/des.py":1050
 *     # Timezone: approach 1 (tz_name)
 *     elif flag == 1:
 *         pos[0] += 10  # set new position             # <<<<<<<<<<<<<<
 *         size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx = pos[0]
 */
    __pyx_t_3 = 0;
    (__pyx_v_pos[__pyx_t_3]) = ((__pyx_v_pos[__pyx_t_3]) + 10);

    /* "serializor/unicode/des.py":1051
 *     elif flag == 1:
 *         pos[0] += 10  # set new position
 *         size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         idx = pos[0]
 *         end: cython.ulonglong = idx + size
 */
    __pyx_t_9 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_9 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 1051, __pyx_L1_error)
    __pyx_v_size = __pyx_t_9;

    /* "serializor/unicode/des.py":1052
 *         pos[0] += 10  # set new position
 *         size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx = pos[0]             # <<<<<<<<<<<<<<
 *         end: cython.ulonglong = idx + size
 *         tz_name: str = data[idx:end]
 */
    __pyx_v_idx = (__pyx_v_pos[0]);

    /* "serializor/unicode/des.py":1053
 *         size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx = pos[0]
 *         end: cython.ulonglong = idx + size             # <<<<<<<<<<<<<<
 *         tz_name: str = data[idx:end]
 *         pos[0] = end  # set new position
 */
    __pyx_v_end = (__pyx_v_idx + __pyx_v_size);

    /* "serializor/unicode/des.py":1054
 *         idx = pos[0]
 *         end: cython.ulonglong = idx + size
 *         tz_name: str = data[idx:end]             # <<<<<<<<<<<<<<
 *         pos[0] = end  # set new position
 *         return typeref.PD_TIMESTAMP(val, tz=tz_name)
 */
    __pyx_t_4 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1054, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_v_tz_name = ((PyObject*)__pyx_t_4);
    __pyx_t_4 = 0;

    /* "serializor/unicode/des.py":1055
 *         end: cython.ulonglong = idx + size
 *         tz_name: str = data[idx:end]
 *         pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *         return typeref.PD_TIMESTAMP(val, tz=tz_name)
 *     # Timezone: approach 2 (tz_offset)
 */
    (__pyx_v_pos[0]) = __pyx_v_end;

    /* "serializor/unicode/des.py":1056
 *         tz_name: str = data[idx:end]
 *         pos[0] = end  # set new position
 *         return typeref.PD_TIMESTAMP(val, tz=tz_name)             # <<<<<<<<<<<<<<
 *     # Timezone: approach 2 (tz_offset)
 *     else:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyInt_From_PY_LONG_LONG(__pyx_v_val); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1056, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1056, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4)) __PYX_ERR(1, 1056, __pyx_L1_error);
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1056, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_tz, __pyx_v_tz_name) < 0) __PYX_ERR(1, 1056, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_v_10serializor_7typeref_PD_TIMESTAMP, __pyx_t_6, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1056, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1049
 *         return typeref.PD_TIMESTAMP(val)
 *     # Timezone: approach 1 (tz_name)
 *     elif flag == 1:             # <<<<<<<<<<<<<<
 *         pos[0] += 10  # set new position
 *         size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 */
    break;
    default:

    /* "serializor/unicode/des.py":1060
 *     else:
 *         offset = datetime.timedelta_new(
 *             unpack_int32(data, idx + 10),  # type: ignore             # <<<<<<<<<<<<<<
 *             unpack_int32(data, idx + 14),  # type: ignore
 *             unpack_int32(data, idx + 18),  # type: ignore
 */
    __pyx_t_10 = __pyx_f_10serializor_7unicode_3des_unpack_int32(__pyx_v_data, (__pyx_v_idx + 10)); if (unlikely(__pyx_t_10 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 1060, __pyx_L1_error)

    /* "serializor/unicode/des.py":1061
 *         offset = datetime.timedelta_new(
 *             unpack_int32(data, idx + 10),  # type: ignore
 *             unpack_int32(data, idx + 14),  # type: ignore             # <<<<<<<<<<<<<<
 *             unpack_int32(data, idx + 18),  # type: ignore
 *         )
 */
    __pyx_t_11 = __pyx_f_10serializor_7unicode_3des_unpack_int32(__pyx_v_data, (__pyx_v_idx + 14)); if (unlikely(__pyx_t_11 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 1061, __pyx_L1_error)

    /* "serializor/unicode/des.py":1062
 *             unpack_int32(data, idx + 10),  # type: ignore
 *             unpack_int32(data, idx + 14),  # type: ignore
 *             unpack_int32(data, idx + 18),  # type: ignore             # <<<<<<<<<<<<<<
 *         )
 *         pos[0] += 22  # set new position
 */
    __pyx_t_12 = __pyx_f_10serializor_7unicode_3des_unpack_int32(__pyx_v_data, (__pyx_v_idx + 18)); if (unlikely(__pyx_t_12 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 1062, __pyx_L1_error)

    /* "serializor/unicode/des.py":1059
 *     # Timezone: approach 2 (tz_offset)
 *     else:
 *         offset = datetime.timedelta_new(             # <<<<<<<<<<<<<<
 *             unpack_int32(data, idx + 10),  # type: ignore
 *             unpack_int32(data, idx + 14),  # type: ignore
 */
    __pyx_t_5 = ((PyObject *)__pyx_f_7cpython_8datetime_timedelta_new(__pyx_t_10, __pyx_t_11, __pyx_t_12)); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1059, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_v_offset = ((PyDateTime_Delta *)__pyx_t_5);
    __pyx_t_5 = 0;

    /* "serializor/unicode/des.py":1064
 *             unpack_int32(data, idx + 18),  # type: ignore
 *         )
 *         pos[0] += 22  # set new position             # <<<<<<<<<<<<<<
 *         tzinfo = datetime.timezone_new(offset, None)
 *         return typeref.PD_TIMESTAMP(val, tzinfo=tzinfo)
 */
    __pyx_t_3 = 0;
    (__pyx_v_pos[__pyx_t_3]) = ((__pyx_v_pos[__pyx_t_3]) + 22);

    /* "serializor/unicode/des.py":1065
 *         )
 *         pos[0] += 22  # set new position
 *         tzinfo = datetime.timezone_new(offset, None)             # <<<<<<<<<<<<<<
 *         return typeref.PD_TIMESTAMP(val, tzinfo=tzinfo)
 * 
 */
    __pyx_t_13.__pyx_n = 1;
    __pyx_t_13.name = Py_None;
    __pyx_t_5 = __pyx_f_7cpython_8datetime_timezone_new(((PyObject *)__pyx_v_offset), &__pyx_t_13); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1065, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_v_tzinfo = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "serializor/unicode/des.py":1066
 *         pos[0] += 22  # set new position
 *         tzinfo = datetime.timezone_new(offset, None)
 *         return typeref.PD_TIMESTAMP(val, tzinfo=tzinfo)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_5 = __Pyx_PyInt_From_PY_LONG_LONG(__pyx_v_val); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1066, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1066, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5)) __PYX_ERR(1, 1066, __pyx_L1_error);
    __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1066, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_tzinfo, __pyx_v_tzinfo) < 0) __PYX_ERR(1, 1066, __pyx_L1_error)
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_v_10serializor_7typeref_PD_TIMESTAMP, __pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1066, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;
    break;
  }

  /* "serializor/unicode/des.py":1034
 * 
 * # Pandas Types --------------------------------------------------------------------------------
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_pd_timestamp(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("serializor.unicode.des._des_pd_timestamp", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_tz_name);
  __Pyx_XDECREF((PyObject *)__pyx_v_offset);
  __Pyx_XDECREF(__pyx_v_tzinfo);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":1069
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_pd_timedelta(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_pd_timedelta(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  Py_ssize_t __pyx_v_idx;
  PY_LONG_LONG __pyx_v_val;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PY_LONG_LONG __pyx_t_1;
  long __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  unsigned int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_pd_timedelta", 1);

  /* "serializor/unicode/des.py":1076
 *     Position updates to the next byte after deserialization.
 *     """
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip identifier             # <<<<<<<<<<<<<<
 *     val: cython.longlong = unpack_int64(data, idx)  # type: ignore
 *     pos[0] += 9  # set new position
 */
  __pyx_v_idx = ((__pyx_v_pos[0]) + 1);

  /* "serializor/unicode/des.py":1077
 *     """
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip identifier
 *     val: cython.longlong = unpack_int64(data, idx)  # type: ignore             # <<<<<<<<<<<<<<
 *     pos[0] += 9  # set new position
 *     return typeref.PD_TIMEDELTA(val)
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_int64(__pyx_v_data, __pyx_v_idx); if (unlikely(__pyx_t_1 == ((PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 1077, __pyx_L1_error)
  __pyx_v_val = __pyx_t_1;

  /* "serializor/unicode/des.py":1078
 *     idx: cython.Py_ssize_t = pos[0] + 1  # skip identifier
 *     val: cython.longlong = unpack_int64(data, idx)  # type: ignore
 *     pos[0] += 9  # set new position             # <<<<<<<<<<<<<<
 *     return typeref.PD_TIMEDELTA(val)
 * 
 */
  __pyx_t_2 = 0;
  (__pyx_v_pos[__pyx_t_2]) = ((__pyx_v_pos[__pyx_t_2]) + 9);

  /* "serializor/unicode/des.py":1079
 *     val: cython.longlong = unpack_int64(data, idx)  # type: ignore
 *     pos[0] += 9  # set new position
 *     return typeref.PD_TIMEDELTA(val)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __Pyx_PyInt_From_PY_LONG_LONG(__pyx_v_val); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1079, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_10serializor_7typeref_PD_TIMEDELTA);
  __pyx_t_5 = __pyx_v_10serializor_7typeref_PD_TIMEDELTA; __pyx_t_6 = NULL;
  __pyx_t_7 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
      __pyx_t_7 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_4};
    __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1079, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":1069
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_pd_timedelta(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("serializor.unicode.des._des_pd_timedelta", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":1082
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_series(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_series(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  unsigned char __pyx_v_has_name;
  PyObject *__pyx_v_name = NULL;
  unsigned char __pyx_v_arr_dtype;
  PyObject *__pyx_v_arr = NULL;
  unsigned char __pyx_v_flag;
  unsigned PY_LONG_LONG __pyx_v_size;
  Py_ssize_t __pyx_v_idx;
  Py_ssize_t __pyx_v_end;
  PyObject *__pyx_v_tz_name = 0;
  PyObject *__pyx_v_series = NULL;
  PyDateTime_Delta *__pyx_v_offset = NULL;
  PyObject *__pyx_v_tzinfo = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  unsigned char __pyx_t_1;
  long __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  Py_UCS4 __pyx_t_6;
  unsigned PY_LONG_LONG __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  unsigned int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  int __pyx_t_13;
  struct __pyx_opt_args_7cpython_8datetime_timezone_new __pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_series", 1);

  /* "serializor/unicode/des.py":1090
 *     """
 *     # Deserialize name
 *     has_name: cython.uchar = unpack_uint8(data, pos[0] + 1)  # type: ignore             # <<<<<<<<<<<<<<
 *     pos[0] += 2  # skip identifier & has_name
 *     name = None if has_name == 0 else _des_data(data, pos)
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, ((__pyx_v_pos[0]) + 1)); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 1090, __pyx_L1_error)
  __pyx_v_has_name = __pyx_t_1;

  /* "serializor/unicode/des.py":1091
 *     # Deserialize name
 *     has_name: cython.uchar = unpack_uint8(data, pos[0] + 1)  # type: ignore
 *     pos[0] += 2  # skip identifier & has_name             # <<<<<<<<<<<<<<
 *     name = None if has_name == 0 else _des_data(data, pos)
 * 
 */
  __pyx_t_2 = 0;
  (__pyx_v_pos[__pyx_t_2]) = ((__pyx_v_pos[__pyx_t_2]) + 2);

  /* "serializor/unicode/des.py":1092
 *     has_name: cython.uchar = unpack_uint8(data, pos[0] + 1)  # type: ignore
 *     pos[0] += 2  # skip identifier & has_name
 *     name = None if has_name == 0 else _des_data(data, pos)             # <<<<<<<<<<<<<<
 * 
 *     # Deserialize values
 */
  __pyx_t_4 = (__pyx_v_has_name == 0);
  if (__pyx_t_4) {
    __Pyx_INCREF(Py_None);
    __pyx_t_3 = Py_None;
  } else {
    __pyx_t_5 = __pyx_f_10serializor_7unicode_3des__des_data(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1092, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = __pyx_t_5;
    __pyx_t_5 = 0;
  }
  __pyx_v_name = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "serializor/unicode/des.py":1095
 * 
 *     # Deserialize values
 *     arr_dtype: cython.uchar = data[pos[0]]             # <<<<<<<<<<<<<<
 *     # . ndarray[object]
 *     if arr_dtype == identifier.NDARRAY_OBJECT:
 */
  __pyx_t_6 = __Pyx_GetItemInt_Unicode(__pyx_v_data, (__pyx_v_pos[0]), Py_ssize_t, 1, PyInt_FromSsize_t, 0, 0, 0); if (unlikely(__pyx_t_6 == (Py_UCS4)-1)) __PYX_ERR(1, 1095, __pyx_L1_error)
  __pyx_v_arr_dtype = __pyx_t_6;

  /* "serializor/unicode/des.py":1097
 *     arr_dtype: cython.uchar = data[pos[0]]
 *     # . ndarray[object]
 *     if arr_dtype == identifier.NDARRAY_OBJECT:             # <<<<<<<<<<<<<<
 *         arr = _des_ndarray_object(data, pos)
 *     # . ndarray[int]
 */
  __pyx_t_4 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_OBJECT);
  if (__pyx_t_4) {

    /* "serializor/unicode/des.py":1098
 *     # . ndarray[object]
 *     if arr_dtype == identifier.NDARRAY_OBJECT:
 *         arr = _des_ndarray_object(data, pos)             # <<<<<<<<<<<<<<
 *     # . ndarray[int]
 *     elif arr_dtype == identifier.NDARRAY_INT:
 */
    __pyx_t_3 = __pyx_f_10serializor_7unicode_3des__des_ndarray_object(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1098, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_arr = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "serializor/unicode/des.py":1097
 *     arr_dtype: cython.uchar = data[pos[0]]
 *     # . ndarray[object]
 *     if arr_dtype == identifier.NDARRAY_OBJECT:             # <<<<<<<<<<<<<<
 *         arr = _des_ndarray_object(data, pos)
 *     # . ndarray[int]
 */
    goto __pyx_L3;
  }

  /* "serializor/unicode/des.py":1100
 *         arr = _des_ndarray_object(data, pos)
 *     # . ndarray[int]
 *     elif arr_dtype == identifier.NDARRAY_INT:             # <<<<<<<<<<<<<<
 *         arr = _des_ndarray_numeric(data, pos)
 *     # . ndarray[uint]
 */
  __pyx_t_4 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_INT);
  if (__pyx_t_4) {

    /* "serializor/unicode/des.py":1101
 *     # . ndarray[int]
 *     elif arr_dtype == identifier.NDARRAY_INT:
 *         arr = _des_ndarray_numeric(data, pos)             # <<<<<<<<<<<<<<
 *     # . ndarray[uint]
 *     elif arr_dtype == identifier.NDARRAY_UINT:
 */
    __pyx_t_3 = __pyx_f_10serializor_7unicode_3des__des_ndarray_numeric(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1101, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_arr = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "serializor/unicode/des.py":1100
 *         arr = _des_ndarray_object(data, pos)
 *     # . ndarray[int]
 *     elif arr_dtype == identifier.NDARRAY_INT:             # <<<<<<<<<<<<<<
 *         arr = _des_ndarray_numeric(data, pos)
 *     # . ndarray[uint]
 */
    goto __pyx_L3;
  }

  /* "serializor/unicode/des.py":1103
 *         arr = _des_ndarray_numeric(data, pos)
 *     # . ndarray[uint]
 *     elif arr_dtype == identifier.NDARRAY_UINT:             # <<<<<<<<<<<<<<
 *         arr = _des_ndarray_numeric(data, pos)
 *     # . ndarray[float]
 */
  __pyx_t_4 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_UINT);
  if (__pyx_t_4) {

    /* "serializor/unicode/des.py":1104
 *     # . ndarray[uint]
 *     elif arr_dtype == identifier.NDARRAY_UINT:
 *         arr = _des_ndarray_numeric(data, pos)             # <<<<<<<<<<<<<<
 *     # . ndarray[float]
 *     elif arr_dtype == identifier.NDARRAY_FLOAT:
 */
    __pyx_t_3 = __pyx_f_10serializor_7unicode_3des__des_ndarray_numeric(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1104, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_arr = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "serializor/unicode/des.py":1103
 *         arr = _des_ndarray_numeric(data, pos)
 *     # . ndarray[uint]
 *     elif arr_dtype == identifier.NDARRAY_UINT:             # <<<<<<<<<<<<<<
 *         arr = _des_ndarray_numeric(data, pos)
 *     # . ndarray[float]
 */
    goto __pyx_L3;
  }

  /* "serializor/unicode/des.py":1106
 *         arr = _des_ndarray_numeric(data, pos)
 *     # . ndarray[float]
 *     elif arr_dtype == identifier.NDARRAY_FLOAT:             # <<<<<<<<<<<<<<
 *         arr = _des_ndarray_numeric(data, pos)
 *     # . ndarray[bool]
 */
  __pyx_t_4 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_FLOAT);
  if (__pyx_t_4) {

    /* "serializor/unicode/des.py":1107
 *     # . ndarray[float]
 *     elif arr_dtype == identifier.NDARRAY_FLOAT:
 *         arr = _des_ndarray_numeric(data, pos)             # <<<<<<<<<<<<<<
 *     # . ndarray[bool]
 *     elif arr_dtype == identifier.NDARRAY_BOOL:
 */
    __pyx_t_3 = __pyx_f_10serializor_7unicode_3des__des_ndarray_numeric(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1107, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_arr = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "serializor/unicode/des.py":1106
 *         arr = _des_ndarray_numeric(data, pos)
 *     # . ndarray[float]
 *     elif arr_dtype == identifier.NDARRAY_FLOAT:             # <<<<<<<<<<<<<<
 *         arr = _des_ndarray_numeric(data, pos)
 *     # . ndarray[bool]
 */
    goto __pyx_L3;
  }

  /* "serializor/unicode/des.py":1109
 *         arr = _des_ndarray_numeric(data, pos)
 *     # . ndarray[bool]
 *     elif arr_dtype == identifier.NDARRAY_BOOL:             # <<<<<<<<<<<<<<
 *         arr = _des_ndarray_numeric(data, pos)
 *     # . ndarray[datetime64]
 */
  __pyx_t_4 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_BOOL);
  if (__pyx_t_4) {

    /* "serializor/unicode/des.py":1110
 *     # . ndarray[bool]
 *     elif arr_dtype == identifier.NDARRAY_BOOL:
 *         arr = _des_ndarray_numeric(data, pos)             # <<<<<<<<<<<<<<
 *     # . ndarray[datetime64]
 *     elif arr_dtype == identifier.NDARRAY_DT64:
 */
    __pyx_t_3 = __pyx_f_10serializor_7unicode_3des__des_ndarray_numeric(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1110, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_arr = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "serializor/unicode/des.py":1109
 *         arr = _des_ndarray_numeric(data, pos)
 *     # . ndarray[bool]
 *     elif arr_dtype == identifier.NDARRAY_BOOL:             # <<<<<<<<<<<<<<
 *         arr = _des_ndarray_numeric(data, pos)
 *     # . ndarray[datetime64]
 */
    goto __pyx_L3;
  }

  /* "serializor/unicode/des.py":1112
 *         arr = _des_ndarray_numeric(data, pos)
 *     # . ndarray[datetime64]
 *     elif arr_dtype == identifier.NDARRAY_DT64:             # <<<<<<<<<<<<<<
 *         flag: cython.uchar = unpack_uint8(data, pos[0] + 1)  # type: ignore
 *         pos[0] += 2  # skip arr dtype & flag
 */
  __pyx_t_4 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_DT64);
  if (__pyx_t_4) {

    /* "serializor/unicode/des.py":1113
 *     # . ndarray[datetime64]
 *     elif arr_dtype == identifier.NDARRAY_DT64:
 *         flag: cython.uchar = unpack_uint8(data, pos[0] + 1)  # type: ignore             # <<<<<<<<<<<<<<
 *         pos[0] += 2  # skip arr dtype & flag
 *         # . Without timezone
 */
    __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, ((__pyx_v_pos[0]) + 1)); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 1113, __pyx_L1_error)
    __pyx_v_flag = __pyx_t_1;

    /* "serializor/unicode/des.py":1114
 *     elif arr_dtype == identifier.NDARRAY_DT64:
 *         flag: cython.uchar = unpack_uint8(data, pos[0] + 1)  # type: ignore
 *         pos[0] += 2  # skip arr dtype & flag             # <<<<<<<<<<<<<<
 *         # . Without timezone
 *         if flag == 0:
 */
    __pyx_t_2 = 0;
    (__pyx_v_pos[__pyx_t_2]) = ((__pyx_v_pos[__pyx_t_2]) + 2);

    /* "serializor/unicode/des.py":1116
 *         pos[0] += 2  # skip arr dtype & flag
 *         # . Without timezone
 *         if flag == 0:             # <<<<<<<<<<<<<<
 *             arr = _des_ndarray_nptime(data, pos, True)
 *         # . Timezone: approach 1 (tz_name)
 */
    switch (__pyx_v_flag) {
      case 0:

      /* "serializor/unicode/des.py":1117
 *         # . Without timezone
 *         if flag == 0:
 *             arr = _des_ndarray_nptime(data, pos, True)             # <<<<<<<<<<<<<<
 *         # . Timezone: approach 1 (tz_name)
 *         elif flag == 1:
 */
      __pyx_t_3 = __pyx_f_10serializor_7unicode_3des__des_ndarray_nptime(__pyx_v_data, __pyx_v_pos, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1117, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_v_arr = __pyx_t_3;
      __pyx_t_3 = 0;

      /* "serializor/unicode/des.py":1116
 *         pos[0] += 2  # skip arr dtype & flag
 *         # . Without timezone
 *         if flag == 0:             # <<<<<<<<<<<<<<
 *             arr = _des_ndarray_nptime(data, pos, True)
 *         # . Timezone: approach 1 (tz_name)
 */
      break;
      case 1:

      /* "serializor/unicode/des.py":1120
 *         # . Timezone: approach 1 (tz_name)
 *         elif flag == 1:
 *             size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *             idx: cython.Py_ssize_t = pos[0]
 *             end: cython.Py_ssize_t = idx + size
 */
      __pyx_t_7 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_7 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 1120, __pyx_L1_error)
      __pyx_v_size = __pyx_t_7;

      /* "serializor/unicode/des.py":1121
 *         elif flag == 1:
 *             size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *             idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *             end: cython.Py_ssize_t = idx + size
 *             tz_name: str = data[idx:end]
 */
      __pyx_v_idx = (__pyx_v_pos[0]);

      /* "serializor/unicode/des.py":1122
 *             size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *             idx: cython.Py_ssize_t = pos[0]
 *             end: cython.Py_ssize_t = idx + size             # <<<<<<<<<<<<<<
 *             tz_name: str = data[idx:end]
 *             pos[0] = end  # set new position
 */
      __pyx_v_end = (__pyx_v_idx + __pyx_v_size);

      /* "serializor/unicode/des.py":1123
 *             idx: cython.Py_ssize_t = pos[0]
 *             end: cython.Py_ssize_t = idx + size
 *             tz_name: str = data[idx:end]             # <<<<<<<<<<<<<<
 *             pos[0] = end  # set new position
 *             arr = _des_ndarray_nptime(data, pos, True)
 */
      __pyx_t_3 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1123, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_v_tz_name = ((PyObject*)__pyx_t_3);
      __pyx_t_3 = 0;

      /* "serializor/unicode/des.py":1124
 *             end: cython.Py_ssize_t = idx + size
 *             tz_name: str = data[idx:end]
 *             pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *             arr = _des_ndarray_nptime(data, pos, True)
 *             series = typeref.SERIES(arr, name=name, copy=False)
 */
      (__pyx_v_pos[0]) = __pyx_v_end;

      /* "serializor/unicode/des.py":1125
 *             tz_name: str = data[idx:end]
 *             pos[0] = end  # set new position
 *             arr = _des_ndarray_nptime(data, pos, True)             # <<<<<<<<<<<<<<
 *             series = typeref.SERIES(arr, name=name, copy=False)
 *             return series.dt.tz_localize("UTC").dt.tz_convert(tz_name)  # exit
 */
      __pyx_t_3 = __pyx_f_10serializor_7unicode_3des__des_ndarray_nptime(__pyx_v_data, __pyx_v_pos, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1125, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_v_arr = __pyx_t_3;
      __pyx_t_3 = 0;

      /* "serializor/unicode/des.py":1126
 *             pos[0] = end  # set new position
 *             arr = _des_ndarray_nptime(data, pos, True)
 *             series = typeref.SERIES(arr, name=name, copy=False)             # <<<<<<<<<<<<<<
 *             return series.dt.tz_localize("UTC").dt.tz_convert(tz_name)  # exit
 *         # . Timezone: approach 2 (tz_offset)
 */
      __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1126, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_v_arr);
      __Pyx_GIVEREF(__pyx_v_arr);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_arr)) __PYX_ERR(1, 1126, __pyx_L1_error);
      __pyx_t_5 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1126, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_name, __pyx_v_name) < 0) __PYX_ERR(1, 1126, __pyx_L1_error)
      if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_copy, Py_False) < 0) __PYX_ERR(1, 1126, __pyx_L1_error)
      __pyx_t_8 = __Pyx_PyObject_Call(__pyx_v_10serializor_7typeref_SERIES, __pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 1126, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_v_series = __pyx_t_8;
      __pyx_t_8 = 0;

      /* "serializor/unicode/des.py":1127
 *             arr = _des_ndarray_nptime(data, pos, True)
 *             series = typeref.SERIES(arr, name=name, copy=False)
 *             return series.dt.tz_localize("UTC").dt.tz_convert(tz_name)  # exit             # <<<<<<<<<<<<<<
 *         # . Timezone: approach 2 (tz_offset)
 *         else:
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_series, __pyx_n_s_dt); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1127, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_tz_localize); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 1127, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = NULL;
      __pyx_t_10 = 0;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_9))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_9);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_9, function);
          __pyx_t_10 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_n_u_UTC};
        __pyx_t_5 = __Pyx_PyObject_FastCall(__pyx_t_9, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1127, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      }
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_dt); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 1127, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_tz_convert); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1127, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_9 = NULL;
      __pyx_t_10 = 0;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_9)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
          __pyx_t_10 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_v_tz_name};
        __pyx_t_8 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 1127, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __pyx_r = __pyx_t_8;
      __pyx_t_8 = 0;
      goto __pyx_L0;

      /* "serializor/unicode/des.py":1119
 *             arr = _des_ndarray_nptime(data, pos, True)
 *         # . Timezone: approach 1 (tz_name)
 *         elif flag == 1:             # <<<<<<<<<<<<<<
 *             size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *             idx: cython.Py_ssize_t = pos[0]
 */
      break;
      default:

      /* "serializor/unicode/des.py":1130
 *         # . Timezone: approach 2 (tz_offset)
 *         else:
 *             idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *             offset = datetime.timedelta_new(
 *                 unpack_int32(data, idx),  # type: ignore
 */
      __pyx_v_idx = (__pyx_v_pos[0]);

      /* "serializor/unicode/des.py":1132
 *             idx: cython.Py_ssize_t = pos[0]
 *             offset = datetime.timedelta_new(
 *                 unpack_int32(data, idx),  # type: ignore             # <<<<<<<<<<<<<<
 *                 unpack_int32(data, idx + 4),  # type: ignore
 *                 unpack_int32(data, idx + 8),  # type: ignore
 */
      __pyx_t_11 = __pyx_f_10serializor_7unicode_3des_unpack_int32(__pyx_v_data, __pyx_v_idx); if (unlikely(__pyx_t_11 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 1132, __pyx_L1_error)

      /* "serializor/unicode/des.py":1133
 *             offset = datetime.timedelta_new(
 *                 unpack_int32(data, idx),  # type: ignore
 *                 unpack_int32(data, idx + 4),  # type: ignore             # <<<<<<<<<<<<<<
 *                 unpack_int32(data, idx + 8),  # type: ignore
 *             )
 */
      __pyx_t_12 = __pyx_f_10serializor_7unicode_3des_unpack_int32(__pyx_v_data, (__pyx_v_idx + 4)); if (unlikely(__pyx_t_12 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 1133, __pyx_L1_error)

      /* "serializor/unicode/des.py":1134
 *                 unpack_int32(data, idx),  # type: ignore
 *                 unpack_int32(data, idx + 4),  # type: ignore
 *                 unpack_int32(data, idx + 8),  # type: ignore             # <<<<<<<<<<<<<<
 *             )
 *             pos[0] += 12  # set new position
 */
      __pyx_t_13 = __pyx_f_10serializor_7unicode_3des_unpack_int32(__pyx_v_data, (__pyx_v_idx + 8)); if (unlikely(__pyx_t_13 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 1134, __pyx_L1_error)

      /* "serializor/unicode/des.py":1131
 *         else:
 *             idx: cython.Py_ssize_t = pos[0]
 *             offset = datetime.timedelta_new(             # <<<<<<<<<<<<<<
 *                 unpack_int32(data, idx),  # type: ignore
 *                 unpack_int32(data, idx + 4),  # type: ignore
 */
      __pyx_t_8 = ((PyObject *)__pyx_f_7cpython_8datetime_timedelta_new(__pyx_t_11, __pyx_t_12, __pyx_t_13)); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 1131, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_v_offset = ((PyDateTime_Delta *)__pyx_t_8);
      __pyx_t_8 = 0;

      /* "serializor/unicode/des.py":1136
 *                 unpack_int32(data, idx + 8),  # type: ignore
 *             )
 *             pos[0] += 12  # set new position             # <<<<<<<<<<<<<<
 *             tzinfo = datetime.timezone_new(offset, None)
 *             arr = _des_ndarray_nptime(data, pos, True)
 */
      __pyx_t_2 = 0;
      (__pyx_v_pos[__pyx_t_2]) = ((__pyx_v_pos[__pyx_t_2]) + 12);

      /* "serializor/unicode/des.py":1137
 *             )
 *             pos[0] += 12  # set new position
 *             tzinfo = datetime.timezone_new(offset, None)             # <<<<<<<<<<<<<<
 *             arr = _des_ndarray_nptime(data, pos, True)
 *             series = typeref.SERIES(arr, name=name, copy=False)
 */
      __pyx_t_14.__pyx_n = 1;
      __pyx_t_14.name = Py_None;
      __pyx_t_8 = __pyx_f_7cpython_8datetime_timezone_new(((PyObject *)__pyx_v_offset), &__pyx_t_14); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 1137, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_v_tzinfo = __pyx_t_8;
      __pyx_t_8 = 0;

      /* "serializor/unicode/des.py":1138
 *             pos[0] += 12  # set new position
 *             tzinfo = datetime.timezone_new(offset, None)
 *             arr = _des_ndarray_nptime(data, pos, True)             # <<<<<<<<<<<<<<
 *             series = typeref.SERIES(arr, name=name, copy=False)
 *             return series.dt.tz_localize("UTC").dt.tz_convert(tzinfo)  # exit
 */
      __pyx_t_8 = __pyx_f_10serializor_7unicode_3des__des_ndarray_nptime(__pyx_v_data, __pyx_v_pos, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 1138, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_v_arr = __pyx_t_8;
      __pyx_t_8 = 0;

      /* "serializor/unicode/des.py":1139
 *             tzinfo = datetime.timezone_new(offset, None)
 *             arr = _des_ndarray_nptime(data, pos, True)
 *             series = typeref.SERIES(arr, name=name, copy=False)             # <<<<<<<<<<<<<<
 *             return series.dt.tz_localize("UTC").dt.tz_convert(tzinfo)  # exit
 *     # . ndarray[timedelta64]
 */
      __pyx_t_8 = PyTuple_New(1); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 1139, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_INCREF(__pyx_v_arr);
      __Pyx_GIVEREF(__pyx_v_arr);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_v_arr)) __PYX_ERR(1, 1139, __pyx_L1_error);
      __pyx_t_5 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1139, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_name, __pyx_v_name) < 0) __PYX_ERR(1, 1139, __pyx_L1_error)
      if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_copy, Py_False) < 0) __PYX_ERR(1, 1139, __pyx_L1_error)
      __pyx_t_9 = __Pyx_PyObject_Call(__pyx_v_10serializor_7typeref_SERIES, __pyx_t_8, __pyx_t_5); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 1139, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_v_series = __pyx_t_9;
      __pyx_t_9 = 0;

      /* "serializor/unicode/des.py":1140
 *             arr = _des_ndarray_nptime(data, pos, True)
 *             series = typeref.SERIES(arr, name=name, copy=False)
 *             return series.dt.tz_localize("UTC").dt.tz_convert(tzinfo)  # exit             # <<<<<<<<<<<<<<
 *     # . ndarray[timedelta64]
 *     elif arr_dtype == identifier.NDARRAY_TD64:
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_series, __pyx_n_s_dt); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 1140, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_tz_localize); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1140, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = NULL;
      __pyx_t_10 = 0;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_10 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_n_u_UTC};
        __pyx_t_5 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1140, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_dt); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1140, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_tz_convert); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1140, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = NULL;
      __pyx_t_10 = 0;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
          __pyx_t_10 = 1;
        }
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_tzinfo};
        __pyx_t_9 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_10, 1+__pyx_t_10);
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 1140, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __pyx_r = __pyx_t_9;
      __pyx_t_9 = 0;
      goto __pyx_L0;
      break;
    }

    /* "serializor/unicode/des.py":1112
 *         arr = _des_ndarray_numeric(data, pos)
 *     # . ndarray[datetime64]
 *     elif arr_dtype == identifier.NDARRAY_DT64:             # <<<<<<<<<<<<<<
 *         flag: cython.uchar = unpack_uint8(data, pos[0] + 1)  # type: ignore
 *         pos[0] += 2  # skip arr dtype & flag
 */
    goto __pyx_L3;
  }

  /* "serializor/unicode/des.py":1142
 *             return series.dt.tz_localize("UTC").dt.tz_convert(tzinfo)  # exit
 *     # . ndarray[timedelta64]
 *     elif arr_dtype == identifier.NDARRAY_TD64:             # <<<<<<<<<<<<<<
 *         arr = _des_ndarray_nptime(data, pos, False)
 *     # . ndarray[complex]
 */
  __pyx_t_4 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_TD64);
  if (__pyx_t_4) {

    /* "serializor/unicode/des.py":1143
 *     # . ndarray[timedelta64]
 *     elif arr_dtype == identifier.NDARRAY_TD64:
 *         arr = _des_ndarray_nptime(data, pos, False)             # <<<<<<<<<<<<<<
 *     # . ndarray[complex]
 *     elif arr_dtype == identifier.NDARRAY_COMPLEX:
 */
    __pyx_t_9 = __pyx_f_10serializor_7unicode_3des__des_ndarray_nptime(__pyx_v_data, __pyx_v_pos, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 1143, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arr = __pyx_t_9;
    __pyx_t_9 = 0;

    /* "serializor/unicode/des.py":1142
 *             return series.dt.tz_localize("UTC").dt.tz_convert(tzinfo)  # exit
 *     # . ndarray[timedelta64]
 *     elif arr_dtype == identifier.NDARRAY_TD64:             # <<<<<<<<<<<<<<
 *         arr = _des_ndarray_nptime(data, pos, False)
 *     # . ndarray[complex]
 */
    goto __pyx_L3;
  }

  /* "serializor/unicode/des.py":1145
 *         arr = _des_ndarray_nptime(data, pos, False)
 *     # . ndarray[complex]
 *     elif arr_dtype == identifier.NDARRAY_COMPLEX:             # <<<<<<<<<<<<<<
 *         arr = _des_ndarray_complex(data, pos)
 *     # . ndarray[bytes]
 */
  __pyx_t_4 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_COMPLEX);
  if (__pyx_t_4) {

    /* "serializor/unicode/des.py":1146
 *     # . ndarray[complex]
 *     elif arr_dtype == identifier.NDARRAY_COMPLEX:
 *         arr = _des_ndarray_complex(data, pos)             # <<<<<<<<<<<<<<
 *     # . ndarray[bytes]
 *     elif arr_dtype == identifier.NDARRAY_BYTES:
 */
    __pyx_t_9 = __pyx_f_10serializor_7unicode_3des__des_ndarray_complex(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 1146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arr = __pyx_t_9;
    __pyx_t_9 = 0;

    /* "serializor/unicode/des.py":1145
 *         arr = _des_ndarray_nptime(data, pos, False)
 *     # . ndarray[complex]
 *     elif arr_dtype == identifier.NDARRAY_COMPLEX:             # <<<<<<<<<<<<<<
 *         arr = _des_ndarray_complex(data, pos)
 *     # . ndarray[bytes]
 */
    goto __pyx_L3;
  }

  /* "serializor/unicode/des.py":1148
 *         arr = _des_ndarray_complex(data, pos)
 *     # . ndarray[bytes]
 *     elif arr_dtype == identifier.NDARRAY_BYTES:             # <<<<<<<<<<<<<<
 *         arr = _des_ndarray_bytes(data, pos)
 *     # . ndarray[str]
 */
  __pyx_t_4 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_BYTES);
  if (__pyx_t_4) {

    /* "serializor/unicode/des.py":1149
 *     # . ndarray[bytes]
 *     elif arr_dtype == identifier.NDARRAY_BYTES:
 *         arr = _des_ndarray_bytes(data, pos)             # <<<<<<<<<<<<<<
 *     # . ndarray[str]
 *     elif arr_dtype == identifier.NDARRAY_UNICODE:
 */
    __pyx_t_9 = __pyx_f_10serializor_7unicode_3des__des_ndarray_bytes(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 1149, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arr = __pyx_t_9;
    __pyx_t_9 = 0;

    /* "serializor/unicode/des.py":1148
 *         arr = _des_ndarray_complex(data, pos)
 *     # . ndarray[bytes]
 *     elif arr_dtype == identifier.NDARRAY_BYTES:             # <<<<<<<<<<<<<<
 *         arr = _des_ndarray_bytes(data, pos)
 *     # . ndarray[str]
 */
    goto __pyx_L3;
  }

  /* "serializor/unicode/des.py":1151
 *         arr = _des_ndarray_bytes(data, pos)
 *     # . ndarray[str]
 *     elif arr_dtype == identifier.NDARRAY_UNICODE:             # <<<<<<<<<<<<<<
 *         arr = _des_ndarray_unicode(data, pos)
 *     # . invalid dtype
 */
  __pyx_t_4 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_UNICODE);
  if (likely(__pyx_t_4)) {

    /* "serializor/unicode/des.py":1152
 *     # . ndarray[str]
 *     elif arr_dtype == identifier.NDARRAY_UNICODE:
 *         arr = _des_ndarray_unicode(data, pos)             # <<<<<<<<<<<<<<
 *     # . invalid dtype
 *     else:
 */
    __pyx_t_9 = __pyx_f_10serializor_7unicode_3des__des_ndarray_unicode(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 1152, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arr = __pyx_t_9;
    __pyx_t_9 = 0;

    /* "serializor/unicode/des.py":1151
 *         arr = _des_ndarray_bytes(data, pos)
 *     # . ndarray[str]
 *     elif arr_dtype == identifier.NDARRAY_UNICODE:             # <<<<<<<<<<<<<<
 *         arr = _des_ndarray_unicode(data, pos)
 *     # . invalid dtype
 */
    goto __pyx_L3;
  }

  /* "serializor/unicode/des.py":1155
 *     # . invalid dtype
 *     else:
 *         raise ValueError("unsupported <'pandas.Series'> dtype '%s'." % chr(arr_dtype))             # <<<<<<<<<<<<<<
 * 
 *     # Generate Series
 */
  /*else*/ {
    __pyx_t_9 = __Pyx_PyInt_From_unsigned_char(__pyx_v_arr_dtype); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 1155, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_builtin_chr, __pyx_t_9); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1155, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyUnicode_FormatSafe(__pyx_kp_u_unsupported_pandas_Series_dtype, __pyx_t_5); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 1155, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_9); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1155, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(1, 1155, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "serializor/unicode/des.py":1158
 * 
 *     # Generate Series
 *     return typeref.SERIES(arr, name=name, copy=False)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1158, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_v_arr);
  __Pyx_GIVEREF(__pyx_v_arr);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_arr)) __PYX_ERR(1, 1158, __pyx_L1_error);
  __pyx_t_9 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 1158, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_name, __pyx_v_name) < 0) __PYX_ERR(1, 1158, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_copy, Py_False) < 0) __PYX_ERR(1, 1158, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_v_10serializor_7typeref_SERIES, __pyx_t_5, __pyx_t_9); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1158, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":1082
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_series(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("serializor.unicode.des._des_series", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XDECREF(__pyx_v_arr);
  __Pyx_XDECREF(__pyx_v_tz_name);
  __Pyx_XDECREF(__pyx_v_series);
  __Pyx_XDECREF((PyObject *)__pyx_v_offset);
  __Pyx_XDECREF(__pyx_v_tzinfo);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":1161
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_datetime_index(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_datetime_index(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  unsigned char __pyx_v_has_name;
  PyObject *__pyx_v_name = NULL;
  unsigned char __pyx_v_flag;
  PyObject *__pyx_v_arr = NULL;
  unsigned PY_LONG_LONG __pyx_v_size;
  Py_ssize_t __pyx_v_idx;
  Py_ssize_t __pyx_v_end;
  PyObject *__pyx_v_tz_name = NULL;
  PyObject *__pyx_v_dindex = NULL;
  PyDateTime_Delta *__pyx_v_offset = NULL;
  PyObject *__pyx_v_tzinfo = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  unsigned char __pyx_t_1;
  long __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  unsigned PY_LONG_LONG __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  unsigned int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  struct __pyx_opt_args_7cpython_8datetime_timezone_new __pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_datetime_index", 1);

  /* "serializor/unicode/des.py":1169
 *     """
 *     # Deserialize name
 *     has_name: cython.uchar = unpack_uint8(data, pos[0] + 1)  # type: ignore             # <<<<<<<<<<<<<<
 *     pos[0] += 2  # skip identifier & has_name
 *     name = None if has_name == 0 else _des_data(data, pos)
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, ((__pyx_v_pos[0]) + 1)); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 1169, __pyx_L1_error)
  __pyx_v_has_name = __pyx_t_1;

  /* "serializor/unicode/des.py":1170
 *     # Deserialize name
 *     has_name: cython.uchar = unpack_uint8(data, pos[0] + 1)  # type: ignore
 *     pos[0] += 2  # skip identifier & has_name             # <<<<<<<<<<<<<<
 *     name = None if has_name == 0 else _des_data(data, pos)
 * 
 */
  __pyx_t_2 = 0;
  (__pyx_v_pos[__pyx_t_2]) = ((__pyx_v_pos[__pyx_t_2]) + 2);

  /* "serializor/unicode/des.py":1171
 *     has_name: cython.uchar = unpack_uint8(data, pos[0] + 1)  # type: ignore
 *     pos[0] += 2  # skip identifier & has_name
 *     name = None if has_name == 0 else _des_data(data, pos)             # <<<<<<<<<<<<<<
 * 
 *     # Deserialize values
 */
  __pyx_t_4 = (__pyx_v_has_name == 0);
  if (__pyx_t_4) {
    __Pyx_INCREF(Py_None);
    __pyx_t_3 = Py_None;
  } else {
    __pyx_t_5 = __pyx_f_10serializor_7unicode_3des__des_data(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1171, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = __pyx_t_5;
    __pyx_t_5 = 0;
  }
  __pyx_v_name = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "serializor/unicode/des.py":1174
 * 
 *     # Deserialize values
 *     flag: cython.uchar = unpack_uint8(data, pos[0])  # type: ignore             # <<<<<<<<<<<<<<
 *     pos[0] += 1  # skip arr dtype & flag
 *     # . Without timezone
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_pos[0])); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 1174, __pyx_L1_error)
  __pyx_v_flag = __pyx_t_1;

  /* "serializor/unicode/des.py":1175
 *     # Deserialize values
 *     flag: cython.uchar = unpack_uint8(data, pos[0])  # type: ignore
 *     pos[0] += 1  # skip arr dtype & flag             # <<<<<<<<<<<<<<
 *     # . Without timezone
 *     if flag == 0:
 */
  __pyx_t_2 = 0;
  (__pyx_v_pos[__pyx_t_2]) = ((__pyx_v_pos[__pyx_t_2]) + 1);

  /* "serializor/unicode/des.py":1177
 *     pos[0] += 1  # skip arr dtype & flag
 *     # . Without timezone
 *     if flag == 0:             # <<<<<<<<<<<<<<
 *         arr = _des_ndarray_nptime(data, pos, True)
 *         return typeref.DATETIMEINDEX(arr, name=name, copy=False)
 */
  switch (__pyx_v_flag) {
    case 0:

    /* "serializor/unicode/des.py":1178
 *     # . Without timezone
 *     if flag == 0:
 *         arr = _des_ndarray_nptime(data, pos, True)             # <<<<<<<<<<<<<<
 *         return typeref.DATETIMEINDEX(arr, name=name, copy=False)
 *     # . Timezone: approach 1 (tz_name)
 */
    __pyx_t_3 = __pyx_f_10serializor_7unicode_3des__des_ndarray_nptime(__pyx_v_data, __pyx_v_pos, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1178, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_arr = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "serializor/unicode/des.py":1179
 *     if flag == 0:
 *         arr = _des_ndarray_nptime(data, pos, True)
 *         return typeref.DATETIMEINDEX(arr, name=name, copy=False)             # <<<<<<<<<<<<<<
 *     # . Timezone: approach 1 (tz_name)
 *     elif flag == 1:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_arr);
    __Pyx_GIVEREF(__pyx_v_arr);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_arr)) __PYX_ERR(1, 1179, __pyx_L1_error);
    __pyx_t_5 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_name, __pyx_v_name) < 0) __PYX_ERR(1, 1179, __pyx_L1_error)
    if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_copy, Py_False) < 0) __PYX_ERR(1, 1179, __pyx_L1_error)
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_v_10serializor_7typeref_DATETIMEINDEX, __pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1177
 *     pos[0] += 1  # skip arr dtype & flag
 *     # . Without timezone
 *     if flag == 0:             # <<<<<<<<<<<<<<
 *         arr = _des_ndarray_nptime(data, pos, True)
 *         return typeref.DATETIMEINDEX(arr, name=name, copy=False)
 */
    break;
    case 1:

    /* "serializor/unicode/des.py":1182
 *     # . Timezone: approach 1 (tz_name)
 *     elif flag == 1:
 *         size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + size
 */
    __pyx_t_7 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_7 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 1182, __pyx_L1_error)
    __pyx_v_size = __pyx_t_7;

    /* "serializor/unicode/des.py":1183
 *     elif flag == 1:
 *         size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *         end: cython.Py_ssize_t = idx + size
 *         tz_name = data[idx:end]
 */
    __pyx_v_idx = (__pyx_v_pos[0]);

    /* "serializor/unicode/des.py":1184
 *         size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + size             # <<<<<<<<<<<<<<
 *         tz_name = data[idx:end]
 *         pos[0] = end  # set new position
 */
    __pyx_v_end = (__pyx_v_idx + __pyx_v_size);

    /* "serializor/unicode/des.py":1185
 *         idx: cython.Py_ssize_t = pos[0]
 *         end: cython.Py_ssize_t = idx + size
 *         tz_name = data[idx:end]             # <<<<<<<<<<<<<<
 *         pos[0] = end  # set new position
 *         arr = _des_ndarray_nptime(data, pos, True)
 */
    __pyx_t_6 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1185, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_v_tz_name = ((PyObject*)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":1186
 *         end: cython.Py_ssize_t = idx + size
 *         tz_name = data[idx:end]
 *         pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *         arr = _des_ndarray_nptime(data, pos, True)
 *         dindex = typeref.DATETIMEINDEX(arr, name=name, copy=False)
 */
    (__pyx_v_pos[0]) = __pyx_v_end;

    /* "serializor/unicode/des.py":1187
 *         tz_name = data[idx:end]
 *         pos[0] = end  # set new position
 *         arr = _des_ndarray_nptime(data, pos, True)             # <<<<<<<<<<<<<<
 *         dindex = typeref.DATETIMEINDEX(arr, name=name, copy=False)
 *         return dindex.tz_localize("UTC").tz_convert(tz_name)
 */
    __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__des_ndarray_nptime(__pyx_v_data, __pyx_v_pos, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_v_arr = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "serializor/unicode/des.py":1188
 *         pos[0] = end  # set new position
 *         arr = _des_ndarray_nptime(data, pos, True)
 *         dindex = typeref.DATETIMEINDEX(arr, name=name, copy=False)             # <<<<<<<<<<<<<<
 *         return dindex.tz_localize("UTC").tz_convert(tz_name)
 *     # . Timezone: approach 2 (tz_offset)
 */
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v_arr);
    __Pyx_GIVEREF(__pyx_v_arr);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_arr)) __PYX_ERR(1, 1188, __pyx_L1_error);
    __pyx_t_5 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_name, __pyx_v_name) < 0) __PYX_ERR(1, 1188, __pyx_L1_error)
    if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_copy, Py_False) < 0) __PYX_ERR(1, 1188, __pyx_L1_error)
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_v_10serializor_7typeref_DATETIMEINDEX, __pyx_t_6, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_dindex = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "serializor/unicode/des.py":1189
 *         arr = _des_ndarray_nptime(data, pos, True)
 *         dindex = typeref.DATETIMEINDEX(arr, name=name, copy=False)
 *         return dindex.tz_localize("UTC").tz_convert(tz_name)             # <<<<<<<<<<<<<<
 *     # . Timezone: approach 2 (tz_offset)
 *     else:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_dindex, __pyx_n_s_tz_localize); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    __pyx_t_9 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
        __pyx_t_9 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_n_u_UTC};
      __pyx_t_5 = __Pyx_PyObject_FastCall(__pyx_t_6, __pyx_callargs+1-__pyx_t_9, 1+__pyx_t_9);
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1189, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_tz_convert); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    __pyx_t_9 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
        __pyx_t_9 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_v_tz_name};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_6, __pyx_callargs+1-__pyx_t_9, 1+__pyx_t_9);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1189, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1181
 *         return typeref.DATETIMEINDEX(arr, name=name, copy=False)
 *     # . Timezone: approach 1 (tz_name)
 *     elif flag == 1:             # <<<<<<<<<<<<<<
 *         size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *         idx: cython.Py_ssize_t = pos[0]
 */
    break;
    default:

    /* "serializor/unicode/des.py":1192
 *     # . Timezone: approach 2 (tz_offset)
 *     else:
 *         idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *         offset = datetime.timedelta_new(
 *             unpack_int32(data, idx),  # type: ignore
 */
    __pyx_v_idx = (__pyx_v_pos[0]);

    /* "serializor/unicode/des.py":1194
 *         idx: cython.Py_ssize_t = pos[0]
 *         offset = datetime.timedelta_new(
 *             unpack_int32(data, idx),  # type: ignore             # <<<<<<<<<<<<<<
 *             unpack_int32(data, idx + 4),  # type: ignore
 *             unpack_int32(data, idx + 8),  # type: ignore
 */
    __pyx_t_10 = __pyx_f_10serializor_7unicode_3des_unpack_int32(__pyx_v_data, __pyx_v_idx); if (unlikely(__pyx_t_10 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 1194, __pyx_L1_error)

    /* "serializor/unicode/des.py":1195
 *         offset = datetime.timedelta_new(
 *             unpack_int32(data, idx),  # type: ignore
 *             unpack_int32(data, idx + 4),  # type: ignore             # <<<<<<<<<<<<<<
 *             unpack_int32(data, idx + 8),  # type: ignore
 *         )
 */
    __pyx_t_11 = __pyx_f_10serializor_7unicode_3des_unpack_int32(__pyx_v_data, (__pyx_v_idx + 4)); if (unlikely(__pyx_t_11 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 1195, __pyx_L1_error)

    /* "serializor/unicode/des.py":1196
 *             unpack_int32(data, idx),  # type: ignore
 *             unpack_int32(data, idx + 4),  # type: ignore
 *             unpack_int32(data, idx + 8),  # type: ignore             # <<<<<<<<<<<<<<
 *         )
 *         pos[0] += 12
 */
    __pyx_t_12 = __pyx_f_10serializor_7unicode_3des_unpack_int32(__pyx_v_data, (__pyx_v_idx + 8)); if (unlikely(__pyx_t_12 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 1196, __pyx_L1_error)

    /* "serializor/unicode/des.py":1193
 *     else:
 *         idx: cython.Py_ssize_t = pos[0]
 *         offset = datetime.timedelta_new(             # <<<<<<<<<<<<<<
 *             unpack_int32(data, idx),  # type: ignore
 *             unpack_int32(data, idx + 4),  # type: ignore
 */
    __pyx_t_3 = ((PyObject *)__pyx_f_7cpython_8datetime_timedelta_new(__pyx_t_10, __pyx_t_11, __pyx_t_12)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_offset = ((PyDateTime_Delta *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "serializor/unicode/des.py":1198
 *             unpack_int32(data, idx + 8),  # type: ignore
 *         )
 *         pos[0] += 12             # <<<<<<<<<<<<<<
 *         tzinfo = datetime.timezone_new(offset, None)
 *         arr = _des_ndarray_nptime(data, pos, True)
 */
    __pyx_t_2 = 0;
    (__pyx_v_pos[__pyx_t_2]) = ((__pyx_v_pos[__pyx_t_2]) + 12);

    /* "serializor/unicode/des.py":1199
 *         )
 *         pos[0] += 12
 *         tzinfo = datetime.timezone_new(offset, None)             # <<<<<<<<<<<<<<
 *         arr = _des_ndarray_nptime(data, pos, True)
 *         dindex = typeref.DATETIMEINDEX(arr, name=name, copy=False)
 */
    __pyx_t_13.__pyx_n = 1;
    __pyx_t_13.name = Py_None;
    __pyx_t_3 = __pyx_f_7cpython_8datetime_timezone_new(((PyObject *)__pyx_v_offset), &__pyx_t_13); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1199, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_tzinfo = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "serializor/unicode/des.py":1200
 *         pos[0] += 12
 *         tzinfo = datetime.timezone_new(offset, None)
 *         arr = _des_ndarray_nptime(data, pos, True)             # <<<<<<<<<<<<<<
 *         dindex = typeref.DATETIMEINDEX(arr, name=name, copy=False)
 *         return dindex.tz_localize("UTC").tz_convert(tzinfo)
 */
    __pyx_t_3 = __pyx_f_10serializor_7unicode_3des__des_ndarray_nptime(__pyx_v_data, __pyx_v_pos, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1200, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_arr = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "serializor/unicode/des.py":1201
 *         tzinfo = datetime.timezone_new(offset, None)
 *         arr = _des_ndarray_nptime(data, pos, True)
 *         dindex = typeref.DATETIMEINDEX(arr, name=name, copy=False)             # <<<<<<<<<<<<<<
 *         return dindex.tz_localize("UTC").tz_convert(tzinfo)
 * 
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1201, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_arr);
    __Pyx_GIVEREF(__pyx_v_arr);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_arr)) __PYX_ERR(1, 1201, __pyx_L1_error);
    __pyx_t_6 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1201, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_name, __pyx_v_name) < 0) __PYX_ERR(1, 1201, __pyx_L1_error)
    if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_copy, Py_False) < 0) __PYX_ERR(1, 1201, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_v_10serializor_7typeref_DATETIMEINDEX, __pyx_t_3, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1201, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_dindex = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "serializor/unicode/des.py":1202
 *         arr = _des_ndarray_nptime(data, pos, True)
 *         dindex = typeref.DATETIMEINDEX(arr, name=name, copy=False)
 *         return dindex.tz_localize("UTC").tz_convert(tzinfo)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_dindex, __pyx_n_s_tz_localize); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1202, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = NULL;
    __pyx_t_9 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_9 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_n_u_UTC};
      __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_9, 1+__pyx_t_9);
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1202, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_tz_convert); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1202, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = NULL;
    __pyx_t_9 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_9 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_v_tzinfo};
      __pyx_t_5 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_9, 1+__pyx_t_9);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1202, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
    break;
  }

  /* "serializor/unicode/des.py":1161
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_datetime_index(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("serializor.unicode.des._des_datetime_index", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XDECREF(__pyx_v_arr);
  __Pyx_XDECREF(__pyx_v_tz_name);
  __Pyx_XDECREF(__pyx_v_dindex);
  __Pyx_XDECREF((PyObject *)__pyx_v_offset);
  __Pyx_XDECREF(__pyx_v_tzinfo);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":1205
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_timedelta_index(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_timedelta_index(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  unsigned char __pyx_v_has_name;
  PyObject *__pyx_v_name = NULL;
  PyObject *__pyx_v_arr = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  unsigned char __pyx_t_1;
  long __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_timedelta_index", 1);

  /* "serializor/unicode/des.py":1213
 *     """
 *     # Deserialize name
 *     has_name: cython.uchar = unpack_uint8(data, pos[0] + 1)  # type: ignore             # <<<<<<<<<<<<<<
 *     pos[0] += 2  # skip identifier & has_name
 *     name = None if has_name == 0 else _des_data(data, pos)
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, ((__pyx_v_pos[0]) + 1)); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 1213, __pyx_L1_error)
  __pyx_v_has_name = __pyx_t_1;

  /* "serializor/unicode/des.py":1214
 *     # Deserialize name
 *     has_name: cython.uchar = unpack_uint8(data, pos[0] + 1)  # type: ignore
 *     pos[0] += 2  # skip identifier & has_name             # <<<<<<<<<<<<<<
 *     name = None if has_name == 0 else _des_data(data, pos)
 * 
 */
  __pyx_t_2 = 0;
  (__pyx_v_pos[__pyx_t_2]) = ((__pyx_v_pos[__pyx_t_2]) + 2);

  /* "serializor/unicode/des.py":1215
 *     has_name: cython.uchar = unpack_uint8(data, pos[0] + 1)  # type: ignore
 *     pos[0] += 2  # skip identifier & has_name
 *     name = None if has_name == 0 else _des_data(data, pos)             # <<<<<<<<<<<<<<
 * 
 *     # Deserialize values
 */
  __pyx_t_4 = (__pyx_v_has_name == 0);
  if (__pyx_t_4) {
    __Pyx_INCREF(Py_None);
    __pyx_t_3 = Py_None;
  } else {
    __pyx_t_5 = __pyx_f_10serializor_7unicode_3des__des_data(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1215, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = __pyx_t_5;
    __pyx_t_5 = 0;
  }
  __pyx_v_name = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "serializor/unicode/des.py":1218
 * 
 *     # Deserialize values
 *     arr = _des_ndarray_nptime(data, pos, False)             # <<<<<<<<<<<<<<
 *     return typeref.TIMEDELTAINDEX(arr, name=name, copy=False)
 * 
 */
  __pyx_t_3 = __pyx_f_10serializor_7unicode_3des__des_ndarray_nptime(__pyx_v_data, __pyx_v_pos, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1218, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_arr = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "serializor/unicode/des.py":1219
 *     # Deserialize values
 *     arr = _des_ndarray_nptime(data, pos, False)
 *     return typeref.TIMEDELTAINDEX(arr, name=name, copy=False)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_arr);
  __Pyx_GIVEREF(__pyx_v_arr);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_arr)) __PYX_ERR(1, 1219, __pyx_L1_error);
  __pyx_t_5 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_name, __pyx_v_name) < 0) __PYX_ERR(1, 1219, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_copy, Py_False) < 0) __PYX_ERR(1, 1219, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_v_10serializor_7typeref_TIMEDELTAINDEX, __pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":1205
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_timedelta_index(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("serializor.unicode.des._des_timedelta_index", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XDECREF(__pyx_v_arr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":1222
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_dataframe(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_dataframe(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  unsigned char __pyx_v_has_cols;
  PyObject *__pyx_v_cols = 0;
  unsigned char __pyx_v_has_row;
  PyObject *__pyx_v_values = 0;
  PyObject *__pyx_v_col = NULL;
  unsigned char __pyx_v_arr_dtype;
  unsigned char __pyx_v_flag;
  unsigned PY_LONG_LONG __pyx_v_size;
  Py_ssize_t __pyx_v_idx;
  Py_ssize_t __pyx_v_end;
  PyObject *__pyx_v_tz_name = 0;
  PyObject *__pyx_v_arr = NULL;
  PyObject *__pyx_v_dindex = NULL;
  PyDateTime_Delta *__pyx_v_offset = NULL;
  PyObject *__pyx_v_tzinfo = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  unsigned char __pyx_t_1;
  long __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  unsigned int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_UCS4 __pyx_t_9;
  unsigned PY_LONG_LONG __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  struct __pyx_opt_args_7cpython_8datetime_timezone_new __pyx_t_16;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_dataframe", 1);

  /* "serializor/unicode/des.py":1230
 *     """
 *     # Deserialize columns name
 *     has_cols: cython.uchar = unpack_uint8(data, pos[0] + 1)  # type: ignore             # <<<<<<<<<<<<<<
 *     pos[0] += 2  # skip identifier & has_cols
 *     # . empty w/o columns
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, ((__pyx_v_pos[0]) + 1)); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 1230, __pyx_L1_error)
  __pyx_v_has_cols = __pyx_t_1;

  /* "serializor/unicode/des.py":1231
 *     # Deserialize columns name
 *     has_cols: cython.uchar = unpack_uint8(data, pos[0] + 1)  # type: ignore
 *     pos[0] += 2  # skip identifier & has_cols             # <<<<<<<<<<<<<<
 *     # . empty w/o columns
 *     if has_cols == 0:
 */
  __pyx_t_2 = 0;
  (__pyx_v_pos[__pyx_t_2]) = ((__pyx_v_pos[__pyx_t_2]) + 2);

  /* "serializor/unicode/des.py":1233
 *     pos[0] += 2  # skip identifier & has_cols
 *     # . empty w/o columns
 *     if has_cols == 0:             # <<<<<<<<<<<<<<
 *         return typeref.DATAFRAME()  # exit
 *     cols: list = _des_list(data, pos)
 */
  __pyx_t_3 = (__pyx_v_has_cols == 0);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":1234
 *     # . empty w/o columns
 *     if has_cols == 0:
 *         return typeref.DATAFRAME()  # exit             # <<<<<<<<<<<<<<
 *     cols: list = _des_list(data, pos)
 *     has_row: cython.uchar = unpack_uint8(data, pos[0])  # type: ignore
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_10serializor_7typeref_DATAFRAME);
    __pyx_t_5 = __pyx_v_10serializor_7typeref_DATAFRAME; __pyx_t_6 = NULL;
    __pyx_t_7 = 0;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_7 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, NULL};
      __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_7, 0+__pyx_t_7);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1234, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1233
 *     pos[0] += 2  # skip identifier & has_cols
 *     # . empty w/o columns
 *     if has_cols == 0:             # <<<<<<<<<<<<<<
 *         return typeref.DATAFRAME()  # exit
 *     cols: list = _des_list(data, pos)
 */
  }

  /* "serializor/unicode/des.py":1235
 *     if has_cols == 0:
 *         return typeref.DATAFRAME()  # exit
 *     cols: list = _des_list(data, pos)             # <<<<<<<<<<<<<<
 *     has_row: cython.uchar = unpack_uint8(data, pos[0])  # type: ignore
 *     pos[0] += 1  # skip has_row
 */
  __pyx_t_4 = __pyx_f_10serializor_7unicode_3des__des_list(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v_cols = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "serializor/unicode/des.py":1236
 *         return typeref.DATAFRAME()  # exit
 *     cols: list = _des_list(data, pos)
 *     has_row: cython.uchar = unpack_uint8(data, pos[0])  # type: ignore             # <<<<<<<<<<<<<<
 *     pos[0] += 1  # skip has_row
 *     # . empty w/o rows
 */
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, (__pyx_v_pos[0])); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 1236, __pyx_L1_error)
  __pyx_v_has_row = __pyx_t_1;

  /* "serializor/unicode/des.py":1237
 *     cols: list = _des_list(data, pos)
 *     has_row: cython.uchar = unpack_uint8(data, pos[0])  # type: ignore
 *     pos[0] += 1  # skip has_row             # <<<<<<<<<<<<<<
 *     # . empty w/o rows
 *     if has_row == 0:
 */
  __pyx_t_2 = 0;
  (__pyx_v_pos[__pyx_t_2]) = ((__pyx_v_pos[__pyx_t_2]) + 1);

  /* "serializor/unicode/des.py":1239
 *     pos[0] += 1  # skip has_row
 *     # . empty w/o rows
 *     if has_row == 0:             # <<<<<<<<<<<<<<
 *         return typeref.DATAFRAME(columns=cols)  # exit
 * 
 */
  __pyx_t_3 = (__pyx_v_has_row == 0);
  if (__pyx_t_3) {

    /* "serializor/unicode/des.py":1240
 *     # . empty w/o rows
 *     if has_row == 0:
 *         return typeref.DATAFRAME(columns=cols)  # exit             # <<<<<<<<<<<<<<
 * 
 *     # Deserialize columns values
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1240, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_columns, __pyx_v_cols) < 0) __PYX_ERR(1, 1240, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_v_10serializor_7typeref_DATAFRAME, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1240, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1239
 *     pos[0] += 1  # skip has_row
 *     # . empty w/o rows
 *     if has_row == 0:             # <<<<<<<<<<<<<<
 *         return typeref.DATAFRAME(columns=cols)  # exit
 * 
 */
  }

  /* "serializor/unicode/des.py":1243
 * 
 *     # Deserialize columns values
 *     values: dict = {}             # <<<<<<<<<<<<<<
 *     for col in cols:
 *         arr_dtype: cython.uchar = data[pos[0]]
 */
  __pyx_t_5 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_v_values = ((PyObject*)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "serializor/unicode/des.py":1244
 *     # Deserialize columns values
 *     values: dict = {}
 *     for col in cols:             # <<<<<<<<<<<<<<
 *         arr_dtype: cython.uchar = data[pos[0]]
 *         # . ndarray[object]
 */
  if (unlikely(__pyx_v_cols == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(1, 1244, __pyx_L1_error)
  }
  __pyx_t_5 = __pyx_v_cols; __Pyx_INCREF(__pyx_t_5);
  __pyx_t_8 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_5);
      #if !CYTHON_ASSUME_SAFE_MACROS
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(1, 1244, __pyx_L1_error)
      #endif
      if (__pyx_t_8 >= __pyx_temp) break;
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_4 = PyList_GET_ITEM(__pyx_t_5, __pyx_t_8); __Pyx_INCREF(__pyx_t_4); __pyx_t_8++; if (unlikely((0 < 0))) __PYX_ERR(1, 1244, __pyx_L1_error)
    #else
    __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_5, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1244, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_col, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "serializor/unicode/des.py":1245
 *     values: dict = {}
 *     for col in cols:
 *         arr_dtype: cython.uchar = data[pos[0]]             # <<<<<<<<<<<<<<
 *         # . ndarray[object]
 *         if arr_dtype == identifier.NDARRAY_OBJECT:
 */
    __pyx_t_9 = __Pyx_GetItemInt_Unicode(__pyx_v_data, (__pyx_v_pos[0]), Py_ssize_t, 1, PyInt_FromSsize_t, 0, 0, 0); if (unlikely(__pyx_t_9 == (Py_UCS4)-1)) __PYX_ERR(1, 1245, __pyx_L1_error)
    __pyx_v_arr_dtype = __pyx_t_9;

    /* "serializor/unicode/des.py":1247
 *         arr_dtype: cython.uchar = data[pos[0]]
 *         # . ndarray[object]
 *         if arr_dtype == identifier.NDARRAY_OBJECT:             # <<<<<<<<<<<<<<
 *             values[col] = _des_ndarray_object(data, pos)
 *         # . ndarray[int]
 */
    __pyx_t_3 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_OBJECT);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":1248
 *         # . ndarray[object]
 *         if arr_dtype == identifier.NDARRAY_OBJECT:
 *             values[col] = _des_ndarray_object(data, pos)             # <<<<<<<<<<<<<<
 *         # . ndarray[int]
 *         elif arr_dtype == identifier.NDARRAY_INT:
 */
      __pyx_t_4 = __pyx_f_10serializor_7unicode_3des__des_ndarray_object(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1248, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (unlikely((PyDict_SetItem(__pyx_v_values, __pyx_v_col, __pyx_t_4) < 0))) __PYX_ERR(1, 1248, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "serializor/unicode/des.py":1247
 *         arr_dtype: cython.uchar = data[pos[0]]
 *         # . ndarray[object]
 *         if arr_dtype == identifier.NDARRAY_OBJECT:             # <<<<<<<<<<<<<<
 *             values[col] = _des_ndarray_object(data, pos)
 *         # . ndarray[int]
 */
      goto __pyx_L7;
    }

    /* "serializor/unicode/des.py":1250
 *             values[col] = _des_ndarray_object(data, pos)
 *         # . ndarray[int]
 *         elif arr_dtype == identifier.NDARRAY_INT:             # <<<<<<<<<<<<<<
 *             values[col] = _des_ndarray_numeric(data, pos)
 *         # . ndarray[uint]
 */
    __pyx_t_3 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_INT);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":1251
 *         # . ndarray[int]
 *         elif arr_dtype == identifier.NDARRAY_INT:
 *             values[col] = _des_ndarray_numeric(data, pos)             # <<<<<<<<<<<<<<
 *         # . ndarray[uint]
 *         elif arr_dtype == identifier.NDARRAY_UINT:
 */
      __pyx_t_4 = __pyx_f_10serializor_7unicode_3des__des_ndarray_numeric(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1251, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (unlikely((PyDict_SetItem(__pyx_v_values, __pyx_v_col, __pyx_t_4) < 0))) __PYX_ERR(1, 1251, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "serializor/unicode/des.py":1250
 *             values[col] = _des_ndarray_object(data, pos)
 *         # . ndarray[int]
 *         elif arr_dtype == identifier.NDARRAY_INT:             # <<<<<<<<<<<<<<
 *             values[col] = _des_ndarray_numeric(data, pos)
 *         # . ndarray[uint]
 */
      goto __pyx_L7;
    }

    /* "serializor/unicode/des.py":1253
 *             values[col] = _des_ndarray_numeric(data, pos)
 *         # . ndarray[uint]
 *         elif arr_dtype == identifier.NDARRAY_UINT:             # <<<<<<<<<<<<<<
 *             values[col] = _des_ndarray_numeric(data, pos)
 *         # . ndarray[float]
 */
    __pyx_t_3 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_UINT);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":1254
 *         # . ndarray[uint]
 *         elif arr_dtype == identifier.NDARRAY_UINT:
 *             values[col] = _des_ndarray_numeric(data, pos)             # <<<<<<<<<<<<<<
 *         # . ndarray[float]
 *         elif arr_dtype == identifier.NDARRAY_FLOAT:
 */
      __pyx_t_4 = __pyx_f_10serializor_7unicode_3des__des_ndarray_numeric(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1254, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (unlikely((PyDict_SetItem(__pyx_v_values, __pyx_v_col, __pyx_t_4) < 0))) __PYX_ERR(1, 1254, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "serializor/unicode/des.py":1253
 *             values[col] = _des_ndarray_numeric(data, pos)
 *         # . ndarray[uint]
 *         elif arr_dtype == identifier.NDARRAY_UINT:             # <<<<<<<<<<<<<<
 *             values[col] = _des_ndarray_numeric(data, pos)
 *         # . ndarray[float]
 */
      goto __pyx_L7;
    }

    /* "serializor/unicode/des.py":1256
 *             values[col] = _des_ndarray_numeric(data, pos)
 *         # . ndarray[float]
 *         elif arr_dtype == identifier.NDARRAY_FLOAT:             # <<<<<<<<<<<<<<
 *             values[col] = _des_ndarray_numeric(data, pos)
 *         # . ndarray[bool]
 */
    __pyx_t_3 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_FLOAT);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":1257
 *         # . ndarray[float]
 *         elif arr_dtype == identifier.NDARRAY_FLOAT:
 *             values[col] = _des_ndarray_numeric(data, pos)             # <<<<<<<<<<<<<<
 *         # . ndarray[bool]
 *         elif arr_dtype == identifier.NDARRAY_BOOL:
 */
      __pyx_t_4 = __pyx_f_10serializor_7unicode_3des__des_ndarray_numeric(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1257, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (unlikely((PyDict_SetItem(__pyx_v_values, __pyx_v_col, __pyx_t_4) < 0))) __PYX_ERR(1, 1257, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "serializor/unicode/des.py":1256
 *             values[col] = _des_ndarray_numeric(data, pos)
 *         # . ndarray[float]
 *         elif arr_dtype == identifier.NDARRAY_FLOAT:             # <<<<<<<<<<<<<<
 *             values[col] = _des_ndarray_numeric(data, pos)
 *         # . ndarray[bool]
 */
      goto __pyx_L7;
    }

    /* "serializor/unicode/des.py":1259
 *             values[col] = _des_ndarray_numeric(data, pos)
 *         # . ndarray[bool]
 *         elif arr_dtype == identifier.NDARRAY_BOOL:             # <<<<<<<<<<<<<<
 *             values[col] = _des_ndarray_numeric(data, pos)
 *         # . ndarray[datetime64]
 */
    __pyx_t_3 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_BOOL);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":1260
 *         # . ndarray[bool]
 *         elif arr_dtype == identifier.NDARRAY_BOOL:
 *             values[col] = _des_ndarray_numeric(data, pos)             # <<<<<<<<<<<<<<
 *         # . ndarray[datetime64]
 *         elif arr_dtype == identifier.NDARRAY_DT64:
 */
      __pyx_t_4 = __pyx_f_10serializor_7unicode_3des__des_ndarray_numeric(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1260, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (unlikely((PyDict_SetItem(__pyx_v_values, __pyx_v_col, __pyx_t_4) < 0))) __PYX_ERR(1, 1260, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "serializor/unicode/des.py":1259
 *             values[col] = _des_ndarray_numeric(data, pos)
 *         # . ndarray[bool]
 *         elif arr_dtype == identifier.NDARRAY_BOOL:             # <<<<<<<<<<<<<<
 *             values[col] = _des_ndarray_numeric(data, pos)
 *         # . ndarray[datetime64]
 */
      goto __pyx_L7;
    }

    /* "serializor/unicode/des.py":1262
 *             values[col] = _des_ndarray_numeric(data, pos)
 *         # . ndarray[datetime64]
 *         elif arr_dtype == identifier.NDARRAY_DT64:             # <<<<<<<<<<<<<<
 *             flag: cython.uchar = unpack_uint8(data, pos[0] + 1)  # type: ignore
 *             pos[0] += 2  # skip arr dtype & flag
 */
    __pyx_t_3 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_DT64);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":1263
 *         # . ndarray[datetime64]
 *         elif arr_dtype == identifier.NDARRAY_DT64:
 *             flag: cython.uchar = unpack_uint8(data, pos[0] + 1)  # type: ignore             # <<<<<<<<<<<<<<
 *             pos[0] += 2  # skip arr dtype & flag
 *             # . Without timezone
 */
      __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(__pyx_v_data, ((__pyx_v_pos[0]) + 1)); if (unlikely(__pyx_t_1 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 1263, __pyx_L1_error)
      __pyx_v_flag = __pyx_t_1;

      /* "serializor/unicode/des.py":1264
 *         elif arr_dtype == identifier.NDARRAY_DT64:
 *             flag: cython.uchar = unpack_uint8(data, pos[0] + 1)  # type: ignore
 *             pos[0] += 2  # skip arr dtype & flag             # <<<<<<<<<<<<<<
 *             # . Without timezone
 *             if flag == 0:
 */
      __pyx_t_2 = 0;
      (__pyx_v_pos[__pyx_t_2]) = ((__pyx_v_pos[__pyx_t_2]) + 2);

      /* "serializor/unicode/des.py":1266
 *             pos[0] += 2  # skip arr dtype & flag
 *             # . Without timezone
 *             if flag == 0:             # <<<<<<<<<<<<<<
 *                 values[col] = _des_ndarray_nptime(data, pos, True)
 *             # . Timezone: approach 1 (tz_name)
 */
      switch (__pyx_v_flag) {
        case 0:

        /* "serializor/unicode/des.py":1267
 *             # . Without timezone
 *             if flag == 0:
 *                 values[col] = _des_ndarray_nptime(data, pos, True)             # <<<<<<<<<<<<<<
 *             # . Timezone: approach 1 (tz_name)
 *             elif flag == 1:
 */
        __pyx_t_4 = __pyx_f_10serializor_7unicode_3des__des_ndarray_nptime(__pyx_v_data, __pyx_v_pos, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1267, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        if (unlikely((PyDict_SetItem(__pyx_v_values, __pyx_v_col, __pyx_t_4) < 0))) __PYX_ERR(1, 1267, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "serializor/unicode/des.py":1266
 *             pos[0] += 2  # skip arr dtype & flag
 *             # . Without timezone
 *             if flag == 0:             # <<<<<<<<<<<<<<
 *                 values[col] = _des_ndarray_nptime(data, pos, True)
 *             # . Timezone: approach 1 (tz_name)
 */
        break;
        case 1:

        /* "serializor/unicode/des.py":1270
 *             # . Timezone: approach 1 (tz_name)
 *             elif flag == 1:
 *                 size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore             # <<<<<<<<<<<<<<
 *                 idx: cython.Py_ssize_t = pos[0]
 *                 end: cython.Py_ssize_t = idx + size
 */
        __pyx_t_10 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(__pyx_v_data, __pyx_v_pos); if (unlikely(__pyx_t_10 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 1270, __pyx_L1_error)
        __pyx_v_size = __pyx_t_10;

        /* "serializor/unicode/des.py":1271
 *             elif flag == 1:
 *                 size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *                 idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *                 end: cython.Py_ssize_t = idx + size
 *                 tz_name: str = data[idx:end]
 */
        __pyx_v_idx = (__pyx_v_pos[0]);

        /* "serializor/unicode/des.py":1272
 *                 size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *                 idx: cython.Py_ssize_t = pos[0]
 *                 end: cython.Py_ssize_t = idx + size             # <<<<<<<<<<<<<<
 *                 tz_name: str = data[idx:end]
 *                 pos[0] = end  # set new position
 */
        __pyx_v_end = (__pyx_v_idx + __pyx_v_size);

        /* "serializor/unicode/des.py":1273
 *                 idx: cython.Py_ssize_t = pos[0]
 *                 end: cython.Py_ssize_t = idx + size
 *                 tz_name: str = data[idx:end]             # <<<<<<<<<<<<<<
 *                 pos[0] = end  # set new position
 *                 arr = _des_ndarray_nptime(data, pos, True)
 */
        __pyx_t_4 = __Pyx_PyUnicode_Substring(__pyx_v_data, __pyx_v_idx, __pyx_v_end); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1273, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_XDECREF_SET(__pyx_v_tz_name, ((PyObject*)__pyx_t_4));
        __pyx_t_4 = 0;

        /* "serializor/unicode/des.py":1274
 *                 end: cython.Py_ssize_t = idx + size
 *                 tz_name: str = data[idx:end]
 *                 pos[0] = end  # set new position             # <<<<<<<<<<<<<<
 *                 arr = _des_ndarray_nptime(data, pos, True)
 *                 dindex = typeref.DATETIMEINDEX(arr, copy=False)
 */
        (__pyx_v_pos[0]) = __pyx_v_end;

        /* "serializor/unicode/des.py":1275
 *                 tz_name: str = data[idx:end]
 *                 pos[0] = end  # set new position
 *                 arr = _des_ndarray_nptime(data, pos, True)             # <<<<<<<<<<<<<<
 *                 dindex = typeref.DATETIMEINDEX(arr, copy=False)
 *                 values[col] = dindex.tz_localize("UTC").tz_convert(tz_name)
 */
        __pyx_t_4 = __pyx_f_10serializor_7unicode_3des__des_ndarray_nptime(__pyx_v_data, __pyx_v_pos, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1275, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_XDECREF_SET(__pyx_v_arr, __pyx_t_4);
        __pyx_t_4 = 0;

        /* "serializor/unicode/des.py":1276
 *                 pos[0] = end  # set new position
 *                 arr = _des_ndarray_nptime(data, pos, True)
 *                 dindex = typeref.DATETIMEINDEX(arr, copy=False)             # <<<<<<<<<<<<<<
 *                 values[col] = dindex.tz_localize("UTC").tz_convert(tz_name)
 *             # . Timezone: approach 2 (tz_offset)
 */
        __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1276, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_INCREF(__pyx_v_arr);
        __Pyx_GIVEREF(__pyx_v_arr);
        if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_arr)) __PYX_ERR(1, 1276, __pyx_L1_error);
        __pyx_t_6 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1276, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_copy, Py_False) < 0) __PYX_ERR(1, 1276, __pyx_L1_error)
        __pyx_t_11 = __Pyx_PyObject_Call(__pyx_v_10serializor_7typeref_DATETIMEINDEX, __pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 1276, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF_SET(__pyx_v_dindex, __pyx_t_11);
        __pyx_t_11 = 0;

        /* "serializor/unicode/des.py":1277
 *                 arr = _des_ndarray_nptime(data, pos, True)
 *                 dindex = typeref.DATETIMEINDEX(arr, copy=False)
 *                 values[col] = dindex.tz_localize("UTC").tz_convert(tz_name)             # <<<<<<<<<<<<<<
 *             # . Timezone: approach 2 (tz_offset)
 *             else:
 */
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_dindex, __pyx_n_s_tz_localize); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1277, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_12 = NULL;
        __pyx_t_7 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_12)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_12);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
            __pyx_t_7 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_n_u_UTC};
          __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1277, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        }
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_tz_convert); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1277, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_6 = NULL;
        __pyx_t_7 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
            __pyx_t_7 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_v_tz_name};
          __pyx_t_11 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 1277, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        }
        if (unlikely((PyDict_SetItem(__pyx_v_values, __pyx_v_col, __pyx_t_11) < 0))) __PYX_ERR(1, 1277, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

        /* "serializor/unicode/des.py":1269
 *                 values[col] = _des_ndarray_nptime(data, pos, True)
 *             # . Timezone: approach 1 (tz_name)
 *             elif flag == 1:             # <<<<<<<<<<<<<<
 *                 size: cython.ulonglong = dec_encoded_int(data, pos)  # type: ignore
 *                 idx: cython.Py_ssize_t = pos[0]
 */
        break;
        default:

        /* "serializor/unicode/des.py":1280
 *             # . Timezone: approach 2 (tz_offset)
 *             else:
 *                 idx: cython.Py_ssize_t = pos[0]             # <<<<<<<<<<<<<<
 *                 offset = datetime.timedelta_new(
 *                     unpack_int32(data, idx),  # type: ignore
 */
        __pyx_v_idx = (__pyx_v_pos[0]);

        /* "serializor/unicode/des.py":1282
 *                 idx: cython.Py_ssize_t = pos[0]
 *                 offset = datetime.timedelta_new(
 *                     unpack_int32(data, idx),  # type: ignore             # <<<<<<<<<<<<<<
 *                     unpack_int32(data, idx + 4),  # type: ignore
 *                     unpack_int32(data, idx + 8),  # type: ignore
 */
        __pyx_t_13 = __pyx_f_10serializor_7unicode_3des_unpack_int32(__pyx_v_data, __pyx_v_idx); if (unlikely(__pyx_t_13 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 1282, __pyx_L1_error)

        /* "serializor/unicode/des.py":1283
 *                 offset = datetime.timedelta_new(
 *                     unpack_int32(data, idx),  # type: ignore
 *                     unpack_int32(data, idx + 4),  # type: ignore             # <<<<<<<<<<<<<<
 *                     unpack_int32(data, idx + 8),  # type: ignore
 *                 )
 */
        __pyx_t_14 = __pyx_f_10serializor_7unicode_3des_unpack_int32(__pyx_v_data, (__pyx_v_idx + 4)); if (unlikely(__pyx_t_14 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 1283, __pyx_L1_error)

        /* "serializor/unicode/des.py":1284
 *                     unpack_int32(data, idx),  # type: ignore
 *                     unpack_int32(data, idx + 4),  # type: ignore
 *                     unpack_int32(data, idx + 8),  # type: ignore             # <<<<<<<<<<<<<<
 *                 )
 *                 pos[0] += 12  # set new position
 */
        __pyx_t_15 = __pyx_f_10serializor_7unicode_3des_unpack_int32(__pyx_v_data, (__pyx_v_idx + 8)); if (unlikely(__pyx_t_15 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 1284, __pyx_L1_error)

        /* "serializor/unicode/des.py":1281
 *             else:
 *                 idx: cython.Py_ssize_t = pos[0]
 *                 offset = datetime.timedelta_new(             # <<<<<<<<<<<<<<
 *                     unpack_int32(data, idx),  # type: ignore
 *                     unpack_int32(data, idx + 4),  # type: ignore
 */
        __pyx_t_11 = ((PyObject *)__pyx_f_7cpython_8datetime_timedelta_new(__pyx_t_13, __pyx_t_14, __pyx_t_15)); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 1281, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_XDECREF_SET(__pyx_v_offset, ((PyDateTime_Delta *)__pyx_t_11));
        __pyx_t_11 = 0;

        /* "serializor/unicode/des.py":1286
 *                     unpack_int32(data, idx + 8),  # type: ignore
 *                 )
 *                 pos[0] += 12  # set new position             # <<<<<<<<<<<<<<
 *                 tzinfo = datetime.timezone_new(offset, None)
 *                 arr = _des_ndarray_nptime(data, pos, True)
 */
        __pyx_t_2 = 0;
        (__pyx_v_pos[__pyx_t_2]) = ((__pyx_v_pos[__pyx_t_2]) + 12);

        /* "serializor/unicode/des.py":1287
 *                 )
 *                 pos[0] += 12  # set new position
 *                 tzinfo = datetime.timezone_new(offset, None)             # <<<<<<<<<<<<<<
 *                 arr = _des_ndarray_nptime(data, pos, True)
 *                 dindex = typeref.DATETIMEINDEX(arr, copy=False)
 */
        __pyx_t_16.__pyx_n = 1;
        __pyx_t_16.name = Py_None;
        __pyx_t_11 = __pyx_f_7cpython_8datetime_timezone_new(((PyObject *)__pyx_v_offset), &__pyx_t_16); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 1287, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_XDECREF_SET(__pyx_v_tzinfo, __pyx_t_11);
        __pyx_t_11 = 0;

        /* "serializor/unicode/des.py":1288
 *                 pos[0] += 12  # set new position
 *                 tzinfo = datetime.timezone_new(offset, None)
 *                 arr = _des_ndarray_nptime(data, pos, True)             # <<<<<<<<<<<<<<
 *                 dindex = typeref.DATETIMEINDEX(arr, copy=False)
 *                 values[col] = dindex.tz_localize("UTC").tz_convert(tzinfo)
 */
        __pyx_t_11 = __pyx_f_10serializor_7unicode_3des__des_ndarray_nptime(__pyx_v_data, __pyx_v_pos, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 1288, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_XDECREF_SET(__pyx_v_arr, __pyx_t_11);
        __pyx_t_11 = 0;

        /* "serializor/unicode/des.py":1289
 *                 tzinfo = datetime.timezone_new(offset, None)
 *                 arr = _des_ndarray_nptime(data, pos, True)
 *                 dindex = typeref.DATETIMEINDEX(arr, copy=False)             # <<<<<<<<<<<<<<
 *                 values[col] = dindex.tz_localize("UTC").tz_convert(tzinfo)
 *         # . ndarray[timedelta64]
 */
        __pyx_t_11 = PyTuple_New(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 1289, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_INCREF(__pyx_v_arr);
        __Pyx_GIVEREF(__pyx_v_arr);
        if (__Pyx_PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_v_arr)) __PYX_ERR(1, 1289, __pyx_L1_error);
        __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1289, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_copy, Py_False) < 0) __PYX_ERR(1, 1289, __pyx_L1_error)
        __pyx_t_6 = __Pyx_PyObject_Call(__pyx_v_10serializor_7typeref_DATETIMEINDEX, __pyx_t_11, __pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1289, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF_SET(__pyx_v_dindex, __pyx_t_6);
        __pyx_t_6 = 0;

        /* "serializor/unicode/des.py":1290
 *                 arr = _des_ndarray_nptime(data, pos, True)
 *                 dindex = typeref.DATETIMEINDEX(arr, copy=False)
 *                 values[col] = dindex.tz_localize("UTC").tz_convert(tzinfo)             # <<<<<<<<<<<<<<
 *         # . ndarray[timedelta64]
 *         elif arr_dtype == identifier.NDARRAY_TD64:
 */
        __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_dindex, __pyx_n_s_tz_localize); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 1290, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_12 = NULL;
        __pyx_t_7 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_11))) {
          __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_11);
          if (likely(__pyx_t_12)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
            __Pyx_INCREF(__pyx_t_12);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_11, function);
            __pyx_t_7 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_n_u_UTC};
          __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_11, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1290, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
        __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_tz_convert); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 1290, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = NULL;
        __pyx_t_7 = 0;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_11))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_11);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_11, function);
            __pyx_t_7 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_tzinfo};
          __pyx_t_6 = __Pyx_PyObject_FastCall(__pyx_t_11, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1290, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
        if (unlikely((PyDict_SetItem(__pyx_v_values, __pyx_v_col, __pyx_t_6) < 0))) __PYX_ERR(1, 1290, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        break;
      }

      /* "serializor/unicode/des.py":1262
 *             values[col] = _des_ndarray_numeric(data, pos)
 *         # . ndarray[datetime64]
 *         elif arr_dtype == identifier.NDARRAY_DT64:             # <<<<<<<<<<<<<<
 *             flag: cython.uchar = unpack_uint8(data, pos[0] + 1)  # type: ignore
 *             pos[0] += 2  # skip arr dtype & flag
 */
      goto __pyx_L7;
    }

    /* "serializor/unicode/des.py":1292
 *                 values[col] = dindex.tz_localize("UTC").tz_convert(tzinfo)
 *         # . ndarray[timedelta64]
 *         elif arr_dtype == identifier.NDARRAY_TD64:             # <<<<<<<<<<<<<<
 *             values[col] = _des_ndarray_nptime(data, pos, False)
 *         # . ndarray[complex]
 */
    __pyx_t_3 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_TD64);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":1293
 *         # . ndarray[timedelta64]
 *         elif arr_dtype == identifier.NDARRAY_TD64:
 *             values[col] = _des_ndarray_nptime(data, pos, False)             # <<<<<<<<<<<<<<
 *         # . ndarray[complex]
 *         elif arr_dtype == identifier.NDARRAY_COMPLEX:
 */
      __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__des_ndarray_nptime(__pyx_v_data, __pyx_v_pos, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1293, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (unlikely((PyDict_SetItem(__pyx_v_values, __pyx_v_col, __pyx_t_6) < 0))) __PYX_ERR(1, 1293, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "serializor/unicode/des.py":1292
 *                 values[col] = dindex.tz_localize("UTC").tz_convert(tzinfo)
 *         # . ndarray[timedelta64]
 *         elif arr_dtype == identifier.NDARRAY_TD64:             # <<<<<<<<<<<<<<
 *             values[col] = _des_ndarray_nptime(data, pos, False)
 *         # . ndarray[complex]
 */
      goto __pyx_L7;
    }

    /* "serializor/unicode/des.py":1295
 *             values[col] = _des_ndarray_nptime(data, pos, False)
 *         # . ndarray[complex]
 *         elif arr_dtype == identifier.NDARRAY_COMPLEX:             # <<<<<<<<<<<<<<
 *             values[col] = _des_ndarray_complex(data, pos)
 *         # . ndarray[bytes]
 */
    __pyx_t_3 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_COMPLEX);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":1296
 *         # . ndarray[complex]
 *         elif arr_dtype == identifier.NDARRAY_COMPLEX:
 *             values[col] = _des_ndarray_complex(data, pos)             # <<<<<<<<<<<<<<
 *         # . ndarray[bytes]
 *         elif arr_dtype == identifier.NDARRAY_BYTES:
 */
      __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__des_ndarray_complex(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1296, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (unlikely((PyDict_SetItem(__pyx_v_values, __pyx_v_col, __pyx_t_6) < 0))) __PYX_ERR(1, 1296, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "serializor/unicode/des.py":1295
 *             values[col] = _des_ndarray_nptime(data, pos, False)
 *         # . ndarray[complex]
 *         elif arr_dtype == identifier.NDARRAY_COMPLEX:             # <<<<<<<<<<<<<<
 *             values[col] = _des_ndarray_complex(data, pos)
 *         # . ndarray[bytes]
 */
      goto __pyx_L7;
    }

    /* "serializor/unicode/des.py":1298
 *             values[col] = _des_ndarray_complex(data, pos)
 *         # . ndarray[bytes]
 *         elif arr_dtype == identifier.NDARRAY_BYTES:             # <<<<<<<<<<<<<<
 *             values[col] = _des_ndarray_bytes(data, pos)
 *         # . ndarray[str]
 */
    __pyx_t_3 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_BYTES);
    if (__pyx_t_3) {

      /* "serializor/unicode/des.py":1299
 *         # . ndarray[bytes]
 *         elif arr_dtype == identifier.NDARRAY_BYTES:
 *             values[col] = _des_ndarray_bytes(data, pos)             # <<<<<<<<<<<<<<
 *         # . ndarray[str]
 *         elif arr_dtype == identifier.NDARRAY_UNICODE:
 */
      __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__des_ndarray_bytes(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1299, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (unlikely((PyDict_SetItem(__pyx_v_values, __pyx_v_col, __pyx_t_6) < 0))) __PYX_ERR(1, 1299, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "serializor/unicode/des.py":1298
 *             values[col] = _des_ndarray_complex(data, pos)
 *         # . ndarray[bytes]
 *         elif arr_dtype == identifier.NDARRAY_BYTES:             # <<<<<<<<<<<<<<
 *             values[col] = _des_ndarray_bytes(data, pos)
 *         # . ndarray[str]
 */
      goto __pyx_L7;
    }

    /* "serializor/unicode/des.py":1301
 *             values[col] = _des_ndarray_bytes(data, pos)
 *         # . ndarray[str]
 *         elif arr_dtype == identifier.NDARRAY_UNICODE:             # <<<<<<<<<<<<<<
 *             values[col] = _des_ndarray_unicode(data, pos)
 *         # . invalid dtype
 */
    __pyx_t_3 = (__pyx_v_arr_dtype == __pyx_v_10serializor_10identifier_NDARRAY_UNICODE);
    if (likely(__pyx_t_3)) {

      /* "serializor/unicode/des.py":1302
 *         # . ndarray[str]
 *         elif arr_dtype == identifier.NDARRAY_UNICODE:
 *             values[col] = _des_ndarray_unicode(data, pos)             # <<<<<<<<<<<<<<
 *         # . invalid dtype
 *         else:
 */
      __pyx_t_6 = __pyx_f_10serializor_7unicode_3des__des_ndarray_unicode(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1302, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (unlikely((PyDict_SetItem(__pyx_v_values, __pyx_v_col, __pyx_t_6) < 0))) __PYX_ERR(1, 1302, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "serializor/unicode/des.py":1301
 *             values[col] = _des_ndarray_bytes(data, pos)
 *         # . ndarray[str]
 *         elif arr_dtype == identifier.NDARRAY_UNICODE:             # <<<<<<<<<<<<<<
 *             values[col] = _des_ndarray_unicode(data, pos)
 *         # . invalid dtype
 */
      goto __pyx_L7;
    }

    /* "serializor/unicode/des.py":1305
 *         # . invalid dtype
 *         else:
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 "unsupported <'pandas.DataFrame'> column's dtype '%s'." % chr(arr_dtype)
 *             )
 */
    /*else*/ {

      /* "serializor/unicode/des.py":1306
 *         else:
 *             raise ValueError(
 *                 "unsupported <'pandas.DataFrame'> column's dtype '%s'." % chr(arr_dtype)             # <<<<<<<<<<<<<<
 *             )
 * 
 */
      __pyx_t_6 = __Pyx_PyInt_From_unsigned_char(__pyx_v_arr_dtype); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1306, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_11 = __Pyx_PyObject_CallOneArg(__pyx_builtin_chr, __pyx_t_6); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 1306, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyUnicode_FormatSafe(__pyx_kp_u_unsupported_pandas_DataFrame_col, __pyx_t_11); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1306, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

      /* "serializor/unicode/des.py":1305
 *         # . invalid dtype
 *         else:
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 "unsupported <'pandas.DataFrame'> column's dtype '%s'." % chr(arr_dtype)
 *             )
 */
      __pyx_t_11 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_6); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 1305, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_Raise(__pyx_t_11, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __PYX_ERR(1, 1305, __pyx_L1_error)
    }
    __pyx_L7:;

    /* "serializor/unicode/des.py":1244
 *     # Deserialize columns values
 *     values: dict = {}
 *     for col in cols:             # <<<<<<<<<<<<<<
 *         arr_dtype: cython.uchar = data[pos[0]]
 *         # . ndarray[object]
 */
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "serializor/unicode/des.py":1310
 * 
 *     # Generate DataFrame
 *     return typeref.DATAFRAME(values, columns=cols, copy=False)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1310, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_v_values);
  __Pyx_GIVEREF(__pyx_v_values);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_values)) __PYX_ERR(1, 1310, __pyx_L1_error);
  __pyx_t_11 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 1310, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_columns, __pyx_v_cols) < 0) __PYX_ERR(1, 1310, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_copy, Py_False) < 0) __PYX_ERR(1, 1310, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_v_10serializor_7typeref_DATAFRAME, __pyx_t_5, __pyx_t_11); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 1310, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "serializor/unicode/des.py":1222
 * 
 * 
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_dataframe(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("serializor.unicode.des._des_dataframe", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cols);
  __Pyx_XDECREF(__pyx_v_values);
  __Pyx_XDECREF(__pyx_v_col);
  __Pyx_XDECREF(__pyx_v_tz_name);
  __Pyx_XDECREF(__pyx_v_arr);
  __Pyx_XDECREF(__pyx_v_dindex);
  __Pyx_XDECREF((PyObject *)__pyx_v_offset);
  __Pyx_XDECREF(__pyx_v_tzinfo);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":1314
 * 
 * # Deserialize ---------------------------------------------------------------------------------
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_data(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

static PyObject *__pyx_f_10serializor_7unicode_3des__des_data(PyObject *__pyx_v_data, Py_ssize_t *__pyx_v_pos) {
  char __pyx_v_dtype_id;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_des_data", 1);

  /* "serializor/unicode/des.py":1321
 *     Position updates to the next byte after deserialization.
 *     """
 *     dtype_id: cython.char = read_char(data, pos[0])             # <<<<<<<<<<<<<<
 * 
 *     # Basic Types
 */
  __pyx_v_dtype_id = PyUnicode_READ_CHAR(__pyx_v_data, (__pyx_v_pos[0]));

  /* "serializor/unicode/des.py":1325
 *     # Basic Types
 *     # <'str'>
 *     if dtype_id == identifier.STR:             # <<<<<<<<<<<<<<
 *         return _des_str(data, pos)
 *     # <'int'>
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_STR);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1326
 *     # <'str'>
 *     if dtype_id == identifier.STR:
 *         return _des_str(data, pos)             # <<<<<<<<<<<<<<
 *     # <'int'>
 *     if dtype_id == identifier.INT:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_str(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1326, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1325
 *     # Basic Types
 *     # <'str'>
 *     if dtype_id == identifier.STR:             # <<<<<<<<<<<<<<
 *         return _des_str(data, pos)
 *     # <'int'>
 */
  }

  /* "serializor/unicode/des.py":1328
 *         return _des_str(data, pos)
 *     # <'int'>
 *     if dtype_id == identifier.INT:             # <<<<<<<<<<<<<<
 *         return _des_int(data, pos)
 *     # <'float'>
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_INT);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1329
 *     # <'int'>
 *     if dtype_id == identifier.INT:
 *         return _des_int(data, pos)             # <<<<<<<<<<<<<<
 *     # <'float'>
 *     if dtype_id == identifier.FLOAT:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_int(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1329, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1328
 *         return _des_str(data, pos)
 *     # <'int'>
 *     if dtype_id == identifier.INT:             # <<<<<<<<<<<<<<
 *         return _des_int(data, pos)
 *     # <'float'>
 */
  }

  /* "serializor/unicode/des.py":1331
 *         return _des_int(data, pos)
 *     # <'float'>
 *     if dtype_id == identifier.FLOAT:             # <<<<<<<<<<<<<<
 *         return _des_float(data, pos)
 *     # <'bool'>
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_FLOAT);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1332
 *     # <'float'>
 *     if dtype_id == identifier.FLOAT:
 *         return _des_float(data, pos)             # <<<<<<<<<<<<<<
 *     # <'bool'>
 *     if dtype_id == identifier.BOOL:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_float(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1332, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1331
 *         return _des_int(data, pos)
 *     # <'float'>
 *     if dtype_id == identifier.FLOAT:             # <<<<<<<<<<<<<<
 *         return _des_float(data, pos)
 *     # <'bool'>
 */
  }

  /* "serializor/unicode/des.py":1334
 *         return _des_float(data, pos)
 *     # <'bool'>
 *     if dtype_id == identifier.BOOL:             # <<<<<<<<<<<<<<
 *         return _des_bool(data, pos)
 *     # <'NoneType'>
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_BOOL);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1335
 *     # <'bool'>
 *     if dtype_id == identifier.BOOL:
 *         return _des_bool(data, pos)             # <<<<<<<<<<<<<<
 *     # <'NoneType'>
 *     if dtype_id == identifier.NONE:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_bool(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1335, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1334
 *         return _des_float(data, pos)
 *     # <'bool'>
 *     if dtype_id == identifier.BOOL:             # <<<<<<<<<<<<<<
 *         return _des_bool(data, pos)
 *     # <'NoneType'>
 */
  }

  /* "serializor/unicode/des.py":1337
 *         return _des_bool(data, pos)
 *     # <'NoneType'>
 *     if dtype_id == identifier.NONE:             # <<<<<<<<<<<<<<
 *         return _des_none(data, pos)
 * 
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_NONE);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1338
 *     # <'NoneType'>
 *     if dtype_id == identifier.NONE:
 *         return _des_none(data, pos)             # <<<<<<<<<<<<<<
 * 
 *     # Date&Time Types
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_none(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1338, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1337
 *         return _des_bool(data, pos)
 *     # <'NoneType'>
 *     if dtype_id == identifier.NONE:             # <<<<<<<<<<<<<<
 *         return _des_none(data, pos)
 * 
 */
  }

  /* "serializor/unicode/des.py":1342
 *     # Date&Time Types
 *     # <'datetime.datetime'>
 *     if dtype_id == identifier.DATETIME:             # <<<<<<<<<<<<<<
 *         return _des_datetime(data, pos)
 *     # <'datetime.date'>
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_DATETIME);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1343
 *     # <'datetime.datetime'>
 *     if dtype_id == identifier.DATETIME:
 *         return _des_datetime(data, pos)             # <<<<<<<<<<<<<<
 *     # <'datetime.date'>
 *     if dtype_id == identifier.DATE:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_datetime(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1343, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1342
 *     # Date&Time Types
 *     # <'datetime.datetime'>
 *     if dtype_id == identifier.DATETIME:             # <<<<<<<<<<<<<<
 *         return _des_datetime(data, pos)
 *     # <'datetime.date'>
 */
  }

  /* "serializor/unicode/des.py":1345
 *         return _des_datetime(data, pos)
 *     # <'datetime.date'>
 *     if dtype_id == identifier.DATE:             # <<<<<<<<<<<<<<
 *         return _des_date(data, pos)
 *     # <'datetime.time'>
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_DATE);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1346
 *     # <'datetime.date'>
 *     if dtype_id == identifier.DATE:
 *         return _des_date(data, pos)             # <<<<<<<<<<<<<<
 *     # <'datetime.time'>
 *     if dtype_id == identifier.TIME:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_date(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1346, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1345
 *         return _des_datetime(data, pos)
 *     # <'datetime.date'>
 *     if dtype_id == identifier.DATE:             # <<<<<<<<<<<<<<
 *         return _des_date(data, pos)
 *     # <'datetime.time'>
 */
  }

  /* "serializor/unicode/des.py":1348
 *         return _des_date(data, pos)
 *     # <'datetime.time'>
 *     if dtype_id == identifier.TIME:             # <<<<<<<<<<<<<<
 *         return _des_time(data, pos)
 *     # <'datetime.timedelta'>
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_TIME);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1349
 *     # <'datetime.time'>
 *     if dtype_id == identifier.TIME:
 *         return _des_time(data, pos)             # <<<<<<<<<<<<<<
 *     # <'datetime.timedelta'>
 *     if dtype_id == identifier.TIMEDELTA:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_time(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1349, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1348
 *         return _des_date(data, pos)
 *     # <'datetime.time'>
 *     if dtype_id == identifier.TIME:             # <<<<<<<<<<<<<<
 *         return _des_time(data, pos)
 *     # <'datetime.timedelta'>
 */
  }

  /* "serializor/unicode/des.py":1351
 *         return _des_time(data, pos)
 *     # <'datetime.timedelta'>
 *     if dtype_id == identifier.TIMEDELTA:             # <<<<<<<<<<<<<<
 *         return _des_timedelta(data, pos)
 *     # <'time.struct_time'>
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_TIMEDELTA);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1352
 *     # <'datetime.timedelta'>
 *     if dtype_id == identifier.TIMEDELTA:
 *         return _des_timedelta(data, pos)             # <<<<<<<<<<<<<<
 *     # <'time.struct_time'>
 *     if dtype_id == identifier.STRUCT_TIME:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_timedelta(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1352, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1351
 *         return _des_time(data, pos)
 *     # <'datetime.timedelta'>
 *     if dtype_id == identifier.TIMEDELTA:             # <<<<<<<<<<<<<<
 *         return _des_timedelta(data, pos)
 *     # <'time.struct_time'>
 */
  }

  /* "serializor/unicode/des.py":1354
 *         return _des_timedelta(data, pos)
 *     # <'time.struct_time'>
 *     if dtype_id == identifier.STRUCT_TIME:             # <<<<<<<<<<<<<<
 *         return _des_struct_time(data, pos)
 *     # <'pandas.Timestamp'>
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_STRUCT_TIME);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1355
 *     # <'time.struct_time'>
 *     if dtype_id == identifier.STRUCT_TIME:
 *         return _des_struct_time(data, pos)             # <<<<<<<<<<<<<<
 *     # <'pandas.Timestamp'>
 *     if dtype_id == identifier.PD_TIMESTAMP:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_struct_time(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1355, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1354
 *         return _des_timedelta(data, pos)
 *     # <'time.struct_time'>
 *     if dtype_id == identifier.STRUCT_TIME:             # <<<<<<<<<<<<<<
 *         return _des_struct_time(data, pos)
 *     # <'pandas.Timestamp'>
 */
  }

  /* "serializor/unicode/des.py":1357
 *         return _des_struct_time(data, pos)
 *     # <'pandas.Timestamp'>
 *     if dtype_id == identifier.PD_TIMESTAMP:             # <<<<<<<<<<<<<<
 *         return _des_pd_timestamp(data, pos)
 *     # <'pandas.Timedelta'>
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_PD_TIMESTAMP);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1358
 *     # <'pandas.Timestamp'>
 *     if dtype_id == identifier.PD_TIMESTAMP:
 *         return _des_pd_timestamp(data, pos)             # <<<<<<<<<<<<<<
 *     # <'pandas.Timedelta'>
 *     if dtype_id == identifier.PD_TIMEDELTA:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_pd_timestamp(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1358, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1357
 *         return _des_struct_time(data, pos)
 *     # <'pandas.Timestamp'>
 *     if dtype_id == identifier.PD_TIMESTAMP:             # <<<<<<<<<<<<<<
 *         return _des_pd_timestamp(data, pos)
 *     # <'pandas.Timedelta'>
 */
  }

  /* "serializor/unicode/des.py":1360
 *         return _des_pd_timestamp(data, pos)
 *     # <'pandas.Timedelta'>
 *     if dtype_id == identifier.PD_TIMEDELTA:             # <<<<<<<<<<<<<<
 *         return _des_pd_timedelta(data, pos)
 *     # <'numpy.datetime64'>
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_PD_TIMEDELTA);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1361
 *     # <'pandas.Timedelta'>
 *     if dtype_id == identifier.PD_TIMEDELTA:
 *         return _des_pd_timedelta(data, pos)             # <<<<<<<<<<<<<<
 *     # <'numpy.datetime64'>
 *     if dtype_id == identifier.DATETIME64:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_pd_timedelta(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1361, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1360
 *         return _des_pd_timestamp(data, pos)
 *     # <'pandas.Timedelta'>
 *     if dtype_id == identifier.PD_TIMEDELTA:             # <<<<<<<<<<<<<<
 *         return _des_pd_timedelta(data, pos)
 *     # <'numpy.datetime64'>
 */
  }

  /* "serializor/unicode/des.py":1363
 *         return _des_pd_timedelta(data, pos)
 *     # <'numpy.datetime64'>
 *     if dtype_id == identifier.DATETIME64:             # <<<<<<<<<<<<<<
 *         return _des_datetime64(data, pos)
 *     # <'numpy.timedelta64'>
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_DATETIME64);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1364
 *     # <'numpy.datetime64'>
 *     if dtype_id == identifier.DATETIME64:
 *         return _des_datetime64(data, pos)             # <<<<<<<<<<<<<<
 *     # <'numpy.timedelta64'>
 *     if dtype_id == identifier.TIMEDELTA64:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_datetime64(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1364, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1363
 *         return _des_pd_timedelta(data, pos)
 *     # <'numpy.datetime64'>
 *     if dtype_id == identifier.DATETIME64:             # <<<<<<<<<<<<<<
 *         return _des_datetime64(data, pos)
 *     # <'numpy.timedelta64'>
 */
  }

  /* "serializor/unicode/des.py":1366
 *         return _des_datetime64(data, pos)
 *     # <'numpy.timedelta64'>
 *     if dtype_id == identifier.TIMEDELTA64:             # <<<<<<<<<<<<<<
 *         return _des_timedelta64(data, pos)
 * 
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_TIMEDELTA64);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1367
 *     # <'numpy.timedelta64'>
 *     if dtype_id == identifier.TIMEDELTA64:
 *         return _des_timedelta64(data, pos)             # <<<<<<<<<<<<<<
 * 
 *     # Numeric Types
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_timedelta64(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1367, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1366
 *         return _des_datetime64(data, pos)
 *     # <'numpy.timedelta64'>
 *     if dtype_id == identifier.TIMEDELTA64:             # <<<<<<<<<<<<<<
 *         return _des_timedelta64(data, pos)
 * 
 */
  }

  /* "serializor/unicode/des.py":1371
 *     # Numeric Types
 *     # <'decimal.Decimal'>
 *     if dtype_id == identifier.DECIMAL:             # <<<<<<<<<<<<<<
 *         return _des_decimal(data, pos)
 *     # <'complex'>
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_DECIMAL);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1372
 *     # <'decimal.Decimal'>
 *     if dtype_id == identifier.DECIMAL:
 *         return _des_decimal(data, pos)             # <<<<<<<<<<<<<<
 *     # <'complex'>
 *     if dtype_id == identifier.COMPLEX:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_decimal(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1372, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1371
 *     # Numeric Types
 *     # <'decimal.Decimal'>
 *     if dtype_id == identifier.DECIMAL:             # <<<<<<<<<<<<<<
 *         return _des_decimal(data, pos)
 *     # <'complex'>
 */
  }

  /* "serializor/unicode/des.py":1374
 *         return _des_decimal(data, pos)
 *     # <'complex'>
 *     if dtype_id == identifier.COMPLEX:             # <<<<<<<<<<<<<<
 *         return _des_complex(data, pos)
 * 
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_COMPLEX);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1375
 *     # <'complex'>
 *     if dtype_id == identifier.COMPLEX:
 *         return _des_complex(data, pos)             # <<<<<<<<<<<<<<
 * 
 *     # Bytes Types
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_complex(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1375, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1374
 *         return _des_decimal(data, pos)
 *     # <'complex'>
 *     if dtype_id == identifier.COMPLEX:             # <<<<<<<<<<<<<<
 *         return _des_complex(data, pos)
 * 
 */
  }

  /* "serializor/unicode/des.py":1379
 *     # Bytes Types
 *     # <'bytes'>
 *     if dtype_id == identifier.BYTES:             # <<<<<<<<<<<<<<
 *         return _des_bytes(data, pos)
 *     # <'bytearray'>
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_BYTES);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1380
 *     # <'bytes'>
 *     if dtype_id == identifier.BYTES:
 *         return _des_bytes(data, pos)             # <<<<<<<<<<<<<<
 *     # <'bytearray'>
 *     if dtype_id == identifier.BYTEARRAY:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_bytes(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1380, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1379
 *     # Bytes Types
 *     # <'bytes'>
 *     if dtype_id == identifier.BYTES:             # <<<<<<<<<<<<<<
 *         return _des_bytes(data, pos)
 *     # <'bytearray'>
 */
  }

  /* "serializor/unicode/des.py":1382
 *         return _des_bytes(data, pos)
 *     # <'bytearray'>
 *     if dtype_id == identifier.BYTEARRAY:             # <<<<<<<<<<<<<<
 *         return _des_bytearray(data, pos)
 * 
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_BYTEARRAY);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1383
 *     # <'bytearray'>
 *     if dtype_id == identifier.BYTEARRAY:
 *         return _des_bytearray(data, pos)             # <<<<<<<<<<<<<<
 * 
 *     # Sequence Types
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_bytearray(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1383, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1382
 *         return _des_bytes(data, pos)
 *     # <'bytearray'>
 *     if dtype_id == identifier.BYTEARRAY:             # <<<<<<<<<<<<<<
 *         return _des_bytearray(data, pos)
 * 
 */
  }

  /* "serializor/unicode/des.py":1387
 *     # Sequence Types
 *     # <'list'>
 *     if dtype_id == identifier.LIST:             # <<<<<<<<<<<<<<
 *         return _des_list(data, pos)
 *     # <'tuple'>
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_LIST);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1388
 *     # <'list'>
 *     if dtype_id == identifier.LIST:
 *         return _des_list(data, pos)             # <<<<<<<<<<<<<<
 *     # <'tuple'>
 *     if dtype_id == identifier.TUPLE:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_list(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1388, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1387
 *     # Sequence Types
 *     # <'list'>
 *     if dtype_id == identifier.LIST:             # <<<<<<<<<<<<<<
 *         return _des_list(data, pos)
 *     # <'tuple'>
 */
  }

  /* "serializor/unicode/des.py":1390
 *         return _des_list(data, pos)
 *     # <'tuple'>
 *     if dtype_id == identifier.TUPLE:             # <<<<<<<<<<<<<<
 *         return _des_tuple(data, pos)
 *     # <'set'>
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_TUPLE);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1391
 *     # <'tuple'>
 *     if dtype_id == identifier.TUPLE:
 *         return _des_tuple(data, pos)             # <<<<<<<<<<<<<<
 *     # <'set'>
 *     if dtype_id == identifier.SET:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_tuple(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1391, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1390
 *         return _des_list(data, pos)
 *     # <'tuple'>
 *     if dtype_id == identifier.TUPLE:             # <<<<<<<<<<<<<<
 *         return _des_tuple(data, pos)
 *     # <'set'>
 */
  }

  /* "serializor/unicode/des.py":1393
 *         return _des_tuple(data, pos)
 *     # <'set'>
 *     if dtype_id == identifier.SET:             # <<<<<<<<<<<<<<
 *         return _des_set(data, pos)
 *     # <'frozenset'>
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_SET);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1394
 *     # <'set'>
 *     if dtype_id == identifier.SET:
 *         return _des_set(data, pos)             # <<<<<<<<<<<<<<
 *     # <'frozenset'>
 *     if dtype_id == identifier.FROZENSET:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_set(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1394, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1393
 *         return _des_tuple(data, pos)
 *     # <'set'>
 *     if dtype_id == identifier.SET:             # <<<<<<<<<<<<<<
 *         return _des_set(data, pos)
 *     # <'frozenset'>
 */
  }

  /* "serializor/unicode/des.py":1396
 *         return _des_set(data, pos)
 *     # <'frozenset'>
 *     if dtype_id == identifier.FROZENSET:             # <<<<<<<<<<<<<<
 *         return _des_frozenset(data, pos)
 *     # <'range'>
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_FROZENSET);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1397
 *     # <'frozenset'>
 *     if dtype_id == identifier.FROZENSET:
 *         return _des_frozenset(data, pos)             # <<<<<<<<<<<<<<
 *     # <'range'>
 *     if dtype_id == identifier.RANGE:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_frozenset(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1397, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1396
 *         return _des_set(data, pos)
 *     # <'frozenset'>
 *     if dtype_id == identifier.FROZENSET:             # <<<<<<<<<<<<<<
 *         return _des_frozenset(data, pos)
 *     # <'range'>
 */
  }

  /* "serializor/unicode/des.py":1399
 *         return _des_frozenset(data, pos)
 *     # <'range'>
 *     if dtype_id == identifier.RANGE:             # <<<<<<<<<<<<<<
 *         return _des_range(data, pos)
 * 
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_RANGE);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1400
 *     # <'range'>
 *     if dtype_id == identifier.RANGE:
 *         return _des_range(data, pos)             # <<<<<<<<<<<<<<
 * 
 *     # Mapping Types
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_range(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1400, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1399
 *         return _des_frozenset(data, pos)
 *     # <'range'>
 *     if dtype_id == identifier.RANGE:             # <<<<<<<<<<<<<<
 *         return _des_range(data, pos)
 * 
 */
  }

  /* "serializor/unicode/des.py":1404
 *     # Mapping Types
 *     # <'dict'>
 *     if dtype_id == identifier.DICT:             # <<<<<<<<<<<<<<
 *         return _des_dict(data, pos)
 * 
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_DICT);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1405
 *     # <'dict'>
 *     if dtype_id == identifier.DICT:
 *         return _des_dict(data, pos)             # <<<<<<<<<<<<<<
 * 
 *     # NumPy Types
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_dict(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1405, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1404
 *     # Mapping Types
 *     # <'dict'>
 *     if dtype_id == identifier.DICT:             # <<<<<<<<<<<<<<
 *         return _des_dict(data, pos)
 * 
 */
  }

  /* "serializor/unicode/des.py":1409
 *     # NumPy Types
 *     # <'numpy.ndarray'>
 *     if dtype_id == identifier.NDARRAY:             # <<<<<<<<<<<<<<
 *         return _des_ndarray(data, pos)
 * 
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_NDARRAY);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1410
 *     # <'numpy.ndarray'>
 *     if dtype_id == identifier.NDARRAY:
 *         return _des_ndarray(data, pos)             # <<<<<<<<<<<<<<
 * 
 *     # Pandas Types
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_ndarray(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1410, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1409
 *     # NumPy Types
 *     # <'numpy.ndarray'>
 *     if dtype_id == identifier.NDARRAY:             # <<<<<<<<<<<<<<
 *         return _des_ndarray(data, pos)
 * 
 */
  }

  /* "serializor/unicode/des.py":1414
 *     # Pandas Types
 *     # <'pandas.Series'>
 *     if dtype_id == identifier.SERIES:             # <<<<<<<<<<<<<<
 *         return _des_series(data, pos)
 *     # <'pandas.DataFrame'>
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_SERIES);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1415
 *     # <'pandas.Series'>
 *     if dtype_id == identifier.SERIES:
 *         return _des_series(data, pos)             # <<<<<<<<<<<<<<
 *     # <'pandas.DataFrame'>
 *     if dtype_id == identifier.DATAFRAME:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_series(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1415, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1414
 *     # Pandas Types
 *     # <'pandas.Series'>
 *     if dtype_id == identifier.SERIES:             # <<<<<<<<<<<<<<
 *         return _des_series(data, pos)
 *     # <'pandas.DataFrame'>
 */
  }

  /* "serializor/unicode/des.py":1417
 *         return _des_series(data, pos)
 *     # <'pandas.DataFrame'>
 *     if dtype_id == identifier.DATAFRAME:             # <<<<<<<<<<<<<<
 *         return _des_dataframe(data, pos)
 *     # <'pandas.DatetimeIndex'>
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_DATAFRAME);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1418
 *     # <'pandas.DataFrame'>
 *     if dtype_id == identifier.DATAFRAME:
 *         return _des_dataframe(data, pos)             # <<<<<<<<<<<<<<
 *     # <'pandas.DatetimeIndex'>
 *     if dtype_id == identifier.DATETIMEINDEX:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_dataframe(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1418, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1417
 *         return _des_series(data, pos)
 *     # <'pandas.DataFrame'>
 *     if dtype_id == identifier.DATAFRAME:             # <<<<<<<<<<<<<<
 *         return _des_dataframe(data, pos)
 *     # <'pandas.DatetimeIndex'>
 */
  }

  /* "serializor/unicode/des.py":1420
 *         return _des_dataframe(data, pos)
 *     # <'pandas.DatetimeIndex'>
 *     if dtype_id == identifier.DATETIMEINDEX:             # <<<<<<<<<<<<<<
 *         return _des_datetime_index(data, pos)
 *     # <'pandas.TimedeltaIndex'>
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_DATETIMEINDEX);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1421
 *     # <'pandas.DatetimeIndex'>
 *     if dtype_id == identifier.DATETIMEINDEX:
 *         return _des_datetime_index(data, pos)             # <<<<<<<<<<<<<<
 *     # <'pandas.TimedeltaIndex'>
 *     if dtype_id == identifier.TIMEDELTAINDEX:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_datetime_index(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1421, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1420
 *         return _des_dataframe(data, pos)
 *     # <'pandas.DatetimeIndex'>
 *     if dtype_id == identifier.DATETIMEINDEX:             # <<<<<<<<<<<<<<
 *         return _des_datetime_index(data, pos)
 *     # <'pandas.TimedeltaIndex'>
 */
  }

  /* "serializor/unicode/des.py":1423
 *         return _des_datetime_index(data, pos)
 *     # <'pandas.TimedeltaIndex'>
 *     if dtype_id == identifier.TIMEDELTAINDEX:             # <<<<<<<<<<<<<<
 *         return _des_timedelta_index(data, pos)
 * 
 */
  __pyx_t_1 = (__pyx_v_dtype_id == __pyx_v_10serializor_10identifier_TIMEDELTAINDEX);
  if (__pyx_t_1) {

    /* "serializor/unicode/des.py":1424
 *     # <'pandas.TimedeltaIndex'>
 *     if dtype_id == identifier.TIMEDELTAINDEX:
 *         return _des_timedelta_index(data, pos)             # <<<<<<<<<<<<<<
 * 
 *     # Invalid 'data'
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_10serializor_7unicode_3des__des_timedelta_index(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1424, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "serializor/unicode/des.py":1423
 *         return _des_datetime_index(data, pos)
 *     # <'pandas.TimedeltaIndex'>
 *     if dtype_id == identifier.TIMEDELTAINDEX:             # <<<<<<<<<<<<<<
 *         return _des_timedelta_index(data, pos)
 * 
 */
  }

  /* "serializor/unicode/des.py":1427
 * 
 *     # Invalid 'data'
 *     raise ValueError("unknown 'data' identifer '%s'." % chr(dtype_id))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_From_char(__pyx_v_dtype_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1427, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_chr, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1427, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyUnicode_FormatSafe(__pyx_kp_u_unknown_data_identifer_s, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1427, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1427, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_Raise(__pyx_t_3, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(1, 1427, __pyx_L1_error)

  /* "serializor/unicode/des.py":1314
 * 
 * # Deserialize ---------------------------------------------------------------------------------
 * @cython.cfunc             # <<<<<<<<<<<<<<
 * @cython.inline(True)
 * def _des_data(data: str, pos: cython.Py_ssize_t[1]) -> object:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("serializor.unicode.des._des_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":1430
 * 
 * 
 * @cython.ccall             # <<<<<<<<<<<<<<
 * def deserialize(data: str) -> object:
 *     """Deserialize the data (str) back to an `<'object'>`."""
 */

static PyObject *__pyx_pw_10serializor_7unicode_3des_1deserialize(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_10serializor_7unicode_3des_deserialize(PyObject *__pyx_v_data, CYTHON_UNUSED int __pyx_skip_dispatch) {
  Py_ssize_t __pyx_v_eof;
  Py_ssize_t __pyx_v_pos[1];
  PyObject *__pyx_v_obj = NULL;
  PyObject *__pyx_v_err = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6[1];
  Py_ssize_t __pyx_t_7;
  Py_UCS4 __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  unsigned int __pyx_t_16;
  int __pyx_t_17;
  char const *__pyx_t_18;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  PyObject *__pyx_t_24 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 1);

  /* "serializor/unicode/des.py":1433
 * def deserialize(data: str) -> object:
 *     """Deserialize the data (str) back to an `<'object'>`."""
 *     try:             # <<<<<<<<<<<<<<
 *         eof: cython.Py_ssize_t = str_len(data)
 *         if eof == 0:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "serializor/unicode/des.py":1434
 *     """Deserialize the data (str) back to an `<'object'>`."""
 *     try:
 *         eof: cython.Py_ssize_t = str_len(data)             # <<<<<<<<<<<<<<
 *         if eof == 0:
 *             raise ValueError("'data' is empty.")
 */
      __pyx_v_eof = PyUnicode_GET_LENGTH(__pyx_v_data);

      /* "serializor/unicode/des.py":1435
 *     try:
 *         eof: cython.Py_ssize_t = str_len(data)
 *         if eof == 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("'data' is empty.")
 *         pos: cython.Py_ssize_t[1] = [0]
 */
      __pyx_t_4 = (__pyx_v_eof == 0);
      if (unlikely(__pyx_t_4)) {

        /* "serializor/unicode/des.py":1436
 *         eof: cython.Py_ssize_t = str_len(data)
 *         if eof == 0:
 *             raise ValueError("'data' is empty.")             # <<<<<<<<<<<<<<
 *         pos: cython.Py_ssize_t[1] = [0]
 *         obj = _des_data(data, pos)
 */
        __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1436, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_Raise(__pyx_t_5, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __PYX_ERR(1, 1436, __pyx_L3_error)

        /* "serializor/unicode/des.py":1435
 *     try:
 *         eof: cython.Py_ssize_t = str_len(data)
 *         if eof == 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("'data' is empty.")
 *         pos: cython.Py_ssize_t[1] = [0]
 */
      }

      /* "serializor/unicode/des.py":1437
 *         if eof == 0:
 *             raise ValueError("'data' is empty.")
 *         pos: cython.Py_ssize_t[1] = [0]             # <<<<<<<<<<<<<<
 *         obj = _des_data(data, pos)
 *         if pos[0] != eof:
 */
      __pyx_t_6[0] = 0;
      memcpy(&(__pyx_v_pos[0]), __pyx_t_6, sizeof(__pyx_v_pos[0]) * (1));

      /* "serializor/unicode/des.py":1438
 *             raise ValueError("'data' is empty.")
 *         pos: cython.Py_ssize_t[1] = [0]
 *         obj = _des_data(data, pos)             # <<<<<<<<<<<<<<
 *         if pos[0] != eof:
 *             raise ValueError("improper [pos] %d / [eof] %d." % (pos[0], eof))
 */
      __pyx_t_5 = __pyx_f_10serializor_7unicode_3des__des_data(__pyx_v_data, __pyx_v_pos); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1438, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_v_obj = __pyx_t_5;
      __pyx_t_5 = 0;

      /* "serializor/unicode/des.py":1439
 *         pos: cython.Py_ssize_t[1] = [0]
 *         obj = _des_data(data, pos)
 *         if pos[0] != eof:             # <<<<<<<<<<<<<<
 *             raise ValueError("improper [pos] %d / [eof] %d." % (pos[0], eof))
 *         return obj
 */
      __pyx_t_4 = ((__pyx_v_pos[0]) != __pyx_v_eof);
      if (unlikely(__pyx_t_4)) {

        /* "serializor/unicode/des.py":1440
 *         obj = _des_data(data, pos)
 *         if pos[0] != eof:
 *             raise ValueError("improper [pos] %d / [eof] %d." % (pos[0], eof))             # <<<<<<<<<<<<<<
 *         return obj
 *     except Exception as err:
 */
        __pyx_t_5 = PyTuple_New(5); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1440, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_7 = 0;
        __pyx_t_8 = 127;
        __Pyx_INCREF(__pyx_kp_u_improper_pos);
        __pyx_t_7 += 15;
        __Pyx_GIVEREF(__pyx_kp_u_improper_pos);
        PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_kp_u_improper_pos);
        __pyx_t_9 = __Pyx_PyUnicode_From_Py_ssize_t((__pyx_v_pos[0]), 0, ' ', 'd'); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 1440, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_9);
        __Pyx_GIVEREF(__pyx_t_9);
        PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_9);
        __pyx_t_9 = 0;
        __Pyx_INCREF(__pyx_kp_u_eof);
        __pyx_t_7 += 9;
        __Pyx_GIVEREF(__pyx_kp_u_eof);
        PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_kp_u_eof);
        __pyx_t_9 = __Pyx_PyUnicode_From_Py_ssize_t(__pyx_v_eof, 0, ' ', 'd'); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 1440, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_9);
        __Pyx_GIVEREF(__pyx_t_9);
        PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_9);
        __pyx_t_9 = 0;
        __Pyx_INCREF(__pyx_kp_u__5);
        __pyx_t_7 += 1;
        __Pyx_GIVEREF(__pyx_kp_u__5);
        PyTuple_SET_ITEM(__pyx_t_5, 4, __pyx_kp_u__5);
        __pyx_t_9 = __Pyx_PyUnicode_Join(__pyx_t_5, 5, __pyx_t_7, __pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 1440, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_9); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1440, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_Raise(__pyx_t_5, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __PYX_ERR(1, 1440, __pyx_L3_error)

        /* "serializor/unicode/des.py":1439
 *         pos: cython.Py_ssize_t[1] = [0]
 *         obj = _des_data(data, pos)
 *         if pos[0] != eof:             # <<<<<<<<<<<<<<
 *             raise ValueError("improper [pos] %d / [eof] %d." % (pos[0], eof))
 *         return obj
 */
      }

      /* "serializor/unicode/des.py":1441
 *         if pos[0] != eof:
 *             raise ValueError("improper [pos] %d / [eof] %d." % (pos[0], eof))
 *         return obj             # <<<<<<<<<<<<<<
 *     except Exception as err:
 *         raise errors.DeserializeError(
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_v_obj);
      __pyx_r = __pyx_v_obj;
      goto __pyx_L7_try_return;

      /* "serializor/unicode/des.py":1433
 * def deserialize(data: str) -> object:
 *     """Deserialize the data (str) back to an `<'object'>`."""
 *     try:             # <<<<<<<<<<<<<<
 *         eof: cython.Py_ssize_t = str_len(data)
 *         if eof == 0:
 */
    }
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "serializor/unicode/des.py":1442
 *             raise ValueError("improper [pos] %d / [eof] %d." % (pos[0], eof))
 *         return obj
 *     except Exception as err:             # <<<<<<<<<<<<<<
 *         raise errors.DeserializeError(
 *             "<'Serializor'>\nFailed to deserialize from: "
 */
    __pyx_t_10 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
    if (__pyx_t_10) {
      __Pyx_AddTraceback("serializor.unicode.des.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_9, &__pyx_t_11) < 0) __PYX_ERR(1, 1442, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_11);
      __Pyx_INCREF(__pyx_t_9);
      __pyx_v_err = __pyx_t_9;
      /*try:*/ {

        /* "serializor/unicode/des.py":1443
 *         return obj
 *     except Exception as err:
 *         raise errors.DeserializeError(             # <<<<<<<<<<<<<<
 *             "<'Serializor'>\nFailed to deserialize from: "
 *             "%s\n%r\nError: %s" % (type(data), data, err)
 */
        __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_n_s_errors); if (unlikely(!__pyx_t_13)) __PYX_ERR(1, 1443, __pyx_L16_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_13, __pyx_n_s_DeserializeError); if (unlikely(!__pyx_t_14)) __PYX_ERR(1, 1443, __pyx_L16_error)
        __Pyx_GOTREF(__pyx_t_14);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

        /* "serializor/unicode/des.py":1444
 *     except Exception as err:
 *         raise errors.DeserializeError(
 *             "<'Serializor'>\nFailed to deserialize from: "             # <<<<<<<<<<<<<<
 *             "%s\n%r\nError: %s" % (type(data), data, err)
 *         ) from err
 */
        __pyx_t_13 = PyTuple_New(6); if (unlikely(!__pyx_t_13)) __PYX_ERR(1, 1444, __pyx_L16_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_7 = 0;
        __pyx_t_8 = 127;
        __Pyx_INCREF(__pyx_kp_u_Serializor_Failed_to_deserializ);
        __pyx_t_7 += 43;
        __Pyx_GIVEREF(__pyx_kp_u_Serializor_Failed_to_deserializ);
        PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_kp_u_Serializor_Failed_to_deserializ);

        /* "serializor/unicode/des.py":1445
 *         raise errors.DeserializeError(
 *             "<'Serializor'>\nFailed to deserialize from: "
 *             "%s\n%r\nError: %s" % (type(data), data, err)             # <<<<<<<<<<<<<<
 *         ) from err
 * 
 */
        __pyx_t_15 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Unicode(((PyObject *)Py_TYPE(__pyx_v_data))), __pyx_empty_unicode); if (unlikely(!__pyx_t_15)) __PYX_ERR(1, 1445, __pyx_L16_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_8 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_15) > __pyx_t_8) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_15) : __pyx_t_8;
        __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_15);
        __Pyx_GIVEREF(__pyx_t_15);
        PyTuple_SET_ITEM(__pyx_t_13, 1, __pyx_t_15);
        __pyx_t_15 = 0;
        __Pyx_INCREF(__pyx_kp_u__6);
        __pyx_t_7 += 1;
        __Pyx_GIVEREF(__pyx_kp_u__6);
        PyTuple_SET_ITEM(__pyx_t_13, 2, __pyx_kp_u__6);
        __pyx_t_15 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_data), __pyx_empty_unicode); if (unlikely(!__pyx_t_15)) __PYX_ERR(1, 1445, __pyx_L16_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_8 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_15) > __pyx_t_8) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_15) : __pyx_t_8;
        __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_15);
        __Pyx_GIVEREF(__pyx_t_15);
        PyTuple_SET_ITEM(__pyx_t_13, 3, __pyx_t_15);
        __pyx_t_15 = 0;
        __Pyx_INCREF(__pyx_kp_u_Error);
        __pyx_t_7 += 8;
        __Pyx_GIVEREF(__pyx_kp_u_Error);
        PyTuple_SET_ITEM(__pyx_t_13, 4, __pyx_kp_u_Error);
        __pyx_t_15 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Unicode(__pyx_v_err), __pyx_empty_unicode); if (unlikely(!__pyx_t_15)) __PYX_ERR(1, 1445, __pyx_L16_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_8 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_15) > __pyx_t_8) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_15) : __pyx_t_8;
        __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_15);
        __Pyx_GIVEREF(__pyx_t_15);
        PyTuple_SET_ITEM(__pyx_t_13, 5, __pyx_t_15);
        __pyx_t_15 = 0;

        /* "serializor/unicode/des.py":1444
 *     except Exception as err:
 *         raise errors.DeserializeError(
 *             "<'Serializor'>\nFailed to deserialize from: "             # <<<<<<<<<<<<<<
 *             "%s\n%r\nError: %s" % (type(data), data, err)
 *         ) from err
 */
        __pyx_t_15 = __Pyx_PyUnicode_Join(__pyx_t_13, 6, __pyx_t_7, __pyx_t_8); if (unlikely(!__pyx_t_15)) __PYX_ERR(1, 1444, __pyx_L16_error)
        __Pyx_GOTREF(__pyx_t_15);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_13 = NULL;
        __pyx_t_16 = 0;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_14))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_14);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_14, function);
            __pyx_t_16 = 1;
          }
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_13, __pyx_t_15};
          __pyx_t_12 = __Pyx_PyObject_FastCall(__pyx_t_14, __pyx_callargs+1-__pyx_t_16, 1+__pyx_t_16);
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          if (unlikely(!__pyx_t_12)) __PYX_ERR(1, 1443, __pyx_L16_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        }

        /* "serializor/unicode/des.py":1446
 *             "<'Serializor'>\nFailed to deserialize from: "
 *             "%s\n%r\nError: %s" % (type(data), data, err)
 *         ) from err             # <<<<<<<<<<<<<<
 * 
 * 
 */
        __Pyx_Raise(__pyx_t_12, 0, 0, __pyx_v_err);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __PYX_ERR(1, 1443, __pyx_L16_error)
      }

      /* "serializor/unicode/des.py":1442
 *             raise ValueError("improper [pos] %d / [eof] %d." % (pos[0], eof))
 *         return obj
 *     except Exception as err:             # <<<<<<<<<<<<<<
 *         raise errors.DeserializeError(
 *             "<'Serializor'>\nFailed to deserialize from: "
 */
      /*finally:*/ {
        __pyx_L16_error:;
        /*exception exit:*/{
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0;
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_22, &__pyx_t_23, &__pyx_t_24);
          if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_19, &__pyx_t_20, &__pyx_t_21) < 0)) __Pyx_ErrFetch(&__pyx_t_19, &__pyx_t_20, &__pyx_t_21);
          __Pyx_XGOTREF(__pyx_t_19);
          __Pyx_XGOTREF(__pyx_t_20);
          __Pyx_XGOTREF(__pyx_t_21);
          __Pyx_XGOTREF(__pyx_t_22);
          __Pyx_XGOTREF(__pyx_t_23);
          __Pyx_XGOTREF(__pyx_t_24);
          __pyx_t_10 = __pyx_lineno; __pyx_t_17 = __pyx_clineno; __pyx_t_18 = __pyx_filename;
          {
            __Pyx_DECREF(__pyx_v_err); __pyx_v_err = 0;
          }
          if (PY_MAJOR_VERSION >= 3) {
            __Pyx_XGIVEREF(__pyx_t_22);
            __Pyx_XGIVEREF(__pyx_t_23);
            __Pyx_XGIVEREF(__pyx_t_24);
            __Pyx_ExceptionReset(__pyx_t_22, __pyx_t_23, __pyx_t_24);
          }
          __Pyx_XGIVEREF(__pyx_t_19);
          __Pyx_XGIVEREF(__pyx_t_20);
          __Pyx_XGIVEREF(__pyx_t_21);
          __Pyx_ErrRestore(__pyx_t_19, __pyx_t_20, __pyx_t_21);
          __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0;
          __pyx_lineno = __pyx_t_10; __pyx_clineno = __pyx_t_17; __pyx_filename = __pyx_t_18;
          goto __pyx_L5_except_error;
        }
      }
    }
    goto __pyx_L5_except_error;

    /* "serializor/unicode/des.py":1433
 * def deserialize(data: str) -> object:
 *     """Deserialize the data (str) back to an `<'object'>`."""
 *     try:             # <<<<<<<<<<<<<<
 *         eof: cython.Py_ssize_t = str_len(data)
 *         if eof == 0:
 */
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L7_try_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
  }

  /* "serializor/unicode/des.py":1430
 * 
 * 
 * @cython.ccall             # <<<<<<<<<<<<<<
 * def deserialize(data: str) -> object:
 *     """Deserialize the data (str) back to an `<'object'>`."""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_AddTraceback("serializor.unicode.des.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_obj);
  __Pyx_XDECREF(__pyx_v_err);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_10serializor_7unicode_3des_1deserialize(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_10serializor_7unicode_3des_deserialize, "Deserialize the data (str) back to an `<'object'>`.");
static PyMethodDef __pyx_mdef_10serializor_7unicode_3des_1deserialize = {"deserialize", (PyCFunction)(void*)(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_10serializor_7unicode_3des_1deserialize, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_10serializor_7unicode_3des_deserialize};
static PyObject *__pyx_pw_10serializor_7unicode_3des_1deserialize(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_data = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deserialize (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_MACROS
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,0};
    if (__pyx_kwds) {
      Py_ssize_t kw_args;
      switch (__pyx_nargs) {
        case  1: values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = __Pyx_NumKwargs_FASTCALL(__pyx_kwds);
      switch (__pyx_nargs) {
        case  0:
        if (likely((values[0] = __Pyx_GetKwValue_FASTCALL(__pyx_kwds, __pyx_kwvalues, __pyx_n_s_data)) != 0)) {
          (void)__Pyx_Arg_NewRef_FASTCALL(values[0]);
          kw_args--;
        }
        else if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 1430, __pyx_L3_error)
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t kwd_pos_args = __pyx_nargs;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values + 0, kwd_pos_args, "deserialize") < 0)) __PYX_ERR(1, 1430, __pyx_L3_error)
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_Arg_FASTCALL(__pyx_args, 0);
    }
    __pyx_v_data = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("deserialize", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 1430, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_AddTraceback("serializor.unicode.des.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyUnicode_Type), 0, "data", 1))) __PYX_ERR(1, 1431, __pyx_L1_error)
  __pyx_r = __pyx_pf_10serializor_7unicode_3des_deserialize(__pyx_self, __pyx_v_data);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  {
    Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
      __Pyx_Arg_XDECREF_FASTCALL(values[__pyx_temp]);
    }
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10serializor_7unicode_3des_deserialize(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("deserialize", 1);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_10serializor_7unicode_3des_deserialize(__pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("serializor.unicode.des.deserialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":1450
 * 
 * ########## The following functions are for testing purpose only ##########
 * def _test_utils() -> None:             # <<<<<<<<<<<<<<
 *     _test_unpack_int8()
 *     _test_unpack_uint8()
 */

/* Python wrapper */
static PyObject *__pyx_pw_10serializor_7unicode_3des_3_test_utils(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_10serializor_7unicode_3des_3_test_utils = {"_test_utils", (PyCFunction)__pyx_pw_10serializor_7unicode_3des_3_test_utils, METH_NOARGS, 0};
static PyObject *__pyx_pw_10serializor_7unicode_3des_3_test_utils(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_test_utils (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10serializor_7unicode_3des_2_test_utils(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10serializor_7unicode_3des_2_test_utils(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  unsigned int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_test_utils", 1);

  /* "serializor/unicode/des.py":1451
 * ########## The following functions are for testing purpose only ##########
 * def _test_utils() -> None:
 *     _test_unpack_int8()             # <<<<<<<<<<<<<<
 *     _test_unpack_uint8()
 *     _test_unpack_int16()
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_test_unpack_int8); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1451, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 0+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1451, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1452
 * def _test_utils() -> None:
 *     _test_unpack_int8()
 *     _test_unpack_uint8()             # <<<<<<<<<<<<<<
 *     _test_unpack_int16()
 *     _test_unpack_uint16()
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_test_unpack_uint8); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1452, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 0+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1452, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1453
 *     _test_unpack_int8()
 *     _test_unpack_uint8()
 *     _test_unpack_int16()             # <<<<<<<<<<<<<<
 *     _test_unpack_uint16()
 *     _test_unpack_int24()
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_test_unpack_int16); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 0+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1453, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1454
 *     _test_unpack_uint8()
 *     _test_unpack_int16()
 *     _test_unpack_uint16()             # <<<<<<<<<<<<<<
 *     _test_unpack_int24()
 *     _test_unpack_uint24()
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_test_unpack_uint16); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1454, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 0+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1454, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1455
 *     _test_unpack_int16()
 *     _test_unpack_uint16()
 *     _test_unpack_int24()             # <<<<<<<<<<<<<<
 *     _test_unpack_uint24()
 *     _test_unpack_int32()
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_test_unpack_int24); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1455, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 0+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1455, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1456
 *     _test_unpack_uint16()
 *     _test_unpack_int24()
 *     _test_unpack_uint24()             # <<<<<<<<<<<<<<
 *     _test_unpack_int32()
 *     _test_unpack_uint32()
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_test_unpack_uint24); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1456, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 0+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1456, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1457
 *     _test_unpack_int24()
 *     _test_unpack_uint24()
 *     _test_unpack_int32()             # <<<<<<<<<<<<<<
 *     _test_unpack_uint32()
 *     _test_unpack_int64()
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_test_unpack_int32); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1457, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 0+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1457, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1458
 *     _test_unpack_uint24()
 *     _test_unpack_int32()
 *     _test_unpack_uint32()             # <<<<<<<<<<<<<<
 *     _test_unpack_int64()
 *     _test_unpack_uint64()
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_test_unpack_uint32); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1458, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 0+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1458, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1459
 *     _test_unpack_int32()
 *     _test_unpack_uint32()
 *     _test_unpack_int64()             # <<<<<<<<<<<<<<
 *     _test_unpack_uint64()
 *     _test_dec_encoded_int()
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_test_unpack_int64); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1459, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 0+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1459, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1460
 *     _test_unpack_uint32()
 *     _test_unpack_int64()
 *     _test_unpack_uint64()             # <<<<<<<<<<<<<<
 *     _test_dec_encoded_int()
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_test_unpack_uint64); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 0+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1460, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1461
 *     _test_unpack_int64()
 *     _test_unpack_uint64()
 *     _test_dec_encoded_int()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_test_dec_encoded_int); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1461, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_2, __pyx_callargs+1-__pyx_t_4, 0+__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1461, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1450
 * 
 * ########## The following functions are for testing purpose only ##########
 * def _test_utils() -> None:             # <<<<<<<<<<<<<<
 *     _test_unpack_int8()
 *     _test_unpack_uint8()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("serializor.unicode.des._test_utils", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":1464
 * 
 * 
 * def _test_unpack_int8() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_10serializor_7unicode_3des_5_test_unpack_int8(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_10serializor_7unicode_3des_5_test_unpack_int8 = {"_test_unpack_int8", (PyCFunction)__pyx_pw_10serializor_7unicode_3des_5_test_unpack_int8, METH_NOARGS, 0};
static PyObject *__pyx_pw_10serializor_7unicode_3des_5_test_unpack_int8(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_test_unpack_int8 (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10serializor_7unicode_3des_4_test_unpack_int8(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10serializor_7unicode_3des_4_test_unpack_int8(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_v_struct = NULL;
  PyObject *__pyx_v_val = NULL;
  PyObject *__pyx_v_b = NULL;
  signed char __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  long __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  unsigned int __pyx_t_6;
  signed char __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_UCS4 __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_test_unpack_int8", 1);

  /* "serializor/unicode/des.py":1465
 * 
 * def _test_unpack_int8() -> None:
 *     import struct             # <<<<<<<<<<<<<<
 * 
 *     for val in range(-128, 128):
 */
  __pyx_t_1 = __Pyx_ImportDottedModule(__pyx_n_s_struct, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1465, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_struct = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1467
 *     import struct
 * 
 *     for val in range(-128, 128):             # <<<<<<<<<<<<<<
 *         b = struct.pack("<b", val).decode("latin1")
 *         i = unpack_int8(b, 0)  # type: ignore
 */
  for (__pyx_t_2 = -128L; __pyx_t_2 < 0x80; __pyx_t_2+=1) {
    __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1467, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_val, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "serializor/unicode/des.py":1468
 * 
 *     for val in range(-128, 128):
 *         b = struct.pack("<b", val).decode("latin1")             # <<<<<<<<<<<<<<
 *         i = unpack_int8(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_pack); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1468, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    __pyx_t_6 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_6 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_5, __pyx_kp_u_b, __pyx_v_val};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_6, 2+__pyx_t_6);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1468, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_decode); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1468, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    __pyx_t_6 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_6 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_n_u_latin1};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_6, 1+__pyx_t_6);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1468, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_XDECREF_SET(__pyx_v_b, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "serializor/unicode/des.py":1469
 *     for val in range(-128, 128):
 *         b = struct.pack("<b", val).decode("latin1")
 *         i = unpack_int8(b, 0)  # type: ignore             # <<<<<<<<<<<<<<
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")
 */
    if (!(likely(PyUnicode_CheckExact(__pyx_v_b))||((__pyx_v_b) == Py_None) || __Pyx_RaiseUnexpectedTypeError("unicode", __pyx_v_b))) __PYX_ERR(1, 1469, __pyx_L1_error)
    __pyx_t_7 = __pyx_f_10serializor_7unicode_3des_unpack_int8(((PyObject*)__pyx_v_b), 0); if (unlikely(__pyx_t_7 == ((signed char)-1) && PyErr_Occurred())) __PYX_ERR(1, 1469, __pyx_L1_error)
    __pyx_v_i = __pyx_t_7;

    /* "serializor/unicode/des.py":1470
 *         b = struct.pack("<b", val).decode("latin1")
 *         i = unpack_int8(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"             # <<<<<<<<<<<<<<
 *         print("- pass: %s" % val, end="\r")
 *     print("Pass Unpack int8".ljust(80))
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(__pyx_assertions_enabled())) {
      __pyx_t_1 = __Pyx_PyInt_From_signed_char(__pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1470, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_val, __pyx_t_1, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1470, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(1, 1470, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_8)) {
        __pyx_t_4 = PyTuple_New(4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1470, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_9 = 0;
        __pyx_t_10 = 127;
        __Pyx_INCREF(__pyx_kp_u_val);
        __pyx_t_9 += 4;
        __Pyx_GIVEREF(__pyx_kp_u_val);
        PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_kp_u_val);
        __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_v_val, __pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1470, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_10 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) > __pyx_t_10) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) : __pyx_t_10;
        __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1);
        __Pyx_GIVEREF(__pyx_t_1);
        PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
        __pyx_t_1 = 0;
        __Pyx_INCREF(__pyx_kp_u_vs_i);
        __pyx_t_9 += 6;
        __Pyx_GIVEREF(__pyx_kp_u_vs_i);
        PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_kp_u_vs_i);
        __pyx_t_1 = __Pyx_PyUnicode_From_signed_char(__pyx_v_i, 0, ' ', 'd'); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1470, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1);
        __Pyx_GIVEREF(__pyx_t_1);
        PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_1);
        __pyx_t_1 = 0;
        __pyx_t_1 = __Pyx_PyUnicode_Join(__pyx_t_4, 4, __pyx_t_9, __pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1470, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_Raise(__pyx_builtin_AssertionError, __pyx_t_1, 0, 0);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __PYX_ERR(1, 1470, __pyx_L1_error)
      }
    }
    #else
    if ((1)); else __PYX_ERR(1, 1470, __pyx_L1_error)
    #endif

    /* "serializor/unicode/des.py":1471
 *         i = unpack_int8(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")             # <<<<<<<<<<<<<<
 *     print("Pass Unpack int8".ljust(80))
 * 
 */
    __pyx_t_1 = __Pyx_PyUnicode_FormatSafe(__pyx_kp_u_pass_s, __pyx_v_val); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1471, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1471, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1)) __PYX_ERR(1, 1471, __pyx_L1_error);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1471, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_end, __pyx_kp_u__8) < 0) __PYX_ERR(1, 1471, __pyx_L1_error)
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_t_4, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1471, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }

  /* "serializor/unicode/des.py":1472
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")
 *     print("Pass Unpack int8".ljust(80))             # <<<<<<<<<<<<<<
 * 
 *     del struct
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_Pass_Unpack_int8, __pyx_n_s_ljust); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1472, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = NULL;
  __pyx_t_6 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
      __pyx_t_6 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_int_80};
    __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_6, 1+__pyx_t_6);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1472, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1472, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1474
 *     print("Pass Unpack int8".ljust(80))
 * 
 *     del struct             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_DECREF(__pyx_v_struct); __pyx_v_struct = 0;

  /* "serializor/unicode/des.py":1464
 * 
 * 
 * def _test_unpack_int8() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("serializor.unicode.des._test_unpack_int8", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_struct);
  __Pyx_XDECREF(__pyx_v_val);
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":1477
 * 
 * 
 * def _test_unpack_uint8() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_10serializor_7unicode_3des_7_test_unpack_uint8(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_10serializor_7unicode_3des_7_test_unpack_uint8 = {"_test_unpack_uint8", (PyCFunction)__pyx_pw_10serializor_7unicode_3des_7_test_unpack_uint8, METH_NOARGS, 0};
static PyObject *__pyx_pw_10serializor_7unicode_3des_7_test_unpack_uint8(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_test_unpack_uint8 (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10serializor_7unicode_3des_6_test_unpack_uint8(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10serializor_7unicode_3des_6_test_unpack_uint8(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_v_struct = NULL;
  PyObject *__pyx_v_val = NULL;
  PyObject *__pyx_v_b = NULL;
  unsigned char __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  long __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  unsigned int __pyx_t_6;
  unsigned char __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_UCS4 __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_test_unpack_uint8", 1);

  /* "serializor/unicode/des.py":1478
 * 
 * def _test_unpack_uint8() -> None:
 *     import struct             # <<<<<<<<<<<<<<
 * 
 *     for val in range(0, 256):
 */
  __pyx_t_1 = __Pyx_ImportDottedModule(__pyx_n_s_struct, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1478, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_struct = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1480
 *     import struct
 * 
 *     for val in range(0, 256):             # <<<<<<<<<<<<<<
 *         b = struct.pack("<B", val).decode("latin1")
 *         i = unpack_uint8(b, 0)  # type: ignore
 */
  for (__pyx_t_2 = 0; __pyx_t_2 < 0x100; __pyx_t_2+=1) {
    __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1480, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_val, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "serializor/unicode/des.py":1481
 * 
 *     for val in range(0, 256):
 *         b = struct.pack("<B", val).decode("latin1")             # <<<<<<<<<<<<<<
 *         i = unpack_uint8(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_pack); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1481, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    __pyx_t_6 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_6 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_5, __pyx_kp_u_B, __pyx_v_val};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_6, 2+__pyx_t_6);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1481, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_decode); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1481, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    __pyx_t_6 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_6 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_n_u_latin1};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_6, 1+__pyx_t_6);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1481, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_XDECREF_SET(__pyx_v_b, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "serializor/unicode/des.py":1482
 *     for val in range(0, 256):
 *         b = struct.pack("<B", val).decode("latin1")
 *         i = unpack_uint8(b, 0)  # type: ignore             # <<<<<<<<<<<<<<
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")
 */
    if (!(likely(PyUnicode_CheckExact(__pyx_v_b))||((__pyx_v_b) == Py_None) || __Pyx_RaiseUnexpectedTypeError("unicode", __pyx_v_b))) __PYX_ERR(1, 1482, __pyx_L1_error)
    __pyx_t_7 = __pyx_f_10serializor_7unicode_3des_unpack_uint8(((PyObject*)__pyx_v_b), 0); if (unlikely(__pyx_t_7 == ((unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 1482, __pyx_L1_error)
    __pyx_v_i = __pyx_t_7;

    /* "serializor/unicode/des.py":1483
 *         b = struct.pack("<B", val).decode("latin1")
 *         i = unpack_uint8(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"             # <<<<<<<<<<<<<<
 *         print("- pass: %s" % val, end="\r")
 *     print("Pass Unpack uint8".ljust(80))
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(__pyx_assertions_enabled())) {
      __pyx_t_1 = __Pyx_PyInt_From_unsigned_char(__pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1483, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_val, __pyx_t_1, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1483, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(1, 1483, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_8)) {
        __pyx_t_4 = PyTuple_New(4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1483, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_9 = 0;
        __pyx_t_10 = 127;
        __Pyx_INCREF(__pyx_kp_u_val);
        __pyx_t_9 += 4;
        __Pyx_GIVEREF(__pyx_kp_u_val);
        PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_kp_u_val);
        __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_v_val, __pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1483, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_10 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) > __pyx_t_10) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) : __pyx_t_10;
        __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1);
        __Pyx_GIVEREF(__pyx_t_1);
        PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
        __pyx_t_1 = 0;
        __Pyx_INCREF(__pyx_kp_u_vs_i);
        __pyx_t_9 += 6;
        __Pyx_GIVEREF(__pyx_kp_u_vs_i);
        PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_kp_u_vs_i);
        __pyx_t_1 = __Pyx_PyUnicode_From_unsigned_char(__pyx_v_i, 0, ' ', 'd'); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1483, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1);
        __Pyx_GIVEREF(__pyx_t_1);
        PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_1);
        __pyx_t_1 = 0;
        __pyx_t_1 = __Pyx_PyUnicode_Join(__pyx_t_4, 4, __pyx_t_9, __pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1483, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_Raise(__pyx_builtin_AssertionError, __pyx_t_1, 0, 0);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __PYX_ERR(1, 1483, __pyx_L1_error)
      }
    }
    #else
    if ((1)); else __PYX_ERR(1, 1483, __pyx_L1_error)
    #endif

    /* "serializor/unicode/des.py":1484
 *         i = unpack_uint8(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")             # <<<<<<<<<<<<<<
 *     print("Pass Unpack uint8".ljust(80))
 * 
 */
    __pyx_t_1 = __Pyx_PyUnicode_FormatSafe(__pyx_kp_u_pass_s, __pyx_v_val); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1484, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1484, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1)) __PYX_ERR(1, 1484, __pyx_L1_error);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1484, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_end, __pyx_kp_u__8) < 0) __PYX_ERR(1, 1484, __pyx_L1_error)
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_t_4, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1484, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }

  /* "serializor/unicode/des.py":1485
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")
 *     print("Pass Unpack uint8".ljust(80))             # <<<<<<<<<<<<<<
 * 
 *     del struct
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_Pass_Unpack_uint8, __pyx_n_s_ljust); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1485, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = NULL;
  __pyx_t_6 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
      __pyx_t_6 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_int_80};
    __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_6, 1+__pyx_t_6);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1485, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1485, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1487
 *     print("Pass Unpack uint8".ljust(80))
 * 
 *     del struct             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_DECREF(__pyx_v_struct); __pyx_v_struct = 0;

  /* "serializor/unicode/des.py":1477
 * 
 * 
 * def _test_unpack_uint8() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("serializor.unicode.des._test_unpack_uint8", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_struct);
  __Pyx_XDECREF(__pyx_v_val);
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":1490
 * 
 * 
 * def _test_unpack_int16() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_10serializor_7unicode_3des_9_test_unpack_int16(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_10serializor_7unicode_3des_9_test_unpack_int16 = {"_test_unpack_int16", (PyCFunction)__pyx_pw_10serializor_7unicode_3des_9_test_unpack_int16, METH_NOARGS, 0};
static PyObject *__pyx_pw_10serializor_7unicode_3des_9_test_unpack_int16(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_test_unpack_int16 (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10serializor_7unicode_3des_8_test_unpack_int16(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10serializor_7unicode_3des_8_test_unpack_int16(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_v_struct = NULL;
  PyObject *__pyx_v_val = NULL;
  PyObject *__pyx_v_b = NULL;
  short __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  long __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  unsigned int __pyx_t_6;
  short __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_UCS4 __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_test_unpack_int16", 1);

  /* "serializor/unicode/des.py":1491
 * 
 * def _test_unpack_int16() -> None:
 *     import struct             # <<<<<<<<<<<<<<
 * 
 *     for val in range(-32768, 32768):
 */
  __pyx_t_1 = __Pyx_ImportDottedModule(__pyx_n_s_struct, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1491, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_struct = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1493
 *     import struct
 * 
 *     for val in range(-32768, 32768):             # <<<<<<<<<<<<<<
 *         b = struct.pack("<h", val).decode("latin1")
 *         i = unpack_int16(b, 0)  # type: ignore
 */
  for (__pyx_t_2 = -32768L; __pyx_t_2 < 0x8000; __pyx_t_2+=1) {
    __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1493, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_val, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "serializor/unicode/des.py":1494
 * 
 *     for val in range(-32768, 32768):
 *         b = struct.pack("<h", val).decode("latin1")             # <<<<<<<<<<<<<<
 *         i = unpack_int16(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_pack); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1494, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    __pyx_t_6 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_6 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_5, __pyx_kp_u_h_2, __pyx_v_val};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_6, 2+__pyx_t_6);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1494, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_decode); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1494, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    __pyx_t_6 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_6 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_n_u_latin1};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_6, 1+__pyx_t_6);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1494, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_XDECREF_SET(__pyx_v_b, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "serializor/unicode/des.py":1495
 *     for val in range(-32768, 32768):
 *         b = struct.pack("<h", val).decode("latin1")
 *         i = unpack_int16(b, 0)  # type: ignore             # <<<<<<<<<<<<<<
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")
 */
    if (!(likely(PyUnicode_CheckExact(__pyx_v_b))||((__pyx_v_b) == Py_None) || __Pyx_RaiseUnexpectedTypeError("unicode", __pyx_v_b))) __PYX_ERR(1, 1495, __pyx_L1_error)
    __pyx_t_7 = __pyx_f_10serializor_7unicode_3des_unpack_int16(((PyObject*)__pyx_v_b), 0); if (unlikely(__pyx_t_7 == ((short)-1) && PyErr_Occurred())) __PYX_ERR(1, 1495, __pyx_L1_error)
    __pyx_v_i = __pyx_t_7;

    /* "serializor/unicode/des.py":1496
 *         b = struct.pack("<h", val).decode("latin1")
 *         i = unpack_int16(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"             # <<<<<<<<<<<<<<
 *         print("- pass: %s" % val, end="\r")
 *     print("Pass Unpack int16".ljust(80))
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(__pyx_assertions_enabled())) {
      __pyx_t_1 = __Pyx_PyInt_From_short(__pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1496, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_val, __pyx_t_1, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1496, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(1, 1496, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_8)) {
        __pyx_t_4 = PyTuple_New(4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1496, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_9 = 0;
        __pyx_t_10 = 127;
        __Pyx_INCREF(__pyx_kp_u_val);
        __pyx_t_9 += 4;
        __Pyx_GIVEREF(__pyx_kp_u_val);
        PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_kp_u_val);
        __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_v_val, __pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1496, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_10 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) > __pyx_t_10) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) : __pyx_t_10;
        __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1);
        __Pyx_GIVEREF(__pyx_t_1);
        PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
        __pyx_t_1 = 0;
        __Pyx_INCREF(__pyx_kp_u_vs_i);
        __pyx_t_9 += 6;
        __Pyx_GIVEREF(__pyx_kp_u_vs_i);
        PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_kp_u_vs_i);
        __pyx_t_1 = __Pyx_PyUnicode_From_short(__pyx_v_i, 0, ' ', 'd'); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1496, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1);
        __Pyx_GIVEREF(__pyx_t_1);
        PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_1);
        __pyx_t_1 = 0;
        __pyx_t_1 = __Pyx_PyUnicode_Join(__pyx_t_4, 4, __pyx_t_9, __pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1496, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_Raise(__pyx_builtin_AssertionError, __pyx_t_1, 0, 0);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __PYX_ERR(1, 1496, __pyx_L1_error)
      }
    }
    #else
    if ((1)); else __PYX_ERR(1, 1496, __pyx_L1_error)
    #endif

    /* "serializor/unicode/des.py":1497
 *         i = unpack_int16(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")             # <<<<<<<<<<<<<<
 *     print("Pass Unpack int16".ljust(80))
 * 
 */
    __pyx_t_1 = __Pyx_PyUnicode_FormatSafe(__pyx_kp_u_pass_s, __pyx_v_val); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1497, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1497, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1)) __PYX_ERR(1, 1497, __pyx_L1_error);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1497, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_end, __pyx_kp_u__8) < 0) __PYX_ERR(1, 1497, __pyx_L1_error)
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_t_4, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1497, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }

  /* "serializor/unicode/des.py":1498
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")
 *     print("Pass Unpack int16".ljust(80))             # <<<<<<<<<<<<<<
 * 
 *     del struct
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_Pass_Unpack_int16, __pyx_n_s_ljust); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1498, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = NULL;
  __pyx_t_6 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
      __pyx_t_6 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_int_80};
    __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_6, 1+__pyx_t_6);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1498, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1498, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1500
 *     print("Pass Unpack int16".ljust(80))
 * 
 *     del struct             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_DECREF(__pyx_v_struct); __pyx_v_struct = 0;

  /* "serializor/unicode/des.py":1490
 * 
 * 
 * def _test_unpack_int16() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("serializor.unicode.des._test_unpack_int16", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_struct);
  __Pyx_XDECREF(__pyx_v_val);
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":1503
 * 
 * 
 * def _test_unpack_uint16() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_10serializor_7unicode_3des_11_test_unpack_uint16(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_10serializor_7unicode_3des_11_test_unpack_uint16 = {"_test_unpack_uint16", (PyCFunction)__pyx_pw_10serializor_7unicode_3des_11_test_unpack_uint16, METH_NOARGS, 0};
static PyObject *__pyx_pw_10serializor_7unicode_3des_11_test_unpack_uint16(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_test_unpack_uint16 (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10serializor_7unicode_3des_10_test_unpack_uint16(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10serializor_7unicode_3des_10_test_unpack_uint16(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_v_struct = NULL;
  PyObject *__pyx_v_val = NULL;
  PyObject *__pyx_v_b = NULL;
  unsigned short __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  long __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  unsigned int __pyx_t_6;
  unsigned short __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_UCS4 __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_test_unpack_uint16", 1);

  /* "serializor/unicode/des.py":1504
 * 
 * def _test_unpack_uint16() -> None:
 *     import struct             # <<<<<<<<<<<<<<
 * 
 *     for val in range(0, 65536):
 */
  __pyx_t_1 = __Pyx_ImportDottedModule(__pyx_n_s_struct, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1504, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_struct = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1506
 *     import struct
 * 
 *     for val in range(0, 65536):             # <<<<<<<<<<<<<<
 *         b = struct.pack("<H", val).decode("latin1")
 *         i = unpack_uint16(b, 0)  # type: ignore
 */
  for (__pyx_t_2 = 0; __pyx_t_2 < 0x10000; __pyx_t_2+=1) {
    __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1506, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_val, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "serializor/unicode/des.py":1507
 * 
 *     for val in range(0, 65536):
 *         b = struct.pack("<H", val).decode("latin1")             # <<<<<<<<<<<<<<
 *         i = unpack_uint16(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_pack); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1507, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    __pyx_t_6 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_6 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_5, __pyx_kp_u_H, __pyx_v_val};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_6, 2+__pyx_t_6);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1507, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_decode); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1507, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    __pyx_t_6 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_6 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_n_u_latin1};
      __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_6, 1+__pyx_t_6);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1507, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_XDECREF_SET(__pyx_v_b, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "serializor/unicode/des.py":1508
 *     for val in range(0, 65536):
 *         b = struct.pack("<H", val).decode("latin1")
 *         i = unpack_uint16(b, 0)  # type: ignore             # <<<<<<<<<<<<<<
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")
 */
    if (!(likely(PyUnicode_CheckExact(__pyx_v_b))||((__pyx_v_b) == Py_None) || __Pyx_RaiseUnexpectedTypeError("unicode", __pyx_v_b))) __PYX_ERR(1, 1508, __pyx_L1_error)
    __pyx_t_7 = __pyx_f_10serializor_7unicode_3des_unpack_uint16(((PyObject*)__pyx_v_b), 0); if (unlikely(__pyx_t_7 == ((unsigned short)-1) && PyErr_Occurred())) __PYX_ERR(1, 1508, __pyx_L1_error)
    __pyx_v_i = __pyx_t_7;

    /* "serializor/unicode/des.py":1509
 *         b = struct.pack("<H", val).decode("latin1")
 *         i = unpack_uint16(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"             # <<<<<<<<<<<<<<
 *         print("- pass: %s" % val, end="\r")
 *     print("Pass Unpack uint16".ljust(80))
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(__pyx_assertions_enabled())) {
      __pyx_t_1 = __Pyx_PyInt_From_unsigned_short(__pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1509, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_val, __pyx_t_1, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1509, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(1, 1509, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_8)) {
        __pyx_t_4 = PyTuple_New(4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1509, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_9 = 0;
        __pyx_t_10 = 127;
        __Pyx_INCREF(__pyx_kp_u_val);
        __pyx_t_9 += 4;
        __Pyx_GIVEREF(__pyx_kp_u_val);
        PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_kp_u_val);
        __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_v_val, __pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1509, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_10 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) > __pyx_t_10) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) : __pyx_t_10;
        __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1);
        __Pyx_GIVEREF(__pyx_t_1);
        PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
        __pyx_t_1 = 0;
        __Pyx_INCREF(__pyx_kp_u_vs_i);
        __pyx_t_9 += 6;
        __Pyx_GIVEREF(__pyx_kp_u_vs_i);
        PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_kp_u_vs_i);
        __pyx_t_1 = __Pyx_PyUnicode_From_unsigned_short(__pyx_v_i, 0, ' ', 'd'); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1509, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1);
        __Pyx_GIVEREF(__pyx_t_1);
        PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_1);
        __pyx_t_1 = 0;
        __pyx_t_1 = __Pyx_PyUnicode_Join(__pyx_t_4, 4, __pyx_t_9, __pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1509, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_Raise(__pyx_builtin_AssertionError, __pyx_t_1, 0, 0);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __PYX_ERR(1, 1509, __pyx_L1_error)
      }
    }
    #else
    if ((1)); else __PYX_ERR(1, 1509, __pyx_L1_error)
    #endif

    /* "serializor/unicode/des.py":1510
 *         i = unpack_uint16(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")             # <<<<<<<<<<<<<<
 *     print("Pass Unpack uint16".ljust(80))
 * 
 */
    __pyx_t_1 = __Pyx_PyUnicode_FormatSafe(__pyx_kp_u_pass_s, __pyx_v_val); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1510, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1510, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1)) __PYX_ERR(1, 1510, __pyx_L1_error);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1510, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_end, __pyx_kp_u__8) < 0) __PYX_ERR(1, 1510, __pyx_L1_error)
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_t_4, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1510, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }

  /* "serializor/unicode/des.py":1511
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")
 *     print("Pass Unpack uint16".ljust(80))             # <<<<<<<<<<<<<<
 * 
 *     del struct
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_Pass_Unpack_uint16, __pyx_n_s_ljust); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1511, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = NULL;
  __pyx_t_6 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
      __pyx_t_6 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_int_80};
    __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_1, __pyx_callargs+1-__pyx_t_6, 1+__pyx_t_6);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1511, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1511, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1513
 *     print("Pass Unpack uint16".ljust(80))
 * 
 *     del struct             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_DECREF(__pyx_v_struct); __pyx_v_struct = 0;

  /* "serializor/unicode/des.py":1503
 * 
 * 
 * def _test_unpack_uint16() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("serializor.unicode.des._test_unpack_uint16", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_struct);
  __Pyx_XDECREF(__pyx_v_val);
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":1516
 * 
 * 
 * def _test_unpack_int24() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_10serializor_7unicode_3des_13_test_unpack_int24(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_10serializor_7unicode_3des_13_test_unpack_int24 = {"_test_unpack_int24", (PyCFunction)__pyx_pw_10serializor_7unicode_3des_13_test_unpack_int24, METH_NOARGS, 0};
static PyObject *__pyx_pw_10serializor_7unicode_3des_13_test_unpack_int24(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_test_unpack_int24 (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10serializor_7unicode_3des_12_test_unpack_int24(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10serializor_7unicode_3des_12_test_unpack_int24(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_v_struct = NULL;
  PyObject *__pyx_v_val = NULL;
  PyObject *__pyx_v_b = NULL;
  int __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  unsigned int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_UCS4 __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_test_unpack_int24", 1);

  /* "serializor/unicode/des.py":1517
 * 
 * def _test_unpack_int24() -> None:
 *     import struct             # <<<<<<<<<<<<<<
 * 
 *     for val in (-8388608, -8388607, 0, 8388606, 8388607):
 */
  __pyx_t_1 = __Pyx_ImportDottedModule(__pyx_n_s_struct, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1517, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_struct = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1519
 *     import struct
 * 
 *     for val in (-8388608, -8388607, 0, 8388606, 8388607):             # <<<<<<<<<<<<<<
 *         b = (struct.pack("<i", val)[:3]).decode("latin1")
 *         i = unpack_int24(b, 0)  # type: ignore
 */
  __pyx_t_1 = __pyx_tuple__9; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= 5) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely((0 < 0))) __PYX_ERR(1, 1519, __pyx_L1_error)
    #else
    __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1519, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_val, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "serializor/unicode/des.py":1520
 * 
 *     for val in (-8388608, -8388607, 0, 8388606, 8388607):
 *         b = (struct.pack("<i", val)[:3]).decode("latin1")             # <<<<<<<<<<<<<<
 *         i = unpack_int24(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_pack); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1520, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    __pyx_t_7 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_7 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_6, __pyx_kp_u_i, __pyx_v_val};
      __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_7, 2+__pyx_t_7);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1520, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __pyx_t_5 = __Pyx_PyObject_GetSlice(__pyx_t_4, 0, 3, NULL, NULL, &__pyx_slice__10, 0, 1, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1520, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_decode); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1520, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    __pyx_t_7 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_7 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_n_u_latin1};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1520, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_XDECREF_SET(__pyx_v_b, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "serializor/unicode/des.py":1521
 *     for val in (-8388608, -8388607, 0, 8388606, 8388607):
 *         b = (struct.pack("<i", val)[:3]).decode("latin1")
 *         i = unpack_int24(b, 0)  # type: ignore             # <<<<<<<<<<<<<<
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")
 */
    if (!(likely(PyUnicode_CheckExact(__pyx_v_b))||((__pyx_v_b) == Py_None) || __Pyx_RaiseUnexpectedTypeError("unicode", __pyx_v_b))) __PYX_ERR(1, 1521, __pyx_L1_error)
    __pyx_t_8 = __pyx_f_10serializor_7unicode_3des_unpack_int24(((PyObject*)__pyx_v_b), 0); if (unlikely(__pyx_t_8 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 1521, __pyx_L1_error)
    __pyx_v_i = __pyx_t_8;

    /* "serializor/unicode/des.py":1522
 *         b = (struct.pack("<i", val)[:3]).decode("latin1")
 *         i = unpack_int24(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"             # <<<<<<<<<<<<<<
 *         print("- pass: %s" % val, end="\r")
 *     print("Pass Unpack int24".ljust(80))
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(__pyx_assertions_enabled())) {
      __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_i); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1522, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_val, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1522, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(1, 1522, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_9)) {
        __pyx_t_4 = PyTuple_New(4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1522, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_10 = 0;
        __pyx_t_11 = 127;
        __Pyx_INCREF(__pyx_kp_u_val);
        __pyx_t_10 += 4;
        __Pyx_GIVEREF(__pyx_kp_u_val);
        PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_kp_u_val);
        __pyx_t_3 = __Pyx_PyObject_FormatSimple(__pyx_v_val, __pyx_empty_unicode); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1522, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) : __pyx_t_11;
        __pyx_t_10 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);
        __pyx_t_3 = 0;
        __Pyx_INCREF(__pyx_kp_u_vs_i);
        __pyx_t_10 += 6;
        __Pyx_GIVEREF(__pyx_kp_u_vs_i);
        PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_kp_u_vs_i);
        __pyx_t_3 = __Pyx_PyUnicode_From_int(__pyx_v_i, 0, ' ', 'd'); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1522, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_10 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyUnicode_Join(__pyx_t_4, 4, __pyx_t_10, __pyx_t_11); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1522, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_Raise(__pyx_builtin_AssertionError, __pyx_t_3, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __PYX_ERR(1, 1522, __pyx_L1_error)
      }
    }
    #else
    if ((1)); else __PYX_ERR(1, 1522, __pyx_L1_error)
    #endif

    /* "serializor/unicode/des.py":1523
 *         i = unpack_int24(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")             # <<<<<<<<<<<<<<
 *     print("Pass Unpack int24".ljust(80))
 * 
 */
    __pyx_t_3 = __Pyx_PyUnicode_FormatSafe(__pyx_kp_u_pass_s, __pyx_v_val); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1523, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1523, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3)) __PYX_ERR(1, 1523, __pyx_L1_error);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1523, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_end, __pyx_kp_u__8) < 0) __PYX_ERR(1, 1523, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1523, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "serializor/unicode/des.py":1519
 *     import struct
 * 
 *     for val in (-8388608, -8388607, 0, 8388606, 8388607):             # <<<<<<<<<<<<<<
 *         b = (struct.pack("<i", val)[:3]).decode("latin1")
 *         i = unpack_int24(b, 0)  # type: ignore
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1524
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")
 *     print("Pass Unpack int24".ljust(80))             # <<<<<<<<<<<<<<
 * 
 *     del struct
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_Pass_Unpack_int24, __pyx_n_s_ljust); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1524, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = NULL;
  __pyx_t_7 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
      __pyx_t_7 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_int_80};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1524, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1524, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "serializor/unicode/des.py":1526
 *     print("Pass Unpack int24".ljust(80))
 * 
 *     del struct             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_DECREF(__pyx_v_struct); __pyx_v_struct = 0;

  /* "serializor/unicode/des.py":1516
 * 
 * 
 * def _test_unpack_int24() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("serializor.unicode.des._test_unpack_int24", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_struct);
  __Pyx_XDECREF(__pyx_v_val);
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":1529
 * 
 * 
 * def _test_unpack_uint24() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_10serializor_7unicode_3des_15_test_unpack_uint24(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_10serializor_7unicode_3des_15_test_unpack_uint24 = {"_test_unpack_uint24", (PyCFunction)__pyx_pw_10serializor_7unicode_3des_15_test_unpack_uint24, METH_NOARGS, 0};
static PyObject *__pyx_pw_10serializor_7unicode_3des_15_test_unpack_uint24(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_test_unpack_uint24 (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10serializor_7unicode_3des_14_test_unpack_uint24(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10serializor_7unicode_3des_14_test_unpack_uint24(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_v_struct = NULL;
  PyObject *__pyx_v_val = NULL;
  PyObject *__pyx_v_b = NULL;
  unsigned int __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  unsigned int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_UCS4 __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_test_unpack_uint24", 1);

  /* "serializor/unicode/des.py":1530
 * 
 * def _test_unpack_uint24() -> None:
 *     import struct             # <<<<<<<<<<<<<<
 * 
 *     for val in (0, 1, 16777213, 16777214, 16777215):
 */
  __pyx_t_1 = __Pyx_ImportDottedModule(__pyx_n_s_struct, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1530, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_struct = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1532
 *     import struct
 * 
 *     for val in (0, 1, 16777213, 16777214, 16777215):             # <<<<<<<<<<<<<<
 *         b = (struct.pack("<I", val)[:3]).decode("latin1")
 *         i = unpack_uint24(b, 0)  # type: ignore
 */
  __pyx_t_1 = __pyx_tuple__11; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= 5) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely((0 < 0))) __PYX_ERR(1, 1532, __pyx_L1_error)
    #else
    __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1532, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_val, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "serializor/unicode/des.py":1533
 * 
 *     for val in (0, 1, 16777213, 16777214, 16777215):
 *         b = (struct.pack("<I", val)[:3]).decode("latin1")             # <<<<<<<<<<<<<<
 *         i = unpack_uint24(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_pack); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1533, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    __pyx_t_7 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_7 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_6, __pyx_kp_u_I, __pyx_v_val};
      __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_7, 2+__pyx_t_7);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1533, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __pyx_t_5 = __Pyx_PyObject_GetSlice(__pyx_t_4, 0, 3, NULL, NULL, &__pyx_slice__10, 0, 1, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1533, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_decode); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1533, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    __pyx_t_7 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_7 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_n_u_latin1};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1533, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_XDECREF_SET(__pyx_v_b, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "serializor/unicode/des.py":1534
 *     for val in (0, 1, 16777213, 16777214, 16777215):
 *         b = (struct.pack("<I", val)[:3]).decode("latin1")
 *         i = unpack_uint24(b, 0)  # type: ignore             # <<<<<<<<<<<<<<
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")
 */
    if (!(likely(PyUnicode_CheckExact(__pyx_v_b))||((__pyx_v_b) == Py_None) || __Pyx_RaiseUnexpectedTypeError("unicode", __pyx_v_b))) __PYX_ERR(1, 1534, __pyx_L1_error)
    __pyx_t_7 = __pyx_f_10serializor_7unicode_3des_unpack_uint24(((PyObject*)__pyx_v_b), 0); if (unlikely(__pyx_t_7 == ((unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(1, 1534, __pyx_L1_error)
    __pyx_v_i = __pyx_t_7;

    /* "serializor/unicode/des.py":1535
 *         b = (struct.pack("<I", val)[:3]).decode("latin1")
 *         i = unpack_uint24(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"             # <<<<<<<<<<<<<<
 *         print("- pass: %s" % val, end="\r")
 *     print("Pass Unpack uint24".ljust(80))
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(__pyx_assertions_enabled())) {
      __pyx_t_3 = __Pyx_PyInt_From_unsigned_int(__pyx_v_i); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1535, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_val, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1535, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(1, 1535, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_8)) {
        __pyx_t_4 = PyTuple_New(4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1535, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_9 = 0;
        __pyx_t_10 = 127;
        __Pyx_INCREF(__pyx_kp_u_val);
        __pyx_t_9 += 4;
        __Pyx_GIVEREF(__pyx_kp_u_val);
        PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_kp_u_val);
        __pyx_t_3 = __Pyx_PyObject_FormatSimple(__pyx_v_val, __pyx_empty_unicode); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1535, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_10 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) > __pyx_t_10) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) : __pyx_t_10;
        __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);
        __pyx_t_3 = 0;
        __Pyx_INCREF(__pyx_kp_u_vs_i);
        __pyx_t_9 += 6;
        __Pyx_GIVEREF(__pyx_kp_u_vs_i);
        PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_kp_u_vs_i);
        __pyx_t_3 = __Pyx_PyUnicode_From_unsigned_int(__pyx_v_i, 0, ' ', 'd'); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1535, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyUnicode_Join(__pyx_t_4, 4, __pyx_t_9, __pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1535, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_Raise(__pyx_builtin_AssertionError, __pyx_t_3, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __PYX_ERR(1, 1535, __pyx_L1_error)
      }
    }
    #else
    if ((1)); else __PYX_ERR(1, 1535, __pyx_L1_error)
    #endif

    /* "serializor/unicode/des.py":1536
 *         i = unpack_uint24(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")             # <<<<<<<<<<<<<<
 *     print("Pass Unpack uint24".ljust(80))
 * 
 */
    __pyx_t_3 = __Pyx_PyUnicode_FormatSafe(__pyx_kp_u_pass_s, __pyx_v_val); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1536, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1536, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3)) __PYX_ERR(1, 1536, __pyx_L1_error);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1536, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_end, __pyx_kp_u__8) < 0) __PYX_ERR(1, 1536, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1536, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "serializor/unicode/des.py":1532
 *     import struct
 * 
 *     for val in (0, 1, 16777213, 16777214, 16777215):             # <<<<<<<<<<<<<<
 *         b = (struct.pack("<I", val)[:3]).decode("latin1")
 *         i = unpack_uint24(b, 0)  # type: ignore
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1537
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")
 *     print("Pass Unpack uint24".ljust(80))             # <<<<<<<<<<<<<<
 * 
 *     del struct
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_Pass_Unpack_uint24, __pyx_n_s_ljust); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1537, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = NULL;
  __pyx_t_7 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
      __pyx_t_7 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_int_80};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1537, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1537, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "serializor/unicode/des.py":1539
 *     print("Pass Unpack uint24".ljust(80))
 * 
 *     del struct             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_DECREF(__pyx_v_struct); __pyx_v_struct = 0;

  /* "serializor/unicode/des.py":1529
 * 
 * 
 * def _test_unpack_uint24() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("serializor.unicode.des._test_unpack_uint24", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_struct);
  __Pyx_XDECREF(__pyx_v_val);
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":1542
 * 
 * 
 * def _test_unpack_int32() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_10serializor_7unicode_3des_17_test_unpack_int32(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_10serializor_7unicode_3des_17_test_unpack_int32 = {"_test_unpack_int32", (PyCFunction)__pyx_pw_10serializor_7unicode_3des_17_test_unpack_int32, METH_NOARGS, 0};
static PyObject *__pyx_pw_10serializor_7unicode_3des_17_test_unpack_int32(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_test_unpack_int32 (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10serializor_7unicode_3des_16_test_unpack_int32(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10serializor_7unicode_3des_16_test_unpack_int32(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_v_struct = NULL;
  PyObject *__pyx_v_val = NULL;
  PyObject *__pyx_v_b = NULL;
  int __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  unsigned int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_UCS4 __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_test_unpack_int32", 1);

  /* "serializor/unicode/des.py":1543
 * 
 * def _test_unpack_int32() -> None:
 *     import struct             # <<<<<<<<<<<<<<
 * 
 *     for val in (-2147483648, -2147483647, 0, 2147483646, 2147483647):
 */
  __pyx_t_1 = __Pyx_ImportDottedModule(__pyx_n_s_struct, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1543, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_struct = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1545
 *     import struct
 * 
 *     for val in (-2147483648, -2147483647, 0, 2147483646, 2147483647):             # <<<<<<<<<<<<<<
 *         b = struct.pack("<i", val).decode("latin1")
 *         i = unpack_int32(b, 0)  # type: ignore
 */
  __pyx_t_1 = __pyx_tuple__12; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= 5) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely((0 < 0))) __PYX_ERR(1, 1545, __pyx_L1_error)
    #else
    __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1545, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_val, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "serializor/unicode/des.py":1546
 * 
 *     for val in (-2147483648, -2147483647, 0, 2147483646, 2147483647):
 *         b = struct.pack("<i", val).decode("latin1")             # <<<<<<<<<<<<<<
 *         i = unpack_int32(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_pack); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1546, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    __pyx_t_7 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_7 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_6, __pyx_kp_u_i, __pyx_v_val};
      __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_7, 2+__pyx_t_7);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1546, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_decode); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1546, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    __pyx_t_7 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_7 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_n_u_latin1};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1546, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_XDECREF_SET(__pyx_v_b, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "serializor/unicode/des.py":1547
 *     for val in (-2147483648, -2147483647, 0, 2147483646, 2147483647):
 *         b = struct.pack("<i", val).decode("latin1")
 *         i = unpack_int32(b, 0)  # type: ignore             # <<<<<<<<<<<<<<
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")
 */
    if (!(likely(PyUnicode_CheckExact(__pyx_v_b))||((__pyx_v_b) == Py_None) || __Pyx_RaiseUnexpectedTypeError("unicode", __pyx_v_b))) __PYX_ERR(1, 1547, __pyx_L1_error)
    __pyx_t_8 = __pyx_f_10serializor_7unicode_3des_unpack_int32(((PyObject*)__pyx_v_b), 0); if (unlikely(__pyx_t_8 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(1, 1547, __pyx_L1_error)
    __pyx_v_i = __pyx_t_8;

    /* "serializor/unicode/des.py":1548
 *         b = struct.pack("<i", val).decode("latin1")
 *         i = unpack_int32(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"             # <<<<<<<<<<<<<<
 *         print("- pass: %s" % val, end="\r")
 *     print("Pass Unpack int32".ljust(80))
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(__pyx_assertions_enabled())) {
      __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_i); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1548, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = PyObject_RichCompare(__pyx_v_val, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1548, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(1, 1548, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_9)) {
        __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1548, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_10 = 0;
        __pyx_t_11 = 127;
        __Pyx_INCREF(__pyx_kp_u_val);
        __pyx_t_10 += 4;
        __Pyx_GIVEREF(__pyx_kp_u_val);
        PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_kp_u_val);
        __pyx_t_3 = __Pyx_PyObject_FormatSimple(__pyx_v_val, __pyx_empty_unicode); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1548, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) : __pyx_t_11;
        __pyx_t_10 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_3);
        __pyx_t_3 = 0;
        __Pyx_INCREF(__pyx_kp_u_vs_i);
        __pyx_t_10 += 6;
        __Pyx_GIVEREF(__pyx_kp_u_vs_i);
        PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_kp_u_vs_i);
        __pyx_t_3 = __Pyx_PyUnicode_From_int(__pyx_v_i, 0, ' ', 'd'); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1548, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_10 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyUnicode_Join(__pyx_t_5, 4, __pyx_t_10, __pyx_t_11); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1548, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_Raise(__pyx_builtin_AssertionError, __pyx_t_3, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __PYX_ERR(1, 1548, __pyx_L1_error)
      }
    }
    #else
    if ((1)); else __PYX_ERR(1, 1548, __pyx_L1_error)
    #endif

    /* "serializor/unicode/des.py":1549
 *         i = unpack_int32(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")             # <<<<<<<<<<<<<<
 *     print("Pass Unpack int32".ljust(80))
 * 
 */
    __pyx_t_3 = __Pyx_PyUnicode_FormatSafe(__pyx_kp_u_pass_s, __pyx_v_val); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1549, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1549, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3)) __PYX_ERR(1, 1549, __pyx_L1_error);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1549, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_end, __pyx_kp_u__8) < 0) __PYX_ERR(1, 1549, __pyx_L1_error)
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_t_5, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1549, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "serializor/unicode/des.py":1545
 *     import struct
 * 
 *     for val in (-2147483648, -2147483647, 0, 2147483646, 2147483647):             # <<<<<<<<<<<<<<
 *         b = struct.pack("<i", val).decode("latin1")
 *         i = unpack_int32(b, 0)  # type: ignore
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1550
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")
 *     print("Pass Unpack int32".ljust(80))             # <<<<<<<<<<<<<<
 * 
 *     del struct
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_Pass_Unpack_int32, __pyx_n_s_ljust); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1550, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = NULL;
  __pyx_t_7 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
      __pyx_t_7 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_int_80};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1550, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1550, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "serializor/unicode/des.py":1552
 *     print("Pass Unpack int32".ljust(80))
 * 
 *     del struct             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_DECREF(__pyx_v_struct); __pyx_v_struct = 0;

  /* "serializor/unicode/des.py":1542
 * 
 * 
 * def _test_unpack_int32() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("serializor.unicode.des._test_unpack_int32", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_struct);
  __Pyx_XDECREF(__pyx_v_val);
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":1555
 * 
 * 
 * def _test_unpack_uint32() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_10serializor_7unicode_3des_19_test_unpack_uint32(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_10serializor_7unicode_3des_19_test_unpack_uint32 = {"_test_unpack_uint32", (PyCFunction)__pyx_pw_10serializor_7unicode_3des_19_test_unpack_uint32, METH_NOARGS, 0};
static PyObject *__pyx_pw_10serializor_7unicode_3des_19_test_unpack_uint32(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_test_unpack_uint32 (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10serializor_7unicode_3des_18_test_unpack_uint32(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10serializor_7unicode_3des_18_test_unpack_uint32(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_v_struct = NULL;
  PyObject *__pyx_v_val = NULL;
  PyObject *__pyx_v_b = NULL;
  unsigned int __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  unsigned int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_UCS4 __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_test_unpack_uint32", 1);

  /* "serializor/unicode/des.py":1556
 * 
 * def _test_unpack_uint32() -> None:
 *     import struct             # <<<<<<<<<<<<<<
 * 
 *     for val in (0, 1, 4294967293, 4294967294, 4294967295):
 */
  __pyx_t_1 = __Pyx_ImportDottedModule(__pyx_n_s_struct, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1556, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_struct = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1558
 *     import struct
 * 
 *     for val in (0, 1, 4294967293, 4294967294, 4294967295):             # <<<<<<<<<<<<<<
 *         b = struct.pack("<I", val).decode("latin1")
 *         i = unpack_uint32(b, 0)  # type: ignore
 */
  __pyx_t_1 = __pyx_tuple__13; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= 5) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely((0 < 0))) __PYX_ERR(1, 1558, __pyx_L1_error)
    #else
    __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1558, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_val, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "serializor/unicode/des.py":1559
 * 
 *     for val in (0, 1, 4294967293, 4294967294, 4294967295):
 *         b = struct.pack("<I", val).decode("latin1")             # <<<<<<<<<<<<<<
 *         i = unpack_uint32(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_pack); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1559, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    __pyx_t_7 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_7 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_6, __pyx_kp_u_I, __pyx_v_val};
      __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_7, 2+__pyx_t_7);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1559, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_decode); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1559, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    __pyx_t_7 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_7 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_n_u_latin1};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1559, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_XDECREF_SET(__pyx_v_b, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "serializor/unicode/des.py":1560
 *     for val in (0, 1, 4294967293, 4294967294, 4294967295):
 *         b = struct.pack("<I", val).decode("latin1")
 *         i = unpack_uint32(b, 0)  # type: ignore             # <<<<<<<<<<<<<<
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")
 */
    if (!(likely(PyUnicode_CheckExact(__pyx_v_b))||((__pyx_v_b) == Py_None) || __Pyx_RaiseUnexpectedTypeError("unicode", __pyx_v_b))) __PYX_ERR(1, 1560, __pyx_L1_error)
    __pyx_t_7 = __pyx_f_10serializor_7unicode_3des_unpack_uint32(((PyObject*)__pyx_v_b), 0); if (unlikely(__pyx_t_7 == ((unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(1, 1560, __pyx_L1_error)
    __pyx_v_i = __pyx_t_7;

    /* "serializor/unicode/des.py":1561
 *         b = struct.pack("<I", val).decode("latin1")
 *         i = unpack_uint32(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"             # <<<<<<<<<<<<<<
 *         print("- pass: %s" % val, end="\r")
 *     print("Pass Unpack uint32".ljust(80))
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(__pyx_assertions_enabled())) {
      __pyx_t_3 = __Pyx_PyInt_From_unsigned_int(__pyx_v_i); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1561, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = PyObject_RichCompare(__pyx_v_val, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1561, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(1, 1561, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_8)) {
        __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1561, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_9 = 0;
        __pyx_t_10 = 127;
        __Pyx_INCREF(__pyx_kp_u_val);
        __pyx_t_9 += 4;
        __Pyx_GIVEREF(__pyx_kp_u_val);
        PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_kp_u_val);
        __pyx_t_3 = __Pyx_PyObject_FormatSimple(__pyx_v_val, __pyx_empty_unicode); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1561, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_10 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) > __pyx_t_10) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) : __pyx_t_10;
        __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_3);
        __pyx_t_3 = 0;
        __Pyx_INCREF(__pyx_kp_u_vs_i);
        __pyx_t_9 += 6;
        __Pyx_GIVEREF(__pyx_kp_u_vs_i);
        PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_kp_u_vs_i);
        __pyx_t_3 = __Pyx_PyUnicode_From_unsigned_int(__pyx_v_i, 0, ' ', 'd'); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1561, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyUnicode_Join(__pyx_t_5, 4, __pyx_t_9, __pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1561, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_Raise(__pyx_builtin_AssertionError, __pyx_t_3, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __PYX_ERR(1, 1561, __pyx_L1_error)
      }
    }
    #else
    if ((1)); else __PYX_ERR(1, 1561, __pyx_L1_error)
    #endif

    /* "serializor/unicode/des.py":1562
 *         i = unpack_uint32(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")             # <<<<<<<<<<<<<<
 *     print("Pass Unpack uint32".ljust(80))
 * 
 */
    __pyx_t_3 = __Pyx_PyUnicode_FormatSafe(__pyx_kp_u_pass_s, __pyx_v_val); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1562, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1562, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3)) __PYX_ERR(1, 1562, __pyx_L1_error);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1562, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_end, __pyx_kp_u__8) < 0) __PYX_ERR(1, 1562, __pyx_L1_error)
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_t_5, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1562, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "serializor/unicode/des.py":1558
 *     import struct
 * 
 *     for val in (0, 1, 4294967293, 4294967294, 4294967295):             # <<<<<<<<<<<<<<
 *         b = struct.pack("<I", val).decode("latin1")
 *         i = unpack_uint32(b, 0)  # type: ignore
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1563
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")
 *     print("Pass Unpack uint32".ljust(80))             # <<<<<<<<<<<<<<
 * 
 *     del struct
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_Pass_Unpack_uint32, __pyx_n_s_ljust); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1563, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = NULL;
  __pyx_t_7 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
      __pyx_t_7 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_int_80};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1563, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1563, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "serializor/unicode/des.py":1565
 *     print("Pass Unpack uint32".ljust(80))
 * 
 *     del struct             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_DECREF(__pyx_v_struct); __pyx_v_struct = 0;

  /* "serializor/unicode/des.py":1555
 * 
 * 
 * def _test_unpack_uint32() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("serializor.unicode.des._test_unpack_uint32", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_struct);
  __Pyx_XDECREF(__pyx_v_val);
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":1568
 * 
 * 
 * def _test_unpack_int64() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_10serializor_7unicode_3des_21_test_unpack_int64(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_10serializor_7unicode_3des_21_test_unpack_int64 = {"_test_unpack_int64", (PyCFunction)__pyx_pw_10serializor_7unicode_3des_21_test_unpack_int64, METH_NOARGS, 0};
static PyObject *__pyx_pw_10serializor_7unicode_3des_21_test_unpack_int64(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_test_unpack_int64 (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10serializor_7unicode_3des_20_test_unpack_int64(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10serializor_7unicode_3des_20_test_unpack_int64(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_v_struct = NULL;
  PyObject *__pyx_v_val = NULL;
  PyObject *__pyx_v_b = NULL;
  PY_LONG_LONG __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  unsigned int __pyx_t_7;
  PY_LONG_LONG __pyx_t_8;
  int __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_UCS4 __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_test_unpack_int64", 1);

  /* "serializor/unicode/des.py":1569
 * 
 * def _test_unpack_int64() -> None:
 *     import struct             # <<<<<<<<<<<<<<
 * 
 *     for val in (
 */
  __pyx_t_1 = __Pyx_ImportDottedModule(__pyx_n_s_struct, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1569, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_struct = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1571
 *     import struct
 * 
 *     for val in (             # <<<<<<<<<<<<<<
 *         -9223372036854775808,
 *         -9223372036854775807,
 */
  __pyx_t_1 = __pyx_tuple__14; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= 5) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely((0 < 0))) __PYX_ERR(1, 1571, __pyx_L1_error)
    #else
    __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1571, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_val, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "serializor/unicode/des.py":1578
 *         9223372036854775807,
 *     ):
 *         b = struct.pack("<q", val).decode("latin1")             # <<<<<<<<<<<<<<
 *         i = unpack_int64(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_pack); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1578, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    __pyx_t_7 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_7 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_6, __pyx_kp_u_q, __pyx_v_val};
      __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_7, 2+__pyx_t_7);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1578, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_decode); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1578, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    __pyx_t_7 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_7 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_n_u_latin1};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1578, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_XDECREF_SET(__pyx_v_b, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "serializor/unicode/des.py":1579
 *     ):
 *         b = struct.pack("<q", val).decode("latin1")
 *         i = unpack_int64(b, 0)  # type: ignore             # <<<<<<<<<<<<<<
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")
 */
    if (!(likely(PyUnicode_CheckExact(__pyx_v_b))||((__pyx_v_b) == Py_None) || __Pyx_RaiseUnexpectedTypeError("unicode", __pyx_v_b))) __PYX_ERR(1, 1579, __pyx_L1_error)
    __pyx_t_8 = __pyx_f_10serializor_7unicode_3des_unpack_int64(((PyObject*)__pyx_v_b), 0); if (unlikely(__pyx_t_8 == ((PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 1579, __pyx_L1_error)
    __pyx_v_i = __pyx_t_8;

    /* "serializor/unicode/des.py":1580
 *         b = struct.pack("<q", val).decode("latin1")
 *         i = unpack_int64(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"             # <<<<<<<<<<<<<<
 *         print("- pass: %s" % val, end="\r")
 *     print("Pass Unpack int64".ljust(80))
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(__pyx_assertions_enabled())) {
      __pyx_t_3 = __Pyx_PyInt_From_PY_LONG_LONG(__pyx_v_i); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1580, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = PyObject_RichCompare(__pyx_v_val, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1580, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(1, 1580, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_9)) {
        __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1580, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_10 = 0;
        __pyx_t_11 = 127;
        __Pyx_INCREF(__pyx_kp_u_val);
        __pyx_t_10 += 4;
        __Pyx_GIVEREF(__pyx_kp_u_val);
        PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_kp_u_val);
        __pyx_t_3 = __Pyx_PyObject_FormatSimple(__pyx_v_val, __pyx_empty_unicode); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1580, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) : __pyx_t_11;
        __pyx_t_10 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_3);
        __pyx_t_3 = 0;
        __Pyx_INCREF(__pyx_kp_u_vs_i);
        __pyx_t_10 += 6;
        __Pyx_GIVEREF(__pyx_kp_u_vs_i);
        PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_kp_u_vs_i);
        __pyx_t_3 = __Pyx_PyUnicode_From_PY_LONG_LONG(__pyx_v_i, 0, ' ', 'd'); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1580, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_10 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyUnicode_Join(__pyx_t_5, 4, __pyx_t_10, __pyx_t_11); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1580, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_Raise(__pyx_builtin_AssertionError, __pyx_t_3, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __PYX_ERR(1, 1580, __pyx_L1_error)
      }
    }
    #else
    if ((1)); else __PYX_ERR(1, 1580, __pyx_L1_error)
    #endif

    /* "serializor/unicode/des.py":1581
 *         i = unpack_int64(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")             # <<<<<<<<<<<<<<
 *     print("Pass Unpack int64".ljust(80))
 * 
 */
    __pyx_t_3 = __Pyx_PyUnicode_FormatSafe(__pyx_kp_u_pass_s, __pyx_v_val); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1581, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1581, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3)) __PYX_ERR(1, 1581, __pyx_L1_error);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1581, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_end, __pyx_kp_u__8) < 0) __PYX_ERR(1, 1581, __pyx_L1_error)
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_t_5, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1581, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "serializor/unicode/des.py":1571
 *     import struct
 * 
 *     for val in (             # <<<<<<<<<<<<<<
 *         -9223372036854775808,
 *         -9223372036854775807,
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1582
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")
 *     print("Pass Unpack int64".ljust(80))             # <<<<<<<<<<<<<<
 * 
 *     del struct
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_Pass_Unpack_int64, __pyx_n_s_ljust); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1582, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = NULL;
  __pyx_t_7 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
      __pyx_t_7 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_int_80};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1582, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1582, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "serializor/unicode/des.py":1584
 *     print("Pass Unpack int64".ljust(80))
 * 
 *     del struct             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_DECREF(__pyx_v_struct); __pyx_v_struct = 0;

  /* "serializor/unicode/des.py":1568
 * 
 * 
 * def _test_unpack_int64() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("serializor.unicode.des._test_unpack_int64", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_struct);
  __Pyx_XDECREF(__pyx_v_val);
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":1587
 * 
 * 
 * def _test_unpack_uint64() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_10serializor_7unicode_3des_23_test_unpack_uint64(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_10serializor_7unicode_3des_23_test_unpack_uint64 = {"_test_unpack_uint64", (PyCFunction)__pyx_pw_10serializor_7unicode_3des_23_test_unpack_uint64, METH_NOARGS, 0};
static PyObject *__pyx_pw_10serializor_7unicode_3des_23_test_unpack_uint64(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_test_unpack_uint64 (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10serializor_7unicode_3des_22_test_unpack_uint64(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10serializor_7unicode_3des_22_test_unpack_uint64(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_v_struct = NULL;
  PyObject *__pyx_v_val = NULL;
  PyObject *__pyx_v_b = NULL;
  unsigned PY_LONG_LONG __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  unsigned int __pyx_t_7;
  unsigned PY_LONG_LONG __pyx_t_8;
  int __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_UCS4 __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_test_unpack_uint64", 1);

  /* "serializor/unicode/des.py":1588
 * 
 * def _test_unpack_uint64() -> None:
 *     import struct             # <<<<<<<<<<<<<<
 * 
 *     for val in (
 */
  __pyx_t_1 = __Pyx_ImportDottedModule(__pyx_n_s_struct, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1588, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_struct = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1590
 *     import struct
 * 
 *     for val in (             # <<<<<<<<<<<<<<
 *         0,
 *         1,
 */
  __pyx_t_1 = __pyx_tuple__15; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= 5) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely((0 < 0))) __PYX_ERR(1, 1590, __pyx_L1_error)
    #else
    __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1590, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_val, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "serializor/unicode/des.py":1597
 *         18446744073709551615,
 *     ):
 *         b = struct.pack("<Q", val).decode("latin1")             # <<<<<<<<<<<<<<
 *         i = unpack_uint64(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_pack); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1597, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    __pyx_t_7 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_7 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_6, __pyx_kp_u_Q, __pyx_v_val};
      __pyx_t_4 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_7, 2+__pyx_t_7);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1597, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_decode); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1597, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    __pyx_t_7 = 0;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_7 = 1;
      }
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_n_u_latin1};
      __pyx_t_3 = __Pyx_PyObject_FastCall(__pyx_t_5, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1597, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_XDECREF_SET(__pyx_v_b, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "serializor/unicode/des.py":1598
 *     ):
 *         b = struct.pack("<Q", val).decode("latin1")
 *         i = unpack_uint64(b, 0)  # type: ignore             # <<<<<<<<<<<<<<
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")
 */
    if (!(likely(PyUnicode_CheckExact(__pyx_v_b))||((__pyx_v_b) == Py_None) || __Pyx_RaiseUnexpectedTypeError("unicode", __pyx_v_b))) __PYX_ERR(1, 1598, __pyx_L1_error)
    __pyx_t_8 = __pyx_f_10serializor_7unicode_3des_unpack_uint64(((PyObject*)__pyx_v_b), 0); if (unlikely(__pyx_t_8 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 1598, __pyx_L1_error)
    __pyx_v_i = __pyx_t_8;

    /* "serializor/unicode/des.py":1599
 *         b = struct.pack("<Q", val).decode("latin1")
 *         i = unpack_uint64(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"             # <<<<<<<<<<<<<<
 *         print("- pass: %s" % val, end="\r")
 *     print("Pass Unpack uint64".ljust(80))
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(__pyx_assertions_enabled())) {
      __pyx_t_3 = __Pyx_PyInt_From_unsigned_PY_LONG_LONG(__pyx_v_i); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1599, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = PyObject_RichCompare(__pyx_v_val, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1599, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(1, 1599, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_9)) {
        __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1599, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_10 = 0;
        __pyx_t_11 = 127;
        __Pyx_INCREF(__pyx_kp_u_val);
        __pyx_t_10 += 4;
        __Pyx_GIVEREF(__pyx_kp_u_val);
        PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_kp_u_val);
        __pyx_t_3 = __Pyx_PyObject_FormatSimple(__pyx_v_val, __pyx_empty_unicode); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1599, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) : __pyx_t_11;
        __pyx_t_10 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_3);
        __pyx_t_3 = 0;
        __Pyx_INCREF(__pyx_kp_u_vs_i);
        __pyx_t_10 += 6;
        __Pyx_GIVEREF(__pyx_kp_u_vs_i);
        PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_kp_u_vs_i);
        __pyx_t_3 = __Pyx_PyUnicode_From_unsigned_PY_LONG_LONG(__pyx_v_i, 0, ' ', 'd'); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1599, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_10 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyUnicode_Join(__pyx_t_5, 4, __pyx_t_10, __pyx_t_11); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1599, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_Raise(__pyx_builtin_AssertionError, __pyx_t_3, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __PYX_ERR(1, 1599, __pyx_L1_error)
      }
    }
    #else
    if ((1)); else __PYX_ERR(1, 1599, __pyx_L1_error)
    #endif

    /* "serializor/unicode/des.py":1600
 *         i = unpack_uint64(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")             # <<<<<<<<<<<<<<
 *     print("Pass Unpack uint64".ljust(80))
 * 
 */
    __pyx_t_3 = __Pyx_PyUnicode_FormatSafe(__pyx_kp_u_pass_s, __pyx_v_val); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1600, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1600, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3)) __PYX_ERR(1, 1600, __pyx_L1_error);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1600, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_end, __pyx_kp_u__8) < 0) __PYX_ERR(1, 1600, __pyx_L1_error)
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_t_5, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1600, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "serializor/unicode/des.py":1590
 *     import struct
 * 
 *     for val in (             # <<<<<<<<<<<<<<
 *         0,
 *         1,
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1601
 *         assert val == i, f"val {val} vs i {i}"
 *         print("- pass: %s" % val, end="\r")
 *     print("Pass Unpack uint64".ljust(80))             # <<<<<<<<<<<<<<
 * 
 *     del struct
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_Pass_Unpack_uint64, __pyx_n_s_ljust); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1601, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = NULL;
  __pyx_t_7 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
      __pyx_t_7 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_int_80};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_4, __pyx_callargs+1-__pyx_t_7, 1+__pyx_t_7);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1601, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1601, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "serializor/unicode/des.py":1603
 *     print("Pass Unpack uint64".ljust(80))
 * 
 *     del struct             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_DECREF(__pyx_v_struct); __pyx_v_struct = 0;

  /* "serializor/unicode/des.py":1587
 * 
 * 
 * def _test_unpack_uint64() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("serializor.unicode.des._test_unpack_uint64", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_struct);
  __Pyx_XDECREF(__pyx_v_val);
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "serializor/unicode/des.py":1606
 * 
 * 
 * def _test_dec_encoded_int() -> None:             # <<<<<<<<<<<<<<
 *     for enc, num in [
 *         ("", 251),
 */

/* Python wrapper */
static PyObject *__pyx_pw_10serializor_7unicode_3des_25_test_dec_encoded_int(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_10serializor_7unicode_3des_25_test_dec_encoded_int = {"_test_dec_encoded_int", (PyCFunction)__pyx_pw_10serializor_7unicode_3des_25_test_dec_encoded_int, METH_NOARGS, 0};
static PyObject *__pyx_pw_10serializor_7unicode_3des_25_test_dec_encoded_int(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_test_dec_encoded_int (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_10serializor_7unicode_3des_24_test_dec_encoded_int(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_10serializor_7unicode_3des_24_test_dec_encoded_int(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_v_enc = NULL;
  PyObject *__pyx_v_num = NULL;
  unsigned PY_LONG_LONG __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6[1];
  unsigned PY_LONG_LONG __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_UCS4 __pyx_t_10;
  unsigned int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_test_dec_encoded_int", 1);

  /* "serializor/unicode/des.py":1607
 * 
 * def _test_dec_encoded_int() -> None:
 *     for enc, num in [             # <<<<<<<<<<<<<<
 *         ("", 251),
 *         ("\x00", 252),
 */
  __pyx_t_1 = __pyx_tuple__30; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= 7) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely((0 < 0))) __PYX_ERR(1, 1607, __pyx_L1_error)
    #else
    __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1607, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    if (likely(__pyx_t_3 != Py_None)) {
      PyObject* sequence = __pyx_t_3;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(1, 1607, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_5 = PyTuple_GET_ITEM(sequence, 1); 
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      #else
      __pyx_t_4 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1607, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1607, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      #endif
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else {
      __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(1, 1607, __pyx_L1_error)
    }
    __Pyx_XDECREF_SET(__pyx_v_enc, __pyx_t_4);
    __pyx_t_4 = 0;
    __Pyx_XDECREF_SET(__pyx_v_num, __pyx_t_5);
    __pyx_t_5 = 0;

    /* "serializor/unicode/des.py":1616
 *         ("\x00\x00\x00\x00\x02\x00\x00\x00", 8589934592),
 *     ]:
 *         i = dec_encoded_int(enc, [0])  # type: ignore             # <<<<<<<<<<<<<<
 *         assert num == i, f"num {num} vs i {i} - encode: {enc}"
 *     print("Pass Dec Encoded Integer".ljust(80))
 */
    if (!(likely(PyUnicode_CheckExact(__pyx_v_enc))||((__pyx_v_enc) == Py_None) || __Pyx_RaiseUnexpectedTypeError("unicode", __pyx_v_enc))) __PYX_ERR(1, 1616, __pyx_L1_error)
    __pyx_t_6[0] = 0;
    __pyx_t_7 = __pyx_f_10serializor_7unicode_3des_dec_encoded_int(((PyObject*)__pyx_v_enc), __pyx_t_6); if (unlikely(__pyx_t_7 == ((unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(1, 1616, __pyx_L1_error)
    __pyx_v_i = __pyx_t_7;

    /* "serializor/unicode/des.py":1617
 *     ]:
 *         i = dec_encoded_int(enc, [0])  # type: ignore
 *         assert num == i, f"num {num} vs i {i} - encode: {enc}"             # <<<<<<<<<<<<<<
 *     print("Pass Dec Encoded Integer".ljust(80))
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(__pyx_assertions_enabled())) {
      __pyx_t_3 = __Pyx_PyInt_From_unsigned_PY_LONG_LONG(__pyx_v_i); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1617, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_5 = PyObject_RichCompare(__pyx_v_num, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1617, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(1, 1617, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_8)) {
        __pyx_t_5 = PyTuple_New(6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1617, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_9 = 0;
        __pyx_t_10 = 127;
        __Pyx_INCREF(__pyx_kp_u_num);
        __pyx_t_9 += 4;
        __Pyx_GIVEREF(__pyx_kp_u_num);
        PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_kp_u_num);
        __pyx_t_3 = __Pyx_PyObject_FormatSimple(__pyx_v_num, __pyx_empty_unicode); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1617, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_10 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) > __pyx_t_10) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) : __pyx_t_10;
        __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_3);
        __pyx_t_3 = 0;
        __Pyx_INCREF(__pyx_kp_u_vs_i);
        __pyx_t_9 += 6;
        __Pyx_GIVEREF(__pyx_kp_u_vs_i);
        PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_kp_u_vs_i);
        __pyx_t_3 = __Pyx_PyUnicode_From_unsigned_PY_LONG_LONG(__pyx_v_i, 0, ' ', 'd'); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1617, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_3);
        __pyx_t_3 = 0;
        __Pyx_INCREF(__pyx_kp_u_encode);
        __pyx_t_9 += 11;
        __Pyx_GIVEREF(__pyx_kp_u_encode);
        PyTuple_SET_ITEM(__pyx_t_5, 4, __pyx_kp_u_encode);
        __pyx_t_3 = __Pyx_PyObject_FormatSimple(__pyx_v_enc, __pyx_empty_unicode); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1617, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_10 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) > __pyx_t_10) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) : __pyx_t_10;
        __pyx_t_9 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_5, 5, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyUnicode_Join(__pyx_t_5, 6, __pyx_t_9, __pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1617, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_Raise(__pyx_builtin_AssertionError, __pyx_t_3, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __PYX_ERR(1, 1617, __pyx_L1_error)
      }
    }
    #else
    if ((1)); else __PYX_ERR(1, 1617, __pyx_L1_error)
    #endif

    /* "serializor/unicode/des.py":1607
 * 
 * def _test_dec_encoded_int() -> None:
 *     for enc, num in [             # <<<<<<<<<<<<<<
 *         ("", 251),
 *         ("\x00", 252),
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "serializor/unicode/des.py":1618
 *         i = dec_encoded_int(enc, [0])  # type: ignore
 *         assert num == i, f"num {num} vs i {i} - encode: {enc}"
 *     print("Pass Dec Encoded Integer".ljust(80))             # <<<<<<<<<<<<<<
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_Pass_Dec_Encoded_Integer, __pyx_n_s_ljust); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1618, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = NULL;
  __pyx_t_11 = 0;
  #if CYTHON_UNPACK_METHODS
  if (likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_11 = 1;
    }
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_int_80};
    __pyx_t_1 = __Pyx_PyObject_FastCall(__pyx_t_3, __pyx_callargs+1-__pyx_t_11, 1+__pyx_t_11);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1618, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1618, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "serializor/unicode/des.py":1606
 * 
 * 
 * def _test_dec_encoded_int() -> None:             # <<<<<<<<<<<<<<
 *     for enc, num in [
 *         ("", 251),
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("serializor.unicode.des._test_dec_encoded_int", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_enc);
  __Pyx_XDECREF(__pyx_v_num);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif
/* #### Code section: pystring_table ### */

static int __Pyx_CreateStringTabAndInitStrings(void) {
  __Pyx_StringTabEntry __pyx_string_tab[] = {
    {&__pyx_n_s_AssertionError, __pyx_k_AssertionError, sizeof(__pyx_k_AssertionError), 0, 0, 1, 1},
    {&__pyx_kp_u_B, __pyx_k_B, sizeof(__pyx_k_B), 0, 1, 0, 0},
    {&__pyx_n_u_D, __pyx_k_D, sizeof(__pyx_k_D), 0, 1, 0, 1},
    {&__pyx_n_s_DeserializeError, __pyx_k_DeserializeError, sizeof(__pyx_k_DeserializeError), 0, 0, 1, 1},
    {&__pyx_kp_u_Error, __pyx_k_Error, sizeof(__pyx_k_Error), 0, 1, 0, 0},
    {&__pyx_kp_u_Failed_to_parse_arr_time_unit_fr, __pyx_k_Failed_to_parse_arr_time_unit_fr, sizeof(__pyx_k_Failed_to_parse_arr_time_unit_fr), 0, 1, 0, 0},
    {&__pyx_kp_u_H, __pyx_k_H, sizeof(__pyx_k_H), 0, 1, 0, 0},
    {&__pyx_kp_u_I, __pyx_k_I, sizeof(__pyx_k_I), 0, 1, 0, 0},
    {&__pyx_n_s_ImportError, __pyx_k_ImportError, sizeof(__pyx_k_ImportError), 0, 0, 1, 1},
    {&__pyx_kp_u_Invalid_encoded_integer_value_d, __pyx_k_Invalid_encoded_integer_value_d, sizeof(__pyx_k_Invalid_encoded_integer_value_d), 0, 1, 0, 0},
    {&__pyx_n_u_M, __pyx_k_M, sizeof(__pyx_k_M), 0, 1, 0, 1},
    {&__pyx_n_s_None, __pyx_k_None, sizeof(__pyx_k_None), 0, 0, 1, 1},
    {&__pyx_kp_u_Pass_Dec_Encoded_Integer, __pyx_k_Pass_Dec_Encoded_Integer, sizeof(__pyx_k_Pass_Dec_Encoded_Integer), 0, 1, 0, 0},
    {&__pyx_kp_u_Pass_Unpack_int16, __pyx_k_Pass_Unpack_int16, sizeof(__pyx_k_Pass_Unpack_int16), 0, 1, 0, 0},
    {&__pyx_kp_u_Pass_Unpack_int24, __pyx_k_Pass_Unpack_int24, sizeof(__pyx_k_Pass_Unpack_int24), 0, 1, 0, 0},
    {&__pyx_kp_u_Pass_Unpack_int32, __pyx_k_Pass_Unpack_int32, sizeof(__pyx_k_Pass_Unpack_int32), 0, 1, 0, 0},
    {&__pyx_kp_u_Pass_Unpack_int64, __pyx_k_Pass_Unpack_int64, sizeof(__pyx_k_Pass_Unpack_int64), 0, 1, 0, 0},
    {&__pyx_kp_u_Pass_Unpack_int8, __pyx_k_Pass_Unpack_int8, sizeof(__pyx_k_Pass_Unpack_int8), 0, 1, 0, 0},
    {&__pyx_kp_u_Pass_Unpack_uint16, __pyx_k_Pass_Unpack_uint16, sizeof(__pyx_k_Pass_Unpack_uint16), 0, 1, 0, 0},
    {&__pyx_kp_u_Pass_Unpack_uint24, __pyx_k_Pass_Unpack_uint24, sizeof(__pyx_k_Pass_Unpack_uint24), 0, 1, 0, 0},
    {&__pyx_kp_u_Pass_Unpack_uint32, __pyx_k_Pass_Unpack_uint32, sizeof(__pyx_k_Pass_Unpack_uint32), 0, 1, 0, 0},
    {&__pyx_kp_u_Pass_Unpack_uint64, __pyx_k_Pass_Unpack_uint64, sizeof(__pyx_k_Pass_Unpack_uint64), 0, 1, 0, 0},
    {&__pyx_kp_u_Pass_Unpack_uint8, __pyx_k_Pass_Unpack_uint8, sizeof(__pyx_k_Pass_Unpack_uint8), 0, 1, 0, 0},
    {&__pyx_kp_u_Q, __pyx_k_Q, sizeof(__pyx_k_Q), 0, 1, 0, 0},
    {&__pyx_n_s_RuntimeError, __pyx_k_RuntimeError, sizeof(__pyx_k_RuntimeError), 0, 0, 1, 1},
    {&__pyx_kp_u_S_d, __pyx_k_S_d, sizeof(__pyx_k_S_d), 0, 1, 0, 0},
    {&__pyx_kp_u_Serializor_Failed_to_deserializ, __pyx_k_Serializor_Failed_to_deserializ, sizeof(__pyx_k_Serializor_Failed_to_deserializ), 0, 1, 0, 0},
    {&__pyx_kp_u_Time_zones_are_not_available_fro, __pyx_k_Time_zones_are_not_available_fro, sizeof(__pyx_k_Time_zones_are_not_available_fro), 0, 1, 0, 0},
    {&__pyx_n_u_UTC, __pyx_k_UTC, sizeof(__pyx_k_UTC), 0, 1, 0, 1},
    {&__pyx_kp_u_U_d, __pyx_k_U_d, sizeof(__pyx_k_U_d), 0, 1, 0, 0},
    {&__pyx_kp_u_Unsupported_numpy_time_unit_d, __pyx_k_Unsupported_numpy_time_unit_d, sizeof(__pyx_k_Unsupported_numpy_time_unit_d), 0, 1, 0, 0},
    {&__pyx_kp_u_Unsupported_numpy_time_unit_s, __pyx_k_Unsupported_numpy_time_unit_s, sizeof(__pyx_k_Unsupported_numpy_time_unit_s), 0, 1, 0, 0},
    {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
    {&__pyx_n_u_W, __pyx_k_W, sizeof(__pyx_k_W), 0, 1, 0, 1},
    {&__pyx_n_u_Y, __pyx_k_Y, sizeof(__pyx_k_Y), 0, 1, 0, 1},
    {&__pyx_n_s_ZoneInfo, __pyx_k_ZoneInfo, sizeof(__pyx_k_ZoneInfo), 0, 0, 1, 1},
    {&__pyx_n_u__16, __pyx_k__16, sizeof(__pyx_k__16), 0, 1, 0, 1},
    {&__pyx_kp_u__18, __pyx_k__18, sizeof(__pyx_k__18), 0, 1, 0, 0},
    {&__pyx_n_u__20, __pyx_k__20, sizeof(__pyx_k__20), 0, 1, 0, 1},
    {&__pyx_kp_u__22, __pyx_k__22, sizeof(__pyx_k__22), 0, 1, 0, 0},
    {&__pyx_n_u__24, __pyx_k__24, sizeof(__pyx_k__24), 0, 1, 0, 1},
    {&__pyx_kp_u__26, __pyx_k__26, sizeof(__pyx_k__26), 0, 1, 0, 0},
    {&__pyx_kp_u__28, __pyx_k__28, sizeof(__pyx_k__28), 0, 1, 0, 0},
    {&__pyx_n_s__47, __pyx_k__47, sizeof(__pyx_k__47), 0, 0, 1, 1},
    {&__pyx_kp_u__5, __pyx_k__5, sizeof(__pyx_k__5), 0, 1, 0, 0},
    {&__pyx_kp_u__6, __pyx_k__6, sizeof(__pyx_k__6), 0, 1, 0, 0},
    {&__pyx_n_s__7, __pyx_k__7, sizeof(__pyx_k__7), 0, 0, 1, 1},
    {&__pyx_kp_u__8, __pyx_k__8, sizeof(__pyx_k__8), 0, 1, 0, 0},
    {&__pyx_n_s_all, __pyx_k_all, sizeof(__pyx_k_all), 0, 0, 1, 1},
    {&__pyx_n_u_as, __pyx_k_as, sizeof(__pyx_k_as), 0, 1, 0, 1},
    {&__pyx_n_s_asyncio_coroutines, __pyx_k_asyncio_coroutines, sizeof(__pyx_k_asyncio_coroutines), 0, 0, 1, 1},
    {&__pyx_kp_u_b, __pyx_k_b, sizeof(__pyx_k_b), 0, 1, 0, 0},
    {&__pyx_n_s_b_2, __pyx_k_b_2, sizeof(__pyx_k_b_2), 0, 0, 1, 1},
    {&__pyx_n_s_chr, __pyx_k_chr, sizeof(__pyx_k_chr), 0, 0, 1, 1},
    {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
    {&__pyx_n_s_columns, __pyx_k_columns, sizeof(__pyx_k_columns), 0, 0, 1, 1},
    {&__pyx_n_s_copy, __pyx_k_copy, sizeof(__pyx_k_copy), 0, 0, 1, 1},
    {&__pyx_n_u_d, __pyx_k_d, sizeof(__pyx_k_d), 0, 1, 0, 1},
    {&__pyx_n_s_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 0, 1, 1},
    {&__pyx_kp_u_data_is_empty, __pyx_k_data_is_empty, sizeof(__pyx_k_data_is_empty), 0, 1, 0, 0},
    {&__pyx_n_s_datetime, __pyx_k_datetime, sizeof(__pyx_k_datetime), 0, 0, 1, 1},
    {&__pyx_kp_u_datetime64_s, __pyx_k_datetime64_s, sizeof(__pyx_k_datetime64_s), 0, 1, 0, 0},
    {&__pyx_n_s_decode, __pyx_k_decode, sizeof(__pyx_k_decode), 0, 0, 1, 1},
    {&__pyx_n_s_deserialize, __pyx_k_deserialize, sizeof(__pyx_k_deserialize), 0, 0, 1, 1},
    {&__pyx_n_u_deserialize, __pyx_k_deserialize, sizeof(__pyx_k_deserialize), 0, 1, 0, 1},
    {&__pyx_n_s_dt, __pyx_k_dt, sizeof(__pyx_k_dt), 0, 0, 1, 1},
    {&__pyx_n_s_dtype, __pyx_k_dtype, sizeof(__pyx_k_dtype), 0, 0, 1, 1},
    {&__pyx_n_s_empty, __pyx_k_empty, sizeof(__pyx_k_empty), 0, 0, 1, 1},
    {&__pyx_n_s_enc, __pyx_k_enc, sizeof(__pyx_k_enc), 0, 0, 1, 1},
    {&__pyx_kp_u_encode, __pyx_k_encode, sizeof(__pyx_k_encode), 0, 1, 0, 0},
    {&__pyx_n_s_end, __pyx_k_end, sizeof(__pyx_k_end), 0, 0, 1, 1},
    {&__pyx_kp_u_eof, __pyx_k_eof, sizeof(__pyx_k_eof), 0, 1, 0, 0},
    {&__pyx_n_s_errors, __pyx_k_errors, sizeof(__pyx_k_errors), 0, 0, 1, 1},
    {&__pyx_n_u_fs, __pyx_k_fs, sizeof(__pyx_k_fs), 0, 1, 0, 1},
    {&__pyx_n_u_h, __pyx_k_h, sizeof(__pyx_k_h), 0, 1, 0, 1},
    {&__pyx_kp_u_h_2, __pyx_k_h_2, sizeof(__pyx_k_h_2), 0, 1, 0, 0},
    {&__pyx_kp_u_i, __pyx_k_i, sizeof(__pyx_k_i), 0, 1, 0, 0},
    {&__pyx_n_s_i_2, __pyx_k_i_2, sizeof(__pyx_k_i_2), 0, 0, 1, 1},
    {&__pyx_n_s_identifier, __pyx_k_identifier, sizeof(__pyx_k_identifier), 0, 0, 1, 1},
    {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
    {&__pyx_kp_u_improper_pos, __pyx_k_improper_pos, sizeof(__pyx_k_improper_pos), 0, 1, 0, 0},
    {&__pyx_n_s_initializing, __pyx_k_initializing, sizeof(__pyx_k_initializing), 0, 0, 1, 1},
    {&__pyx_n_s_is_coroutine, __pyx_k_is_coroutine, sizeof(__pyx_k_is_coroutine), 0, 0, 1, 1},
    {&__pyx_n_u_latin1, __pyx_k_latin1, sizeof(__pyx_k_latin1), 0, 1, 0, 1},
    {&__pyx_n_s_ljust, __pyx_k_ljust, sizeof(__pyx_k_ljust), 0, 0, 1, 1},
    {&__pyx_n_s_loads, __pyx_k_loads, sizeof(__pyx_k_loads), 0, 0, 1, 1},
    {&__pyx_n_s_loads_2, __pyx_k_loads_2, sizeof(__pyx_k_loads_2), 0, 0, 1, 1},
    {&__pyx_n_u_m, __pyx_k_m, sizeof(__pyx_k_m), 0, 1, 0, 1},
    {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
    {&__pyx_n_u_ms, __pyx_k_ms, sizeof(__pyx_k_ms), 0, 1, 0, 1},
    {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
    {&__pyx_n_s_name_2, __pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 0, 1, 1},
    {&__pyx_n_s_np, __pyx_k_np, sizeof(__pyx_k_np), 0, 0, 1, 1},
    {&__pyx_n_u_ns, __pyx_k_ns, sizeof(__pyx_k_ns), 0, 1, 0, 1},
    {&__pyx_kp_u_num, __pyx_k_num, sizeof(__pyx_k_num), 0, 1, 0, 0},
    {&__pyx_n_s_num_2, __pyx_k_num_2, sizeof(__pyx_k_num_2), 0, 0, 1, 1},
    {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
    {&__pyx_kp_u_numpy__core_multiarray_failed_to, __pyx_k_numpy__core_multiarray_failed_to, sizeof(__pyx_k_numpy__core_multiarray_failed_to), 0, 1, 0, 0},
    {&__pyx_kp_u_numpy__core_umath_failed_to_impo, __pyx_k_numpy__core_umath_failed_to_impo, sizeof(__pyx_k_numpy__core_umath_failed_to_impo), 0, 1, 0, 0},
    {&__pyx_n_s_orjson, __pyx_k_orjson, sizeof(__pyx_k_orjson), 0, 0, 1, 1},
    {&__pyx_n_s_pack, __pyx_k_pack, sizeof(__pyx_k_pack), 0, 0, 1, 1},
    {&__pyx_kp_u_pass_s, __pyx_k_pass_s, sizeof(__pyx_k_pass_s), 0, 1, 0, 0},
    {&__pyx_n_s_print, __pyx_k_print, sizeof(__pyx_k_print), 0, 0, 1, 1},
    {&__pyx_n_u_ps, __pyx_k_ps, sizeof(__pyx_k_ps), 0, 1, 0, 1},
    {&__pyx_kp_u_q, __pyx_k_q, sizeof(__pyx_k_q), 0, 1, 0, 0},
    {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
    {&__pyx_n_s_return, __pyx_k_return, sizeof(__pyx_k_return), 0, 0, 1, 1},
    {&__pyx_n_u_s, __pyx_k_s, sizeof(__pyx_k_s), 0, 1, 0, 1},
    {&__pyx_n_s_serializor, __pyx_k_serializor, sizeof(__pyx_k_serializor), 0, 0, 1, 1},
    {&__pyx_n_s_serializor_unicode_des, __pyx_k_serializor_unicode_des, sizeof(__pyx_k_serializor_unicode_des), 0, 0, 1, 1},
    {&__pyx_n_s_spec, __pyx_k_spec, sizeof(__pyx_k_spec), 0, 0, 1, 1},
    {&__pyx_kp_s_src_serializor_unicode_des_py, __pyx_k_src_serializor_unicode_des_py, sizeof(__pyx_k_src_serializor_unicode_des_py), 0, 0, 1, 0},
    {&__pyx_n_s_str, __pyx_k_str, sizeof(__pyx_k_str), 0, 0, 1, 1},
    {&__pyx_n_s_struct, __pyx_k_struct, sizeof(__pyx_k_struct), 0, 0, 1, 1},
    {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
    {&__pyx_n_s_test_dec_encoded_int, __pyx_k_test_dec_encoded_int, sizeof(__pyx_k_test_dec_encoded_int), 0, 0, 1, 1},
    {&__pyx_n_s_test_unpack_int16, __pyx_k_test_unpack_int16, sizeof(__pyx_k_test_unpack_int16), 0, 0, 1, 1},
    {&__pyx_n_s_test_unpack_int24, __pyx_k_test_unpack_int24, sizeof(__pyx_k_test_unpack_int24), 0, 0, 1, 1},
    {&__pyx_n_s_test_unpack_int32, __pyx_k_test_unpack_int32, sizeof(__pyx_k_test_unpack_int32), 0, 0, 1, 1},
    {&__pyx_n_s_test_unpack_int64, __pyx_k_test_unpack_int64, sizeof(__pyx_k_test_unpack_int64), 0, 0, 1, 1},
    {&__pyx_n_s_test_unpack_int8, __pyx_k_test_unpack_int8, sizeof(__pyx_k_test_unpack_int8), 0, 0, 1, 1},
    {&__pyx_n_s_test_unpack_uint16, __pyx_k_test_unpack_uint16, sizeof(__pyx_k_test_unpack_uint16), 0, 0, 1, 1},
    {&__pyx_n_s_test_unpack_uint24, __pyx_k_test_unpack_uint24, sizeof(__pyx_k_test_unpack_uint24), 0, 0, 1, 1},
    {&__pyx_n_s_test_unpack_uint32, __pyx_k_test_unpack_uint32, sizeof(__pyx_k_test_unpack_uint32), 0, 0, 1, 1},
    {&__pyx_n_s_test_unpack_uint64, __pyx_k_test_unpack_uint64, sizeof(__pyx_k_test_unpack_uint64), 0, 0, 1, 1},
    {&__pyx_n_s_test_unpack_uint8, __pyx_k_test_unpack_uint8, sizeof(__pyx_k_test_unpack_uint8), 0, 0, 1, 1},
    {&__pyx_n_s_test_utils, __pyx_k_test_utils, sizeof(__pyx_k_test_utils), 0, 0, 1, 1},
    {&__pyx_kp_u_timedelta64_s, __pyx_k_timedelta64_s, sizeof(__pyx_k_timedelta64_s), 0, 1, 0, 0},
    {&__pyx_n_s_typeref, __pyx_k_typeref, sizeof(__pyx_k_typeref), 0, 0, 1, 1},
    {&__pyx_n_s_tz, __pyx_k_tz, sizeof(__pyx_k_tz), 0, 0, 1, 1},
    {&__pyx_n_s_tz_convert, __pyx_k_tz_convert, sizeof(__pyx_k_tz_convert), 0, 0, 1, 1},
    {&__pyx_n_s_tz_localize, __pyx_k_tz_localize, sizeof(__pyx_k_tz_localize), 0, 0, 1, 1},
    {&__pyx_n_s_tzinfo, __pyx_k_tzinfo, sizeof(__pyx_k_tzinfo), 0, 0, 1, 1},
    {&__pyx_kp_u_unknown_data_identifer_s, __pyx_k_unknown_data_identifer_s, sizeof(__pyx_k_unknown_data_identifer_s), 0, 1, 0, 0},
    {&__pyx_kp_u_unsupported_numpy_ndarray_dimens, __pyx_k_unsupported_numpy_ndarray_dimens, sizeof(__pyx_k_unsupported_numpy_ndarray_dimens), 0, 1, 0, 0},
    {&__pyx_kp_u_unsupported_numpy_ndarray_dtype, __pyx_k_unsupported_numpy_ndarray_dtype, sizeof(__pyx_k_unsupported_numpy_ndarray_dtype), 0, 1, 0, 0},
    {&__pyx_kp_u_unsupported_pandas_DataFrame_col, __pyx_k_unsupported_pandas_DataFrame_col, sizeof(__pyx_k_unsupported_pandas_DataFrame_col), 0, 1, 0, 0},
    {&__pyx_kp_u_unsupported_pandas_Series_dtype, __pyx_k_unsupported_pandas_Series_dtype, sizeof(__pyx_k_unsupported_pandas_Series_dtype), 0, 1, 0, 0},
    {&__pyx_n_u_us, __pyx_k_us, sizeof(__pyx_k_us), 0, 1, 0, 1},
    {&__pyx_n_s_utils, __pyx_k_utils, sizeof(__pyx_k_utils), 0, 0, 1, 1},
    {&__pyx_kp_u_val, __pyx_k_val, sizeof(__pyx_k_val), 0, 1, 0, 0},
    {&__pyx_n_s_val_2, __pyx_k_val_2, sizeof(__pyx_k_val_2), 0, 0, 1, 1},
    {&__pyx_kp_u_vs_i, __pyx_k_vs_i, sizeof(__pyx_k_vs_i), 0, 1, 0, 0},
    {&__pyx_n_s_zoneinfo, __pyx_k_zoneinfo, sizeof(__pyx_k_zoneinfo), 0, 0, 1, 1},
    {0, 0, 0, 0, 0, 0, 0}
  };
  return __Pyx_InitStrings(__pyx_string_tab);
}
/* #### Code section: cached_builtins ### */
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 119, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(1, 295, __pyx_L1_error)
  __pyx_builtin_chr = __Pyx_GetBuiltinName(__pyx_n_s_chr); if (!__pyx_builtin_chr) __PYX_ERR(1, 432, __pyx_L1_error)
  __pyx_builtin_AssertionError = __Pyx_GetBuiltinName(__pyx_n_s_AssertionError); if (!__pyx_builtin_AssertionError) __PYX_ERR(1, 1470, __pyx_L1_error)
  __pyx_builtin_print = __Pyx_GetBuiltinName(__pyx_n_s_print); if (!__pyx_builtin_print) __PYX_ERR(1, 1471, __pyx_L1_error)
  __pyx_builtin_ImportError = __Pyx_GetBuiltinName(__pyx_n_s_ImportError); if (!__pyx_builtin_ImportError) __PYX_ERR(3, 1025, __pyx_L1_error)
  __pyx_builtin_RuntimeError = __Pyx_GetBuiltinName(__pyx_n_s_RuntimeError); if (!__pyx_builtin_RuntimeError) __PYX_ERR(5, 308, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cached_constants ### */

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1025
 *         __pyx_import_array()
 *     except Exception:
 *         raise ImportError("numpy._core.multiarray failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_umath() except -1:
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_u_numpy__core_multiarray_failed_to); if (unlikely(!__pyx_tuple_)) __PYX_ERR(3, 1025, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "../../../../../tmp/build-env-4z4oxfkx/lib/python3.12/site-packages/numpy/__init__.cython-30.pxd":1031
 *         _import_umath()
 *     except Exception:
 *         raise ImportError("numpy._core.umath failed to import")             # <<<<<<<<<<<<<<
 * 
 * cdef inline int import_ufunc() except -1:
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_u_numpy__core_umath_failed_to_impo); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(3, 1031, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "datetime.pxd":308
 * cdef inline object timezone_new(object offset, object name=None):
 *     if PY_VERSION_HEX < 0x030700b1:
 *         raise RuntimeError('Time zones are not available from the C-API.')             # <<<<<<<<<<<<<<
 *     return __Pyx_TimeZone_FromOffsetAndName(offset, <PyObject*>name if name is not None else NULL)
 * 
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_u_Time_zones_are_not_available_fro); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(5, 308, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "serializor/unicode/des.py":1436
 *         eof: cython.Py_ssize_t = str_len(data)
 *         if eof == 0:
 *             raise ValueError("'data' is empty.")             # <<<<<<<<<<<<<<
 *         pos: cython.Py_ssize_t[1] = [0]
 *         obj = _des_data(data, pos)
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_u_data_is_empty); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(1, 1436, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "serializor/unicode/des.py":1519
 *     import struct
 * 
 *     for val in (-8388608, -8388607, 0, 8388606, 8388607):             # <<<<<<<<<<<<<<
 *         b = (struct.pack("<i", val)[:3]).decode("latin1")
 *         i = unpack_int24(b, 0)  # type: ignore
 */
  __pyx_tuple__9 = PyTuple_Pack(5, __pyx_int_neg_8388608, __pyx_int_neg_8388607, __pyx_int_0, __pyx_int_8388606, __pyx_int_8388607); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(1, 1519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "serializor/unicode/des.py":1520
 * 
 *     for val in (-8388608, -8388607, 0, 8388606, 8388607):
 *         b = (struct.pack("<i", val)[:3]).decode("latin1")             # <<<<<<<<<<<<<<
 *         i = unpack_int24(b, 0)  # type: ignore
 *         assert val == i, f"val {val} vs i {i}"
 */
  __pyx_slice__10 = PySlice_New(Py_None, __pyx_int_3, Py_None); if (unlikely(!__pyx_slice__10)) __PYX_ERR(1, 1520, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__10);
  __Pyx_GIVEREF(__pyx_slice__10);

  /* "serializor/unicode/des.py":1532
 *     import struct
 * 
 *     for val in (0, 1, 16777213, 16777214, 16777215):             # <<<<<<<<<<<<<<
 *         b = (struct.pack("<I", val)[:3]).decode("latin1")
 *         i = unpack_uint24(b, 0)  # type: ignore
 */
  __pyx_tuple__11 = PyTuple_Pack(5, __pyx_int_0, __pyx_int_1, __pyx_int_16777213, __pyx_int_16777214, __pyx_int_16777215); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(1, 1532, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "serializor/unicode/des.py":1545
 *     import struct
 * 
 *     for val in (-2147483648, -2147483647, 0, 2147483646, 2147483647):             # <<<<<<<<<<<<<<
 *         b = struct.pack("<i", val).decode("latin1")
 *         i = unpack_int32(b, 0)  # type: ignore
 */
  __pyx_tuple__12 = PyTuple_Pack(5, __pyx_int_neg_2147483648, __pyx_int_neg_2147483647, __pyx_int_0, __pyx_int_2147483646, __pyx_int_2147483647); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(1, 1545, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "serializor/unicode/des.py":1558
 *     import struct
 * 
 *     for val in (0, 1, 4294967293, 4294967294, 4294967295):             # <<<<<<<<<<<<<<
 *         b = struct.pack("<I", val).decode("latin1")
 *         i = unpack_uint32(b, 0)  # type: ignore
 */
  __pyx_tuple__13 = PyTuple_Pack(5, __pyx_int_0, __pyx_int_1, __pyx_int_4294967293, __pyx_int_4294967294, __pyx_int_4294967295); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(1, 1558, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "serializor/unicode/des.py":1572
 * 
 *     for val in (
 *         -9223372036854775808,             # <<<<<<<<<<<<<<
 *         -9223372036854775807,
 *         0,
 */
  __pyx_tuple__14 = PyTuple_Pack(5, __pyx_int_neg_9223372036854775808, __pyx_int_neg_9223372036854775807, __pyx_int_0, __pyx_int_0x7ffffffffffffffe, __pyx_int_0x7fffffffffffffff); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(1, 1572, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);

  /* "serializor/unicode/des.py":1591
 * 
 *     for val in (
 *         0,             # <<<<<<<<<<<<<<
 *         1,
 *         18446744073709551613,
 */
  __pyx_tuple__15 = PyTuple_Pack(5, __pyx_int_0, __pyx_int_1, __pyx_int_0xfffffffffffffffd, __pyx_int_0xfffffffffffffffe, __pyx_int_0xffffffffffffffff); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(1, 1591, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "serializor/unicode/des.py":1608
 * def _test_dec_encoded_int() -> None:
 *     for enc, num in [
 *         ("", 251),             # <<<<<<<<<<<<<<
 *         ("\x00", 252),
 *         ("", 65535),
 */
  __pyx_tuple__17 = PyTuple_Pack(2, __pyx_n_u__16, __pyx_int_251); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(1, 1608, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "serializor/unicode/des.py":1609
 *     for enc, num in [
 *         ("", 251),
 *         ("\x00", 252),             # <<<<<<<<<<<<<<
 *         ("", 65535),
 *         ("\x00\x00\x01\x00", 65536),
 */
  __pyx_tuple__19 = PyTuple_Pack(2, __pyx_kp_u__18, __pyx_int_252); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(1, 1609, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);

  /* "serializor/unicode/des.py":1610
 *         ("", 251),
 *         ("\x00", 252),
 *         ("", 65535),             # <<<<<<<<<<<<<<
 *         ("\x00\x00\x01\x00", 65536),
 *         ("", 4294967295),
 */
  __pyx_tuple__21 = PyTuple_Pack(2, __pyx_n_u__20, __pyx_int_65535); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(1, 1610, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);

  /* "serializor/unicode/des.py":1611
 *         ("\x00", 252),
 *         ("", 65535),
 *         ("\x00\x00\x01\x00", 65536),             # <<<<<<<<<<<<<<
 *         ("", 4294967295),
 *         ("\x00\x00\x00\x00\x01\x00\x00\x00", 4294967296),
 */
  __pyx_tuple__23 = PyTuple_Pack(2, __pyx_kp_u__22, __pyx_int_65536); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(1, 1611, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);

  /* "serializor/unicode/des.py":1612
 *         ("", 65535),
 *         ("\x00\x00\x01\x00", 65536),
 *         ("", 4294967295),             # <<<<<<<<<<<<<<
 *         ("\x00\x00\x00\x00\x01\x00\x00\x00", 4294967296),
 *         ("\x00\x00\x00\x00\x02\x00\x00\x00", 8589934592),
 */
  __pyx_tuple__25 = PyTuple_Pack(2, __pyx_n_u__24, __pyx_int_4294967295); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(1, 1612, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);

  /* "serializor/unicode/des.py":1613
 *         ("\x00\x00\x01\x00", 65536),
 *         ("", 4294967295),
 *         ("\x00\x00\x00\x00\x01\x00\x00\x00", 4294967296),             # <<<<<<<<<<<<<<
 *         ("\x00\x00\x00\x00\x02\x00\x00\x00", 8589934592),
 *     ]:
 */
  __pyx_tuple__27 = PyTuple_Pack(2, __pyx_kp_u__26, __pyx_int_4294967296); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(1, 1613, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);

  /* "serializor/unicode/des.py":1614
 *         ("", 4294967295),
 *         ("\x00\x00\x00\x00\x01\x00\x00\x00", 4294967296),
 *         ("\x00\x00\x00\x00\x02\x00\x00\x00", 8589934592),             # <<<<<<<<<<<<<<
 *     ]:
 *         i = dec_encoded_int(enc, [0])  # type: ignore
 */
  __pyx_tuple__29 = PyTuple_Pack(2, __pyx_kp_u__28, __pyx_int_8589934592); if (unlikely(!__pyx_tuple__29)) __PYX_ERR(1, 1614, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);

  /* "serializor/unicode/des.py":1607
 * 
 * def _test_dec_encoded_int() -> None:
 *     for enc, num in [             # <<<<<<<<<<<<<<
 *         ("", 251),
 *         ("\x00", 252),
 */
  __pyx_tuple__30 = PyTuple_Pack(7, __pyx_tuple__17, __pyx_tuple__19, __pyx_tuple__21, __pyx_tuple__23, __pyx_tuple__25, __pyx_tuple__27, __pyx_tuple__29); if (unlikely(!__pyx_tuple__30)) __PYX_ERR(1, 1607, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__30);
  __Pyx_GIVEREF(__pyx_tuple__30);

  /* "serializor/unicode/des.py":1430
 * 
 * 
 * @cython.ccall             # <<<<<<<<<<<<<<
 * def deserialize(data: str) -> object:
 *     """Deserialize the data (str) back to an `<'object'>`."""
 */
  __pyx_tuple__31 = PyTuple_Pack(1, __pyx_n_s_data); if (unlikely(!__pyx_tuple__31)) __PYX_ERR(1, 1430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__31);
  __Pyx_GIVEREF(__pyx_tuple__31);
  __pyx_codeobj__32 = (PyObject*)__Pyx_PyCode_New(1, 0, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__31, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_serializor_unicode_des_py, __pyx_n_s_deserialize, 1430, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__32)) __PYX_ERR(1, 1430, __pyx_L1_error)

  /* "serializor/unicode/des.py":1450
 * 
 * ########## The following functions are for testing purpose only ##########
 * def _test_utils() -> None:             # <<<<<<<<<<<<<<
 *     _test_unpack_int8()
 *     _test_unpack_uint8()
 */
  __pyx_codeobj__33 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 0, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_serializor_unicode_des_py, __pyx_n_s_test_utils, 1450, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__33)) __PYX_ERR(1, 1450, __pyx_L1_error)

  /* "serializor/unicode/des.py":1464
 * 
 * 
 * def _test_unpack_int8() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */
  __pyx_tuple__34 = PyTuple_Pack(4, __pyx_n_s_struct, __pyx_n_s_val_2, __pyx_n_s_b_2, __pyx_n_s_i_2); if (unlikely(!__pyx_tuple__34)) __PYX_ERR(1, 1464, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__34);
  __Pyx_GIVEREF(__pyx_tuple__34);
  __pyx_codeobj__35 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__34, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_serializor_unicode_des_py, __pyx_n_s_test_unpack_int8, 1464, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__35)) __PYX_ERR(1, 1464, __pyx_L1_error)

  /* "serializor/unicode/des.py":1477
 * 
 * 
 * def _test_unpack_uint8() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */
  __pyx_codeobj__36 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__34, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_serializor_unicode_des_py, __pyx_n_s_test_unpack_uint8, 1477, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__36)) __PYX_ERR(1, 1477, __pyx_L1_error)

  /* "serializor/unicode/des.py":1490
 * 
 * 
 * def _test_unpack_int16() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */
  __pyx_codeobj__37 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__34, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_serializor_unicode_des_py, __pyx_n_s_test_unpack_int16, 1490, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__37)) __PYX_ERR(1, 1490, __pyx_L1_error)

  /* "serializor/unicode/des.py":1503
 * 
 * 
 * def _test_unpack_uint16() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */
  __pyx_codeobj__38 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__34, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_serializor_unicode_des_py, __pyx_n_s_test_unpack_uint16, 1503, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__38)) __PYX_ERR(1, 1503, __pyx_L1_error)

  /* "serializor/unicode/des.py":1516
 * 
 * 
 * def _test_unpack_int24() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */
  __pyx_codeobj__39 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__34, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_serializor_unicode_des_py, __pyx_n_s_test_unpack_int24, 1516, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__39)) __PYX_ERR(1, 1516, __pyx_L1_error)

  /* "serializor/unicode/des.py":1529
 * 
 * 
 * def _test_unpack_uint24() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */
  __pyx_codeobj__40 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__34, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_serializor_unicode_des_py, __pyx_n_s_test_unpack_uint24, 1529, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__40)) __PYX_ERR(1, 1529, __pyx_L1_error)

  /* "serializor/unicode/des.py":1542
 * 
 * 
 * def _test_unpack_int32() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */
  __pyx_codeobj__41 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__34, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_serializor_unicode_des_py, __pyx_n_s_test_unpack_int32, 1542, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__41)) __PYX_ERR(1, 1542, __pyx_L1_error)

  /* "serializor/unicode/des.py":1555
 * 
 * 
 * def _test_unpack_uint32() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */
  __pyx_codeobj__42 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__34, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_serializor_unicode_des_py, __pyx_n_s_test_unpack_uint32, 1555, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__42)) __PYX_ERR(1, 1555, __pyx_L1_error)

  /* "serializor/unicode/des.py":1568
 * 
 * 
 * def _test_unpack_int64() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */
  __pyx_codeobj__43 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__34, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_serializor_unicode_des_py, __pyx_n_s_test_unpack_int64, 1568, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__43)) __PYX_ERR(1, 1568, __pyx_L1_error)

  /* "serializor/unicode/des.py":1587
 * 
 * 
 * def _test_unpack_uint64() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */
  __pyx_codeobj__44 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__34, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_serializor_unicode_des_py, __pyx_n_s_test_unpack_uint64, 1587, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__44)) __PYX_ERR(1, 1587, __pyx_L1_error)

  /* "serializor/unicode/des.py":1606
 * 
 * 
 * def _test_dec_encoded_int() -> None:             # <<<<<<<<<<<<<<
 *     for enc, num in [
 *         ("", 251),
 */
  __pyx_tuple__45 = PyTuple_Pack(3, __pyx_n_s_enc, __pyx_n_s_num_2, __pyx_n_s_i_2); if (unlikely(!__pyx_tuple__45)) __PYX_ERR(1, 1606, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__45);
  __Pyx_GIVEREF(__pyx_tuple__45);
  __pyx_codeobj__46 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__45, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_src_serializor_unicode_des_py, __pyx_n_s_test_dec_encoded_int, 1606, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__46)) __PYX_ERR(1, 1606, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}
/* #### Code section: init_constants ### */

static CYTHON_SMALL_CODE int __Pyx_InitConstants(void) {
  if (__Pyx_CreateStringTabAndInitStrings() < 0) __PYX_ERR(1, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_3 = PyInt_FromLong(3); if (unlikely(!__pyx_int_3)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_80 = PyInt_FromLong(80); if (unlikely(!__pyx_int_80)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_251 = PyInt_FromLong(251); if (unlikely(!__pyx_int_251)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_252 = PyInt_FromLong(252); if (unlikely(!__pyx_int_252)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_65535 = PyInt_FromLong(65535L); if (unlikely(!__pyx_int_65535)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_65536 = PyInt_FromLong(65536L); if (unlikely(!__pyx_int_65536)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_8388606 = PyInt_FromLong(8388606L); if (unlikely(!__pyx_int_8388606)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_8388607 = PyInt_FromLong(8388607L); if (unlikely(!__pyx_int_8388607)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_16777213 = PyInt_FromLong(16777213L); if (unlikely(!__pyx_int_16777213)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_16777214 = PyInt_FromLong(16777214L); if (unlikely(!__pyx_int_16777214)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_16777215 = PyInt_FromLong(16777215L); if (unlikely(!__pyx_int_16777215)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_2147483646 = PyInt_FromLong(2147483646L); if (unlikely(!__pyx_int_2147483646)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_2147483647 = PyInt_FromLong(2147483647L); if (unlikely(!__pyx_int_2147483647)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_4294967293 = PyInt_FromString((char *)"4294967293", 0, 0); if (unlikely(!__pyx_int_4294967293)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_4294967294 = PyInt_FromString((char *)"4294967294", 0, 0); if (unlikely(!__pyx_int_4294967294)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_4294967295 = PyInt_FromString((char *)"4294967295", 0, 0); if (unlikely(!__pyx_int_4294967295)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_4294967296 = PyInt_FromString((char *)"4294967296", 0, 0); if (unlikely(!__pyx_int_4294967296)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_8589934592 = PyInt_FromString((char *)"8589934592", 0, 0); if (unlikely(!__pyx_int_8589934592)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_0x7ffffffffffffffe = PyInt_FromString((char *)"0x7ffffffffffffffe", 0, 0); if (unlikely(!__pyx_int_0x7ffffffffffffffe)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_0x7fffffffffffffff = PyInt_FromString((char *)"0x7fffffffffffffff", 0, 0); if (unlikely(!__pyx_int_0x7fffffffffffffff)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_0xfffffffffffffffd = PyInt_FromString((char *)"0xfffffffffffffffd", 0, 0); if (unlikely(!__pyx_int_0xfffffffffffffffd)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_0xfffffffffffffffe = PyInt_FromString((char *)"0xfffffffffffffffe", 0, 0); if (unlikely(!__pyx_int_0xfffffffffffffffe)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_0xffffffffffffffff = PyInt_FromString((char *)"0xffffffffffffffff", 0, 0); if (unlikely(!__pyx_int_0xffffffffffffffff)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_neg_8388607 = PyInt_FromLong(-8388607L); if (unlikely(!__pyx_int_neg_8388607)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_neg_8388608 = PyInt_FromLong(-8388608L); if (unlikely(!__pyx_int_neg_8388608)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_neg_2147483647 = PyInt_FromLong(-2147483647L); if (unlikely(!__pyx_int_neg_2147483647)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_neg_2147483648 = PyInt_FromLong(-2147483648L); if (unlikely(!__pyx_int_neg_2147483648)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_neg_9223372036854775807 = PyInt_FromString((char *)"-9223372036854775807", 0, 0); if (unlikely(!__pyx_int_neg_9223372036854775807)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_neg_9223372036854775808 = PyInt_FromString((char *)"-9223372036854775808", 0, 0); if (unlikely(!__pyx_int_neg_9223372036854775808)) __PYX_ERR(1, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: init_globals ### */

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  /* AssertionsEnabled.init */
  if (likely(__Pyx_init_assertions_enabled() == 0)); else

if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L1_error)

  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: init_module ### */

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  if (__Pyx_ExportFunction("deserialize", (void (*)(void))__pyx_f_10serializor_7unicode_3des_deserialize, "PyObject *(PyObject *, int __pyx_skip_dispatch)") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType_3_0_11(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyTypeObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(PyTypeObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_0_11); if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(7, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4bool_bool = __Pyx_ImportType_3_0_11(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "bool", sizeof(PyBoolObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyBoolObject),__Pyx_ImportType_CheckSize_Warn_3_0_11); if (!__pyx_ptype_7cpython_4bool_bool) __PYX_ERR(8, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_7complex_complex = __Pyx_ImportType_3_0_11(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "complex", sizeof(PyComplexObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyComplexObject),__Pyx_ImportType_CheckSize_Warn_3_0_11); if (!__pyx_ptype_7cpython_7complex_complex) __PYX_ERR(9, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("numpy"); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 271, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType_3_0_11(__pyx_t_1, "numpy", "dtype", sizeof(PyArray_Descr), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyArray_Descr),__Pyx_ImportType_CheckSize_Ignore_3_0_11); if (!__pyx_ptype_5numpy_dtype) __PYX_ERR(3, 271, __pyx_L1_error)
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType_3_0_11(__pyx_t_1, "numpy", "flatiter", sizeof(PyArrayIterObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyArrayIterObject),__Pyx_ImportType_CheckSize_Ignore_3_0_11); if (!__pyx_ptype_5numpy_flatiter) __PYX_ERR(3, 316, __pyx_L1_error)
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType_3_0_11(__pyx_t_1, "numpy", "broadcast", sizeof(PyArrayMultiIterObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyArrayMultiIterObject),__Pyx_ImportType_CheckSize_Ignore_3_0_11); if (!__pyx_ptype_5numpy_broadcast) __PYX_ERR(3, 320, __pyx_L1_error)
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType_3_0_11(__pyx_t_1, "numpy", "ndarray", sizeof(PyArrayObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyArrayObject),__Pyx_ImportType_CheckSize_Ignore_3_0_11); if (!__pyx_ptype_5numpy_ndarray) __PYX_ERR(3, 359, __pyx_L1_error)
  __pyx_ptype_5numpy_generic = __Pyx_ImportType_3_0_11(__pyx_t_1, "numpy", "generic", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_11); if (!__pyx_ptype_5numpy_generic) __PYX_ERR(3, 847, __pyx_L1_error)
  __pyx_ptype_5numpy_number = __Pyx_ImportType_3_0_11(__pyx_t_1, "numpy", "number", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_11); if (!__pyx_ptype_5numpy_number) __PYX_ERR(3, 849, __pyx_L1_error)
  __pyx_ptype_5numpy_integer = __Pyx_ImportType_3_0_11(__pyx_t_1, "numpy", "integer", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_11); if (!__pyx_ptype_5numpy_integer) __PYX_ERR(3, 851, __pyx_L1_error)
  __pyx_ptype_5numpy_signedinteger = __Pyx_ImportType_3_0_11(__pyx_t_1, "numpy", "signedinteger", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_11); if (!__pyx_ptype_5numpy_signedinteger) __PYX_ERR(3, 853, __pyx_L1_error)
  __pyx_ptype_5numpy_unsignedinteger = __Pyx_ImportType_3_0_11(__pyx_t_1, "numpy", "unsignedinteger", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_11); if (!__pyx_ptype_5numpy_unsignedinteger) __PYX_ERR(3, 855, __pyx_L1_error)
  __pyx_ptype_5numpy_inexact = __Pyx_ImportType_3_0_11(__pyx_t_1, "numpy", "inexact", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_11); if (!__pyx_ptype_5numpy_inexact) __PYX_ERR(3, 857, __pyx_L1_error)
  __pyx_ptype_5numpy_floating = __Pyx_ImportType_3_0_11(__pyx_t_1, "numpy", "floating", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_11); if (!__pyx_ptype_5numpy_floating) __PYX_ERR(3, 859, __pyx_L1_error)
  __pyx_ptype_5numpy_complexfloating = __Pyx_ImportType_3_0_11(__pyx_t_1, "numpy", "complexfloating", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_11); if (!__pyx_ptype_5numpy_complexfloating) __PYX_ERR(3, 861, __pyx_L1_error)
  __pyx_ptype_5numpy_flexible = __Pyx_ImportType_3_0_11(__pyx_t_1, "numpy", "flexible", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_11); if (!__pyx_ptype_5numpy_flexible) __PYX_ERR(3, 863, __pyx_L1_error)
  __pyx_ptype_5numpy_character = __Pyx_ImportType_3_0_11(__pyx_t_1, "numpy", "character", sizeof(PyObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyObject),__Pyx_ImportType_CheckSize_Warn_3_0_11); if (!__pyx_ptype_5numpy_character) __PYX_ERR(3, 865, __pyx_L1_error)
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType_3_0_11(__pyx_t_1, "numpy", "ufunc", sizeof(PyUFuncObject), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyUFuncObject),__Pyx_ImportType_CheckSize_Ignore_3_0_11); if (!__pyx_ptype_5numpy_ufunc) __PYX_ERR(3, 929, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("datetime"); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_8datetime_date = __Pyx_ImportType_3_0_11(__pyx_t_1, "datetime", "date", sizeof(PyDateTime_Date), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyDateTime_Date),__Pyx_ImportType_CheckSize_Warn_3_0_11); if (!__pyx_ptype_7cpython_8datetime_date) __PYX_ERR(5, 70, __pyx_L1_error)
  __pyx_ptype_7cpython_8datetime_time = __Pyx_ImportType_3_0_11(__pyx_t_1, "datetime", "time", sizeof(PyDateTime_Time), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyDateTime_Time),__Pyx_ImportType_CheckSize_Warn_3_0_11); if (!__pyx_ptype_7cpython_8datetime_time) __PYX_ERR(5, 83, __pyx_L1_error)
  __pyx_ptype_7cpython_8datetime_datetime = __Pyx_ImportType_3_0_11(__pyx_t_1, "datetime", "datetime", sizeof(PyDateTime_DateTime), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyDateTime_DateTime),__Pyx_ImportType_CheckSize_Warn_3_0_11); if (!__pyx_ptype_7cpython_8datetime_datetime) __PYX_ERR(5, 109, __pyx_L1_error)
  __pyx_ptype_7cpython_8datetime_timedelta = __Pyx_ImportType_3_0_11(__pyx_t_1, "datetime", "timedelta", sizeof(PyDateTime_Delta), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyDateTime_Delta),__Pyx_ImportType_CheckSize_Warn_3_0_11); if (!__pyx_ptype_7cpython_8datetime_timedelta) __PYX_ERR(5, 147, __pyx_L1_error)
  __pyx_ptype_7cpython_8datetime_tzinfo = __Pyx_ImportType_3_0_11(__pyx_t_1, "datetime", "tzinfo", sizeof(PyDateTime_TZInfo), __PYX_GET_STRUCT_ALIGNMENT_3_0_11(PyDateTime_TZInfo),__Pyx_ImportType_CheckSize_Warn_3_0_11); if (!__pyx_ptype_7cpython_8datetime_tzinfo) __PYX_ERR(5, 160, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __pyx_t_1 = PyImport_ImportModule("serializor.unicode.ser"); if (!__pyx_t_1) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "OPT_SERIALIZE_NUMPY", (void **)&__pyx_vp_10serializor_7unicode_3ser_OPT_SERIALIZE_NUMPY, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "UINT8_ENCODE_VALUE", (void **)&__pyx_vp_10serializor_7unicode_3ser_UINT8_ENCODE_VALUE, "Py_UCS4") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "UINT16_ENCODE_VALUE", (void **)&__pyx_vp_10serializor_7unicode_3ser_UINT16_ENCODE_VALUE, "Py_UCS4") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "UINT32_ENCODE_VALUE", (void **)&__pyx_vp_10serializor_7unicode_3ser_UINT32_ENCODE_VALUE, "Py_UCS4") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "UINT64_ENCODE_VALUE", (void **)&__pyx_vp_10serializor_7unicode_3ser_UINT64_ENCODE_VALUE, "Py_UCS4") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "DATETIME_ID", (void **)&__pyx_vp_10serializor_7unicode_3ser_DATETIME_ID, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "DATE_ID", (void **)&__pyx_vp_10serializor_7unicode_3ser_DATE_ID, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "TIME_ID", (void **)&__pyx_vp_10serializor_7unicode_3ser_TIME_ID, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "TIMEDELTA_ID", (void **)&__pyx_vp_10serializor_7unicode_3ser_TIMEDELTA_ID, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "STRUCT_TIME_ID", (void **)&__pyx_vp_10serializor_7unicode_3ser_STRUCT_TIME_ID, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "COMPLEX_ID", (void **)&__pyx_vp_10serializor_7unicode_3ser_COMPLEX_ID, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "LIST_ID", (void **)&__pyx_vp_10serializor_7unicode_3ser_LIST_ID, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "TUPLE_ID", (void **)&__pyx_vp_10serializor_7unicode_3ser_TUPLE_ID, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "SET_ID", (void **)&__pyx_vp_10serializor_7unicode_3ser_SET_ID, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "FROZENSET_ID", (void **)&__pyx_vp_10serializor_7unicode_3ser_FROZENSET_ID, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "RANGE_ID", (void **)&__pyx_vp_10serializor_7unicode_3ser_RANGE_ID, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "DICT_ID", (void **)&__pyx_vp_10serializor_7unicode_3ser_DICT_ID, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "DT64_ID", (void **)&__pyx_vp_10serializor_7unicode_3ser_DT64_ID, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "TD64_ID", (void **)&__pyx_vp_10serializor_7unicode_3ser_TD64_ID, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_ID", (void **)&__pyx_vp_10serializor_7unicode_3ser_NDARRAY_ID, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_OBJECT_DT", (void **)&__pyx_vp_10serializor_7unicode_3ser_NDARRAY_OBJECT_DT, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_INT_DT", (void **)&__pyx_vp_10serializor_7unicode_3ser_NDARRAY_INT_DT, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_UINT_DT", (void **)&__pyx_vp_10serializor_7unicode_3ser_NDARRAY_UINT_DT, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_FLOAT_DT", (void **)&__pyx_vp_10serializor_7unicode_3ser_NDARRAY_FLOAT_DT, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_BOOL_DT", (void **)&__pyx_vp_10serializor_7unicode_3ser_NDARRAY_BOOL_DT, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_DT64_DT", (void **)&__pyx_vp_10serializor_7unicode_3ser_NDARRAY_DT64_DT, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_TD64_DT", (void **)&__pyx_vp_10serializor_7unicode_3ser_NDARRAY_TD64_DT, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_COMPLEX_DT", (void **)&__pyx_vp_10serializor_7unicode_3ser_NDARRAY_COMPLEX_DT, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_BYTES_DT", (void **)&__pyx_vp_10serializor_7unicode_3ser_NDARRAY_BYTES_DT, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_UNICODE_DT", (void **)&__pyx_vp_10serializor_7unicode_3ser_NDARRAY_UNICODE_DT, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_OBJECT_IDDT", (void **)&__pyx_vp_10serializor_7unicode_3ser_NDARRAY_OBJECT_IDDT, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_INT_IDDT", (void **)&__pyx_vp_10serializor_7unicode_3ser_NDARRAY_INT_IDDT, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_UINT_IDDT", (void **)&__pyx_vp_10serializor_7unicode_3ser_NDARRAY_UINT_IDDT, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_FLOAT_IDDT", (void **)&__pyx_vp_10serializor_7unicode_3ser_NDARRAY_FLOAT_IDDT, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_BOOL_IDDT", (void **)&__pyx_vp_10serializor_7unicode_3ser_NDARRAY_BOOL_IDDT, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_DT64_IDDT", (void **)&__pyx_vp_10serializor_7unicode_3ser_NDARRAY_DT64_IDDT, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_TD64_IDDT", (void **)&__pyx_vp_10serializor_7unicode_3ser_NDARRAY_TD64_IDDT, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_COMPLEX_IDDT", (void **)&__pyx_vp_10serializor_7unicode_3ser_NDARRAY_COMPLEX_IDDT, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_BYTES_IDDT", (void **)&__pyx_vp_10serializor_7unicode_3ser_NDARRAY_BYTES_IDDT, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_UNICODE_IDDT", (void **)&__pyx_vp_10serializor_7unicode_3ser_NDARRAY_UNICODE_IDDT, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "PD_TIMESTAMP_ID", (void **)&__pyx_vp_10serializor_7unicode_3ser_PD_TIMESTAMP_ID, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "PD_TIMEDELTA_ID", (void **)&__pyx_vp_10serializor_7unicode_3ser_PD_TIMEDELTA_ID, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "SERIES_ID", (void **)&__pyx_vp_10serializor_7unicode_3ser_SERIES_ID, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "DATETIMEINDEX_ID", (void **)&__pyx_vp_10serializor_7unicode_3ser_DATETIMEINDEX_ID, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "TIMEDELTAINDEX_ID", (void **)&__pyx_vp_10serializor_7unicode_3ser_TIMEDELTAINDEX_ID, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "DATAFRAME_ID", (void **)&__pyx_vp_10serializor_7unicode_3ser_DATAFRAME_ID, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "TRUE_VALUE", (void **)&__pyx_vp_10serializor_7unicode_3ser_TRUE_VALUE, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "FALSE_VALUE", (void **)&__pyx_vp_10serializor_7unicode_3ser_FALSE_VALUE, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NONE_VALUE", (void **)&__pyx_vp_10serializor_7unicode_3ser_NONE_VALUE, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("serializor.identifier"); if (!__pyx_t_1) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "STR", (void **)&__pyx_vp_10serializor_10identifier_STR, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "BOOL", (void **)&__pyx_vp_10serializor_10identifier_BOOL, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "FLOAT", (void **)&__pyx_vp_10serializor_10identifier_FLOAT, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "INT", (void **)&__pyx_vp_10serializor_10identifier_INT, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NONE", (void **)&__pyx_vp_10serializor_10identifier_NONE, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "DATE", (void **)&__pyx_vp_10serializor_10identifier_DATE, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "TIME", (void **)&__pyx_vp_10serializor_10identifier_TIME, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "DATETIME", (void **)&__pyx_vp_10serializor_10identifier_DATETIME, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "TIMEDELTA", (void **)&__pyx_vp_10serializor_10identifier_TIMEDELTA, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "STRUCT_TIME", (void **)&__pyx_vp_10serializor_10identifier_STRUCT_TIME, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "DECIMAL", (void **)&__pyx_vp_10serializor_10identifier_DECIMAL, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "COMPLEX", (void **)&__pyx_vp_10serializor_10identifier_COMPLEX, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "BYTES", (void **)&__pyx_vp_10serializor_10identifier_BYTES, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "BYTEARRAY", (void **)&__pyx_vp_10serializor_10identifier_BYTEARRAY, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "LIST", (void **)&__pyx_vp_10serializor_10identifier_LIST, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "TUPLE", (void **)&__pyx_vp_10serializor_10identifier_TUPLE, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "SET", (void **)&__pyx_vp_10serializor_10identifier_SET, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "FROZENSET", (void **)&__pyx_vp_10serializor_10identifier_FROZENSET, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "RANGE", (void **)&__pyx_vp_10serializor_10identifier_RANGE, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "DICT", (void **)&__pyx_vp_10serializor_10identifier_DICT, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "DATETIME64", (void **)&__pyx_vp_10serializor_10identifier_DATETIME64, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "TIMEDELTA64", (void **)&__pyx_vp_10serializor_10identifier_TIMEDELTA64, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY", (void **)&__pyx_vp_10serializor_10identifier_NDARRAY, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_OBJECT", (void **)&__pyx_vp_10serializor_10identifier_NDARRAY_OBJECT, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_INT", (void **)&__pyx_vp_10serializor_10identifier_NDARRAY_INT, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_UINT", (void **)&__pyx_vp_10serializor_10identifier_NDARRAY_UINT, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_FLOAT", (void **)&__pyx_vp_10serializor_10identifier_NDARRAY_FLOAT, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_BOOL", (void **)&__pyx_vp_10serializor_10identifier_NDARRAY_BOOL, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_DT64", (void **)&__pyx_vp_10serializor_10identifier_NDARRAY_DT64, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_TD64", (void **)&__pyx_vp_10serializor_10identifier_NDARRAY_TD64, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_COMPLEX", (void **)&__pyx_vp_10serializor_10identifier_NDARRAY_COMPLEX, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_BYTES", (void **)&__pyx_vp_10serializor_10identifier_NDARRAY_BYTES, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NDARRAY_UNICODE", (void **)&__pyx_vp_10serializor_10identifier_NDARRAY_UNICODE, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "SERIES", (void **)&__pyx_vp_10serializor_10identifier_SERIES, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "DATAFRAME", (void **)&__pyx_vp_10serializor_10identifier_DATAFRAME, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "PD_TIMESTAMP", (void **)&__pyx_vp_10serializor_10identifier_PD_TIMESTAMP, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "PD_TIMEDELTA", (void **)&__pyx_vp_10serializor_10identifier_PD_TIMEDELTA, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "DATETIMEINDEX", (void **)&__pyx_vp_10serializor_10identifier_DATETIMEINDEX, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "TIMEDELTAINDEX", (void **)&__pyx_vp_10serializor_10identifier_TIMEDELTAINDEX, "char") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "_duplicates_test", (void **)&__pyx_vp_10serializor_10identifier__duplicates_test, "int") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("serializor.typeref"); if (!__pyx_t_1) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "NONE", (void **)&__pyx_vp_10serializor_7typeref_NONE, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "DICT_KEYS", (void **)&__pyx_vp_10serializor_7typeref_DICT_KEYS, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "DICT_VALUES", (void **)&__pyx_vp_10serializor_7typeref_DICT_VALUES, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "DICT_ITEMS", (void **)&__pyx_vp_10serializor_7typeref_DICT_ITEMS, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "DECIMAL", (void **)&__pyx_vp_10serializor_7typeref_DECIMAL, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "STRUCT_TIME", (void **)&__pyx_vp_10serializor_7typeref_STRUCT_TIME, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "DEQUE", (void **)&__pyx_vp_10serializor_7typeref_DEQUE, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "FLOAT16", (void **)&__pyx_vp_10serializor_7typeref_FLOAT16, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "FLOAT32", (void **)&__pyx_vp_10serializor_7typeref_FLOAT32, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "FLOAT64", (void **)&__pyx_vp_10serializor_7typeref_FLOAT64, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "INT8", (void **)&__pyx_vp_10serializor_7typeref_INT8, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "INT16", (void **)&__pyx_vp_10serializor_7typeref_INT16, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "INT32", (void **)&__pyx_vp_10serializor_7typeref_INT32, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "INT64", (void **)&__pyx_vp_10serializor_7typeref_INT64, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "UINT8", (void **)&__pyx_vp_10serializor_7typeref_UINT8, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "UINT16", (void **)&__pyx_vp_10serializor_7typeref_UINT16, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "UINT32", (void **)&__pyx_vp_10serializor_7typeref_UINT32, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "UINT64", (void **)&__pyx_vp_10serializor_7typeref_UINT64, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "COMPLEX64", (void **)&__pyx_vp_10serializor_7typeref_COMPLEX64, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "COMPLEX128", (void **)&__pyx_vp_10serializor_7typeref_COMPLEX128, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "DATETIME64", (void **)&__pyx_vp_10serializor_7typeref_DATETIME64, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "TIMEDELTA64", (void **)&__pyx_vp_10serializor_7typeref_TIMEDELTA64, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "STR_", (void **)&__pyx_vp_10serializor_7typeref_STR_, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "BOOL_", (void **)&__pyx_vp_10serializor_7typeref_BOOL_, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "BYTES_", (void **)&__pyx_vp_10serializor_7typeref_BYTES_, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "RECORD", (void **)&__pyx_vp_10serializor_7typeref_RECORD, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "SERIES", (void **)&__pyx_vp_10serializor_7typeref_SERIES, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "DATAFRAME", (void **)&__pyx_vp_10serializor_7typeref_DATAFRAME, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "PD_TIMESTAMP", (void **)&__pyx_vp_10serializor_7typeref_PD_TIMESTAMP, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "PD_TIMEDELTA", (void **)&__pyx_vp_10serializor_7typeref_PD_TIMEDELTA, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "DATETIMEINDEX", (void **)&__pyx_vp_10serializor_7typeref_DATETIMEINDEX, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "TIMEDELTAINDEX", (void **)&__pyx_vp_10serializor_7typeref_TIMEDELTAINDEX, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "CYTIMES_AVAILABLE", (void **)&__pyx_vp_10serializor_7typeref_CYTIMES_AVAILABLE, "int") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "PYDT", (void **)&__pyx_vp_10serializor_7typeref_PYDT, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "PDDT", (void **)&__pyx_vp_10serializor_7typeref_PDDT, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("serializor.utils"); if (!__pyx_t_1) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr_3_0_11(__pyx_t_1, "AVAILABLE_TIMEZONES", (void **)&__pyx_vp_10serializor_5utils_AVAILABLE_TIMEZONES, "PyObject *") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_des(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_des},
  {0, NULL}
};
#endif

#ifdef __cplusplus
namespace {
  struct PyModuleDef __pyx_moduledef =
  #else
  static struct PyModuleDef __pyx_moduledef =
  #endif
  {
      PyModuleDef_HEAD_INIT,
      "des",
      0, /* m_doc */
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      0, /* m_size */
    #elif CYTHON_USE_MODULE_STATE
      sizeof(__pyx_mstate), /* m_size */
    #else
      -1, /* m_size */
    #endif
      __pyx_methods /* m_methods */,
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      __pyx_moduledef_slots, /* m_slots */
    #else
      NULL, /* m_reload */
    #endif
    #if CYTHON_USE_MODULE_STATE
      __pyx_m_traverse, /* m_traverse */
      __pyx_m_clear, /* m_clear */
      NULL /* m_free */
    #else
      NULL, /* m_traverse */
      NULL, /* m_clear */
      NULL /* m_free */
    #endif
  };
  #ifdef __cplusplus
} /* anonymous namespace */
#endif
#endif

#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initdes(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initdes(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_des(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_des(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *module, const char* from_name, const char* to_name, int allow_none)
#else
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none)
#endif
{
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
#if CYTHON_COMPILING_IN_LIMITED_API
            result = PyModule_AddObject(module, to_name, value);
#else
            result = PyDict_SetItemString(moddict, to_name, value);
#endif
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    CYTHON_UNUSED_VAR(def);
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
#if CYTHON_COMPILING_IN_LIMITED_API
    moddict = module;
#else
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
#endif
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_des(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  int stringtab_initialized = 0;
  #if CYTHON_USE_MODULE_STATE
  int pystate_addmodule_run = 0;
  #endif
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'des' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("des", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  if (unlikely(!__pyx_m)) __PYX_ERR(1, 1, __pyx_L1_error)
  #elif CYTHON_USE_MODULE_STATE
  __pyx_t_1 = PyModule_Create(&__pyx_moduledef); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1, __pyx_L1_error)
  {
    int add_module_result = PyState_AddModule(__pyx_t_1, &__pyx_moduledef);
    __pyx_t_1 = 0; /* transfer ownership from __pyx_t_1 to "des" pseudovariable */
    if (unlikely((add_module_result < 0))) __PYX_ERR(1, 1, __pyx_L1_error)
    pystate_addmodule_run = 1;
  }
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  if (unlikely(!__pyx_m)) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #endif
  CYTHON_UNUSED_VAR(__pyx_t_1);
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(1, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = __Pyx_PyImport_AddModuleRef(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_cython_runtime = __Pyx_PyImport_AddModuleRef((const char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(1, 1, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_des(void)", 0);
  if (__Pyx_check_binary_version(__PYX_LIMITED_VERSION_HEX, __Pyx_get_runtime_version(), CYTHON_COMPILING_IN_LIMITED_API) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(1, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init(__pyx_m) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init(__pyx_m) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init(__pyx_m) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init(__pyx_m) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init(__pyx_m) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init(__pyx_m) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitConstants() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  stringtab_initialized = 1;
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_serializor__unicode__des) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name_2, __pyx_n_s_main) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(1, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "serializor.unicode.des")) {
      if (unlikely((PyDict_SetItemString(modules, "serializor.unicode.des", __pyx_m) < 0))) __PYX_ERR(1, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  if (unlikely((__Pyx_modinit_function_export_code() < 0))) __PYX_ERR(1, 1, __pyx_L1_error)
  (void)__Pyx_modinit_type_init_code();
  if (unlikely((__Pyx_modinit_type_import_code() < 0))) __PYX_ERR(1, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_variable_import_code() < 0))) __PYX_ERR(1, 1, __pyx_L1_error)
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif

  /* "serializor/unicode/des.py":15
 * from cython.cimports.serializor import identifier, typeref, utils  # type: ignore
 * 
 * np.import_array()             # <<<<<<<<<<<<<<
 * np.import_umath()
 * datetime.import_datetime()
 */
  __pyx_t_2 = __pyx_f_5numpy_import_array(); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 15, __pyx_L1_error)

  /* "serializor/unicode/des.py":16
 * 
 * np.import_array()
 * np.import_umath()             # <<<<<<<<<<<<<<
 * datetime.import_datetime()
 * 
 */
  __pyx_t_2 = __pyx_f_5numpy_import_umath(); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 16, __pyx_L1_error)

  /* "serializor/unicode/des.py":17
 * np.import_array()
 * np.import_umath()
 * datetime.import_datetime()             # <<<<<<<<<<<<<<
 * 
 * # Python imports
 */
  __pyx_f_7cpython_8datetime_import_datetime();

  /* "serializor/unicode/des.py":20
 * 
 * # Python imports
 * import numpy as np, datetime             # <<<<<<<<<<<<<<
 * from zoneinfo import ZoneInfo
 * from orjson import loads as _loads
 */
  __pyx_t_3 = __Pyx_ImportDottedModule(__pyx_n_s_numpy, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_np, __pyx_t_3) < 0) __PYX_ERR(1, 20, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_ImportDottedModule(__pyx_n_s_datetime, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_datetime, __pyx_t_3) < 0) __PYX_ERR(1, 20, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "serializor/unicode/des.py":21
 * # Python imports
 * import numpy as np, datetime
 * from zoneinfo import ZoneInfo             # <<<<<<<<<<<<<<
 * from orjson import loads as _loads
 * from serializor import identifier, typeref, utils, errors
 */
  __pyx_t_3 = PyList_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_n_s_ZoneInfo);
  __Pyx_GIVEREF(__pyx_n_s_ZoneInfo);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 0, __pyx_n_s_ZoneInfo)) __PYX_ERR(1, 21, __pyx_L1_error);
  __pyx_t_4 = __Pyx_Import(__pyx_n_s_zoneinfo, __pyx_t_3, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_ImportFrom(__pyx_t_4, __pyx_n_s_ZoneInfo); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ZoneInfo, __pyx_t_3) < 0) __PYX_ERR(1, 21, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "serializor/unicode/des.py":22
 * import numpy as np, datetime
 * from zoneinfo import ZoneInfo
 * from orjson import loads as _loads             # <<<<<<<<<<<<<<
 * from serializor import identifier, typeref, utils, errors
 * 
 */
  __pyx_t_4 = PyList_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_n_s_loads_2);
  __Pyx_GIVEREF(__pyx_n_s_loads_2);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 0, __pyx_n_s_loads_2)) __PYX_ERR(1, 22, __pyx_L1_error);
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_orjson, __pyx_t_4, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_loads_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_loads, __pyx_t_4) < 0) __PYX_ERR(1, 22, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "serializor/unicode/des.py":23
 * from zoneinfo import ZoneInfo
 * from orjson import loads as _loads
 * from serializor import identifier, typeref, utils, errors             # <<<<<<<<<<<<<<
 * 
 * __all__ = ["deserialize"]
 */
  __pyx_t_3 = PyList_New(4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_n_s_identifier);
  __Pyx_GIVEREF(__pyx_n_s_identifier);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 0, __pyx_n_s_identifier)) __PYX_ERR(1, 23, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_s_typeref);
  __Pyx_GIVEREF(__pyx_n_s_typeref);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 1, __pyx_n_s_typeref)) __PYX_ERR(1, 23, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_s_utils);
  __Pyx_GIVEREF(__pyx_n_s_utils);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 2, __pyx_n_s_utils)) __PYX_ERR(1, 23, __pyx_L1_error);
  __Pyx_INCREF(__pyx_n_s_errors);
  __Pyx_GIVEREF(__pyx_n_s_errors);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 3, __pyx_n_s_errors)) __PYX_ERR(1, 23, __pyx_L1_error);
  __pyx_t_4 = __Pyx_Import(__pyx_n_s_serializor, __pyx_t_3, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_ImportFrom(__pyx_t_4, __pyx_n_s_identifier); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_identifier, __pyx_t_3) < 0) __PYX_ERR(1, 23, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_ImportFrom(__pyx_t_4, __pyx_n_s_typeref); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_typeref, __pyx_t_3) < 0) __PYX_ERR(1, 23, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_ImportFrom(__pyx_t_4, __pyx_n_s_utils); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_utils, __pyx_t_3) < 0) __PYX_ERR(1, 23, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_ImportFrom(__pyx_t_4, __pyx_n_s_errors); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_errors, __pyx_t_3) < 0) __PYX_ERR(1, 23, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "serializor/unicode/des.py":25
 * from serializor import identifier, typeref, utils, errors
 * 
 * __all__ = ["deserialize"]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_4 = PyList_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_n_u_deserialize);
  __Pyx_GIVEREF(__pyx_n_u_deserialize);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 0, __pyx_n_u_deserialize)) __PYX_ERR(1, 25, __pyx_L1_error);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_all, __pyx_t_4) < 0) __PYX_ERR(1, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "serializor/unicode/des.py":1430
 * 
 * 
 * @cython.ccall             # <<<<<<<<<<<<<<
 * def deserialize(data: str) -> object:
 *     """Deserialize the data (str) back to an `<'object'>`."""
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_data, __pyx_n_s_str) < 0) __PYX_ERR(1, 1430, __pyx_L1_error)
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_10serializor_7unicode_3des_1deserialize, 0, __pyx_n_s_deserialize, NULL, __pyx_n_s_serializor_unicode_des, __pyx_d, ((PyObject *)__pyx_codeobj__32)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_3, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_deserialize, __pyx_t_3) < 0) __PYX_ERR(1, 1430, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "serializor/unicode/des.py":1450
 * 
 * ########## The following functions are for testing purpose only ##########
 * def _test_utils() -> None:             # <<<<<<<<<<<<<<
 *     _test_unpack_int8()
 *     _test_unpack_uint8()
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_return, __pyx_n_s_None) < 0) __PYX_ERR(1, 1450, __pyx_L1_error)
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_10serializor_7unicode_3des_3_test_utils, 0, __pyx_n_s_test_utils, NULL, __pyx_n_s_serializor_unicode_des, __pyx_d, ((PyObject *)__pyx_codeobj__33)); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_4, __pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test_utils, __pyx_t_4) < 0) __PYX_ERR(1, 1450, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "serializor/unicode/des.py":1464
 * 
 * 
 * def _test_unpack_int8() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1464, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_return, __pyx_n_s_None) < 0) __PYX_ERR(1, 1464, __pyx_L1_error)
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_10serializor_7unicode_3des_5_test_unpack_int8, 0, __pyx_n_s_test_unpack_int8, NULL, __pyx_n_s_serializor_unicode_des, __pyx_d, ((PyObject *)__pyx_codeobj__35)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1464, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_3, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test_unpack_int8, __pyx_t_3) < 0) __PYX_ERR(1, 1464, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "serializor/unicode/des.py":1477
 * 
 * 
 * def _test_unpack_uint8() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1477, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_return, __pyx_n_s_None) < 0) __PYX_ERR(1, 1477, __pyx_L1_error)
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_10serializor_7unicode_3des_7_test_unpack_uint8, 0, __pyx_n_s_test_unpack_uint8, NULL, __pyx_n_s_serializor_unicode_des, __pyx_d, ((PyObject *)__pyx_codeobj__36)); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1477, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_4, __pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test_unpack_uint8, __pyx_t_4) < 0) __PYX_ERR(1, 1477, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "serializor/unicode/des.py":1490
 * 
 * 
 * def _test_unpack_int16() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1490, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_return, __pyx_n_s_None) < 0) __PYX_ERR(1, 1490, __pyx_L1_error)
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_10serializor_7unicode_3des_9_test_unpack_int16, 0, __pyx_n_s_test_unpack_int16, NULL, __pyx_n_s_serializor_unicode_des, __pyx_d, ((PyObject *)__pyx_codeobj__37)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1490, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_3, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test_unpack_int16, __pyx_t_3) < 0) __PYX_ERR(1, 1490, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "serializor/unicode/des.py":1503
 * 
 * 
 * def _test_unpack_uint16() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1503, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_return, __pyx_n_s_None) < 0) __PYX_ERR(1, 1503, __pyx_L1_error)
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_10serializor_7unicode_3des_11_test_unpack_uint16, 0, __pyx_n_s_test_unpack_uint16, NULL, __pyx_n_s_serializor_unicode_des, __pyx_d, ((PyObject *)__pyx_codeobj__38)); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1503, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_4, __pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test_unpack_uint16, __pyx_t_4) < 0) __PYX_ERR(1, 1503, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "serializor/unicode/des.py":1516
 * 
 * 
 * def _test_unpack_int24() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1516, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_return, __pyx_n_s_None) < 0) __PYX_ERR(1, 1516, __pyx_L1_error)
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_10serializor_7unicode_3des_13_test_unpack_int24, 0, __pyx_n_s_test_unpack_int24, NULL, __pyx_n_s_serializor_unicode_des, __pyx_d, ((PyObject *)__pyx_codeobj__39)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1516, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_3, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test_unpack_int24, __pyx_t_3) < 0) __PYX_ERR(1, 1516, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "serializor/unicode/des.py":1529
 * 
 * 
 * def _test_unpack_uint24() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_return, __pyx_n_s_None) < 0) __PYX_ERR(1, 1529, __pyx_L1_error)
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_10serializor_7unicode_3des_15_test_unpack_uint24, 0, __pyx_n_s_test_unpack_uint24, NULL, __pyx_n_s_serializor_unicode_des, __pyx_d, ((PyObject *)__pyx_codeobj__40)); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_4, __pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test_unpack_uint24, __pyx_t_4) < 0) __PYX_ERR(1, 1529, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "serializor/unicode/des.py":1542
 * 
 * 
 * def _test_unpack_int32() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1542, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_return, __pyx_n_s_None) < 0) __PYX_ERR(1, 1542, __pyx_L1_error)
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_10serializor_7unicode_3des_17_test_unpack_int32, 0, __pyx_n_s_test_unpack_int32, NULL, __pyx_n_s_serializor_unicode_des, __pyx_d, ((PyObject *)__pyx_codeobj__41)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1542, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_3, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test_unpack_int32, __pyx_t_3) < 0) __PYX_ERR(1, 1542, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "serializor/unicode/des.py":1555
 * 
 * 
 * def _test_unpack_uint32() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1555, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_return, __pyx_n_s_None) < 0) __PYX_ERR(1, 1555, __pyx_L1_error)
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_10serializor_7unicode_3des_19_test_unpack_uint32, 0, __pyx_n_s_test_unpack_uint32, NULL, __pyx_n_s_serializor_unicode_des, __pyx_d, ((PyObject *)__pyx_codeobj__42)); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1555, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_4, __pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test_unpack_uint32, __pyx_t_4) < 0) __PYX_ERR(1, 1555, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "serializor/unicode/des.py":1568
 * 
 * 
 * def _test_unpack_int64() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_return, __pyx_n_s_None) < 0) __PYX_ERR(1, 1568, __pyx_L1_error)
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_10serializor_7unicode_3des_21_test_unpack_int64, 0, __pyx_n_s_test_unpack_int64, NULL, __pyx_n_s_serializor_unicode_des, __pyx_d, ((PyObject *)__pyx_codeobj__43)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_3, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test_unpack_int64, __pyx_t_3) < 0) __PYX_ERR(1, 1568, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "serializor/unicode/des.py":1587
 * 
 * 
 * def _test_unpack_uint64() -> None:             # <<<<<<<<<<<<<<
 *     import struct
 * 
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1587, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_return, __pyx_n_s_None) < 0) __PYX_ERR(1, 1587, __pyx_L1_error)
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_10serializor_7unicode_3des_23_test_unpack_uint64, 0, __pyx_n_s_test_unpack_uint64, NULL, __pyx_n_s_serializor_unicode_des, __pyx_d, ((PyObject *)__pyx_codeobj__44)); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1587, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_4, __pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test_unpack_uint64, __pyx_t_4) < 0) __PYX_ERR(1, 1587, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "serializor/unicode/des.py":1606
 * 
 * 
 * def _test_dec_encoded_int() -> None:             # <<<<<<<<<<<<<<
 *     for enc, num in [
 *         ("", 251),
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1606, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_return, __pyx_n_s_None) < 0) __PYX_ERR(1, 1606, __pyx_L1_error)
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_10serializor_7unicode_3des_25_test_dec_encoded_int, 0, __pyx_n_s_test_dec_encoded_int, NULL, __pyx_n_s_serializor_unicode_des, __pyx_d, ((PyObject *)__pyx_codeobj__46)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1606, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_3, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test_dec_encoded_int, __pyx_t_3) < 0) __PYX_ERR(1, 1606, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "serializor/unicode/des.py":1
 * # cython: language_level=3             # <<<<<<<<<<<<<<
 * # cython: wraparound=False
 * # cython: boundscheck=False
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_3) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  if (__pyx_m) {
    if (__pyx_d && stringtab_initialized) {
      __Pyx_AddTraceback("init serializor.unicode.des", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    #if !CYTHON_USE_MODULE_STATE
    Py_CLEAR(__pyx_m);
    #else
    Py_DECREF(__pyx_m);
    if (pystate_addmodule_run) {
      PyObject *tp, *value, *tb;
      PyErr_Fetch(&tp, &value, &tb);
      PyState_RemoveModule(&__pyx_moduledef);
      PyErr_Restore(tp, value, tb);
    }
    #endif
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init serializor.unicode.des");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}
/* #### Code section: cleanup_globals ### */
/* #### Code section: cleanup_module ### */
/* #### Code section: main_method ### */
/* #### Code section: utility_code_pragmas ### */
#ifdef _MSC_VER
#pragma warning( push )
/* Warning 4127: conditional expression is constant
 * Cython uses constant conditional expressions to allow in inline functions to be optimized at
 * compile-time, so this warning is not useful
 */
#pragma warning( disable : 4127 )
#endif



/* #### Code section: utility_code_def ### */

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    int result;
    PyObject *exc_type;
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *current_exception = tstate->current_exception;
    if (unlikely(!current_exception)) return 0;
    exc_type = (PyObject*) Py_TYPE(current_exception);
    if (exc_type == err) return 1;
#else
    exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
#endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(exc_type);
    #endif
    if (unlikely(PyTuple_Check(err))) {
        result = __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    } else {
        result = __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
    }
    #if CYTHON_AVOID_BORROWED_REFS
    Py_DECREF(exc_type);
    #endif
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *tmp_value;
    assert(type == NULL || (value != NULL && type == (PyObject*) Py_TYPE(value)));
    if (value) {
        #if CYTHON_COMPILING_IN_CPYTHON
        if (unlikely(((PyBaseExceptionObject*) value)->traceback != tb))
        #endif
            PyException_SetTraceback(value, tb);
    }
    tmp_value = tstate->current_exception;
    tstate->current_exception = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
#else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject* exc_value;
    exc_value = tstate->current_exception;
    tstate->current_exception = 0;
    *value = exc_value;
    *type = NULL;
    *tb = NULL;
    if (exc_value) {
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        #if CYTHON_COMPILING_IN_CPYTHON
        *tb = ((PyBaseExceptionObject*) exc_value)->traceback;
        Py_XINCREF(*tb);
        #else
        *tb = PyException_GetTraceback(exc_value);
        #endif
    }
#else
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#endif
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* PyObjectGetAttrStrNoError */
#if __PYX_LIMITED_VERSION_HEX < 0x030d00A1
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d00A1
    (void) PyObject_GetOptionalAttr(obj, attr_name, &result);
    return result;
#else
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
#endif
}

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStrNoError(__pyx_b, name);
    if (unlikely(!result) && !PyErr_Occurred()) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* decode_c_string */
static CYTHON_INLINE PyObject* __Pyx_decode_c_string(
         const char* cstring, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    Py_ssize_t length;
    if (unlikely((start < 0) | (stop < 0))) {
        size_t slen = strlen(cstring);
        if (unlikely(slen > (size_t) PY_SSIZE_T_MAX)) {
            PyErr_SetString(PyExc_OverflowError,
                            "c-string too long to convert to Python");
            return NULL;
        }
        length = (Py_ssize_t) slen;
        if (start < 0) {
            start += length;
            if (start < 0)
                start = 0;
        }
        if (stop < 0)
            stop += length;
    }
    if (unlikely(stop <= start))
        return __Pyx_NewRef(__pyx_empty_unicode);
    length = stop - start;
    cstring += start;
    if (decode_func) {
        return decode_func(cstring, length, errors);
    } else {
        return PyUnicode_Decode(cstring, length, encoding, errors);
    }
}

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL && !CYTHON_VECTORCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    #if PY_MAJOR_VERSION < 3
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object"))) {
        return NULL;
    }
    #else
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object"))) {
        return NULL;
    }
    #endif
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    #if PY_MAJOR_VERSION < 3
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    #else
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    #endif
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = __Pyx_CyOrPyCFunction_GET_FUNCTION(func);
    self = __Pyx_CyOrPyCFunction_GET_SELF(func);
    #if PY_MAJOR_VERSION < 3
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    #else
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    #endif
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectFastCall */
#if PY_VERSION_HEX < 0x03090000 || CYTHON_COMPILING_IN_LIMITED_API
static PyObject* __Pyx_PyObject_FastCall_fallback(PyObject *func, PyObject **args, size_t nargs, PyObject *kwargs) {
    PyObject *argstuple;
    PyObject *result = 0;
    size_t i;
    argstuple = PyTuple_New((Py_ssize_t)nargs);
    if (unlikely(!argstuple)) return NULL;
    for (i = 0; i < nargs; i++) {
        Py_INCREF(args[i]);
        if (__Pyx_PyTuple_SET_ITEM(argstuple, (Py_ssize_t)i, args[i]) < 0) goto bad;
    }
    result = __Pyx_PyObject_Call(func, argstuple, kwargs);
  bad:
    Py_DECREF(argstuple);
    return result;
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject **args, size_t _nargs, PyObject *kwargs) {
    Py_ssize_t nargs = __Pyx_PyVectorcall_NARGS(_nargs);
#if CYTHON_COMPILING_IN_CPYTHON
    if (nargs == 0 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_NOARGS))
            return __Pyx_PyObject_CallMethO(func, NULL);
    }
    else if (nargs == 1 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_O))
            return __Pyx_PyObject_CallMethO(func, args[0]);
    }
#endif
    #if PY_VERSION_HEX < 0x030800B1
    #if CYTHON_FAST_PYCCALL
    if (PyCFunction_Check(func)) {
        if (kwargs) {
            return _PyCFunction_FastCallDict(func, args, nargs, kwargs);
        } else {
            return _PyCFunction_FastCallKeywords(func, args, nargs, NULL);
        }
    }
    #if PY_VERSION_HEX >= 0x030700A1
    if (!kwargs && __Pyx_IS_TYPE(func, &PyMethodDescr_Type)) {
        return _PyMethodDescr_FastCallKeywords(func, args, nargs, NULL);
    }
    #endif
    #endif
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs);
    }
    #endif
    #endif
    if (kwargs == NULL) {
        #if CYTHON_VECTORCALL
        #if PY_VERSION_HEX < 0x03090000
        vectorcallfunc f = _PyVectorcall_Function(func);
        #else
        vectorcallfunc f = PyVectorcall_Function(func);
        #endif
        if (f) {
            return f(func, args, (size_t)nargs, NULL);
        }
        #elif defined(__Pyx_CyFunction_USED) && CYTHON_BACKPORT_VECTORCALL
        if (__Pyx_CyFunction_CheckExact(func)) {
            __pyx_vectorcallfunc f = __Pyx_CyFunction_func_vectorcall(func);
            if (f) return f(func, args, (size_t)nargs, NULL);
        }
        #endif
    }
    if (nargs == 0) {
        return __Pyx_PyObject_Call(func, __pyx_empty_tuple, kwargs);
    }
    #if PY_VERSION_HEX >= 0x03090000 && !CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_VectorcallDict(func, args, (size_t)nargs, kwargs);
    #else
    return __Pyx_PyObject_FastCall_fallback(func, args, (size_t)nargs, kwargs);
    #endif
}

/* PyObjectCallOneArg */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *args[2] = {NULL, arg};
    return __Pyx_PyObject_FastCall(func, args+1, 1 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    __Pyx_PyThreadState_declare
    CYTHON_UNUSED_VAR(cause);
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
      #if PY_VERSION_HEX >= 0x030C00A6
        PyException_SetTraceback(value, tb);
      #elif CYTHON_FAST_THREAD_STATE
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#else
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_value == NULL || exc_info->exc_value == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    PyObject *exc_value = exc_info->exc_value;
    if (exc_value == NULL || exc_value == Py_None) {
        *value = NULL;
        *type = NULL;
        *tb = NULL;
    } else {
        *value = exc_value;
        Py_INCREF(*value);
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        *tb = PyException_GetTraceback(exc_value);
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #endif
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    PyObject *tmp_value = exc_info->exc_value;
    exc_info->exc_value = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
  #else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
  #endif
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type = NULL, *local_value, *local_tb = NULL;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if PY_VERSION_HEX >= 0x030C00A6
    local_value = tstate->current_exception;
    tstate->current_exception = 0;
    if (likely(local_value)) {
        local_type = (PyObject*) Py_TYPE(local_value);
        Py_INCREF(local_type);
        local_tb = PyException_GetTraceback(local_value);
    }
  #else
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
  #endif
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE && PY_VERSION_HEX >= 0x030C00A6
    if (unlikely(tstate->current_exception))
#elif CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
      #if PY_VERSION_HEX >= 0x030B00a4
        tmp_value = exc_info->exc_value;
        exc_info->exc_value = local_value;
        tmp_type = NULL;
        tmp_tb = NULL;
        Py_XDECREF(local_type);
        Py_XDECREF(local_tb);
      #else
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
      #endif
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    __Pyx_TypeName obj_type_name;
    __Pyx_TypeName type_name;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    obj_type_name = __Pyx_PyType_GetName(Py_TYPE(obj));
    type_name = __Pyx_PyType_GetName(type);
    PyErr_Format(PyExc_TypeError,
                 "Cannot convert " __Pyx_FMT_TYPENAME " to " __Pyx_FMT_TYPENAME,
                 obj_type_name, type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    __Pyx_DECREF_TypeName(type_name);
    return 0;
}

/* RaiseUnexpectedTypeError */
static int
__Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj)
{
    __Pyx_TypeName obj_type_name = __Pyx_PyType_GetName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError, "Expected %s, got " __Pyx_FMT_TYPENAME,
                 expected, obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* BytesEquals */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS && (PY_VERSION_HEX < 0x030B0000)
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && PY_VERSION_HEX < 0x030d0000
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#elif CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(!__pyx_m)) {
        return NULL;
    }
    result = PyObject_GetAttr(__pyx_m, name);
    if (likely(result)) {
        return result;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyUnicode_Substring */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Substring(
            PyObject* text, Py_ssize_t start, Py_ssize_t stop) {
    Py_ssize_t length;
    if (unlikely(__Pyx_PyUnicode_READY(text) == -1)) return NULL;
    length = __Pyx_PyUnicode_GET_LENGTH(text);
    if (start < 0) {
        start += length;
        if (start < 0)
            start = 0;
    }
    if (stop < 0)
        stop += length;
    else if (stop > length)
        stop = length;
    if (stop <= start)
        return __Pyx_NewRef(__pyx_empty_unicode);
    if (start == 0 && stop == length)
        return __Pyx_NewRef(text);
#if CYTHON_PEP393_ENABLED
    return PyUnicode_FromKindAndData(PyUnicode_KIND(text),
        PyUnicode_1BYTE_DATA(text) + start*PyUnicode_KIND(text), stop-start);
#else
    return PyUnicode_FromUnicode(PyUnicode_AS_UNICODE(text)+start, stop-start);
#endif
}

/* pybytes_as_double */
static double __Pyx_SlowPyString_AsDouble(PyObject *obj) {
    PyObject *float_value;
#if PY_MAJOR_VERSION >= 3
    float_value = PyFloat_FromString(obj);
#else
    float_value = PyFloat_FromString(obj, 0);
#endif
    if (likely(float_value)) {
#if CYTHON_ASSUME_SAFE_MACROS
        double value = PyFloat_AS_DOUBLE(float_value);
#else
        double value = PyFloat_AsDouble(float_value);
#endif
        Py_DECREF(float_value);
        return value;
    }
    return (double)-1;
}
static const char* __Pyx__PyBytes_AsDouble_Copy(const char* start, char* buffer, Py_ssize_t length) {
    int last_was_punctuation = 1;
    Py_ssize_t i;
    for (i=0; i < length; i++) {
        char chr = start[i];
        int is_punctuation = (chr == '_') | (chr == '.') | (chr == 'e') | (chr == 'E');
        *buffer = chr;
        buffer += (chr != '_');
        if (unlikely(last_was_punctuation & is_punctuation)) goto parse_failure;
        last_was_punctuation = is_punctuation;
    }
    if (unlikely(last_was_punctuation)) goto parse_failure;
    *buffer = '\0';
    return buffer;
parse_failure:
    return NULL;
}
static double __Pyx__PyBytes_AsDouble_inf_nan(const char* start, Py_ssize_t length) {
    int matches = 1;
    char sign = start[0];
    int is_signed = (sign == '+') | (sign == '-');
    start += is_signed;
    length -= is_signed;
    switch (start[0]) {
        #ifdef Py_NAN
        case 'n':
        case 'N':
            if (unlikely(length != 3)) goto parse_failure;
            matches &= (start[1] == 'a' || start[1] == 'A');
            matches &= (start[2] == 'n' || start[2] == 'N');
            if (unlikely(!matches)) goto parse_failure;
            return (sign == '-') ? -Py_NAN : Py_NAN;
        #endif
        case 'i':
        case 'I':
            if (unlikely(length < 3)) goto parse_failure;
            matches &= (start[1] == 'n' || start[1] == 'N');
            matches &= (start[2] == 'f' || start[2] == 'F');
            if (likely(length == 3 && matches))
                return (sign == '-') ? -Py_HUGE_VAL : Py_HUGE_VAL;
            if (unlikely(length != 8)) goto parse_failure;
            matches &= (start[3] == 'i' || start[3] == 'I');
            matches &= (start[4] == 'n' || start[4] == 'N');
            matches &= (start[5] == 'i' || start[5] == 'I');
            matches &= (start[6] == 't' || start[6] == 'T');
            matches &= (start[7] == 'y' || start[7] == 'Y');
            if (unlikely(!matches)) goto parse_failure;
            return (sign == '-') ? -Py_HUGE_VAL : Py_HUGE_VAL;
        case '.': case '0': case '1': case '2': case '3': case '4': case '5': case '6': case '7': case '8': case '9':
            break;
        default:
            goto parse_failure;
    }
    return 0.0;
parse_failure:
    return -1.0;
}
static CYTHON_INLINE int __Pyx__PyBytes_AsDouble_IsSpace(char ch) {
    return (ch == 0x20) | !((ch < 0x9) | (ch > 0xd));
}
CYTHON_UNUSED static double __Pyx__PyBytes_AsDouble(PyObject *obj, const char* start, Py_ssize_t length) {
    double value;
    Py_ssize_t i, digits;
    const char *last = start + length;
    char *end;
    while (__Pyx__PyBytes_AsDouble_IsSpace(*start))
        start++;
    while (start < last - 1 && __Pyx__PyBytes_AsDouble_IsSpace(last[-1]))
        last--;
    length = last - start;
    if (unlikely(length <= 0)) goto fallback;
    value = __Pyx__PyBytes_AsDouble_inf_nan(start, length);
    if (unlikely(value == -1.0)) goto fallback;
    if (value != 0.0) return value;
    digits = 0;
    for (i=0; i < length; digits += start[i++] != '_');
    if (likely(digits == length)) {
        value = PyOS_string_to_double(start, &end, NULL);
    } else if (digits < 40) {
        char number[40];
        last = __Pyx__PyBytes_AsDouble_Copy(start, number, length);
        if (unlikely(!last)) goto fallback;
        value = PyOS_string_to_double(number, &end, NULL);
    } else {
        char *number = (char*) PyMem_Malloc((digits + 1) * sizeof(char));
        if (unlikely(!number)) goto fallback;
        last = __Pyx__PyBytes_AsDouble_Copy(start, number, length);
        if (unlikely(!last)) {
            PyMem_Free(number);
            goto fallback;
        }
        value = PyOS_string_to_double(number, &end, NULL);
        PyMem_Free(number);
    }
    if (likely(end == last) || (value == (double)-1 && PyErr_Occurred())) {
        return value;
    }
fallback:
    return __Pyx_SlowPyString_AsDouble(obj);
}

/* pyfrozenset_new */
static CYTHON_INLINE PyObject* __Pyx_PyFrozenSet_New(PyObject* it) {
    if (it) {
        PyObject* result;
#if CYTHON_COMPILING_IN_PYPY
        PyObject* args;
        args = PyTuple_Pack(1, it);
        if (unlikely(!args))
            return NULL;
        result = PyObject_Call((PyObject*)&PyFrozenSet_Type, args, NULL);
        Py_DECREF(args);
        return result;
#else
        if (PyFrozenSet_CheckExact(it)) {
            Py_INCREF(it);
            return it;
        }
        result = PyFrozenSet_New(it);
        if (unlikely(!result))
            return NULL;
        if ((PY_VERSION_HEX >= 0x031000A1) || likely(PySet_GET_SIZE(result)))
            return result;
        Py_DECREF(result);
#endif
    }
#if CYTHON_USE_TYPE_SLOTS
    return PyFrozenSet_Type.tp_new(&PyFrozenSet_Type, __pyx_empty_tuple, NULL);
#else
    return PyObject_Call((PyObject*)&PyFrozenSet_Type, __pyx_empty_tuple, NULL);
#endif
}

/* IterNext */
static PyObject *__Pyx_PyIter_Next2Default(PyObject* defval) {
    PyObject* exc_type;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    exc_type = __Pyx_PyErr_CurrentExceptionType();
    if (unlikely(exc_type)) {
        if (!defval || unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)))
            return NULL;
        __Pyx_PyErr_Clear();
        Py_INCREF(defval);
        return defval;
    }
    if (defval) {
        Py_INCREF(defval);
        return defval;
    }
    __Pyx_PyErr_SetNone(PyExc_StopIteration);
    return NULL;
}
static void __Pyx_PyIter_Next_ErrorNoIterator(PyObject *iterator) {
    __Pyx_TypeName iterator_type_name = __Pyx_PyType_GetName(Py_TYPE(iterator));
    PyErr_Format(PyExc_TypeError,
        __Pyx_FMT_TYPENAME " object is not an iterator", iterator_type_name);
    __Pyx_DECREF_TypeName(iterator_type_name);
}
static CYTHON_INLINE PyObject *__Pyx_PyIter_Next2(PyObject* iterator, PyObject* defval) {
    PyObject* next;
    iternextfunc iternext = Py_TYPE(iterator)->tp_iternext;
    if (likely(iternext)) {
#if CYTHON_USE_TYPE_SLOTS || CYTHON_COMPILING_IN_PYPY
        next = iternext(iterator);
        if (likely(next))
            return next;
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030d0000
        if (unlikely(iternext == &_PyObject_NextNotImplemented))
            return NULL;
#endif
#else
        next = PyIter_Next(iterator);
        if (likely(next))
            return next;
#endif
    } else if (CYTHON_USE_TYPE_SLOTS || unlikely(!PyIter_Check(iterator))) {
        __Pyx_PyIter_Next_ErrorNoIterator(iterator);
        return NULL;
    }
#if !CYTHON_USE_TYPE_SLOTS
    else {
        next = PyIter_Next(iterator);
        if (likely(next))
            return next;
    }
#endif
    return __Pyx_PyIter_Next2Default(defval);
}

/* GetItemIntUnicode */
static CYTHON_INLINE Py_UCS4 __Pyx_GetItemInt_Unicode_Fast(PyObject* ustring, Py_ssize_t i,
                                                           int wraparound, int boundscheck) {
    Py_ssize_t length;
    if (unlikely(__Pyx_PyUnicode_READY(ustring) < 0)) return (Py_UCS4)-1;
    if (wraparound | boundscheck) {
        length = __Pyx_PyUnicode_GET_LENGTH(ustring);
        if (wraparound & unlikely(i < 0)) i += length;
        if ((!boundscheck) || likely(__Pyx_is_valid_index(i, length))) {
            return __Pyx_PyUnicode_READ_CHAR(ustring, i);
        } else {
            PyErr_SetString(PyExc_IndexError, "string index out of range");
            return (Py_UCS4)-1;
        }
    } else {
        return __Pyx_PyUnicode_READ_CHAR(ustring, i);
    }
}

/* CIntToDigits */
static const char DIGIT_PAIRS_10[2*10*10+1] = {
    "00010203040506070809"
    "10111213141516171819"
    "20212223242526272829"
    "30313233343536373839"
    "40414243444546474849"
    "50515253545556575859"
    "60616263646566676869"
    "70717273747576777879"
    "80818283848586878889"
    "90919293949596979899"
};
static const char DIGIT_PAIRS_8[2*8*8+1] = {
    "0001020304050607"
    "1011121314151617"
    "2021222324252627"
    "3031323334353637"
    "4041424344454647"
    "5051525354555657"
    "6061626364656667"
    "7071727374757677"
};
static const char DIGITS_HEX[2*16+1] = {
    "0123456789abcdef"
    "0123456789ABCDEF"
};

/* BuildPyUnicode */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, char* chars, int clength,
                                                int prepend_sign, char padding_char) {
    PyObject *uval;
    Py_ssize_t uoffset = ulength - clength;
#if CYTHON_USE_UNICODE_INTERNALS
    Py_ssize_t i;
#if CYTHON_PEP393_ENABLED
    void *udata;
    uval = PyUnicode_New(ulength, 127);
    if (unlikely(!uval)) return NULL;
    udata = PyUnicode_DATA(uval);
#else
    Py_UNICODE *udata;
    uval = PyUnicode_FromUnicode(NULL, ulength);
    if (unlikely(!uval)) return NULL;
    udata = PyUnicode_AS_UNICODE(uval);
#endif
    if (uoffset > 0) {
        i = 0;
        if (prepend_sign) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, 0, '-');
            i++;
        }
        for (; i < uoffset; i++) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, i, padding_char);
        }
    }
    for (i=0; i < clength; i++) {
        __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, uoffset+i, chars[i]);
    }
#else
    {
        PyObject *sign = NULL, *padding = NULL;
        uval = NULL;
        if (uoffset > 0) {
            prepend_sign = !!prepend_sign;
            if (uoffset > prepend_sign) {
                padding = PyUnicode_FromOrdinal(padding_char);
                if (likely(padding) && uoffset > prepend_sign + 1) {
                    PyObject *tmp;
                    PyObject *repeat = PyInt_FromSsize_t(uoffset - prepend_sign);
                    if (unlikely(!repeat)) goto done_or_error;
                    tmp = PyNumber_Multiply(padding, repeat);
                    Py_DECREF(repeat);
                    Py_DECREF(padding);
                    padding = tmp;
                }
                if (unlikely(!padding)) goto done_or_error;
            }
            if (prepend_sign) {
                sign = PyUnicode_FromOrdinal('-');
                if (unlikely(!sign)) goto done_or_error;
            }
        }
        uval = PyUnicode_DecodeASCII(chars, clength, NULL);
        if (likely(uval) && padding) {
            PyObject *tmp = PyNumber_Add(padding, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
        if (likely(uval) && sign) {
            PyObject *tmp = PyNumber_Add(sign, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
done_or_error:
        Py_XDECREF(padding);
        Py_XDECREF(sign);
    }
#endif
    return uval;
}

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_Py_ssize_t(Py_ssize_t value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(Py_ssize_t)*3+2];
    char *dpos, *end = digits + sizeof(Py_ssize_t)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    Py_ssize_t remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const Py_ssize_t neg_one = (Py_ssize_t) -1, const_zero = (Py_ssize_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (Py_ssize_t) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (Py_ssize_t) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (Py_ssize_t) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* JoinPyUnicode */
static PyObject* __Pyx_PyUnicode_Join(PyObject* value_tuple, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char) {
#if CYTHON_USE_UNICODE_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    PyObject *result_uval;
    int result_ukind, kind_shift;
    Py_ssize_t i, char_pos;
    void *result_udata;
    CYTHON_MAYBE_UNUSED_VAR(max_char);
#if CYTHON_PEP393_ENABLED
    result_uval = PyUnicode_New(result_ulength, max_char);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = (max_char <= 255) ? PyUnicode_1BYTE_KIND : (max_char <= 65535) ? PyUnicode_2BYTE_KIND : PyUnicode_4BYTE_KIND;
    kind_shift = (result_ukind == PyUnicode_4BYTE_KIND) ? 2 : result_ukind - 1;
    result_udata = PyUnicode_DATA(result_uval);
#else
    result_uval = PyUnicode_FromUnicode(NULL, result_ulength);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = sizeof(Py_UNICODE);
    kind_shift = (result_ukind == 4) ? 2 : result_ukind - 1;
    result_udata = PyUnicode_AS_UNICODE(result_uval);
#endif
    assert(kind_shift == 2 || kind_shift == 1 || kind_shift == 0);
    char_pos = 0;
    for (i=0; i < value_count; i++) {
        int ukind;
        Py_ssize_t ulength;
        void *udata;
        PyObject *uval = PyTuple_GET_ITEM(value_tuple, i);
        if (unlikely(__Pyx_PyUnicode_READY(uval)))
            goto bad;
        ulength = __Pyx_PyUnicode_GET_LENGTH(uval);
        if (unlikely(!ulength))
            continue;
        if (unlikely((PY_SSIZE_T_MAX >> kind_shift) - ulength < char_pos))
            goto overflow;
        ukind = __Pyx_PyUnicode_KIND(uval);
        udata = __Pyx_PyUnicode_DATA(uval);
        if (!CYTHON_PEP393_ENABLED || ukind == result_ukind) {
            memcpy((char *)result_udata + (char_pos << kind_shift), udata, (size_t) (ulength << kind_shift));
        } else {
            #if PY_VERSION_HEX >= 0x030d0000
            if (unlikely(PyUnicode_CopyCharacters(result_uval, char_pos, uval, 0, ulength) < 0)) goto bad;
            #elif CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030300F0 || defined(_PyUnicode_FastCopyCharacters)
            _PyUnicode_FastCopyCharacters(result_uval, char_pos, uval, 0, ulength);
            #else
            Py_ssize_t j;
            for (j=0; j < ulength; j++) {
                Py_UCS4 uchar = __Pyx_PyUnicode_READ(ukind, udata, j);
                __Pyx_PyUnicode_WRITE(result_ukind, result_udata, char_pos+j, uchar);
            }
            #endif
        }
        char_pos += ulength;
    }
    return result_uval;
overflow:
    PyErr_SetString(PyExc_OverflowError, "join() result is too long for a Python string");
bad:
    Py_DECREF(result_uval);
    return NULL;
#else
    CYTHON_UNUSED_VAR(max_char);
    CYTHON_UNUSED_VAR(result_ulength);
    CYTHON_UNUSED_VAR(value_count);
    return PyUnicode_Join(__pyx_empty_unicode, value_tuple);
#endif
}

/* PyObjectFormatAndDecref */
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f) {
    if (unlikely(!s)) return NULL;
    if (likely(PyUnicode_CheckExact(s))) return s;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyString_CheckExact(s))) {
        PyObject *result = PyUnicode_FromEncodedObject(s, NULL, "strict");
        Py_DECREF(s);
        return result;
    }
    #endif
    return __Pyx_PyObject_FormatAndDecref(s, f);
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f) {
    PyObject *result;
    if (unlikely(!s)) return NULL;
    result = PyObject_Format(s, f);
    Py_DECREF(s);
    return result;
}

/* SwapException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_value = exc_info->exc_value;
    exc_info->exc_value = *value;
    if (tmp_value == NULL || tmp_value == Py_None) {
        Py_XDECREF(tmp_value);
        tmp_value = NULL;
        tmp_type = NULL;
        tmp_tb = NULL;
    } else {
        tmp_type = (PyObject*) Py_TYPE(tmp_value);
        Py_INCREF(tmp_type);
        #if CYTHON_COMPILING_IN_CPYTHON
        tmp_tb = ((PyBaseExceptionObject*) tmp_value)->traceback;
        Py_XINCREF(tmp_tb);
        #else
        tmp_tb = PyException_GetTraceback(tmp_value);
        #endif
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
  #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
  #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* TupleAndListFromArray */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE void __Pyx_copy_object_array(PyObject *const *CYTHON_RESTRICT src, PyObject** CYTHON_RESTRICT dest, Py_ssize_t length) {
    PyObject *v;
    Py_ssize_t i;
    for (i = 0; i < length; i++) {
        v = dest[i] = src[i];
        Py_INCREF(v);
    }
}
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        Py_INCREF(__pyx_empty_tuple);
        return __pyx_empty_tuple;
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyTupleObject*)res)->ob_item, n);
    return res;
}
static CYTHON_INLINE PyObject *
__Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return PyList_New(0);
    }
    res = PyList_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyListObject*)res)->ob_item, n);
    return res;
}
#endif

/* fastcall */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s)
{
    Py_ssize_t i, n = PyTuple_GET_SIZE(kwnames);
    for (i = 0; i < n; i++)
    {
        if (s == PyTuple_GET_ITEM(kwnames, i)) return kwvalues[i];
    }
    for (i = 0; i < n; i++)
    {
        int eq = __Pyx_PyUnicode_Equals(s, PyTuple_GET_ITEM(kwnames, i), Py_EQ);
        if (unlikely(eq != 0)) {
            if (unlikely(eq < 0)) return NULL;
            return kwvalues[i];
        }
    }
    return NULL;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues) {
    Py_ssize_t i, nkwargs = PyTuple_GET_SIZE(kwnames);
    PyObject *dict;
    dict = PyDict_New();
    if (unlikely(!dict))
        return NULL;
    for (i=0; i<nkwargs; i++) {
        PyObject *key = PyTuple_GET_ITEM(kwnames, i);
        if (unlikely(PyDict_SetItem(dict, key, kwvalues[i]) < 0))
            goto bad;
    }
    return dict;
bad:
    Py_DECREF(dict);
    return NULL;
}
#endif
#endif

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject *const *kwvalues,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    int kwds_is_tuple = CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds));
    while (1) {
        Py_XDECREF(key); key = NULL;
        Py_XDECREF(value); value = NULL;
        if (kwds_is_tuple) {
            Py_ssize_t size;
#if CYTHON_ASSUME_SAFE_MACROS
            size = PyTuple_GET_SIZE(kwds);
#else
            size = PyTuple_Size(kwds);
            if (size < 0) goto bad;
#endif
            if (pos >= size) break;
#if CYTHON_AVOID_BORROWED_REFS
            key = __Pyx_PySequence_ITEM(kwds, pos);
            if (!key) goto bad;
#elif CYTHON_ASSUME_SAFE_MACROS
            key = PyTuple_GET_ITEM(kwds, pos);
#else
            key = PyTuple_GetItem(kwds, pos);
            if (!key) goto bad;
#endif
            value = kwvalues[pos];
            pos++;
        }
        else
        {
            if (!PyDict_Next(kwds, &pos, &key, &value)) break;
#if CYTHON_AVOID_BORROWED_REFS
            Py_INCREF(key);
#endif
        }
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
#if CYTHON_AVOID_BORROWED_REFS
            Py_INCREF(value);
            Py_DECREF(key);
#endif
            key = NULL;
            value = NULL;
            continue;
        }
#if !CYTHON_AVOID_BORROWED_REFS
        Py_INCREF(key);
#endif
        Py_INCREF(value);
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
#if CYTHON_AVOID_BORROWED_REFS
                    value = NULL;
#endif
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key)
                );
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
#if CYTHON_AVOID_BORROWED_REFS
                    value = NULL;
#endif
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    Py_XDECREF(key);
    Py_XDECREF(value);
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    #if PY_MAJOR_VERSION < 3
    PyErr_Format(PyExc_TypeError,
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
    PyErr_Format(PyExc_TypeError,
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    Py_XDECREF(key);
    Py_XDECREF(value);
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    __Pyx_TypeName type_name;
    __Pyx_TypeName obj_type_name;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    type_name = __Pyx_PyType_GetName(type);
    obj_type_name = __Pyx_PyType_GetName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected " __Pyx_FMT_TYPENAME
        ", got " __Pyx_FMT_TYPENAME ")", name, type_name, obj_type_name);
    __Pyx_DECREF_TypeName(type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *module = 0;
    PyObject *empty_dict = 0;
    PyObject *empty_list = 0;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (unlikely(!py_import))
        goto bad;
    if (!from_list) {
        empty_list = PyList_New(0);
        if (unlikely(!empty_list))
            goto bad;
        from_list = empty_list;
    }
    #endif
    empty_dict = PyDict_New();
    if (unlikely(!empty_dict))
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.') != NULL) {
                module = PyImport_ImportModuleLevelObject(
                    name, __pyx_d, empty_dict, from_list, 1);
                if (unlikely(!module)) {
                    if (unlikely(!PyErr_ExceptionMatches(PyExc_ImportError)))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (unlikely(!py_level))
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, __pyx_d, empty_dict, from_list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, __pyx_d, empty_dict, from_list, level);
            #endif
        }
    }
bad:
    Py_XDECREF(empty_dict);
    Py_XDECREF(empty_list);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    return module;
}

/* ImportDottedModule */
#if PY_MAJOR_VERSION >= 3
static PyObject *__Pyx__ImportDottedModule_Error(PyObject *name, PyObject *parts_tuple, Py_ssize_t count) {
    PyObject *partial_name = NULL, *slice = NULL, *sep = NULL;
    if (unlikely(PyErr_Occurred())) {
        PyErr_Clear();
    }
    if (likely(PyTuple_GET_SIZE(parts_tuple) == count)) {
        partial_name = name;
    } else {
        slice = PySequence_GetSlice(parts_tuple, 0, count);
        if (unlikely(!slice))
            goto bad;
        sep = PyUnicode_FromStringAndSize(".", 1);
        if (unlikely(!sep))
            goto bad;
        partial_name = PyUnicode_Join(sep, slice);
    }
    PyErr_Format(
#if PY_MAJOR_VERSION < 3
        PyExc_ImportError,
        "No module named '%s'", PyString_AS_STRING(partial_name));
#else
#if PY_VERSION_HEX >= 0x030600B1
        PyExc_ModuleNotFoundError,
#else
        PyExc_ImportError,
#endif
        "No module named '%U'", partial_name);
#endif
bad:
    Py_XDECREF(sep);
    Py_XDECREF(slice);
    Py_XDECREF(partial_name);
    return NULL;
}
#endif
#if PY_MAJOR_VERSION >= 3
static PyObject *__Pyx__ImportDottedModule_Lookup(PyObject *name) {
    PyObject *imported_module;
#if PY_VERSION_HEX < 0x030700A1 || (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM  < 0x07030400)
    PyObject *modules = PyImport_GetModuleDict();
    if (unlikely(!modules))
        return NULL;
    imported_module = __Pyx_PyDict_GetItemStr(modules, name);
    Py_XINCREF(imported_module);
#else
    imported_module = PyImport_GetModule(name);
#endif
    return imported_module;
}
#endif
#if PY_MAJOR_VERSION >= 3
static PyObject *__Pyx_ImportDottedModule_WalkParts(PyObject *module, PyObject *name, PyObject *parts_tuple) {
    Py_ssize_t i, nparts;
    nparts = PyTuple_GET_SIZE(parts_tuple);
    for (i=1; i < nparts && module; i++) {
        PyObject *part, *submodule;
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        part = PyTuple_GET_ITEM(parts_tuple, i);
#else
        part = PySequence_ITEM(parts_tuple, i);
#endif
        submodule = __Pyx_PyObject_GetAttrStrNoError(module, part);
#if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
        Py_DECREF(part);
#endif
        Py_DECREF(module);
        module = submodule;
    }
    if (unlikely(!module)) {
        return __Pyx__ImportDottedModule_Error(name, parts_tuple, i);
    }
    return module;
}
#endif
static PyObject *__Pyx__ImportDottedModule(PyObject *name, PyObject *parts_tuple) {
#if PY_MAJOR_VERSION < 3
    PyObject *module, *from_list, *star = __pyx_n_s__7;
    CYTHON_UNUSED_VAR(parts_tuple);
    from_list = PyList_New(1);
    if (unlikely(!from_list))
        return NULL;
    Py_INCREF(star);
    PyList_SET_ITEM(from_list, 0, star);
    module = __Pyx_Import(name, from_list, 0);
    Py_DECREF(from_list);
    return module;
#else
    PyObject *imported_module;
    PyObject *module = __Pyx_Import(name, NULL, 0);
    if (!parts_tuple || unlikely(!module))
        return module;
    imported_module = __Pyx__ImportDottedModule_Lookup(name);
    if (likely(imported_module)) {
        Py_DECREF(module);
        return imported_module;
    }
    PyErr_Clear();
    return __Pyx_ImportDottedModule_WalkParts(module, name, parts_tuple);
#endif
}
static PyObject *__Pyx_ImportDottedModule(PyObject *name, PyObject *parts_tuple) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030400B1
    PyObject *module = __Pyx__ImportDottedModule_Lookup(name);
    if (likely(module)) {
        PyObject *spec = __Pyx_PyObject_GetAttrStrNoError(module, __pyx_n_s_spec);
        if (likely(spec)) {
            PyObject *unsafe = __Pyx_PyObject_GetAttrStrNoError(spec, __pyx_n_s_initializing);
            if (likely(!unsafe || !__Pyx_PyObject_IsTrue(unsafe))) {
                Py_DECREF(spec);
                spec = NULL;
            }
            Py_XDECREF(unsafe);
        }
        if (likely(!spec)) {
            PyErr_Clear();
            return module;
        }
        Py_DECREF(spec);
        Py_DECREF(module);
    } else if (PyErr_Occurred()) {
        PyErr_Clear();
    }
#endif
    return __Pyx__ImportDottedModule(name, parts_tuple);
}

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_signed_char(signed char value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(signed char)*3+2];
    char *dpos, *end = digits + sizeof(signed char)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    signed char remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const signed char neg_one = (signed char) -1, const_zero = (signed char) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (signed char) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (signed char) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (signed char) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_unsigned_char(unsigned char value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(unsigned char)*3+2];
    char *dpos, *end = digits + sizeof(unsigned char)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    unsigned char remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const unsigned char neg_one = (unsigned char) -1, const_zero = (unsigned char) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (unsigned char) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (unsigned char) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (unsigned char) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_short(short value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(short)*3+2];
    char *dpos, *end = digits + sizeof(short)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    short remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const short neg_one = (short) -1, const_zero = (short) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (short) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (short) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (short) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_unsigned_short(unsigned short value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(unsigned short)*3+2];
    char *dpos, *end = digits + sizeof(unsigned short)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    unsigned short remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const unsigned short neg_one = (unsigned short) -1, const_zero = (unsigned short) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (unsigned short) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (unsigned short) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (unsigned short) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* SliceObject */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, int wraparound) {
    __Pyx_TypeName obj_type_name;
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#else
    CYTHON_UNUSED_VAR(wraparound);
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#else
    CYTHON_UNUSED_VAR(wraparound);
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    obj_type_name = __Pyx_PyType_GetName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "'" __Pyx_FMT_TYPENAME "' object is unsliceable", obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
bad:
    return NULL;
}

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_int(int value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(int)*3+2];
    char *dpos, *end = digits + sizeof(int)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    int remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (int) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (int) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (int) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_unsigned_int(unsigned int value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(unsigned int)*3+2];
    char *dpos, *end = digits + sizeof(unsigned int)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    unsigned int remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const unsigned int neg_one = (unsigned int) -1, const_zero = (unsigned int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (unsigned int) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (unsigned int) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (unsigned int) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_PY_LONG_LONG(PY_LONG_LONG value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(PY_LONG_LONG)*3+2];
    char *dpos, *end = digits + sizeof(PY_LONG_LONG)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    PY_LONG_LONG remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const PY_LONG_LONG neg_one = (PY_LONG_LONG) -1, const_zero = (PY_LONG_LONG) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (PY_LONG_LONG) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (PY_LONG_LONG) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (PY_LONG_LONG) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_From_unsigned_PY_LONG_LONG(unsigned PY_LONG_LONG value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(unsigned PY_LONG_LONG)*3+2];
    char *dpos, *end = digits + sizeof(unsigned PY_LONG_LONG)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    unsigned PY_LONG_LONG remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const unsigned PY_LONG_LONG neg_one = (unsigned PY_LONG_LONG) -1, const_zero = (unsigned PY_LONG_LONG) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (unsigned PY_LONG_LONG) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (unsigned PY_LONG_LONG) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (unsigned PY_LONG_LONG) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* RaiseNoneIterError */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType_3_0_11
#define __PYX_HAVE_RT_ImportType_3_0_11
static PyTypeObject *__Pyx_ImportType_3_0_11(PyObject *module, const char *module_name, const char *class_name,
    size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_0_11 check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
    Py_ssize_t itemsize;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_basicsize;
    PyObject *py_itemsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#if !CYTHON_COMPILING_IN_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
    itemsize = ((PyTypeObject *)result)->tp_itemsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
    py_itemsize = PyObject_GetAttrString(result, "__itemsize__");
    if (!py_itemsize)
        goto bad;
    itemsize = PyLong_AsSsize_t(py_itemsize);
    Py_DECREF(py_itemsize);
    py_itemsize = 0;
    if (itemsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (itemsize) {
        if (size % alignment) {
            alignment = size % alignment;
        }
        if (itemsize < (Py_ssize_t)alignment)
            itemsize = (Py_ssize_t)alignment;
    }
    if ((size_t)(basicsize + itemsize) < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize+itemsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error_3_0_11 &&
            ((size_t)basicsize > size || (size_t)(basicsize + itemsize) < size)) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd-%zd from PyObject",
            module_name, class_name, size, basicsize, basicsize+itemsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn_3_0_11 && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        const char* module_name_str = 0;
        PyObject* module_name = 0;
        PyObject* module_dot = 0;
        PyObject* full_name = 0;
        PyErr_Clear();
        module_name_str = PyModule_GetName(module);
        if (unlikely(!module_name_str)) { goto modbad; }
        module_name = PyUnicode_FromString(module_name_str);
        if (unlikely(!module_name)) { goto modbad; }
        module_dot = PyUnicode_Concat(module_name, __pyx_kp_u__5);
        if (unlikely(!module_dot)) { goto modbad; }
        full_name = PyUnicode_Concat(module_dot, name);
        if (unlikely(!full_name)) { goto modbad; }
        #if PY_VERSION_HEX < 0x030700A1 || (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM  < 0x07030400)
        {
            PyObject *modules = PyImport_GetModuleDict();
            if (unlikely(!modules))
                goto modbad;
            value = PyObject_GetItem(modules, full_name);
        }
        #else
        value = PyImport_GetModule(full_name);
        #endif
      modbad:
        Py_XDECREF(full_name);
        Py_XDECREF(module_dot);
        Py_XDECREF(module_name);
    }
    if (unlikely(!value)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* FixUpExtensionType */
#if CYTHON_USE_TYPE_SPECS
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type) {
#if PY_VERSION_HEX > 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    CYTHON_UNUSED_VAR(spec);
    CYTHON_UNUSED_VAR(type);
#else
    const PyType_Slot *slot = spec->slots;
    while (slot && slot->slot && slot->slot != Py_tp_members)
        slot++;
    if (slot && slot->slot == Py_tp_members) {
        int changed = 0;
#if !(PY_VERSION_HEX <= 0x030900b1 && CYTHON_COMPILING_IN_CPYTHON)
        const
#endif
            PyMemberDef *memb = (PyMemberDef*) slot->pfunc;
        while (memb && memb->name) {
            if (memb->name[0] == '_' && memb->name[1] == '_') {
#if PY_VERSION_HEX < 0x030900b1
                if (strcmp(memb->name, "__weaklistoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_weaklistoffset = memb->offset;
                    changed = 1;
                }
                else if (strcmp(memb->name, "__dictoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_dictoffset = memb->offset;
                    changed = 1;
                }
#if CYTHON_METH_FASTCALL
                else if (strcmp(memb->name, "__vectorcalloffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
#if PY_VERSION_HEX >= 0x030800b4
                    type->tp_vectorcall_offset = memb->offset;
#else
                    type->tp_print = (printfunc) memb->offset;
#endif
                    changed = 1;
                }
#endif
#else
                if ((0));
#endif
#if PY_VERSION_HEX <= 0x030900b1 && CYTHON_COMPILING_IN_CPYTHON
                else if (strcmp(memb->name, "__module__") == 0) {
                    PyObject *descr;
                    assert(memb->type == T_OBJECT);
                    assert(memb->flags == 0 || memb->flags == READONLY);
                    descr = PyDescr_NewMember(type, memb);
                    if (unlikely(!descr))
                        return -1;
                    if (unlikely(PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr) < 0)) {
                        Py_DECREF(descr);
                        return -1;
                    }
                    Py_DECREF(descr);
                    changed = 1;
                }
#endif
            }
            memb++;
        }
        if (changed)
            PyType_Modified(type);
    }
#endif
    return 0;
}
#endif

/* FetchSharedCythonModule */
static PyObject *__Pyx_FetchSharedCythonABIModule(void) {
    return __Pyx_PyImport_AddModuleRef((char*) __PYX_ABI_MODULE_NAME);
}

/* FetchCommonType */
static int __Pyx_VerifyCachedType(PyObject *cached_type,
                               const char *name,
                               Py_ssize_t basicsize,
                               Py_ssize_t expected_basicsize) {
    if (!PyType_Check(cached_type)) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s is not a type object", name);
        return -1;
    }
    if (basicsize != expected_basicsize) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s has the wrong size, try recompiling",
            name);
        return -1;
    }
    return 0;
}
#if !CYTHON_USE_TYPE_SPECS
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* abi_module;
    const char* object_name;
    PyTypeObject *cached_type = NULL;
    abi_module = __Pyx_FetchSharedCythonABIModule();
    if (!abi_module) return NULL;
    object_name = strrchr(type->tp_name, '.');
    object_name = object_name ? object_name+1 : type->tp_name;
    cached_type = (PyTypeObject*) PyObject_GetAttrString(abi_module, object_name);
    if (cached_type) {
        if (__Pyx_VerifyCachedType(
              (PyObject *)cached_type,
              object_name,
              cached_type->tp_basicsize,
              type->tp_basicsize) < 0) {
            goto bad;
        }
        goto done;
    }
    if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
    PyErr_Clear();
    if (PyType_Ready(type) < 0) goto bad;
    if (PyObject_SetAttrString(abi_module, object_name, (PyObject *)type) < 0)
        goto bad;
    Py_INCREF(type);
    cached_type = type;
done:
    Py_DECREF(abi_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}
#else
static PyTypeObject *__Pyx_FetchCommonTypeFromSpec(PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *abi_module, *cached_type = NULL;
    const char* object_name = strrchr(spec->name, '.');
    object_name = object_name ? object_name+1 : spec->name;
    abi_module = __Pyx_FetchSharedCythonABIModule();
    if (!abi_module) return NULL;
    cached_type = PyObject_GetAttrString(abi_module, object_name);
    if (cached_type) {
        Py_ssize_t basicsize;
#if CYTHON_COMPILING_IN_LIMITED_API
        PyObject *py_basicsize;
        py_basicsize = PyObject_GetAttrString(cached_type, "__basicsize__");
        if (unlikely(!py_basicsize)) goto bad;
        basicsize = PyLong_AsSsize_t(py_basicsize);
        Py_DECREF(py_basicsize);
        py_basicsize = 0;
        if (unlikely(basicsize == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
#else
        basicsize = likely(PyType_Check(cached_type)) ? ((PyTypeObject*) cached_type)->tp_basicsize : -1;
#endif
        if (__Pyx_VerifyCachedType(
              cached_type,
              object_name,
              basicsize,
              spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    }
    if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
    PyErr_Clear();
    CYTHON_UNUSED_VAR(module);
    cached_type = __Pyx_PyType_FromModuleAndSpec(abi_module, spec, bases);
    if (unlikely(!cached_type)) goto bad;
    if (unlikely(__Pyx_fix_up_extension_type_from_spec(spec, (PyTypeObject *) cached_type) < 0)) goto bad;
    if (PyObject_SetAttrString(abi_module, object_name, cached_type) < 0) goto bad;
done:
    Py_DECREF(abi_module);
    assert(cached_type == NULL || PyType_Check(cached_type));
    return (PyTypeObject *) cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}
#endif

/* PyVectorcallFastCallDict */
#if CYTHON_METH_FASTCALL
static PyObject *__Pyx_PyVectorcall_FastCallDict_kw(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    PyObject *res = NULL;
    PyObject *kwnames;
    PyObject **newargs;
    PyObject **kwvalues;
    Py_ssize_t i, pos;
    size_t j;
    PyObject *key, *value;
    unsigned long keys_are_strings;
    Py_ssize_t nkw = PyDict_GET_SIZE(kw);
    newargs = (PyObject **)PyMem_Malloc((nargs + (size_t)nkw) * sizeof(args[0]));
    if (unlikely(newargs == NULL)) {
        PyErr_NoMemory();
        return NULL;
    }
    for (j = 0; j < nargs; j++) newargs[j] = args[j];
    kwnames = PyTuple_New(nkw);
    if (unlikely(kwnames == NULL)) {
        PyMem_Free(newargs);
        return NULL;
    }
    kwvalues = newargs + nargs;
    pos = i = 0;
    keys_are_strings = Py_TPFLAGS_UNICODE_SUBCLASS;
    while (PyDict_Next(kw, &pos, &key, &value)) {
        keys_are_strings &= Py_TYPE(key)->tp_flags;
        Py_INCREF(key);
        Py_INCREF(value);
        PyTuple_SET_ITEM(kwnames, i, key);
        kwvalues[i] = value;
        i++;
    }
    if (unlikely(!keys_are_strings)) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        goto cleanup;
    }
    res = vc(func, newargs, nargs, kwnames);
cleanup:
    Py_DECREF(kwnames);
    for (i = 0; i < nkw; i++)
        Py_DECREF(kwvalues[i]);
    PyMem_Free(newargs);
    return res;
}
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    if (likely(kw == NULL) || PyDict_GET_SIZE(kw) == 0) {
        return vc(func, args, nargs, NULL);
    }
    return __Pyx_PyVectorcall_FastCallDict_kw(func, vc, args, nargs, kw);
}
#endif

/* CythonFunctionShared */
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void *cfunc) {
    if (__Pyx_CyFunction_Check(func)) {
        return PyCFunction_GetFunction(((__pyx_CyFunctionObject*)func)->func) == (PyCFunction) cfunc;
    } else if (PyCFunction_Check(func)) {
        return PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
    }
    return 0;
}
#else
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void *cfunc) {
    return __Pyx_CyOrPyCFunction_Check(func) && __Pyx_CyOrPyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
}
#endif
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj) {
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    __Pyx_Py_XDECREF_SET(
        __Pyx_CyFunction_GetClassObj(f),
            ((classobj) ? __Pyx_NewRef(classobj) : NULL));
#else
    __Pyx_Py_XDECREF_SET(
        ((PyCMethodObject *) (f))->mm_class,
        (PyTypeObject*)((classobj) ? __Pyx_NewRef(classobj) : NULL));
#endif
}
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, void *closure)
{
    CYTHON_UNUSED_VAR(closure);
    if (unlikely(op->func_doc == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_doc = PyObject_GetAttrString(op->func, "__doc__");
        if (unlikely(!op->func_doc)) return NULL;
#else
        if (((PyCFunctionObject*)op)->m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(((PyCFunctionObject*)op)->m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(((PyCFunctionObject*)op)->m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
#endif
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_doc, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(op->func_name == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_name = PyObject_GetAttrString(op->func, "__name__");
#elif PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(((PyCFunctionObject*)op)->m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(((PyCFunctionObject*)op)->m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_name, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_qualname, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_dict, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(op);
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = __Pyx_PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = __Pyx_PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyTuple_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__defaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->defaults_tuple, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = op->defaults_tuple;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__kwdefaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(op->defaults_kwdict, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = op->defaults_kwdict;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value || value == Py_None) {
        value = NULL;
    } else if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    __Pyx_Py_XDECREF_SET(op->func_annotations, value);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = op->func_annotations;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_is_coroutine(__pyx_CyFunctionObject *op, void *context) {
    int is_coroutine;
    CYTHON_UNUSED_VAR(context);
    if (op->func_is_coroutine) {
        return __Pyx_NewRef(op->func_is_coroutine);
    }
    is_coroutine = op->flags & __Pyx_CYFUNCTION_COROUTINE;
#if PY_VERSION_HEX >= 0x03050000
    if (is_coroutine) {
        PyObject *module, *fromlist, *marker = __pyx_n_s_is_coroutine;
        fromlist = PyList_New(1);
        if (unlikely(!fromlist)) return NULL;
        Py_INCREF(marker);
#if CYTHON_ASSUME_SAFE_MACROS
        PyList_SET_ITEM(fromlist, 0, marker);
#else
        if (unlikely(PyList_SetItem(fromlist, 0, marker) < 0)) {
            Py_DECREF(marker);
            Py_DECREF(fromlist);
            return NULL;
        }
#endif
        module = PyImport_ImportModuleLevelObject(__pyx_n_s_asyncio_coroutines, NULL, NULL, fromlist, 0);
        Py_DECREF(fromlist);
        if (unlikely(!module)) goto ignore;
        op->func_is_coroutine = __Pyx_PyObject_GetAttrStr(module, marker);
        Py_DECREF(module);
        if (likely(op->func_is_coroutine)) {
            return __Pyx_NewRef(op->func_is_coroutine);
        }
ignore:
        PyErr_Clear();
    }
#endif
    op->func_is_coroutine = __Pyx_PyBool_FromLong(is_coroutine);
    return __Pyx_NewRef(op->func_is_coroutine);
}
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *
__Pyx_CyFunction_get_module(__pyx_CyFunctionObject *op, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_GetAttrString(op->func, "__module__");
}
static int
__Pyx_CyFunction_set_module(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_SetAttrString(op->func, "__module__", value);
}
#endif
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {(char *) "_is_coroutine", (getter)__Pyx_CyFunction_get_is_coroutine, 0, 0, 0},
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", (getter)__Pyx_CyFunction_get_module, (setter)__Pyx_CyFunction_set_module, 0, 0},
#endif
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
#if !CYTHON_COMPILING_IN_LIMITED_API
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), 0, 0},
#endif
#if CYTHON_USE_TYPE_SPECS
    {(char *) "__dictoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_dict), READONLY, 0},
#if CYTHON_METH_FASTCALL
#if CYTHON_BACKPORT_VECTORCALL
    {(char *) "__vectorcalloffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_vectorcall), READONLY, 0},
#else
#if !CYTHON_COMPILING_IN_LIMITED_API
    {(char *) "__vectorcalloffset__", T_PYSSIZET, offsetof(PyCFunctionObject, vectorcall), READONLY, 0},
#endif
#endif
#endif
#if PY_VERSION_HEX < 0x030500A0 || CYTHON_COMPILING_IN_LIMITED_API
    {(char *) "__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_weakreflist), READONLY, 0},
#else
    {(char *) "__weaklistoffset__", T_PYSSIZET, offsetof(PyCFunctionObject, m_weakreflist), READONLY, 0},
#endif
#endif
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, PyObject *args)
{
    CYTHON_UNUSED_VAR(args);
#if PY_MAJOR_VERSION >= 3
    Py_INCREF(m->func_qualname);
    return m->func_qualname;
#else
    return PyString_FromString(((PyCFunctionObject*)m)->m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0 || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) (((PyCFunctionObject*)cyfunc)->m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
#if !CYTHON_COMPILING_IN_LIMITED_API
    PyCFunctionObject *cf = (PyCFunctionObject*) op;
#endif
    if (unlikely(op == NULL))
        return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    op->func = PyCFunction_NewEx(ml, (PyObject*)op, module);
    if (unlikely(!op->func)) return NULL;
#endif
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
#if !CYTHON_COMPILING_IN_LIMITED_API
    cf->m_ml = ml;
    cf->m_self = (PyObject *) op;
#endif
    Py_XINCREF(closure);
    op->func_closure = closure;
#if !CYTHON_COMPILING_IN_LIMITED_API
    Py_XINCREF(module);
    cf->m_module = module;
#endif
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    op->func_classobj = NULL;
#else
    ((PyCMethodObject*)op)->mm_class = NULL;
#endif
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults_size = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    op->func_is_coroutine = NULL;
#if CYTHON_METH_FASTCALL
    switch (ml->ml_flags & (METH_VARARGS | METH_FASTCALL | METH_NOARGS | METH_O | METH_KEYWORDS | METH_METHOD)) {
    case METH_NOARGS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_NOARGS;
        break;
    case METH_O:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_O;
        break;
    case METH_METHOD | METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD;
        break;
    case METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS;
        break;
    case METH_VARARGS | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = NULL;
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        Py_DECREF(op);
        return NULL;
    }
#endif
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(m->func);
#else
    Py_CLEAR(((PyCFunctionObject*)m)->m_module);
#endif
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
#if !CYTHON_COMPILING_IN_LIMITED_API
#if PY_VERSION_HEX < 0x030900B1
    Py_CLEAR(__Pyx_CyFunction_GetClassObj(m));
#else
    {
        PyObject *cls = (PyObject*) ((PyCMethodObject *) (m))->mm_class;
        ((PyCMethodObject *) (m))->mm_class = NULL;
        Py_XDECREF(cls);
    }
#endif
#endif
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    Py_CLEAR(m->func_is_coroutine);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    __Pyx_PyHeapTypeObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(m->func);
#else
    Py_VISIT(((PyCFunctionObject*)m)->m_module);
#endif
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
#if !CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(__Pyx_CyFunction_GetClassObj(m));
#endif
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    Py_VISIT(m->func_is_coroutine);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *f = ((__pyx_CyFunctionObject*)func)->func;
    PyObject *py_name = NULL;
    PyCFunction meth;
    int flags;
    meth = PyCFunction_GetFunction(f);
    if (unlikely(!meth)) return NULL;
    flags = PyCFunction_GetFlags(f);
    if (unlikely(flags < 0)) return NULL;
#else
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    int flags = f->m_ml->ml_flags;
#endif
    Py_ssize_t size;
    switch (flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void*)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_MACROS
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 0))
                return (*meth)(self, NULL);
#if CYTHON_COMPILING_IN_LIMITED_API
            py_name = __Pyx_CyFunction_get_name((__pyx_CyFunctionObject*)func, NULL);
            if (!py_name) return NULL;
            PyErr_Format(PyExc_TypeError,
                "%.200S() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                py_name, size);
            Py_DECREF(py_name);
#else
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
#endif
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_MACROS
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = __Pyx_PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
#if CYTHON_COMPILING_IN_LIMITED_API
            py_name = __Pyx_CyFunction_get_name((__pyx_CyFunctionObject*)func, NULL);
            if (!py_name) return NULL;
            PyErr_Format(PyExc_TypeError,
                "%.200S() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                py_name, size);
            Py_DECREF(py_name);
#else
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
#endif
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        return NULL;
    }
#if CYTHON_COMPILING_IN_LIMITED_API
    py_name = __Pyx_CyFunction_get_name((__pyx_CyFunctionObject*)func, NULL);
    if (!py_name) return NULL;
    PyErr_Format(PyExc_TypeError, "%.200S() takes no keyword arguments",
                 py_name);
    Py_DECREF(py_name);
#else
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
#endif
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *self, *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)func)->func);
    if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
    self = ((PyCFunctionObject*)func)->m_self;
#endif
    result = __Pyx_CyFunction_CallMethod(func, self, arg, kw);
    return result;
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
#if CYTHON_METH_FASTCALL
     __pyx_vectorcallfunc vc = __Pyx_CyFunction_func_vectorcall(cyfunc);
    if (vc) {
#if CYTHON_ASSUME_SAFE_MACROS
        return __Pyx_PyVectorcall_FastCallDict(func, vc, &PyTuple_GET_ITEM(args, 0), (size_t)PyTuple_GET_SIZE(args), kw);
#else
        (void) &__Pyx_PyVectorcall_FastCallDict;
        return PyVectorcall_Call(func, args, kw);
#endif
    }
#endif
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
#if CYTHON_ASSUME_SAFE_MACROS
        argc = PyTuple_GET_SIZE(args);
#else
        argc = PyTuple_Size(args);
        if (unlikely(!argc) < 0) return NULL;
#endif
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
#if PY_MAJOR_VERSION > 2
            PyErr_Format(PyExc_TypeError,
                         "unbound method %.200S() needs an argument",
                         cyfunc->func_qualname);
#else
            PyErr_SetString(PyExc_TypeError,
                            "unbound method needs an argument");
#endif
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE int __Pyx_CyFunction_Vectorcall_CheckArgs(__pyx_CyFunctionObject *cyfunc, Py_ssize_t nargs, PyObject *kwnames)
{
    int ret = 0;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        if (unlikely(nargs < 1)) {
            PyErr_Format(PyExc_TypeError, "%.200s() needs an argument",
                         ((PyCFunctionObject*)cyfunc)->m_ml->ml_name);
            return -1;
        }
        ret = 1;
    }
    if (unlikely(kwnames) && unlikely(PyTuple_GET_SIZE(kwnames))) {
        PyErr_Format(PyExc_TypeError,
                     "%.200s() takes no keyword arguments", ((PyCFunctionObject*)cyfunc)->m_ml->ml_name);
        return -1;
    }
    return ret;
}
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 0)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
            def->ml_name, nargs);
        return NULL;
    }
    return def->ml_meth(self, NULL);
}
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 1)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
            def->ml_name, nargs);
        return NULL;
    }
    return def->ml_meth(self, args[0]);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    return ((__Pyx_PyCFunctionFastWithKeywords)(void(*)(void))def->ml_meth)(self, args, nargs, kwnames);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyMethodDef* def = ((PyCFunctionObject*)cyfunc)->m_ml;
    PyTypeObject *cls = (PyTypeObject *) __Pyx_CyFunction_GetClassObj(cyfunc);
#if CYTHON_BACKPORT_VECTORCALL
    Py_ssize_t nargs = (Py_ssize_t)nargsf;
#else
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
#endif
    PyObject *self;
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
        self = ((PyCFunctionObject*)cyfunc)->m_self;
        break;
    default:
        return NULL;
    }
    return ((__Pyx_PyCMethod)(void(*)(void))def->ml_meth)(self, cls, args, (size_t)nargs, kwnames);
}
#endif
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_CyFunctionType_slots[] = {
    {Py_tp_dealloc, (void *)__Pyx_CyFunction_dealloc},
    {Py_tp_repr, (void *)__Pyx_CyFunction_repr},
    {Py_tp_call, (void *)__Pyx_CyFunction_CallAsMethod},
    {Py_tp_traverse, (void *)__Pyx_CyFunction_traverse},
    {Py_tp_clear, (void *)__Pyx_CyFunction_clear},
    {Py_tp_methods, (void *)__pyx_CyFunction_methods},
    {Py_tp_members, (void *)__pyx_CyFunction_members},
    {Py_tp_getset, (void *)__pyx_CyFunction_getsets},
    {Py_tp_descr_get, (void *)__Pyx_PyMethod_New},
    {0, 0},
};
static PyType_Spec __pyx_CyFunctionType_spec = {
    __PYX_TYPE_MODULE_PREFIX "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
#ifdef Py_TPFLAGS_METHOD_DESCRIPTOR
    Py_TPFLAGS_METHOD_DESCRIPTOR |
#endif
#if (defined(_Py_TPFLAGS_HAVE_VECTORCALL) && CYTHON_METH_FASTCALL)
    _Py_TPFLAGS_HAVE_VECTORCALL |
#endif
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    __pyx_CyFunctionType_slots
};
#else
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    __PYX_TYPE_MODULE_PREFIX "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
#if !CYTHON_METH_FASTCALL
    0,
#elif CYTHON_BACKPORT_VECTORCALL
    (printfunc)offsetof(__pyx_CyFunctionObject, func_vectorcall),
#else
    offsetof(PyCFunctionObject, vectorcall),
#endif
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
#ifdef Py_TPFLAGS_METHOD_DESCRIPTOR
    Py_TPFLAGS_METHOD_DESCRIPTOR |
#endif
#if defined(_Py_TPFLAGS_HAVE_VECTORCALL) && CYTHON_METH_FASTCALL
    _Py_TPFLAGS_HAVE_VECTORCALL |
#endif
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_PyMethod_New,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
    0,
#endif
#if __PYX_NEED_TP_PRINT_SLOT
    0,
#endif
#if PY_VERSION_HEX >= 0x030C0000
    0,
#endif
#if PY_VERSION_HEX >= 0x030d00A4
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
    0,
#endif
};
#endif
static int __pyx_CyFunction_init(PyObject *module) {
#if CYTHON_USE_TYPE_SPECS
    __pyx_CyFunctionType = __Pyx_FetchCommonTypeFromSpec(module, &__pyx_CyFunctionType_spec, NULL);
#else
    CYTHON_UNUSED_VAR(module);
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
#endif
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    m->defaults_size = size;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    CYTHON_MAYBE_UNUSED_VAR(tstate);
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStrNoError(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
#if !CYTHON_COMPILING_IN_LIMITED_API
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}
#endif

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyCode_Replace_For_AddTraceback(PyObject *code, PyObject *scratch_dict,
                                                       PyObject *firstlineno, PyObject *name) {
    PyObject *replace = NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_firstlineno", firstlineno))) return NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_name", name))) return NULL;
    replace = PyObject_GetAttrString(code, "replace");
    if (likely(replace)) {
        PyObject *result;
        result = PyObject_Call(replace, __pyx_empty_tuple, scratch_dict);
        Py_DECREF(replace);
        return result;
    }
    PyErr_Clear();
    #if __PYX_LIMITED_VERSION_HEX < 0x030780000
    {
        PyObject *compiled = NULL, *result = NULL;
        if (unlikely(PyDict_SetItemString(scratch_dict, "code", code))) return NULL;
        if (unlikely(PyDict_SetItemString(scratch_dict, "type", (PyObject*)(&PyType_Type)))) return NULL;
        compiled = Py_CompileString(
            "out = type(code)(\n"
            "  code.co_argcount, code.co_kwonlyargcount, code.co_nlocals, code.co_stacksize,\n"
            "  code.co_flags, code.co_code, code.co_consts, code.co_names,\n"
            "  code.co_varnames, code.co_filename, co_name, co_firstlineno,\n"
            "  code.co_lnotab)\n", "<dummy>", Py_file_input);
        if (!compiled) return NULL;
        result = PyEval_EvalCode(compiled, scratch_dict, scratch_dict);
        Py_DECREF(compiled);
        if (!result) PyErr_Print();
        Py_DECREF(result);
        result = PyDict_GetItemString(scratch_dict, "out");
        if (result) Py_INCREF(result);
        return result;
    }
    #else
    return NULL;
    #endif
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyObject *code_object = NULL, *py_py_line = NULL, *py_funcname = NULL, *dict = NULL;
    PyObject *replace = NULL, *getframe = NULL, *frame = NULL;
    PyObject *exc_type, *exc_value, *exc_traceback;
    int success = 0;
    if (c_line) {
        (void) __pyx_cfilenm;
        (void) __Pyx_CLineForTraceback(__Pyx_PyThreadState_Current, c_line);
    }
    PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
    code_object = Py_CompileString("_getframe()", filename, Py_eval_input);
    if (unlikely(!code_object)) goto bad;
    py_py_line = PyLong_FromLong(py_line);
    if (unlikely(!py_py_line)) goto bad;
    py_funcname = PyUnicode_FromString(funcname);
    if (unlikely(!py_funcname)) goto bad;
    dict = PyDict_New();
    if (unlikely(!dict)) goto bad;
    {
        PyObject *old_code_object = code_object;
        code_object = __Pyx_PyCode_Replace_For_AddTraceback(code_object, dict, py_py_line, py_funcname);
        Py_DECREF(old_code_object);
    }
    if (unlikely(!code_object)) goto bad;
    getframe = PySys_GetObject("_getframe");
    if (unlikely(!getframe)) goto bad;
    if (unlikely(PyDict_SetItemString(dict, "_getframe", getframe))) goto bad;
    frame = PyEval_EvalCode(code_object, dict, dict);
    if (unlikely(!frame) || frame == Py_None) goto bad;
    success = 1;
  bad:
    PyErr_Restore(exc_type, exc_value, exc_traceback);
    Py_XDECREF(code_object);
    Py_XDECREF(py_py_line);
    Py_XDECREF(py_funcname);
    Py_XDECREF(dict);
    Py_XDECREF(replace);
    if (success) {
        PyTraceBack_Here(
            (struct _frame*)frame);
    }
    Py_XDECREF(frame);
}
#else
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}
#endif

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* Declarations */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
#else
    static CYTHON_INLINE int __Pyx_c_eq_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sum_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_diff_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prod_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabsf(b.real) >= fabsf(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                float r = b.imag / b.real;
                float s = (float)(1.0) / (b.real + b.imag * r);
                return __pyx_t_float_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            float r = b.real / b.imag;
            float s = (float)(1.0) / (b.imag + b.real * r);
            return __pyx_t_float_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quot_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        if (b.imag == 0) {
            return __pyx_t_float_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            float denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_float_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_neg_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_float(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conj_float(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_abs_float(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_pow_float(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_float(a, a);
                    case 3:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, a);
                    case 4:
                        z = __Pyx_c_prod_float(a, a);
                        return __Pyx_c_prod_float(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if ((b.imag == 0) && (a.real >= 0)) {
                    z.real = powf(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2f(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_float(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

/* Declarations */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
#else
    static CYTHON_INLINE int __Pyx_c_eq_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabs(b.real) >= fabs(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                double r = b.imag / b.real;
                double s = (double)(1.0) / (b.real + b.imag * r);
                return __pyx_t_double_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            double r = b.real / b.imag;
            double s = (double)(1.0) / (b.imag + b.real * r);
            return __pyx_t_double_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            double denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_double_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_double(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj_double(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs_double(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow_double(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_double(a, a);
                    case 3:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, a);
                    case 4:
                        z = __Pyx_c_prod_double(a, a);
                        return __Pyx_c_prod_double(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if ((b.imag == 0) && (a.real >= 0)) {
                    z.real = pow(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_double(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

/* Declarations */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_long_double_complex __pyx_t_long_double_complex_from_parts(long double x, long double y) {
      return ::std::complex< long double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_long_double_complex __pyx_t_long_double_complex_from_parts(long double x, long double y) {
      return x + y*(__pyx_t_long_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_long_double_complex __pyx_t_long_double_complex_from_parts(long double x, long double y) {
      __pyx_t_long_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

/* Arithmetic */
#if CYTHON_CCOMPLEX && (1) && (!0 || __cplusplus)
#else
    static CYTHON_INLINE int __Pyx_c_eq_long__double(__pyx_t_long_double_complex a, __pyx_t_long_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_sum_long__double(__pyx_t_long_double_complex a, __pyx_t_long_double_complex b) {
        __pyx_t_long_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_diff_long__double(__pyx_t_long_double_complex a, __pyx_t_long_double_complex b) {
        __pyx_t_long_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_prod_long__double(__pyx_t_long_double_complex a, __pyx_t_long_double_complex b) {
        __pyx_t_long_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    #if 1
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_quot_long__double(__pyx_t_long_double_complex a, __pyx_t_long_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_long_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else if (fabsl(b.real) >= fabsl(b.imag)) {
            if (b.real == 0 && b.imag == 0) {
                return __pyx_t_long_double_complex_from_parts(a.real / b.real, a.imag / b.imag);
            } else {
                long double r = b.imag / b.real;
                long double s = (long double)(1.0) / (b.real + b.imag * r);
                return __pyx_t_long_double_complex_from_parts(
                    (a.real + a.imag * r) * s, (a.imag - a.real * r) * s);
            }
        } else {
            long double r = b.real / b.imag;
            long double s = (long double)(1.0) / (b.imag + b.real * r);
            return __pyx_t_long_double_complex_from_parts(
                (a.real * r + a.imag) * s, (a.imag * r - a.real) * s);
        }
    }
    #else
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_quot_long__double(__pyx_t_long_double_complex a, __pyx_t_long_double_complex b) {
        if (b.imag == 0) {
            return __pyx_t_long_double_complex_from_parts(a.real / b.real, a.imag / b.real);
        } else {
            long double denom = b.real * b.real + b.imag * b.imag;
            return __pyx_t_long_double_complex_from_parts(
                (a.real * b.real + a.imag * b.imag) / denom,
                (a.imag * b.real - a.real * b.imag) / denom);
        }
    }
    #endif
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_neg_long__double(__pyx_t_long_double_complex a) {
        __pyx_t_long_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero_long__double(__pyx_t_long_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_conj_long__double(__pyx_t_long_double_complex a) {
        __pyx_t_long_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE long double __Pyx_c_abs_long__double(__pyx_t_long_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtl(z.real*z.real + z.imag*z.imag);
          #else
            return hypotl(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_long_double_complex __Pyx_c_pow_long__double(__pyx_t_long_double_complex a, __pyx_t_long_double_complex b) {
            __pyx_t_long_double_complex z;
            long double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    long double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        return __Pyx_c_prod_long__double(a, a);
                    case 3:
                        z = __Pyx_c_prod_long__double(a, a);
                        return __Pyx_c_prod_long__double(z, a);
                    case 4:
                        z = __Pyx_c_prod_long__double(a, a);
                        return __Pyx_c_prod_long__double(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                } else if ((b.imag == 0) && (a.real >= 0)) {
                    z.real = powl(a.real, b.real);
                    z.imag = 0;
                    return z;
                } else if (a.real > 0) {
                    r = a.real;
                    theta = 0;
                } else {
                    r = -a.real;
                    theta = atan2l(0.0, -1.0);
                }
            } else {
                r = __Pyx_c_abs_long__double(a);
                theta = atan2l(a.imag, a.real);
            }
            lnr = logl(r);
            z_r = expl(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosl(z_theta);
            z.imag = z_r * sinl(z_theta);
            return z;
        }
    #endif
#endif

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_char(unsigned char value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const unsigned char neg_one = (unsigned char) -1, const_zero = (unsigned char) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(unsigned char) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(unsigned char) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned char) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(unsigned char) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned char) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(unsigned char),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(unsigned char));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_PY_LONG_LONG(unsigned PY_LONG_LONG value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const unsigned PY_LONG_LONG neg_one = (unsigned PY_LONG_LONG) -1, const_zero = (unsigned PY_LONG_LONG) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(unsigned PY_LONG_LONG) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(unsigned PY_LONG_LONG) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned PY_LONG_LONG) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(unsigned PY_LONG_LONG) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned PY_LONG_LONG) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(unsigned PY_LONG_LONG),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(unsigned PY_LONG_LONG));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
static CYTHON_INLINE unsigned PY_LONG_LONG __Pyx_PyInt_As_unsigned_PY_LONG_LONG(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const unsigned PY_LONG_LONG neg_one = (unsigned PY_LONG_LONG) -1, const_zero = (unsigned PY_LONG_LONG) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(unsigned PY_LONG_LONG) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (unsigned PY_LONG_LONG) val;
        }
    }
#endif
    if (unlikely(!PyLong_Check(x))) {
        unsigned PY_LONG_LONG val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (unsigned PY_LONG_LONG) -1;
        val = __Pyx_PyInt_As_unsigned_PY_LONG_LONG(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(unsigned PY_LONG_LONG) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(unsigned PY_LONG_LONG) >= 2 * PyLong_SHIFT)) {
                            return (unsigned PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(unsigned PY_LONG_LONG) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(unsigned PY_LONG_LONG) >= 3 * PyLong_SHIFT)) {
                            return (unsigned PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(unsigned PY_LONG_LONG) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(unsigned PY_LONG_LONG) >= 4 * PyLong_SHIFT)) {
                            return (unsigned PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (unsigned PY_LONG_LONG) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(unsigned PY_LONG_LONG) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(unsigned PY_LONG_LONG, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(unsigned PY_LONG_LONG) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(unsigned PY_LONG_LONG, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(unsigned PY_LONG_LONG) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(unsigned PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT)) {
                            return (unsigned PY_LONG_LONG) (((unsigned PY_LONG_LONG)-1)*(((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(unsigned PY_LONG_LONG) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(unsigned PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT)) {
                            return (unsigned PY_LONG_LONG) ((((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(unsigned PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(unsigned PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT)) {
                            return (unsigned PY_LONG_LONG) (((unsigned PY_LONG_LONG)-1)*(((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(unsigned PY_LONG_LONG) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(unsigned PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT)) {
                            return (unsigned PY_LONG_LONG) ((((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(unsigned PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(unsigned PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT)) {
                            return (unsigned PY_LONG_LONG) (((unsigned PY_LONG_LONG)-1)*(((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(unsigned PY_LONG_LONG) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(unsigned PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT)) {
                            return (unsigned PY_LONG_LONG) ((((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(unsigned PY_LONG_LONG) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(unsigned PY_LONG_LONG, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(unsigned PY_LONG_LONG) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(unsigned PY_LONG_LONG, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
        }
    }
    {
        unsigned PY_LONG_LONG val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (unsigned PY_LONG_LONG) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (unsigned PY_LONG_LONG) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (unsigned PY_LONG_LONG) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (unsigned PY_LONG_LONG) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(unsigned PY_LONG_LONG) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((unsigned PY_LONG_LONG) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(unsigned PY_LONG_LONG) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((unsigned PY_LONG_LONG) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((unsigned PY_LONG_LONG) 1) << (sizeof(unsigned PY_LONG_LONG) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (unsigned PY_LONG_LONG) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to unsigned PY_LONG_LONG");
    return (unsigned PY_LONG_LONG) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to unsigned PY_LONG_LONG");
    return (unsigned PY_LONG_LONG) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_PY_LONG_LONG(PY_LONG_LONG value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const PY_LONG_LONG neg_one = (PY_LONG_LONG) -1, const_zero = (PY_LONG_LONG) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(PY_LONG_LONG) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(PY_LONG_LONG) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(PY_LONG_LONG) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(PY_LONG_LONG) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(PY_LONG_LONG) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(PY_LONG_LONG),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(PY_LONG_LONG));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_char(char value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const char neg_one = (char) -1, const_zero = (char) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(char) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(char) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(char) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(char) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(char) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(char),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(char));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(int));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_signed_char(signed char value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const signed char neg_one = (signed char) -1, const_zero = (signed char) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(signed char) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(signed char) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(signed char) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(signed char) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(signed char) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(signed char),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(signed char));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_short(short value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const short neg_one = (short) -1, const_zero = (short) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(short) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(short) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(short) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(short) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(short) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(short),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(short));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_short(unsigned short value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const unsigned short neg_one = (unsigned short) -1, const_zero = (unsigned short) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(unsigned short) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(unsigned short) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned short) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(unsigned short) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned short) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(unsigned short),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(unsigned short));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_int(unsigned int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const unsigned int neg_one = (unsigned int) -1, const_zero = (unsigned int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(unsigned int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(unsigned int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(unsigned int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(unsigned int),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(unsigned int));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL;
        PyObject *py_bytes = NULL, *arg_tuple = NULL, *kwds = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(long));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        arg_tuple = PyTuple_Pack(2, py_bytes, order_str);
        if (!arg_tuple) goto limited_bad;
        if (!is_unsigned) {
            kwds = PyDict_New();
            if (!kwds) goto limited_bad;
            if (PyDict_SetItemString(kwds, "signed", __Pyx_NewRef(Py_True))) goto limited_bad;
        }
        result = PyObject_Call(from_bytes, arg_tuple, kwds);
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(arg_tuple);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* FormatTypeName */
#if CYTHON_COMPILING_IN_LIMITED_API
static __Pyx_TypeName
__Pyx_PyType_GetName(PyTypeObject* tp)
{
    PyObject *name = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_n_s_name_2);
    if (unlikely(name == NULL) || unlikely(!PyUnicode_Check(name))) {
        PyErr_Clear();
        Py_XDECREF(name);
        name = __Pyx_NewRef(__pyx_n_s__47);
    }
    return name;
}
#endif

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(long) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    }
#endif
    if (unlikely(!PyLong_Check(x))) {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 2 * PyLong_SHIFT)) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 3 * PyLong_SHIFT)) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 4 * PyLong_SHIFT)) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (long) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(long) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(long) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(long) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(long) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(long) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
        }
    }
    {
        long val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (long) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (long) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (long) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (long) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(long) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((long) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(long) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((long) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((long) 1) << (sizeof(long) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (long) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if ((sizeof(int) < sizeof(long))) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    }
#endif
    if (unlikely(!PyLong_Check(x))) {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 2 * PyLong_SHIFT)) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 3 * PyLong_SHIFT)) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 4 * PyLong_SHIFT)) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (int) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(int) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(int) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(int) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(int) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
        } else if ((sizeof(int) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
        }
    }
    {
        int val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (int) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (int) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (int) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (int) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(int) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((int) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(int) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((int) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((int) 1) << (sizeof(int) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (int) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = __Pyx_PyType_GetSlot(a, tp_base, PyTypeObject*);
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (cls == a || cls == b) return 1;
    mro = cls->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            PyObject *base = PyTuple_GET_ITEM(mro, i);
            if (base == (PyObject *)a || base == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(cls, a) || __Pyx_InBases(cls, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    if (exc_type1) {
        return __Pyx_IsAnySubtype2((PyTypeObject*)err, (PyTypeObject*)exc_type1, (PyTypeObject*)exc_type2);
    } else {
        return __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static unsigned long __Pyx_get_runtime_version(void) {
#if __PYX_LIMITED_VERSION_HEX >= 0x030B00A4
    return Py_Version & ~0xFFUL;
#else
    const char* rt_version = Py_GetVersion();
    unsigned long version = 0;
    unsigned long factor = 0x01000000UL;
    unsigned int digit = 0;
    int i = 0;
    while (factor) {
        while ('0' <= rt_version[i] && rt_version[i] <= '9') {
            digit = digit * 10 + (unsigned int) (rt_version[i] - '0');
            ++i;
        }
        version += factor * digit;
        if (rt_version[i] != '.')
            break;
        digit = 0;
        factor >>= 8;
        ++i;
    }
    return version;
#endif
}
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer) {
    const unsigned long MAJOR_MINOR = 0xFFFF0000UL;
    if ((rt_version & MAJOR_MINOR) == (ct_version & MAJOR_MINOR))
        return 0;
    if (likely(allow_newer && (rt_version & MAJOR_MINOR) > (ct_version & MAJOR_MINOR)))
        return 1;
    {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compile time Python version %d.%d "
                      "of module '%.100s' "
                      "%s "
                      "runtime version %d.%d",
                       (int) (ct_version >> 24), (int) ((ct_version >> 16) & 0xFF),
                       __Pyx_MODULE_NAME,
                       (allow_newer) ? "was newer than" : "does not match",
                       (int) (rt_version >> 24), (int) ((rt_version >> 16) & 0xFF)
       );
        return PyErr_WarnEx(NULL, message, 1);
    }
}

/* FunctionExport */
static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(__pyx_m, (char *)"__pyx_capi__");
    if (!d) {
        PyErr_Clear();
        d = PyDict_New();
        if (!d)
            goto bad;
        Py_INCREF(d);
        if (PyModule_AddObject(__pyx_m, (char *)"__pyx_capi__", d) < 0)
            goto bad;
    }
    tmp.fp = f;
    cobj = PyCapsule_New(tmp.p, sig, 0);
    if (!cobj)
        goto bad;
    if (PyDict_SetItemString(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

/* VoidPtrImport */
#ifndef __PYX_HAVE_RT_ImportVoidPtr_3_0_11
#define __PYX_HAVE_RT_ImportVoidPtr_3_0_11
static int __Pyx_ImportVoidPtr_3_0_11(PyObject *module, const char *name, void **p, const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, name);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C variable %.200s",
                PyModule_GetName(module), name);
        goto bad;
    }
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
#if PY_MAJOR_VERSION >= 3
static int __Pyx_InitString(__Pyx_StringTabEntry t, PyObject **str) {
    if (t.is_unicode | t.is_str) {
        if (t.intern) {
            *str = PyUnicode_InternFromString(t.s);
        } else if (t.encoding) {
            *str = PyUnicode_Decode(t.s, t.n - 1, t.encoding, NULL);
        } else {
            *str = PyUnicode_FromStringAndSize(t.s, t.n - 1);
        }
    } else {
        *str = PyBytes_FromStringAndSize(t.s, t.n - 1);
    }
    if (!*str)
        return -1;
    if (PyObject_Hash(*str) == -1)
        return -1;
    return 0;
}
#endif
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION >= 3
        __Pyx_InitString(*t, t->p);
        #else
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        #endif
        ++t;
    }
    return 0;
}

#include <string.h>
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s) {
    size_t len = strlen(s);
    if (unlikely(len > (size_t) PY_SSIZE_T_MAX)) {
        PyErr_SetString(PyExc_OverflowError, "byte string is too long");
        return -1;
    }
    return (Py_ssize_t) len;
}
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return __Pyx_PyUnicode_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return PyByteArray_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY && !CYTHON_COMPILING_IN_LIMITED_API) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
    __Pyx_TypeName result_type_name = __Pyx_PyType_GetName(Py_TYPE(result));
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ").  "
                "The ability to return an instance of a strict subclass of int is deprecated, "
                "and may be removed in a future version of Python.",
                result_type_name)) {
            __Pyx_DECREF_TypeName(result_type_name);
            Py_DECREF(result);
            return NULL;
        }
        __Pyx_DECREF_TypeName(result_type_name);
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type " __Pyx_FMT_TYPENAME ")",
                 type_name, type_name, result_type_name);
    __Pyx_DECREF_TypeName(result_type_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(__Pyx_PyLong_IsCompact(b))) {
        return __Pyx_PyLong_CompactValue(b);
    } else {
      const digit* digits = __Pyx_PyLong_Digits(b);
      const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(b);
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


/* #### Code section: utility_code_pragmas_end ### */
#ifdef _MSC_VER
#pragma warning( pop )
#endif



/* #### Code section: end ### */
#endif /* Py_PYTHON_H */
